# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Supported printer types used in resource_printer."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


CONFIG = 'config'
CSV = 'csv'
DEFAULT = 'default'
DIFF = 'diff'
DISABLE = 'disable'
FLATTENED = 'flattened'
GET = 'get'
JSON = 'json'
LIST = 'list'
MULTI = 'multi'
NONE = 'none'
OBJECT = 'object'
TABLE = 'table'
TEXT = 'text'
VALUE = 'value'
YAML = 'yaml'
