# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/appengine_v1/proto/app_yaml.proto

from cloudsdk.google.protobuf.internal import enum_type_wrapper
from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cloudsdk.google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/appengine_v1/proto/app_yaml.proto',
  package='google.appengine.v1',
  syntax='proto3',
  serialized_options=b'\n\027com.google.appengine.v1B\014AppYamlProtoP\001Z<google.golang.org/genproto/googleapis/appengine/v1;appengine',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.google/cloud/appengine_v1/proto/app_yaml.proto\x12\x13google.appengine.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/api/annotations.proto\"\xe0\x01\n\x10\x41piConfigHandler\x12=\n\x10\x61uth_fail_action\x18\x01 \x01(\x0e\x32#.google.appengine.v1.AuthFailAction\x12\x34\n\x05login\x18\x02 \x01(\x0e\x32%.google.appengine.v1.LoginRequirement\x12\x0e\n\x06script\x18\x03 \x01(\t\x12:\n\x0esecurity_level\x18\x04 \x01(\x0e\x32\".google.appengine.v1.SecurityLevel\x12\x0b\n\x03url\x18\x05 \x01(\t\"\x8f\x02\n\x0c\x45rrorHandler\x12?\n\nerror_code\x18\x01 \x01(\x0e\x32+.google.appengine.v1.ErrorHandler.ErrorCode\x12\x13\n\x0bstatic_file\x18\x02 \x01(\t\x12\x11\n\tmime_type\x18\x03 \x01(\t\"\x95\x01\n\tErrorCode\x12\x1a\n\x16\x45RROR_CODE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x45RROR_CODE_DEFAULT\x10\x00\x12\x19\n\x15\x45RROR_CODE_OVER_QUOTA\x10\x01\x12\x1d\n\x19\x45RROR_CODE_DOS_API_DENIAL\x10\x02\x12\x16\n\x12\x45RROR_CODE_TIMEOUT\x10\x03\x1a\x02\x10\x01\"\xcd\x05\n\x06UrlMap\x12\x11\n\turl_regex\x18\x01 \x01(\t\x12?\n\x0cstatic_files\x18\x02 \x01(\x0b\x32\'.google.appengine.v1.StaticFilesHandlerH\x00\x12\x34\n\x06script\x18\x03 \x01(\x0b\x32\".google.appengine.v1.ScriptHandlerH\x00\x12?\n\x0c\x61pi_endpoint\x18\x04 \x01(\x0b\x32\'.google.appengine.v1.ApiEndpointHandlerH\x00\x12:\n\x0esecurity_level\x18\x05 \x01(\x0e\x32\".google.appengine.v1.SecurityLevel\x12\x34\n\x05login\x18\x06 \x01(\x0e\x32%.google.appengine.v1.LoginRequirement\x12=\n\x10\x61uth_fail_action\x18\x07 \x01(\x0e\x32#.google.appengine.v1.AuthFailAction\x12Y\n\x1bredirect_http_response_code\x18\x08 \x01(\x0e\x32\x34.google.appengine.v1.UrlMap.RedirectHttpResponseCode\"\xdb\x01\n\x18RedirectHttpResponseCode\x12+\n\'REDIRECT_HTTP_RESPONSE_CODE_UNSPECIFIED\x10\x00\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_301\x10\x01\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_302\x10\x02\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_303\x10\x03\x12#\n\x1fREDIRECT_HTTP_RESPONSE_CODE_307\x10\x04\x42\x0e\n\x0chandler_type\"\xc0\x02\n\x12StaticFilesHandler\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x19\n\x11upload_path_regex\x18\x02 \x01(\t\x12N\n\x0chttp_headers\x18\x03 \x03(\x0b\x32\x38.google.appengine.v1.StaticFilesHandler.HttpHeadersEntry\x12\x11\n\tmime_type\x18\x04 \x01(\t\x12-\n\nexpiration\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1d\n\x15require_matching_file\x18\x06 \x01(\x08\x12\x1c\n\x14\x61pplication_readable\x18\x07 \x01(\x08\x1a\x32\n\x10HttpHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"$\n\rScriptHandler\x12\x13\n\x0bscript_path\x18\x01 \x01(\t\")\n\x12\x41piEndpointHandler\x12\x13\n\x0bscript_path\x18\x01 \x01(\t\"\xeb\x01\n\x0bHealthCheck\x12\x1c\n\x14\x64isable_health_check\x18\x01 \x01(\x08\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11healthy_threshold\x18\x03 \x01(\r\x12\x1b\n\x13unhealthy_threshold\x18\x04 \x01(\r\x12\x19\n\x11restart_threshold\x18\x05 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xf7\x01\n\x0eReadinessCheck\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11\x66\x61ilure_threshold\x18\x03 \x01(\r\x12\x19\n\x11success_threshold\x18\x04 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x34\n\x11\x61pp_start_timeout\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xf2\x01\n\rLivenessCheck\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x19\n\x11\x66\x61ilure_threshold\x18\x03 \x01(\r\x12\x19\n\x11success_threshold\x18\x04 \x01(\r\x12\x31\n\x0e\x63heck_interval\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x30\n\rinitial_delay\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\"(\n\x07Library\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t*t\n\x0e\x41uthFailAction\x12 \n\x1c\x41UTH_FAIL_ACTION_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x41UTH_FAIL_ACTION_REDIRECT\x10\x01\x12!\n\x1d\x41UTH_FAIL_ACTION_UNAUTHORIZED\x10\x02*b\n\x10LoginRequirement\x12\x15\n\x11LOGIN_UNSPECIFIED\x10\x00\x12\x12\n\x0eLOGIN_OPTIONAL\x10\x01\x12\x0f\n\x0bLOGIN_ADMIN\x10\x02\x12\x12\n\x0eLOGIN_REQUIRED\x10\x03*y\n\rSecurityLevel\x12\x16\n\x12SECURE_UNSPECIFIED\x10\x00\x12\x12\n\x0eSECURE_DEFAULT\x10\x00\x12\x10\n\x0cSECURE_NEVER\x10\x01\x12\x13\n\x0fSECURE_OPTIONAL\x10\x02\x12\x11\n\rSECURE_ALWAYS\x10\x03\x1a\x02\x10\x01\x42g\n\x17\x63om.google.appengine.v1B\x0c\x41ppYamlProtoP\x01Z<google.golang.org/genproto/googleapis/appengine/v1;appengineb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])

_AUTHFAILACTION = _descriptor.EnumDescriptor(
  name='AuthFailAction',
  full_name='google.appengine.v1.AuthFailAction',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_REDIRECT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL_ACTION_UNAUTHORIZED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2533,
  serialized_end=2649,
)
_sym_db.RegisterEnumDescriptor(_AUTHFAILACTION)

AuthFailAction = enum_type_wrapper.EnumTypeWrapper(_AUTHFAILACTION)
_LOGINREQUIREMENT = _descriptor.EnumDescriptor(
  name='LoginRequirement',
  full_name='google.appengine.v1.LoginRequirement',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LOGIN_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_OPTIONAL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_ADMIN', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGIN_REQUIRED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2651,
  serialized_end=2749,
)
_sym_db.RegisterEnumDescriptor(_LOGINREQUIREMENT)

LoginRequirement = enum_type_wrapper.EnumTypeWrapper(_LOGINREQUIREMENT)
_SECURITYLEVEL = _descriptor.EnumDescriptor(
  name='SecurityLevel',
  full_name='google.appengine.v1.SecurityLevel',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SECURE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_DEFAULT', index=1, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_NEVER', index=2, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_OPTIONAL', index=3, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE_ALWAYS', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\020\001',
  serialized_start=2751,
  serialized_end=2872,
)
_sym_db.RegisterEnumDescriptor(_SECURITYLEVEL)

SecurityLevel = enum_type_wrapper.EnumTypeWrapper(_SECURITYLEVEL)
AUTH_FAIL_ACTION_UNSPECIFIED = 0
AUTH_FAIL_ACTION_REDIRECT = 1
AUTH_FAIL_ACTION_UNAUTHORIZED = 2
LOGIN_UNSPECIFIED = 0
LOGIN_OPTIONAL = 1
LOGIN_ADMIN = 2
LOGIN_REQUIRED = 3
SECURE_UNSPECIFIED = 0
SECURE_DEFAULT = 0
SECURE_NEVER = 1
SECURE_OPTIONAL = 2
SECURE_ALWAYS = 3


_ERRORHANDLER_ERRORCODE = _descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='google.appengine.v1.ErrorHandler.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_DEFAULT', index=1, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_OVER_QUOTA', index=2, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_DOS_API_DENIAL', index=3, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CODE_TIMEOUT', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\020\001',
  serialized_start=483,
  serialized_end=632,
)
_sym_db.RegisterEnumDescriptor(_ERRORHANDLER_ERRORCODE)

_URLMAP_REDIRECTHTTPRESPONSECODE = _descriptor.EnumDescriptor(
  name='RedirectHttpResponseCode',
  full_name='google.appengine.v1.UrlMap.RedirectHttpResponseCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_301', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_302', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_303', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIRECT_HTTP_RESPONSE_CODE_307', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1117,
  serialized_end=1336,
)
_sym_db.RegisterEnumDescriptor(_URLMAP_REDIRECTHTTPRESPONSECODE)


_APICONFIGHANDLER = _descriptor.Descriptor(
  name='ApiConfigHandler',
  full_name='google.appengine.v1.ApiConfigHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='auth_fail_action', full_name='google.appengine.v1.ApiConfigHandler.auth_fail_action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='login', full_name='google.appengine.v1.ApiConfigHandler.login', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='script', full_name='google.appengine.v1.ApiConfigHandler.script', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_level', full_name='google.appengine.v1.ApiConfigHandler.security_level', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='url', full_name='google.appengine.v1.ApiConfigHandler.url', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=358,
)


_ERRORHANDLER = _descriptor.Descriptor(
  name='ErrorHandler',
  full_name='google.appengine.v1.ErrorHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='google.appengine.v1.ErrorHandler.error_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_file', full_name='google.appengine.v1.ErrorHandler.static_file', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mime_type', full_name='google.appengine.v1.ErrorHandler.mime_type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ERRORHANDLER_ERRORCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=361,
  serialized_end=632,
)


_URLMAP = _descriptor.Descriptor(
  name='UrlMap',
  full_name='google.appengine.v1.UrlMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='url_regex', full_name='google.appengine.v1.UrlMap.url_regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_files', full_name='google.appengine.v1.UrlMap.static_files', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='script', full_name='google.appengine.v1.UrlMap.script', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='api_endpoint', full_name='google.appengine.v1.UrlMap.api_endpoint', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_level', full_name='google.appengine.v1.UrlMap.security_level', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='login', full_name='google.appengine.v1.UrlMap.login', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='auth_fail_action', full_name='google.appengine.v1.UrlMap.auth_fail_action', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='redirect_http_response_code', full_name='google.appengine.v1.UrlMap.redirect_http_response_code', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _URLMAP_REDIRECTHTTPRESPONSECODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='handler_type', full_name='google.appengine.v1.UrlMap.handler_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=635,
  serialized_end=1352,
)


_STATICFILESHANDLER_HTTPHEADERSENTRY = _descriptor.Descriptor(
  name='HttpHeadersEntry',
  full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='google.appengine.v1.StaticFilesHandler.HttpHeadersEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1625,
  serialized_end=1675,
)

_STATICFILESHANDLER = _descriptor.Descriptor(
  name='StaticFilesHandler',
  full_name='google.appengine.v1.StaticFilesHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.StaticFilesHandler.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upload_path_regex', full_name='google.appengine.v1.StaticFilesHandler.upload_path_regex', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='http_headers', full_name='google.appengine.v1.StaticFilesHandler.http_headers', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mime_type', full_name='google.appengine.v1.StaticFilesHandler.mime_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expiration', full_name='google.appengine.v1.StaticFilesHandler.expiration', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='require_matching_file', full_name='google.appengine.v1.StaticFilesHandler.require_matching_file', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='application_readable', full_name='google.appengine.v1.StaticFilesHandler.application_readable', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STATICFILESHANDLER_HTTPHEADERSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1355,
  serialized_end=1675,
)


_SCRIPTHANDLER = _descriptor.Descriptor(
  name='ScriptHandler',
  full_name='google.appengine.v1.ScriptHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script_path', full_name='google.appengine.v1.ScriptHandler.script_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1677,
  serialized_end=1713,
)


_APIENDPOINTHANDLER = _descriptor.Descriptor(
  name='ApiEndpointHandler',
  full_name='google.appengine.v1.ApiEndpointHandler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script_path', full_name='google.appengine.v1.ApiEndpointHandler.script_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1715,
  serialized_end=1756,
)


_HEALTHCHECK = _descriptor.Descriptor(
  name='HealthCheck',
  full_name='google.appengine.v1.HealthCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='disable_health_check', full_name='google.appengine.v1.HealthCheck.disable_health_check', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.HealthCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='healthy_threshold', full_name='google.appengine.v1.HealthCheck.healthy_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unhealthy_threshold', full_name='google.appengine.v1.HealthCheck.unhealthy_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='restart_threshold', full_name='google.appengine.v1.HealthCheck.restart_threshold', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.HealthCheck.check_interval', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.HealthCheck.timeout', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1759,
  serialized_end=1994,
)


_READINESSCHECK = _descriptor.Descriptor(
  name='ReadinessCheck',
  full_name='google.appengine.v1.ReadinessCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.ReadinessCheck.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.ReadinessCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failure_threshold', full_name='google.appengine.v1.ReadinessCheck.failure_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='success_threshold', full_name='google.appengine.v1.ReadinessCheck.success_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.ReadinessCheck.check_interval', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.ReadinessCheck.timeout', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='app_start_timeout', full_name='google.appengine.v1.ReadinessCheck.app_start_timeout', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1997,
  serialized_end=2244,
)


_LIVENESSCHECK = _descriptor.Descriptor(
  name='LivenessCheck',
  full_name='google.appengine.v1.LivenessCheck',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.LivenessCheck.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='host', full_name='google.appengine.v1.LivenessCheck.host', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failure_threshold', full_name='google.appengine.v1.LivenessCheck.failure_threshold', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='success_threshold', full_name='google.appengine.v1.LivenessCheck.success_threshold', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='check_interval', full_name='google.appengine.v1.LivenessCheck.check_interval', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='google.appengine.v1.LivenessCheck.timeout', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_delay', full_name='google.appengine.v1.LivenessCheck.initial_delay', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2247,
  serialized_end=2489,
)


_LIBRARY = _descriptor.Descriptor(
  name='Library',
  full_name='google.appengine.v1.Library',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='google.appengine.v1.Library.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='google.appengine.v1.Library.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2491,
  serialized_end=2531,
)

_APICONFIGHANDLER.fields_by_name['auth_fail_action'].enum_type = _AUTHFAILACTION
_APICONFIGHANDLER.fields_by_name['login'].enum_type = _LOGINREQUIREMENT
_APICONFIGHANDLER.fields_by_name['security_level'].enum_type = _SECURITYLEVEL
_ERRORHANDLER.fields_by_name['error_code'].enum_type = _ERRORHANDLER_ERRORCODE
_ERRORHANDLER_ERRORCODE.containing_type = _ERRORHANDLER
_URLMAP.fields_by_name['static_files'].message_type = _STATICFILESHANDLER
_URLMAP.fields_by_name['script'].message_type = _SCRIPTHANDLER
_URLMAP.fields_by_name['api_endpoint'].message_type = _APIENDPOINTHANDLER
_URLMAP.fields_by_name['security_level'].enum_type = _SECURITYLEVEL
_URLMAP.fields_by_name['login'].enum_type = _LOGINREQUIREMENT
_URLMAP.fields_by_name['auth_fail_action'].enum_type = _AUTHFAILACTION
_URLMAP.fields_by_name['redirect_http_response_code'].enum_type = _URLMAP_REDIRECTHTTPRESPONSECODE
_URLMAP_REDIRECTHTTPRESPONSECODE.containing_type = _URLMAP
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['static_files'])
_URLMAP.fields_by_name['static_files'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['script'])
_URLMAP.fields_by_name['script'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_URLMAP.oneofs_by_name['handler_type'].fields.append(
  _URLMAP.fields_by_name['api_endpoint'])
_URLMAP.fields_by_name['api_endpoint'].containing_oneof = _URLMAP.oneofs_by_name['handler_type']
_STATICFILESHANDLER_HTTPHEADERSENTRY.containing_type = _STATICFILESHANDLER
_STATICFILESHANDLER.fields_by_name['http_headers'].message_type = _STATICFILESHANDLER_HTTPHEADERSENTRY
_STATICFILESHANDLER.fields_by_name['expiration'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_HEALTHCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_HEALTHCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_READINESSCHECK.fields_by_name['app_start_timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['check_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LIVENESSCHECK.fields_by_name['initial_delay'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['ApiConfigHandler'] = _APICONFIGHANDLER
DESCRIPTOR.message_types_by_name['ErrorHandler'] = _ERRORHANDLER
DESCRIPTOR.message_types_by_name['UrlMap'] = _URLMAP
DESCRIPTOR.message_types_by_name['StaticFilesHandler'] = _STATICFILESHANDLER
DESCRIPTOR.message_types_by_name['ScriptHandler'] = _SCRIPTHANDLER
DESCRIPTOR.message_types_by_name['ApiEndpointHandler'] = _APIENDPOINTHANDLER
DESCRIPTOR.message_types_by_name['HealthCheck'] = _HEALTHCHECK
DESCRIPTOR.message_types_by_name['ReadinessCheck'] = _READINESSCHECK
DESCRIPTOR.message_types_by_name['LivenessCheck'] = _LIVENESSCHECK
DESCRIPTOR.message_types_by_name['Library'] = _LIBRARY
DESCRIPTOR.enum_types_by_name['AuthFailAction'] = _AUTHFAILACTION
DESCRIPTOR.enum_types_by_name['LoginRequirement'] = _LOGINREQUIREMENT
DESCRIPTOR.enum_types_by_name['SecurityLevel'] = _SECURITYLEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ApiConfigHandler = _reflection.GeneratedProtocolMessageType('ApiConfigHandler', (_message.Message,), {
  'DESCRIPTOR' : _APICONFIGHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """\ `Google Cloud Endpoints
  <https://cloud.google.com/appengine/docs/python/endpoints/>`__
  configuration for API handlers.
  
  Attributes:
      auth_fail_action:
          Action to take when users access resources that require
          authentication. Defaults to ``redirect``.
      login:
          Level of login required to access this resource. Defaults to
          ``optional``.
      script:
          Path to the script from the application root directory.
      security_level:
          Security (HTTPS) enforcement for this URL.
      url:
          URL to serve the endpoint at.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ApiConfigHandler)
  })
_sym_db.RegisterMessage(ApiConfigHandler)

ErrorHandler = _reflection.GeneratedProtocolMessageType('ErrorHandler', (_message.Message,), {
  'DESCRIPTOR' : _ERRORHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Custom static error page to be served when an error occurs.
  
  Attributes:
      error_code:
          Error condition this handler applies to.
      static_file:
          Static file content to be served for this error.
      mime_type:
          MIME type of file. Defaults to ``text/html``.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ErrorHandler)
  })
_sym_db.RegisterMessage(ErrorHandler)

UrlMap = _reflection.GeneratedProtocolMessageType('UrlMap', (_message.Message,), {
  'DESCRIPTOR' : _URLMAP,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """URL pattern and description of how the URL should be handled. App
  Engine can handle URLs by executing application code or by serving
  static files uploaded with the version, such as images, CSS, or
  JavaScript.
  
  Attributes:
      url_regex:
          URL prefix. Uses regular expression syntax, which means regexp
          special characters must be escaped, but should not contain
          groupings. All URLs that begin with this prefix are handled by
          this handler, using the portion of the URL after the prefix as
          part of the file path.
      handler_type:
          Type of handler for this URL pattern.
      static_files:
          Returns the contents of a file, such as an image, as the
          response.
      script:
          Executes a script to handle the requests that match this URL
          pattern. Only the ``auto`` value is supported for Node.js in
          the App Engine standard environment, for example ``"script":
          "auto"``.
      api_endpoint:
          Uses API Endpoints to handle requests.
      security_level:
          Security (HTTPS) enforcement for this URL.
      login:
          Level of login required to access this resource. Not supported
          for Node.js in the App Engine standard environment.
      auth_fail_action:
          Action to take when users access resources that require
          authentication. Defaults to ``redirect``.
      redirect_http_response_code:
          \ ``30x`` code to use when performing redirects for the
          ``secure`` field. Defaults to ``302``.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.UrlMap)
  })
_sym_db.RegisterMessage(UrlMap)

StaticFilesHandler = _reflection.GeneratedProtocolMessageType('StaticFilesHandler', (_message.Message,), {

  'HttpHeadersEntry' : _reflection.GeneratedProtocolMessageType('HttpHeadersEntry', (_message.Message,), {
    'DESCRIPTOR' : _STATICFILESHANDLER_HTTPHEADERSENTRY,
    '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
    # @@protoc_insertion_point(class_scope:google.appengine.v1.StaticFilesHandler.HttpHeadersEntry)
    })
  ,
  'DESCRIPTOR' : _STATICFILESHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Files served directly to the user for a given URL, such as images, CSS
  stylesheets, or JavaScript source files. Static file handlers describe
  which files in the application directory are static files, and which
  URLs serve them.
  
  Attributes:
      path:
          Path to the static files matched by the URL pattern, from the
          application root directory. The path can refer to text matched
          in groupings in the URL pattern.
      upload_path_regex:
          Regular expression that matches the file paths for all files
          that should be referenced by this handler.
      http_headers:
          HTTP headers to use for all responses from these URLs.
      mime_type:
          MIME type used to serve all files served by this handler.
          Defaults to file-specific MIME types, which are derived from
          each file’s filename extension.
      expiration:
          Time a static file served by this handler should be cached by
          web proxies and browsers.
      require_matching_file:
          Whether this handler should match the request if the file
          referenced by the handler does not exist.
      application_readable:
          Whether files should also be uploaded as code data. By
          default, files declared in static file handlers are uploaded
          as static data and are only served to end users; they cannot
          be read by the application. If enabled, uploads are charged
          against both your code and static data storage resource
          quotas.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.StaticFilesHandler)
  })
_sym_db.RegisterMessage(StaticFilesHandler)
_sym_db.RegisterMessage(StaticFilesHandler.HttpHeadersEntry)

ScriptHandler = _reflection.GeneratedProtocolMessageType('ScriptHandler', (_message.Message,), {
  'DESCRIPTOR' : _SCRIPTHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Executes a script to handle the request that matches the URL pattern.
  
  Attributes:
      script_path:
          Path to the script from the application root directory.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ScriptHandler)
  })
_sym_db.RegisterMessage(ScriptHandler)

ApiEndpointHandler = _reflection.GeneratedProtocolMessageType('ApiEndpointHandler', (_message.Message,), {
  'DESCRIPTOR' : _APIENDPOINTHANDLER,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Uses Google Cloud Endpoints to handle requests.
  
  Attributes:
      script_path:
          Path to the script from the application root directory.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ApiEndpointHandler)
  })
_sym_db.RegisterMessage(ApiEndpointHandler)

HealthCheck = _reflection.GeneratedProtocolMessageType('HealthCheck', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Health checking configuration for VM instances. Unhealthy instances
  are killed and replaced with new instances. Only applicable for
  instances in App Engine flexible environment.
  
  Attributes:
      disable_health_check:
          Whether to explicitly disable health checks for this instance.
      host:
          Host header to send when performing an HTTP health check.
          Example: “myapp.appspot.com”
      healthy_threshold:
          Number of consecutive successful health checks required before
          receiving traffic.
      unhealthy_threshold:
          Number of consecutive failed health checks required before
          removing traffic.
      restart_threshold:
          Number of consecutive failed health checks required before an
          instance is restarted.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the health check is considered failed.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.HealthCheck)
  })
_sym_db.RegisterMessage(HealthCheck)

ReadinessCheck = _reflection.GeneratedProtocolMessageType('ReadinessCheck', (_message.Message,), {
  'DESCRIPTOR' : _READINESSCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Readiness checking configuration for VM instances. Unhealthy instances
  are removed from traffic rotation.
  
  Attributes:
      path:
          The request path.
      host:
          Host header to send when performing a HTTP Readiness check.
          Example: “myapp.appspot.com”
      failure_threshold:
          Number of consecutive failed checks required before removing
          traffic.
      success_threshold:
          Number of consecutive successful checks required before
          receiving traffic.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the check is considered failed.
      app_start_timeout:
          A maximum time limit on application initialization, measured
          from moment the application successfully replies to a
          healthcheck until it is ready to serve traffic.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.ReadinessCheck)
  })
_sym_db.RegisterMessage(ReadinessCheck)

LivenessCheck = _reflection.GeneratedProtocolMessageType('LivenessCheck', (_message.Message,), {
  'DESCRIPTOR' : _LIVENESSCHECK,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Health checking configuration for VM instances. Unhealthy instances
  are killed and replaced with new instances.
  
  Attributes:
      path:
          The request path.
      host:
          Host header to send when performing a HTTP Liveness check.
          Example: “myapp.appspot.com”
      failure_threshold:
          Number of consecutive failed checks required before
          considering the VM unhealthy.
      success_threshold:
          Number of consecutive successful checks required before
          considering the VM healthy.
      check_interval:
          Interval between health checks.
      timeout:
          Time before the check is considered failed.
      initial_delay:
          The initial delay before starting to execute the checks.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.LivenessCheck)
  })
_sym_db.RegisterMessage(LivenessCheck)

Library = _reflection.GeneratedProtocolMessageType('Library', (_message.Message,), {
  'DESCRIPTOR' : _LIBRARY,
  '__module__' : 'google.cloud.appengine_v1.proto.app_yaml_pb2'
  ,
  '__doc__': """Third-party Python runtime library that is required by the
  application.
  
  Attributes:
      name:
          Name of the library. Example: “django”.
      version:
          Version of the library to select, or “latest”.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.Library)
  })
_sym_db.RegisterMessage(Library)


DESCRIPTOR._options = None
_SECURITYLEVEL._options = None
_ERRORHANDLER_ERRORCODE._options = None
_STATICFILESHANDLER_HTTPHEADERSENTRY._options = None
# @@protoc_insertion_point(module_scope)
