# -*- coding: utf-8 -*- #
"""Cloud SDK static completion CLI tree."""
# pylint: disable=line-too-long,bad-continuation
STATIC_COMPLETION_CLI_TREE = {
  "commands": {
    "access-approval": {
      "commands": {
        "requests": {
          "commands": {
            "approve": {
              "commands": {},
              "flags": {}
            },
            "dismiss": {
              "commands": {},
              "flags": {}
            },
            "get": {
              "commands": {},
              "flags": {}
            },
            "invalidate": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--state": "value"
              }
            }
          },
          "flags": {}
        },
        "service-account": {
          "commands": {
            "get": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "settings": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "get": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--active_key_version": "value",
                "--enrolled_services": "value",
                "--folder": "value",
                "--notification_emails": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "access-context-manager": {
      "commands": {
        "authorized-orgs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--asset_type": [
                  "asset-type-credential-strength",
                  "asset-type-device",
                  "asset-type-unspecified"
                ],
                "--async": "bool",
                "--authorization_direction": [
                  "authorization-direction-from",
                  "authorization-direction-to",
                  "authorization-direction-unspecified"
                ],
                "--authorization_type": [
                  "authorization-type-trust",
                  "authorization-type-unspecified"
                ],
                "--orgs": "value",
                "--policy": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--policy": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--policy": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--policy": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-orgs": "value",
                "--clear-orgs": "bool",
                "--policy": "value",
                "--remove-orgs": "value",
                "--set-orgs": "value"
              }
            }
          },
          "flags": {}
        },
        "cloud-bindings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--group-key": "value",
                "--level": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--binding": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--binding": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--binding": "value",
                "--level": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "levels": {
          "commands": {
            "conditions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--level": "value",
                    "--policy": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--basic-level-spec": "value",
                "--combine-function": [
                  "and",
                  "or"
                ],
                "--custom-level-spec": "value",
                "--description": "value",
                "--policy": "value",
                "--title": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--policy": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--policy": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--policy": "value",
                "--sort-by": "value"
              }
            },
            "replace-all": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--source-file": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--basic-level-spec": "value",
                "--combine-function": [
                  "and",
                  "or"
                ],
                "--custom-level-spec": "value",
                "--description": "value",
                "--policy": "value",
                "--title": "value"
              }
            }
          },
          "flags": {}
        },
        "perimeters": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--access-levels": "value",
                "--async": "bool",
                "--description": "value",
                "--egress-policies": "value",
                "--enable-vpc-accessible-services": "bool",
                "--ingress-policies": "value",
                "--perimeter-type": [
                  "bridge",
                  "regular"
                ],
                "--policy": "value",
                "--resources": "value",
                "--restricted-services": "value",
                "--title": "value",
                "--vpc-allowed-services": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--policy": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--policy": "value"
              }
            },
            "dry-run": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--access-levels": "value",
                    "--async": "bool",
                    "--egress-policies": "value",
                    "--enable-vpc-accessible-services": "bool",
                    "--ingress-policies": "value",
                    "--perimeter-access-levels": "value",
                    "--perimeter-description": "value",
                    "--perimeter-egress-policies": "value",
                    "--perimeter-enable-vpc-accessible-services": "bool",
                    "--perimeter-ingress-policies": "value",
                    "--perimeter-resources": "value",
                    "--perimeter-restricted-services": "value",
                    "--perimeter-title": "value",
                    "--perimeter-type": "value",
                    "--perimeter-vpc-allowed-services": "value",
                    "--policy": "value",
                    "--resources": "value",
                    "--restricted-services": "value",
                    "--vpc-allowed-services": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "drop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "enforce": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "enforce-all": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--policy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-access-levels": "value",
                    "--add-resources": "value",
                    "--add-restricted-services": "value",
                    "--add-vpc-allowed-services": "value",
                    "--async": "bool",
                    "--clear-access-levels": "bool",
                    "--clear-egress-policies": "bool",
                    "--clear-ingress-policies": "bool",
                    "--clear-resources": "bool",
                    "--clear-restricted-services": "bool",
                    "--clear-vpc-allowed-services": "bool",
                    "--enable-vpc-accessible-services": "bool",
                    "--policy": "value",
                    "--remove-access-levels": "value",
                    "--remove-resources": "value",
                    "--remove-restricted-services": "value",
                    "--remove-vpc-allowed-services": "value",
                    "--set-egress-policies": "value",
                    "--set-ingress-policies": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--policy": "value",
                "--sort-by": "value"
              }
            },
            "replace-all": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--source-file": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-access-levels": "value",
                "--add-resources": "value",
                "--add-restricted-services": "value",
                "--add-vpc-allowed-services": "value",
                "--clear-access-levels": "bool",
                "--clear-egress-policies": "bool",
                "--clear-ingress-policies": "bool",
                "--clear-resources": "bool",
                "--clear-restricted-services": "bool",
                "--clear-vpc-allowed-services": "bool",
                "--description": "value",
                "--enable-vpc-accessible-services": "bool",
                "--policy": "value",
                "--remove-access-levels": "value",
                "--remove-resources": "value",
                "--remove-restricted-services": "value",
                "--remove-vpc-allowed-services": "value",
                "--set-access-levels": "value",
                "--set-egress-policies": "value",
                "--set-ingress-policies": "value",
                "--set-resources": "value",
                "--set-restricted-services": "value",
                "--title": "value",
                "--type": [
                  "bridge",
                  "regular"
                ]
              }
            }
          },
          "flags": {}
        },
        "policies": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--organization": "value",
                "--scopes": "value",
                "--title": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--title": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "active-directory": {
      "commands": {
        "domains": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--domain": "value",
                    "--labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--domain": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--domain": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--domain": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--domain": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--admin-name": "value",
                "--async": "bool",
                "--authorized-networks": "value",
                "--enable-audit-logs": "bool",
                "--labels": "value",
                "--region": "value",
                "--reserved-ip-range": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "describe-ldaps-settings": {
              "commands": {},
              "flags": {}
            },
            "extend-schema": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--ldif-file": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "reset-admin-password": {
              "commands": {},
              "flags": {}
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backup": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "trusts": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--direction": [
                      "bidirectional",
                      "inbound",
                      "outbound",
                      "trust-direction-unspecified"
                    ],
                    "--handshake-secret": "value",
                    "--selective-authentication": "bool",
                    "--target-dns-ip-addresses": "value",
                    "--target-domain-name": "value",
                    "--type": [
                      "external",
                      "forest",
                      "trust-type-unspecified"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--target-domain-name": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--target-dns-ip-addresses": "value",
                    "--target-domain-name": "value"
                  }
                },
                "validate-state": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--target-domain-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-authorized-networks": "value",
                "--add-region": "value",
                "--async": "bool",
                "--clear-labels": "bool",
                "--enable-audit-logs": "bool",
                "--remove-authorized-networks": "value",
                "--remove-labels": "value",
                "--remove-region": "value",
                "--update-labels": "value"
              }
            },
            "update-ldaps-settings": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--certificate-password": "value",
                "--certificate-pfx-file": "value",
                "--clear-certificates": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "peerings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--authorized-network": "value",
                "--domain": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "ai": {
      "commands": {
        "custom-jobs": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--args": "value",
                "--command": "value",
                "--config": "value",
                "--display-name": "value",
                "--enable-dashboard-access": "bool",
                "--enable-web-access": "bool",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--network": "value",
                "--python-package-uris": "value",
                "--region": "value",
                "--service-account": "value",
                "--worker-pool-spec": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "local-run": {
              "commands": {},
              "flags": {
                "--executor-image-uri": "value",
                "--extra-dirs": "value",
                "--extra-packages": "value",
                "--gpu": "bool",
                "--local-package-path": "value",
                "--output-image-uri": "value",
                "--python-module": "value",
                "--requirements": "value",
                "--script": "value",
                "--service-account-key-file": "value"
              }
            },
            "stream-logs": {
              "commands": {},
              "flags": {
                "--allow-multiline-logs": "bool",
                "--polling-interval": "value",
                "--region": "dynamic",
                "--task-name": "value"
              }
            }
          },
          "flags": {}
        },
        "endpoints": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--encryption-kms-key-name": "value",
                "--endpoint-id": "value",
                "--labels": "value",
                "--network": "value",
                "--region": "value",
                "--request-response-logging-rate": "value",
                "--request-response-logging-table": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "deploy-model": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--autoscaling-metric-specs": "value",
                "--deployed-model-id": "value",
                "--disable-container-logging": "bool",
                "--display-name": "value",
                "--enable-access-logging": "bool",
                "--machine-type": "value",
                "--max-replica-count": "value",
                "--min-replica-count": "value",
                "--model": "value",
                "--region": "dynamic",
                "--service-account": "value",
                "--traffic-split": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "explain": {
              "commands": {},
              "flags": {
                "--deployed-model-id": "value",
                "--json-request": "value",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "predict": {
              "commands": {},
              "flags": {
                "--json-request": "value",
                "--region": "dynamic"
              }
            },
            "raw-predict": {
              "commands": {},
              "flags": {
                "--http-headers": "value",
                "--region": "dynamic",
                "--request": "value"
              }
            },
            "undeploy-model": {
              "commands": {},
              "flags": {
                "--deployed-model-id": "value",
                "--region": "dynamic",
                "--traffic-split": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--clear-traffic-split": "bool",
                "--description": "value",
                "--disable-request-response-logging": "bool",
                "--display-name": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--request-response-logging-rate": "value",
                "--request-response-logging-table": "value",
                "--traffic-split": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "hp-tuning-jobs": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--algorithm": [
                  "algorithm-unspecified",
                  "grid-search",
                  "random-search"
                ],
                "--config": "value",
                "--display-name": "value",
                "--enable-dashboard-access": "bool",
                "--enable-web-access": "bool",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--max-trial-count": "value",
                "--network": "value",
                "--parallel-trial-count": "value",
                "--region": "value",
                "--service-account": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "stream-logs": {
              "commands": {},
              "flags": {
                "--allow-multiline-logs": "bool",
                "--polling-interval": "value",
                "--region": "dynamic",
                "--task-name": "value"
              }
            }
          },
          "flags": {}
        },
        "index-endpoints": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--network": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "deploy-index": {
              "commands": {},
              "flags": {
                "--deployed-index-id": "value",
                "--display-name": "value",
                "--index": "value",
                "--max-replica-count": "value",
                "--min-replica-count": "value",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "mutate-deployed-index": {
              "commands": {},
              "flags": {
                "--deployed-index-id": "value",
                "--max-replica-count": "value",
                "--min-replica-count": "value",
                "--region": "dynamic"
              }
            },
            "undeploy-index": {
              "commands": {},
              "flags": {
                "--deployed-index-id": "value",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--display-name": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "indexes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--metadata-file": "value",
                "--metadata-schema-uri": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--display-name": "value",
                "--metadata-file": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "model-monitoring-jobs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--analysis-instance-schema": "value",
                "--anomaly-cloud-logging": "bool",
                "--bigquery-uri": "value",
                "--data-format": "value",
                "--dataset": "value",
                "--display-name": "value",
                "--emails": "value",
                "--endpoint": "value",
                "--feature-attribution-thresholds": "value",
                "--feature-thresholds": "value",
                "--gcs-uris": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--log-ttl": "value",
                "--monitoring-config-from-file": "value",
                "--monitoring-frequency": "value",
                "--predict-instance-schema": "value",
                "--prediction-sampling-rate": "value",
                "--region": "value",
                "--sample-predict-request": "value",
                "--target-field": "value",
                "--training-sampling-rate": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "pause": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "resume": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--analysis-instance-schema": "value",
                "--anomaly-cloud-logging": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--emails": "value",
                "--feature-attribution-thresholds": "value",
                "--feature-thresholds": "value",
                "--log-ttl": "value",
                "--monitoring-config-from-file": "value",
                "--monitoring-frequency": "value",
                "--prediction-sampling-rate": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "models": {
          "commands": {
            "copy": {
              "commands": {},
              "flags": {
                "--destination-model-id": "value",
                "--destination-parent-model": "value",
                "--kms-key-name": "value",
                "--region": "value",
                "--source-model": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "delete-version": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-version": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "upload": {
              "commands": {},
              "flags": {
                "--artifact-uri": "value",
                "--container-args": "value",
                "--container-command": "value",
                "--container-env-vars": "value",
                "--container-health-route": "value",
                "--container-image-uri": "value",
                "--container-ports": "value",
                "--container-predict-route": "value",
                "--description": "value",
                "--display-name": "value",
                "--explanation-metadata-file": "value",
                "--explanation-method": "value",
                "--explanation-path-count": "value",
                "--explanation-step-count": "value",
                "--labels": "value",
                "--model-id": "value",
                "--parent-model": "value",
                "--region": "value",
                "--smooth-grad-noise-sigma": "value",
                "--smooth-grad-noise-sigma-by-feature": "value",
                "--smooth-grad-noisy-sample-count": "value",
                "--version-aliases": "value",
                "--version-description": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--index": "value",
                "--index-endpoint": "value",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "tensorboards": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--display-name": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "ai-platform": {
      "commands": {
        "jobs": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {
                "--summarize": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "stream-logs": {
              "commands": {},
              "flags": {
                "--allow-multiline-logs": "bool",
                "--polling-interval": "value",
                "--task-name": "value"
              }
            },
            "submit": {
              "commands": {
                "prediction": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--data-format": [
                      "text",
                      "tf-record",
                      "tf-record-gzip"
                    ],
                    "--input-paths": "value",
                    "--labels": "value",
                    "--max-worker-count": "value",
                    "--model": "value",
                    "--model-dir": "value",
                    "--output-path": "value",
                    "--region": "value",
                    "--runtime-version": "value",
                    "--signature-name": "value"
                  }
                },
                "training": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--config": "value",
                    "--enable-web-access": "bool",
                    "--job-dir": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-image-uri": "value",
                    "--master-machine-type": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--packages": "value",
                    "--parameter-server-accelerator": "value",
                    "--parameter-server-count": "value",
                    "--parameter-server-image-uri": "value",
                    "--parameter-server-machine-type": "value",
                    "--python-version": "value",
                    "--region": "dynamic",
                    "--runtime-version": "value",
                    "--scale-tier": [
                      "basic",
                      "basic-gpu",
                      "basic-tpu",
                      "custom",
                      "premium-1",
                      "standard-1"
                    ],
                    "--service-account": "value",
                    "--staging-bucket": "value",
                    "--stream-logs": "bool",
                    "--use-chief-in-tf-config": "value",
                    "--worker-accelerator": "value",
                    "--worker-count": "value",
                    "--worker-image-uri": "value",
                    "--worker-machine-type": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "local": {
          "commands": {
            "predict": {
              "commands": {},
              "flags": {
                "--framework": [
                  "scikit-learn",
                  "tensorflow",
                  "xgboost"
                ],
                "--json-instances": "value",
                "--json-request": "value",
                "--model-dir": "value",
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "train": {
              "commands": {},
              "flags": {
                "--distributed": "bool",
                "--evaluator-count": "value",
                "--job-dir": "value",
                "--module-name": "value",
                "--package-path": "value",
                "--parameter-server-count": "value",
                "--start-port": "value",
                "--worker-count": "value"
              }
            }
          },
          "flags": {}
        },
        "models": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--enable-logging": "bool",
                "--labels": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--regions": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            }
          },
          "flags": {}
        },
        "predict": {
          "commands": {},
          "flags": {
            "--json-instances": "value",
            "--json-request": "value",
            "--model": "value",
            "--region": [
              "asia-east1",
              "asia-northeast1",
              "asia-southeast1",
              "australia-southeast1",
              "europe-west1",
              "europe-west2",
              "europe-west3",
              "europe-west4",
              "global",
              "northamerica-northeast1",
              "us-central1",
              "us-east1",
              "us-east4",
              "us-west1"
            ],
            "--signature-name": "value",
            "--text-instances": "value"
          }
        },
        "versions": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--async": "bool",
                "--config": "value",
                "--description": "value",
                "--framework": [
                  "scikit-learn",
                  "tensorflow",
                  "xgboost"
                ],
                "--labels": "value",
                "--machine-type": "value",
                "--max-nodes": "value",
                "--metric-targets": "value",
                "--min-nodes": "value",
                "--model": "value",
                "--origin": "value",
                "--python-version": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--runtime-version": "value",
                "--staging-bucket": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--model": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-default": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--config": "value",
                "--description": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "alloydb": {
      "commands": {
        "backups": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "clusters": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--automated-backup-days-of-week": "value",
                "--automated-backup-encryption-key": "dynamic",
                "--automated-backup-encryption-key-keyring": "dynamic",
                "--automated-backup-encryption-key-location": "dynamic",
                "--automated-backup-encryption-key-project": "dynamic",
                "--automated-backup-retention-count": "value",
                "--automated-backup-retention-period": "value",
                "--automated-backup-start-times": "value",
                "--automated-backup-window": "value",
                "--disable-automated-backup": "bool",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--network": "value",
                "--password": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backup": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--network": "value",
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--automated-backup-days-of-week": "value",
                "--automated-backup-encryption-key": "dynamic",
                "--automated-backup-encryption-key-keyring": "dynamic",
                "--automated-backup-encryption-key-location": "dynamic",
                "--automated-backup-encryption-key-project": "dynamic",
                "--automated-backup-retention-count": "value",
                "--automated-backup-retention-period": "value",
                "--automated-backup-start-times": "value",
                "--automated-backup-window": "value",
                "--clear-automated-backup": "bool",
                "--disable-automated-backup": "bool",
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--availability-type": [
                  "REGIONAL"
                ],
                "--cluster": "value",
                "--cpu-count": [
                  2,
                  4,
                  8,
                  16,
                  32,
                  64
                ],
                "--database-flags": "value",
                "--insights-config-query-plans-per-minute": "value",
                "--insights-config-query-string-length": "value",
                "--insights-config-record-application-tags": "bool",
                "--insights-config-record-client-address": "bool",
                "--instance-type": [
                  "PRIMARY",
                  "READ_POOL"
                ],
                "--read-pool-node-count": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--region": "value",
                "--view": [
                  "basic",
                  "full"
                ]
              }
            },
            "failover": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "restart": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--availability-type": [
                  "REGIONAL"
                ],
                "--cluster": "value",
                "--cpu-count": [
                  2,
                  4,
                  8,
                  16,
                  32,
                  64
                ],
                "--database-flags": "value",
                "--insights-config-query-plans-per-minute": "value",
                "--insights-config-query-string-length": "value",
                "--insights-config-record-application-tags": "bool",
                "--insights-config-record-client-address": "bool",
                "--read-pool-node-count": "value",
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "alpha": {
      "commands": {
        "access-approval": {
          "commands": {
            "requests": {
              "commands": {
                "approve": {
                  "commands": {},
                  "flags": {}
                },
                "dismiss": {
                  "commands": {},
                  "flags": {}
                },
                "get": {
                  "commands": {},
                  "flags": {}
                },
                "invalidate": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--state": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "get": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--active_key_version": "value",
                    "--enrolled_services": "value",
                    "--folder": "value",
                    "--notification_emails": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "access-context-manager": {
          "commands": {
            "authorized-orgs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--asset_type": [
                      "asset-type-credential-strength",
                      "asset-type-device",
                      "asset-type-unspecified"
                    ],
                    "--async": "bool",
                    "--authorization_direction": [
                      "authorization-direction-from",
                      "authorization-direction-to",
                      "authorization-direction-unspecified"
                    ],
                    "--authorization_type": [
                      "authorization-type-trust",
                      "authorization-type-unspecified"
                    ],
                    "--orgs": "value",
                    "--policy": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-orgs": "value",
                    "--clear-orgs": "bool",
                    "--policy": "value",
                    "--remove-orgs": "value",
                    "--set-orgs": "value"
                  }
                }
              },
              "flags": {}
            },
            "cloud-bindings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--dry-run-level": "value",
                    "--group-key": "value",
                    "--level": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--binding": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--binding": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--binding": "value",
                    "--dry-run-level": "value",
                    "--level": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "levels": {
              "commands": {
                "conditions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--level": "value",
                        "--policy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--policy": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--basic-level-spec": "value",
                    "--combine-function": [
                      "and",
                      "or"
                    ],
                    "--custom-level-spec": "value",
                    "--description": "value",
                    "--policy": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "replace-all": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--source-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--basic-level-spec": "value",
                    "--combine-function": [
                      "and",
                      "or"
                    ],
                    "--custom-level-spec": "value",
                    "--description": "value",
                    "--policy": "value",
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            },
            "perimeters": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--policy": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--access-levels": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--egress-policies": "value",
                    "--enable-vpc-accessible-services": "bool",
                    "--ingress-policies": "value",
                    "--perimeter-type": [
                      "bridge",
                      "regular"
                    ],
                    "--policy": "value",
                    "--resources": "value",
                    "--restricted-services": "value",
                    "--title": "value",
                    "--vpc-allowed-services": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "dry-run": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--access-levels": "value",
                        "--async": "bool",
                        "--egress-policies": "value",
                        "--enable-vpc-accessible-services": "bool",
                        "--ingress-policies": "value",
                        "--perimeter-access-levels": "value",
                        "--perimeter-description": "value",
                        "--perimeter-egress-policies": "value",
                        "--perimeter-enable-vpc-accessible-services": "bool",
                        "--perimeter-ingress-policies": "value",
                        "--perimeter-resources": "value",
                        "--perimeter-restricted-services": "value",
                        "--perimeter-title": "value",
                        "--perimeter-type": "value",
                        "--perimeter-vpc-allowed-services": "value",
                        "--policy": "value",
                        "--resources": "value",
                        "--restricted-services": "value",
                        "--vpc-allowed-services": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--policy": "value"
                      }
                    },
                    "drop": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "enforce": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "enforce-all": {
                      "commands": {},
                      "flags": {
                        "--etag": "value",
                        "--policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--policy": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-access-levels": "value",
                        "--add-resources": "value",
                        "--add-restricted-services": "value",
                        "--add-vpc-allowed-services": "value",
                        "--async": "bool",
                        "--clear-access-levels": "bool",
                        "--clear-egress-policies": "bool",
                        "--clear-ingress-policies": "bool",
                        "--clear-resources": "bool",
                        "--clear-restricted-services": "bool",
                        "--clear-vpc-allowed-services": "bool",
                        "--enable-vpc-accessible-services": "bool",
                        "--policy": "value",
                        "--remove-access-levels": "value",
                        "--remove-resources": "value",
                        "--remove-restricted-services": "value",
                        "--remove-vpc-allowed-services": "value",
                        "--set-egress-policies": "value",
                        "--set-ingress-policies": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "replace-all": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--source-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-access-levels": "value",
                    "--add-resources": "value",
                    "--add-restricted-services": "value",
                    "--add-vpc-allowed-services": "value",
                    "--clear-access-levels": "bool",
                    "--clear-egress-policies": "bool",
                    "--clear-ingress-policies": "bool",
                    "--clear-resources": "bool",
                    "--clear-restricted-services": "bool",
                    "--clear-vpc-allowed-services": "bool",
                    "--description": "value",
                    "--enable-vpc-accessible-services": "bool",
                    "--policy": "value",
                    "--remove-access-levels": "value",
                    "--remove-resources": "value",
                    "--remove-restricted-services": "value",
                    "--remove-vpc-allowed-services": "value",
                    "--set-access-levels": "value",
                    "--set-egress-policies": "value",
                    "--set-ingress-policies": "value",
                    "--set-resources": "value",
                    "--set-restricted-services": "value",
                    "--title": "value",
                    "--type": [
                      "bridge",
                      "regular"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "policies": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--organization": "value",
                    "--scopes": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "active-directory": {
          "commands": {
            "domains": {
              "commands": {
                "backups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--domain": "value",
                        "--labels": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--domain": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--domain": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--domain": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--domain": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-name": "value",
                    "--async": "bool",
                    "--authorized-networks": "value",
                    "--enable-audit-logs": "bool",
                    "--labels": "value",
                    "--region": "value",
                    "--reserved-ip-range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "describe-ldaps-settings": {
                  "commands": {},
                  "flags": {}
                },
                "extend-schema": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--ldif-file": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "migration": {
                  "commands": {
                    "check-permissions": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--disable-sid-filtering-domains": "value",
                        "--onprem-domains": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "reset-admin-password": {
                  "commands": {},
                  "flags": {}
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "sql-integrations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--domain": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--domain": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "trusts": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--direction": [
                          "bidirectional",
                          "inbound",
                          "outbound",
                          "trust-direction-unspecified"
                        ],
                        "--handshake-secret": "value",
                        "--selective-authentication": "bool",
                        "--target-dns-ip-addresses": "value",
                        "--target-domain-name": "value",
                        "--type": [
                          "external",
                          "forest",
                          "trust-type-unspecified"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-domain-name": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-dns-ip-addresses": "value",
                        "--target-domain-name": "value"
                      }
                    },
                    "validate-state": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-domain-name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-authorized-networks": "value",
                    "--add-region": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--enable-audit-logs": "bool",
                    "--remove-authorized-networks": "value",
                    "--remove-labels": "value",
                    "--remove-region": "value",
                    "--update-labels": "value"
                  }
                },
                "update-ldaps-settings": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-password": "value",
                    "--certificate-pfx-file": "value",
                    "--clear-certificates": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "peerings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorized-network": "value",
                    "--domain": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ai": {
          "commands": {
            "custom-jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--command": "value",
                    "--config": "value",
                    "--display-name": "value",
                    "--enable-dashboard-access": "bool",
                    "--enable-web-access": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--network": "value",
                    "--python-package-uris": "value",
                    "--region": "value",
                    "--service-account": "value",
                    "--worker-pool-spec": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "local-run": {
                  "commands": {},
                  "flags": {
                    "--executor-image-uri": "value",
                    "--extra-dirs": "value",
                    "--extra-packages": "value",
                    "--gpu": "bool",
                    "--local-package-path": "value",
                    "--output-image-uri": "value",
                    "--python-module": "value",
                    "--requirements": "value",
                    "--script": "value",
                    "--service-account-key-file": "value"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--region": "dynamic",
                    "--task-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--encryption-kms-key-name": "value",
                    "--endpoint-id": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--region": "value",
                    "--request-response-logging-rate": "value",
                    "--request-response-logging-table": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "deploy-model": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--autoscaling-metric-specs": "value",
                    "--deployed-model-id": "value",
                    "--display-name": "value",
                    "--enable-access-logging": "bool",
                    "--enable-container-logging": "bool",
                    "--machine-type": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--model": "value",
                    "--region": "dynamic",
                    "--service-account": "value",
                    "--shared-resources": "dynamic",
                    "--shared-resources-region": "dynamic",
                    "--traffic-split": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "explain": {
                  "commands": {},
                  "flags": {
                    "--deployed-model-id": "value",
                    "--json-request": "value",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "predict": {
                  "commands": {},
                  "flags": {
                    "--json-request": "value",
                    "--region": "dynamic"
                  }
                },
                "raw-predict": {
                  "commands": {},
                  "flags": {
                    "--http-headers": "value",
                    "--region": "dynamic",
                    "--request": "value"
                  }
                },
                "undeploy-model": {
                  "commands": {},
                  "flags": {
                    "--deployed-model-id": "value",
                    "--region": "dynamic",
                    "--traffic-split": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-traffic-split": "bool",
                    "--description": "value",
                    "--disable-request-response-logging": "bool",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--request-response-logging-rate": "value",
                    "--request-response-logging-table": "value",
                    "--traffic-split": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "hp-tuning-jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--algorithm": [
                      "algorithm-unspecified",
                      "grid-search",
                      "random-search"
                    ],
                    "--config": "value",
                    "--display-name": "value",
                    "--enable-dashboard-access": "bool",
                    "--enable-web-access": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--max-trial-count": "value",
                    "--network": "value",
                    "--parallel-trial-count": "value",
                    "--region": "value",
                    "--service-account": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--region": "dynamic",
                    "--task-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "index-endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "deploy-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--display-name": "value",
                    "--index": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mutate-deployed-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--region": "dynamic"
                  }
                },
                "undeploy-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "indexes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--metadata-file": "value",
                    "--metadata-schema-uri": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-datapoints": {
                  "commands": {},
                  "flags": {
                    "--datapoint-ids": "value",
                    "--datapoints-from-file": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--metadata-file": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                },
                "upsert-datapoints": {
                  "commands": {},
                  "flags": {
                    "--datapoints-from-file": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "model-monitoring-jobs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--analysis-instance-schema": "value",
                    "--anomaly-cloud-logging": "bool",
                    "--bigquery-uri": "value",
                    "--data-format": "value",
                    "--dataset": "value",
                    "--display-name": "value",
                    "--emails": "value",
                    "--endpoint": "value",
                    "--feature-attribution-thresholds": "value",
                    "--feature-thresholds": "value",
                    "--gcs-uris": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--log-ttl": "value",
                    "--monitoring-config-from-file": "value",
                    "--monitoring-frequency": "value",
                    "--predict-instance-schema": "value",
                    "--prediction-sampling-rate": "value",
                    "--region": "value",
                    "--sample-predict-request": "value",
                    "--target-field": "value",
                    "--training-sampling-rate": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--analysis-instance-schema": "value",
                    "--anomaly-cloud-logging": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--emails": "value",
                    "--feature-attribution-thresholds": "value",
                    "--feature-thresholds": "value",
                    "--log-ttl": "value",
                    "--monitoring-config-from-file": "value",
                    "--monitoring-frequency": "value",
                    "--prediction-sampling-rate": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "copy": {
                  "commands": {},
                  "flags": {
                    "--destination-model-id": "value",
                    "--destination-parent-model": "value",
                    "--kms-key-name": "value",
                    "--region": "value",
                    "--source-model": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "delete-version": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-version": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--artifact-uri": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-env-vars": "value",
                    "--container-health-route": "value",
                    "--container-image-uri": "value",
                    "--container-ports": "value",
                    "--container-predict-route": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--explanation-metadata-file": "value",
                    "--explanation-method": "value",
                    "--explanation-modality": "value",
                    "--explanation-nearest-neighbor-search-config-file": "value",
                    "--explanation-neighbor-count": "value",
                    "--explanation-path-count": "value",
                    "--explanation-query": "value",
                    "--explanation-step-count": "value",
                    "--labels": "value",
                    "--model-id": "value",
                    "--parent-model": "value",
                    "--region": "value",
                    "--smooth-grad-noise-sigma": "value",
                    "--smooth-grad-noise-sigma-by-feature": "value",
                    "--smooth-grad-noisy-sample-count": "value",
                    "--uris": "value",
                    "--version-aliases": "value",
                    "--version-description": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--index": "value",
                    "--index-endpoint": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-experiments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--tensorboard-id": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-runs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-time-series": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--plugin-data": "value",
                    "--plugin-name": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--type": [
                      "blob-sequence",
                      "scalar",
                      "tensor"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic",
                    "--uri": "bool"
                  }
                },
                "read": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--max-data-points": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--plugin-data": "value",
                    "--plugin-name": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tensorboards": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ai-platform": {
          "commands": {
            "explain": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--text-instances": "value"
              }
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--summarize": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--task-name": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "prediction": {
                      "commands": {},
                      "flags": {
                        "--accelerator-count": "value",
                        "--accelerator-type": [
                          "nvidia-tesla-k80",
                          "nvidia-tesla-p100"
                        ],
                        "--batch-size": "value",
                        "--data-format": [
                          "text",
                          "tf-record",
                          "tf-record-gzip"
                        ],
                        "--input-paths": "value",
                        "--labels": "value",
                        "--max-worker-count": "value",
                        "--model": "value",
                        "--model-dir": "value",
                        "--output-path": "value",
                        "--region": "value",
                        "--runtime-version": "value",
                        "--signature-name": "value"
                      }
                    },
                    "training": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config": "value",
                        "--enable-web-access": "bool",
                        "--job-dir": "value",
                        "--kms-key": "dynamic",
                        "--kms-keyring": "dynamic",
                        "--kms-location": "dynamic",
                        "--kms-project": "dynamic",
                        "--labels": "value",
                        "--master-accelerator": "value",
                        "--master-image-uri": "value",
                        "--master-machine-type": "value",
                        "--module-name": "value",
                        "--network": "value",
                        "--package-path": "value",
                        "--packages": "value",
                        "--parameter-server-accelerator": "value",
                        "--parameter-server-count": "value",
                        "--parameter-server-image-uri": "value",
                        "--parameter-server-machine-type": "value",
                        "--python-version": "value",
                        "--region": "dynamic",
                        "--runtime-version": "value",
                        "--scale-tier": [
                          "basic",
                          "basic-gpu",
                          "basic-tpu",
                          "custom",
                          "premium-1",
                          "standard-1"
                        ],
                        "--service-account": "value",
                        "--staging-bucket": "value",
                        "--stream-logs": "bool",
                        "--tpu-tf-version": "value",
                        "--use-chief-in-tf-config": "value",
                        "--worker-accelerator": "value",
                        "--worker-count": "value",
                        "--worker-image-uri": "value",
                        "--worker-machine-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "local": {
              "commands": {
                "predict": {
                  "commands": {},
                  "flags": {
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--json-instances": "value",
                    "--json-request": "value",
                    "--model-dir": "value",
                    "--signature-name": "value",
                    "--text-instances": "value"
                  }
                },
                "train": {
                  "commands": {},
                  "flags": {
                    "--distributed": "bool",
                    "--evaluator-count": "value",
                    "--job-dir": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--parameter-server-count": "value",
                    "--start-port": "value",
                    "--worker-count": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-console-logging": "bool",
                    "--enable-logging": "bool",
                    "--labels": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--regions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "predict": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "versions": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--command": "value",
                    "--config": "value",
                    "--description": "value",
                    "--env-vars": "value",
                    "--explanation-method": [
                      "integrated-gradients",
                      "sampled-shapley",
                      "xrai"
                    ],
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--health-route": "value",
                    "--image": "value",
                    "--labels": "value",
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--metric-targets": "value",
                    "--min-nodes": "value",
                    "--model": "value",
                    "--num-integral-steps": "value",
                    "--num-paths": "value",
                    "--origin": "value",
                    "--package-uris": "value",
                    "--ports": "value",
                    "--predict-route": "value",
                    "--prediction-class": "value",
                    "--python-version": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--runtime-version": "value",
                    "--service-account": "value",
                    "--staging-bucket": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--model": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-default": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bigquery-table-name": "value",
                    "--clear-labels": "bool",
                    "--config": "value",
                    "--description": "value",
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--sampling-percentage": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "alloydb": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automated-backup-days-of-week": "value",
                    "--automated-backup-encryption-key": "dynamic",
                    "--automated-backup-encryption-key-keyring": "dynamic",
                    "--automated-backup-encryption-key-location": "dynamic",
                    "--automated-backup-encryption-key-project": "dynamic",
                    "--automated-backup-retention-count": "value",
                    "--automated-backup-retention-period": "value",
                    "--automated-backup-start-times": "value",
                    "--automated-backup-window": "value",
                    "--continuous-backup-encryption-key": "dynamic",
                    "--continuous-backup-encryption-key-keyring": "dynamic",
                    "--continuous-backup-encryption-key-location": "dynamic",
                    "--continuous-backup-encryption-key-project": "dynamic",
                    "--continuous-backup-recovery-window-days": "value",
                    "--disable-automated-backup": "bool",
                    "--enable-continuous-backup": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--network": "value",
                    "--password": "value",
                    "--region": "value"
                  }
                },
                "create-secondary": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--primary-cluster": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "promote": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--network": "value",
                    "--point-in-time": "value",
                    "--region": "value",
                    "--source-cluster": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automated-backup-days-of-week": "value",
                    "--automated-backup-encryption-key": "dynamic",
                    "--automated-backup-encryption-key-keyring": "dynamic",
                    "--automated-backup-encryption-key-location": "dynamic",
                    "--automated-backup-encryption-key-project": "dynamic",
                    "--automated-backup-retention-count": "value",
                    "--automated-backup-retention-period": "value",
                    "--automated-backup-start-times": "value",
                    "--automated-backup-window": "value",
                    "--clear-automated-backup": "bool",
                    "--clear-continuous-backup-encryption-key": "bool",
                    "--continuous-backup-encryption-key": "dynamic",
                    "--continuous-backup-encryption-key-keyring": "dynamic",
                    "--continuous-backup-encryption-key-location": "dynamic",
                    "--continuous-backup-encryption-key-project": "dynamic",
                    "--continuous-backup-recovery-window-days": "value",
                    "--disable-automated-backup": "bool",
                    "--enable-continuous-backup": "bool",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--availability-type": [
                      "REGIONAL"
                    ],
                    "--cluster": "value",
                    "--cpu-count": [
                      2,
                      4,
                      8,
                      16,
                      32,
                      64
                    ],
                    "--database-flags": "value",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--instance-type": [
                      "PRIMARY",
                      "READ_POOL"
                    ],
                    "--read-pool-node-count": "value",
                    "--region": "value"
                  }
                },
                "create-secondary": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--availability-type": [
                      "REGIONAL"
                    ],
                    "--cluster": "value",
                    "--cpu-count": [
                      2,
                      4,
                      8,
                      16,
                      32,
                      64
                    ],
                    "--database-flags": "value",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--read-pool-node-count": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "anthos": {
          "commands": {
            "apply": {
              "commands": {},
              "flags": {}
            },
            "auth": {
              "commands": {
                "login": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--dry-run": "bool",
                    "--kubeconfig": "value",
                    "--login-config": "value",
                    "--login-config-cert": "value",
                    "--set-preferred-auth": "bool",
                    "--user": "value"
                  }
                }
              },
              "flags": {}
            },
            "config": {
              "commands": {
                "controller": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cluster-ipv4-cidr-block": "value",
                        "--cluster-named-range": "value",
                        "--experimental-features": "value",
                        "--full-management": "bool",
                        "--location": "dynamic",
                        "--man-block": "value",
                        "--man-blocks": "value",
                        "--master-ipv4-cidr-block": "value",
                        "--network": "value",
                        "--services-ipv4-cidr-block": "value",
                        "--services-named-range": "value",
                        "--subnet": "value",
                        "--use-private-endpoint": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-config-connector-identity": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--full-name": "bool",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--man-block": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "sync": {
                  "commands": {
                    "repo": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--cluster": "value",
                            "--managed-resources": "value",
                            "--source": "value",
                            "--sync-name": "value",
                            "--sync-namespace": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--membership": "value",
                            "--namespace": "value",
                            "--selector": "value",
                            "--status": "value",
                            "--targets": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "resources": {
                      "commands": {
                        "list": {
                          "commands": {},
                          "flags": {
                            "--cluster": "value",
                            "--membership": "value",
                            "--sync-name": "value",
                            "--sync-namespace": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "create-login-config": {
              "commands": {},
              "flags": {
                "--kubeconfig": "value",
                "--merge-from": "value",
                "--output": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--output-directory": "value"
              }
            }
          },
          "flags": {}
        },
        "api-gateway": {
          "commands": {
            "api-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool",
                    "--backend-auth-service-account": "value",
                    "--display-name": "value",
                    "--grpc-files": "value",
                    "--labels": "value",
                    "--openapi-spec": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--view": [
                      "BASIC",
                      "FULL"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "apis": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--managed-service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "gateways": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--api-config": "value",
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--api-config": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "apigee": {
          "commands": {
            "apis": {
              "commands": {
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value",
                    "--override": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--revision": "value",
                    "--verbose": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "undeploy": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "applications": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--developer": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "archives": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bundle-file": "value",
                    "--environment": "value",
                    "--labels": "value",
                    "--organization": "value",
                    "--source": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--environment": "value",
                    "--organization": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "deployments": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--revision": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "developers": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "environments": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "organizations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "provision": {
                  "commands": {},
                  "flags": {
                    "--analytics-region": "value",
                    "--async": "bool",
                    "--authorized-network": "value",
                    "--runtime-location": "value"
                  }
                }
              },
              "flags": {}
            },
            "products": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--all-environments": "bool",
                    "--all-proxies": "bool",
                    "--apis": "value",
                    "--attributes": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--environments": "value",
                    "--internal-access": "bool",
                    "--manual-approval": "bool",
                    "--oauth-scopes": "value",
                    "--organization": "value",
                    "--private-access": "bool",
                    "--public-access": "bool",
                    "--quota": "value",
                    "--quota-interval": "value",
                    "--quota-unit": [
                      "day",
                      "hour",
                      "minute",
                      "month"
                    ],
                    "--resources": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-api": "value",
                    "--add-attribute": "value",
                    "--add-environment": "value",
                    "--add-oauth-scope": "value",
                    "--add-resource": "value",
                    "--all-apis": "bool",
                    "--all-environments": "bool",
                    "--all-resources": "bool",
                    "--automatic-approval": "bool",
                    "--clear-attributes": "bool",
                    "--clear-description": "bool",
                    "--clear-oauth-scopes": "bool",
                    "--clear-quota": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--internal-access": "bool",
                    "--manual-approval": "bool",
                    "--organization": "value",
                    "--private-access": "bool",
                    "--public-access": "bool",
                    "--quota": "value",
                    "--quota-interval": "value",
                    "--quota-unit": [
                      "day",
                      "hour",
                      "minute",
                      "month"
                    ],
                    "--remove-api": "value",
                    "--remove-attribute": "value",
                    "--remove-environment": "value",
                    "--remove-oauth-scope": "value",
                    "--remove-resource": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "app": {
          "commands": {
            "domain-mappings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate-id": "value",
                    "--certificate-management": [
                      "automatic",
                      "manual"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate-id": "value",
                    "--certificate-management": [
                      "automatic",
                      "manual"
                    ],
                    "--no-certificate-id": "bool"
                  }
                }
              },
              "flags": {}
            },
            "ssl-certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--display-name": "value",
                    "--private-key": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--display-name": "value",
                    "--private-key": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--service-account": "value",
                "--split-health-checks": "bool"
              }
            }
          },
          "flags": {}
        },
        "artifacts": {
          "commands": {
            "apt": {
              "commands": {
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--gcs-source": "value",
                    "--location": "value"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            },
            "docker": {
              "commands": {
                "images": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--delete-tags": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--metadata-filter": "value",
                        "--show-all-metadata": "bool",
                        "--show-build-details": "bool",
                        "--show-deployment": "bool",
                        "--show-image-basis": "bool",
                        "--show-package-vulnerability": "bool",
                        "--show-provenance": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--include-tags": "bool",
                        "--limit": "value",
                        "--occurrence-filter": "value",
                        "--page-size": "value",
                        "--show-occurrences": "bool",
                        "--show-occurrences-from": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "tags": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "files": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value",
                    "--tag": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "packages": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "print-settings": {
              "commands": {
                "apt": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "gradle": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "mvn": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "npm": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value",
                    "--scope": "value"
                  }
                },
                "python": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "yum": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--repository": "value"
                  }
                }
              },
              "flags": {}
            },
            "repositories": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--location": "value",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--allow-snapshot-overwrites": "bool",
                    "--async": "bool",
                    "--description": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--repository-format": "value",
                    "--version-policy": [
                      "none",
                      "release",
                      "snapshot"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable-upgrade-redirection": {
                  "commands": {},
                  "flags": {}
                },
                "enable-upgrade-redirection": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                }
              },
              "flags": {}
            },
            "versions": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--delete-tags": "bool",
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpcsc-config": {
              "commands": {
                "allow": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "deny": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "yum": {
              "commands": {
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--gcs-source": "value",
                    "--location": "value"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "asset": {
          "commands": {
            "analyze-org-policies": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "analyze-org-policy-governed-containers": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "analyze-org-policy-governed-resources": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--asset-types": "value",
                "--bigquery-dataset": "dynamic",
                "--bigquery-table": "dynamic",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--folder": "value",
                "--organization": "value",
                "--output-bigquery-force": "bool",
                "--output-path": "value",
                "--output-path-prefix": "value",
                "--partition-key": [
                  "read-time",
                  "request-time"
                ],
                "--per-asset-type": "bool",
                "--relationship-types": "value",
                "--snapshot-time": "value"
              }
            },
            "feeds": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--asset-names": "value",
                    "--asset-types": "value",
                    "--condition-description": "value",
                    "--condition-expression": "value",
                    "--condition-title": "value",
                    "--content-type": [
                      "access-policy",
                      "iam-policy",
                      "org-policy",
                      "os-inventory",
                      "relationship",
                      "resource"
                    ],
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value",
                    "--relationship-types": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-asset-names": "value",
                    "--add-asset-types": "value",
                    "--add-relationship-types": "value",
                    "--clear-asset-names": "bool",
                    "--clear-asset-types": "bool",
                    "--clear-condition-description": "bool",
                    "--clear-condition-expression": "bool",
                    "--clear-condition-title": "bool",
                    "--clear-content-type": "bool",
                    "--clear-relationship-types": "bool",
                    "--condition-description": "value",
                    "--condition-expression": "value",
                    "--condition-title": "value",
                    "--content-type": [
                      "access-policy",
                      "iam-policy",
                      "org-policy",
                      "os-inventory",
                      "relationship",
                      "resource"
                    ],
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value",
                    "--remove-asset-names": "value",
                    "--remove-asset-types": "value",
                    "--remove-relationship-types": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-history": {
              "commands": {},
              "flags": {
                "--asset-names": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--end-time": "value",
                "--organization": "value",
                "--relationship-types": "value",
                "--start-time": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--asset-types": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--relationship-types": "value",
                "--snapshot-time": "value",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "query": {
              "commands": {},
              "flags": {
                "--bigquery-dataset": "dynamic",
                "--bigquery-table": "dynamic",
                "--end-time": "value",
                "--folder": "value",
                "--job-reference": "value",
                "--organization": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--snapshot-time": "value",
                "--start-time": "value",
                "--statement": "value",
                "--timeout": "value",
                "--write-disposition": [
                  "write-append",
                  "write-empty",
                  "write-truncate"
                ]
              }
            },
            "saved-queries": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--folder": "value",
                    "--labels": "value",
                    "--organization": "value",
                    "--query-file-path": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--query-file-path": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "assured": {
          "commands": {
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "workloads": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--compliance-regime": [
                      "ASSURED_WORKLOADS_FOR_PARTNERS",
                      "CA_REGIONS_AND_SUPPORT",
                      "CJIS",
                      "EU_REGIONS_AND_SUPPORT",
                      "FEDRAMP_HIGH",
                      "FEDRAMP_MODERATE",
                      "HIPAA",
                      "HITRUST",
                      "IL4",
                      "ITAR",
                      "US_REGIONAL_ACCESS"
                    ],
                    "--display-name": "value",
                    "--enable-sovereign-controls": "value",
                    "--external-identifier": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--next-rotation-time": "value",
                    "--organization": "value",
                    "--partner": [
                      "LOCAL_CONTROLS_BY_S3NS",
                      "SOVEREIGN_CONTROLS_BY_T_SYSTEMS"
                    ],
                    "--provisioned-resources-parent": "value",
                    "--resource-settings": "value",
                    "--rotation-period": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "violations": {
                  "commands": {
                    "acknowledge": {
                      "commands": {},
                      "flags": {
                        "--comment": "value",
                        "--location": "value",
                        "--organization": "value",
                        "--workload": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--organization": "value",
                        "--workload": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--workload": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "auth": {
          "commands": {
            "activate-service-account": {
              "commands": {},
              "flags": {
                "--key-file": "value",
                "--password-file": "value",
                "--prompt-for-password": "bool"
              }
            },
            "application-default": {
              "commands": {
                "login": {
                  "commands": {},
                  "flags": {
                    "--client-id-file": "value",
                    "--disable-quota-project": "bool",
                    "--launch-browser": "bool",
                    "--login-config": "value",
                    "--no-browser": "bool",
                    "--scopes": "value"
                  }
                },
                "print-access-token": {
                  "commands": {},
                  "flags": {
                    "--scopes": "value"
                  }
                },
                "revoke": {
                  "commands": {},
                  "flags": {}
                },
                "set-quota-project": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "configure-docker": {
              "commands": {},
              "flags": {}
            },
            "enterprise-certificate-config": {
              "commands": {
                "create": {
                  "commands": {
                    "linux": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--label": "value",
                        "--module": "value",
                        "--output-file": "value",
                        "--slot": "value",
                        "--tls-offload": "value",
                        "--user-pin": "value"
                      }
                    },
                    "macos": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--issuer": "value",
                        "--output-file": "value",
                        "--tls-offload": "value"
                      }
                    },
                    "windows": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--issuer": "value",
                        "--output-file": "value",
                        "--provider": "value",
                        "--store": "value",
                        "--tls-offload": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--filter-account": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "login": {
              "commands": {},
              "flags": {
                "--activate": "bool",
                "--brief": "bool",
                "--cred-file": "value",
                "--enable-gdrive-access": "bool",
                "--force": "bool",
                "--launch-browser": "bool",
                "--login-config": "value",
                "--no-browser": "bool",
                "--update-adc": "bool"
              }
            },
            "print-access-token": {
              "commands": {},
              "flags": {}
            },
            "print-identity-token": {
              "commands": {},
              "flags": {
                "--audiences": "value",
                "--include-email": "bool",
                "--include-license": "bool",
                "--token-format": [
                  "full",
                  "standard"
                ]
              }
            },
            "revoke": {
              "commands": {},
              "flags": {
                "--all": "bool"
              }
            }
          },
          "flags": {}
        },
        "batch": {
          "commands": {
            "jobs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "submit": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--container-commands-file": "value",
                    "--container-entrypoint": "value",
                    "--container-image-uri": "value",
                    "--location": "value",
                    "--machine-type": "value",
                    "--network": "value",
                    "--priority": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--script-file-path": "value",
                    "--script-text": "value",
                    "--subnetwork": "value"
                  }
                }
              },
              "flags": {}
            },
            "tasks": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--job": "value",
                    "--location": "value",
                    "--task_group": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--job": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "beyondcorp": {
          "commands": {
            "app": {
              "commands": {
                "connections": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--application-endpoint": "value",
                        "--application-endpoint-host": "value",
                        "--application-endpoint-port": "value",
                        "--async": "bool",
                        "--connectors": "value",
                        "--display-name": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--type": [
                          "tcp"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--application-endpoint": "value",
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--connectors": "value",
                        "--display-name": "value",
                        "--location": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "connectors": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--display-name": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--member": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--details": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--display-name": "value",
                        "--location": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "legacy": {
                  "commands": {
                    "connections": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--application-endpoint": "value",
                            "--application-endpoint-host": "value",
                            "--application-endpoint-port": "value",
                            "--async": "bool",
                            "--connectors": "value",
                            "--display-name": "value",
                            "--labels": "value",
                            "--location": "dynamic",
                            "--type": [
                              "tcp"
                            ]
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--location": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "dynamic",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--application-endpoint": "value",
                            "--async": "bool",
                            "--clear-labels": "bool",
                            "--connectors": "value",
                            "--display-name": "value",
                            "--location": "dynamic",
                            "--remove-labels": "value",
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "connectors": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--display-name": "value",
                            "--labels": "value",
                            "--location": "dynamic",
                            "--member": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--location": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--details": "bool",
                            "--location": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "dynamic",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--clear-labels": "bool",
                            "--display-name": "value",
                            "--location": "dynamic",
                            "--remove-labels": "value",
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "client-connector": {
              "commands": {
                "gateways": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--client-connector-service": "value",
                        "--location": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "services": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config-from-file": "value",
                        "--display-name": "value",
                        "--egress-peered-vpc": "value",
                        "--ingress-config": "value",
                        "--location": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config-from-file": "value",
                        "--display-name": "value",
                        "--ingress-config": "value",
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "bigtable": {
          "commands": {
            "app-profiles": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--instance": "value",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--force": "bool",
                    "--instance": "dynamic",
                    "--restrict-to": "value",
                    "--route-any": "bool",
                    "--route-to": "dynamic",
                    "--transactional-writes": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--force": "bool",
                    "--instance": "dynamic",
                    "--restrict-to": "value",
                    "--route-any": "bool",
                    "--route-to": "dynamic",
                    "--transactional-writes": "bool"
                  }
                }
              },
              "flags": {}
            },
            "backups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "copy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-backup": "dynamic",
                    "--destination-cluster": "dynamic",
                    "--destination-instance": "dynamic",
                    "--destination-project": "dynamic",
                    "--expiration-date": "value",
                    "--retention-period": "value",
                    "--source-backup": "dynamic",
                    "--source-cluster": "dynamic",
                    "--source-instance": "dynamic",
                    "--source-project": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "dynamic",
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value",
                    "--table": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--cluster": "dynamic",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-cpu-target": "value",
                    "--autoscaling-max-nodes": "value",
                    "--autoscaling-min-nodes": "value",
                    "--autoscaling-storage-target": "value",
                    "--instance": "dynamic",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--num-nodes": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instances": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-cpu-target": "value",
                    "--autoscaling-max-nodes": "value",
                    "--autoscaling-min-nodes": "value",
                    "--autoscaling-storage-target": "value",
                    "--disable-autoscaling": "bool",
                    "--instance": "dynamic",
                    "--num-nodes": "value"
                  }
                }
              },
              "flags": {}
            },
            "hot-tablets": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--end-time": "value",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--start-time": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "dynamic",
                    "--cluster-config": "value",
                    "--cluster-num-nodes": "value",
                    "--cluster-storage-type": [
                      "hdd",
                      "ssd"
                    ],
                    "--cluster-zone": "value",
                    "--display-name": "value",
                    "--instance-type": [
                      "DEVELOPMENT",
                      "PRODUCTION"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "tables": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--instance": "dynamic",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "config": {
                      "commands": {
                        "export": {
                          "commands": {},
                          "flags": {
                            "--all": "bool",
                            "--instance": "dynamic",
                            "--path": "value",
                            "--resource-format": [
                              "krm",
                              "terraform"
                            ]
                          }
                        }
                      },
                      "flags": {}
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--change-stream-retention-period": "value",
                        "--column-families": "value",
                        "--deletion-protection": "bool",
                        "--instance": "dynamic",
                        "--splits": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic",
                        "--view": [
                          "encryption",
                          "full",
                          "name",
                          "replication",
                          "schema",
                          "stats"
                        ]
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instances": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--instance": "dynamic",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "restore": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--destination": "dynamic",
                        "--destination-instance": "dynamic",
                        "--source": "dynamic",
                        "--source-cluster": "dynamic",
                        "--source-instance": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deletion-protection": "bool",
                        "--instance": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "billing": {
          "commands": {
            "accounts": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "projects": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "link": {
                      "commands": {},
                      "flags": {
                        "--account-id": "dynamic",
                        "--billing-account": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--billing-account": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "unlink": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "budgets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--all-updates-rule-monitoring-notification-channels": "value",
                    "--all-updates-rule-pubsub-topic": "value",
                    "--billing-account": "value",
                    "--budget-amount": "value",
                    "--calendar-period": [
                      "calendar-period-unspecified",
                      "month",
                      "quarter",
                      "year"
                    ],
                    "--credit-types-treatment": [
                      "credit-types-treatment-unspecified",
                      "exclude-all-credits",
                      "include-all-credits",
                      "include-specified-credits"
                    ],
                    "--disable-default-iam-recipients": "bool",
                    "--display-name": "value",
                    "--end-date": "value",
                    "--filter-credit-types": "value",
                    "--filter-labels": "value",
                    "--filter-projects": "value",
                    "--filter-services": "value",
                    "--filter-subaccounts": "value",
                    "--last-period-amount": "bool",
                    "--start-date": "value",
                    "--threshold-rule": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-threshold-rule": "value",
                    "--all-updates-rule-monitoring-notification-channels": "value",
                    "--all-updates-rule-pubsub-topic": "value",
                    "--billing-account": "value",
                    "--budget-amount": "value",
                    "--calendar-period": [
                      "calendar-period-unspecified",
                      "month",
                      "quarter",
                      "year"
                    ],
                    "--clear-threshold-rules": "bool",
                    "--credit-types-treatment": [
                      "credit-types-treatment-unspecified",
                      "exclude-all-credits",
                      "include-all-credits",
                      "include-specified-credits"
                    ],
                    "--disable-default-iam-recipients": "bool",
                    "--display-name": "value",
                    "--end-date": "value",
                    "--filter-credit-types": "value",
                    "--filter-labels": "value",
                    "--filter-projects": "value",
                    "--filter-services": "value",
                    "--filter-subaccounts": "value",
                    "--last-period-amount": "bool",
                    "--start-date": "value",
                    "--threshold-rules-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "projects": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "link": {
                  "commands": {},
                  "flags": {
                    "--account-id": "dynamic",
                    "--billing-account": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "unlink": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "blockchain-node-engine": {
          "commands": {
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "nodes": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "bms": {
          "commands": {
            "instances": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "disable-serial-console": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "enable-serial-console": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "reset": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--enable-hyperthreading": "bool",
                    "--os-image": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "networks": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "list-ip-reservations": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-ip-range-reservation": "value",
                    "--async": "bool",
                    "--clear-ip-range-reservations": "bool",
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-ip-range-reservation": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "nfs-shares": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--allowed-client": "value",
                    "--async": "bool",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--size-gib": "value",
                    "--storage-type": [
                      "HDD",
                      "SSD"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-allowed-client": "value",
                    "--async": "bool",
                    "--clear-allowed-clients": "bool",
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-allowed-client": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "serial-console-ssh-keys": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value"
                  }
                },
                "remove": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ssh-keys": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value"
                  }
                },
                "remove": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "volumes": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "luns": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--volume": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "dynamic",
                        "--volume": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic",
                    "--snapshot": "value"
                  }
                },
                "snapshot": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--region": "dynamic",
                    "--snapshot-name": "value"
                  }
                },
                "snapshots": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--volume": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--volume": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "dynamic",
                        "--volume": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "bq": {
          "commands": {
            "datasets": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--overwrite": "bool",
                    "--permissions-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--remove-tables": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--permissions-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--show-all-users": "bool",
                    "--show-config": "bool",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "tables": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--dataset": "dynamic",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "copy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination": "dynamic",
                    "--destination-dataset": "dynamic",
                    "--job-id": "value",
                    "--overwrite": "bool",
                    "--source": "dynamic",
                    "--source-dataset": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "dynamic",
                    "--description": "value",
                    "--expiration": "value",
                    "--overwrite": "bool",
                    "--schema": "value",
                    "--schema-file": "value",
                    "--use-legacy-sql": "bool",
                    "--view": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "dynamic"
                  }
                },
                "insert": {
                  "commands": {},
                  "flags": {
                    "--data": "value",
                    "--dataset": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "show-rows": {
                  "commands": {},
                  "flags": {
                    "--dataset": "dynamic",
                    "--limit": "value",
                    "--start": "value",
                    "--table": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-columns": "value",
                    "--add-columns-file": "value",
                    "--dataset": "dynamic",
                    "--description": "value",
                    "--expiration": "value",
                    "--relax-columns": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "builds": {
          "commands": {
            "approve": {
              "commands": {},
              "flags": {
                "--comment": "value",
                "--url": "value"
              }
            },
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "connections": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {
                    "github": {
                      "commands": {},
                      "flags": {
                        "--app-installation-id": "value",
                        "--async": "bool",
                        "--authorizer-token-secret-version": "value",
                        "--region": "dynamic"
                      }
                    },
                    "github-enterprise": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--app-installation-id": "value",
                        "--app-slug": "value",
                        "--async": "bool",
                        "--host-uri": "value",
                        "--private-key-secret-version": "value",
                        "--region": "dynamic",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {
                    "github": {
                      "commands": {},
                      "flags": {
                        "--app-installation-id": "value",
                        "--async": "bool",
                        "--authorizer-token-secret-version": "value",
                        "--region": "dynamic"
                      }
                    },
                    "github-enterprise": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--app-installation-id": "value",
                        "--app-slug": "value",
                        "--async": "bool",
                        "--host-uri": "value",
                        "--private-key-secret-version": "value",
                        "--region": "dynamic",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "deploy": {
              "commands": {
                "configure": {
                  "commands": {
                    "gke": {
                      "commands": {},
                      "flags": {
                        "--app-name": "value",
                        "--branch-pattern": "value",
                        "--cluster": "value",
                        "--comment-control": "bool",
                        "--config": "value",
                        "--dockerfile": "value",
                        "--expose": "value",
                        "--gcs-config-staging-dir": "value",
                        "--location": "value",
                        "--namespace": "value",
                        "--preview-expiry": "value",
                        "--pull-request-pattern": "value",
                        "--pull-request-preview": "bool",
                        "--repo-name": "value",
                        "--repo-owner": "value",
                        "--repo-type": [
                          "bitbucket_mirrored",
                          "csr",
                          "github",
                          "github_mirrored"
                        ],
                        "--tag-pattern": "value",
                        "--timeout": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "gke": {
                  "commands": {},
                  "flags": {
                    "--app-name": "value",
                    "--app-version": "value",
                    "--async": "bool",
                    "--cluster": "value",
                    "--config": "value",
                    "--expose": "value",
                    "--gcs-staging-dir": "value",
                    "--image": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--no-source": "bool",
                    "--tag": "value",
                    "--timeout": "value"
                  }
                }
              },
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "enterprise-config": {
              "commands": {
                "bitbucketserver": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-access-token-secret-version": "value",
                        "--api-key": "value",
                        "--host-uri": "value",
                        "--name": "value",
                        "--peered-network": "value",
                        "--read-access-token-secret-version": "value",
                        "--region": "value",
                        "--ssl-ca-file": "value",
                        "--user-name": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--admin-access-token-secret-version": "value",
                        "--api-key": "value",
                        "--host-uri": "value",
                        "--read-access-token-secret-version": "value",
                        "--region": "value",
                        "--ssl-ca-file": "value",
                        "--user-name": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "github": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--gcs-bucket": "value",
                        "--gcs-object": "value",
                        "--generation": "value",
                        "--host-uri": "value",
                        "--oauth-client-id-name": "value",
                        "--oauth-client-id-version-name": "value",
                        "--oauth-secret-name": "value",
                        "--oauth-secret-version-name": "value",
                        "--peered-network": "value",
                        "--private-key-name": "value",
                        "--private-key-version-name": "value",
                        "--webhook-key": "value",
                        "--webhook-secret-name": "value",
                        "--webhook-secret-version-name": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--gcs-bucket": "value",
                        "--gcs-object": "value",
                        "--generation": "value",
                        "--host-uri": "value",
                        "--oauth-client-id-name": "value",
                        "--oauth-client-id-version-name": "value",
                        "--oauth-secret-name": "value",
                        "--oauth-secret-version-name": "value",
                        "--private-key-name": "value",
                        "--private-key-version-name": "value",
                        "--webhook-key": "value",
                        "--webhook-secret-name": "value",
                        "--webhook-secret-version-name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "gitlab": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--api-access-token-secret-version": "value",
                        "--api-key-secret-version": "value",
                        "--host-uri": "value",
                        "--name": "value",
                        "--read-access-token-secret-version": "value",
                        "--region": "value",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--user-name": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--api-access-token-secret-version": "value",
                        "--api-key-secret-version": "value",
                        "--host-uri": "value",
                        "--read-access-token-secret-version": "value",
                        "--region": "value",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--user-name": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--ongoing": "bool",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "log": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--stream": "bool"
              }
            },
            "reject": {
              "commands": {},
              "flags": {
                "--comment": "value",
                "--url": "value"
              }
            },
            "repositories": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--connection": "dynamic",
                    "--region": "dynamic",
                    "--remote-uri": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--connection": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--connection": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--connection": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "submit": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--config": "value",
                "--disk-size": "value",
                "--gcs-log-dir": "value",
                "--gcs-source-staging-dir": "value",
                "--ignore-file": "value",
                "--machine-type": [
                  "e2-highcpu-32",
                  "e2-highcpu-8",
                  "n1-highcpu-32",
                  "n1-highcpu-8"
                ],
                "--no-cache": "bool",
                "--no-source": "bool",
                "--pack": "value",
                "--region": "value",
                "--substitutions": "value",
                "--suppress-logs": "bool",
                "--tag": "value",
                "--timeout": "value",
                "--worker-pool": "value"
              }
            },
            "triggers": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {
                    "bitbucketserver": {
                      "commands": {},
                      "flags": {
                        "--bitbucket-server-config-resource": "value",
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--project-key": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--repo-slug": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "cloud-source-repositories": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "github": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": [
                          "COMMENTS_DISABLED",
                          "COMMENTS_ENABLED",
                          "COMMENTS_ENABLED_FOR_EXTERNAL_CONTRIBUTORS_ONLY"
                        ],
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--enterprise-config": "value",
                        "--ignored-files": "value",
                        "--include-logs-with-status": "bool",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--repo-name": "value",
                        "--repo-owner": "value",
                        "--repository": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "gitlab-enterprise": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--gitlab-config-resource": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--project-namespace": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "manual": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--subscription-filter": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--topic": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "webhook": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--secret": "value",
                        "--service-account": "value",
                        "--subscription-filter": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--trigger-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--branch": "value",
                    "--region": "value",
                    "--sha": "value",
                    "--tag": "value"
                  }
                }
              },
              "flags": {}
            },
            "worker-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--config-from-file": "value",
                    "--no-public-egress": "bool",
                    "--peered-network": "value",
                    "--peered-network-ip-range": "value",
                    "--region": "value",
                    "--worker-disk-size": "value",
                    "--worker-machine-type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config-from-file": "value",
                    "--no-public-egress": "bool",
                    "--public-egress": "bool",
                    "--region": "value",
                    "--worker-disk-size": "value",
                    "--worker-machine-type": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "certificate-manager": {
          "commands": {
            "certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-file": "value",
                    "--description": "value",
                    "--dns-authorizations": "value",
                    "--domains": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--private-key-file": "value",
                    "--scope": [
                      "default",
                      "edge-cache"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-file": "value",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "dynamic",
                    "--private-key-file": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "dns-authorizations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--domain": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "issuance-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--ca-pool": "value",
                    "--description": "value",
                    "--key-algorithm": [
                      "ecdsa-p256",
                      "rsa-2048"
                    ],
                    "--labels": "value",
                    "--lifetime": "value",
                    "--location": "value",
                    "--rotation-window-percentage": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "maps": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "entries": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--certificates": "dynamic",
                        "--description": "value",
                        "--hostname": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--set-primary": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--map": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--map": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--certificates": "dynamic",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "trust-configs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--etag": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "cloud-shell": {
          "commands": {
            "get-mount-command": {
              "commands": {},
              "flags": {
                "--force-key-file-overwrite": "bool",
                "--ssh-key-file": "bool"
              }
            },
            "scp": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--recurse": "bool",
                "--scp-flag": "value",
                "--ssh-key-file": "bool"
              }
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--authorize-session": "bool",
                "--command": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--ssh-flag": "value",
                "--ssh-key-file": "bool"
              }
            }
          },
          "flags": {}
        },
        "code": {
          "commands": {
            "clean-up": {
              "commands": {},
              "flags": {
                "--minikube-profile": "value"
              }
            },
            "dev": {
              "commands": {},
              "flags": {
                "--allow-secret-manager": "bool",
                "--application-default-credential": "bool",
                "--builder": "value",
                "--cloudsql-instances": "value",
                "--cpu": "value",
                "--dockerfile": "value",
                "--env-vars": "value",
                "--env-vars-file": "value",
                "--image": "value",
                "--kube-context": "value",
                "--local-port": "value",
                "--memory": "value",
                "--minikube-profile": "value",
                "--minikube-vm-driver": "value",
                "--namespace": "value",
                "--secrets": "value",
                "--service-account": "value",
                "--service-name": "value",
                "--source": "value",
                "--stop-cluster": "bool"
              }
            }
          },
          "flags": {}
        },
        "composer": {
          "commands": {
            "environments": {
              "commands": {
                "check-upgrade": {
                  "commands": {},
                  "flags": {
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--image-version": "value",
                    "--location": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--airflow-configs": "value",
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--cloud-sql-ipv4-cidr": "value",
                    "--cloud-sql-machine-type": "value",
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--composer-network-ipv4-cidr": "value",
                    "--connection-subnetwork": "value",
                    "--connection-type": [
                      "connection-type-unspecified",
                      "private-service-connect",
                      "vpc-peering"
                    ],
                    "--disk-size": "value",
                    "--enable-cloud-data-lineage-integration": "bool",
                    "--enable-ip-alias": "bool",
                    "--enable-ip-masq-agent": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-environment": "bool",
                    "--enable-privately-used-public-ips": "bool",
                    "--enable-scheduled-snapshot-creation": "bool",
                    "--enable-triggerer": "bool",
                    "--env-variables": "value",
                    "--environment-size": [
                      "large",
                      "medium",
                      "small",
                      "unspecified"
                    ],
                    "--image-version": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--location": "value",
                    "--machine-type": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--max-pods-per-node": "value",
                    "--max-workers": "value",
                    "--min-workers": "value",
                    "--network": "value",
                    "--node-count": "value",
                    "--oauth-scopes": "value",
                    "--python-version": [
                      "2",
                      "3"
                    ],
                    "--scheduler-count": "value",
                    "--scheduler-cpu": "value",
                    "--scheduler-memory": "value",
                    "--scheduler-storage": "value",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--snapshot-creation-schedule": "value",
                    "--snapshot-location": "value",
                    "--snapshot-schedule-timezone": "value",
                    "--subnetwork": "value",
                    "--tags": "value",
                    "--triggerer-count": "value",
                    "--triggerer-cpu": "value",
                    "--triggerer-memory": "value",
                    "--web-server-allow-all": "bool",
                    "--web-server-allow-ip": "value",
                    "--web-server-cpu": "value",
                    "--web-server-deny-all": "bool",
                    "--web-server-ipv4-cidr": "value",
                    "--web-server-machine-type": "value",
                    "--web-server-memory": "value",
                    "--web-server-storage": "value",
                    "--worker-cpu": "value",
                    "--worker-memory": "value",
                    "--worker-storage": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--locations": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-packages": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--tree": "bool"
                  }
                },
                "list-upgrades": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "restart-web-server": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "snapshots": {
                  "commands": {
                    "load": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--skip-airflow-overrides-setting": "bool",
                        "--skip-environment-variables-setting": "bool",
                        "--skip-gcs-data-copying": "bool",
                        "--skip-pypi-packages-installation": "bool",
                        "--snapshot-path": "value"
                      }
                    },
                    "save": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--snapshot-location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "storage": {
                  "commands": {
                    "dags": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "data": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "plugins": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--clear-airflow-configs": "bool",
                    "--clear-env-variables": "bool",
                    "--clear-labels": "bool",
                    "--clear-pypi-packages": "bool",
                    "--cloud-sql-machine-type": "value",
                    "--disable-cloud-data-lineage-integration": "bool",
                    "--disable-master-authorized-networks": "bool",
                    "--disable-scheduled-snapshot-creation": "bool",
                    "--disable-triggerer": "bool",
                    "--enable-cloud-data-lineage-integration": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-scheduled-snapshot-creation": "bool",
                    "--enable-triggerer": "bool",
                    "--environment-size": [
                      "large",
                      "medium",
                      "small",
                      "unspecified"
                    ],
                    "--image-version": "value",
                    "--location": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--max-workers": "value",
                    "--min-workers": "value",
                    "--node-count": "value",
                    "--remove-airflow-configs": "value",
                    "--remove-env-variables": "value",
                    "--remove-labels": "value",
                    "--remove-pypi-packages": "value",
                    "--scheduler-count": "value",
                    "--scheduler-cpu": "value",
                    "--scheduler-memory": "value",
                    "--scheduler-storage": "value",
                    "--snapshot-creation-schedule": "value",
                    "--snapshot-location": "value",
                    "--snapshot-schedule-timezone": "value",
                    "--triggerer-count": "value",
                    "--triggerer-cpu": "value",
                    "--triggerer-memory": "value",
                    "--update-airflow-configs": "value",
                    "--update-env-variables": "value",
                    "--update-labels": "value",
                    "--update-pypi-package": "value",
                    "--update-pypi-packages-from-file": "value",
                    "--update-web-server-allow-ip": "value",
                    "--web-server-allow-all": "bool",
                    "--web-server-cpu": "value",
                    "--web-server-deny-all": "bool",
                    "--web-server-machine-type": "value",
                    "--web-server-memory": "value",
                    "--web-server-storage": "value",
                    "--worker-cpu": "value",
                    "--worker-memory": "value",
                    "--worker-storage": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--locations": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "compute": {
          "commands": {
            "accelerator-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "addresses": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--addresses": "value",
                    "--description": "value",
                    "--endpoint-type": [
                      "NETLB",
                      "VM"
                    ],
                    "--global": "bool",
                    "--ip-version": [
                      "IPV4",
                      "IPV6"
                    ],
                    "--network": "value",
                    "--network-tier": "value",
                    "--prefix-length": "value",
                    "--purpose": [
                      "GCE_ENDPOINT",
                      "IPSEC_INTERCONNECT",
                      "PRIVATE_SERVICE_CONNECT",
                      "SHARED_LOADBALANCER_VIP",
                      "VPC_PEERING"
                    ],
                    "--region": "dynamic",
                    "--subnet": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--new-name": "value",
                    "--region": "dynamic",
                    "--target-project": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "backend-buckets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-file": "value",
                    "--key-name": "value"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--enable-cdn": "bool",
                    "--gcs-bucket-name": "dynamic",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--signed-url-cache-max-age": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "delete-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-name": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--edge-security-policy": "dynamic",
                    "--enable-cdn": "bool",
                    "--gcs-bucket-name": "dynamic",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--no-bypass-cache-on-request-headers": "bool",
                    "--no-client-ttl": "bool",
                    "--no-custom-response-headers": "bool",
                    "--no-default-ttl": "bool",
                    "--no-max-ttl": "bool",
                    "--no-negative-caching-policies": "bool",
                    "--no-serve-while-stale": "bool",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--signed-url-cache-max-age": "value"
                  }
                }
              },
              "flags": {}
            },
            "backend-services": {
              "commands": {
                "add-backend": {
                  "commands": {},
                  "flags": {
                    "--balancing-mode": [
                      "CONNECTION",
                      "RATE",
                      "UTILIZATION"
                    ],
                    "--capacity-scaler": "value",
                    "--description": "value",
                    "--failover": "bool",
                    "--global": "bool",
                    "--global-network-endpoint-group": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--max-connections": "value",
                    "--max-connections-per-endpoint": "value",
                    "--max-connections-per-instance": "value",
                    "--max-rate": "value",
                    "--max-rate-per-endpoint": "value",
                    "--max-rate-per-instance": "value",
                    "--max-utilization": "value",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-region": "dynamic",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "value"
                  }
                },
                "add-service-bindings": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--service-bindings": "dynamic"
                  }
                },
                "add-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-file": "value",
                    "--key-name": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--affinity-cookie-ttl": "value",
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-host": "bool",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-include-named-cookie": "value",
                    "--cache-key-include-protocol": "bool",
                    "--cache-key-include-query-string": "bool",
                    "--cache-key-query-string-blacklist": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--connection-drain-on-failover": "bool",
                    "--connection-draining-timeout": "value",
                    "--connection-persistence-on-unhealthy-backends": [
                      "ALWAYS_PERSIST",
                      "DEFAULT_FOR_PROTOCOL",
                      "NEVER_PERSIST"
                    ],
                    "--custom-request-header": "value",
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--drop-traffic-if-unhealthy": "bool",
                    "--enable-cdn": "bool",
                    "--enable-logging": "bool",
                    "--enable-strong-affinity": "bool",
                    "--failover-ratio": "value",
                    "--global": "bool",
                    "--global-health-checks": "bool",
                    "--health-checks": "dynamic",
                    "--health-checks-region": "dynamic",
                    "--http-health-checks": "dynamic",
                    "--https-health-checks": "dynamic",
                    "--iap": "value",
                    "--idle-timeout-sec": "value",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--locality-lb-policy": [
                      "INVALID_LB_POLICY",
                      "LEAST_REQUEST",
                      "MAGLEV",
                      "ORIGINAL_DESTINATION",
                      "RANDOM",
                      "RING_HASH",
                      "ROUND_ROBIN",
                      "WEIGHTED_MAGLEV"
                    ],
                    "--logging-optional": [
                      "CUSTOM",
                      "EXCLUDE_ALL_OPTIONAL",
                      "INCLUDE_ALL_OPTIONAL"
                    ],
                    "--logging-optional-fields": "value",
                    "--logging-sample-rate": "value",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--network": "value",
                    "--port-name": "value",
                    "--protocol": "value",
                    "--region": "dynamic",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--service-bindings": "dynamic",
                    "--service-lb-policy": "value",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_NO_DESTINATION",
                      "CLIENT_IP_PORT_PROTO",
                      "CLIENT_IP_PROTO",
                      "GENERATED_COOKIE",
                      "HEADER_FIELD",
                      "HTTP_COOKIE",
                      "NONE"
                    ],
                    "--signed-url-cache-max-age": "value",
                    "--subsetting-policy": [
                      "CONSISTENT_HASH_SUBSETTING",
                      "NONE"
                    ],
                    "--subsetting-subset-size": "value",
                    "--timeout": "value",
                    "--tracking-mode": [
                      "PER_CONNECTION",
                      "PER_SESSION"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-name": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "edit": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "get-health": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-backend": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--global-network-endpoint-group": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-region": "dynamic",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "value"
                  }
                },
                "remove-service-bindings": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--service-bindings": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "set-security-policy": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--security-policy": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--affinity-cookie-ttl": "value",
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-host": "bool",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-include-named-cookie": "value",
                    "--cache-key-include-protocol": "bool",
                    "--cache-key-include-query-string": "bool",
                    "--cache-key-query-string-blacklist": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--connection-drain-on-failover": "bool",
                    "--connection-draining-timeout": "value",
                    "--connection-persistence-on-unhealthy-backends": [
                      "ALWAYS_PERSIST",
                      "DEFAULT_FOR_PROTOCOL",
                      "NEVER_PERSIST"
                    ],
                    "--custom-request-header": "value",
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--drop-traffic-if-unhealthy": "bool",
                    "--edge-security-policy": "dynamic",
                    "--enable-cdn": "bool",
                    "--enable-logging": "bool",
                    "--enable-strong-affinity": "bool",
                    "--failover-ratio": "value",
                    "--global": "bool",
                    "--global-health-checks": "bool",
                    "--health-checks": "dynamic",
                    "--health-checks-region": "dynamic",
                    "--http-health-checks": "dynamic",
                    "--https-health-checks": "dynamic",
                    "--iap": "value",
                    "--idle-timeout-sec": "value",
                    "--locality-lb-policy": [
                      "INVALID_LB_POLICY",
                      "LEAST_REQUEST",
                      "MAGLEV",
                      "ORIGINAL_DESTINATION",
                      "RANDOM",
                      "RING_HASH",
                      "ROUND_ROBIN",
                      "WEIGHTED_MAGLEV"
                    ],
                    "--logging-optional": [
                      "CUSTOM",
                      "EXCLUDE_ALL_OPTIONAL",
                      "INCLUDE_ALL_OPTIONAL"
                    ],
                    "--logging-optional-fields": "value",
                    "--logging-sample-rate": "value",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--no-bypass-cache-on-request-headers": "bool",
                    "--no-client-ttl": "bool",
                    "--no-custom-request-headers": "bool",
                    "--no-custom-response-headers": "bool",
                    "--no-default-ttl": "bool",
                    "--no-health-checks": "bool",
                    "--no-max-ttl": "bool",
                    "--no-negative-caching-policies": "bool",
                    "--no-serve-while-stale": "bool",
                    "--no-service-bindings": "bool",
                    "--no-service-lb-policy": "bool",
                    "--port-name": "value",
                    "--protocol": "value",
                    "--region": "dynamic",
                    "--request-coalescing": "bool",
                    "--security-policy": "dynamic",
                    "--serve-while-stale": "value",
                    "--service-bindings": "dynamic",
                    "--service-lb-policy": "value",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_NO_DESTINATION",
                      "CLIENT_IP_PORT_PROTO",
                      "CLIENT_IP_PROTO",
                      "GENERATED_COOKIE",
                      "HEADER_FIELD",
                      "HTTP_COOKIE",
                      "NONE"
                    ],
                    "--signed-url-cache-max-age": "value",
                    "--subsetting-policy": [
                      "CONSISTENT_HASH_SUBSETTING",
                      "NONE"
                    ],
                    "--subsetting-subset-size": "value",
                    "--timeout": "value",
                    "--tracking-mode": [
                      "PER_CONNECTION",
                      "PER_SESSION"
                    ]
                  }
                },
                "update-backend": {
                  "commands": {},
                  "flags": {
                    "--balancing-mode": [
                      "CONNECTION",
                      "RATE",
                      "UTILIZATION"
                    ],
                    "--capacity-scaler": "value",
                    "--description": "value",
                    "--failover": "bool",
                    "--global": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--max-connections": "value",
                    "--max-connections-per-endpoint": "value",
                    "--max-connections-per-instance": "value",
                    "--max-rate": "value",
                    "--max-rate-per-endpoint": "value",
                    "--max-rate-per-instance": "value",
                    "--max-utilization": "value",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "commitments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--auto-renew": "bool",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--merge-source-commitments": "value",
                    "--min-cpu-platform": "value",
                    "--plan": [
                      "12-month",
                      "36-month"
                    ],
                    "--region": "dynamic",
                    "--require-specific-reservation": "bool",
                    "--reservation": "dynamic",
                    "--reservation-zone": "dynamic",
                    "--reservations-from-file": "value",
                    "--resources": "value",
                    "--resources-accelerator": "value",
                    "--share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--split-source-commitment": "value",
                    "--type": [
                      "accelerator-optimized",
                      "compute-optimized",
                      "compute-optimized-c2d",
                      "compute-optimized-c3",
                      "general-purpose",
                      "general-purpose-e2",
                      "general-purpose-n2",
                      "general-purpose-n2d",
                      "general-purpose-t2d",
                      "graphics-optimized",
                      "memory-optimized",
                      "memory-optimized-m3"
                    ],
                    "--vm-count": "value"
                  }
                },
                "create-license": {
                  "commands": {},
                  "flags": {
                    "--amount": "value",
                    "--cores-per-license": "value",
                    "--license": "value",
                    "--plan": [
                      "12-month",
                      "36-month"
                    ],
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--auto-renew": "bool",
                    "--plan": [
                      "36-month"
                    ],
                    "--region": "dynamic"
                  }
                },
                "update-reservations": {
                  "commands": {},
                  "flags": {
                    "--dest-accelerator": "value",
                    "--dest-local-ssd": "value",
                    "--dest-reservation": "value",
                    "--dest-share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--dest-share-with": "value",
                    "--region": "dynamic",
                    "--reservations-from-file": "value",
                    "--source-accelerator": "value",
                    "--source-local-ssd": "value",
                    "--source-reservation": "value",
                    "--source-share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--source-share-with": "value"
                  }
                }
              },
              "flags": {}
            },
            "config-ssh": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--remove": "bool",
                "--ssh-config-file": "value",
                "--ssh-key-file": "value"
              }
            },
            "connect-to-serial-port": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--extra-args": "value",
                "--force-key-file-overwrite": "bool",
                "--port": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--zone": "dynamic"
              }
            },
            "copy-files": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--plain": "bool",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--zone": "value"
              }
            },
            "diagnose": {
              "commands": {
                "export-logs": {
                  "commands": {},
                  "flags": {
                    "--collect-process-traces": "bool",
                    "--zone": "dynamic"
                  }
                },
                "routes": {
                  "commands": {},
                  "flags": {
                    "--container": "value",
                    "--dry-run": "bool",
                    "--external-route-ip": "value",
                    "--force-key-file-overwrite": "bool",
                    "--plain": "bool",
                    "--regexp": "value",
                    "--reverse-traceroute": "bool",
                    "--ssh-flag": "value",
                    "--ssh-key-expiration": "value",
                    "--ssh-key-expire-after": "value",
                    "--ssh-key-file": "value",
                    "--strict-host-key-checking": [
                      "ask",
                      "no",
                      "yes"
                    ],
                    "--user": "value",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "disk-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            },
            "disks": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "add-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "bulk": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--source-consistency-group-policy": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--erase-windows-vss-signature": "bool",
                    "--guest-os-features": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-family-scope": [
                      "global",
                      "zonal"
                    ],
                    "--image-project": "value",
                    "--interface": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--licenses": "value",
                    "--multi-writer": "bool",
                    "--physical-block-size": [
                      "16384",
                      "4096"
                    ],
                    "--primary-disk": "dynamic",
                    "--primary-disk-project": "value",
                    "--primary-disk-region": "dynamic",
                    "--primary-disk-zone": "dynamic",
                    "--provisioned-iops": "value",
                    "--provisioned-throughput": "value",
                    "--region": "dynamic",
                    "--replica-zones": "value",
                    "--require-csek-key-create": "bool",
                    "--resource-policies": "value",
                    "--size": "value",
                    "--source-disk": "dynamic",
                    "--source-disk-region": "dynamic",
                    "--source-disk-zone": "dynamic",
                    "--source-instant-snapshot": "dynamic",
                    "--source-snapshot": "dynamic",
                    "--type": "dynamic",
                    "--user-licenses": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "remove-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--size": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "snapshot": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--chain-name": "value",
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--snapshot-names": "value",
                    "--storage-location": "value",
                    "--zone": "dynamic"
                  }
                },
                "start-async-replication": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--secondary-disk": "dynamic",
                    "--secondary-disk-project": "value",
                    "--secondary-disk-region": "dynamic",
                    "--secondary-disk-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "stop-async-replication": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--secondary-disk": "dynamic",
                    "--secondary-disk-region": "dynamic",
                    "--secondary-disk-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "stop-group-async-replication": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-architecture": "bool",
                    "--clear-labels": "bool",
                    "--clear-user-licenses": "bool",
                    "--provisioned-iops": "value",
                    "--provisioned-throughput": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--update-labels": "value",
                    "--update-user-licenses": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "external-vpn-gateways": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--interfaces": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "firewall-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--folder": "value",
                        "--name": "value",
                        "--organization": "value",
                        "--replace-association-on-target": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "clone-rules": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--source-firewall-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--short-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-rules": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "apply_security_profile_group",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--layer4-configs": "value",
                        "--organization": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-threat-intelligence": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "apply_security_profile_group",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--organization": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-threat-intelligence": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "firewall-rules": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "ALLOW",
                      "DENY"
                    ],
                    "--allow": "value",
                    "--description": "value",
                    "--destination-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "IN",
                      "INGRESS",
                      "OUT"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--logging-metadata": [
                      "exclude-all",
                      "include-all"
                    ],
                    "--network": "value",
                    "--priority": "value",
                    "--rules": "value",
                    "--source-ranges": "value",
                    "--source-service-accounts": "value",
                    "--source-tags": "value",
                    "--target-service-accounts": "value",
                    "--target-tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "migrate": {
                  "commands": {},
                  "flags": {
                    "--export-tag-mapping": "bool",
                    "--source-network": "dynamic",
                    "--tag-mapping-file": "value",
                    "--target-firewall-policy": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allow": "value",
                    "--description": "value",
                    "--destination-ranges": "value",
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--logging-metadata": [
                      "exclude-all",
                      "include-all"
                    ],
                    "--priority": "value",
                    "--rules": "value",
                    "--source-ranges": "value",
                    "--source-service-accounts": "value",
                    "--source-tags": "value",
                    "--target-service-accounts": "value",
                    "--target-tags": "value"
                  }
                }
              },
              "flags": {}
            },
            "forwarding-rules": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--global": "bool",
                        "--path": "value",
                        "--region": "dynamic",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--address": "dynamic",
                    "--address-region": "dynamic",
                    "--allow-global-access": "bool",
                    "--allow-psc-global-access": "bool",
                    "--backend-service": "value",
                    "--backend-service-region": "dynamic",
                    "--description": "value",
                    "--disable-automate-dns-zone": "bool",
                    "--global": "bool",
                    "--global-address": "bool",
                    "--global-backend-service": "bool",
                    "--global-target-http-proxy": "bool",
                    "--global-target-https-proxy": "bool",
                    "--global-target-tcp-proxy": "bool",
                    "--ip-protocol": [
                      "AH",
                      "ALL",
                      "ESP",
                      "ICMP",
                      "L3_DEFAULT",
                      "SCTP",
                      "TCP",
                      "UDP"
                    ],
                    "--ip-version": [
                      "IPV4",
                      "IPV6"
                    ],
                    "--is-mirroring-collector": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--network": "value",
                    "--network-tier": "value",
                    "--port-range": "value",
                    "--ports": "value",
                    "--region": "dynamic",
                    "--service-directory-registration": "value",
                    "--service-label": "value",
                    "--source-ip-ranges": "value",
                    "--subnet": "value",
                    "--subnet-region": "dynamic",
                    "--target-google-apis-bundle": "value",
                    "--target-grpc-proxy": "value",
                    "--target-http-proxy": "value",
                    "--target-http-proxy-region": "dynamic",
                    "--target-https-proxy": "value",
                    "--target-https-proxy-region": "dynamic",
                    "--target-instance": "value",
                    "--target-instance-zone": "dynamic",
                    "--target-pool": "value",
                    "--target-pool-region": "dynamic",
                    "--target-service-attachment": "value",
                    "--target-service-attachment-region": "dynamic",
                    "--target-ssl-proxy": "value",
                    "--target-tcp-proxy": "value",
                    "--target-tcp-proxy-region": "dynamic",
                    "--target-vpn-gateway": "value",
                    "--target-vpn-gateway-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-target": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "value",
                    "--backend-service-region": "dynamic",
                    "--global": "bool",
                    "--global-backend-service": "bool",
                    "--global-target-http-proxy": "bool",
                    "--global-target-https-proxy": "bool",
                    "--global-target-tcp-proxy": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--network": "value",
                    "--region": "dynamic",
                    "--subnet": "value",
                    "--subnet-region": "value",
                    "--target-grpc-proxy": "value",
                    "--target-http-proxy": "value",
                    "--target-http-proxy-region": "dynamic",
                    "--target-https-proxy": "value",
                    "--target-https-proxy-region": "dynamic",
                    "--target-instance": "value",
                    "--target-instance-zone": "dynamic",
                    "--target-pool": "value",
                    "--target-pool-region": "dynamic",
                    "--target-ssl-proxy": "value",
                    "--target-tcp-proxy": "value",
                    "--target-tcp-proxy-region": "dynamic",
                    "--target-vpn-gateway": "value",
                    "--target-vpn-gateway-region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allow-global-access": "bool",
                    "--allow-psc-global-access": "bool",
                    "--clear-labels": "bool",
                    "--global": "bool",
                    "--network-tier": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--source-ip-ranges": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "future-reservations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--description": "value",
                    "--duration": "value",
                    "--end-time": "value",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--maintenance-freeze-duration": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--min-cpu-platform": "value",
                    "--name-prefix": "value",
                    "--planning-status": [
                      "DRAFT",
                      "SUBMITTED"
                    ],
                    "--share-setting": [
                      "local",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--source-instance-template": "value",
                    "--start-time": "value",
                    "--total-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--duration": "value",
                    "--end-time": "value",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--min-cpu-platform": "value",
                    "--planning-status": [
                      "DRAFT",
                      "SUBMITTED"
                    ],
                    "--start-time": "value",
                    "--total-count": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "health-checks": {
              "commands": {
                "create": {
                  "commands": {
                    "grpc": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--grpc-service-name": "value",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "http2": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "https": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "ssl": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "tcp": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--protocol": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {
                    "grpc": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--grpc-service-name": "value",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "http2": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "https": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool",
                        "--weight-report-mode": [
                          "DISABLE",
                          "DRY_RUN",
                          "ENABLE"
                        ]
                      }
                    },
                    "ssl": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "tcp": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "http-health-checks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                }
              },
              "flags": {}
            },
            "https-health-checks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                }
              },
              "flags": {}
            },
            "images": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--family": "value",
                    "--forbidden-database-file": "value",
                    "--force": "bool",
                    "--force-create": "bool",
                    "--guest-os-features": "value",
                    "--key-exchange-key-file": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--licenses": "value",
                    "--platform-key-file": "value",
                    "--require-csek-key-create": "bool",
                    "--signature-database-file": "value",
                    "--source-disk": "dynamic",
                    "--source-disk-project": "value",
                    "--source-disk-zone": "dynamic",
                    "--source-image": "value",
                    "--source-image-family": "value",
                    "--source-image-project": "value",
                    "--source-snapshot": "dynamic",
                    "--source-uri": "value",
                    "--storage-location": "value",
                    "--user-licenses": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "deprecate": {
                  "commands": {},
                  "flags": {
                    "--delete-in": "value",
                    "--delete-on": "value",
                    "--deprecate-in": "value",
                    "--deprecate-on": "value",
                    "--obsolete-in": "value",
                    "--obsolete-on": "value",
                    "--replacement": "dynamic",
                    "--state": [
                      "ACTIVE",
                      "DELETED",
                      "DEPRECATED",
                      "OBSOLETE"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "describe-from-family": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--compute-service-account": "value",
                    "--destination-uri": "value",
                    "--export-format": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--log-location": "value",
                    "--network": "value",
                    "--subnet": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--aws-access-key-id": "value",
                    "--aws-ami-export-location": "value",
                    "--aws-ami-id": "value",
                    "--aws-region": "value",
                    "--aws-secret-access-key": "value",
                    "--aws-session-token": "value",
                    "--aws-source-ami-file-path": "value",
                    "--byol": "bool",
                    "--compute-service-account": "value",
                    "--data-disk": "bool",
                    "--description": "value",
                    "--family": "value",
                    "--guest-environment": "bool",
                    "--guest-os-features": "value",
                    "--log-location": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--source-file": "value",
                    "--source-image": "dynamic",
                    "--storage-location": "value",
                    "--subnet": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--preview-images": "bool",
                    "--regexp": "value",
                    "--show-deprecated": "bool",
                    "--sort-by": "value",
                    "--standard-images": "bool",
                    "--uri": "bool"
                  }
                },
                "packages": {
                  "commands": {
                    "diff": {
                      "commands": {},
                      "flags": {
                        "--base-image": "dynamic",
                        "--base-project": "dynamic",
                        "--diff-image": "dynamic",
                        "--diff-project": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--show-added-packages": "bool",
                        "--show-removed-packages": "bool",
                        "--show-updated-packages": "bool",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--image": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--clear-labels": "bool",
                    "--clear-user-licenses": "bool",
                    "--description": "value",
                    "--family": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--update-user-licenses": "value"
                  }
                },
                "vulnerabilities": {
                  "commands": {
                    "describe-note": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--image": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instance-groups": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-named-ports": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "list-instances": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "managed": {
                  "commands": {
                    "abandon-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "all-instances-config": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--labels": "value",
                            "--metadata": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--labels": "value",
                            "--metadata": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--base-instance-name": "value",
                        "--description": "value",
                        "--force-update-on-repair": "bool",
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--instance-redistribution-type": [
                          "none",
                          "proactive"
                        ],
                        "--list-managed-instances-results": [
                          "pageless",
                          "paginated"
                        ],
                        "--region": "dynamic",
                        "--size": "value",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--target-distribution-shape": [
                          "any",
                          "any-single-zone",
                          "balanced",
                          "even"
                        ],
                        "--target-pool": "value",
                        "--template": "value",
                        "--update-policy-max-surge": "value",
                        "--update-policy-max-unavailable": "value",
                        "--update-policy-min-ready": "value",
                        "--update-policy-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-most-disruptive-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--update-policy-type": [
                          "opportunistic",
                          "proactive"
                        ],
                        "--zone": "dynamic",
                        "--zones": "dynamic"
                      }
                    },
                    "create-instance": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--region": "dynamic",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--stateful-metadata": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "delete-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--skip-instances-on-validation-error": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe-instance": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "export-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--autoscaling-file": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "get-named-ports": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "instance-configs": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--instance": "value",
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--stateful-disk": "value",
                            "--stateful-external-ip": "value",
                            "--stateful-internal-ip": "value",
                            "--stateful-metadata": "value",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--instances": "value",
                            "--region": "dynamic",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--region": "dynamic",
                            "--sort-by": "value",
                            "--zone": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--instance": "value",
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--remove-stateful-disks": "value",
                            "--remove-stateful-external-ips": "value",
                            "--remove-stateful-internal-ips": "value",
                            "--remove-stateful-metadata": "value",
                            "--stateful-disk": "value",
                            "--stateful-external-ip": "value",
                            "--stateful-internal-ip": "value",
                            "--stateful-metadata": "value",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--regions": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zones": "value"
                      }
                    },
                    "list-errors": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "recreate-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "resize": {
                      "commands": {},
                      "flags": {
                        "--creation-retries": "bool",
                        "--region": "dynamic",
                        "--size": "value",
                        "--stopped-size": "value",
                        "--suspended-size": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "resume-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "rolling-action": {
                      "commands": {
                        "replace": {
                          "commands": {},
                          "flags": {
                            "--max-surge": "value",
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--region": "dynamic",
                            "--replacement-method": [
                              "recreate",
                              "substitute"
                            ],
                            "--zone": "dynamic"
                          }
                        },
                        "restart": {
                          "commands": {},
                          "flags": {
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "start-update": {
                          "commands": {},
                          "flags": {
                            "--canary-version": "value",
                            "--force": "bool",
                            "--max-surge": "value",
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--minimal-action": [
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--most-disruptive-allowed-action": [
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--replacement-method": [
                              "recreate",
                              "substitute"
                            ],
                            "--type": [
                              "opportunistic",
                              "proactive"
                            ],
                            "--zone": "dynamic"
                          }
                        },
                        "stop-proactive-update": {
                          "commands": {},
                          "flags": {
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "set-autohealing": {
                      "commands": {},
                      "flags": {
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "set-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--autoscaling-file": "value",
                        "--cool-down-period": "value",
                        "--cpu-utilization-predictive-method": [
                          "none",
                          "optimize-availability",
                          "standard"
                        ],
                        "--custom-metric-utilization": "value",
                        "--description": "value",
                        "--max-num-replicas": "value",
                        "--min-num-replicas": "value",
                        "--mode": [
                          "off",
                          "on",
                          "only-scale-out",
                          "only-up"
                        ],
                        "--region": "dynamic",
                        "--remove-stackdriver-metric": "value",
                        "--scale-based-on-cpu": "bool",
                        "--scale-based-on-load-balancing": "bool",
                        "--scale-in-control": "value",
                        "--schedule-cron": "value",
                        "--schedule-description": "value",
                        "--schedule-duration-sec": "value",
                        "--schedule-min-required-replicas": "value",
                        "--schedule-time-zone": "value",
                        "--set-schedule": "value",
                        "--stackdriver-metric-filter": "value",
                        "--stackdriver-metric-single-instance-assignment": "value",
                        "--stackdriver-metric-utilization-target": "value",
                        "--stackdriver-metric-utilization-target-type": [
                          "delta-per-minute",
                          "delta-per-second",
                          "gauge"
                        ],
                        "--target-cpu-utilization": "value",
                        "--target-load-balancing-utilization": "value",
                        "--update-stackdriver-metric": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-instance-template": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--template": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-named-ports": {
                      "commands": {},
                      "flags": {
                        "--named-ports": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "set-standby-policy": {
                      "commands": {},
                      "flags": {
                        "--initial-delay": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "set-target-pools": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--target-pools": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "start-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "stop-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "stop-instances": {
                      "commands": {},
                      "flags": {
                        "--force": "bool",
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "suspend-instances": {
                      "commands": {},
                      "flags": {
                        "--force": "bool",
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-autohealing": "bool",
                        "--description": "value",
                        "--force-update-on-repair": "bool",
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--instance-redistribution-type": [
                          "none",
                          "proactive"
                        ],
                        "--list-managed-instances-results": [
                          "pageless",
                          "paginated"
                        ],
                        "--region": "dynamic",
                        "--remove-stateful-disks": "value",
                        "--remove-stateful-external-ips": "value",
                        "--remove-stateful-internal-ips": "value",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--target-distribution-shape": [
                          "any",
                          "any-single-zone",
                          "balanced",
                          "even"
                        ],
                        "--update-policy-max-surge": "value",
                        "--update-policy-max-unavailable": "value",
                        "--update-policy-min-ready": "value",
                        "--update-policy-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-most-disruptive-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--update-policy-type": [
                          "opportunistic",
                          "proactive"
                        ],
                        "--zone": "dynamic"
                      }
                    },
                    "update-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--clear-scale-down-control": "bool",
                        "--clear-scale-in-control": "bool",
                        "--cpu-utilization-predictive-method": [
                          "none",
                          "optimize-availability",
                          "standard"
                        ],
                        "--disable-schedule": "value",
                        "--enable-schedule": "value",
                        "--max-num-replicas": "value",
                        "--min-num-replicas": "value",
                        "--mode": [
                          "off",
                          "on",
                          "only-scale-out",
                          "only-up"
                        ],
                        "--region": "dynamic",
                        "--remove-schedule": "value",
                        "--scale-in-control": "value",
                        "--schedule-cron": "value",
                        "--schedule-description": "value",
                        "--schedule-duration-sec": "value",
                        "--schedule-min-required-replicas": "value",
                        "--schedule-time-zone": "value",
                        "--set-schedule": "value",
                        "--update-schedule": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "update-instances": {
                      "commands": {},
                      "flags": {
                        "--all-instances": "bool",
                        "--instances": "value",
                        "--minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--most-disruptive-allowed-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "wait-until": {
                      "commands": {},
                      "flags": {
                        "--all-instances-config-effective": "bool",
                        "--region": "dynamic",
                        "--stable": "bool",
                        "--timeout": "value",
                        "--version-target-reached": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "wait-until-stable": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-named-ports": {
                  "commands": {},
                  "flags": {
                    "--named-ports": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "unmanaged": {
                  "commands": {
                    "add-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "get-named-ports": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zones": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-named-ports": {
                      "commands": {},
                      "flags": {
                        "--named-ports": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instance-templates": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--configure-disk": "value",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-template-region": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-address": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--ipv6-prefix-length": "value",
                    "--key-revocation-action-type": [
                      "none",
                      "stop"
                    ],
                    "--labels": "value",
                    "--local-nvdimm": "value",
                    "--local-ssd": "value",
                    "--local-ssd-recovery-timeout": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--mesh": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--numa-node-count": "value",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--region": "dynamic",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--service-proxy": "value",
                    "--service-proxy-labels": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance": "dynamic",
                    "--source-instance-zone": "dynamic",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--subnet-region": "value",
                    "--tags": "value",
                    "--termination-time": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value"
                  }
                },
                "create-with-container": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-address": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--ipv6-prefix-length": "value",
                    "--labels": "value",
                    "--local-nvdimm": "value",
                    "--local-ssd": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--region": "dynamic",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--subnet-region": "value",
                    "--tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-access-config": {
                  "commands": {},
                  "flags": {
                    "--access-config-name": "value",
                    "--address": "value",
                    "--network-interface": "value",
                    "--network-tier": "value",
                    "--no-public-dns": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--public-dns": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-metadata": {
                  "commands": {},
                  "flags": {
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-tags": {
                  "commands": {},
                  "flags": {
                    "--tags": "value",
                    "--zone": "dynamic"
                  }
                },
                "attach-disk": {
                  "commands": {},
                  "flags": {
                    "--boot": "bool",
                    "--csek-key-file": "value",
                    "--device-name": "value",
                    "--disk": "value",
                    "--disk-scope": [
                      "regional",
                      "zonal"
                    ],
                    "--force-attach": "bool",
                    "--mode": [
                      "ro",
                      "rw"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "bulk": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator": "value",
                        "--async": "bool",
                        "--boot-disk-auto-delete": "bool",
                        "--boot-disk-device-name": "value",
                        "--boot-disk-kms-key": "dynamic",
                        "--boot-disk-kms-keyring": "dynamic",
                        "--boot-disk-kms-location": "dynamic",
                        "--boot-disk-kms-project": "dynamic",
                        "--boot-disk-provisioned-iops": "value",
                        "--boot-disk-size": "value",
                        "--boot-disk-type": "value",
                        "--can-ip-forward": "bool",
                        "--confidential-compute": "bool",
                        "--count": "value",
                        "--create-disk": "value",
                        "--custom-cpu": "value",
                        "--custom-extensions": "bool",
                        "--custom-memory": "value",
                        "--custom-vm-type": "value",
                        "--description": "value",
                        "--disk": "value",
                        "--enable-display-device": "bool",
                        "--enable-nested-virtualization": "bool",
                        "--enable-uefi-networking": "bool",
                        "--erase-windows-vss-signature": "bool",
                        "--host-error-timeout-seconds": "value",
                        "--image": "value",
                        "--image-family": "value",
                        "--image-project": "value",
                        "--instance-termination-action": [
                          "DELETE",
                          "STOP"
                        ],
                        "--labels": "value",
                        "--local-ssd": "value",
                        "--local-ssd-recovery-timeout": "value",
                        "--location-policy": "value",
                        "--machine-type": "dynamic",
                        "--maintenance-interval": [
                          "PERIODIC",
                          "RECURRENT"
                        ],
                        "--maintenance-policy": [
                          "MIGRATE",
                          "TERMINATE"
                        ],
                        "--max-count-per-zone": "value",
                        "--max-run-duration": "value",
                        "--metadata": "value",
                        "--metadata-from-file": "value",
                        "--min-count": "value",
                        "--min-cpu-platform": "value",
                        "--min-node-cpu": "value",
                        "--name-pattern": "value",
                        "--network": "value",
                        "--network-interface": "value",
                        "--network-performance-configs": "value",
                        "--network-tier": "value",
                        "--no-address": "bool",
                        "--no-public-dns": "bool",
                        "--no-scopes": "bool",
                        "--no-service-account": "bool",
                        "--numa-node-count": "value",
                        "--on-host-maintenance": [
                          "MIGRATE",
                          "TERMINATE"
                        ],
                        "--post-key-revocation-action-type": [
                          "noop",
                          "shutdown"
                        ],
                        "--predefined-names": "value",
                        "--preemptible": "bool",
                        "--provisioning-model": [
                          "SPOT",
                          "STANDARD"
                        ],
                        "--public-dns": "bool",
                        "--region": "value",
                        "--reservation": "value",
                        "--reservation-affinity": [
                          "any",
                          "none",
                          "specific"
                        ],
                        "--resource-manager-tags": "value",
                        "--resource-policies": "value",
                        "--restart-on-failure": "bool",
                        "--scopes": "value",
                        "--secure-tags": "value",
                        "--service-account": "value",
                        "--shielded-integrity-monitoring": "bool",
                        "--shielded-secure-boot": "bool",
                        "--shielded-vtpm": "bool",
                        "--source-instance-template": "dynamic",
                        "--source-snapshot": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--subnet": "value",
                        "--tags": "value",
                        "--target-distribution-shape": [
                          "ANY",
                          "ANY_SINGLE_ZONE",
                          "BALANCED"
                        ],
                        "--termination-time": "value",
                        "--threads-per-core": "value",
                        "--visible-core-count": "value",
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--async": "bool",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--create-disk": "value",
                    "--csek-key-file": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--deletion-protection": "bool",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-display-device": "bool",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--erase-windows-vss-signature": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--hostname": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-family-scope": [
                      "global",
                      "zonal"
                    ],
                    "--image-project": "value",
                    "--instance-kms-key": "dynamic",
                    "--instance-kms-keyring": "dynamic",
                    "--instance-kms-location": "dynamic",
                    "--instance-kms-project": "dynamic",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-address": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--ipv6-prefix-length": "value",
                    "--ipv6-public-ptr-domain": "value",
                    "--key-revocation-action-type": [
                      "none",
                      "stop"
                    ],
                    "--labels": "value",
                    "--local-nvdimm": "value",
                    "--local-ssd": "value",
                    "--local-ssd-recovery-timeout": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-freeze-duration": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-interface-from-file": "value",
                    "--network-interface-from-json-string": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-dns": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--node-project": "value",
                    "--numa-node-count": "value",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-dns": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--require-csek-key-create": "bool",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--secure-tags": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--source-machine-image": "dynamic",
                    "--source-machine-image-csek-key-file": "value",
                    "--source-snapshot": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--termination-time": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "create-with-container": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-nested-virtualization": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-address": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--ipv6-prefix-length": "value",
                    "--labels": "value",
                    "--local-nvdimm": "value",
                    "--local-ssd": "value",
                    "--local-ssd-recovery-timeout": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-dns": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--numa-node-count": "value",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-dns": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--delete-disks": [
                      "all",
                      "boot",
                      "data"
                    ],
                    "--keep-disks": [
                      "all",
                      "boot",
                      "data"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "delete-access-config": {
                  "commands": {},
                  "flags": {
                    "--access-config-name": "value",
                    "--network-interface": "value",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--guest-attributes": "value",
                    "--zone": "dynamic"
                  }
                },
                "detach-disk": {
                  "commands": {},
                  "flags": {
                    "--device-name": "value",
                    "--disk": "value",
                    "--disk-scope": [
                      "regional",
                      "zonal"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--zone": "value"
                  }
                },
                "get-guest-attributes": {
                  "commands": {},
                  "flags": {
                    "--query-path": "value",
                    "--zone": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "get-serial-port-output": {
                  "commands": {},
                  "flags": {
                    "--port": "value",
                    "--start": "value",
                    "--zone": "dynamic"
                  }
                },
                "get-shielded-identity": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--byol": "bool",
                    "--can-ip-forward": "bool",
                    "--compute-service-account": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--deletion-protection": "bool",
                    "--description": "value",
                    "--guest-environment": "bool",
                    "--guest-os-features": "value",
                    "--hostname": "value",
                    "--labels": "value",
                    "--log-location": "value",
                    "--machine-type": "dynamic",
                    "--network": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--private-network-ip": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--source-uri": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "network-interfaces": {
                  "commands": {
                    "get-effective-firewalls": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network-interface": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--aliases": "value",
                        "--external-ipv6-address": "value",
                        "--external-ipv6-prefix-length": "value",
                        "--internal-ipv6-address": "value",
                        "--internal-ipv6-prefix-length": "value",
                        "--ipv6-address": "value",
                        "--ipv6-network-tier": [
                          "PREMIUM"
                        ],
                        "--ipv6-prefix-length": "value",
                        "--network": "value",
                        "--network-interface": "value",
                        "--private-network-ip": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--subnetwork": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "ops-agents": {
                  "commands": {
                    "policies": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--agent-rules": "value",
                            "--description": "value",
                            "--group-labels": "value",
                            "--instances": "value",
                            "--os-types": "value",
                            "--zones": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {}
                        },
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--agent-rules": "value",
                            "--clear-group-labels": "bool",
                            "--clear-instances": "bool",
                            "--clear-zones": "bool",
                            "--description": "value",
                            "--etag": "value",
                            "--group-labels": "value",
                            "--instances": "value",
                            "--os-types": "value",
                            "--zones": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "os-inventory": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--inventory-filter": "value",
                        "--kernel-version": "value",
                        "--limit": "value",
                        "--os-shortname": "value",
                        "--os-version": "value",
                        "--package-name": "value",
                        "--package-version": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "perform-maintenance": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-metadata": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--keys": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-tags": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--tags": "value",
                    "--zone": "dynamic"
                  }
                },
                "reset": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "dynamic"
                  }
                },
                "send-diagnostic-interrupt": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "set-disk-auto-delete": {
                  "commands": {},
                  "flags": {
                    "--auto-delete": "bool",
                    "--device-name": "value",
                    "--disk": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "set-machine-type": {
                  "commands": {},
                  "flags": {
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--machine-type": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "set-min-cpu-platform": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--min-cpu-platform": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-name": {
                  "commands": {},
                  "flags": {
                    "--new-name": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-scheduling": {
                  "commands": {},
                  "flags": {
                    "--clear-instance-termination-action": "bool",
                    "--clear-max-run-duration": "bool",
                    "--clear-min-node-cpu": "bool",
                    "--clear-node-affinities": "bool",
                    "--clear-termination-time": "bool",
                    "--host-error-timeout-seconds": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--local-ssd-recovery-timeout": "value",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--min-node-cpu": "value",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--preemptible": "bool",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--restart-on-failure": "bool",
                    "--termination-time": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-scopes": {
                  "commands": {},
                  "flags": {
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--zone": "dynamic"
                  }
                },
                "simulate-maintenance-event": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--with-extended-notifications": "value",
                    "--zone": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csek-key-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-local-ssd": "value",
                    "--zone": "dynamic"
                  }
                },
                "suspend": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-local-ssd": "value",
                    "--zone": "dynamic"
                  }
                },
                "tail-serial-port-output": {
                  "commands": {},
                  "flags": {
                    "--port": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-node-affinities": "bool",
                    "--deletion-protection": "bool",
                    "--enable-display-device": "bool",
                    "--min-cpu-platform": "value",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--remove-labels": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-learn-integrity-policy": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--update-labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "update-access-config": {
                  "commands": {},
                  "flags": {
                    "--ipv6-public-ptr-domain": "value",
                    "--network-interface": "value",
                    "--network-tier": "value",
                    "--no-ipv6-public-ptr": "bool",
                    "--no-public-dns": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--public-dns": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--zone": "dynamic"
                  }
                },
                "update-container": {
                  "commands": {},
                  "flags": {
                    "--clear-container-args": "bool",
                    "--clear-container-command": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--remove-container-env": "value",
                    "--remove-container-mounts": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-learn-integrity-policy": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--zone": "dynamic"
                  }
                },
                "update-from-file": {
                  "commands": {},
                  "flags": {
                    "--clear-secure-tag": "bool",
                    "--minimal-action": "value",
                    "--most-disruptive-allowed-action": "value",
                    "--source": "value",
                    "--zone": "dynamic"
                  }
                },
                "vulnerabilities": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--instance-zone": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instant-snapshots": {
              "commands": {
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--region": "dynamic",
                    "--source-disk": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            },
            "interconnects": {
              "commands": {
                "attachments": {
                  "commands": {
                    "dedicated": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--bandwidth": [
                              "100m",
                              "10g",
                              "1g",
                              "200m",
                              "20g",
                              "2g",
                              "300m",
                              "400m",
                              "500m",
                              "50g",
                              "50m",
                              "5g"
                            ],
                            "--candidate-ipv6-subnets": "value",
                            "--candidate-subnets": "value",
                            "--cloud-router-ipv6-interface-id": "value",
                            "--customer-router-ipv6-interface-id": "value",
                            "--description": "value",
                            "--dry-run": "bool",
                            "--enable-admin": "bool",
                            "--encryption": [
                              "IPSEC",
                              "NONE"
                            ],
                            "--interconnect": "dynamic",
                            "--ipsec-internal-addresses": "value",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--stack-type": [
                              "IPV4_IPV6",
                              "IPV4_ONLY"
                            ],
                            "--subnet-length": [
                              29,
                              30
                            ],
                            "--vlan": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--bandwidth": [
                              "100m",
                              "10g",
                              "1g",
                              "200m",
                              "20g",
                              "2g",
                              "300m",
                              "400m",
                              "500m",
                              "50g",
                              "50m",
                              "5g"
                            ],
                            "--candidate-ipv6-subnets": "value",
                            "--clear-labels": "bool",
                            "--cloud-router-ipv6-interface-id": "value",
                            "--customer-router-ipv6-interface-id": "value",
                            "--description": "value",
                            "--enable-admin": "bool",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--remove-labels": "value",
                            "--stack-type": [
                              "IPV4_IPV6",
                              "IPV4_ONLY"
                            ],
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "partner": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--description": "value",
                            "--dry-run": "bool",
                            "--edge-availability-domain": [
                              "any",
                              "availability-domain-1",
                              "availability-domain-2"
                            ],
                            "--enable-admin": "bool",
                            "--encryption": [
                              "IPSEC",
                              "NONE"
                            ],
                            "--ipsec-internal-addresses": "value",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--clear-labels": "bool",
                            "--description": "value",
                            "--enable-admin": "bool",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--remove-labels": "value",
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-enabled": "bool",
                    "--customer-name": "value",
                    "--description": "value",
                    "--interconnect-type": [
                      "DEDICATED",
                      "IT_PRIVATE",
                      "PARTNER"
                    ],
                    "--link-type": [
                      "LINK_TYPE_ETHERNET_100G_LR",
                      "LINK_TYPE_ETHERNET_10G_LR"
                    ],
                    "--location": "dynamic",
                    "--noc-contact-email": "value",
                    "--remote-location": "dynamic",
                    "--requested-link-count": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-diagnostics": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "macsec": {
                  "commands": {
                    "add-key": {
                      "commands": {},
                      "flags": {
                        "--key-name": "value",
                        "--start-time": "value"
                      }
                    },
                    "get-config": {
                      "commands": {},
                      "flags": {}
                    },
                    "remove-key": {
                      "commands": {},
                      "flags": {
                        "--key-name": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--enabled": "bool",
                        "--fail-open": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "remote-locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--admin-enabled": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--noc-contact-email": "value",
                    "--remove-labels": "value",
                    "--requested-link-count": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "machine-images": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--require-csek-key-create": "bool",
                    "--source-disk-csek-key": "value",
                    "--source-instance": "dynamic",
                    "--source-instance-zone": "dynamic",
                    "--storage-location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--byol": "bool",
                    "--can-ip-forward": "bool",
                    "--compute-service-account": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--guest-environment": "bool",
                    "--guest-flush": "bool",
                    "--guest-os-features": "value",
                    "--labels": "value",
                    "--log-location": "value",
                    "--machine-type": "dynamic",
                    "--network": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--source-uri": "value",
                    "--storage-location": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "machine-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-attachments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--description": "value",
                    "--producer-accept-list": "value",
                    "--producer-reject-list": "value",
                    "--region": "dynamic",
                    "--subnets": "dynamic",
                    "--subnets-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "network-edge-security-services": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--region": "dynamic",
                    "--security-policy": "dynamic",
                    "--security-policy-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--region": "dynamic",
                    "--security-policy": "dynamic",
                    "--security-policy-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-endpoint-groups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--app-engine-app": "bool",
                    "--app-engine-service": "value",
                    "--app-engine-url-mask": "value",
                    "--app-engine-version": "value",
                    "--cloud-function-name": "value",
                    "--cloud-function-url-mask": "value",
                    "--cloud-run-service": "value",
                    "--cloud-run-tag": "value",
                    "--cloud-run-url-mask": "value",
                    "--default-port": "value",
                    "--global": "bool",
                    "--network": "value",
                    "--network-endpoint-type": [
                      "gce-vm-ip",
                      "gce-vm-ip-port",
                      "gce-vm-primary-ip",
                      "internet-fqdn-port",
                      "internet-ip-port",
                      "non-gcp-private-ip-port",
                      "private-service-connect",
                      "serverless"
                    ],
                    "--psc-target-service": "value",
                    "--region": "dynamic",
                    "--serverless-deployment-platform": "value",
                    "--serverless-deployment-resource": "value",
                    "--serverless-deployment-url-mask": "value",
                    "--serverless-deployment-version": "value",
                    "--subnet": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "list-network-endpoints": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-endpoint": "value",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--remove-endpoint": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-firewall-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--name": "value",
                        "--network": "value",
                        "--replace-association-on-target": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "clone-rules": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source-firewall-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "get-effective-firewalls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "apply_security_profile_group",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--layer4-configs": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-secure-tags": "value",
                        "--src-threat-intelligence": "value",
                        "--target-secure-tags": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "apply_security_profile_group",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-secure-tags": "value",
                        "--src-threat-intelligence": "value",
                        "--target-secure-tags": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "networks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bgp-routing-mode": [
                      "global",
                      "regional"
                    ],
                    "--description": "value",
                    "--enable-ula-internal-ipv6": "bool",
                    "--internal-ipv6-range": "value",
                    "--mtu": "value",
                    "--network-firewall-policy-enforcement-order": [
                      "AFTER_CLASSIC_FIREWALL",
                      "BEFORE_CLASSIC_FIREWALL"
                    ],
                    "--range": "value",
                    "--subnet-mode": [
                      "auto",
                      "custom",
                      "legacy"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-effective-firewalls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-ip-addresses": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--types": "value"
                  }
                },
                "list-ip-owners": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--ip-cidr-range": "value",
                    "--limit": "value",
                    "--owner-projects": "value",
                    "--owner-types": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--subnet-name": "value",
                    "--subnet-region": "value",
                    "--uri": "bool"
                  }
                },
                "peerings": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-create-routes": "bool",
                        "--export-custom-routes": "bool",
                        "--export-subnet-routes-with-public-ip": "bool",
                        "--import-custom-routes": "bool",
                        "--import-subnet-routes-with-public-ip": "bool",
                        "--network": "value",
                        "--peer-network": "value",
                        "--peer-project": "value",
                        "--stack-type": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--network": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-routes": {
                      "commands": {},
                      "flags": {
                        "--direction": [
                          "INCOMING",
                          "OUTGOING"
                        ],
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--export-custom-routes": "bool",
                        "--export-subnet-routes-with-public-ip": "bool",
                        "--import-custom-routes": "bool",
                        "--import-subnet-routes-with-public-ip": "bool",
                        "--network": "value",
                        "--stack-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "subnets": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "config": {
                      "commands": {
                        "export": {
                          "commands": {},
                          "flags": {
                            "--all": "bool",
                            "--path": "value",
                            "--region": "value",
                            "--resource-format": [
                              "krm",
                              "terraform"
                            ]
                          }
                        }
                      },
                      "flags": {}
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--description": "value",
                        "--enable-flow-logs": "bool",
                        "--enable-l2": "bool",
                        "--enable-private-ip-google-access": "bool",
                        "--external-ipv6-prefix": "value",
                        "--flow-sampling": "value",
                        "--ipv6-access-type": [
                          "EXTERNAL",
                          "INTERNAL"
                        ],
                        "--logging-aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--logging-filter-expr": "value",
                        "--logging-flow-sampling": "value",
                        "--logging-metadata": [
                          "custom",
                          "exclude-all",
                          "include-all"
                        ],
                        "--logging-metadata-fields": "value",
                        "--metadata": [
                          "exclude-all-metadata",
                          "include-all-metadata"
                        ],
                        "--network": "dynamic",
                        "--private-ipv6-google-access-type": [
                          "disable",
                          "enable-bidirectional-access",
                          "enable-outbound-vm-access"
                        ],
                        "--purpose": [
                          "AGGREGATE",
                          "GLOBAL_MANAGED_PROXY",
                          "INTERNAL_HTTPS_LOAD_BALANCER",
                          "PRIVATE",
                          "PRIVATE_NAT",
                          "PRIVATE_SERVICE_CONNECT",
                          "REGIONAL_MANAGED_PROXY"
                        ],
                        "--range": "value",
                        "--region": "dynamic",
                        "--reserved-internal-range": "value",
                        "--role": [
                          "ACTIVE",
                          "BACKUP"
                        ],
                        "--secondary-range": "value",
                        "--secondary-range-with-reserved-internal-range": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--vlan": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "expand-ip-range": {
                      "commands": {},
                      "flags": {
                        "--prefix-length": "value",
                        "--region": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--regions": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-usable": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--service-project": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-secondary-ranges": "value",
                        "--add-secondary-ranges-with-reserved-internal-range": "value",
                        "--aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--drain-timeout": "value",
                        "--enable-flow-logs": "bool",
                        "--enable-private-ip-google-access": "bool",
                        "--external-ipv6-prefix": "value",
                        "--flow-sampling": "value",
                        "--ipv6-access-type": [
                          "EXTERNAL",
                          "INTERNAL"
                        ],
                        "--logging-aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--logging-filter-expr": "value",
                        "--logging-flow-sampling": "value",
                        "--logging-metadata": [
                          "custom",
                          "exclude-all",
                          "include-all"
                        ],
                        "--logging-metadata-fields": "value",
                        "--metadata": [
                          "exclude-all-metadata",
                          "include-all-metadata"
                        ],
                        "--private-ipv6-google-access-type": [
                          "disable",
                          "enable-bidirectional-access",
                          "enable-outbound-vm-access"
                        ],
                        "--purpose": [
                          "REGIONAL_MANAGED_PROXY"
                        ],
                        "--region": "dynamic",
                        "--remove-secondary-ranges": "value",
                        "--role": [
                          "ACTIVE"
                        ],
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bgp-routing-mode": [
                      "global",
                      "regional"
                    ],
                    "--enable-ula-internal-ipv6": "bool",
                    "--internal-ipv6-range": "value",
                    "--mtu": "value",
                    "--network-firewall-policy-enforcement-order": [
                      "AFTER_CLASSIC_FIREWALL",
                      "BEFORE_CLASSIC_FIREWALL"
                    ],
                    "--switch-to-custom-subnet-mode": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            },
            "org-security-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--name": "value",
                        "--organization": "value",
                        "--replace-association-on-target": "bool",
                        "--security-policy": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "copy-rules": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--source-security-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-rules": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-ports": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--enable-logging": "bool",
                        "--layer4-configs": "value",
                        "--organization": "value",
                        "--security-policy": "value",
                        "--src-ip-ranges": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-ports": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--enable-logging": "bool",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--organization": "value",
                        "--security-policy": "value",
                        "--src-ip-ranges": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "os-config": {
              "commands": {
                "guest-policies": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--file": "value",
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "lookup": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--file": "value",
                        "--folder": "value",
                        "--organization": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "instance-os-policies-compliances": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "inventories": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "os-policy-assignment-reports": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--assignment-id": "value",
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "os-policy-assignments": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--file": "value",
                        "--location": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-revisions": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "operations": {
                      "commands": {
                        "cancel": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--os-policy-assignment": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--os-policy-assignment": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--file": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "os-upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auto-rollback": "bool",
                    "--create-machine-backup": "bool",
                    "--log-location": "value",
                    "--source-os": [
                      "windows-2008r2"
                    ],
                    "--target-os": [
                      "windows-2012r2"
                    ],
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "patch-deployments": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "pause": {
                      "commands": {},
                      "flags": {}
                    },
                    "resume": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "patch-jobs": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "execute": {
                      "commands": {},
                      "flags": {
                        "--apt-dist": "bool",
                        "--apt-excludes": "value",
                        "--apt-exclusive-packages": "value",
                        "--async": "bool",
                        "--description": "value",
                        "--display-name": "value",
                        "--dry-run": "bool",
                        "--duration": "value",
                        "--instance-filter": "value",
                        "--instance-filter-all": "bool",
                        "--instance-filter-group-labels": "value",
                        "--instance-filter-name-prefixes": "value",
                        "--instance-filter-names": "value",
                        "--instance-filter-zones": "value",
                        "--mig-instances-allowed": "bool",
                        "--post-patch-linux-executable": "value",
                        "--post-patch-linux-success-codes": "value",
                        "--post-patch-windows-executable": "value",
                        "--post-patch-windows-success-codes": "value",
                        "--pre-patch-linux-executable": "value",
                        "--pre-patch-linux-success-codes": "value",
                        "--pre-patch-windows-executable": "value",
                        "--pre-patch-windows-success-codes": "value",
                        "--reboot-config": [
                          "always",
                          "default",
                          "never"
                        ],
                        "--retry": "bool",
                        "--rollout-disruption-budget": "value",
                        "--rollout-disruption-budget-percent": "value",
                        "--rollout-mode": [
                          "concurrent-zones",
                          "zone-by-zone"
                        ],
                        "--windows-classifications": "value",
                        "--windows-excludes": "value",
                        "--windows-exclusive-patches": "value",
                        "--yum-excludes": "value",
                        "--yum-exclusive-packages": "value",
                        "--yum-minimal": "bool",
                        "--yum-security": "bool",
                        "--zypper-categories": "value",
                        "--zypper-excludes": "value",
                        "--zypper-exclusive-patches": "value",
                        "--zypper-severities": "value",
                        "--zypper-with-optional": "bool",
                        "--zypper-with-update": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-instance-details": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "troubleshoot": {
                  "commands": {},
                  "flags": {
                    "--enable-log-analysis": "bool",
                    "--zone": "dynamic"
                  }
                },
                "vulnerability-reports": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "os-login": {
              "commands": {
                "describe-profile": {
                  "commands": {},
                  "flags": {}
                },
                "remove-profile": {
                  "commands": {},
                  "flags": {
                    "--operating-system": [
                      "linux",
                      "windows"
                    ]
                  }
                },
                "ssh-keys": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value",
                        "--ttl": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value",
                        "--ttl": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "packet-mirrorings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--collector-ilb": "value",
                    "--description": "value",
                    "--enable": "bool",
                    "--filter-cidr-ranges": "value",
                    "--filter-direction": [
                      "both",
                      "egress",
                      "ingress"
                    ],
                    "--filter-protocols": "value",
                    "--mirrored-instances": "value",
                    "--mirrored-subnets": "value",
                    "--mirrored-tags": "value",
                    "--network": "value",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-filter-cidr-ranges": "value",
                    "--add-filter-protocols": "value",
                    "--add-mirrored-instances": "value",
                    "--add-mirrored-subnets": "value",
                    "--add-mirrored-tags": "value",
                    "--async": "bool",
                    "--clear-filter-cidr-ranges": "bool",
                    "--clear-filter-protocols": "bool",
                    "--clear-mirrored-instances": "bool",
                    "--clear-mirrored-subnets": "bool",
                    "--clear-mirrored-tags": "bool",
                    "--collector-ilb": "value",
                    "--description": "value",
                    "--enable": "bool",
                    "--filter-direction": [
                      "both",
                      "egress",
                      "ingress"
                    ],
                    "--region": "dynamic",
                    "--remove-filter-cidr-ranges": "value",
                    "--remove-filter-protocols": "value",
                    "--remove-mirrored-instances": "value",
                    "--remove-mirrored-subnets": "value",
                    "--remove-mirrored-tags": "value",
                    "--set-filter-cidr-ranges": "value",
                    "--set-filter-protocols": "value",
                    "--set-mirrored-instances": "value",
                    "--set-mirrored-subnets": "value",
                    "--set-mirrored-tags": "value"
                  }
                }
              },
              "flags": {}
            },
            "project-info": {
              "commands": {
                "add-metadata": {
                  "commands": {},
                  "flags": {
                    "--metadata": "value",
                    "--metadata-from-file": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "remove-metadata": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--keys": "value"
                  }
                },
                "set-default-service-account": {
                  "commands": {},
                  "flags": {
                    "--no-service-account": "bool",
                    "--service-account": "value"
                  }
                },
                "set-usage-bucket": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--no-bucket": "bool",
                    "--prefix": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--default-network-tier": [
                      "FIXED_STANDARD",
                      "PREMIUM",
                      "STANDARD"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "public-advertised-prefixes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--dns-verification-ip": "value",
                    "--pdp-scope": [
                      "GLOBAL",
                      "REGIONAL"
                    ],
                    "--range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--announce-prefix": "bool",
                    "--status": [
                      "ptr-configured"
                    ],
                    "--withdraw-prefix": "bool"
                  }
                }
              },
              "flags": {}
            },
            "public-delegated-prefixes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-live-migration": "bool",
                    "--global": "bool",
                    "--public-advertised-prefix": "value",
                    "--range": "value",
                    "--region": "dynamic"
                  }
                },
                "delegated-sub-prefixes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--create-addresses": "bool",
                        "--delegatee-project": "value",
                        "--description": "value",
                        "--global-public-delegated-prefix": "bool",
                        "--public-delegated-prefix": "value",
                        "--public-delegated-prefix-region": "dynamic",
                        "--range": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--global-public-delegated-prefix": "bool",
                        "--public-delegated-prefix": "value",
                        "--public-delegated-prefix-region": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--announce-prefix": "bool",
                    "--region": "dynamic",
                    "--withdraw-prefix": "bool"
                  }
                }
              },
              "flags": {}
            },
            "queued-resources": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--async": "bool",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--count": "value",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-display-device": "bool",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--erase-windows-vss-signature": "bool",
                    "--host-error-timeout-seconds": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--local-ssd-recovery-timeout": "value",
                    "--location-policy": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-count": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--name-pattern": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-dns": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--numa-node-count": "value",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--predefined-names": "value",
                    "--preemptible": "bool",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-dns": "bool",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--secure-tags": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--source-snapshot": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--target-distribution-shape": [
                      "ANY",
                      "ANY_SINGLE_ZONE",
                      "BALANCED"
                    ],
                    "--termination-time": "value",
                    "--threads-per-core": "value",
                    "--valid-until-duration": "value",
                    "--valid-until-time": "value",
                    "--visible-core-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "reservations": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--description": "value",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--maintenance-freeze-duration": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--min-cpu-platform": "value",
                    "--require-specific-reservation": "bool",
                    "--resource-policies": "value",
                    "--share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--source-instance-template": "value",
                    "--vm-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-share-with": "value",
                    "--remove-share-with": "value",
                    "--share-with": "value",
                    "--vm-count": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "reset-windows-password": {
              "commands": {},
              "flags": {
                "--user": "value",
                "--zone": "dynamic"
              }
            },
            "resource-policies": {
              "commands": {
                "create": {
                  "commands": {
                    "disk-consistency-group": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--region": "dynamic"
                      }
                    },
                    "group-placement": {
                      "commands": {},
                      "flags": {
                        "--availability-domain-count": "value",
                        "--collocation": [
                          "clustered",
                          "collocated",
                          "unspecified-collocation"
                        ],
                        "--description": "value",
                        "--max-distance": "value",
                        "--region": "dynamic",
                        "--scope": [
                          "host",
                          "unspecified-scope"
                        ],
                        "--tpu-topology": "value",
                        "--vm-count": "value"
                      }
                    },
                    "instance-schedule": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--end-date": "value",
                        "--initiation-date": "value",
                        "--region": "dynamic",
                        "--timezone": "value",
                        "--vm-start-schedule": "value",
                        "--vm-stop-schedule": "value"
                      }
                    },
                    "snapshot-schedule": {
                      "commands": {},
                      "flags": {
                        "--daily-schedule": "bool",
                        "--description": "value",
                        "--guest-flush": "bool",
                        "--hourly-schedule": "value",
                        "--max-retention-days": "value",
                        "--on-source-disk-delete": [
                          "apply-retention-policy",
                          "keep-auto-snapshots"
                        ],
                        "--region": "dynamic",
                        "--snapshot-labels": "value",
                        "--start-time": "value",
                        "--storage-location": "value",
                        "--weekly-schedule": [
                          "friday",
                          "monday",
                          "saturday",
                          "sunday",
                          "thursday",
                          "tuesday",
                          "wednesday"
                        ],
                        "--weekly-schedule-from-file": "value"
                      }
                    },
                    "vm-maintenance": {
                      "commands": {
                        "concurrency-limit": {
                          "commands": {},
                          "flags": {
                            "--description": "value",
                            "--max-percent": "value",
                            "--region": "dynamic"
                          }
                        },
                        "maintenance-window": {
                          "commands": {},
                          "flags": {
                            "--daily-window": "bool",
                            "--description": "value",
                            "--region": "dynamic",
                            "--start-time": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "create-snapshot-schedule": {
                  "commands": {},
                  "flags": {
                    "--daily-schedule": "bool",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--hourly-schedule": "value",
                    "--max-retention-days": "value",
                    "--on-source-disk-delete": [
                      "apply-retention-policy",
                      "keep-auto-snapshots"
                    ],
                    "--region": "dynamic",
                    "--snapshot-labels": "value",
                    "--start-time": "value",
                    "--storage-location": "value",
                    "--weekly-schedule": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--weekly-schedule-from-file": "value"
                  }
                },
                "create-vm-maintenance": {
                  "commands": {},
                  "flags": {
                    "--daily-window": "bool",
                    "--description": "value",
                    "--region": "dynamic",
                    "--start-time": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {
                    "snapshot-schedule": {
                      "commands": {},
                      "flags": {
                        "--daily-schedule": "bool",
                        "--description": "value",
                        "--hourly-schedule": "value",
                        "--region": "dynamic",
                        "--snapshot-labels": "value",
                        "--start-time": "value",
                        "--weekly-schedule": [
                          "friday",
                          "monday",
                          "saturday",
                          "sunday",
                          "thursday",
                          "tuesday",
                          "wednesday"
                        ],
                        "--weekly-schedule-from-file": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "routers": {
              "commands": {
                "add-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--advertised-route-priority": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--async": "bool",
                    "--bfd-min-receive-interval": "value",
                    "--bfd-min-transmit-interval": "value",
                    "--bfd-multiplier": "value",
                    "--bfd-session-initialization-mode": [
                      "ACTIVE",
                      "DISABLED",
                      "PASSIVE"
                    ],
                    "--enable-ipv6": "bool",
                    "--enabled": "bool",
                    "--instance": "dynamic",
                    "--instance-zone": "dynamic",
                    "--interface": "value",
                    "--ipv6-nexthop-address": "value",
                    "--md5-authentication-key": "value",
                    "--peer-asn": "value",
                    "--peer-ip-address": "value",
                    "--peer-ipv6-nexthop-address": "value",
                    "--peer-name": "value",
                    "--region": "dynamic",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "add-interface": {
                  "commands": {},
                  "flags": {
                    "--interconnect-attachment": "dynamic",
                    "--interconnect-attachment-region": "dynamic",
                    "--interface-name": "value",
                    "--ip-address": "value",
                    "--mask-length": "value",
                    "--redundant-interface": "value",
                    "--region": "dynamic",
                    "--subnetwork": "dynamic",
                    "--subnetwork-region": "dynamic",
                    "--vpn-tunnel": "dynamic",
                    "--vpn-tunnel-region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--asn": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--encrypted-interconnect-router": "bool",
                    "--keepalive-interval": "value",
                    "--network": "dynamic",
                    "--region": "dynamic",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-nat-mapping-info": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--nat-name": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "nats": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-allocate-nat-external-ips": "bool",
                        "--enable-dynamic-port-allocation": "bool",
                        "--enable-endpoint-independent-mapping": "bool",
                        "--enable-logging": "bool",
                        "--endpoint-types": "value",
                        "--icmp-idle-timeout": "value",
                        "--log-filter": [
                          "ALL",
                          "ERRORS_ONLY",
                          "TRANSLATIONS_ONLY"
                        ],
                        "--max-ports-per-vm": "value",
                        "--min-ports-per-vm": "value",
                        "--nat-all-subnet-ip-ranges": "bool",
                        "--nat-custom-subnet-ip-ranges": "value",
                        "--nat-external-ip-pool": "value",
                        "--nat-primary-subnet-ip-ranges": "bool",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--rules": "value",
                        "--tcp-established-idle-timeout": "value",
                        "--tcp-time-wait-timeout": "value",
                        "--tcp-transitory-idle-timeout": "value",
                        "--type": [
                          "PRIVATE",
                          "PUBLIC"
                        ],
                        "--udp-idle-timeout": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "rules": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--match": "value",
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--source-nat-active-ips": "value",
                            "--source-nat-active-ranges": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--clear-source-nat-drain-ips": "bool",
                            "--clear-source-nat-drain-ranges": "bool",
                            "--match": "value",
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--source-nat-active-ips": "value",
                            "--source-nat-active-ranges": "value",
                            "--source-nat-drain-ips": "value",
                            "--source-nat-drain-ranges": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-allocate-nat-external-ips": "bool",
                        "--clear-icmp-idle-timeout": "bool",
                        "--clear-max-ports-per-vm": "bool",
                        "--clear-min-ports-per-vm": "bool",
                        "--clear-nat-external-drain-ip-pool": "bool",
                        "--clear-tcp-established-idle-timeout": "bool",
                        "--clear-tcp-time-wait-timeout": "bool",
                        "--clear-tcp-transitory-idle-timeout": "bool",
                        "--clear-udp-idle-timeout": "bool",
                        "--enable-dynamic-port-allocation": "bool",
                        "--enable-endpoint-independent-mapping": "bool",
                        "--enable-logging": "bool",
                        "--icmp-idle-timeout": "value",
                        "--log-filter": [
                          "ALL",
                          "ERRORS_ONLY",
                          "TRANSLATIONS_ONLY"
                        ],
                        "--max-ports-per-vm": "value",
                        "--min-ports-per-vm": "value",
                        "--nat-all-subnet-ip-ranges": "bool",
                        "--nat-custom-subnet-ip-ranges": "value",
                        "--nat-external-drain-ip-pool": "value",
                        "--nat-external-ip-pool": "value",
                        "--nat-primary-subnet-ip-ranges": "bool",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--rules": "value",
                        "--tcp-established-idle-timeout": "value",
                        "--tcp-time-wait-timeout": "value",
                        "--tcp-transitory-idle-timeout": "value",
                        "--udp-idle-timeout": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--peer-name": "value",
                    "--peer-names": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-interface": {
                  "commands": {},
                  "flags": {
                    "--interface-name": "value",
                    "--interface-names": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-advertisement-groups": "value",
                    "--add-advertisement-ranges": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--asn": "value",
                    "--async": "bool",
                    "--keepalive-interval": "value",
                    "--region": "dynamic",
                    "--remove-advertisement-groups": "value",
                    "--remove-advertisement-ranges": "value",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "update-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--add-advertisement-groups": "value",
                    "--add-advertisement-ranges": "value",
                    "--advertised-route-priority": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--async": "bool",
                    "--bfd-min-receive-interval": "value",
                    "--bfd-min-transmit-interval": "value",
                    "--bfd-multiplier": "value",
                    "--bfd-session-initialization-mode": [
                      "ACTIVE",
                      "DISABLED",
                      "PASSIVE"
                    ],
                    "--clear-md5-authentication-key": "bool",
                    "--enable-ipv6": "bool",
                    "--enabled": "bool",
                    "--interface": "value",
                    "--ip-address": "value",
                    "--ipv6-nexthop-address": "value",
                    "--md5-authentication-key": "value",
                    "--peer-asn": "value",
                    "--peer-ip-address": "value",
                    "--peer-ipv6-nexthop-address": "value",
                    "--peer-name": "value",
                    "--region": "dynamic",
                    "--remove-advertisement-groups": "value",
                    "--remove-advertisement-ranges": "value",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "update-interface": {
                  "commands": {},
                  "flags": {
                    "--interconnect-attachment": "dynamic",
                    "--interconnect-attachment-region": "dynamic",
                    "--interface-name": "value",
                    "--ip-address": "value",
                    "--mask-length": "value",
                    "--region": "dynamic",
                    "--vpn-tunnel": "dynamic",
                    "--vpn-tunnel-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "routes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--destination-range": "value",
                    "--network": "value",
                    "--next-hop-address": "value",
                    "--next-hop-gateway": "dynamic",
                    "--next-hop-ilb": "value",
                    "--next-hop-ilb-region": "value",
                    "--next-hop-instance": "value",
                    "--next-hop-instance-zone": "value",
                    "--next-hop-vpn-tunnel": "value",
                    "--next-hop-vpn-tunnel-region": "value",
                    "--priority": "value",
                    "--tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "scp": {
              "commands": {},
              "flags": {
                "--compress": "bool",
                "--dest-group": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--internal-ip": "bool",
                "--network": "value",
                "--plain": "bool",
                "--port": "value",
                "--recurse": "bool",
                "--region": "value",
                "--scp-flag": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--tunnel-through-iap": "bool",
                "--zone": "value"
              }
            },
            "security-policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--type": [
                      "CLOUD_ARMOR",
                      "CLOUD_ARMOR_EDGE",
                      "CLOUD_ARMOR_INTERNAL_SERVICE",
                      "CLOUD_ARMOR_NETWORK"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-preconfigured-expression-sets": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "add-preconfig-waf-exclusion": {
                      "commands": {},
                      "flags": {
                        "--request-cookie-to-exclude": "value",
                        "--request-header-to-exclude": "value",
                        "--request-query-param-to-exclude": "value",
                        "--request-uri-to-exclude": "value",
                        "--security-policy": "dynamic",
                        "--target-rule-ids": "value",
                        "--target-rule-set": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "deny-403",
                          "deny-404",
                          "deny-502",
                          "fairshare",
                          "rate-based-ban",
                          "redirect",
                          "redirect-to-recaptcha",
                          "throttle"
                        ],
                        "--ban-duration-sec": "value",
                        "--ban-threshold-count": "value",
                        "--ban-threshold-interval-sec": "value",
                        "--conform-action": [
                          "allow"
                        ],
                        "--description": "value",
                        "--enforce-on-key": [
                          "all",
                          "http-cookie",
                          "http-header",
                          "http-path",
                          "ip",
                          "region-code",
                          "sni",
                          "xff-ip"
                        ],
                        "--enforce-on-key-configs": "value",
                        "--enforce-on-key-name": "value",
                        "--exceed-action": [
                          "deny",
                          "deny-403",
                          "deny-404",
                          "deny-429",
                          "deny-502",
                          "redirect"
                        ],
                        "--exceed-action-rpc-status-code": "value",
                        "--exceed-action-rpc-status-message": "value",
                        "--exceed-redirect-target": "value",
                        "--exceed-redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--expression": "value",
                        "--preview": "bool",
                        "--rate-limit-threshold-count": "value",
                        "--rate-limit-threshold-interval-sec": "value",
                        "--redirect-target": "value",
                        "--redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--request-headers-to-add": "value",
                        "--security-policy": "dynamic",
                        "--src-ip-ranges": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--security-policy": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--security-policy": "dynamic"
                      }
                    },
                    "remove-preconfig-waf-exclusion": {
                      "commands": {},
                      "flags": {
                        "--request-cookie-to-exclude": "value",
                        "--request-header-to-exclude": "value",
                        "--request-query-param-to-exclude": "value",
                        "--request-uri-to-exclude": "value",
                        "--security-policy": "dynamic",
                        "--target-rule-ids": "value",
                        "--target-rule-set": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "deny-403",
                          "deny-404",
                          "deny-502",
                          "fairshare",
                          "rate-based-ban",
                          "redirect",
                          "redirect-to-recaptcha",
                          "throttle"
                        ],
                        "--ban-duration-sec": "value",
                        "--ban-threshold-count": "value",
                        "--ban-threshold-interval-sec": "value",
                        "--conform-action": [
                          "allow"
                        ],
                        "--description": "value",
                        "--enforce-on-key": [
                          "all",
                          "http-cookie",
                          "http-header",
                          "http-path",
                          "ip",
                          "region-code",
                          "sni",
                          "xff-ip"
                        ],
                        "--enforce-on-key-configs": "value",
                        "--enforce-on-key-name": "value",
                        "--exceed-action": [
                          "deny",
                          "deny-403",
                          "deny-404",
                          "deny-429",
                          "deny-502",
                          "redirect"
                        ],
                        "--exceed-action-rpc-status-code": "value",
                        "--exceed-action-rpc-status-message": "value",
                        "--exceed-redirect-target": "value",
                        "--exceed-redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--expression": "value",
                        "--preview": "bool",
                        "--rate-limit-threshold-count": "value",
                        "--rate-limit-threshold-interval-sec": "value",
                        "--redirect-target": "value",
                        "--redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--request-headers-to-add": "value",
                        "--security-policy": "dynamic",
                        "--src-ip-ranges": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ddos-protection": [
                      "ADVANCED",
                      "ADVANCED_PREVIEW",
                      "STANDARD"
                    ],
                    "--description": "value",
                    "--enable-layer7-ddos-defense": "bool",
                    "--enable-ml": "bool",
                    "--global": "bool",
                    "--json-custom-content-types": "value",
                    "--json-parsing": [
                      "DISABLED",
                      "STANDARD"
                    ],
                    "--layer7-ddos-defense-auto-deploy-confidence-threshold": "value",
                    "--layer7-ddos-defense-auto-deploy-expiration-sec": "value",
                    "--layer7-ddos-defense-auto-deploy-impacted-baseline-threshold": "value",
                    "--layer7-ddos-defense-auto-deploy-load-threshold": "value",
                    "--layer7-ddos-defense-rule-visibility": [
                      "PREMIUM",
                      "STANDARD"
                    ],
                    "--log-level": [
                      "NORMAL",
                      "VERBOSE"
                    ],
                    "--network-ddos-protection": [
                      "ADVANCED",
                      "ADVANCED_PREVIEW",
                      "STANDARD"
                    ],
                    "--recaptcha-redirect-site-key": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "service-attachments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--consumer-accept-list": "value",
                    "--consumer-reject-list": "value",
                    "--description": "value",
                    "--domain-names": "value",
                    "--enable-proxy-protocol": "bool",
                    "--nat-subnets": "dynamic",
                    "--nat-subnets-region": "dynamic",
                    "--producer-forwarding-rule": "dynamic",
                    "--producer-forwarding-rule-region": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--consumer-accept-list": "value",
                    "--consumer-reject-list": "value",
                    "--description": "value",
                    "--enable-proxy-protocol": "bool",
                    "--nat-subnets": "dynamic",
                    "--nat-subnets-region": "dynamic",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "shared-vpc": {
              "commands": {
                "associated-projects": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--host-project": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--host-project": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {}
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "get-host-project": {
                  "commands": {},
                  "flags": {}
                },
                "list-associated-resources": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "organizations": {
                  "commands": {
                    "list-host-projects": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "sign-url": {
              "commands": {},
              "flags": {
                "--expires-in": "value",
                "--key-file": "value",
                "--key-name": "value",
                "--validate": "bool"
              }
            },
            "snapshots": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--chain-name": "value",
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--labels": "value",
                    "--max-retention-days": "value",
                    "--snapshot-type": [
                      "ARCHIVE",
                      "STANDARD"
                    ],
                    "--source-disk": "dynamic",
                    "--source-disk-key-file": "value",
                    "--source-disk-region": "dynamic",
                    "--source-disk-zone": "dynamic",
                    "--source-instant-snapshot": "dynamic",
                    "--source-instant-snapshot-region": "dynamic",
                    "--source-instant-snapshot-zone": "dynamic",
                    "--storage-location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "sole-tenancy": {
              "commands": {
                "node-groups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--zone": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--autoscaler-mode": [
                          "off",
                          "on",
                          "only-scale-out"
                        ],
                        "--description": "value",
                        "--maintenance-policy": [
                          "default",
                          "migrate-within-node-group",
                          "restart-in-place"
                        ],
                        "--maintenance-window-start-time": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-template": "value",
                        "--share-setting": [
                          "local",
                          "organization",
                          "projects"
                        ],
                        "--share-with": "value",
                        "--target-size": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--share-settings": "bool",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-nodes": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--zone": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "simulate-maintenance-event": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--nodes": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-nodes": "value",
                        "--autoscaler-mode": [
                          "off",
                          "on",
                          "only-scale-out"
                        ],
                        "--delete-nodes": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-template": "value",
                        "--share-setting": [
                          "local",
                          "organization",
                          "projects"
                        ],
                        "--share-with": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-templates": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator": "value",
                        "--cpu-overcommit-type": [
                          "enabled",
                          "none"
                        ],
                        "--description": "value",
                        "--disk": "value",
                        "--node-affinity-labels": "value",
                        "--node-requirements": "value",
                        "--node-type": "value",
                        "--region": "dynamic",
                        "--server-binding": [
                          "restart-node-on-any-server",
                          "restart-node-on-minimal-servers"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--command": "value",
                "--container": "value",
                "--dest-group": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--internal-ip": "bool",
                "--network": "value",
                "--plain": "bool",
                "--region": "value",
                "--ssh-flag": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--troubleshoot": "bool",
                "--tunnel-through-iap": "bool",
                "--zone": "dynamic"
              }
            },
            "ssl-certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--description": "value",
                    "--domains": "value",
                    "--global": "bool",
                    "--private-key": "value",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "ssl-policies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--custom-features": "value",
                    "--description": "value",
                    "--global": "bool",
                    "--min-tls-version": [
                      "1.0",
                      "1.1",
                      "1.2"
                    ],
                    "--profile": [
                      "COMPATIBLE",
                      "CUSTOM",
                      "MODERN",
                      "RESTRICTED"
                    ],
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-available-features": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--custom-features": "value",
                    "--global": "bool",
                    "--min-tls-version": [
                      "1.0",
                      "1.1",
                      "1.2"
                    ],
                    "--profile": [
                      "COMPATIBLE",
                      "CUSTOM",
                      "MODERN",
                      "RESTRICTED"
                    ],
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "start-iap-tunnel": {
              "commands": {},
              "flags": {
                "--dest-group": "value",
                "--iap-tunnel-disable-connection-check": "bool",
                "--local-host-port": "value",
                "--network": "value",
                "--region": "value",
                "--zone": "dynamic"
              }
            },
            "target-grpc-proxies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global-url-map": "bool",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic",
                    "--validate-for-proxyless": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "target-http-proxies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--global-url-map": "bool",
                    "--proxy-bind": "bool",
                    "--region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--global-url-map": "bool",
                    "--region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-https-proxies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate-map": "dynamic",
                    "--description": "value",
                    "--global": "bool",
                    "--global-ssl-certificates": "bool",
                    "--global-ssl-policy": "bool",
                    "--global-url-map": "bool",
                    "--proxy-bind": "bool",
                    "--quic-override": [
                      "DISABLE",
                      "ENABLE",
                      "NONE"
                    ],
                    "--region": "dynamic",
                    "--ssl-certificates": "dynamic",
                    "--ssl-certificates-region": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate-map": "dynamic",
                    "--clear-certificate-map": "bool",
                    "--clear-ssl-policy": "bool",
                    "--global": "bool",
                    "--global-ssl-certificates": "bool",
                    "--global-ssl-policy": "bool",
                    "--global-url-map": "bool",
                    "--quic-override": [
                      "DISABLE",
                      "ENABLE",
                      "NONE"
                    ],
                    "--region": "dynamic",
                    "--ssl-certificates": "dynamic",
                    "--ssl-certificates-region": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-instances": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--instance": "dynamic",
                    "--instance-zone": "dynamic",
                    "--network": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-pools": {
              "commands": {
                "add-health-checks": {
                  "commands": {},
                  "flags": {
                    "--http-health-check": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "add-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "dynamic",
                    "--instances-zone": "dynamic",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backup-pool": "value",
                    "--description": "value",
                    "--failover-ratio": "value",
                    "--health-check": "value",
                    "--http-health-check": "dynamic",
                    "--region": "dynamic",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_PROTO",
                      "NONE"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-health": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-health-checks": {
                  "commands": {},
                  "flags": {
                    "--http-health-check": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "remove-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "dynamic",
                    "--instances-zone": "dynamic",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "set-backup": {
                  "commands": {},
                  "flags": {
                    "--backup-pool": "dynamic",
                    "--failover-ratio": "value",
                    "--no-backup-pool": "bool",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-ssl-proxies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--certificate-map": "dynamic",
                    "--description": "value",
                    "--global-ssl-policy": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--ssl-certificates": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--certificate-map": "dynamic",
                    "--clear-certificate-map": "bool",
                    "--clear-ssl-certificates": "bool",
                    "--clear-ssl-policy": "bool",
                    "--global-ssl-policy": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--ssl-certificates": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-tcp-proxies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--backend-service-region": "dynamic",
                    "--description": "value",
                    "--global": "bool",
                    "--global-backend-service": "bool",
                    "--proxy-bind": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "target-vpn-gateways": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--network": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tpus": {
              "commands": {
                "accelerator-types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-type": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--model-base-path": "value",
                    "--model-config-file": "value",
                    "--model-name": "value",
                    "--network": "value",
                    "--platform-config-file": "value",
                    "--preemptible": "bool",
                    "--range": "value",
                    "--reserved": "bool",
                    "--use-service-networking": "bool",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "execution-groups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--disk-size": "value",
                        "--dry-run": "bool",
                        "--forward-ports": "bool",
                        "--gce-image": "value",
                        "--machine-type": "value",
                        "--name": "value",
                        "--network": "value",
                        "--preemptible": "bool",
                        "--preemptible-vm": "bool",
                        "--tf-version": "value",
                        "--tpu-only": "bool",
                        "--use-dl-images": "bool",
                        "--use-with-notebook": "bool",
                        "--vm-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--tpu-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "resume": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--network": "value",
                        "--preemptible": "bool",
                        "--tf-version": "value",
                        "--vm-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "suspend": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "queued-resources": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--async": "bool",
                        "--best-effort": "bool",
                        "--guaranteed": "bool",
                        "--internal-ips": "bool",
                        "--node-count": "value",
                        "--node-id": "value",
                        "--node-prefix": "value",
                        "--reserved": "bool",
                        "--runtime-version": "value",
                        "--valid-after-duration": "value",
                        "--valid-after-time": "value",
                        "--valid-until-duration": "value",
                        "--valid-until-time": "value",
                        "--zone": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--zone": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "reimage": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "topologies": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "tpu-vm": {
                  "commands": {
                    "accelerator-types": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--zone": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "attach-disk": {
                      "commands": {},
                      "flags": {
                        "--disk": "value",
                        "--mode": [
                          "read-only",
                          "read-write"
                        ],
                        "--zone": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--async": "bool",
                        "--data-disk": "value",
                        "--description": "value",
                        "--internal-ips": "bool",
                        "--labels": "value",
                        "--metadata": "value",
                        "--metadata-from-file": "value",
                        "--network": "value",
                        "--preemptible": "bool",
                        "--range": "value",
                        "--reserved": "bool",
                        "--scopes": "value",
                        "--service-account": "value",
                        "--shielded-secure-boot": "bool",
                        "--subnetwork": "value",
                        "--tags": "value",
                        "--topology": "value",
                        "--type": [
                          "v2",
                          "v3",
                          "v4"
                        ],
                        "--zone": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--zone": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "detach-disk": {
                      "commands": {},
                      "flags": {
                        "--disk": "value",
                        "--zone": "value"
                      }
                    },
                    "get-guest-attributes": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--query-path": "value",
                        "--sort-by": "value",
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "scp": {
                      "commands": {},
                      "flags": {
                        "--batch-size": "value",
                        "--compress": "bool",
                        "--dry-run": "bool",
                        "--force-key-file-overwrite": "bool",
                        "--internal-ip": "bool",
                        "--plain": "bool",
                        "--recurse": "bool",
                        "--scp-flag": "value",
                        "--ssh-key-expiration": "value",
                        "--ssh-key-expire-after": "value",
                        "--ssh-key-file": "value",
                        "--strict-host-key-checking": [
                          "ask",
                          "no",
                          "yes"
                        ],
                        "--tunnel-through-iap": "bool",
                        "--worker": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "service-identity": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "simulate-maintenance-event": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--workers": "value",
                        "--zone": "value"
                      }
                    },
                    "ssh": {
                      "commands": {},
                      "flags": {
                        "--batch-size": "value",
                        "--command": "value",
                        "--dry-run": "bool",
                        "--force-key-file-overwrite": "bool",
                        "--internal-ip": "bool",
                        "--output-directory": "value",
                        "--plain": "bool",
                        "--ssh-flag": "value",
                        "--ssh-key-expiration": "value",
                        "--ssh-key-expire-after": "value",
                        "--ssh-key-file": "value",
                        "--strict-host-key-checking": [
                          "ask",
                          "no",
                          "yes"
                        ],
                        "--tunnel-through-iap": "bool",
                        "--worker": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "start": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--zone": "value"
                      }
                    },
                    "stop": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--zone": "value"
                      }
                    },
                    "versions": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--zone": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "versions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "url-maps": {
              "commands": {
                "add-host-rule": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--hosts": "value",
                    "--path-matcher-name": "value",
                    "--region": "dynamic"
                  }
                },
                "add-path-matcher": {
                  "commands": {},
                  "flags": {
                    "--backend-bucket-path-rules": "value",
                    "--backend-service-path-rules": "value",
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--delete-orphaned-path-matcher": "bool",
                    "--description": "value",
                    "--existing-host": "value",
                    "--global": "bool",
                    "--new-hosts": "value",
                    "--path-matcher-name": "value",
                    "--path-rules": "value",
                    "--region": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--global": "bool",
                        "--path": "value",
                        "--region": "dynamic",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "edit": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "invalidate-cdn-cache": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--global": "bool",
                    "--host": "value",
                    "--path": "value",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-cdn-cache-invalidations": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-host-rule": {
                  "commands": {},
                  "flags": {
                    "--delete-orphaned-path-matcher": "bool",
                    "--global": "bool",
                    "--host": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-path-matcher": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--path-matcher-name": "value",
                    "--region": "dynamic"
                  }
                },
                "set-default-service": {
                  "commands": {},
                  "flags": {
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "validate": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED"
                    ],
                    "--region": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpn-gateways": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--gateway-ip-version": [
                      "IPV4",
                      "IPV6"
                    ],
                    "--interconnect-attachments": "value",
                    "--network": "dynamic",
                    "--region": "dynamic",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpn-tunnels": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--ike-version": [
                      1,
                      2
                    ],
                    "--interface": [
                      0,
                      1
                    ],
                    "--local-traffic-selector": "value",
                    "--peer-address": "value",
                    "--peer-external-gateway": "dynamic",
                    "--peer-external-gateway-interface": [
                      0,
                      1,
                      2,
                      3
                    ],
                    "--peer-gcp-gateway": "dynamic",
                    "--peer-gcp-gateway-region": "dynamic",
                    "--region": "dynamic",
                    "--remote-traffic-selector": "value",
                    "--router": "dynamic",
                    "--router-region": "dynamic",
                    "--shared-secret": "value",
                    "--target-vpn-gateway": "dynamic",
                    "--target-vpn-gateway-region": "dynamic",
                    "--vpn-gateway": "dynamic",
                    "--vpn-gateway-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "config": {
          "commands": {
            "configurations": {
              "commands": {
                "activate": {
                  "commands": {},
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--activate": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--all": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                },
                "rename": {
                  "commands": {},
                  "flags": {
                    "--new-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "get": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            },
            "set": {
              "commands": {},
              "flags": {
                "--installation": "bool"
              }
            },
            "unset": {
              "commands": {},
              "flags": {
                "--installation": "bool"
              }
            }
          },
          "flags": {}
        },
        "container": {
          "commands": {
            "attached": {
              "commands": {
                "clusters": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--ignore-errors": "bool",
                        "--location": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "generate-install-manifest": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--output-file": "value",
                        "--platform-version": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--context": "value",
                        "--distribution": "value",
                        "--fleet-membership": "value",
                        "--fleet-membership-location": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--platform-version": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "register": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--context": "value",
                        "--description": "value",
                        "--distribution": "value",
                        "--enable-managed-prometheus": "bool",
                        "--fleet-project": "value",
                        "--has-private-issuer": "bool",
                        "--issuer-url": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--logging": "value",
                        "--platform-version": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--clear-description": "bool",
                        "--description": "value",
                        "--disable-managed-prometheus": "bool",
                        "--enable-managed-prometheus": "bool",
                        "--location": "value",
                        "--logging": "value",
                        "--platform-version": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-server-config": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "aws": {
              "commands": {
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--aws-region": "value",
                        "--cluster-version": "value",
                        "--config-encryption-kms-key-arn": "value",
                        "--database-encryption-kms-key-arn": "value",
                        "--description": "value",
                        "--enable-managed-prometheus": "bool",
                        "--fleet-project": "value",
                        "--iam-instance-profile": "value",
                        "--instance-placement": [
                          "dedicated",
                          "default",
                          "host"
                        ],
                        "--instance-type": "value",
                        "--location": "value",
                        "--logging": "value",
                        "--main-volume-iops": "value",
                        "--main-volume-kms-key-arn": "value",
                        "--main-volume-size": "value",
                        "--main-volume-type": [
                          "gp2",
                          "gp3"
                        ],
                        "--pod-address-cidr-blocks": "value",
                        "--proxy-secret-arn": "value",
                        "--proxy-secret-version-id": "value",
                        "--role-arn": "value",
                        "--role-session-name": "value",
                        "--root-volume-iops": "value",
                        "--root-volume-kms-key-arn": "value",
                        "--root-volume-size": "value",
                        "--root-volume-type": [
                          "gp2",
                          "gp3"
                        ],
                        "--security-group-ids": "value",
                        "--service-address-cidr-blocks": "value",
                        "--ssh-ec2-key-pair": "value",
                        "--subnet-ids": "value",
                        "--tags": "value",
                        "--validate-only": "bool",
                        "--vpc-id": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--location": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--private-endpoint": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--clear-annotations": "bool",
                        "--clear-description": "bool",
                        "--clear-proxy-config": "bool",
                        "--clear-security-group-ids": "bool",
                        "--clear-ssh-ec2-key-pair": "bool",
                        "--clear-tags": "bool",
                        "--cluster-version": "value",
                        "--config-encryption-kms-key-arn": "value",
                        "--description": "value",
                        "--disable-managed-prometheus": "bool",
                        "--enable-managed-prometheus": "bool",
                        "--iam-instance-profile": "value",
                        "--instance-type": "value",
                        "--location": "value",
                        "--logging": "value",
                        "--proxy-secret-arn": "value",
                        "--proxy-secret-version-id": "value",
                        "--role-arn": "value",
                        "--role-session-name": "value",
                        "--root-volume-iops": "value",
                        "--root-volume-kms-key-arn": "value",
                        "--root-volume-size": "value",
                        "--root-volume-type": [
                          "gp2",
                          "gp3"
                        ],
                        "--security-group-ids": "value",
                        "--ssh-ec2-key-pair": "value",
                        "--tags": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-server-config": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "node-pools": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--autoscaling-metrics": "value",
                        "--autoscaling-metrics-granularity": "value",
                        "--cluster": "value",
                        "--config-encryption-kms-key-arn": "value",
                        "--iam-instance-profile": "value",
                        "--image-type": "value",
                        "--instance-placement": [
                          "dedicated",
                          "default",
                          "host"
                        ],
                        "--instance-type": "value",
                        "--location": "value",
                        "--max-nodes": "value",
                        "--max-pods-per-node": "value",
                        "--min-nodes": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--node-version": "value",
                        "--proxy-secret-arn": "value",
                        "--proxy-secret-version-id": "value",
                        "--root-volume-iops": "value",
                        "--root-volume-kms-key-arn": "value",
                        "--root-volume-size": "value",
                        "--root-volume-type": [
                          "gp2",
                          "gp3"
                        ],
                        "--security-group-ids": "value",
                        "--ssh-ec2-key-pair": "value",
                        "--subnet-id": "value",
                        "--tags": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--cluster": "value",
                        "--location": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--autoscaling-metrics": "value",
                        "--autoscaling-metrics-granularity": "value",
                        "--clear-annotations": "bool",
                        "--clear-autoscaling-metrics": "bool",
                        "--clear-proxy-config": "bool",
                        "--clear-security-group-ids": "bool",
                        "--clear-ssh-ec2-key-pair": "bool",
                        "--clear-tags": "bool",
                        "--cluster": "value",
                        "--config-encryption-kms-key-arn": "value",
                        "--iam-instance-profile": "value",
                        "--location": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-version": "value",
                        "--proxy-secret-arn": "value",
                        "--proxy-secret-version-id": "value",
                        "--root-volume-iops": "value",
                        "--root-volume-kms-key-arn": "value",
                        "--root-volume-size": "value",
                        "--root-volume-type": [
                          "gp2",
                          "gp3"
                        ],
                        "--security-group-ids": "value",
                        "--ssh-ec2-key-pair": "value",
                        "--tags": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "azure": {
              "commands": {
                "clients": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--application-id": "value",
                        "--async": "bool",
                        "--location": "value",
                        "--tenant-id": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-public-cert": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--output-file": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--azure-application-id": "value",
                        "--azure-region": "value",
                        "--azure-tenant-id": "value",
                        "--client": "value",
                        "--cluster-version": "value",
                        "--config-encryption-key-id": "value",
                        "--config-encryption-public-key": "value",
                        "--database-encryption-key-id": "value",
                        "--description": "value",
                        "--enable-managed-prometheus": "bool",
                        "--endpoint-subnet-id": "value",
                        "--fleet-project": "value",
                        "--location": "value",
                        "--logging": "value",
                        "--main-volume-size": "value",
                        "--pod-address-cidr-blocks": "value",
                        "--proxy-resource-group-id": "value",
                        "--proxy-secret-id": "value",
                        "--replica-placements": "value",
                        "--resource-group-id": "value",
                        "--root-volume-size": "value",
                        "--service-address-cidr-blocks": "value",
                        "--service-load-balancer-subnet-id": "value",
                        "--ssh-public-key": "value",
                        "--subnet-id": "value",
                        "--tags": "value",
                        "--validate-only": "bool",
                        "--vm-size": "value",
                        "--vnet-id": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--private-endpoint": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--azure-application-id": "value",
                        "--azure-tenant-id": "value",
                        "--clear-annotations": "bool",
                        "--clear-client": "bool",
                        "--clear-description": "bool",
                        "--client": "value",
                        "--cluster-version": "value",
                        "--description": "value",
                        "--disable-managed-prometheus": "bool",
                        "--enable-managed-prometheus": "bool",
                        "--location": "value",
                        "--logging": "value",
                        "--ssh-public-key": "value",
                        "--validate-only": "bool",
                        "--vm-size": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-server-config": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "node-pools": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--azure-availability-zone": "value",
                        "--cluster": "value",
                        "--config-encryption-key-id": "value",
                        "--config-encryption-public-key": "value",
                        "--image-type": "value",
                        "--location": "value",
                        "--max-nodes": "value",
                        "--max-pods-per-node": "value",
                        "--min-nodes": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--node-version": "value",
                        "--proxy-resource-group-id": "value",
                        "--proxy-secret-id": "value",
                        "--root-volume-size": "value",
                        "--ssh-public-key": "value",
                        "--subnet-id": "value",
                        "--tags": "value",
                        "--validate-only": "bool",
                        "--vm-size": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--clear-annotations": "bool",
                        "--cluster": "value",
                        "--location": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-version": "value",
                        "--ssh-public-key": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "backup-restore": {
              "commands": {
                "backup-plans": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-delete-lock-days": "value",
                        "--backup-retain-days": "value",
                        "--cluster": "value",
                        "--cron-schedule": "value",
                        "--deactivated": "bool",
                        "--description": "value",
                        "--encryption-key": "value",
                        "--include-secrets": "bool",
                        "--include-volume-data": "bool",
                        "--labels": "value",
                        "--location": "value",
                        "--locked": "bool",
                        "--paused": "bool",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-delete-lock-days": "value",
                        "--backup-retain-days": "value",
                        "--clear-labels": "bool",
                        "--cron-schedule": "value",
                        "--deactivated": "bool",
                        "--description": "value",
                        "--encryption-key": "value",
                        "--etag": "value",
                        "--include-secrets": "bool",
                        "--include-volume-data": "bool",
                        "--location": "value",
                        "--locked": "bool",
                        "--paused": "bool",
                        "--remove-labels": "value",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "backups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--delete-lock-days": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--retain-days": "value",
                        "--wait-for-completion": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--clear-labels": "bool",
                        "--delete-lock-days": "value",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--retain-days": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "restore-plans": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--cluster": "value",
                        "--cluster-resource-conflict-policy": [
                          "use-backup-version",
                          "use-existing-version"
                        ],
                        "--cluster-resource-restore-scope": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--namespaced-resource-restore-mode": [
                          "delete-and-restore",
                          "fail-on-conflict"
                        ],
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--substitution-rules-file": "value",
                        "--volume-data-restore-policy": [
                          "no-volume-data-restoration",
                          "restore-volume-data-from-backup",
                          "reuse-volume-handle-from-backup"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--cluster-resource-conflict-policy": [
                          "use-backup-version",
                          "use-existing-version"
                        ],
                        "--cluster-resource-restore-scope": "value",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--namespaced-resource-restore-mode": [
                          "delete-and-restore",
                          "fail-on-conflict"
                        ],
                        "--remove-labels": "value",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--substitution-rules-file": "value",
                        "--update-labels": "value",
                        "--volume-data-restore-policy": [
                          "no-volume-data-restoration",
                          "restore-volume-data-from-backup",
                          "reuse-volume-handle-from-backup"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "restores": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--restore-plan": "value",
                        "--wait-for-completion": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--restore-plan": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "volume-backups": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--backup": "value",
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--backup": "value",
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "volume-restores": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "bare-metal": {
              "commands": {
                "admin-clusters": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "enroll": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "query-version-config": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--api-server-args": "value",
                        "--async": "bool",
                        "--control-plane-node-configs": "value",
                        "--control-plane-node-labels": "value",
                        "--control-plane-node-taints": "value",
                        "--description": "value",
                        "--enable-application-logs": "bool",
                        "--island-mode-service-address-cidr-blocks": "value",
                        "--location": "value",
                        "--login-user": "value",
                        "--maintenance-address-cidr-blocks": "value",
                        "--max-pods-per-node": "value",
                        "--no-proxy": "value",
                        "--uri": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--api-server-args": "value",
                        "--async": "bool",
                        "--container-runtime": "value",
                        "--control-plane-load-balancer-port": "value",
                        "--control-plane-node-configs": "value",
                        "--control-plane-node-labels": "value",
                        "--control-plane-node-taints": "value",
                        "--control-plane-vip": "value",
                        "--description": "value",
                        "--enable-application-logs": "bool",
                        "--enable-manual-lb": "bool",
                        "--ingress-vip": "value",
                        "--island-mode-pod-address-cidr-blocks": "value",
                        "--island-mode-service-address-cidr-blocks": "value",
                        "--location": "value",
                        "--login-user": "value",
                        "--lvp-node-mounts-config-path": "value",
                        "--lvp-node-mounts-config-storage-class": "value",
                        "--lvp-share-path": "value",
                        "--lvp-share-storage-class": "value",
                        "--maintenance-address-cidr-blocks": "value",
                        "--max-pods-per-node": "value",
                        "--metal-lb-address-pools": "value",
                        "--metal-lb-load-balancer-node-configs": "value",
                        "--metal-lb-load-balancer-node-labels": "value",
                        "--metal-lb-load-balancer-node-taints": "value",
                        "--no-proxy": "value",
                        "--shared-path-pv-count": "value",
                        "--uri": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--force": "bool",
                        "--ignore-errors": "bool",
                        "--location": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "enroll": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "query-version-config": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--allow-missing": "bool",
                        "--api-server-args": "value",
                        "--async": "bool",
                        "--control-plane-node-configs": "value",
                        "--control-plane-node-labels": "value",
                        "--control-plane-node-taints": "value",
                        "--description": "value",
                        "--enable-application-logs": "bool",
                        "--island-mode-service-address-cidr-blocks": "value",
                        "--location": "value",
                        "--login-user": "value",
                        "--maintenance-address-cidr-blocks": "value",
                        "--metal-lb-address-pools": "value",
                        "--metal-lb-load-balancer-node-configs": "value",
                        "--metal-lb-load-balancer-node-labels": "value",
                        "--metal-lb-load-balancer-node-taints": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-pools": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--cluster": "value",
                        "--display-name": "value",
                        "--location": "value",
                        "--node-configs": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--cluster": "value",
                        "--location": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--cluster": "value",
                        "--display-name": "value",
                        "--location": "value",
                        "--node-configs": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "binauthz": {
              "commands": {
                "attestations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--payload-file": "value",
                        "--public-key-id": "value",
                        "--signature-file": "value",
                        "--validate": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "sign-and-create": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--keyversion": "value",
                        "--keyversion-key": "value",
                        "--keyversion-keyring": "value",
                        "--keyversion-location": "value",
                        "--keyversion-project": "value",
                        "--public-key-id-override": "value",
                        "--validate": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "attestors": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--attestation-authority-note": "value",
                        "--attestation-authority-note-project": "value",
                        "--description": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "public-keys": {
                      "commands": {
                        "add": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value",
                            "--comment": "value",
                            "--keyversion": "value",
                            "--keyversion-key": "value",
                            "--keyversion-keyring": "value",
                            "--keyversion-location": "value",
                            "--keyversion-project": "value",
                            "--pgp-public-key-file": "value",
                            "--pkix-public-key-algorithm": [
                              "ec-sign-p256-sha256",
                              "ec-sign-p384-sha384",
                              "ec-sign-p521-sha512",
                              "ecdsa-p256-sha256",
                              "ecdsa-p384-sha384",
                              "ecdsa-p521-sha512",
                              "rsa-pss-2048-sha256",
                              "rsa-pss-3072-sha256",
                              "rsa-pss-4096-sha256",
                              "rsa-pss-4096-sha512",
                              "rsa-sign-pkcs1-2048-sha256",
                              "rsa-sign-pkcs1-3072-sha256",
                              "rsa-sign-pkcs1-4096-sha256",
                              "rsa-sign-pkcs1-4096-sha512",
                              "rsa-sign-pss-2048-sha256",
                              "rsa-sign-pss-3072-sha256",
                              "rsa-sign-pss-4096-sha256",
                              "rsa-sign-pss-4096-sha512"
                            ],
                            "--pkix-public-key-file": "value",
                            "--public-key-id-override": "value"
                          }
                        },
                        "remove": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value",
                            "--comment": "value",
                            "--pgp-public-key-file": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "continuous-validation": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {}
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "create-signature-payload": {
                  "commands": {},
                  "flags": {
                    "--artifact-url": "value"
                  }
                },
                "policy": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {}
                    },
                    "export-system-policy": {
                      "commands": {},
                      "flags": {
                        "--location": [
                          "asia-east1",
                          "asia-east2",
                          "asia-northeast1",
                          "asia-northeast2",
                          "asia-northeast3",
                          "asia-south1",
                          "asia-south2",
                          "asia-southeast1",
                          "asia-southeast2",
                          "australia-southeast1",
                          "australia-southeast2",
                          "europe-central2",
                          "europe-north1",
                          "europe-southwest1",
                          "europe-west1",
                          "europe-west12",
                          "europe-west2",
                          "europe-west3",
                          "europe-west4",
                          "europe-west5",
                          "europe-west6",
                          "europe-west8",
                          "europe-west9",
                          "global",
                          "me-central1",
                          "me-west1",
                          "northamerica-northeast1",
                          "northamerica-northeast2",
                          "southamerica-east1",
                          "southamerica-west1",
                          "us-central1",
                          "us-central2",
                          "us-east1",
                          "us-east4",
                          "us-east5",
                          "us-east7",
                          "us-south1",
                          "us-west1",
                          "us-west2",
                          "us-west3",
                          "us-west4"
                        ]
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--strict-validation": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--location": "value",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--additional-zones": "value",
                    "--addons": "value",
                    "--allow-route-overlap": "bool",
                    "--async": "bool",
                    "--autoprovisioning-config-file": "value",
                    "--autoprovisioning-image-type": "value",
                    "--autoprovisioning-locations": "value",
                    "--autoprovisioning-max-surge-upgrade": "value",
                    "--autoprovisioning-max-unavailable-upgrade": "value",
                    "--autoprovisioning-min-cpu-platform": "value",
                    "--autoprovisioning-network-tags": "value",
                    "--autoprovisioning-node-pool-soak-duration": "value",
                    "--autoprovisioning-scopes": "value",
                    "--autoprovisioning-service-account": "value",
                    "--autoprovisioning-standard-rollout-policy": "value",
                    "--autoscaling-profile": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--boot-disk-kms-key": "value",
                    "--cloud-run-config": "value",
                    "--cluster-dns": [
                      "clouddns",
                      "default"
                    ],
                    "--cluster-dns-domain": "value",
                    "--cluster-dns-scope": [
                      "cluster",
                      "vpc"
                    ],
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--cluster-version": "value",
                    "--create-subnetwork": "value",
                    "--database-encryption-key": "value",
                    "--default-max-pods-per-node": "value",
                    "--disable-default-snat": "bool",
                    "--disable-pod-cidr-overprovision": "bool",
                    "--disk-size": "value",
                    "--disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--enable-autoprovisioning": "bool",
                    "--enable-autoprovisioning-autorepair": "bool",
                    "--enable-autoprovisioning-autoupgrade": "bool",
                    "--enable-autoprovisioning-blue-green-upgrade": "bool",
                    "--enable-autoprovisioning-surge-upgrade": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-basic-auth": "bool",
                    "--enable-binauthz": "bool",
                    "--enable-cloud-logging": "bool",
                    "--enable-cloud-monitoring": "bool",
                    "--enable-cloud-run-alpha": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-cost-allocation": "bool",
                    "--enable-dataplane-v2": "bool",
                    "--enable-gke-oidc": "bool",
                    "--enable-google-cloud-access": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-identity-service": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-intra-node-visibility": "bool",
                    "--enable-ip-alias": "bool",
                    "--enable-kubernetes-alpha": "bool",
                    "--enable-l4-ilb-subsetting": "bool",
                    "--enable-legacy-authorization": "bool",
                    "--enable-logging-monitoring-system-only": "bool",
                    "--enable-managed-prometheus": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-master-global-access": "bool",
                    "--enable-network-egress-metering": "bool",
                    "--enable-network-policy": "bool",
                    "--enable-pod-security-policy": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-resource-consumption-metering": "bool",
                    "--enable-service-externalips": "bool",
                    "--enable-shielded-nodes": "bool",
                    "--enable-stackdriver-kubernetes": "bool",
                    "--enable-tpu": "bool",
                    "--enable-tpu-service-networking": "bool",
                    "--enable-vertical-pod-autoscaling": "bool",
                    "--ephemeral-storage": "value",
                    "--ephemeral-storage-local-ssd": "value",
                    "--gateway-api": [
                      "disabled",
                      "standard"
                    ],
                    "--identity-provider": "value",
                    "--image-type": "value",
                    "--ipv6-access-type": [
                      "external",
                      "internal"
                    ],
                    "--issue-client-certificate": "bool",
                    "--istio-config": "value",
                    "--labels": "value",
                    "--linux-sysctls": "value",
                    "--local-nvme-ssd-block": "value",
                    "--local-ssd-count": "value",
                    "--local-ssd-volumes": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging": "value",
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--machine-type": "value",
                    "--maintenance-window": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--max-accelerator": "value",
                    "--max-cpu": "value",
                    "--max-memory": "value",
                    "--max-nodes": "value",
                    "--max-nodes-per-pool": "value",
                    "--max-pods-per-node": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-accelerator": "value",
                    "--min-cpu": "value",
                    "--min-cpu-platform": "value",
                    "--min-memory": "value",
                    "--min-nodes": "value",
                    "--monitoring": "value",
                    "--network": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-pool-name": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--notification-config": "value",
                    "--num-nodes": "value",
                    "--password": "value",
                    "--placement-type": [
                      "COMPACT",
                      "UNSPECIFIED"
                    ],
                    "--preemptible": "bool",
                    "--private-cluster": "bool",
                    "--private-endpoint-subnetwork": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "disabled",
                      "outbound-only"
                    ],
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-usage-bigquery-dataset": "value",
                    "--scopes": "value",
                    "--security-group": "value",
                    "--security-profile": "value",
                    "--security-profile-runtime-rules": "bool",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--spot": "bool",
                    "--stack-type": [
                      "ipv4",
                      "ipv4-ipv6"
                    ],
                    "--subnetwork": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--tpu-ipv4-cidr": "value",
                    "--username": "value",
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--workload-pool": "value",
                    "--zone": "value"
                  }
                },
                "create-auto": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoprovisioning-network-tags": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--boot-disk-kms-key": "value",
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--cluster-version": "value",
                    "--create-subnetwork": "value",
                    "--database-encryption-key": "value",
                    "--enable-google-cloud-access": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-nodes": "bool",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--network": "value",
                    "--private-endpoint-subnetwork": "value",
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--scopes": "value",
                    "--security-group": "value",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--subnetwork": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--internal-ip": "bool",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--node-pool": "value",
                    "--num-nodes": "value",
                    "--region": "value",
                    "--size": "value",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-maintenance-exclusion-end": "value",
                    "--add-maintenance-exclusion-name": "value",
                    "--add-maintenance-exclusion-scope": "value",
                    "--add-maintenance-exclusion-start": "value",
                    "--additional-zones": "value",
                    "--async": "bool",
                    "--autoprovisioning-config-file": "value",
                    "--autoprovisioning-image-type": "value",
                    "--autoprovisioning-locations": "value",
                    "--autoprovisioning-max-surge-upgrade": "value",
                    "--autoprovisioning-max-unavailable-upgrade": "value",
                    "--autoprovisioning-min-cpu-platform": "value",
                    "--autoprovisioning-network-tags": "value",
                    "--autoprovisioning-node-pool-soak-duration": "value",
                    "--autoprovisioning-scopes": "value",
                    "--autoprovisioning-service-account": "value",
                    "--autoprovisioning-standard-rollout-policy": "value",
                    "--autoscaling-profile": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--clear-maintenance-window": "bool",
                    "--clear-resource-usage-bigquery-dataset": "bool",
                    "--cloud-run-config": "value",
                    "--cluster-dns": [
                      "clouddns",
                      "default"
                    ],
                    "--cluster-dns-domain": "value",
                    "--cluster-dns-scope": [
                      "cluster",
                      "vpc"
                    ],
                    "--complete-credential-rotation": "bool",
                    "--complete-ip-rotation": "bool",
                    "--database-encryption-key": "value",
                    "--disable-database-encryption": "bool",
                    "--disable-default-snat": "bool",
                    "--disable-managed-prometheus": "bool",
                    "--disable-workload-identity": "bool",
                    "--enable-autoprovisioning": "bool",
                    "--enable-autoprovisioning-autorepair": "bool",
                    "--enable-autoprovisioning-autoupgrade": "bool",
                    "--enable-autoprovisioning-blue-green-upgrade": "bool",
                    "--enable-autoprovisioning-surge-upgrade": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-basic-auth": "bool",
                    "--enable-binauthz": "bool",
                    "--enable-cost-allocation": "bool",
                    "--enable-gke-oidc": "bool",
                    "--enable-google-cloud-access": "bool",
                    "--enable-identity-service": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-intra-node-visibility": "bool",
                    "--enable-l4-ilb-subsetting": "bool",
                    "--enable-legacy-authorization": "bool",
                    "--enable-logging-monitoring-system-only": "bool",
                    "--enable-managed-prometheus": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-master-global-access": "bool",
                    "--enable-network-egress-metering": "bool",
                    "--enable-network-policy": "bool",
                    "--enable-pod-security-policy": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-resource-consumption-metering": "bool",
                    "--enable-service-externalips": "bool",
                    "--enable-shielded-nodes": "bool",
                    "--enable-stackdriver-kubernetes": "bool",
                    "--enable-tpu": "bool",
                    "--enable-tpu-service-networking": "bool",
                    "--enable-vertical-pod-autoscaling": "bool",
                    "--gateway-api": [
                      "disabled",
                      "standard"
                    ],
                    "--generate-password": "bool",
                    "--identity-provider": "value",
                    "--istio-config": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging": "value",
                    "--logging-service": "value",
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--maintenance-window": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--max-accelerator": "value",
                    "--max-cpu": "value",
                    "--max-memory": "value",
                    "--max-nodes": "value",
                    "--min-accelerator": "value",
                    "--min-cpu": "value",
                    "--min-memory": "value",
                    "--min-nodes": "value",
                    "--monitoring": "value",
                    "--monitoring-service": "value",
                    "--node-locations": "value",
                    "--node-pool": "value",
                    "--notification-config": "value",
                    "--password": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "disabled",
                      "outbound-only"
                    ],
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--remove-labels": "value",
                    "--remove-maintenance-exclusion": "value",
                    "--resource-usage-bigquery-dataset": "value",
                    "--security-group": "value",
                    "--security-profile": "value",
                    "--set-password": "bool",
                    "--stack-type": [
                      "ipv4",
                      "ipv4-ipv6"
                    ],
                    "--start-credential-rotation": "bool",
                    "--start-ip-rotation": "bool",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--tpu-ipv4-cidr": "value",
                    "--update-addons": "value",
                    "--update-labels": "value",
                    "--username": "value",
                    "--workload-pool": "value",
                    "--zone": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster-version": "value",
                    "--image-type": "value",
                    "--master": "bool",
                    "--node-pool": "value",
                    "--region": "value",
                    "--security-profile": "value",
                    "--security-profile-runtime-rules": "bool",
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "fleet": {
              "commands": {
                "cloudrun": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "config-management": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "fetch-for-apply": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "status": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "unmanage": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "upgrade": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "version": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "features": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "identity-service": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "bool",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "ingress": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "bindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "generate-gateway-rbac": {
                      "commands": {},
                      "flags": {
                        "--anthos-support": "bool",
                        "--apply": "bool",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--membership": "value",
                        "--rbac-output-file": "value",
                        "--revoke": "bool",
                        "--role": "value",
                        "--users": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "register": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--enable-workload-identity": "bool",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--has-private-issuer": "bool",
                        "--install-connect-agent": "bool",
                        "--internal-ip": "bool",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--manifest-output-file": "value",
                        "--proxy": "value",
                        "--public-issuer-url": "value",
                        "--service-account-key-file": "value"
                      }
                    },
                    "unregister": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--uninstall-connect-agent": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "mesh": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--control-plane": [
                          "automatic",
                          "manual"
                        ],
                        "--location": "value",
                        "--management": [
                          "automatic",
                          "manual"
                        ],
                        "--membership": "value",
                        "--memberships": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "multi-cluster-services": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "namespaces": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--membership": "value",
                        "--membership-location": "value",
                        "--set-namespace-in-config": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rbacrolebindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--namespace": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "policycontroller": {
                  "commands": {},
                  "flags": {}
                },
                "scopes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "service-directory": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "get-server-config": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "hub": {
              "commands": {
                "cloudrun": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "config-management": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "fetch-for-apply": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "status": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "unmanage": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "upgrade": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "version": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "features": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "identity-service": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "bool",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "ingress": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "bindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "generate-gateway-rbac": {
                      "commands": {},
                      "flags": {
                        "--anthos-support": "bool",
                        "--apply": "bool",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--membership": "value",
                        "--rbac-output-file": "value",
                        "--revoke": "bool",
                        "--role": "value",
                        "--users": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "register": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--enable-workload-identity": "bool",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--has-private-issuer": "bool",
                        "--install-connect-agent": "bool",
                        "--internal-ip": "bool",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--manifest-output-file": "value",
                        "--proxy": "value",
                        "--public-issuer-url": "value",
                        "--service-account-key-file": "value"
                      }
                    },
                    "unregister": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--uninstall-connect-agent": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "mesh": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--control-plane": [
                          "automatic",
                          "manual"
                        ],
                        "--location": "value",
                        "--management": [
                          "automatic",
                          "manual"
                        ],
                        "--membership": "value",
                        "--memberships": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "multi-cluster-services": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "namespaces": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--membership": "value",
                        "--membership-location": "value",
                        "--set-namespace-in-config": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rbacrolebindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--namespace": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "policycontroller": {
                  "commands": {},
                  "flags": {}
                },
                "scopes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "service-directory": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "images": {
              "commands": {
                "add-tag": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force-delete-tags": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--metadata-filter": "value",
                    "--show-all-metadata": "bool",
                    "--show-build-details": "bool",
                    "--show-deployment": "bool",
                    "--show-image-basis": "bool",
                    "--show-package-vulnerability": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-tags": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--occurrence-filter": "value",
                    "--page-size": "value",
                    "--show-occurrences": "bool",
                    "--show-occurrences-from": "value",
                    "--sort-by": "value"
                  }
                },
                "untag": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "node-pools": {
              "commands": {
                "complete-upgrade": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--cluster": "value",
                        "--location": "value",
                        "--path": "value",
                        "--region": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--async": "bool",
                    "--boot-disk-kms-key": "value",
                    "--cluster": "value",
                    "--create-pod-ipv4-range": "value",
                    "--disable-pod-cidr-overprovision": "bool",
                    "--disk-size": "value",
                    "--disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--enable-autoprovisioning": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-blue-green-upgrade": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-surge-upgrade": "bool",
                    "--ephemeral-storage": "value",
                    "--ephemeral-storage-local-ssd": "value",
                    "--image-type": "value",
                    "--labels": "value",
                    "--linux-sysctls": "value",
                    "--local-nvme-ssd-block": "value",
                    "--local-ssd-count": "value",
                    "--local-ssd-volumes": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--max-pods-per-node": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-nodes": "value",
                    "--network-performance-configs": "value",
                    "--node-group": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-pool-soak-duration": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--num-nodes": "value",
                    "--placement-type": [
                      "COMPACT",
                      "UNSPECIFIED"
                    ],
                    "--pod-ipv4-range": "value",
                    "--preemptible": "bool",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--sandbox": "value",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--spot": "bool",
                    "--standard-rollout-policy": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--windows-os-version": [
                      "ltsc2019",
                      "ltsc2022"
                    ],
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value",
                    "--respect-pdb": "value",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--enable-autoprovisioning": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-blue-green-upgrade": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-surge-upgrade": "bool",
                    "--labels": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--max-nodes": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--min-nodes": "value",
                    "--network-performance-configs": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-pool-soak-duration": "value",
                    "--node-taints": "value",
                    "--region": "value",
                    "--standard-rollout-policy": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--windows-os-version": [
                      "ltsc2019",
                      "ltsc2022"
                    ],
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "subnets": {
              "commands": {
                "list-usable": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network-project": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "vmware": {
              "commands": {
                "admin-clusters": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "enroll": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--required-platform-version": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--admin-users": "value",
                        "--annotations": "value",
                        "--async": "bool",
                        "--control-plane-node-port": "value",
                        "--control-plane-vip": "value",
                        "--cpus": "value",
                        "--description": "value",
                        "--disable-aag-config": "bool",
                        "--disable-vsphere-csi": "bool",
                        "--dns-search-domains": "value",
                        "--dns-servers": "value",
                        "--enable-advanced-networking": "bool",
                        "--enable-auto-repair": "bool",
                        "--enable-auto-resize": "bool",
                        "--enable-dataplane-v2": "bool",
                        "--enable-dhcp": "bool",
                        "--enable-vm-tracking": "bool",
                        "--f5-config-address": "value",
                        "--f5-config-partition": "value",
                        "--f5-config-snat-pool": "value",
                        "--ingress-http-node-port": "value",
                        "--ingress-https-node-port": "value",
                        "--ingress-vip": "value",
                        "--konnectivity-server-node-port": "value",
                        "--location": "value",
                        "--memory": "value",
                        "--metal-lb-config-address-pools": "value",
                        "--ntp-servers": "value",
                        "--pod-address-cidr-blocks": "value",
                        "--replicas": "value",
                        "--service-address-cidr-blocks": "value",
                        "--static-ip-config-ip-blocks": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--force": "bool",
                        "--ignore-errors": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "enroll": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "query-version-config": {
                      "commands": {},
                      "flags": {
                        "--admin-cluster-membership": "value",
                        "--admin-cluster-membership-location": "value",
                        "--admin-cluster-membership-project": "value",
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-annotations": "value",
                        "--admin-users": "value",
                        "--async": "bool",
                        "--cpus": "value",
                        "--description": "value",
                        "--disable-aag-config": "bool",
                        "--disable-auto-repair": "bool",
                        "--disable-auto-resize": "bool",
                        "--disable-vsphere-csi": "bool",
                        "--enable-aag-config": "bool",
                        "--enable-auto-repair": "bool",
                        "--enable-auto-resize": "bool",
                        "--enable-vsphere-csi": "bool",
                        "--location": "value",
                        "--memory": "value",
                        "--metal-lb-config-address-pools": "value",
                        "--remove-annotations": "value",
                        "--static-ip-config-ip-blocks": "value"
                      }
                    },
                    "upgrade": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-pools": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--annotations": "value",
                        "--async": "bool",
                        "--boot-disk-size": "value",
                        "--cluster": "value",
                        "--cpus": "value",
                        "--display-name": "value",
                        "--enable-load-balancer": "bool",
                        "--image": "value",
                        "--image-type": "value",
                        "--location": "value",
                        "--max-replicas": "value",
                        "--memory": "value",
                        "--min-replicas": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--replicas": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--allow-missing": "bool",
                        "--async": "bool",
                        "--cluster": "value",
                        "--ignore-errors": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--boot-disk-size": "value",
                        "--cluster": "value",
                        "--cpus": "value",
                        "--disable-load-balancer": "bool",
                        "--display-name": "value",
                        "--enable-load-balancer": "bool",
                        "--image": "value",
                        "--image-type": "value",
                        "--location": "value",
                        "--max-replicas": "value",
                        "--memory": "value",
                        "--min-replicas": "value",
                        "--node-labels": "value",
                        "--node-taints": "value",
                        "--replicas": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "data-catalog": {
          "commands": {
            "crawler-runs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--crawler": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "crawlers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--buckets": "value",
                    "--bundle-specs": "value",
                    "--crawl-scope": [
                      "bucket",
                      "organization",
                      "project"
                    ],
                    "--description": "value",
                    "--display-name": "value",
                    "--run-option": [
                      "manual",
                      "scheduled"
                    ],
                    "--run-schedule": [
                      "daily",
                      "weekly"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-buckets": "value",
                    "--add-bundle-specs": "value",
                    "--clear-buckets": "bool",
                    "--clear-bundle-specs": "bool",
                    "--crawl-scope": [
                      "bucket",
                      "organization",
                      "project"
                    ],
                    "--description": "value",
                    "--display-name": "value",
                    "--remove-buckets": "value",
                    "--remove-bundle-specs": "value",
                    "--run-option": [
                      "manual",
                      "scheduled"
                    ],
                    "--run-schedule": [
                      "daily",
                      "weekly"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "entries": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--entry-group": "value",
                    "--gcs-file-patterns": "value",
                    "--location": "value",
                    "--schema": "value",
                    "--schema-from-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "lookup": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-file-patterns": "value",
                    "--clear-file-patterns": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--lookup-entry": "value",
                    "--remove-file-patterns": "value",
                    "--schema": "value",
                    "--schema-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "entry-groups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "search": {
              "commands": {},
              "flags": {
                "--include-gcp-public-datasets": "bool",
                "--include-organization-ids": "value",
                "--include-project-ids": "value",
                "--limit": "value",
                "--order-by": "value",
                "--page-size": "value",
                "--restricted-locations": "value"
              }
            },
            "tag-templates": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--field": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "fields": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--display-name": "value",
                        "--location": "value",
                        "--tag-template": "value",
                        "--type": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--force": "bool",
                        "--location": "value",
                        "--tag-template": "value"
                      }
                    },
                    "enum-values": {
                      "commands": {
                        "rename": {
                          "commands": {},
                          "flags": {
                            "--field": "value",
                            "--location": "value",
                            "--new-id": "value",
                            "--tag-template": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "rename": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--new-id": "value",
                        "--tag-template": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--display-name": "value",
                        "--enum-values": "value",
                        "--location": "value",
                        "--required": "bool",
                        "--tag-template": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--scope": "value",
                    "--tag-file": "value",
                    "--tag-template": "value",
                    "--tag-template-location": "value",
                    "--tag-template-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--tag-file": "value",
                    "--tag-template": "value",
                    "--tag-template-location": "value",
                    "--tag-template-project": "value"
                  }
                }
              },
              "flags": {}
            },
            "taxonomies": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "policy-tags": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--taxonomy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--taxonomy": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--taxonomy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--taxonomy": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--taxonomy": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--taxonomy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "database-migration": {
          "commands": {
            "connection-profiles": {
              "commands": {
                "create": {
                  "commands": {
                    "cloudsql": {
                      "commands": {},
                      "flags": {
                        "--activation-policy": [
                          "ALWAYS",
                          "NEVER"
                        ],
                        "--authorized-networks": "value",
                        "--auto-storage-increase": "bool",
                        "--data-disk-size": "value",
                        "--data-disk-type": [
                          "PD_HDD",
                          "PD_SSD"
                        ],
                        "--database-flags": "value",
                        "--database-version": [
                          "MYSQL_5_6",
                          "MYSQL_5_7",
                          "MYSQL_8_0",
                          "POSTGRES_10",
                          "POSTGRES_11",
                          "POSTGRES_12",
                          "POSTGRES_13",
                          "POSTGRES_14",
                          "POSTGRES_9_6"
                        ],
                        "--display-name": "value",
                        "--enable-ip-v4": "bool",
                        "--labels": "value",
                        "--no-async": "bool",
                        "--private-network": "value",
                        "--provider": [
                          "CLOUDSQL",
                          "RDS"
                        ],
                        "--region": "dynamic",
                        "--require-ssl": "bool",
                        "--root-password": "value",
                        "--source-id": "dynamic",
                        "--storage-auto-resize-limit": "value",
                        "--tier": "value",
                        "--user-labels": "value",
                        "--zone": "value"
                      }
                    },
                    "mysql": {
                      "commands": {},
                      "flags": {
                        "--ca-certificate": "value",
                        "--certificate": "value",
                        "--display-name": "value",
                        "--host": "value",
                        "--instance": "value",
                        "--labels": "value",
                        "--no-async": "bool",
                        "--password": "value",
                        "--port": "value",
                        "--private-key": "value",
                        "--prompt-for-password": "bool",
                        "--provider": [
                          "CLOUDSQL",
                          "RDS"
                        ],
                        "--region": "dynamic",
                        "--username": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ca-certificate": "value",
                    "--certificate": "value",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--host": "value",
                    "--instance": "value",
                    "--password": "value",
                    "--port": "value",
                    "--private-key": "value",
                    "--prompt-for-password": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--username": "value"
                  }
                }
              },
              "flags": {}
            },
            "migration-jobs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--destination": "dynamic",
                    "--display-name": "value",
                    "--dump-path": "value",
                    "--labels": "value",
                    "--no-async": "bool",
                    "--peer-vpc": "value",
                    "--region": "dynamic",
                    "--source": "dynamic",
                    "--static-ip": "bool",
                    "--type": [
                      "CONTINUOUS",
                      "ONE_TIME"
                    ],
                    "--vm": "value",
                    "--vm-ip": "value",
                    "--vm-port": "value",
                    "--vpc": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "generate-ssh-script": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--vm": "value",
                    "--vm-machine-type": "value",
                    "--vm-port": "value",
                    "--vm-zone": "value",
                    "--vm-zone-create": "value",
                    "--vpc": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "promote": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--destination": "dynamic",
                    "--display-name": "value",
                    "--dump-path": "value",
                    "--no-async": "bool",
                    "--peer-vpc": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--source": "dynamic",
                    "--static-ip": "bool",
                    "--type": [
                      "CONTINUOUS",
                      "ONE_TIME"
                    ],
                    "--update-labels": "value",
                    "--vm": "value",
                    "--vm-ip": "value",
                    "--vm-port": "value",
                    "--vpc": "value"
                  }
                },
                "verify": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dataflow": {
          "commands": {
            "jobs": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "export-steps": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "resume-unsupported-sdk": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--token": "value"
                  }
                }
              },
              "flags": {}
            },
            "logs": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--after": "value",
                    "--before": "value",
                    "--filter": "value",
                    "--importance": [
                      "debug",
                      "detailed",
                      "error",
                      "warning"
                    ],
                    "--limit": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--changed-after": "value",
                    "--filter": "value",
                    "--hide-committed": "bool",
                    "--limit": "value",
                    "--region": "value",
                    "--source": [
                      "all",
                      "service",
                      "user"
                    ],
                    "--tentative": "bool",
                    "--transform": "value"
                  }
                }
              },
              "flags": {}
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--job-id": "value",
                    "--region": "value",
                    "--snapshot-sources": "value",
                    "--snapshot-ttl": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--job-id": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dataplex": {
          "commands": {
            "assets": {
              "commands": {
                "actions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--asset": "dynamic",
                        "--filter": "value",
                        "--lake": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csv-delimiter": "value",
                    "--csv-disable-type-inference": "bool",
                    "--csv-encoding": "value",
                    "--csv-header-rows": "value",
                    "--description": "value",
                    "--discovery-enabled": "bool",
                    "--discovery-exclude-patterns": "value",
                    "--discovery-include-patterns": "value",
                    "--discovery-schedule": "value",
                    "--display-name": "value",
                    "--json-disable-type-inference": "bool",
                    "--json-encoding": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--resource-name": "value",
                    "--resource-type": [
                      "BIGQUERY_DATASET",
                      "STORAGE_BUCKET"
                    ],
                    "--validate-only": "bool",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value",
                    "--zone": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csv-delimiter": "value",
                    "--csv-disable-type-inference": "bool",
                    "--csv-encoding": "value",
                    "--csv-header-rows": "value",
                    "--description": "value",
                    "--discovery-enabled": "bool",
                    "--discovery-exclude-patterns": "value",
                    "--discovery-include-patterns": "value",
                    "--discovery-schedule": "value",
                    "--display-name": "value",
                    "--json-disable-type-inference": "bool",
                    "--json-encoding": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--validate-only": "bool",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "content": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--data-text": "value",
                    "--description": "value",
                    "--kernel-type": [
                      "PYTHON3"
                    ],
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--path": "value",
                    "--query-engine": [
                      "SPARK"
                    ],
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--data-text": "value",
                    "--description": "value",
                    "--kernel-type": [
                      "PYTHON3"
                    ],
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--path": "value",
                    "--query-engine": [
                      "SPARK"
                    ],
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "data-attribute-bindings": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--path-file-format": [
                      "json",
                      "yaml"
                    ],
                    "--path-file-name": "value",
                    "--paths": "value",
                    "--resource": "value",
                    "--resource-attributes": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--etag": "value",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--path-file-format": [
                      "json",
                      "yaml"
                    ],
                    "--path-file-name": "value",
                    "--paths": "value",
                    "--resource-attributes": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "data-taxonomies": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "attributes": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "value",
                        "--location": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--data-readers": "value",
                        "--data_taxonomy": "value",
                        "--description": "value",
                        "--display-name": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--parent": "value",
                        "--resource-owners": "value",
                        "--resource-readers": "value",
                        "--resource-writers": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--data_taxonomy": "dynamic",
                        "--etag": "value",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "value",
                        "--location": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--data_taxonomy": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--data-readers": "value",
                        "--data_taxonomy": "value",
                        "--description": "value",
                        "--display-name": "value",
                        "--etag": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--parent": "value",
                        "--resource-owners": "value",
                        "--resource-readers": "value",
                        "--resource-writers": "value",
                        "--validate-only": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--etag": "value",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "datascan": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--data-quality-spec-file": "value",
                    "--data-source-entity": "value",
                    "--description": "value",
                    "--disabled": "value",
                    "--display-name": "value",
                    "--field": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--on-demand": "value",
                    "--scan-type": [
                      "PROFILE",
                      "QUALITY"
                    ],
                    "--schedule": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "jobs": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--datascan": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--datascan": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "environments": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--compute-disk-size-gb": "value",
                    "--compute-max-node-count": "value",
                    "--compute-node-count": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--os-image-java-libraries": "value",
                    "--os-image-properties": "value",
                    "--os-image-python-packages": "value",
                    "--os-image-version": "value",
                    "--session-enable-fast-startup": "bool",
                    "--session-max-idle-duration": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "sessions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--environment": "dynamic",
                        "--filter": "value",
                        "--lake": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--compute-disk-size-gb": "value",
                    "--compute-max-node-count": "value",
                    "--compute-node-count": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--os-image-java-libraries": "value",
                    "--os-image-properties": "value",
                    "--os-image-python-packages": "value",
                    "--os-image-version": "value",
                    "--session-enable-fast-startup": "bool",
                    "--session-max-idle-duration": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "lakes": {
              "commands": {
                "actions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--lake": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "authorize": {
                  "commands": {},
                  "flags": {
                    "--bigquery-dataset-resource": "value",
                    "--project-resource": "value",
                    "--secondary-project": "value",
                    "--storage-bucket-resource": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--metastore-service": "value",
                    "--validate-only": "bool"
                  }
                },
                "deauthorize": {
                  "commands": {},
                  "flags": {
                    "--bigquery-dataset-resource": "value",
                    "--project-resource": "value",
                    "--secondary-project": "value",
                    "--storage-bucket-resource": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--metastore-service": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tasks": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--batch-executors-count": "value",
                    "--batch-max-executors-count": "value",
                    "--container-image": "value",
                    "--container-image-java-jars": "value",
                    "--container-image-properties": "value",
                    "--container-image-python-packages": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--execution-args": "value",
                    "--execution-project": "value",
                    "--execution-service-account": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--max-job-execution-lifetime": "value",
                    "--notebook": "value",
                    "--notebook-archive-uris": "value",
                    "--notebook-batch-executors-count": "value",
                    "--notebook-batch-max-executors-count": "value",
                    "--notebook-container-image": "value",
                    "--notebook-container-image-java-jars": "value",
                    "--notebook-container-image-properties": "value",
                    "--notebook-file-uris": "value",
                    "--notebook-vpc-network-name": "value",
                    "--notebook-vpc-network-tags": "value",
                    "--notebook-vpc-sub-network-name": "value",
                    "--spark-archive-uris": "value",
                    "--spark-file-uris": "value",
                    "--spark-main-class": "value",
                    "--spark-main-jar-file-uri": "value",
                    "--spark-python-script-file": "value",
                    "--spark-sql-script": "value",
                    "--spark-sql-script-file": "value",
                    "--trigger-disabled": "bool",
                    "--trigger-max-retires": "value",
                    "--trigger-schedule": "value",
                    "--trigger-start-time": "value",
                    "--trigger-type": [
                      "on-demand",
                      "recurring"
                    ],
                    "--vpc-network-name": "value",
                    "--vpc-network-tags": "value",
                    "--vpc-sub-network-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "jobs": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {
                        "--lake": "dynamic",
                        "--location": "dynamic",
                        "--task": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--lake": "dynamic",
                        "--location": "dynamic",
                        "--task": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--lake": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--task": "dynamic",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--batch-executors-count": "value",
                    "--batch-max-executors-count": "value",
                    "--clear-labels": "bool",
                    "--container-image": "value",
                    "--container-image-java-jars": "value",
                    "--container-image-properties": "value",
                    "--container-image-python-packages": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--execution-args": "value",
                    "--execution-project": "value",
                    "--execution-service-account": "value",
                    "--kms-key": "value",
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--max-job-execution-lifetime": "value",
                    "--notebook": "value",
                    "--notebook-archive-uris": "value",
                    "--notebook-batch-executors-count": "value",
                    "--notebook-batch-max-executors-count": "value",
                    "--notebook-container-image": "value",
                    "--notebook-container-image-java-jars": "value",
                    "--notebook-container-image-properties": "value",
                    "--notebook-file-uris": "value",
                    "--notebook-vpc-network-name": "value",
                    "--notebook-vpc-network-tags": "value",
                    "--notebook-vpc-sub-network-name": "value",
                    "--remove-labels": "value",
                    "--spark-archive-uris": "value",
                    "--spark-file-uris": "value",
                    "--spark-main-class": "value",
                    "--spark-main-jar-file-uri": "value",
                    "--spark-python-script-file": "value",
                    "--spark-sql-script": "value",
                    "--spark-sql-script-file": "value",
                    "--trigger-disabled": "bool",
                    "--trigger-max-retires": "value",
                    "--trigger-schedule": "value",
                    "--trigger-start-time": "value",
                    "--update-labels": "value",
                    "--vpc-network-name": "value",
                    "--vpc-network-tags": "value",
                    "--vpc-sub-network-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "actions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--lake": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csv-delimiter": "value",
                    "--csv-disable-type-inference": "bool",
                    "--csv-encoding": "value",
                    "--csv-header-rows": "value",
                    "--description": "value",
                    "--discovery-enabled": "bool",
                    "--discovery-exclude-patterns": "value",
                    "--discovery-include-patterns": "value",
                    "--discovery-schedule": "value",
                    "--display-name": "value",
                    "--json-disable-type-inference": "bool",
                    "--json-encoding": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--resource-location-type": [
                      "MULTI_REGION",
                      "SINGLE_REGION"
                    ],
                    "--type": [
                      "CURATED",
                      "RAW"
                    ],
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--lake": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csv-delimiter": "value",
                    "--csv-disable-type-inference": "bool",
                    "--csv-encoding": "value",
                    "--csv-header-rows": "value",
                    "--description": "value",
                    "--discovery-enabled": "bool",
                    "--discovery-exclude-patterns": "value",
                    "--discovery-include-patterns": "value",
                    "--discovery-schedule": "value",
                    "--display-name": "value",
                    "--json-disable-type-inference": "bool",
                    "--json-encoding": "value",
                    "--labels": "value",
                    "--lake": "value",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dataproc": {
          "commands": {
            "autoscaling-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-policy": "value",
                    "--bucket": "value",
                    "--dataproc-metastore": "value",
                    "--driver-pool-accelerator": "value",
                    "--driver-pool-boot-disk-size": "value",
                    "--driver-pool-boot-disk-type": "value",
                    "--driver-pool-id": "value",
                    "--driver-pool-local-ssd-interface": "value",
                    "--driver-pool-machine-type": "value",
                    "--driver-pool-min-cpu-platform": "value",
                    "--driver-pool-size": "value",
                    "--enable-component-gateway": "bool",
                    "--enable-kerberos": "bool",
                    "--expiration-time": "value",
                    "--gce-pd-kms-key": "dynamic",
                    "--gce-pd-kms-key-keyring": "dynamic",
                    "--gce-pd-kms-key-location": "dynamic",
                    "--gce-pd-kms-key-project": "dynamic",
                    "--image": "value",
                    "--image-version": "value",
                    "--initialization-action-timeout": "value",
                    "--initialization-actions": "value",
                    "--kerberos-config-file": "value",
                    "--kerberos-kms-key": "dynamic",
                    "--kerberos-kms-key-keyring": "dynamic",
                    "--kerberos-kms-key-location": "dynamic",
                    "--kerberos-kms-key-project": "dynamic",
                    "--kerberos-root-principal-password-uri": "value",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-boot-disk-size": "value",
                    "--master-boot-disk-type": "value",
                    "--master-local-ssd-interface": "value",
                    "--master-machine-type": "value",
                    "--master-min-cpu-platform": "value",
                    "--max-age": "value",
                    "--max-idle": "value",
                    "--metadata": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--node-group": "value",
                    "--num-driver-pool-local-ssds": "value",
                    "--num-master-local-ssds": "value",
                    "--num-masters": "value",
                    "--num-secondary-worker-local-ssds": "value",
                    "--num-secondary-workers": "value",
                    "--num-worker-local-ssds": "value",
                    "--num-workers": "value",
                    "--optional-components": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "inherit-subnetwork",
                      "outbound"
                    ],
                    "--properties": "value",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--scopes": "value",
                    "--secondary-worker-accelerator": "value",
                    "--secondary-worker-boot-disk-size": "value",
                    "--secondary-worker-boot-disk-type": "value",
                    "--secondary-worker-local-ssd-interface": "value",
                    "--secondary-worker-type": [
                      "non-preemptible",
                      "preemptible",
                      "spot"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--single-node": "bool",
                    "--subnet": "value",
                    "--tags": "value",
                    "--temp-bucket": "value",
                    "--worker-accelerator": "value",
                    "--worker-boot-disk-size": "value",
                    "--worker-boot-disk-type": "value",
                    "--worker-local-ssd-interface": "value",
                    "--worker-machine-type": "value",
                    "--worker-min-cpu-platform": "value",
                    "--zone": "value"
                  }
                },
                "create-from-file": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "diagnose": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "gke": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--gke-cluster": "value",
                        "--gke-cluster-location": "value",
                        "--history-server-cluster": "value",
                        "--history-server-cluster-region": "value",
                        "--metastore-service": "value",
                        "--metastore-service-location": "value",
                        "--namespace": "value",
                        "--pools": "value",
                        "--properties": "value",
                        "--region": "value",
                        "--setup-workload-identity": "bool",
                        "--spark-engine-version": "value",
                        "--staging-bucket": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-policy": "value",
                    "--clear-labels": "bool",
                    "--disable-autoscaling": "bool",
                    "--expiration-time": "value",
                    "--graceful-decommission-timeout": "value",
                    "--max-age": "value",
                    "--max-idle": "value",
                    "--no-max-age": "bool",
                    "--no-max-idle": "bool",
                    "--num-secondary-workers": "value",
                    "--num-workers": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "jobs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "kill": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--state-filter": [
                      "active",
                      "inactive"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "hadoop": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--class": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "hive": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "pig": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "presto": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--client-tags": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value"
                      }
                    },
                    "pyspark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--py-files": "value",
                        "--region": "value"
                      }
                    },
                    "spark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--class": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "spark-r": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "spark-sql": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "trino": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--client-tags": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "node-groups": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--graceful-decommission-timeout": "value",
                    "--region": "value",
                    "--size": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--state-filter": [
                      "active",
                      "inactive"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "workflow-templates": {
              "commands": {
                "add-job": {
                  "commands": {
                    "hadoop": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--class": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "hive": {
                      "commands": {},
                      "flags": {
                        "--continue-on-failure": "bool",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "pig": {
                      "commands": {},
                      "flags": {
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "presto": {
                      "commands": {},
                      "flags": {
                        "--client-tags": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "pyspark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--py-files": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--class": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark-r": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark-sql": {
                      "commands": {},
                      "flags": {
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "trino": {
                      "commands": {},
                      "flags": {
                        "--client-tags": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dag-timeout": "value",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "instantiate": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--parameters": "value",
                    "--region": "value"
                  }
                },
                "instantiate-from-file": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-dag-timeout": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "remove-job": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--step-id": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "set-cluster-selector": {
                  "commands": {},
                  "flags": {
                    "--cluster-labels": "value",
                    "--region": "value"
                  }
                },
                "set-dag-timeout": {
                  "commands": {},
                  "flags": {
                    "--dag-timeout": "value",
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "set-managed-cluster": {
                  "commands": {},
                  "flags": {
                    "--autoscaling-policy": "value",
                    "--bucket": "value",
                    "--cluster-name": "value",
                    "--dataproc-metastore": "value",
                    "--enable-component-gateway": "bool",
                    "--enable-kerberos": "bool",
                    "--image": "value",
                    "--image-version": "value",
                    "--initialization-action-timeout": "value",
                    "--initialization-actions": "value",
                    "--kerberos-config-file": "value",
                    "--kerberos-kms-key": "dynamic",
                    "--kerberos-kms-key-keyring": "dynamic",
                    "--kerberos-kms-key-location": "dynamic",
                    "--kerberos-kms-key-project": "dynamic",
                    "--kerberos-root-principal-password-uri": "value",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-boot-disk-size": "value",
                    "--master-boot-disk-type": "value",
                    "--master-local-ssd-interface": "value",
                    "--master-machine-type": "value",
                    "--master-min-cpu-platform": "value",
                    "--metadata": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--node-group": "value",
                    "--num-master-local-ssds": "value",
                    "--num-masters": "value",
                    "--num-secondary-worker-local-ssds": "value",
                    "--num-secondary-workers": "value",
                    "--num-worker-local-ssds": "value",
                    "--num-workers": "value",
                    "--optional-components": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "inherit-subnetwork",
                      "outbound"
                    ],
                    "--properties": "value",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--scopes": "value",
                    "--secondary-worker-accelerator": "value",
                    "--secondary-worker-boot-disk-size": "value",
                    "--secondary-worker-boot-disk-type": "value",
                    "--secondary-worker-local-ssd-interface": "value",
                    "--secondary-worker-type": [
                      "non-preemptible",
                      "preemptible",
                      "spot"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--single-node": "bool",
                    "--subnet": "value",
                    "--tags": "value",
                    "--temp-bucket": "value",
                    "--worker-accelerator": "value",
                    "--worker-boot-disk-size": "value",
                    "--worker-boot-disk-type": "value",
                    "--worker-local-ssd-interface": "value",
                    "--worker-machine-type": "value",
                    "--worker-min-cpu-platform": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "datastore": {
          "commands": {
            "databases": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--kinds": "value",
                "--namespaces": "value",
                "--operation-labels": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--kinds": "value",
                "--namespaces": "value",
                "--operation-labels": "value"
              }
            },
            "indexes": {
              "commands": {
                "cleanup": {
                  "commands": {},
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "deploy": {
          "commands": {
            "apply": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--force": "bool",
                "--region": "value"
              }
            },
            "delivery-pipelines": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "job-runs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "releases": {
              "commands": {
                "abandon": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--build-artifacts": "value",
                    "--delivery-pipeline": "dynamic",
                    "--description": "value",
                    "--disable-initial-rollout": "bool",
                    "--enable-initial-rollout": "bool",
                    "--from-k8s-manifest": "value",
                    "--from-run-manifest": "value",
                    "--gcs-source-staging-dir": "value",
                    "--ignore-file": "value",
                    "--images": "value",
                    "--initial-rollout-annotations": "value",
                    "--initial-rollout-labels": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--skaffold-file": "value",
                    "--skaffold-version": "value",
                    "--source": "value",
                    "--to-target": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "promote": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "dynamic",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout-id": "value",
                    "--to-target": "value"
                  }
                }
              },
              "flags": {}
            },
            "rollouts": {
              "commands": {
                "approve": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reject": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "retry-job": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--job-id": "value",
                    "--phase-id": "value",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "targets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "value",
                    "--list-all-pipelines": "bool",
                    "--region": "dynamic",
                    "--skip-pipeline-lookup": "bool"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "redeploy": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--rollout-id": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--release": "value",
                    "--rollout-id": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "deployment-manager": {
          "commands": {
            "deployments": {
              "commands": {
                "cancel-preview": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--fingerprint": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automatic-rollback-on-error": "bool",
                    "--composite-type": "value",
                    "--config": "value",
                    "--create-policy": [
                      "acquire",
                      "create",
                      "create-or-acquire"
                    ],
                    "--credential": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--preview": "bool",
                    "--properties": "value",
                    "--template": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--delete-policy": [
                      "abandon",
                      "delete"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--fingerprint": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--composite-type": "value",
                    "--config": "value",
                    "--create-policy": [
                      "acquire",
                      "create",
                      "create-or-acquire"
                    ],
                    "--credential": "value",
                    "--delete-policy": [
                      "abandon",
                      "delete"
                    ],
                    "--description": "value",
                    "--fingerprint": "value",
                    "--manifest-id": "value",
                    "--preview": "bool",
                    "--properties": "value",
                    "--remove-labels": "value",
                    "--template": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "manifests": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "resources": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--deployment": "value"
              }
            },
            "type-providers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--api-options-file": "value",
                    "--async": "bool",
                    "--custom-certificate-authority-roots": "value",
                    "--description": "value",
                    "--descriptor-url": "value",
                    "--labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api-options-file": "value",
                    "--async": "bool",
                    "--custom-certificate-authority-roots": "value",
                    "--description": "value",
                    "--descriptor-url": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "types": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--status": [
                      "DEPRECATED",
                      "EXPERIMENTAL",
                      "SUPPORTED"
                    ],
                    "--template": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--provider": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--provider": "value",
                    "--provider-project": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--remove-labels": "value",
                    "--status": [
                      "DEPRECATED",
                      "EXPERIMENTAL",
                      "SUPPORTED"
                    ],
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dialogflow": {
          "commands": {
            "agent": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--replace-all": "bool",
                    "--source": "value"
                  }
                },
                "query": {
                  "commands": {},
                  "flags": {
                    "--audio-encoding": [
                      "audio-encoding-amr",
                      "audio-encoding-amr-wb",
                      "audio-encoding-flac",
                      "audio-encoding-linear-16",
                      "audio-encoding-mulaw",
                      "audio-encoding-ogg-opus",
                      "audio-encoding-speex-with-header-byte",
                      "audio-encoding-unspecified"
                    ],
                    "--language": "value",
                    "--query-audio-file": "value",
                    "--query-text": "value",
                    "--sample-rate": "value",
                    "--session": "value"
                  }
                },
                "train": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                }
              },
              "flags": {}
            },
            "entity-types": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--auto-expand": "bool",
                    "--display-name": "value",
                    "--entities": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-entities": "value",
                    "--auto-expand": "bool",
                    "--clear-entities": "bool",
                    "--display-name": "value",
                    "--remove-entities": "value"
                  }
                }
              },
              "flags": {}
            },
            "intents": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--is-fallback": "bool",
                    "--ml-enabled": "bool",
                    "--other-properties": "value",
                    "--priority": "value",
                    "--responses": "value",
                    "--training-phrases": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--is-fallback": "bool",
                    "--ml-enabled": "bool",
                    "--other-properties": "value",
                    "--priority": "value",
                    "--responses": "value",
                    "--training-phrases": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dlp": {
          "commands": {
            "datasources": {
              "commands": {
                "bigquery": {
                  "commands": {
                    "analyze": {
                      "commands": {},
                      "flags": {
                        "--categorical-stat-field": "value",
                        "--job-id": "value",
                        "--numerical-stat-field": "value",
                        "--output-table": "value",
                        "--output-topics": "value",
                        "--quasi-ids": "value",
                        "--sensitive-attribute": "value"
                      }
                    },
                    "inspect": {
                      "commands": {},
                      "flags": {
                        "--exclude-info-types": "bool",
                        "--identifying-fields": "value",
                        "--include-quote": "bool",
                        "--info-types": "value",
                        "--job-id": "value",
                        "--max-findings": "value",
                        "--max-findings-per-item": "value",
                        "--max-time": "value",
                        "--min-likelihood": [
                          "likely",
                          "possible",
                          "unlikely",
                          "very-likely",
                          "very-unlikely"
                        ],
                        "--min-time": "value",
                        "--output-table": "value",
                        "--output-topics": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "datastore": {
                  "commands": {
                    "inspect": {
                      "commands": {},
                      "flags": {
                        "--exclude-info-types": "bool",
                        "--include-quote": "bool",
                        "--info-types": "value",
                        "--job-id": "value",
                        "--max-findings": "value",
                        "--max-findings-per-item": "value",
                        "--max-time": "value",
                        "--min-likelihood": [
                          "likely",
                          "possible",
                          "unlikely",
                          "very-likely",
                          "very-unlikely"
                        ],
                        "--min-time": "value",
                        "--output-table": "value",
                        "--output-topics": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "gcs": {
                  "commands": {
                    "inspect": {
                      "commands": {},
                      "flags": {
                        "--exclude-info-types": "bool",
                        "--file-size-limit": "value",
                        "--include-quote": "bool",
                        "--info-types": "value",
                        "--job-id": "value",
                        "--max-findings": "value",
                        "--max-findings-per-item": "value",
                        "--max-time": "value",
                        "--min-likelihood": [
                          "likely",
                          "possible",
                          "unlikely",
                          "very-likely",
                          "very-unlikely"
                        ],
                        "--min-time": "value",
                        "--output-table": "value",
                        "--output-topics": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "images": {
              "commands": {
                "inspect": {
                  "commands": {},
                  "flags": {
                    "--exclude-info-types": "bool",
                    "--include-quote": "bool",
                    "--info-types": "value",
                    "--max-findings": "value",
                    "--min-likelihood": [
                      "likely",
                      "possible",
                      "unlikely",
                      "very-likely",
                      "very-unlikely"
                    ]
                  }
                },
                "redact": {
                  "commands": {},
                  "flags": {
                    "--include-quote": "bool",
                    "--info-types": "value",
                    "--min-likelihood": [
                      "likely",
                      "possible",
                      "unlikely",
                      "very-likely",
                      "very-unlikely"
                    ],
                    "--output-file": "value",
                    "--redact-all-text": "bool",
                    "--redact-color": "value"
                  }
                }
              },
              "flags": {}
            },
            "job-triggers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--datastore-kind": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--exclude-info-types": "bool",
                    "--include-quote": "bool",
                    "--info-types": "value",
                    "--input-table": "value",
                    "--max-findings": "value",
                    "--max-findings-per-item": "value",
                    "--min-likelihood": [
                      "likely",
                      "possible",
                      "unlikely",
                      "very-likely",
                      "very-unlikely"
                    ],
                    "--output-table": "value",
                    "--output-topics": "value",
                    "--path": "value",
                    "--trigger-schedule": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--job-type": [
                      "inspect",
                      "risk-analysis"
                    ],
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "text": {
              "commands": {
                "inspect": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--exclude-info-types": "bool",
                    "--include-quote": "bool",
                    "--info-types": "value",
                    "--max-findings": "value",
                    "--min-likelihood": [
                      "likely",
                      "possible",
                      "unlikely",
                      "very-likely",
                      "very-unlikely"
                    ]
                  }
                },
                "redact": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--info-types": "value",
                    "--min-likelihood": [
                      "likely",
                      "possible",
                      "unlikely",
                      "very-likely",
                      "very-unlikely"
                    ],
                    "--output-file": "value",
                    "--remove-findings": "bool",
                    "--replace-with-info-type": "bool",
                    "--replacement-text": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dns": {
          "commands": {
            "active-peering-zones": {
              "commands": {},
              "flags": {}
            },
            "dns-keys": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "managed-zones": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--denial-of-existence": [
                      "nsec",
                      "nsec3"
                    ],
                    "--description": "value",
                    "--dns-name": "value",
                    "--dnssec-state": [
                      "off",
                      "on",
                      "transfer"
                    ],
                    "--forwarding-targets": "value",
                    "--gkeclusters": "value",
                    "--ksk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--ksk-key-length": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--log-dns-queries": "bool",
                    "--managed-reverse-lookup": "bool",
                    "--networks": "value",
                    "--private-forwarding-targets": "value",
                    "--service-directory-namespace": "value",
                    "--target-network": "value",
                    "--target-project": "value",
                    "--visibility": [
                      "private",
                      "public"
                    ],
                    "--zsk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--zsk-key-length": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--policy-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--denial-of-existence": [
                      "nsec",
                      "nsec3"
                    ],
                    "--description": "value",
                    "--dnssec-state": [
                      "off",
                      "on",
                      "transfer"
                    ],
                    "--forwarding-targets": "value",
                    "--gkeclusters": "value",
                    "--ksk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--ksk-key-length": "value",
                    "--location": "value",
                    "--log-dns-queries": "bool",
                    "--managed-reverse-lookup": "bool",
                    "--networks": "value",
                    "--private-forwarding-targets": "value",
                    "--remove-labels": "value",
                    "--target-network": "value",
                    "--target-project": "value",
                    "--update-labels": "value",
                    "--zsk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--zsk-key-length": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "policies": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--alternative-name-servers": "value",
                    "--description": "value",
                    "--enable-inbound-forwarding": "bool",
                    "--enable-logging": "bool",
                    "--networks": "value",
                    "--private-alternative-name-servers": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--alternative-name-servers": "value",
                    "--description": "value",
                    "--enable-inbound-forwarding": "bool",
                    "--enable-logging": "bool",
                    "--networks": "value",
                    "--private-alternative-name-servers": "value"
                  }
                }
              },
              "flags": {}
            },
            "project-info": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "record-sets": {
              "commands": {
                "changes": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--sort-order": [
                          "ascending",
                          "descending"
                        ],
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backup-data-trickle-ratio": "value",
                    "--enable-geo-fencing": "bool",
                    "--enable-health-checking": "bool",
                    "--location": "value",
                    "--routing-policy-backup-data": "value",
                    "--routing-policy-backup-data-type": [
                      "GEO"
                    ],
                    "--routing-policy-primary-data": "value",
                    "--routing_policy_data": "value",
                    "--routing_policy_type": [
                      "FAILOVER",
                      "GEO",
                      "WRR"
                    ],
                    "--rrdatas": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "dynamic",
                    "--zone-file-format": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--delete-all-existing": "bool",
                    "--location": "value",
                    "--replace-origin-ns": "bool",
                    "--zone": "dynamic",
                    "--zone-file-format": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--name": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "transaction": {
                  "commands": {
                    "abort": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "add": {
                      "commands": {},
                      "flags": {
                        "--name": "value",
                        "--transaction-file": "value",
                        "--ttl": "value",
                        "--type": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "execute": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--transaction-file": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--name": "value",
                        "--transaction-file": "value",
                        "--ttl": "value",
                        "--type": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "start": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {
                    "--transaction-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backup-data-trickle-ratio": "value",
                    "--enable-geo-fencing": "bool",
                    "--enable-health-checking": "bool",
                    "--location": "value",
                    "--routing-policy-backup-data": "value",
                    "--routing-policy-backup-data-type": [
                      "GEO"
                    ],
                    "--routing-policy-primary-data": "value",
                    "--routing_policy_data": "value",
                    "--routing_policy_type": [
                      "FAILOVER",
                      "GEO",
                      "WRR"
                    ],
                    "--rrdatas": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "response-policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--gkeclusters": "value",
                    "--location": "value",
                    "--networks": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--behavior": [
                          "behaviorUnspecified",
                          "bypassResponsePolicy"
                        ],
                        "--dns-name": "value",
                        "--local-data": "value",
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--behavior": [
                          "behaviorUnspecified",
                          "bypassResponsePolicy"
                        ],
                        "--dns-name": "value",
                        "--local-data": "value",
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--gkeclusters": "value",
                    "--location": "value",
                    "--networks": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "domains": {
          "commands": {
            "registrations": {
              "commands": {
                "authorization-code": {
                  "commands": {
                    "get": {
                      "commands": {},
                      "flags": {}
                    },
                    "reset": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "configure": {
                  "commands": {
                    "contacts": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--contact-data-from-file": "value",
                        "--contact-privacy": [
                          "private-contact-data",
                          "public-contact-data",
                          "redacted-contact-data"
                        ],
                        "--notices": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "dns": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cloud-dns-zone": "value",
                        "--disable-dnssec": "bool",
                        "--dns-settings-from-file": "value",
                        "--name-servers": "value",
                        "--unsafe-dns-update": "bool",
                        "--use-google-domains-dns": "bool",
                        "--validate-only": "bool"
                      }
                    },
                    "management": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--transfer-lock-state": [
                          "locked",
                          "unlocked"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "get-register-parameters": {
                  "commands": {},
                  "flags": {}
                },
                "get-transfer-parameters": {
                  "commands": {},
                  "flags": {}
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--labels": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-importable-domains": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cloud-dns-zone": "value",
                    "--contact-data-from-file": "value",
                    "--contact-privacy": [
                      "private-contact-data",
                      "public-contact-data",
                      "redacted-contact-data"
                    ],
                    "--disable-dnssec": "bool",
                    "--labels": "value",
                    "--name-servers": "value",
                    "--notices": "value",
                    "--use-google-domains-dns": "bool",
                    "--validate-only": "bool",
                    "--yearly-price": "value"
                  }
                },
                "search-domains": {
                  "commands": {},
                  "flags": {}
                },
                "transfer": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorization-code-from-file": "value",
                    "--cloud-dns-zone": "value",
                    "--contact-data-from-file": "value",
                    "--contact-privacy": [
                      "private-contact-data",
                      "public-contact-data",
                      "redacted-contact-data"
                    ],
                    "--keep-dns-settings": "bool",
                    "--labels": "value",
                    "--notices": "value",
                    "--use-google-domains-dns": "bool",
                    "--validate-only": "bool",
                    "--yearly-price": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "edge-cache": {
          "commands": {
            "keysets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--public-key": "value",
                    "--validation-shared-key": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--public-key": "value",
                    "--validation-shared-key": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "origins": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--failover-origin": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--origin-address": "value",
                    "--port": "value",
                    "--protocol": [
                      "http",
                      "http2",
                      "https"
                    ],
                    "--response-timeout": "value",
                    "--retry-conditions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--failover-origin": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--origin-address": "value",
                    "--port": "value",
                    "--protocol": [
                      "http",
                      "http2",
                      "https"
                    ],
                    "--response-timeout": "value",
                    "--retry-conditions": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "invalidate-cache": {
                  "commands": {},
                  "flags": {
                    "--host": "value",
                    "--location": "value",
                    "--path": "value",
                    "--tags": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--edge-security-policy": "value",
                    "--edge-ssl-certificate": "value",
                    "--enable-logging": "bool",
                    "--labels": "value",
                    "--location": "value",
                    "--logging-sample-rate": "value",
                    "--require-tls": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "edge-cloud": {
          "commands": {
            "container": {
              "commands": {
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--admin-users": "value",
                        "--async": "bool",
                        "--cluster-ipv4-cidr": "value",
                        "--cluster-ipv6-cidr": "value",
                        "--default-max-pods-per-node": "value",
                        "--fleet-project": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--maintenance-window-end": "value",
                        "--maintenance-window-recurrence": "value",
                        "--maintenance-window-start": "value",
                        "--services-ipv4-cidr": "value",
                        "--services-ipv6-cidr": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--auth-provider-cmd-path": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "node-pools": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--cluster": "dynamic",
                            "--labels": "value",
                            "--local-disk-kms-key": "value",
                            "--location": "dynamic",
                            "--machine-filter": "value",
                            "--node-count": "value",
                            "--node-location": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--cluster": "dynamic",
                            "--location": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--cluster": "dynamic",
                            "--location": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--cluster": "dynamic",
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "dynamic",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--clear-labels": "bool",
                            "--cluster": "dynamic",
                            "--location": "dynamic",
                            "--machine-filter": "value",
                            "--node-count": "value",
                            "--remove-labels": "value",
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-maintenance-window": "bool",
                        "--location": "dynamic",
                        "--maintenance-window-end": "value",
                        "--maintenance-window-recurrence": "value",
                        "--maintenance-window-start": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "machines": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "regions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "vpn-connections": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cluster": "value",
                        "--high-availability": "bool",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--nat-gateway-ip": "value",
                        "--router": "value",
                        "--vpc-network": "value",
                        "--vpc-project": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "zones": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "networking": {
              "commands": {
                "interconnects": {
                  "commands": {
                    "attachments": {
                      "commands": {
                        "dedicated": {
                          "commands": {
                            "create": {
                              "commands": {},
                              "flags": {
                                "--async": "bool",
                                "--description": "value",
                                "--interconnect": "value",
                                "--labels": "value",
                                "--location": "dynamic",
                                "--mtu": "value",
                                "--network": "value",
                                "--vlan-id": "value",
                                "--zone": "dynamic"
                              }
                            }
                          },
                          "flags": {}
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--location": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "dynamic",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "get-diagnostics": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "networks": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--mtu": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "get-status": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "routers": {
                  "commands": {
                    "add-bgp-peer": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--interface": "value",
                        "--location": "dynamic",
                        "--peer-asn": "value",
                        "--peer-ipv4-range": "value",
                        "--peer-name": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "add-interface": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--interconnect-attachment": "value",
                        "--interface-name": "value",
                        "--ip-address": "value",
                        "--ip-mask-length": "value",
                        "--location": "dynamic",
                        "--loopback-ip-addresses": "value",
                        "--subnetwork": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--asn": "value",
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--network": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "get-status": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-bgp-peer": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--peer-name": "value",
                        "--peer-names": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-interface": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--interface-name": "value",
                        "--interface-names": "value",
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-advertisement-ranges": "value",
                        "--async": "bool",
                        "--location": "dynamic",
                        "--remove-advertisement-ranges": "value",
                        "--set-advertisement-ranges": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "subnets": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--ipv4-range": "value",
                        "--ipv6-range": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--network": "value",
                        "--vlan-id": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "zones": {
                  "commands": {
                    "init": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "emulators": {
          "commands": {
            "bigtable": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {}
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--host-port": "value"
                  }
                }
              },
              "flags": {}
            },
            "datastore": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "env-unset": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--consistency": "value",
                    "--data-dir": "value",
                    "--host-port": "value",
                    "--store-on-disk": "bool",
                    "--use-firestore-in-datastore-mode": "bool"
                  }
                }
              },
              "flags": {
                "--data-dir": "value"
              }
            },
            "firestore": {
              "commands": {
                "start": {
                  "commands": {},
                  "flags": {
                    "--host-port": "value",
                    "--rules": "value"
                  }
                }
              },
              "flags": {}
            },
            "pubsub": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value",
                    "--host-port": "value"
                  }
                }
              },
              "flags": {
                "--data-dir": "value"
              }
            },
            "spanner": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {}
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--enable-fault-injection": "value",
                    "--host-port": "value",
                    "--rest-port": "value",
                    "--use-docker": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "endpoints": {
          "commands": {
            "configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--service": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "quota": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--dimensions": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--service": "value",
                    "--unit": "value",
                    "--value": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--override-id": "value",
                    "--service": "value",
                    "--unit": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--dimensions": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--service": "value",
                    "--unit": "value",
                    "--value": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "check-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "essential-contacts": {
          "commands": {
            "compute": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--notification-categories": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--email": "value",
                "--folder": "value",
                "--language": "value",
                "--notification-categories": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--language": "value",
                "--notification-categories": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "filestore": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--instance": "value",
                    "--instance-zone": "value",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--network": "value",
                    "--region": "value",
                    "--tier": [
                      "basic-hdd",
                      "basic-ssd",
                      "enterprise",
                      "high-scale-ssd",
                      "premium",
                      "standard"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file-share": "value",
                    "--source-backup": "value",
                    "--source-backup-region": "value",
                    "--source-snapshot": "value",
                    "--source-snapshot-region": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--location": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file-share": "value",
                    "--instance": "value",
                    "--instance-zone": "value",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "firebase": {
          "commands": {
            "test": {
              "commands": {
                "android": {
                  "commands": {
                    "list-device-capacities": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "locales": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "models": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "run": {
                      "commands": {},
                      "flags": {
                        "--additional-apks": "value",
                        "--app": "value",
                        "--app-package": "value",
                        "--async": "bool",
                        "--auto-google-login": "bool",
                        "--client-details": "value",
                        "--device": "value",
                        "--device-ids": "value",
                        "--directories-to-pull": "value",
                        "--environment-variables": "value",
                        "--filter": "value",
                        "--grant-permissions": [
                          "all",
                          "none"
                        ],
                        "--limit": "value",
                        "--locales": "value",
                        "--network-profile": "value",
                        "--num-flaky-test-attempts": "value",
                        "--num-uniform-shards": "value",
                        "--obb-files": "value",
                        "--orientations": "dynamic",
                        "--os-version-ids": "value",
                        "--other-files": "value",
                        "--page-size": "value",
                        "--performance-metrics": "bool",
                        "--record-video": "bool",
                        "--resign": "bool",
                        "--results-bucket": "value",
                        "--results-dir": "value",
                        "--results-history-name": "value",
                        "--robo-directives": "value",
                        "--robo-script": "value",
                        "--scenario-labels": "value",
                        "--scenario-numbers": "value",
                        "--sort-by": "value",
                        "--test": "value",
                        "--test-package": "value",
                        "--test-runner-class": "value",
                        "--test-targets": "value",
                        "--test-targets-for-shard": "value",
                        "--timeout": "value",
                        "--type": [
                          "game-loop",
                          "instrumentation",
                          "robo"
                        ],
                        "--use-orchestrator": "bool"
                      }
                    },
                    "versions": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "ios": {
                  "commands": {
                    "list-device-capacities": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "locales": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "models": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "run": {
                      "commands": {},
                      "flags": {
                        "--additional-ipas": "value",
                        "--app": "value",
                        "--async": "bool",
                        "--client-details": "value",
                        "--device": "value",
                        "--directories-to-pull": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--num-flaky-test-attempts": "value",
                        "--other-files": "value",
                        "--page-size": "value",
                        "--record-video": "bool",
                        "--results-bucket": "value",
                        "--results-dir": "value",
                        "--results-history-name": "value",
                        "--robo-script": "value",
                        "--scenario-numbers": "value",
                        "--sort-by": "value",
                        "--test": "value",
                        "--test-special-entitlements": "bool",
                        "--timeout": "value",
                        "--type": [
                          "game-loop",
                          "robo",
                          "xctest"
                        ],
                        "--xcode-version": "value",
                        "--xctestrun-file": "value"
                      }
                    },
                    "versions": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "ip-blocks": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "network-profiles": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "firestore": {
          "commands": {
            "databases": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--location": "value",
                    "--type": [
                      "datastore-mode",
                      "firestore-native"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--type": [
                      "datastore-mode",
                      "firestore-native"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--collection-ids": "value",
                "--database": "value",
                "--namespace-ids": "value"
              }
            },
            "fields": {
              "commands": {
                "ttls": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--disable-ttl": "bool",
                        "--enable-ttl": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--collection-ids": "value",
                "--database": "value",
                "--namespace-ids": "value"
              }
            },
            "indexes": {
              "commands": {
                "composite": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--api-scope": [
                          "any-api",
                          "datastore-mode-api"
                        ],
                        "--async": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--field-config": "value",
                        "--query-scope": [
                          "collection",
                          "collection-group",
                          "collection-recursive"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--database": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--database": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "fields": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-exemption": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--disable-indexes": "bool",
                        "--index": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "functions": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "add-invoker-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": "dynamic"
              }
            },
            "call": {
              "commands": {},
              "flags": {
                "--cloud-event": "value",
                "--data": "value",
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "deploy": {
              "commands": {},
              "flags": {
                "--allow-unauthenticated": "bool",
                "--build-env-vars-file": "value",
                "--build-worker-pool": "value",
                "--buildpack-stack": "value",
                "--clear-build-env-vars": "bool",
                "--clear-build-worker-pool": "bool",
                "--clear-docker-repository": "bool",
                "--clear-env-vars": "bool",
                "--clear-kms-key": "bool",
                "--clear-labels": "bool",
                "--clear-max-instances": "bool",
                "--clear-min-instances": "bool",
                "--clear-secrets": "bool",
                "--clear-vpc-connector": "bool",
                "--concurrency": "value",
                "--cpu": "value",
                "--docker-registry": [
                  "artifact-registry",
                  "container-registry"
                ],
                "--docker-repository": "value",
                "--egress-settings": [
                  "all",
                  "private-ranges-only"
                ],
                "--entry-point": "value",
                "--env-vars-file": "value",
                "--gen2": "bool",
                "--ignore-file": "value",
                "--ingress-settings": [
                  "all",
                  "internal-and-gclb",
                  "internal-only"
                ],
                "--kms-key": "value",
                "--max-instances": "value",
                "--memory": "value",
                "--min-instances": "value",
                "--region": "dynamic",
                "--remove-build-env-vars": "value",
                "--remove-env-vars": "value",
                "--remove-labels": "value",
                "--remove-secrets": "value",
                "--retry": "bool",
                "--run-service-account": "value",
                "--runtime": "value",
                "--security-level": [
                  "secure-always",
                  "secure-optional"
                ],
                "--serve-all-traffic-latest-revision": "bool",
                "--service-account": "value",
                "--set-build-env-vars": "value",
                "--set-env-vars": "value",
                "--set-secrets": "value",
                "--source": "value",
                "--stage-bucket": "value",
                "--timeout": "value",
                "--trigger-bucket": "value",
                "--trigger-event": "value",
                "--trigger-event-filters": "value",
                "--trigger-event-filters-path-pattern": "value",
                "--trigger-http": "bool",
                "--trigger-location": "dynamic",
                "--trigger-resource": "value",
                "--trigger-service-account": "value",
                "--trigger-topic": "value",
                "--update-build-env-vars": "value",
                "--update-env-vars": "value",
                "--update-labels": "value",
                "--update-secrets": "value",
                "--vpc-connector": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "event-types": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--gen2": "bool"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--gen2": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regions": "value",
                "--sort-by": "value"
              }
            },
            "logs": {
              "commands": {
                "read": {
                  "commands": {},
                  "flags": {
                    "--end-time": "value",
                    "--execution-id": "value",
                    "--filter": "value",
                    "--gen2": "bool",
                    "--limit": "value",
                    "--min-log-level": [
                      "debug",
                      "error",
                      "info"
                    ],
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--start-time": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--gen2": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "remove-invoker-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": "dynamic"
              }
            },
            "runtimes": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "game": {
          "commands": {
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "servers": {
              "commands": {
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--dry-run": "bool",
                        "--gke-cluster": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--namespace": "value",
                        "--preview-time": "value",
                        "--realm": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--realm": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--realm": "value",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--realm": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--realm": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "configs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deployment": "value",
                        "--description": "value",
                        "--fleet-configs-file": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--scaling-configs-file": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deployment": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--deployment": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--deployment": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "deployments": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "describe-rollout": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "fetch-state": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    },
                    "update-rollout": {
                      "commands": {},
                      "flags": {
                        "--clear-config-overrides": "bool",
                        "--clear-default-config": "bool",
                        "--config-overrides-file": "value",
                        "--default-config": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "realms": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--time-zone": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--remove-labels": "value",
                        "--time-zone": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "genomics": {
          "commands": {
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "pipelines": {
              "commands": {
                "run": {
                  "commands": {},
                  "flags": {
                    "--boot-disk-size": "value",
                    "--command-line": "value",
                    "--cpus": "value",
                    "--disk-size": "value",
                    "--docker-image": "value",
                    "--env-vars": "value",
                    "--inputs": "value",
                    "--inputs-from-file": "value",
                    "--labels": "value",
                    "--logging": "value",
                    "--memory": "value",
                    "--network": "value",
                    "--outputs": "value",
                    "--pipeline-file": "value",
                    "--preemptible": "bool",
                    "--regions": "value",
                    "--service-account-email": "value",
                    "--service-account-scopes": "value",
                    "--subnetwork": "value",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "healthcare": {
          "commands": {
            "annotation-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "evaluate": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--golden-store": "value",
                        "--location": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--location": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "consent-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "check-data-access": {
                  "commands": {},
                  "flags": {
                    "--data-id": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--request-attributes": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "evaluate-user-consents": {
                  "commands": {},
                  "flags": {
                    "--consent-list": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--request-attributes": "value",
                    "--resource-attributes": "value",
                    "--response-view": [
                      "basic",
                      "full",
                      "response-view-unspecified"
                    ],
                    "--user-id": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "query-accessible-data": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value",
                    "--request-attributes": "value",
                    "--resource-attributes": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--dataset": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "datasets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--time-zone": "value"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--default-fhir-config": "bool",
                    "--destination-dataset": "value",
                    "--dicom-filter-tags": "value",
                    "--location": "value",
                    "--text-redaction-mode": [
                      "all"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--time-zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "dicom-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value",
                    "--send-for-bulk-import": "bool"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--dataset": "value",
                    "--destination-store": "value",
                    "--dicom-filter-tags": "value",
                    "--location": "value",
                    "--text-redaction-mode": [
                      "all"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--filter-config-gcs-uri": "value",
                        "--location": "value",
                        "--overwrite-table": "bool",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--filter-config-gcs-uri": "value",
                        "--gcs-uri-prefix": "value",
                        "--location": "value",
                        "--mime-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value",
                    "--send-for-bulk-import": "bool"
                  }
                }
              },
              "flags": {}
            },
            "fhir-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--disable-referential-integrity": "bool",
                    "--disable-resource-versioning": "bool",
                    "--enable-update-create": "bool",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--dataset": "value",
                    "--destination-store": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-dataset": "value",
                        "--dataset": "value",
                        "--location": "value",
                        "--recursive-depth": "value",
                        "--resource-type": "value",
                        "--schema-type": [
                          "analytics",
                          "analytics_v2",
                          "lossless"
                        ],
                        "--since": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value",
                        "--resource-type": "value",
                        "--since": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--content-structure": [
                          "bundle",
                          "bundle-pretty",
                          "resource",
                          "resource-pretty"
                        ],
                        "--dataset": "value",
                        "--error-gcs-uri": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--enable-update-create": "bool",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "hl7v2-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--end-time": "value",
                        "--gcs-uri": "value",
                        "--location": "value",
                        "--message-view": [
                          "basic",
                          "full",
                          "parsed-only",
                          "raw-only",
                          "schematized-only"
                        ],
                        "--start-time": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "nlp": {
              "commands": {
                "analyze-entities": {
                  "commands": {},
                  "flags": {
                    "--document-content": "value",
                    "--nlp-service": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "help": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "iam": {
          "commands": {
            "list-grantable-roles": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--page-size": "value"
              }
            },
            "list-testable-permissions": {
              "commands": {},
              "flags": {
                "--filter": "value"
              }
            },
            "policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value",
                    "--policy-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--etag": "value",
                    "--kind": "value"
                  }
                },
                "get": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value"
                  }
                },
                "lint-condition": {
                  "commands": {},
                  "flags": {
                    "--condition-from-file": "value",
                    "--description": "value",
                    "--expression": "value",
                    "--resource-name": "value",
                    "--title": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--filter": "value",
                    "--kind": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--page_token": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value",
                    "--policy-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "roles": {
              "commands": {
                "copy": {
                  "commands": {},
                  "flags": {
                    "--dest-organization": "value",
                    "--dest-project": "value",
                    "--destination": "value",
                    "--source": "value",
                    "--source-organization": "value",
                    "--source-project": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--file": "value",
                    "--organization": "value",
                    "--permissions": "value",
                    "--stage": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-permissions": "value",
                    "--description": "value",
                    "--file": "value",
                    "--organization": "value",
                    "--permissions": "value",
                    "--remove-permissions": "value",
                    "--stage": "value",
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            },
            "service-accounts": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {}
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "keys": {
                  "commands": {
                    "config": {
                      "commands": {
                        "export": {
                          "commands": {},
                          "flags": {
                            "--all": "bool",
                            "--iam-account": "dynamic",
                            "--path": "value",
                            "--resource-format": [
                              "krm",
                              "terraform"
                            ]
                          }
                        }
                      },
                      "flags": {}
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value",
                        "--key-file-type": [
                          "json",
                          "p12"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    },
                    "get-public-key": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value",
                        "--output-file": "value",
                        "--type": [
                          "pem",
                          "raw"
                        ]
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--created-before": "value",
                        "--filter": "value",
                        "--iam-account": "value",
                        "--limit": "value",
                        "--managed-by": [
                          "any",
                          "system",
                          "user"
                        ],
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "upload": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "sign-blob": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value"
                  }
                },
                "sign-jwt": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "workforce-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--session-duration": "value"
                  }
                },
                "create-cred-config": {
                  "commands": {},
                  "flags": {
                    "--credential-source-field-name": "value",
                    "--credential-source-file": "value",
                    "--credential-source-headers": "value",
                    "--credential-source-type": "value",
                    "--credential-source-url": "value",
                    "--executable-command": "value",
                    "--executable-interactive-timeout-millis": "value",
                    "--executable-output-file": "value",
                    "--executable-timeout-millis": "value",
                    "--output-file": "value",
                    "--service-account": "value",
                    "--service-account-token-lifetime-seconds": "value",
                    "--subject-token-type": "value",
                    "--workforce-pool-user-project": "value"
                  }
                },
                "create-login-config": {
                  "commands": {},
                  "flags": {
                    "--activate": "bool",
                    "--output-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "providers": {
                  "commands": {
                    "create-oidc": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--client-id": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--show-deleted": "bool",
                        "--sort-by": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "operations": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--provider": "dynamic",
                            "--workforce-pool": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "update-oidc": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--client-id": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--session-duration": "value"
                  }
                }
              },
              "flags": {}
            },
            "workload-identity-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                },
                "create-cred-config": {
                  "commands": {},
                  "flags": {
                    "--app-id-uri": "value",
                    "--aws": "bool",
                    "--azure": "bool",
                    "--credential-source-field-name": "value",
                    "--credential-source-file": "value",
                    "--credential-source-headers": "value",
                    "--credential-source-type": "value",
                    "--credential-source-url": "value",
                    "--enable-imdsv2": "bool",
                    "--executable-command": "value",
                    "--executable-output-file": "value",
                    "--executable-timeout-millis": "value",
                    "--output-file": "value",
                    "--service-account": "value",
                    "--service-account-token-lifetime-seconds": "value",
                    "--subject-token-type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "providers": {
                  "commands": {
                    "create-aws": {
                      "commands": {},
                      "flags": {
                        "--account-id": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "create-oidc": {
                      "commands": {},
                      "flags": {
                        "--allowed-audiences": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--show-deleted": "bool",
                        "--sort-by": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "operations": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--provider": "dynamic",
                            "--workload-identity-pool": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "update-aws": {
                      "commands": {},
                      "flags": {
                        "--account-id": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "update-oidc": {
                      "commands": {},
                      "flags": {
                        "--allowed-audiences": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "iap": {
          "commands": {
            "oauth-brands": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--application_title": "value",
                    "--support_email": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "oauth-clients": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--display_name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "reset-secret": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "get": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value",
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "compute",
                      "folder",
                      "iap_web",
                      "organization"
                    ],
                    "--service": "value"
                  }
                },
                "set": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value",
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "compute",
                      "folder",
                      "iap_web",
                      "organization"
                    ],
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "tcp": {
              "commands": {
                "dest-groups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--dest-group": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--fqdn-list": "value",
                        "--ip-range-list": "value",
                        "--region": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--dest-group": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--dest-group": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--dest-group": "value",
                        "--region": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--fqdn-list": "value",
                        "--ip-range-list": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "web": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--role": "value",
                    "--service": "value"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--oauth2-client-id": "value",
                    "--oauth2-client-secret": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--role": "value",
                    "--service": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "identity": {
          "commands": {
            "groups": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--dynamic-user-query": "value",
                    "--group-type": [
                      "discussion",
                      "dynamic",
                      "security"
                    ],
                    "--labels": "value",
                    "--organization": "value",
                    "--posix-group": "value",
                    "--with-initial-owner": [
                      "empty",
                      "with-initial-owner"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--expiration": "value",
                        "--group-email": "value",
                        "--member-email": "value",
                        "--organization": "value",
                        "--roles": "value"
                      }
                    },
                    "check-transitive-membership": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "get-membership-graph": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--labels": "value",
                        "--member-email": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--group-email": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--page-token": "value",
                        "--sort-by": "value",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "modify-membership-roles": {
                      "commands": {},
                      "flags": {
                        "--add-roles": "value",
                        "--group-email": "value",
                        "--member-email": "value",
                        "--remove-roles": "value",
                        "--update-roles-params": "value"
                      }
                    },
                    "search-transitive-groups": {
                      "commands": {},
                      "flags": {
                        "--labels": "value",
                        "--member-email": "value",
                        "--page-size": "value",
                        "--page-token": "value"
                      }
                    },
                    "search-transitive-memberships": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--page-size": "value",
                        "--page-token": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "preview": {
                  "commands": {},
                  "flags": {
                    "--custom-field-mask": "value",
                    "--customer": "value",
                    "--max-results": "value",
                    "--page-token": "value",
                    "--projection": [
                      "basic",
                      "custom",
                      "full"
                    ],
                    "--query": "value",
                    "--view-type": [
                      "admin-view",
                      "domain-public"
                    ]
                  }
                },
                "search": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-posix-group": "value",
                    "--clear-description": "bool",
                    "--clear-display-name": "bool",
                    "--clear-posix-groups": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--dynamic-user-query": "value",
                    "--remove-posix-groups": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ids": {
          "commands": {
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--enable-traffic-logs": "bool",
                    "--labels": "value",
                    "--max-wait": "value",
                    "--network": "value",
                    "--severity": [
                      "CRITICAL",
                      "HIGH",
                      "INFORMATIONAL",
                      "LOW",
                      "MEDIUM"
                    ],
                    "--threat-exceptions": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--max-wait": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--max-wait": "value",
                    "--threat-exceptions": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--max-wait": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "immersive-stream": {
          "commands": {
            "xr": {
              "commands": {
                "contents": {
                  "commands": {
                    "build": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--location": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "instances": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--add-region": "value",
                        "--async": "bool",
                        "--content": "value",
                        "--fallback-url": "value",
                        "--location": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-region": "value",
                        "--async": "bool",
                        "--fallback-url": "value",
                        "--location": "value",
                        "--remove-region": "value",
                        "--update-region": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "init": {
          "commands": {},
          "flags": {
            "--console-only": "bool",
            "--no-browser": "bool",
            "--no-launch-browser": "bool",
            "--skip-diagnostics": "bool"
          }
        },
        "interactive": {
          "commands": {},
          "flags": {
            "--context": "value"
          }
        },
        "iot": {
          "commands": {
            "devices": {
              "commands": {
                "commands": {
                  "commands": {
                    "send": {
                      "commands": {},
                      "flags": {
                        "--command-data": "value",
                        "--command-file": "value",
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--subfolder": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "configs": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "get-value": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-data": "value",
                        "--config-file": "value",
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--version-to-update": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--auth-method": [
                      "association-and-device-auth-token",
                      "association-only",
                      "device-auth-token-only"
                    ],
                    "--blocked": "bool",
                    "--device-type": [
                      "gateway",
                      "non-gateway"
                    ],
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--public-key": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "credentials": {
                  "commands": {
                    "clear": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--expiration-time": "value",
                        "--path": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--type": [
                          "es256",
                          "es256-pem",
                          "es256-x509-pem",
                          "rs256",
                          "rsa-pem",
                          "rsa-x509-pem"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--expiration-time": "value",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "gateways": {
                  "commands": {
                    "bind": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--device-region": "value",
                        "--device-registry": "dynamic",
                        "--gateway": "dynamic",
                        "--gateway-region": "value",
                        "--gateway-registry": "dynamic"
                      }
                    },
                    "list-bound-devices": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--gateway": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "unbind": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--device-region": "value",
                        "--device-registry": "dynamic",
                        "--gateway": "dynamic",
                        "--gateway-region": "value",
                        "--gateway-registry": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--device-field-mask": "value",
                    "--device-ids": "value",
                    "--device-num-ids": "value",
                    "--device-type": [
                      "gateway",
                      "non-gateway"
                    ],
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "states": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--auth-method": [
                      "association-and-device-auth-token",
                      "association-only",
                      "device-auth-token-only"
                    ],
                    "--blocked": "bool",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "registries": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--enable-http-config": "bool",
                    "--enable-mqtt-config": "bool",
                    "--event-notification-config": "value",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--public-key-path": "value",
                    "--region": "value",
                    "--state-pubsub-topic": "value"
                  }
                },
                "credentials": {
                  "commands": {
                    "clear": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--path": "value",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--enable-http-config": "bool",
                    "--enable-mqtt-config": "bool",
                    "--event-notification-config": "value",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--region": "value",
                    "--state-pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "kms": {
          "commands": {
            "asymmetric-decrypt": {
              "commands": {},
              "flags": {
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "asymmetric-sign": {
              "commands": {},
              "flags": {
                "--digest-algorithm": [
                  "sha256",
                  "sha384",
                  "sha512"
                ],
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "decrypt": {
              "commands": {},
              "flags": {
                "--additional-authenticated-data-file": "value",
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "ekm-connections": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--hostname": "value",
                    "--location": "dynamic",
                    "--server-certificates-files": "value",
                    "--service-directory-service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--hostname": "value",
                    "--location": "dynamic",
                    "--server-certificates-files": "value",
                    "--service-directory-service": "value"
                  }
                }
              },
              "flags": {}
            },
            "encrypt": {
              "commands": {},
              "flags": {
                "--additional-authenticated-data-file": "value",
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "import-jobs": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--import-method": [
                      "rsa-oaep-3072-sha1-aes-256",
                      "rsa-oaep-3072-sha256",
                      "rsa-oaep-3072-sha256-aes-256",
                      "rsa-oaep-4096-sha1-aes-256",
                      "rsa-oaep-4096-sha256",
                      "rsa-oaep-4096-sha256-aes-256"
                    ],
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--protection-level": [
                      "hsm",
                      "software"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--attestation-file": "value",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "inventory": {
              "commands": {
                "get-protected-resources-summary": {
                  "commands": {},
                  "flags": {
                    "--keyname": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "list-keys": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "search-protected-resources": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyname": "dynamic",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--scope": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "keyrings": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--location": "value",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "keys": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--keyring": "value",
                        "--location": "value",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--crypto-key-backend": "value",
                    "--default-algorithm": [
                      "ec-sign-p256-sha256",
                      "ec-sign-p384-sha384",
                      "ec-sign-secp256k1-sha256",
                      "external-symmetric-encryption",
                      "google-symmetric-encryption",
                      "hmac-sha1",
                      "hmac-sha224",
                      "hmac-sha256",
                      "hmac-sha384",
                      "hmac-sha512",
                      "rsa-decrypt-oaep-2048-sha1",
                      "rsa-decrypt-oaep-2048-sha256",
                      "rsa-decrypt-oaep-3072-sha1",
                      "rsa-decrypt-oaep-3072-sha256",
                      "rsa-decrypt-oaep-4096-sha1",
                      "rsa-decrypt-oaep-4096-sha256",
                      "rsa-decrypt-oaep-4096-sha512",
                      "rsa-sign-pkcs1-2048-sha256",
                      "rsa-sign-pkcs1-3072-sha256",
                      "rsa-sign-pkcs1-4096-sha256",
                      "rsa-sign-pkcs1-4096-sha512",
                      "rsa-sign-pss-2048-sha256",
                      "rsa-sign-pss-3072-sha256",
                      "rsa-sign-pss-4096-sha256",
                      "rsa-sign-pss-4096-sha512",
                      "rsa-sign-raw-pkcs1-2048",
                      "rsa-sign-raw-pkcs1-3072",
                      "rsa-sign-raw-pkcs1-4096"
                    ],
                    "--destroy-scheduled-duration": "value",
                    "--import-only": "bool",
                    "--keyring": "dynamic",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--protection-level": [
                      "external",
                      "external-vpc",
                      "hsm",
                      "software"
                    ],
                    "--purpose": [
                      "asymmetric-encryption",
                      "asymmetric-signing",
                      "encryption",
                      "mac"
                    ],
                    "--rotation-period": "value",
                    "--skip-initial-version-creation": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-rotation-schedule": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value"
                  }
                },
                "set-primary-version": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "set-rotation-schedule": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--rotation-period": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--default-algorithm": [
                      "ec-sign-p256-sha256",
                      "ec-sign-p384-sha384",
                      "ec-sign-secp256k1-sha256",
                      "external-symmetric-encryption",
                      "google-symmetric-encryption",
                      "hmac-sha1",
                      "hmac-sha224",
                      "hmac-sha256",
                      "hmac-sha384",
                      "hmac-sha512",
                      "rsa-decrypt-oaep-2048-sha1",
                      "rsa-decrypt-oaep-2048-sha256",
                      "rsa-decrypt-oaep-3072-sha1",
                      "rsa-decrypt-oaep-3072-sha256",
                      "rsa-decrypt-oaep-4096-sha1",
                      "rsa-decrypt-oaep-4096-sha256",
                      "rsa-decrypt-oaep-4096-sha512",
                      "rsa-sign-pkcs1-2048-sha256",
                      "rsa-sign-pkcs1-3072-sha256",
                      "rsa-sign-pkcs1-4096-sha256",
                      "rsa-sign-pkcs1-4096-sha512",
                      "rsa-sign-pss-2048-sha256",
                      "rsa-sign-pss-3072-sha256",
                      "rsa-sign-pss-4096-sha256",
                      "rsa-sign-pss-4096-sha512",
                      "rsa-sign-raw-pkcs1-2048",
                      "rsa-sign-raw-pkcs1-3072",
                      "rsa-sign-raw-pkcs1-4096"
                    ],
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--primary-version": "dynamic",
                    "--remove-labels": "value",
                    "--remove-rotation-schedule": "bool",
                    "--rotation-period": "value",
                    "--update-labels": "value"
                  }
                },
                "versions": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--ekm-connection-key-path": "value",
                        "--external-key-uri": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--primary": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--attestation-file": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "destroy": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "get-certificate-chain": {
                      "commands": {},
                      "flags": {
                        "--certificate-chain-type": [
                          "all",
                          "cavium",
                          "google-card",
                          "google-partition"
                        ],
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--output-file": "value"
                      }
                    },
                    "get-public-key": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--output-file": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--algorithm": [
                          "ec-sign-p256-sha256",
                          "ec-sign-p384-sha384",
                          "ec-sign-secp256k1-sha256",
                          "google-symmetric-encryption",
                          "hmac-sha1",
                          "hmac-sha224",
                          "hmac-sha256",
                          "hmac-sha384",
                          "hmac-sha512",
                          "rsa-decrypt-oaep-2048-sha1",
                          "rsa-decrypt-oaep-2048-sha256",
                          "rsa-decrypt-oaep-3072-sha1",
                          "rsa-decrypt-oaep-3072-sha256",
                          "rsa-decrypt-oaep-4096-sha1",
                          "rsa-decrypt-oaep-4096-sha256",
                          "rsa-decrypt-oaep-4096-sha512",
                          "rsa-sign-pkcs1-2048-sha256",
                          "rsa-sign-pkcs1-3072-sha256",
                          "rsa-sign-pkcs1-4096-sha256",
                          "rsa-sign-pkcs1-4096-sha512",
                          "rsa-sign-pss-2048-sha256",
                          "rsa-sign-pss-3072-sha256",
                          "rsa-sign-pss-4096-sha256",
                          "rsa-sign-pss-4096-sha512",
                          "rsa-sign-raw-pkcs1-2048",
                          "rsa-sign-raw-pkcs1-3072",
                          "rsa-sign-raw-pkcs1-4096"
                        ],
                        "--import-job": "dynamic",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--public-key-file": "value",
                        "--target-key-file": "value",
                        "--wrapped-key-file": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "restore": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--ekm-connection-key-path": "value",
                        "--external-key-uri": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--state": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "mac-sign": {
              "commands": {},
              "flags": {
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "mac-verify": {
              "commands": {},
              "flags": {
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            }
          },
          "flags": {}
        },
        "lifesciences": {
          "commands": {
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "pipelines": {
              "commands": {
                "run": {
                  "commands": {},
                  "flags": {
                    "--boot-disk-size": "value",
                    "--command-line": "value",
                    "--disk-size": "value",
                    "--docker-image": "value",
                    "--env-vars": "value",
                    "--inputs": "value",
                    "--inputs-from-file": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--logging": "value",
                    "--machine-type": "value",
                    "--network": "value",
                    "--outputs": "value",
                    "--pipeline-file": "value",
                    "--preemptible": "bool",
                    "--regions": "value",
                    "--service-account-email": "value",
                    "--service-account-scopes": "value",
                    "--subnetwork": "value",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "logging": {
          "commands": {
            "buckets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cmek-kms-key-name": "value",
                    "--description": "value",
                    "--enable-analytics": "bool",
                    "--index": "value",
                    "--location": "value",
                    "--restricted-fields": "value",
                    "--retention-days": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-index": "value",
                    "--async": "bool",
                    "--clear-indexes": "bool",
                    "--cmek-kms-key-name": "value",
                    "--description": "value",
                    "--enable-analytics": "bool",
                    "--location": "value",
                    "--locked": "bool",
                    "--remove-indexes": "value",
                    "--restricted-fields": "value",
                    "--retention-days": "value",
                    "--update-index": "value"
                  }
                }
              },
              "flags": {}
            },
            "copy": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--log-filter": "value",
                "--organization": "dynamic"
              }
            },
            "links": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "logs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--sort-by": "value",
                    "--view": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--config-from-file": "value",
                    "--description": "value",
                    "--log-filter": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--config-from-file": "value",
                    "--description": "value",
                    "--log-filter": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--operation-filter": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "read": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--freshness": "value",
                "--limit": "value",
                "--location": "value",
                "--order": [
                  "asc",
                  "desc"
                ],
                "--organization": "dynamic",
                "--resource-names": "value",
                "--view": "value"
              }
            },
            "resource-descriptors": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-kms-key": "bool",
                    "--disable-default-sink": "bool",
                    "--folder": "value",
                    "--kms-key-name": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--organization": "dynamic",
                    "--storage-location": "value"
                  }
                }
              },
              "flags": {}
            },
            "sinks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--custom-writer-identity": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--exclusion": "value",
                    "--folder": "value",
                    "--include-children": "bool",
                    "--log-filter": "value",
                    "--organization": "dynamic",
                    "--use-partitioned-tables": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-exclusion": "value",
                    "--billing-account": "value",
                    "--clear-exclusions": "bool",
                    "--custom-writer-identity": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--folder": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic",
                    "--remove-exclusions": "value",
                    "--update-exclusion": "value",
                    "--use-partitioned-tables": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tail": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--buffer-window": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic",
                "--view": "value"
              }
            },
            "views": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "write": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic",
                "--payload-type": [
                  "json",
                  "text"
                ],
                "--severity": [
                  "ALERT",
                  "CRITICAL",
                  "DEBUG",
                  "DEFAULT",
                  "EMERGENCY",
                  "ERROR",
                  "INFO",
                  "NOTICE",
                  "WARNING"
                ]
              }
            }
          },
          "flags": {}
        },
        "media": {
          "commands": {},
          "flags": {}
        },
        "memcache": {
          "commands": {
            "instances": {
              "commands": {
                "apply-parameters": {
                  "commands": {},
                  "flags": {
                    "--apply-all": "bool",
                    "--async": "bool",
                    "--node-ids": "value",
                    "--region": "dynamic"
                  }
                },
                "apply-software-update": {
                  "commands": {},
                  "flags": {
                    "--apply-all": "bool",
                    "--async": "bool",
                    "--node-ids": "value",
                    "--region": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "dynamic",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorized-network": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--maintenance-window-day": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-duration": "value",
                    "--maintenance-window-start-time": "value",
                    "--memcached-version": [
                      "1.5"
                    ],
                    "--node-count": "value",
                    "--node-cpu": "value",
                    "--node-memory": "value",
                    "--parameters": "value",
                    "--region": "dynamic",
                    "--zones": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reschedule-maintenance": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic",
                    "--reschedule-type": [
                      "immediate",
                      "next-available-window",
                      "specific-time"
                    ],
                    "--schedule-time": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-duration": "value",
                    "--maintenance-window-start-time": "value",
                    "--node-count": "value",
                    "--parameters": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "metastore": {
          "commands": {
            "federations": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backends": "value",
                    "--hive-metastore-version": [
                      "2.3.6",
                      "3.1.2"
                    ],
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-backends": "bool",
                    "--clear-labels": "bool",
                    "--location": "value",
                    "--remove-backends": "value",
                    "--remove-labels": "value",
                    "--update-backends": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "alter-metadata-resource-location": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--location_uri": "value",
                    "--resource_name": "value"
                  }
                },
                "backups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auxiliary-versions": "value",
                    "--auxiliary-versions-from-file": "value",
                    "--consumer-subnetworks": "value",
                    "--data-catalog-sync": "bool",
                    "--database-type": [
                      "mysql",
                      "spanner"
                    ],
                    "--encryption-kms-key": "value",
                    "--endpoint-protocol": [
                      "grpc",
                      "thrift"
                    ],
                    "--hive-metastore-configs": "value",
                    "--hive-metastore-configs-from-file": "value",
                    "--hive-metastore-version": "value",
                    "--instance-size": [
                      "extra_large",
                      "extra_small",
                      "large",
                      "medium",
                      "small"
                    ],
                    "--kerberos-principal": "value",
                    "--keytab": "value",
                    "--krb5-config": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--maintenance-window-day": [
                      "fri",
                      "mon",
                      "sat",
                      "sun",
                      "thu",
                      "tue",
                      "wed"
                    ],
                    "--maintenance-window-hour": "value",
                    "--network": "value",
                    "--network-config-from-file": "value",
                    "--port": "value",
                    "--release-channel": [
                      "canary",
                      "stable"
                    ],
                    "--scaling-factor": "value",
                    "--tier": [
                      "developer",
                      "enterprise"
                    ]
                  }
                },
                "databases": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "tables": {
                      "commands": {
                        "add-iam-policy-binding": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--member": "value",
                            "--role": "dynamic",
                            "--service": "value"
                          }
                        },
                        "get-iam-policy": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--service": "value",
                            "--sort-by": "value"
                          }
                        },
                        "remove-iam-policy-binding": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--member": "value",
                            "--role": "dynamic",
                            "--service": "value"
                          }
                        },
                        "set-iam-policy": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--service": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--destination-folder": "value",
                        "--dump-type": [
                          "avro",
                          "mysql"
                        ],
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--database-dump": "value",
                        "--description": "value",
                        "--dump-type": [
                          "avro",
                          "mysql"
                        ],
                        "--import-id": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "imports": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--database-dump": "value",
                        "--database-dump-type": [
                          "mysql"
                        ],
                        "--database-type": [
                          "mysql"
                        ],
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move-table-to-database": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination_db_name": "value",
                    "--location": "value",
                    "--source_db_name": "value",
                    "--table_name": "value"
                  }
                },
                "query-metadata": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--query": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value",
                    "--location": "value",
                    "--restore-type": [
                      "full",
                      "metadata-only"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-auxiliary-versions": "value",
                    "--async": "bool",
                    "--clear-auxiliary-versions": "bool",
                    "--clear-hive-metastore-configs": "bool",
                    "--clear-labels": "bool",
                    "--data-catalog-sync": "bool",
                    "--endpoint-protocol": [
                      "grpc",
                      "thrift"
                    ],
                    "--instance-size": [
                      "extra_large",
                      "extra_small",
                      "large",
                      "medium",
                      "small"
                    ],
                    "--kerberos-principal": "value",
                    "--keytab": "value",
                    "--krb5-config": "value",
                    "--location": "value",
                    "--maintenance-window-day": [
                      "fri",
                      "mon",
                      "sat",
                      "sun",
                      "thu",
                      "tue",
                      "wed"
                    ],
                    "--maintenance-window-hour": "value",
                    "--port": "value",
                    "--remove-hive-metastore-configs": "value",
                    "--remove-labels": "value",
                    "--scaling-factor": "value",
                    "--tier": [
                      "developer",
                      "enterprise"
                    ],
                    "--update-auxiliary-versions-from-file": "value",
                    "--update-hive-metastore-configs": "value",
                    "--update-hive-metastore-configs-from-file": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ml": {
          "commands": {
            "language": {
              "commands": {
                "analyze-entities": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-entity-sentiment": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-sentiment": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-syntax": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "classify-text": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--language": "value"
                  }
                }
              },
              "flags": {}
            },
            "speech": {
              "commands": {
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "wait": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "recognize": {
                  "commands": {},
                  "flags": {
                    "--additional-language-codes": "value",
                    "--audio-channel-count": "value",
                    "--audio-topic": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--encoding": [
                      "amr",
                      "amr-wb",
                      "encoding-unspecified",
                      "flac",
                      "linear16",
                      "mp3",
                      "mulaw",
                      "ogg-opus",
                      "speex-with-header-byte",
                      "webm-opus"
                    ],
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--include-word-confidence": "bool",
                    "--include-word-time-offsets": "bool",
                    "--interaction-type": [
                      "dictation",
                      "discussion",
                      "phone-call",
                      "presentation",
                      "professionally-produced",
                      "voice-command",
                      "voice-search",
                      "voicemail"
                    ],
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--max-diarization-speaker-count": "value",
                    "--microphone-distance": [
                      "farfield",
                      "midfield",
                      "nearfield"
                    ],
                    "--min-diarization-speaker-count": "value",
                    "--model": [
                      "command_and_search",
                      "default",
                      "latest_long",
                      "latest_short",
                      "medical_conversation",
                      "medical_dictation",
                      "phone_call",
                      "phone_call_enhanced",
                      "video_enhanced"
                    ],
                    "--naics-code": "value",
                    "--original-media-type": [
                      "audio",
                      "video"
                    ],
                    "--original-mime-type": "value",
                    "--recording-device-name": "value",
                    "--recording-device-type": [
                      "indoor",
                      "outdoor",
                      "pc",
                      "phone-line",
                      "smartphone",
                      "vehicle"
                    ],
                    "--sample-rate": "value",
                    "--separate-channel-recognition": "bool"
                  }
                },
                "recognize-long-running": {
                  "commands": {},
                  "flags": {
                    "--additional-language-codes": "value",
                    "--async": "bool",
                    "--audio-channel-count": "value",
                    "--audio-topic": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--encoding": [
                      "amr",
                      "amr-wb",
                      "encoding-unspecified",
                      "flac",
                      "linear16",
                      "mp3",
                      "mulaw",
                      "ogg-opus",
                      "speex-with-header-byte",
                      "webm-opus"
                    ],
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--include-word-confidence": "bool",
                    "--include-word-time-offsets": "bool",
                    "--interaction-type": [
                      "dictation",
                      "discussion",
                      "phone-call",
                      "presentation",
                      "professionally-produced",
                      "voice-command",
                      "voice-search",
                      "voicemail"
                    ],
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--max-diarization-speaker-count": "value",
                    "--microphone-distance": [
                      "farfield",
                      "midfield",
                      "nearfield"
                    ],
                    "--min-diarization-speaker-count": "value",
                    "--model": [
                      "command_and_search",
                      "default",
                      "latest_long",
                      "latest_short",
                      "medical_conversation",
                      "medical_dictation",
                      "phone_call",
                      "phone_call_enhanced",
                      "video_enhanced"
                    ],
                    "--naics-code": "value",
                    "--original-media-type": [
                      "audio",
                      "video"
                    ],
                    "--original-mime-type": "value",
                    "--output-uri": "value",
                    "--recording-device-name": "value",
                    "--recording-device-type": [
                      "indoor",
                      "outdoor",
                      "pc",
                      "phone-line",
                      "smartphone",
                      "vehicle"
                    ],
                    "--sample-rate": "value",
                    "--separate-channel-recognition": "bool"
                  }
                },
                "recognizers": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--display-name": "value",
                        "--enable-automatic-punctuation": "value",
                        "--enable-spoken-emojis": "value",
                        "--enable-spoken-punctuation": "value",
                        "--enable-word-confidence": "value",
                        "--enable-word-time-offsets": "value",
                        "--language-codes": "value",
                        "--location": "value",
                        "--max-speaker-count": "value",
                        "--min-speaker-count": "value",
                        "--model": "value",
                        "--profanity-filter": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--display-name": "value",
                        "--enable-automatic-punctuation": "value",
                        "--enable-spoken-emojis": "value",
                        "--enable-spoken-punctuation": "value",
                        "--enable-word-confidence": "value",
                        "--enable-word-time-offsets": "value",
                        "--language-codes": "value",
                        "--location": "value",
                        "--max-speaker-count": "value",
                        "--min-speaker-count": "value",
                        "--model": "value",
                        "--profanity-filter": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "translate": {
              "commands": {
                "batch-translate-text": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--glossaries": "value",
                    "--models": "value",
                    "--source": "value",
                    "--source-language": "value",
                    "--target-language-codes": "value",
                    "--zone": "value"
                  }
                },
                "detect-language": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--mime-type": "value",
                    "--model": "value",
                    "--zone": "value"
                  }
                },
                "get-supported-languages": {
                  "commands": {},
                  "flags": {
                    "--display-language-code": "value",
                    "--model": "value",
                    "--zone": "value"
                  }
                },
                "translate-text": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--glossary-config": "value",
                    "--mime-type": "value",
                    "--model": "value",
                    "--source-language": "value",
                    "--target-language": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "video": {
              "commands": {
                "detect-explicit-content": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-labels": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--detection-mode": [
                      "frame",
                      "shot",
                      "shot-and-frame"
                    ],
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-object": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-shot-changes": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-text": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--language-hints": "value",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "transcribe-speech": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--audio-tracks": "value",
                    "--diarization-speaker-count": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--enable-word-confidence": "bool",
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                }
              },
              "flags": {}
            },
            "vision": {
              "commands": {
                "detect-document": {
                  "commands": {},
                  "flags": {
                    "--language-hints": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-faces": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-image-properties": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-labels": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-landmarks": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-logos": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-objects": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-product": {
                  "commands": {},
                  "flags": {
                    "--bounding-polygon": "value",
                    "--category": "value",
                    "--filter": "value",
                    "--max-results": "value",
                    "--product-set": "value",
                    "--product-set-location": "value"
                  }
                },
                "detect-safe-search": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text": {
                  "commands": {},
                  "flags": {
                    "--language-hints": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text-pdf": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text-tiff": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-web": {
                  "commands": {},
                  "flags": {
                    "--include-geo-results": "bool",
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "product-search": {
                  "commands": {
                    "images": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--bounding-polygon": "value",
                            "--location": "value",
                            "--product": "value",
                            "--storage-location": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--product": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "product-sets": {
                      "commands": {
                        "add-product": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "create": {
                          "commands": {},
                          "flags": {
                            "--display-name": "value",
                            "--location": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--location": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "list-products": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--product-set": "value",
                            "--sort-by": "value"
                          }
                        },
                        "remove-product": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--display-name": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "products": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--category": [
                              "apparel",
                              "homegoods",
                              "toys"
                            ],
                            "--description": "value",
                            "--display-name": "value",
                            "--location": "value",
                            "--product-labels": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "delete-all": {
                          "commands": {},
                          "flags": {
                            "--force": "bool",
                            "--orphan-products": "bool",
                            "--product-set": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--add-product-labels": "value",
                            "--clear-product-labels": "bool",
                            "--description": "value",
                            "--display-name": "value",
                            "--location": "value",
                            "--remove-product-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "suggest-crop": {
                  "commands": {},
                  "flags": {
                    "--aspect-ratios": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ml-engine": {
          "commands": {
            "explain": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--text-instances": "value"
              }
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--summarize": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--task-name": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "prediction": {
                      "commands": {},
                      "flags": {
                        "--accelerator-count": "value",
                        "--accelerator-type": [
                          "nvidia-tesla-k80",
                          "nvidia-tesla-p100"
                        ],
                        "--batch-size": "value",
                        "--data-format": [
                          "text",
                          "tf-record",
                          "tf-record-gzip"
                        ],
                        "--input-paths": "value",
                        "--labels": "value",
                        "--max-worker-count": "value",
                        "--model": "value",
                        "--model-dir": "value",
                        "--output-path": "value",
                        "--region": "value",
                        "--runtime-version": "value",
                        "--signature-name": "value"
                      }
                    },
                    "training": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config": "value",
                        "--enable-web-access": "bool",
                        "--job-dir": "value",
                        "--kms-key": "dynamic",
                        "--kms-keyring": "dynamic",
                        "--kms-location": "dynamic",
                        "--kms-project": "dynamic",
                        "--labels": "value",
                        "--master-accelerator": "value",
                        "--master-image-uri": "value",
                        "--master-machine-type": "value",
                        "--module-name": "value",
                        "--network": "value",
                        "--package-path": "value",
                        "--packages": "value",
                        "--parameter-server-accelerator": "value",
                        "--parameter-server-count": "value",
                        "--parameter-server-image-uri": "value",
                        "--parameter-server-machine-type": "value",
                        "--python-version": "value",
                        "--region": "dynamic",
                        "--runtime-version": "value",
                        "--scale-tier": [
                          "basic",
                          "basic-gpu",
                          "basic-tpu",
                          "custom",
                          "premium-1",
                          "standard-1"
                        ],
                        "--service-account": "value",
                        "--staging-bucket": "value",
                        "--stream-logs": "bool",
                        "--tpu-tf-version": "value",
                        "--use-chief-in-tf-config": "value",
                        "--worker-accelerator": "value",
                        "--worker-count": "value",
                        "--worker-image-uri": "value",
                        "--worker-machine-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "local": {
              "commands": {
                "predict": {
                  "commands": {},
                  "flags": {
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--json-instances": "value",
                    "--json-request": "value",
                    "--model-dir": "value",
                    "--signature-name": "value",
                    "--text-instances": "value"
                  }
                },
                "train": {
                  "commands": {},
                  "flags": {
                    "--distributed": "bool",
                    "--evaluator-count": "value",
                    "--job-dir": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--parameter-server-count": "value",
                    "--start-port": "value",
                    "--worker-count": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-console-logging": "bool",
                    "--enable-logging": "bool",
                    "--labels": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--regions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "predict": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "versions": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--command": "value",
                    "--config": "value",
                    "--description": "value",
                    "--env-vars": "value",
                    "--explanation-method": [
                      "integrated-gradients",
                      "sampled-shapley",
                      "xrai"
                    ],
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--health-route": "value",
                    "--image": "value",
                    "--labels": "value",
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--metric-targets": "value",
                    "--min-nodes": "value",
                    "--model": "value",
                    "--num-integral-steps": "value",
                    "--num-paths": "value",
                    "--origin": "value",
                    "--package-uris": "value",
                    "--ports": "value",
                    "--predict-route": "value",
                    "--prediction-class": "value",
                    "--python-version": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--runtime-version": "value",
                    "--service-account": "value",
                    "--staging-bucket": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--model": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-default": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bigquery-table-name": "value",
                    "--clear-labels": "bool",
                    "--config": "value",
                    "--description": "value",
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--sampling-percentage": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "monitoring": {
          "commands": {
            "channel-descriptors": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "channels": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--channel-content": "value",
                    "--channel-content-from-file": "value",
                    "--channel-labels": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--enabled": "bool",
                    "--type": "value",
                    "--user-labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--channel-content": "value",
                    "--channel-content-from-file": "value",
                    "--clear-channel-labels": "bool",
                    "--clear-user-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--enabled": "bool",
                    "--fields": "value",
                    "--remove-channel-labels": "value",
                    "--remove-user-labels": "value",
                    "--type": "value",
                    "--update-channel-labels": "value",
                    "--update-user-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "dashboards": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--config-from-file": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--config-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics-scopes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "policies": {
              "commands": {
                "conditions": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--aggregation": "value",
                        "--condition": "value",
                        "--condition-display-name": "value",
                        "--condition-filter": "value",
                        "--condition-from-file": "value",
                        "--duration": "value",
                        "--if": "value",
                        "--trigger-count": "value",
                        "--trigger-percent": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--policy": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--display-name": "value",
                        "--if": "value",
                        "--policy": "value",
                        "--trigger-count": "value",
                        "--trigger-percent": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--aggregation": "value",
                    "--combiner": [
                      "AND",
                      "AND_WITH_MATCHING_RESOURCE",
                      "COMBINE_UNSPECIFIED",
                      "OR"
                    ],
                    "--condition-display-name": "value",
                    "--condition-filter": "value",
                    "--display-name": "value",
                    "--documentation": "value",
                    "--documentation-format": "value",
                    "--documentation-from-file": "value",
                    "--duration": "value",
                    "--enabled": "bool",
                    "--if": "value",
                    "--notification-channels": "value",
                    "--policy": "value",
                    "--policy-from-file": "value",
                    "--trigger-count": "value",
                    "--trigger-percent": "value",
                    "--user-labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-notification-channels": "value",
                    "--clear-notification-channels": "bool",
                    "--clear-user-labels": "bool",
                    "--combiner": [
                      "AND",
                      "AND_WITH_MATCHING_RESOURCE",
                      "COMBINE_UNSPECIFIED",
                      "OR"
                    ],
                    "--display-name": "value",
                    "--documentation": "value",
                    "--documentation-format": "value",
                    "--documentation-from-file": "value",
                    "--enabled": "bool",
                    "--fields": "value",
                    "--policy": "value",
                    "--policy-from-file": "value",
                    "--remove-notification-channels": "value",
                    "--remove-user-labels": "value",
                    "--set-notification-channels": "value",
                    "--update-user-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "snoozes": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--criteria-policies": "value",
                    "--display-name": "value",
                    "--end-time": "value",
                    "--snooze-from-file": "value",
                    "--start-time": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--end-time": "value",
                    "--fields": "value",
                    "--snooze-from-file": "value",
                    "--start-time": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "netapp": {
          "commands": {
            "active-directories": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-operators": "value",
                    "--description": "value",
                    "--dns": "value",
                    "--domain": "value",
                    "--enable-aes": "value",
                    "--enable-ldap-signing": "value",
                    "--encrypt-dc-connections": "value",
                    "--kdc-hostname": "value",
                    "--kdc-ip": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--net-bios-prefix": "value",
                    "--nfs-users-with-ldap": "value",
                    "--organizational-unit": "value",
                    "--password": "value",
                    "--security-operators": "value",
                    "--site": "value",
                    "--username": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-operators": "value",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--dns": "value",
                    "--domain": "value",
                    "--enable-aes": "value",
                    "--enable-ldap-signing": "value",
                    "--encrypt-dc-connections": "value",
                    "--kdc-hostname": "value",
                    "--kdc-ip": "value",
                    "--location": "value",
                    "--net-bios-prefix": "value",
                    "--nfs-users-with-ldap": "value",
                    "--organizational-unit": "value",
                    "--password": "value",
                    "--remove-labels": "value",
                    "--security-operators": "value",
                    "--site": "value",
                    "--update-labels": "value",
                    "--username": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "storage-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--capacity": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--service-level": [
                      "extreme",
                      "premium"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--capacity": "value",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "volumes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--capacity": "value",
                    "--description": "value",
                    "--enable-kerberos": "value",
                    "--enable-ldap": "value",
                    "--export-policy": "value",
                    "--from-snapshot": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--network": "value",
                    "--protocols": "value",
                    "--security-style": [
                      "ntfs",
                      "unix"
                    ],
                    "--share-name": "value",
                    "--smb-settings": "value",
                    "--snap-reserve": "value",
                    "--snapshot-daily": "value",
                    "--snapshot-directory": "value",
                    "--snapshot-hourly": "value",
                    "--snapshot-monthly": "value",
                    "--snapshot-weekly": "value",
                    "--storage-pool": "value",
                    "--unix-permissions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "revert": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--snapshot": "value"
                  }
                },
                "snapshots": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--snapshot-volume": "value",
                        "--volume": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--snapshot-volume": "value",
                        "--volume": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--snapshot-volume": "value",
                        "--volume": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--volume": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--snapshot-volume": "value",
                        "--update-labels": "value",
                        "--volume": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--capacity": "value",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--enable-kerberos": "value",
                    "--enable-ldap": "value",
                    "--export-policy": "value",
                    "--from-snapshot": "value",
                    "--location": "value",
                    "--network": "value",
                    "--protocols": "value",
                    "--remove-labels": "value",
                    "--security-style": [
                      "ntfs",
                      "unix"
                    ],
                    "--share-name": "value",
                    "--smb-settings": "value",
                    "--snap-reserve": "value",
                    "--snapshot-daily": "value",
                    "--snapshot-directory": "value",
                    "--snapshot-hourly": "value",
                    "--snapshot-monthly": "value",
                    "--snapshot-weekly": "value",
                    "--storage-pool": "value",
                    "--unix-permissions": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-connectivity": {
          "commands": {
            "hubs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "internal-ranges": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--ip-cidr-range": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--overlaps": "value",
                    "--peering": [
                      "for-peer",
                      "for-self",
                      "not-shared"
                    ],
                    "--prefix-length": "value",
                    "--region": "value",
                    "--target-cidr-range": "value",
                    "--usage": [
                      "external-to-vpc",
                      "for-vpc"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--ip-cidr-range": "value",
                    "--prefix-length": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "policy-based-routes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--destination-range": "value",
                    "--interconnect-attachment-region": "value",
                    "--ip-protocol": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--next-hop-ilb-ip": "value",
                    "--priority": "value",
                    "--protocol-version": [
                      "ipv4",
                      "protocol-version-unspecified"
                    ],
                    "--source-range": "value",
                    "--tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "spokes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--hub": "value",
                    "--interconnect-attachment": "value",
                    "--labels": "value",
                    "--region": "value",
                    "--router-appliance": "value",
                    "--vpn-tunnel": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--region": "value",
                    "--router-appliance": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-management": {
          "commands": {},
          "flags": {}
        },
        "network-security": {
          "commands": {
            "authorization-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "client-tls-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "gateway-security-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "rules": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--gateway-security-policy": "value",
                        "--location": "value"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--gateway-security-policy": "value",
                        "--location": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--gateway-security-policy": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--gateway-security-policy": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "server-tls-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "url-lists": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-services": {
          "commands": {
            "endpoint-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "gateways": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "grpc-routes": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "http-filters": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "http-routes": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "meshes": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "observability-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "service-bindings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--service-directory-namespace": "value",
                    "--service-directory-region": "value",
                    "--service-directory-service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tcp-routes": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tls-routes": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "notebooks": {
          "commands": {
            "environments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--container-repository": "value",
                    "--container-tag": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "dynamic",
                    "--post-startup-script": "value",
                    "--vm-image-family": "value",
                    "--vm-image-name": "value",
                    "--vm-image-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-core-count": "value",
                    "--accelerator-type": [
                      "NVIDIA_TESLA_A100",
                      "NVIDIA_TESLA_K80",
                      "NVIDIA_TESLA_P100",
                      "NVIDIA_TESLA_P100_VWS",
                      "NVIDIA_TESLA_P4",
                      "NVIDIA_TESLA_P4_VWS",
                      "NVIDIA_TESLA_T4",
                      "NVIDIA_TESLA_T4_VWS",
                      "NVIDIA_TESLA_V100",
                      "TPU_V2",
                      "TPU_V3"
                    ],
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": [
                      "PD_BALANCED",
                      "PD_SSD",
                      "PD_STANDARD"
                    ],
                    "--container-repository": "value",
                    "--container-tag": "value",
                    "--custom-gpu-driver-path": "value",
                    "--disk-encryption": [
                      "CMEK",
                      "GMEK"
                    ],
                    "--environment": "dynamic",
                    "--environment-location": "dynamic",
                    "--install-gpu-driver": "bool",
                    "--instance-owners": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--machine-type": "value",
                    "--metadata": "value",
                    "--network": "dynamic",
                    "--no-proxy-access": "bool",
                    "--no-public-ip": "bool",
                    "--post-startup-script": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "ANY_RESERVATION",
                      "NO_RESERVATION",
                      "SPECIFIC_RESERVATION",
                      "TYPE_UNSPECIFIED"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--subnet": "dynamic",
                    "--subnet-region": "dynamic",
                    "--vm-image-family": "value",
                    "--vm-image-name": "value",
                    "--vm-image-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "diagnose": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--enable-copy-home-files": "bool",
                    "--enable-packet-capture": "bool",
                    "--enable-repair": "bool",
                    "--gcs-bucket": "value",
                    "--location": "dynamic",
                    "--relative-path": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "is-upgradeable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "reset": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--accelerator-core-count": "value",
                    "--accelerator-type": [
                      "NVIDIA_TESLA_A100",
                      "NVIDIA_TESLA_K80",
                      "NVIDIA_TESLA_P100",
                      "NVIDIA_TESLA_P100_VWS",
                      "NVIDIA_TESLA_P4",
                      "NVIDIA_TESLA_P4_VWS",
                      "NVIDIA_TESLA_T4",
                      "NVIDIA_TESLA_T4_VWS",
                      "NVIDIA_TESLA_V100",
                      "TPU_V2",
                      "TPU_V3"
                    ],
                    "--async": "bool",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--machine-type": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "organizations": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "policy-troubleshoot": {
          "commands": {
            "iam": {
              "commands": {},
              "flags": {
                "--destination-ip": "value",
                "--destination-port": "value",
                "--permission": "value",
                "--principal-email": "value",
                "--request-time": "value",
                "--resource-name": "value",
                "--resource-service": "value",
                "--resource-type": "value"
              }
            }
          },
          "flags": {}
        },
        "projects": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--enable-cloud-apis": "bool",
                "--folder": "value",
                "--labels": "value",
                "--name": "value",
                "--organization": "value",
                "--set-as-default": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-ancestors": {
              "commands": {},
              "flags": {}
            },
            "get-ancestors-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--include-deny": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "search": {
              "commands": {},
              "flags": {
                "--limit": "value",
                "--page-size": "value",
                "--query": "value",
                "--sort-by": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--name": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "publicca": {
          "commands": {
            "external-account-keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--key-output-file": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "pubsub": {
          "commands": {
            "lite-operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--done": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--subscription": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-reservations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--throughput-capacity": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list-topics": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--throughput-capacity": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-subscriptions": {
              "commands": {
                "ack-up-to": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--offset": "value",
                    "--partition": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--delivery-requirement": [
                      "deliver-after-stored",
                      "deliver-immediately"
                    ],
                    "--event-time": "value",
                    "--export-dead-letter-topic": "value",
                    "--export-desired-state": [
                      "active",
                      "paused"
                    ],
                    "--export-pubsub-topic": "value",
                    "--location": "value",
                    "--publish-time": "value",
                    "--starting-offset": [
                      "beginning",
                      "end"
                    ],
                    "--topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "seek": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--event-time": "value",
                    "--location": "value",
                    "--publish-time": "value",
                    "--starting-offset": [
                      "beginning",
                      "end"
                    ]
                  }
                },
                "subscribe": {
                  "commands": {},
                  "flags": {
                    "--auto-ack": "bool",
                    "--location": "value",
                    "--num-messages": "value",
                    "--partitions": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--delivery-requirement": [
                      "deliver-after-stored",
                      "deliver-immediately"
                    ],
                    "--export-dead-letter-topic": "value",
                    "--export-desired-state": [
                      "active",
                      "paused"
                    ],
                    "--export-pubsub-topic": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-topics": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--message-retention-period": "value",
                    "--partitions": "value",
                    "--per-partition-bytes": "value",
                    "--per-partition-publish-mib": "value",
                    "--per-partition-subscribe-mib": "value",
                    "--throughput-reservation": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list-subscriptions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "publish": {
                  "commands": {},
                  "flags": {
                    "--attributes": "value",
                    "--event-time": "value",
                    "--location": "value",
                    "--message": "value",
                    "--ordering-key": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--message-retention-period": "value",
                    "--partitions": "value",
                    "--per-partition-bytes": "value",
                    "--per-partition-publish-mib": "value",
                    "--per-partition-subscribe-mib": "value",
                    "--throughput-reservation": "value"
                  }
                }
              },
              "flags": {}
            },
            "schemas": {
              "commands": {
                "commit": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "delete-revision": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "list-revisions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--revision-id": "value"
                  }
                },
                "validate-message": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--message": "value",
                    "--message-encoding": [
                      "binary",
                      "encoding-unspecified",
                      "json"
                    ],
                    "--schema-name": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                },
                "validate-schema": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--subscription": "value",
                    "--subscription-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "subscriptions": {
              "commands": {
                "ack": {
                  "commands": {},
                  "flags": {
                    "--ack-ids": "value"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--bigquery-table": "value",
                    "--dead-letter-topic": "value",
                    "--dead-letter-topic-project": "value",
                    "--drop-unknown-fields": "bool",
                    "--enable-exactly-once-delivery": "bool",
                    "--enable-message-ordering": "bool",
                    "--expiration-period": "value",
                    "--labels": "value",
                    "--max-delivery-attempts": "value",
                    "--max-retry-delay": "value",
                    "--message-filter": "value",
                    "--message-retention-duration": "value",
                    "--min-retry-delay": "value",
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value",
                    "--retain-acked-messages": "bool",
                    "--topic": "value",
                    "--topic-project": "value",
                    "--use-topic-schema": "bool",
                    "--write-metadata": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "modify-ack-deadline": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--ack-ids": "value"
                  }
                },
                "modify-message-ack-deadline": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--ack-ids": "value"
                  }
                },
                "modify-push-config": {
                  "commands": {},
                  "flags": {
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value"
                  }
                },
                "pull": {
                  "commands": {},
                  "flags": {
                    "--auto-ack": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--max-messages": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--wait": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "seek": {
                  "commands": {},
                  "flags": {
                    "--snapshot": "value",
                    "--snapshot-project": "value",
                    "--time": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--bigquery-table": "value",
                    "--clear-bigquery-config": "bool",
                    "--clear-dead-letter-policy": "bool",
                    "--clear-labels": "bool",
                    "--clear-retry-policy": "bool",
                    "--dead-letter-topic": "value",
                    "--dead-letter-topic-project": "value",
                    "--drop-unknown-fields": "bool",
                    "--enable-exactly-once-delivery": "bool",
                    "--expiration-period": "value",
                    "--max-delivery-attempts": "value",
                    "--max-retry-delay": "value",
                    "--message-retention-duration": "value",
                    "--min-retry-delay": "value",
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value",
                    "--remove-labels": "value",
                    "--retain-acked-messages": "bool",
                    "--update-labels": "value",
                    "--use-topic-schema": "bool",
                    "--write-metadata": "bool"
                  }
                }
              },
              "flags": {}
            },
            "topics": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--first-revision-id": "value",
                    "--labels": "value",
                    "--last-revision-id": "value",
                    "--message-encoding": "value",
                    "--message-retention-duration": "value",
                    "--message-storage-policy-allowed-regions": "value",
                    "--schema": "value",
                    "--schema-project": "value",
                    "--topic-encryption-key": "dynamic",
                    "--topic-encryption-key-keyring": "dynamic",
                    "--topic-encryption-key-location": "dynamic",
                    "--topic-encryption-key-project": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "detach-subscription": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-snapshots": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-subscriptions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "publish": {
                  "commands": {},
                  "flags": {
                    "--attribute": "value",
                    "--message": "value",
                    "--ordering-key": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-message-retention-duration": "bool",
                    "--clear-schema-settings": "bool",
                    "--first-revision-id": "value",
                    "--last-revision-id": "value",
                    "--message-encoding": "value",
                    "--message-retention-duration": "value",
                    "--message-storage-policy-allowed-regions": "value",
                    "--recompute-message-storage-policy": "bool",
                    "--remove-labels": "value",
                    "--schema": "value",
                    "--schema-project": "value",
                    "--topic-encryption-key": "dynamic",
                    "--topic-encryption-key-keyring": "dynamic",
                    "--topic-encryption-key-location": "dynamic",
                    "--topic-encryption-key-project": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "recaptcha": {
          "commands": {
            "keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--allow-all-bundle-ids": "bool",
                    "--allow-all-domains": "bool",
                    "--allow-all-package-names": "bool",
                    "--allow-amp-traffic": "bool",
                    "--android": "bool",
                    "--bundle-ids": "value",
                    "--display-name": "value",
                    "--domains": "value",
                    "--integration-type": [
                      "checkbox",
                      "invisible",
                      "score"
                    ],
                    "--ios": "bool",
                    "--labels": "value",
                    "--package-names": "value",
                    "--security-preference": [
                      "balance",
                      "challenge-security-preference-unspecified",
                      "security",
                      "usability"
                    ],
                    "--support-non-google-app-store-distribution": "bool",
                    "--testing-challenge": [
                      "challenge",
                      "nocaptcha"
                    ],
                    "--testing-score": "value",
                    "--waf-feature": [
                      "action-token",
                      "challenge-page",
                      "express",
                      "session-token"
                    ],
                    "--waf-service": [
                      "ca",
                      "fastly",
                      "unspecified"
                    ],
                    "--web": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "migrate": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allow-all-bundle-ids": "bool",
                    "--allow-all-domains": "bool",
                    "--allow-all-package-names": "bool",
                    "--allow-amp-traffic": "bool",
                    "--android": "bool",
                    "--bundle-ids": "value",
                    "--display-name": "value",
                    "--domains": "value",
                    "--ios": "bool",
                    "--labels": "value",
                    "--package-names": "value",
                    "--security-preference": [
                      "balance",
                      "challenge-security-preference-unspecified",
                      "security",
                      "usability"
                    ],
                    "--web": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "recommender": {
          "commands": {
            "insights": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mark-accepted": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--state-metadata": "value"
                  }
                },
                "mark-active": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "mark-dismissed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommendation-change-type": [
                      "dismiss",
                      "leave_unchanged"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "recommendations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--recommender": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mark-active": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value"
                  }
                },
                "mark-claimed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                },
                "mark-dismissed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value"
                  }
                },
                "mark-failed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                },
                "mark-succeeded": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                }
              },
              "flags": {}
            },
            "recommender-configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--recommender": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "redis": {
          "commands": {
            "instances": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--region": "dynamic",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--alternative-zone": "value",
                    "--async": "bool",
                    "--connect-mode": [
                      "connect-mode-unspecified",
                      "direct-peering",
                      "private-service-access"
                    ],
                    "--customer-managed-key": "value",
                    "--display-name": "value",
                    "--enable-auth": "bool",
                    "--labels": "value",
                    "--maintenance-window-day": [
                      "day-of-week-unspecified",
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-hour": "value",
                    "--network": "value",
                    "--persistence-mode": [
                      "disabled",
                      "rdb"
                    ],
                    "--rdb-snapshot-period": [
                      "12h",
                      "1h",
                      "24h",
                      "6h"
                    ],
                    "--rdb-snapshot-start-time": "value",
                    "--read-replicas-mode": [
                      "read-replicas-disabled",
                      "read-replicas-enabled"
                    ],
                    "--redis-config": "value",
                    "--redis-version": [
                      "redis_3_2",
                      "redis_4_0",
                      "redis_5_0",
                      "redis_6_x"
                    ],
                    "--region": "dynamic",
                    "--replica-count": "value",
                    "--reserved-ip-range": "value",
                    "--size": "value",
                    "--tier": [
                      "basic",
                      "standard"
                    ],
                    "--transit-encryption-mode": [
                      "disabled",
                      "server-authentication"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--data-protection-mode": [
                      "force-data-loss",
                      "limited-data-loss"
                    ],
                    "--region": "dynamic"
                  }
                },
                "get-auth-string": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reschedule-maintenance": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic",
                    "--reschedule-type": [
                      "immediate",
                      "next-available-window",
                      "specific-time"
                    ],
                    "--schedule-time": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--enable-auth": "bool",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "day-of-week-unspecified",
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-hour": "value",
                    "--persistence-mode": [
                      "disabled",
                      "rdb"
                    ],
                    "--rdb-snapshot-period": [
                      "12h",
                      "1h",
                      "24h",
                      "6h"
                    ],
                    "--rdb-snapshot-start-time": "value",
                    "--read-replicas-mode": [
                      "read-replicas-disabled",
                      "read-replicas-enabled"
                    ],
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--remove-redis-config": "value",
                    "--replica-count": "value",
                    "--secondary-ip-range": "value",
                    "--size": "value",
                    "--update-labels": "value",
                    "--update-redis-config": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--redis-version": [
                      "redis_4_0",
                      "redis_5_0",
                      "redis_6_x"
                    ],
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "resource-config": {
          "commands": {
            "apply": {
              "commands": {},
              "flags": {}
            },
            "bulk-export": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--on-error": [
                  "continue",
                  "halt",
                  "ignore"
                ],
                "--organization": "value",
                "--path": "value",
                "--resource-format": [
                  "krm",
                  "terraform"
                ],
                "--resource-types": "value",
                "--resource-types-file": "value",
                "--storage-path": "value"
              }
            },
            "list-resource-types": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "terraform": {
              "commands": {
                "generate-import": {
                  "commands": {},
                  "flags": {
                    "--output-dir": "value",
                    "--output-module-file": "value",
                    "--output-script-file": "value"
                  }
                },
                "init-provider": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--tf-billing-project": "value",
                    "--tf-user-project-override": "bool",
                    "--use-gcloud-billing-project": "bool",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "resource-manager": {
          "commands": {
            "folders": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-ancestors-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--include-deny": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "search": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--page-size": "value",
                    "--query": "value",
                    "--sort-by": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "liens": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--origin": "value",
                    "--reason": "value",
                    "--restrictions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "org-policies": {
              "commands": {
                "allow": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "deny": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--effective": "bool",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "disable-enforce": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "enable-enforce": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--show-unset": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-policy": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "bindings": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--parent": "value",
                        "--tag-value": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--parent": "value",
                        "--tag-value": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--effective": "bool",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "holds": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--help-link": "value",
                        "--holder": "value",
                        "--location": "value",
                        "--origin": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--holder": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--origin": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "keys": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--parent": "value",
                        "--purpose": [
                          "GCE_FIREWALL"
                        ],
                        "--purpose-data": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "values": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--parent": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "resource-settings": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--effective": "bool",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-values": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-value": {
              "commands": {},
              "flags": {
                "--value-file": "dynamic"
              }
            },
            "unset-value": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "run": {
          "commands": {
            "deploy": {
              "commands": {},
              "flags": {
                "--add-cloudsql-instances": "value",
                "--add-custom-audiences": "value",
                "--allow-unauthenticated": "bool",
                "--args": "value",
                "--async": "bool",
                "--binary-authorization": "value",
                "--breakglass": "value",
                "--clear-binary-authorization": "bool",
                "--clear-cloudsql-instances": "bool",
                "--clear-config-maps": "bool",
                "--clear-custom-audiences": "bool",
                "--clear-encryption-key-shutdown-hours": "bool",
                "--clear-env-vars": "bool",
                "--clear-key": "bool",
                "--clear-labels": "bool",
                "--clear-post-key-revocation-action-type": "bool",
                "--clear-secrets": "bool",
                "--clear-vpc-connector": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--command": "value",
                "--concurrency": "value",
                "--connectivity": [
                  "external",
                  "internal"
                ],
                "--context": "value",
                "--cpu": "value",
                "--cpu-boost": "bool",
                "--cpu-throttling": "bool",
                "--description": "value",
                "--encryption-key-shutdown-hours": "value",
                "--env-vars-file": "value",
                "--execution-environment": [
                  "gen1",
                  "gen2"
                ],
                "--image": "value",
                "--ingress": [
                  "all",
                  "internal",
                  "internal-and-cloud-load-balancing"
                ],
                "--key": "value",
                "--kubeconfig": "value",
                "--labels": "value",
                "--max-instances": "value",
                "--memory": "value",
                "--min-instances": "value",
                "--namespace": "value",
                "--no-traffic": "bool",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--port": "value",
                "--post-key-revocation-action-type": [
                  "prevent-new",
                  "shut-down"
                ],
                "--region": "value",
                "--remove-cloudsql-instances": "value",
                "--remove-config-maps": "value",
                "--remove-custom-audiences": "value",
                "--remove-env-vars": "value",
                "--remove-labels": "value",
                "--remove-secrets": "value",
                "--revision-suffix": "value",
                "--service-account": "value",
                "--session-affinity": "bool",
                "--set-cloudsql-instances": "value",
                "--set-config-maps": "value",
                "--set-custom-audiences": "value",
                "--set-env-vars": "value",
                "--set-secrets": "value",
                "--source": "value",
                "--tag": "value",
                "--timeout": "value",
                "--update-config-maps": "value",
                "--update-env-vars": "value",
                "--update-labels": "value",
                "--update-secrets": "value",
                "--use-http2": "bool",
                "--vpc-connector": "value",
                "--vpc-egress": [
                  "all",
                  "all-traffic",
                  "private-ranges-only"
                ]
              }
            },
            "domain-mappings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--force-override": "bool",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "integrations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--name": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--service": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool"
                  }
                },
                "types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-service": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--remove-service": "value"
                  }
                }
              },
              "flags": {
                "--region": "value"
              }
            },
            "jobs": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-env-vars": "bool",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--remove-env-vars": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--source": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--update-env-vars": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "execute": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--wait": "bool"
                  }
                },
                "executions": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--job": "value",
                        "--limit": "value",
                        "--namespace": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "logs": {
                      "commands": {
                        "read": {
                          "commands": {},
                          "flags": {
                            "--freshness": "value",
                            "--limit": "value",
                            "--log-filter": "value",
                            "--order": [
                              "asc",
                              "desc"
                            ],
                            "--region": "value"
                          }
                        },
                        "tail": {
                          "commands": {},
                          "flags": {
                            "--log-filter": "value",
                            "--region": "value"
                          }
                        }
                      },
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "tasks": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--region": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--abandoned": "bool",
                            "--cancelled": "bool",
                            "--completed": "bool",
                            "--execution": "value",
                            "--failed": "bool",
                            "--filter": "value",
                            "--limit": "value",
                            "--no-completed": "bool",
                            "--no-started": "bool",
                            "--page-size": "value",
                            "--pending": "bool",
                            "--region": "value",
                            "--running": "bool",
                            "--sort-by": "value",
                            "--started": "bool",
                            "--succeeded": "bool",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--freshness": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--log-filter": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "replace": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-cloudsql-instances": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-binary-authorization": "bool",
                    "--clear-cloudsql-instances": "bool",
                    "--clear-env-vars": "bool",
                    "--clear-labels": "bool",
                    "--clear-secrets": "bool",
                    "--clear-vpc-connector": "bool",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--remove-cloudsql-instances": "value",
                    "--remove-env-vars": "value",
                    "--remove-labels": "value",
                    "--remove-secrets": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--update-env-vars": "value",
                    "--update-labels": "value",
                    "--update-secrets": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                }
              },
              "flags": {
                "--region": "value"
              }
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "revisions": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--freshness": "value",
                        "--kubeconfig": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--log-filter": "value",
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--member": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--freshness": "value",
                        "--kubeconfig": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--log-filter": "value",
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "proxy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--port": "value",
                    "--region": "value",
                    "--tag": "value",
                    "--token": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--member": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "replace": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-cloudsql-instances": "value",
                    "--add-custom-audiences": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-binary-authorization": "bool",
                    "--clear-cloudsql-instances": "bool",
                    "--clear-config-maps": "bool",
                    "--clear-custom-audiences": "bool",
                    "--clear-encryption-key-shutdown-hours": "bool",
                    "--clear-env-vars": "bool",
                    "--clear-key": "bool",
                    "--clear-labels": "bool",
                    "--clear-post-key-revocation-action-type": "bool",
                    "--clear-secrets": "bool",
                    "--clear-vpc-connector": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--command": "value",
                    "--concurrency": "value",
                    "--connectivity": [
                      "external",
                      "internal"
                    ],
                    "--context": "value",
                    "--cpu": "value",
                    "--cpu-boost": "bool",
                    "--cpu-throttling": "bool",
                    "--description": "value",
                    "--encryption-key-shutdown-hours": "value",
                    "--env-vars-file": "value",
                    "--execution-environment": [
                      "gen1",
                      "gen2"
                    ],
                    "--image": "value",
                    "--ingress": [
                      "all",
                      "internal",
                      "internal-and-cloud-load-balancing"
                    ],
                    "--key": "value",
                    "--kubeconfig": "value",
                    "--labels": "value",
                    "--max-instances": "value",
                    "--memory": "value",
                    "--min-instances": "value",
                    "--namespace": "value",
                    "--no-traffic": "bool",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--port": "value",
                    "--post-key-revocation-action-type": [
                      "prevent-new",
                      "shut-down"
                    ],
                    "--region": "value",
                    "--remove-cloudsql-instances": "value",
                    "--remove-config-maps": "value",
                    "--remove-custom-audiences": "value",
                    "--remove-env-vars": "value",
                    "--remove-labels": "value",
                    "--remove-secrets": "value",
                    "--revision-suffix": "value",
                    "--service-account": "value",
                    "--session-affinity": "bool",
                    "--set-cloudsql-instances": "value",
                    "--set-config-maps": "value",
                    "--set-custom-audiences": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--tag": "value",
                    "--timeout": "value",
                    "--update-config-maps": "value",
                    "--update-env-vars": "value",
                    "--update-labels": "value",
                    "--update-secrets": "value",
                    "--use-http2": "bool",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ]
                  }
                },
                "update-traffic": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--breakglass": "value",
                    "--clear-tags": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--remove-tags": "value",
                    "--set-tags": "value",
                    "--to-latest": "bool",
                    "--to-revisions": "value",
                    "--to-tags": "value",
                    "--update-tags": "value"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "scc": {
          "commands": {
            "assets": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "get-parent": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "get-project": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "group": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--filter": "value",
                    "--group-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--field-mask": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--order-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value"
                  }
                },
                "list-marks": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value"
                  }
                },
                "run-discovery": {
                  "commands": {},
                  "flags": {}
                },
                "update-marks": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--security-marks": "value",
                    "--start-time": "value",
                    "--update-mask": "value"
                  }
                }
              },
              "flags": {}
            },
            "custom-modules": {
              "commands": {
                "sha": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--custom-config-from-file": "value",
                        "--display-name": "value",
                        "--enablement-state": [
                          "disabled",
                          "enabled",
                          "enablement-state-unspecified",
                          "inherited"
                        ],
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "get": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "get-effective": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list-descendant": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list-effective": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "test": {
                      "commands": {},
                      "flags": {
                        "--custom-config-from-file": "value",
                        "--display-name": "value",
                        "--enablement-state": [
                          "disabled",
                          "enabled",
                          "enablement-state-unspecified",
                          "inherited"
                        ],
                        "--folder": "value",
                        "--organization": "value",
                        "--test-data-from-file": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--custom-config-from-file": "value",
                        "--display-name": "value",
                        "--enablement-state": [
                          "disabled",
                          "enabled",
                          "enablement-state-unspecified",
                          "inherited"
                        ],
                        "--folder": "value",
                        "--organization": "value",
                        "--update-mask": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "findings": {
              "commands": {
                "bulk-mute": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--category": "value",
                    "--event-time": "value",
                    "--external-uri": "value",
                    "--organization": "value",
                    "--resource-name": "value",
                    "--source": "dynamic",
                    "--source-properties": "value",
                    "--state": [
                      "active",
                      "inactive",
                      "state-unspecified"
                    ]
                  }
                },
                "group": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--filter": "value",
                    "--group-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--field-mask": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--order-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value",
                    "--source": "value"
                  }
                },
                "list-marks": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value",
                    "--source": "dynamic"
                  }
                },
                "set-mute": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--mute": [
                      "muted",
                      "unmuted"
                    ],
                    "--organization": "value",
                    "--source": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--event-time": "value",
                    "--external-uri": "value",
                    "--organization": "value",
                    "--source": "dynamic",
                    "--source-properties": "value",
                    "--state": [
                      "active",
                      "inactive",
                      "state-unspecified"
                    ],
                    "--update-mask": "value"
                  }
                },
                "update-marks": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--security-marks": "value",
                    "--source": "dynamic",
                    "--start-time": "value",
                    "--update-mask": "value"
                  }
                }
              },
              "flags": {}
            },
            "muteconfigs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "get": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--update-mask": "value"
                  }
                }
              },
              "flags": {}
            },
            "notifications": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "describe-explicit": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "services": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value",
                        "--service": [
                          "container-threat-detection",
                          "event-threat-detection",
                          "rapid-vulnerability-detection",
                          "security-health-analytics",
                          "virtual-machine-threat-detection",
                          "web-security-scanner"
                        ]
                      }
                    },
                    "describe-explicit": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value",
                        "--service": [
                          "container-threat-detection",
                          "event-threat-detection",
                          "rapid-vulnerability-detection",
                          "security-health-analytics",
                          "virtual-machine-threat-detection",
                          "web-security-scanner"
                        ]
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value",
                        "--service": [
                          "container-threat-detection",
                          "event-threat-detection",
                          "rapid-vulnerability-detection",
                          "security-health-analytics",
                          "virtual-machine-threat-detection",
                          "web-security-scanner"
                        ]
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--organization": "value",
                        "--service": [
                          "container-threat-detection",
                          "event-threat-detection",
                          "rapid-vulnerability-detection",
                          "security-health-analytics",
                          "virtual-machine-threat-detection",
                          "web-security-scanner"
                        ]
                      }
                    },
                    "inherit": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--service": [
                          "container-threat-detection",
                          "event-threat-detection",
                          "rapid-vulnerability-detection",
                          "security-health-analytics",
                          "virtual-machine-threat-detection",
                          "web-security-scanner"
                        ]
                      }
                    },
                    "modules": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--folder": "value",
                            "--module": "value",
                            "--organization": "value",
                            "--service": [
                              "container-threat-detection",
                              "event-threat-detection",
                              "rapid-vulnerability-detection",
                              "security-health-analytics",
                              "virtual-machine-threat-detection",
                              "web-security-scanner"
                            ]
                          }
                        },
                        "describe-explicit": {
                          "commands": {},
                          "flags": {
                            "--folder": "value",
                            "--module": "value",
                            "--organization": "value",
                            "--service": [
                              "container-threat-detection",
                              "event-threat-detection",
                              "rapid-vulnerability-detection",
                              "security-health-analytics",
                              "virtual-machine-threat-detection",
                              "web-security-scanner"
                            ]
                          }
                        },
                        "disable": {
                          "commands": {},
                          "flags": {
                            "--folder": "value",
                            "--module": "value",
                            "--organization": "value",
                            "--service": [
                              "container-threat-detection",
                              "event-threat-detection",
                              "rapid-vulnerability-detection",
                              "security-health-analytics",
                              "virtual-machine-threat-detection",
                              "web-security-scanner"
                            ]
                          }
                        },
                        "enable": {
                          "commands": {},
                          "flags": {
                            "--folder": "value",
                            "--module": "value",
                            "--organization": "value",
                            "--service": [
                              "container-threat-detection",
                              "event-threat-detection",
                              "rapid-vulnerability-detection",
                              "security-health-analytics",
                              "virtual-machine-threat-detection",
                              "web-security-scanner"
                            ]
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--clear-config": "bool",
                            "--config": "value",
                            "--enablement-state": [
                              "disabled",
                              "enabled"
                            ],
                            "--folder": "value",
                            "--module": "value",
                            "--organization": "value",
                            "--service": [
                              "container-threat-detection",
                              "event-threat-detection",
                              "rapid-vulnerability-detection",
                              "security-health-analytics",
                              "virtual-machine-threat-detection",
                              "web-security-scanner"
                            ]
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "sources": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--source": "value",
                    "--source-display-name": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "scheduler": {
          "commands": {
            "jobs": {
              "commands": {
                "create": {
                  "commands": {
                    "app-engine": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--description": "value",
                        "--headers": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--relative-url": "value",
                        "--schedule": "value",
                        "--service": "value",
                        "--time-zone": "value"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--description": "value",
                        "--headers": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--oauth-service-account-email": "value",
                        "--oauth-token-scope": "value",
                        "--oidc-service-account-email": "value",
                        "--oidc-token-audience": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--uri": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--attributes": "value",
                        "--description": "value",
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--topic": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {
                    "app-engine": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--clear-headers": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-message-body": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-relative-url": "bool",
                        "--clear-service": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--relative-url": "value",
                        "--remove-headers": "value",
                        "--schedule": "value",
                        "--service": "value",
                        "--time-zone": "value",
                        "--update-headers": "value"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--clear-auth-token": "bool",
                        "--clear-headers": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-message-body": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--oauth-service-account-email": "value",
                        "--oauth-token-scope": "value",
                        "--oidc-service-account-email": "value",
                        "--oidc-token-audience": "value",
                        "--remove-headers": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--update-headers": "value",
                        "--uri": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--clear-attributes": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--remove-attributes": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--topic": "value",
                        "--update-attributes": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "secrets": {
          "commands": {
            "config": {
              "commands": {
                "export": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--path": "value",
                    "--resource-format": [
                      "krm",
                      "terraform"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "versions": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ],
                        "--secret": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "service-directory": {
          "commands": {
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--address": "value",
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value",
                    "--network": "value",
                    "--port": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value",
                    "--service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--address": "value",
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value",
                    "--port": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "namespaces": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--namespace": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--namespace": "value",
                    "--role": "value"
                  }
                },
                "resolve": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--location": "value",
                    "--max-endpoints": "value",
                    "--namespace": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "api-keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--allowed-application": "value",
                    "--allowed-bundle-ids": "value",
                    "--allowed-ips": "value",
                    "--allowed-referrers": "value",
                    "--annotations": "value",
                    "--api-target": "value",
                    "--async": "bool",
                    "--display-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-key-string": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "lookup": {
                  "commands": {},
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--key-string": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allowed-application": "value",
                    "--allowed-bundle-ids": "value",
                    "--allowed-ips": "value",
                    "--allowed-referrers": "value",
                    "--annotations": "value",
                    "--api-target": "value",
                    "--async": "bool",
                    "--clear-annotations": "bool",
                    "--clear-restrictions": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "disable": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "identity": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--available": "bool",
                "--enabled": "bool",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--full": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "peered-dns-domains": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dns-suffix": "value",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--network": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "quota": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--dimensions": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--service": "value",
                    "--unit": "value",
                    "--value": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--override-id": "value",
                    "--service": "value",
                    "--unit": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--consumer": "value",
                    "--dimensions": "value",
                    "--force": "bool",
                    "--metric": "value",
                    "--service": "value",
                    "--unit": "value",
                    "--value": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpc-peerings": {
              "commands": {
                "connect": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--ranges": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "disable-vpc-service-controls": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "enable-vpc-service-controls": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--name": "value"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--network": "value",
                    "--ranges": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "source": {
          "commands": {
            "project-configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-topic": "value",
                    "--disable-pushblock": "bool",
                    "--enable-pushblock": "bool",
                    "--message-format": [
                      "json",
                      "protobuf"
                    ],
                    "--remove-topic": "value",
                    "--service-account": "value",
                    "--topic-project": "value",
                    "--update-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "repos": {
              "commands": {
                "clone": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--use-full-gcloud-path": "bool"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-topic": "value",
                    "--message-format": [
                      "json",
                      "protobuf"
                    ],
                    "--remove-topic": "value",
                    "--service-account": "value",
                    "--topic-project": "value",
                    "--update-topic": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "spanner": {
          "commands": {
            "backups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "copy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-backup": "value",
                    "--destination-instance": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-config-default-or-backup-encryption"
                    ],
                    "--expiration-date": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--retention-period": "value",
                    "--source-backup": "value",
                    "--source-instance": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-database-encryption"
                    ],
                    "--expiration-date": "value",
                    "--instance": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--retention-period": "value",
                    "--version-time": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "update-metadata": {
                  "commands": {},
                  "flags": {
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value"
                  }
                }
              },
              "flags": {}
            },
            "databases": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--instance": "dynamic",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database-dialect": [
                      "GOOGLE_STANDARD_SQL",
                      "POSTGRESQL"
                    ],
                    "--ddl": "value",
                    "--ddl-file": "dynamic",
                    "--instance": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value"
                  }
                },
                "ddl": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--ddl": "value",
                        "--ddl-file": "dynamic",
                        "--instance": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                },
                "execute-sql": {
                  "commands": {},
                  "flags": {
                    "--database-role": "value",
                    "--enable-partitioned-dml": "bool",
                    "--instance": "value",
                    "--query-mode": [
                      "NORMAL",
                      "PLAN",
                      "PROFILE"
                    ],
                    "--read-timestamp": "value",
                    "--sql": "value",
                    "--strong": "bool",
                    "--timeout": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-database": "value",
                    "--destination-instance": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-config-default-or-backup-encryption"
                    ],
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--source-backup": "value",
                    "--source-instance": "value"
                  }
                },
                "roles": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "sessions": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--instance": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--server-filter": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                }
              },
              "flags": {}
            },
            "instance-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--add-replicas": "value",
                    "--async": "bool",
                    "--base-config": "value",
                    "--clone-config": "dynamic",
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--replicas": "value",
                    "--skip-replicas": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--etag": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--config": "dynamic",
                    "--description": "value",
                    "--expire-behavior": [
                      "free-to-provisioned",
                      "remove-after-grace-period"
                    ],
                    "--instance-type": [
                      "free-instance",
                      "provisioned"
                    ],
                    "--nodes": "value",
                    "--processing-units": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--expire-behavior": [
                      "free-to-provisioned",
                      "remove-after-grace-period"
                    ],
                    "--instance-type": [
                      "free-instance",
                      "provisioned"
                    ],
                    "--nodes": "value",
                    "--processing-units": "value"
                  }
                }
              },
              "flags": {}
            },
            "migration": {
              "commands": {
                "data": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--log-level": "value",
                    "--prefix": "value",
                    "--session": "value",
                    "--skip-foreign-keys": "bool",
                    "--source": "value",
                    "--source-profile": "value",
                    "--target": "value",
                    "--target-profile": "value",
                    "--write-limit": "value"
                  }
                },
                "schema": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--log-level": "value",
                    "--prefix": "value",
                    "--source": "value",
                    "--source-profile": "value",
                    "--target": "value",
                    "--target-profile": "value"
                  }
                },
                "schema-and-data": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--log-level": "value",
                    "--prefix": "value",
                    "--skip-foreign-keys": "bool",
                    "--source": "value",
                    "--source-profile": "value",
                    "--target": "value",
                    "--target-profile": "value",
                    "--write-limit": "value"
                  }
                },
                "web": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": [
                      "BACKUP",
                      "DATABASE",
                      "DATABASE_CREATE",
                      "DATABASE_RESTORE",
                      "DATABASE_UPDATE_DDL",
                      "INSTANCE",
                      "INSTANCE_CONFIG_CREATE",
                      "INSTANCE_CONFIG_UPDATE"
                    ],
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "rows": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--instance": "value",
                    "--keys": "value",
                    "--table": "value"
                  }
                },
                "insert": {
                  "commands": {},
                  "flags": {
                    "--data": "value",
                    "--database": "value",
                    "--instance": "value",
                    "--table": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--data": "value",
                    "--database": "value",
                    "--instance": "value",
                    "--table": "value"
                  }
                }
              },
              "flags": {}
            },
            "samples": {
              "commands": {
                "backend": {
                  "commands": {},
                  "flags": {
                    "--database-id": "value",
                    "--duration": "value",
                    "--instance-id": "value",
                    "--port": "value"
                  }
                },
                "init": {
                  "commands": {},
                  "flags": {
                    "--database-id": "value",
                    "--instance-id": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--cleanup": "bool",
                    "--database-id": "value",
                    "--duration": "value",
                    "--instance-id": "value",
                    "--skip-init": "bool"
                  }
                },
                "workload": {
                  "commands": {},
                  "flags": {
                    "--duration": "value",
                    "--port": "value",
                    "--target-qps": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "sql": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--instance": "dynamic",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-instance": "dynamic",
                    "--restore-instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "connect": {
              "commands": {},
              "flags": {
                "--database": "value",
                "--port": "value",
                "--user": "value"
              }
            },
            "databases": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--instance": "value",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--charset": "value",
                    "--collation": "value",
                    "--instance": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "patch": {
                  "commands": {},
                  "flags": {
                    "--charset": "value",
                    "--collation": "value",
                    "--diff": "bool",
                    "--instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {
                "bak": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--stripe_count": "value",
                    "--striped": "bool"
                  }
                },
                "csv": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--escape": "value",
                    "--fields-terminated-by": "value",
                    "--lines-terminated-by": "value",
                    "--offload": "bool",
                    "--query": "value",
                    "--quote": "value"
                  }
                },
                "sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--offload": "bool",
                    "--table": "value"
                  }
                }
              },
              "flags": {}
            },
            "flags": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--database-version": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "generate-login-token": {
              "commands": {},
              "flags": {
                "--application-default-credential": "bool",
                "--instance": "dynamic"
              }
            },
            "import": {
              "commands": {
                "bak": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cert-path": "value",
                    "--database": "value",
                    "--prompt-for-pvk-password": "bool",
                    "--pvk-password": "value",
                    "--pvk-path": "value",
                    "--striped": "bool"
                  }
                },
                "csv": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--columns": "value",
                    "--database": "value",
                    "--escape": "value",
                    "--fields-terminated-by": "value",
                    "--lines-terminated-by": "value",
                    "--quote": "value",
                    "--table": "value",
                    "--user": "value"
                  }
                },
                "sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--user": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "clone": {
                  "commands": {},
                  "flags": {
                    "--allocated-ip-range-name": "value",
                    "--async": "bool",
                    "--bin-log-file-name": "value",
                    "--bin-log-position": "value",
                    "--point-in-time": "value",
                    "--restore-database-name": "value"
                  }
                },
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--activation-policy": [
                      "always",
                      "never",
                      "on-demand"
                    ],
                    "--active-directory-domain": "value",
                    "--allocated-ip-range-name": "value",
                    "--assign-ip": "bool",
                    "--async": "bool",
                    "--audit-bucket-path": "value",
                    "--audit-retention-interval": "value",
                    "--audit-upload-interval": "value",
                    "--authorized-networks": "value",
                    "--availability-type": [
                      "regional",
                      "zonal"
                    ],
                    "--backup": "bool",
                    "--backup-location": "value",
                    "--backup-start-time": "value",
                    "--client-certificate-path": "value",
                    "--client-key-path": "value",
                    "--collation": "value",
                    "--connector-enforcement": [
                      "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                      "NOT_REQUIRED",
                      "REQUIRED"
                    ],
                    "--cpu": "value",
                    "--database-flags": "value",
                    "--database-version": "value",
                    "--deletion-protection": "bool",
                    "--deny-maintenance-period-end-date": "value",
                    "--deny-maintenance-period-start-date": "value",
                    "--deny-maintenance-period-time": "value",
                    "--disk-encryption-key": "dynamic",
                    "--disk-encryption-key-keyring": "dynamic",
                    "--disk-encryption-key-location": "dynamic",
                    "--disk-encryption-key-project": "dynamic",
                    "--enable-bin-log": "bool",
                    "--enable-google-private-path": "bool",
                    "--enable-password-policy": "bool",
                    "--enable-point-in-time-recovery": "bool",
                    "--failover-replica-name": "value",
                    "--gce-zone": "value",
                    "--insights-config-query-insights-enabled": "bool",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--labels": "value",
                    "--maintenance-release-channel": [
                      "preview",
                      "production"
                    ],
                    "--maintenance-window-day": [
                      "FRI",
                      "MON",
                      "SAT",
                      "SUN",
                      "THU",
                      "TUE",
                      "WED"
                    ],
                    "--maintenance-window-hour": "value",
                    "--master-ca-certificate-path": "value",
                    "--master-dump-file-path": "value",
                    "--master-instance-name": "value",
                    "--master-password": "value",
                    "--master-username": "value",
                    "--memory": "value",
                    "--network": "value",
                    "--password-policy-complexity": [
                      "COMPLEXITY_DEFAULT",
                      "COMPLEXITY_UNSPECIFIED"
                    ],
                    "--password-policy-disallow-username-substring": "bool",
                    "--password-policy-min-length": "value",
                    "--password-policy-password-change-interval": "value",
                    "--password-policy-reuse-interval": "value",
                    "--prompt-for-master-password": "bool",
                    "--region": "value",
                    "--replica-type": [
                      "FAILOVER",
                      "READ"
                    ],
                    "--replication": [
                      "asynchronous",
                      "synchronous"
                    ],
                    "--require-ssl": "bool",
                    "--retained-backups-count": "value",
                    "--retained-transaction-log-days": "value",
                    "--root-password": "value",
                    "--secondary-zone": "value",
                    "--source-ip-address": "value",
                    "--source-port": "value",
                    "--storage-auto-increase": "bool",
                    "--storage-auto-increase-limit": "value",
                    "--storage-size": "value",
                    "--storage-type": [
                      "HDD",
                      "SSD"
                    ],
                    "--tier": "value",
                    "--time-zone": "value",
                    "--timeout": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--table": "value"
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "patch": {
                  "commands": {},
                  "flags": {
                    "--activation-policy": [
                      "always",
                      "never",
                      "on-demand"
                    ],
                    "--active-directory-domain": "value",
                    "--allocated-ip-range-name": "value",
                    "--assign-ip": "bool",
                    "--async": "bool",
                    "--audit-bucket-path": "value",
                    "--audit-retention-interval": "value",
                    "--audit-upload-interval": "value",
                    "--authorized-gae-apps": "value",
                    "--authorized-networks": "value",
                    "--availability-type": [
                      "regional",
                      "zonal"
                    ],
                    "--backup-location": "value",
                    "--backup-start-time": "value",
                    "--clear-authorized-networks": "bool",
                    "--clear-database-flags": "bool",
                    "--clear-gae-apps": "bool",
                    "--clear-labels": "bool",
                    "--clear-password-policy": "bool",
                    "--connector-enforcement": [
                      "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                      "NOT_REQUIRED",
                      "REQUIRED"
                    ],
                    "--cpu": "value",
                    "--database-flags": "value",
                    "--database-version": "value",
                    "--deletion-protection": "bool",
                    "--deny-maintenance-period-end-date": "value",
                    "--deny-maintenance-period-start-date": "value",
                    "--deny-maintenance-period-time": "value",
                    "--diff": "bool",
                    "--enable-bin-log": "bool",
                    "--enable-database-replication": "bool",
                    "--enable-google-private-path": "bool",
                    "--enable-password-policy": "bool",
                    "--enable-point-in-time-recovery": "bool",
                    "--follow-gae-app": "value",
                    "--gce-zone": "value",
                    "--insights-config-query-insights-enabled": "bool",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--maintenance-release-channel": [
                      "preview",
                      "production"
                    ],
                    "--maintenance-version": "value",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "FRI",
                      "MON",
                      "SAT",
                      "SUN",
                      "THU",
                      "TUE",
                      "WED"
                    ],
                    "--maintenance-window-hour": "value",
                    "--memory": "value",
                    "--network": "value",
                    "--no-backup": "bool",
                    "--password-policy-complexity": [
                      "COMPLEXITY_DEFAULT",
                      "COMPLEXITY_UNSPECIFIED"
                    ],
                    "--password-policy-disallow-username-substring": "bool",
                    "--password-policy-min-length": "value",
                    "--password-policy-password-change-interval": "value",
                    "--password-policy-reuse-interval": "value",
                    "--pricing-plan": [
                      "PACKAGE",
                      "PER_USE"
                    ],
                    "--remove-deny-maintenance-period": "bool",
                    "--remove-labels": "value",
                    "--replication": [
                      "asynchronous",
                      "synchronous"
                    ],
                    "--require-ssl": "bool",
                    "--retained-backups-count": "value",
                    "--retained-transaction-log-days": "value",
                    "--secondary-zone": "value",
                    "--storage-auto-increase": "bool",
                    "--storage-auto-increase-limit": "value",
                    "--storage-size": "value",
                    "--tier": "value",
                    "--update-labels": "value",
                    "--zone": "value"
                  }
                },
                "promote-replica": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "reset-ssl-config": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "restore-backup": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-id": "value",
                    "--backup-instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--timeout": "value"
                  }
                }
              },
              "flags": {}
            },
            "reschedule-maintenance": {
              "commands": {},
              "flags": {
                "--reschedule-type": [
                  "IMMEDIATE",
                  "NEXT_AVAILABLE_WINDOW",
                  "SPECIFIC_TIME"
                ],
                "--schedule-time": "value"
              }
            },
            "ssl": {
              "commands": {
                "client-certs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "server-ca-certs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rollback": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "rotate": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ssl-certs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tiers": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "users": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password": "value",
                    "--password-policy-allowed-failed-attempts": "value",
                    "--password-policy-enable-failed-attempts-check": "bool",
                    "--password-policy-enable-password-verification": "bool",
                    "--password-policy-password-expiration-duration": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--host": "value",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--host": "value",
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-password": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-dual-password": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password": "value",
                    "--prompt-for-password": "bool",
                    "--retain-password": "bool"
                  }
                },
                "set-password-policy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-password-policy": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password-policy-allowed-failed-attempts": "value",
                    "--password-policy-enable-failed-attempts-check": "bool",
                    "--password-policy-enable-password-verification": "bool",
                    "--password-policy-password-expiration-duration": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "storage": {
          "commands": {
            "buckets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--default-encryption-key": "value",
                    "--default-storage-class": "value",
                    "--enable-autoclass": "bool",
                    "--location": "value",
                    "--pap": "bool",
                    "--placement": "value",
                    "--public-access-prevention": "bool",
                    "--recovery-point-objective": [
                      "ASYNC_TURBO",
                      "DEFAULT"
                    ],
                    "--retention-period": "value",
                    "--rpo": [
                      "ASYNC_TURBO",
                      "DEFAULT"
                    ],
                    "--uniform-bucket-level-access": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--continue-on-error": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "notifications": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--custom-attributes": "value",
                        "--event-types": "value",
                        "--object-prefix": "value",
                        "--payload-format": [
                          "json",
                          "none"
                        ],
                        "--skip-topic-setup": "bool",
                        "--topic": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--continue-on-error": "bool",
                    "--etag": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--acl-file": "value",
                    "--add-acl-grant": "value",
                    "--add-default-object-acl-grant": "value",
                    "--additional-headers": "value",
                    "--canned-acl": "value",
                    "--clear-cors": "bool",
                    "--clear-default-encryption-key": "bool",
                    "--clear-labels": "bool",
                    "--clear-lifecycle": "bool",
                    "--clear-log-bucket": "bool",
                    "--clear-log-object-prefix": "bool",
                    "--clear-pap": "bool",
                    "--clear-public-access-prevention": "bool",
                    "--clear-retention-period": "bool",
                    "--clear-web-error-page": "bool",
                    "--clear-web-main-page-suffix": "bool",
                    "--continue-on-error": "bool",
                    "--cors-file": "value",
                    "--default-encryption-key": "value",
                    "--default-event-based-hold": "bool",
                    "--default-object-acl-file": "value",
                    "--enable-autoclass": "bool",
                    "--labels-file": "value",
                    "--lifecycle-file": "value",
                    "--lock-retention-period": "bool",
                    "--log-bucket": "value",
                    "--log-object-prefix": "value",
                    "--pap": "bool",
                    "--predefined-acl": "value",
                    "--predefined-default-object-acl": "value",
                    "--public-access-prevention": "bool",
                    "--recovery-point-objective": [
                      "ASYNC_TURBO",
                      "DEFAULT"
                    ],
                    "--remove-acl-grant": "value",
                    "--remove-default-object-acl-grant": "value",
                    "--remove-labels": "value",
                    "--requester-pays": "bool",
                    "--retention-period": "value",
                    "--rpo": [
                      "ASYNC_TURBO",
                      "DEFAULT"
                    ],
                    "--uniform-bucket-level-access": "bool",
                    "--update-labels": "value",
                    "--versioning": "bool",
                    "--web-error-page": "value",
                    "--web-main-page-suffix": "value"
                  }
                }
              },
              "flags": {}
            },
            "cat": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--decryption-keys": "value",
                "--display-url": "bool",
                "--range": "value"
              }
            },
            "cp": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--all-versions": "bool",
                "--cache-control": "value",
                "--canned-acl": "value",
                "--clear-custom-metadata": "bool",
                "--content-disposition": "value",
                "--content-encoding": "value",
                "--content-language": "value",
                "--content-md5": "value",
                "--content-type": "value",
                "--continue-on-error": "bool",
                "--custom-metadata": "value",
                "--custom-time": "value",
                "--daisy-chain": "bool",
                "--decryption-keys": "value",
                "--do-not-decompress": "bool",
                "--encryption-key": "value",
                "--gzip-in-flight": "value",
                "--gzip-in-flight-all": "bool",
                "--gzip-local": "value",
                "--gzip-local-all": "bool",
                "--if-generation-match": "value",
                "--if-metageneration-match": "value",
                "--ignore-symlinks": "bool",
                "--manifest-path": "value",
                "--no-clobber": "bool",
                "--predefined-acl": "value",
                "--preserve-acl": "bool",
                "--preserve-posix": "bool",
                "--print-created-message": "bool",
                "--read-paths-from-stdin": "bool",
                "--recursive": "bool",
                "--remove-custom-metadata": "value",
                "--skip-unsupported": "bool",
                "--storage-class": "value",
                "--update-custom-metadata": "value"
              }
            },
            "hash": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--decryption-keys": "value",
                "--hex": "bool",
                "--skip-crc32c": "bool",
                "--skip-md5": "bool"
              }
            },
            "ls": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--all-versions": "bool",
                "--buckets": "bool",
                "--decryption-keys": "value",
                "--etag": "bool",
                "--fetch-encrypted-object-hashes": "bool",
                "--full": "bool",
                "--json": "bool",
                "--long": "bool",
                "--readable-sizes": "bool",
                "--recursive": "bool"
              }
            },
            "mv": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--all-versions": "bool",
                "--cache-control": "value",
                "--canned-acl": "value",
                "--clear-custom-metadata": "bool",
                "--content-disposition": "value",
                "--content-encoding": "value",
                "--content-language": "value",
                "--content-md5": "value",
                "--content-type": "value",
                "--continue-on-error": "bool",
                "--custom-metadata": "value",
                "--custom-time": "value",
                "--daisy-chain": "bool",
                "--decryption-keys": "value",
                "--do-not-decompress": "bool",
                "--encryption-key": "value",
                "--gzip-in-flight": "value",
                "--gzip-in-flight-all": "bool",
                "--gzip-local": "value",
                "--gzip-local-all": "bool",
                "--if-generation-match": "value",
                "--if-metageneration-match": "value",
                "--ignore-symlinks": "bool",
                "--manifest-path": "value",
                "--no-clobber": "bool",
                "--predefined-acl": "value",
                "--preserve-acl": "bool",
                "--preserve-posix": "bool",
                "--print-created-message": "bool",
                "--read-paths-from-stdin": "bool",
                "--remove-custom-metadata": "value",
                "--skip-unsupported": "bool",
                "--storage-class": "value",
                "--update-custom-metadata": "value"
              }
            },
            "objects": {
              "commands": {
                "compose": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--if-generation-match": "value",
                    "--if-metageneration-match": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--decryption-keys": "value",
                    "--fetch-encrypted-object-hashes": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--additional-headers": "value",
                    "--decryption-keys": "value",
                    "--fetch-encrypted-object-hashes": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--stat": "bool",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--acl-file": "value",
                    "--add-acl-grant": "value",
                    "--additional-headers": "value",
                    "--all-versions": "bool",
                    "--cache-control": "value",
                    "--canned-acl": "value",
                    "--clear-cache-control": "bool",
                    "--clear-content-disposition": "bool",
                    "--clear-content-encoding": "bool",
                    "--clear-content-language": "bool",
                    "--clear-content-md5": "bool",
                    "--clear-content-type": "bool",
                    "--clear-custom-metadata": "bool",
                    "--clear-custom-time": "bool",
                    "--clear-encryption-key": "bool",
                    "--content-disposition": "value",
                    "--content-encoding": "value",
                    "--content-language": "value",
                    "--content-md5": "value",
                    "--content-type": "value",
                    "--continue-on-error": "bool",
                    "--custom-metadata": "value",
                    "--custom-time": "value",
                    "--decryption-keys": "value",
                    "--encryption-key": "value",
                    "--event-based-hold": "bool",
                    "--if-generation-match": "value",
                    "--if-metageneration-match": "value",
                    "--predefined-acl": "value",
                    "--preserve-acl": "bool",
                    "--read-paths-from-stdin": "bool",
                    "--recursive": "bool",
                    "--remove-acl-grant": "value",
                    "--remove-custom-metadata": "value",
                    "--storage-class": "value",
                    "--temporary-hold": "bool",
                    "--update-custom-metadata": "value"
                  }
                }
              },
              "flags": {}
            },
            "rm": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--all-versions": "bool",
                "--continue-on-error": "bool",
                "--if-generation-match": "value",
                "--if-metageneration-match": "value",
                "--read-paths-from-stdin": "bool",
                "--recursive": "bool"
              }
            },
            "service-agent": {
              "commands": {},
              "flags": {
                "--authorize-cmek": "value"
              }
            }
          },
          "flags": {}
        },
        "survey": {
          "commands": {},
          "flags": {}
        },
        "tasks": {
          "commands": {
            "acknowledge": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value",
                "--schedule-time": "value"
              }
            },
            "cancel-lease": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value",
                "--schedule-time": "value"
              }
            },
            "create-app-engine-task": {
              "commands": {},
              "flags": {
                "--header": "value",
                "--location": "value",
                "--method": "value",
                "--payload-content": "value",
                "--payload-file": "value",
                "--queue": "value",
                "--routing": "value",
                "--schedule-time": "value",
                "--url": "value"
              }
            },
            "create-http-task": {
              "commands": {},
              "flags": {
                "--body-content": "value",
                "--body-file": "value",
                "--header": "value",
                "--location": "value",
                "--method": "value",
                "--oauth-service-account-email": "value",
                "--oauth-token-scope": "value",
                "--oidc-service-account-email": "value",
                "--oidc-token-audience": "value",
                "--queue": "value",
                "--schedule-time": "value",
                "--url": "value"
              }
            },
            "create-pull-task": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--payload-content": "value",
                "--payload-file": "value",
                "--queue": "value",
                "--schedule-time": "value",
                "--tag": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value",
                "--response-view": [
                  "basic",
                  "full",
                  "view-unspecified"
                ]
              }
            },
            "lease": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lease-duration": "value",
                "--limit": "value",
                "--location": "value",
                "--oldest-tag": "bool",
                "--queue": "value",
                "--sort-by": "value",
                "--tag": "value",
                "--uri": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--queue": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "queues": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--http-header-override": "value",
                    "--http-method-override": "value",
                    "--http-oauth-service-account-email-override": "value",
                    "--http-oauth-token-scope-override": "value",
                    "--http-oidc-service-account-email-override": "value",
                    "--http-oidc-token-audience-override": "value",
                    "--http-uri-override": "value",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-tasks": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--max-tasks-dispatched-per-second": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                },
                "create-app-engine-queue": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-tasks": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--max-tasks-dispatched-per-second": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                },
                "create-pull-queue": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-retry-duration": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "purge": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-http-header-override": "bool",
                    "--clear-http-method-override": "bool",
                    "--clear-http-oauth-service-account-email-override": "bool",
                    "--clear-http-oauth-token-scope-override": "bool",
                    "--clear-http-oidc-service-account-email-override": "bool",
                    "--clear-http-oidc-token-audience-override": "bool",
                    "--clear-http-uri-override": "bool",
                    "--clear-max-attempts": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-concurrent-tasks": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-max-tasks-dispatched-per-second": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-routing-override": "bool",
                    "--http-header-override": "value",
                    "--http-method-override": "value",
                    "--http-oauth-service-account-email-override": "value",
                    "--http-oauth-token-scope-override": "value",
                    "--http-oidc-service-account-email-override": "value",
                    "--http-oidc-token-audience-override": "value",
                    "--http-uri-override": "value",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-tasks": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--max-tasks-dispatched-per-second": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                },
                "update-app-engine-queue": {
                  "commands": {},
                  "flags": {
                    "--clear-max-attempts": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-concurrent-tasks": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-max-tasks-dispatched-per-second": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-routing-override": "bool",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-tasks": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--max-tasks-dispatched-per-second": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                },
                "update-pull-queue": {
                  "commands": {},
                  "flags": {
                    "--clear-max-attempts": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-retry-duration": "value"
                  }
                }
              },
              "flags": {}
            },
            "renew-lease": {
              "commands": {},
              "flags": {
                "--lease-duration": "value",
                "--location": "value",
                "--queue": "value",
                "--schedule-time": "value"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value"
              }
            }
          },
          "flags": {}
        },
        "terraform": {
          "commands": {
            "vet": {
              "commands": {},
              "flags": {
                "--policy-library": "value",
                "--region": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        },
        "topic": {
          "commands": {
            "accessibility": {
              "commands": {},
              "flags": {}
            },
            "arg-files": {
              "commands": {},
              "flags": {}
            },
            "cli-trees": {
              "commands": {},
              "flags": {}
            },
            "client-certificate": {
              "commands": {},
              "flags": {}
            },
            "command-conventions": {
              "commands": {},
              "flags": {}
            },
            "configurations": {
              "commands": {},
              "flags": {}
            },
            "datetimes": {
              "commands": {},
              "flags": {}
            },
            "endpoint-override": {
              "commands": {},
              "flags": {}
            },
            "escaping": {
              "commands": {},
              "flags": {}
            },
            "filters": {
              "commands": {},
              "flags": {}
            },
            "flags-file": {
              "commands": {},
              "flags": {}
            },
            "formats": {
              "commands": {},
              "flags": {}
            },
            "gcloudignore": {
              "commands": {},
              "flags": {}
            },
            "offline-help": {
              "commands": {},
              "flags": {}
            },
            "projections": {
              "commands": {},
              "flags": {}
            },
            "resource-keys": {
              "commands": {},
              "flags": {}
            },
            "startup": {
              "commands": {},
              "flags": {}
            },
            "uninstall": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "trace": {
          "commands": {
            "sinks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "transfer": {
          "commands": {
            "agent-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bandwidth-limit": "value",
                    "--display-name": "value",
                    "--no-async": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--names": "value",
                    "--page-size": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bandwidth-limit": "value",
                    "--clear-bandwidth-limit": "bool",
                    "--clear-display-name": "bool",
                    "--display-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "agents": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--ids": "value",
                    "--uninstall": "bool"
                  }
                },
                "install": {
                  "commands": {},
                  "flags": {
                    "--count": "value",
                    "--creds-file": "value",
                    "--enable-multipart": "bool",
                    "--id-prefix": "value",
                    "--logs-directory": "value",
                    "--max-concurrent-small-file-uploads": "value",
                    "--memlock-limit": "value",
                    "--mount-directories": "value",
                    "--pool": "value",
                    "--proxy": "value",
                    "--s3-compatible-mode": "bool"
                  }
                }
              },
              "flags": {}
            },
            "authorize": {
              "commands": {},
              "flags": {
                "--add-missing": "bool",
                "--creds-file": "value"
              }
            },
            "jobs": {
              "commands": {
                "config": {
                  "commands": {
                    "export": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--path": "value",
                        "--resource-format": [
                          "krm",
                          "terraform"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--custom-storage-class": "value",
                    "--delete-from": [
                      "destination-if-unique",
                      "source-after-transfer"
                    ],
                    "--description": "value",
                    "--destination-agent-pool": "value",
                    "--do-not-run": "bool",
                    "--enable-posix-transfer-logs": "bool",
                    "--event-stream-expires": "value",
                    "--event-stream-name": "value",
                    "--event-stream-starts": "value",
                    "--exclude-prefixes": "value",
                    "--include-modified-after-absolute": "value",
                    "--include-modified-after-relative": "value",
                    "--include-modified-before-absolute": "value",
                    "--include-modified-before-relative": "value",
                    "--include-prefixes": "value",
                    "--intermediate-storage-path": "value",
                    "--log-action-states": "value",
                    "--log-actions": "value",
                    "--manifest-file": "value",
                    "--name": "value",
                    "--no-async": "bool",
                    "--notification-event-types": "value",
                    "--notification-payload-format": [
                      "json",
                      "none"
                    ],
                    "--notification-pubsub-topic": "value",
                    "--overwrite-when": [
                      "always",
                      "different",
                      "never"
                    ],
                    "--preserve-metadata": "value",
                    "--schedule-repeats-every": "value",
                    "--schedule-repeats-until": "value",
                    "--schedule-starts": "value",
                    "--source-agent-pool": "value",
                    "--source-auth-method": [
                      "AWS_SIGNATURE_V2",
                      "AWS_SIGNATURE_V4"
                    ],
                    "--source-creds-file": "value",
                    "--source-endpoint": "value",
                    "--source-list-api": [
                      "LIST_OBJECTS",
                      "LIST_OBJECTS_V2"
                    ],
                    "--source-network-protocol": [
                      "HTTP",
                      "HTTPS"
                    ],
                    "--source-request-model": [
                      "PATH_STYLE",
                      "VIRTUAL_HOSTED_STYLE"
                    ],
                    "--source-signing-region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--expand-table": "bool",
                    "--job-names": "value",
                    "--job-statuses": "value",
                    "--limit": "value",
                    "--page-size": "value"
                  }
                },
                "monitor": {
                  "commands": {},
                  "flags": {}
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--no-async": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-custom-storage-class": "bool",
                    "--clear-delete-from": "bool",
                    "--clear-description": "bool",
                    "--clear-destination-agent-pool": "bool",
                    "--clear-event-stream": "bool",
                    "--clear-exclude-prefixes": "bool",
                    "--clear-include-modified-after-absolute": "bool",
                    "--clear-include-modified-after-relative": "bool",
                    "--clear-include-modified-before-absolute": "bool",
                    "--clear-include-modified-before-relative": "bool",
                    "--clear-include-prefixes": "bool",
                    "--clear-intermediate-storage-path": "bool",
                    "--clear-log-config": "bool",
                    "--clear-manifest-file": "bool",
                    "--clear-notification-config": "bool",
                    "--clear-notification-event-types": "bool",
                    "--clear-preserve-metadata": "bool",
                    "--clear-schedule": "bool",
                    "--clear-source-agent-pool": "bool",
                    "--clear-source-auth-method": "bool",
                    "--clear-source-creds-file": "bool",
                    "--clear-source-endpoint": "bool",
                    "--clear-source-list-api": "bool",
                    "--clear-source-network-protocol": "bool",
                    "--clear-source-request-model": "bool",
                    "--clear-source-signing-region": "bool",
                    "--custom-storage-class": "value",
                    "--delete-from": [
                      "destination-if-unique",
                      "source-after-transfer"
                    ],
                    "--description": "value",
                    "--destination": "value",
                    "--destination-agent-pool": "value",
                    "--enable-posix-transfer-logs": "bool",
                    "--event-stream-expires": "value",
                    "--event-stream-name": "value",
                    "--event-stream-starts": "value",
                    "--exclude-prefixes": "value",
                    "--include-modified-after-absolute": "value",
                    "--include-modified-after-relative": "value",
                    "--include-modified-before-absolute": "value",
                    "--include-modified-before-relative": "value",
                    "--include-prefixes": "value",
                    "--intermediate-storage-path": "value",
                    "--log-action-states": "value",
                    "--log-actions": "value",
                    "--manifest-file": "value",
                    "--notification-event-types": "value",
                    "--notification-payload-format": [
                      "json",
                      "none"
                    ],
                    "--notification-pubsub-topic": "value",
                    "--overwrite-when": [
                      "always",
                      "different",
                      "never"
                    ],
                    "--preserve-metadata": "value",
                    "--schedule-repeats-every": "value",
                    "--schedule-repeats-until": "value",
                    "--schedule-starts": "value",
                    "--source": "value",
                    "--source-agent-pool": "value",
                    "--source-auth-method": [
                      "AWS_SIGNATURE_V2",
                      "AWS_SIGNATURE_V4"
                    ],
                    "--source-creds-file": "value",
                    "--source-endpoint": "value",
                    "--source-list-api": [
                      "LIST_OBJECTS",
                      "LIST_OBJECTS_V2"
                    ],
                    "--source-network-protocol": [
                      "HTTP",
                      "HTTPS"
                    ],
                    "--source-request-model": [
                      "PATH_STYLE",
                      "VIRTUAL_HOSTED_STYLE"
                    ],
                    "--source-signing-region": "value",
                    "--status": [
                      "deleted",
                      "disabled",
                      "enabled"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--expand-table": "bool",
                    "--job-names": "value",
                    "--limit": "value",
                    "--operation-names": "value",
                    "--operation-statuses": "value",
                    "--page-size": "value"
                  }
                },
                "monitor": {
                  "commands": {},
                  "flags": {}
                },
                "pause": {
                  "commands": {},
                  "flags": {}
                },
                "resume": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "web-security-scanner": {
          "commands": {
            "scan-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--auth-password": "value",
                    "--auth-type": [
                      "custom",
                      "google",
                      "none"
                    ],
                    "--auth-url": "value",
                    "--auth-user": "value",
                    "--blacklist-patterns": "value",
                    "--display-name": "value",
                    "--export-to-security-center": "bool",
                    "--max-qps": "value",
                    "--risk-level": [
                      "low",
                      "normal"
                    ],
                    "--schedule-interval-days": "value",
                    "--schedule-next-start": "value",
                    "--starting-urls": "value",
                    "--target-platforms": "value",
                    "--user-agent": [
                      "chrome-android",
                      "chrome-linux",
                      "safari-iphone",
                      "user-agent-unspecified"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--auth-password": "value",
                    "--auth-type": [
                      "custom",
                      "google",
                      "none"
                    ],
                    "--auth-url": "value",
                    "--auth-user": "value",
                    "--blacklist-patterns": "value",
                    "--display-name": "value",
                    "--export-to-security-center": "bool",
                    "--max-qps": "value",
                    "--risk-level": [
                      "low",
                      "normal"
                    ],
                    "--schedule-interval-days": "value",
                    "--schedule-next-start": "value",
                    "--starting-urls": "value",
                    "--target-platforms": "value",
                    "--user-agent": [
                      "chrome-android",
                      "chrome-linux",
                      "safari-iphone",
                      "user-agent-unspecified"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "scan-runs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--scan-config": "value"
                  }
                },
                "findings": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--scan-config": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--scan-config": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-crawled-urls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--scan-config": "value",
                    "--scan-run": "value",
                    "--sort-by": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {}
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--scan-config": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "workstations": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--enable-private-endpoint": "bool",
                    "--network": "value",
                    "--region": "value",
                    "--subnetwork": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--cluster": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-custom-image": "value",
                    "--container-env": "value",
                    "--container-predefined-image": [
                      "base-image",
                      "clion",
                      "codeoss",
                      "goland",
                      "intellij",
                      "phpstorm",
                      "pycharm",
                      "rider",
                      "rubymine",
                      "webstorm"
                    ],
                    "--container-run-as-user": "value",
                    "--container-working-dir": "value",
                    "--disable-public-ip-addresses": "bool",
                    "--enable-confidential-compute": "bool",
                    "--idle-timeout": "value",
                    "--machine-type": [
                      "e2-standard-16",
                      "e2-standard-2",
                      "e2-standard-32",
                      "e2-standard-4",
                      "e2-standard-8",
                      "n1-standard-4",
                      "n2-standard-8",
                      "n2d-standard-16",
                      "n2d-standard-2",
                      "n2d-standard-32",
                      "n2d-standard-4",
                      "n2d-standard-8",
                      "t2d-standard-6"
                    ],
                    "--network-tags": "value",
                    "--pd-disk-size": [
                      10,
                      50,
                      100,
                      200,
                      500,
                      1000
                    ],
                    "--pd-disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--pd-reclaim-policy": [
                      "delete",
                      "retain"
                    ],
                    "--pool-size": "value",
                    "--region": "value",
                    "--running-timeout": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--cluster": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-custom-image": "value",
                    "--container-env": "value",
                    "--container-predefined-image": [
                      "base-image",
                      "clion",
                      "codeoss",
                      "goland",
                      "intellij",
                      "phpstorm",
                      "pycharm",
                      "rider",
                      "rubymine",
                      "webstorm"
                    ],
                    "--container-run-as-user": "value",
                    "--container-working-dir": "value",
                    "--disable-public-ip-addresses": "bool",
                    "--enable-confidential-compute": "bool",
                    "--idle-timeout": "value",
                    "--machine-type": [
                      "e2-standard-16",
                      "e2-standard-2",
                      "e2-standard-32",
                      "e2-standard-4",
                      "e2-standard-8",
                      "n1-standard-4",
                      "n2-standard-8",
                      "n2d-standard-16",
                      "n2d-standard-2",
                      "n2d-standard-32",
                      "n2d-standard-4",
                      "n2d-standard-8",
                      "t2d-standard-6"
                    ],
                    "--network-tags": "value",
                    "--pool-size": "value",
                    "--region": "value",
                    "--running-timeout": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--command": "value",
                "--config": "value",
                "--port": "value",
                "--region": "value",
                "--user": "value"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "start-tcp-tunnel": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--local-host-port": "value",
                "--region": "value"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "anthos": {
      "commands": {
        "auth": {
          "commands": {
            "login": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--dry-run": "bool",
                "--kubeconfig": "value",
                "--login-config": "value",
                "--login-config-cert": "value",
                "--set-preferred-auth": "bool",
                "--user": "value"
              }
            }
          },
          "flags": {}
        },
        "config": {
          "commands": {
            "controller": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster-ipv4-cidr-block": "value",
                    "--cluster-named-range": "value",
                    "--full-management": "bool",
                    "--location": "dynamic",
                    "--man-block": "value",
                    "--man-blocks": "value",
                    "--master-ipv4-cidr-block": "value",
                    "--network": "value",
                    "--services-ipv4-cidr-block": "value",
                    "--services-named-range": "value",
                    "--subnet": "value",
                    "--use-private-endpoint": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-config-connector-identity": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--full-name": "bool",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "create-login-config": {
          "commands": {},
          "flags": {
            "--kubeconfig": "value",
            "--merge-from": "value",
            "--output": "value"
          }
        }
      },
      "flags": {}
    },
    "api-gateway": {
      "commands": {
        "api-configs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--async": "bool",
                "--backend-auth-service-account": "value",
                "--display-name": "value",
                "--grpc-files": "value",
                "--labels": "value",
                "--openapi-spec": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--view": [
                  "BASIC",
                  "FULL"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--async": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "apis": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--display-name": "value",
                "--labels": "value",
                "--managed-service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "gateways": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--api-config": "value",
                "--async": "bool",
                "--display-name": "value",
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--api-config": "value",
                "--async": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--location": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "apigee": {
      "commands": {
        "apis": {
          "commands": {
            "deploy": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--environment": "value",
                "--organization": "value",
                "--override": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value",
                "--verbose": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "undeploy": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--environment": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "applications": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--developer": "value",
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "deployments": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--environment": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--api": "value",
                "--environment": "value",
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--revision": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "developers": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "environments": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "organizations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "products": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--all-environments": "bool",
                "--all-proxies": "bool",
                "--apis": "value",
                "--attributes": "value",
                "--description": "value",
                "--display-name": "value",
                "--environments": "value",
                "--internal-access": "bool",
                "--manual-approval": "bool",
                "--oauth-scopes": "value",
                "--organization": "value",
                "--private-access": "bool",
                "--public-access": "bool",
                "--quota": "value",
                "--quota-interval": "value",
                "--quota-unit": [
                  "day",
                  "hour",
                  "minute",
                  "month"
                ],
                "--resources": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-api": "value",
                "--add-attribute": "value",
                "--add-environment": "value",
                "--add-oauth-scope": "value",
                "--add-resource": "value",
                "--all-apis": "bool",
                "--all-environments": "bool",
                "--all-resources": "bool",
                "--automatic-approval": "bool",
                "--clear-attributes": "bool",
                "--clear-description": "bool",
                "--clear-oauth-scopes": "bool",
                "--clear-quota": "bool",
                "--description": "value",
                "--display-name": "value",
                "--internal-access": "bool",
                "--manual-approval": "bool",
                "--organization": "value",
                "--private-access": "bool",
                "--public-access": "bool",
                "--quota": "value",
                "--quota-interval": "value",
                "--quota-unit": [
                  "day",
                  "hour",
                  "minute",
                  "month"
                ],
                "--remove-api": "value",
                "--remove-attribute": "value",
                "--remove-environment": "value",
                "--remove-oauth-scope": "value",
                "--remove-resource": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "app": {
      "commands": {
        "browse": {
          "commands": {},
          "flags": {
            "--launch-browser": "bool",
            "--service": "value"
          }
        },
        "create": {
          "commands": {},
          "flags": {
            "--region": "value",
            "--service-account": "value"
          }
        },
        "deploy": {
          "commands": {},
          "flags": {
            "--appyaml": "value",
            "--bucket": "value",
            "--cache": "bool",
            "--ignore-file": "value",
            "--image-url": "value",
            "--promote": "bool",
            "--service-account": "value",
            "--stop-previous-version": "bool"
          }
        },
        "describe": {
          "commands": {},
          "flags": {}
        },
        "domain-mappings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--certificate-id": "value",
                "--certificate-management": [
                  "automatic",
                  "manual"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--certificate-id": "value",
                "--certificate-management": [
                  "automatic",
                  "manual"
                ],
                "--no-certificate-id": "bool"
              }
            }
          },
          "flags": {}
        },
        "firewall-rules": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--action": [
                  "ALLOW",
                  "DENY"
                ],
                "--description": "value",
                "--source-range": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "test-ip": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--action": [
                  "ALLOW",
                  "DENY"
                ],
                "--description": "value",
                "--source-range": "value"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "disable-debug": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "enable-debug": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--service": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "scp": {
              "commands": {},
              "flags": {
                "--compress": "bool",
                "--recurse": "bool",
                "--service": "value"
              }
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--container": "value",
                "--service": "value"
              }
            }
          },
          "flags": {}
        },
        "logs": {
          "commands": {
            "read": {
              "commands": {},
              "flags": {
                "--level": [
                  "any",
                  "critical",
                  "debug",
                  "error",
                  "info",
                  "warning"
                ],
                "--limit": "value",
                "--logs": "value",
                "--service": "value"
              }
            },
            "tail": {
              "commands": {},
              "flags": {
                "--level": [
                  "any",
                  "critical",
                  "debug",
                  "error",
                  "info",
                  "warning"
                ],
                "--logs": "value",
                "--service": "value"
              }
            }
          },
          "flags": {}
        },
        "open-console": {
          "commands": {},
          "flags": {
            "--logs": "bool",
            "--service": "value"
          }
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--pending": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "browse": {
              "commands": {},
              "flags": {
                "--launch-browser": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-traffic": {
              "commands": {},
              "flags": {
                "--migrate": "bool",
                "--split-by": [
                  "cookie",
                  "ip",
                  "random"
                ],
                "--splits": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--ingress": [
                  "all",
                  "internal-and-cloud-load-balancing",
                  "internal-only"
                ]
              }
            }
          },
          "flags": {}
        },
        "ssl-certificates": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--certificate": "value",
                "--display-name": "value",
                "--private-key": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--certificate": "value",
                "--display-name": "value",
                "--private-key": "value"
              }
            }
          },
          "flags": {}
        },
        "update": {
          "commands": {},
          "flags": {
            "--service-account": "value",
            "--split-health-checks": "bool"
          }
        },
        "versions": {
          "commands": {
            "browse": {
              "commands": {},
              "flags": {
                "--launch-browser": "bool",
                "--service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--hide-no-traffic": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--service": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "migrate": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--service": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "artifacts": {
      "commands": {
        "apt": {
          "commands": {
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--gcs-source": "value",
                "--location": "value"
              }
            },
            "upload": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            }
          },
          "flags": {}
        },
        "docker": {
          "commands": {
            "images": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--delete-tags": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--metadata-filter": "value",
                    "--show-all-metadata": "bool",
                    "--show-build-details": "bool",
                    "--show-deployment": "bool",
                    "--show-image-basis": "bool",
                    "--show-package-vulnerability": "bool",
                    "--show-provenance": "bool"
                  }
                },
                "get-operation": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--include-tags": "bool",
                    "--limit": "value",
                    "--occurrence-filter": "value",
                    "--page-size": "value",
                    "--show-occurrences": "bool",
                    "--show-occurrences-from": "value",
                    "--sort-by": "value"
                  }
                },
                "list-vulnerabilities": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value"
                  }
                },
                "scan": {
                  "commands": {},
                  "flags": {
                    "--additional-package-types": "value",
                    "--async": "bool",
                    "--location": [
                      "asia",
                      "europe",
                      "us"
                    ],
                    "--remote": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "files": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--package": "value",
                "--page-size": "value",
                "--repository": "value",
                "--sort-by": "value",
                "--tag": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "packages": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--repository": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--repository": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--repository": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "print-settings": {
          "commands": {
            "gradle": {
              "commands": {},
              "flags": {
                "--json-key": "value",
                "--location": "value",
                "--repository": "value"
              }
            },
            "mvn": {
              "commands": {},
              "flags": {
                "--json-key": "value",
                "--location": "value",
                "--repository": "value"
              }
            },
            "npm": {
              "commands": {},
              "flags": {
                "--json-key": "value",
                "--location": "value",
                "--repository": "value",
                "--scope": "value"
              }
            },
            "python": {
              "commands": {},
              "flags": {
                "--json-key": "value",
                "--location": "value",
                "--repository": "value"
              }
            }
          },
          "flags": {}
        },
        "repositories": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--allow-snapshot-overwrites": "bool",
                "--async": "bool",
                "--description": "value",
                "--kms-key": "value",
                "--labels": "value",
                "--location": "value",
                "--mode": [
                  "none",
                  "remote-repository",
                  "standard-repository",
                  "virtual-repository"
                ],
                "--remote-docker-repo": [
                  "docker-hub",
                  "none"
                ],
                "--remote-mvn-repo": [
                  "maven-central",
                  "none"
                ],
                "--remote-npm-repo": [
                  "none",
                  "npmjs"
                ],
                "--remote-python-repo": [
                  "none",
                  "pypi"
                ],
                "--remote-repo-config-desc": "value",
                "--repository-format": "value",
                "--upstream-policy-file": "value",
                "--version-policy": [
                  "none",
                  "release",
                  "snapshot"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--location": "value",
                "--remove-labels": "value",
                "--update-labels": "value",
                "--upstream-policy-file": "value"
              }
            }
          },
          "flags": {}
        },
        "tags": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--package": "value",
                "--repository": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--package": "value",
                "--repository": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--package": "value",
                "--page-size": "value",
                "--repository": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--package": "value",
                "--repository": "value"
              }
            }
          },
          "flags": {}
        },
        "versions": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--delete-tags": "bool",
                "--location": "value",
                "--package": "value",
                "--repository": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--package": "value",
                "--repository": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--package": "value",
                "--page-size": "value",
                "--repository": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "vpcsc-config": {
          "commands": {
            "allow": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "deny": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "yum": {
          "commands": {
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--gcs-source": "value",
                "--location": "value"
              }
            },
            "upload": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "asset": {
      "commands": {
        "analyze-iam-policy": {
          "commands": {},
          "flags": {
            "--access-time": "value",
            "--analyze-service-account-impersonation": "bool",
            "--execution-timeout": "value",
            "--expand-groups": "bool",
            "--expand-resources": "bool",
            "--expand-roles": "bool",
            "--folder": "value",
            "--full-resource-name": "value",
            "--identity": "value",
            "--organization": "value",
            "--output-group-edges": "bool",
            "--output-resource-edges": "bool",
            "--permissions": "value",
            "--roles": "value",
            "--saved-analysis-query": "value",
            "--show-response": "bool"
          }
        },
        "analyze-iam-policy-longrunning": {
          "commands": {},
          "flags": {
            "--access-time": "value",
            "--analyze-service-account-impersonation": "bool",
            "--bigquery-dataset": "value",
            "--bigquery-partition-key": [
              "PARTITION_KEY_UNSPECIFIED",
              "REQUEST_TIME"
            ],
            "--bigquery-table-prefix": "value",
            "--bigquery-write-disposition": "value",
            "--expand-groups": "bool",
            "--expand-resources": "bool",
            "--expand-roles": "bool",
            "--folder": "value",
            "--full-resource-name": "value",
            "--gcs-output-path": "value",
            "--identity": "value",
            "--organization": "value",
            "--output-group-edges": "bool",
            "--output-resource-edges": "bool",
            "--permissions": "value",
            "--roles": "value"
          }
        },
        "analyze-move": {
          "commands": {},
          "flags": {
            "--blockers-only": "value",
            "--destination-folder": "value",
            "--destination-organization": "value"
          }
        },
        "export": {
          "commands": {},
          "flags": {
            "--asset-types": "value",
            "--bigquery-dataset": "dynamic",
            "--bigquery-table": "dynamic",
            "--content-type": [
              "access-policy",
              "iam-policy",
              "org-policy",
              "os-inventory",
              "relationship",
              "resource"
            ],
            "--folder": "value",
            "--organization": "value",
            "--output-bigquery-force": "bool",
            "--output-path": "value",
            "--output-path-prefix": "value",
            "--partition-key": [
              "read-time",
              "request-time"
            ],
            "--per-asset-type": "bool",
            "--relationship-types": "value",
            "--snapshot-time": "value"
          }
        },
        "feeds": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--asset-names": "value",
                "--asset-types": "value",
                "--condition-description": "value",
                "--condition-expression": "value",
                "--condition-title": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--folder": "value",
                "--organization": "value",
                "--pubsub-topic": "value",
                "--relationship-types": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-asset-names": "value",
                "--add-asset-types": "value",
                "--add-relationship-types": "value",
                "--clear-asset-names": "bool",
                "--clear-asset-types": "bool",
                "--clear-condition-description": "bool",
                "--clear-condition-expression": "bool",
                "--clear-condition-title": "bool",
                "--clear-content-type": "bool",
                "--clear-relationship-types": "bool",
                "--condition-description": "value",
                "--condition-expression": "value",
                "--condition-title": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--folder": "value",
                "--organization": "value",
                "--pubsub-topic": "value",
                "--remove-asset-names": "value",
                "--remove-asset-types": "value",
                "--remove-relationship-types": "value"
              }
            }
          },
          "flags": {}
        },
        "get-effective-iam-policy": {
          "commands": {},
          "flags": {
            "--names": "value",
            "--scope": "value"
          }
        },
        "get-history": {
          "commands": {},
          "flags": {
            "--asset-names": "value",
            "--content-type": [
              "access-policy",
              "iam-policy",
              "org-policy",
              "os-inventory",
              "relationship",
              "resource"
            ],
            "--end-time": "value",
            "--organization": "value",
            "--relationship-types": "value",
            "--start-time": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--asset-types": "value",
            "--content-type": [
              "access-policy",
              "iam-policy",
              "org-policy",
              "os-inventory",
              "relationship",
              "resource"
            ],
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--organization": "value",
            "--page-size": "value",
            "--relationship-types": "value",
            "--snapshot-time": "value",
            "--sort-by": "value"
          }
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "query": {
          "commands": {},
          "flags": {
            "--bigquery-dataset": "dynamic",
            "--bigquery-table": "dynamic",
            "--end-time": "value",
            "--folder": "value",
            "--job-reference": "value",
            "--organization": "value",
            "--page-size": "value",
            "--page-token": "value",
            "--snapshot-time": "value",
            "--start-time": "value",
            "--statement": "value",
            "--timeout": "value",
            "--write-disposition": [
              "write-append",
              "write-empty",
              "write-truncate"
            ]
          }
        },
        "saved-queries": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--folder": "value",
                "--labels": "value",
                "--organization": "value",
                "--query-file-path": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--folder": "value",
                "--organization": "value",
                "--query-file-path": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "search-all-iam-policies": {
          "commands": {},
          "flags": {
            "--asset-types": "value",
            "--filter": "value",
            "--limit": "value",
            "--order-by": "value",
            "--page-size": "value",
            "--query": "value",
            "--scope": "value",
            "--sort-by": "value"
          }
        },
        "search-all-resources": {
          "commands": {},
          "flags": {
            "--asset-types": "value",
            "--filter": "value",
            "--limit": "value",
            "--order-by": "value",
            "--page-size": "value",
            "--query": "value",
            "--read-mask": "value",
            "--scope": "value",
            "--sort-by": "value"
          }
        }
      },
      "flags": {}
    },
    "assured": {
      "commands": {
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "workloads": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--compliance-regime": [
                  "ASSURED_WORKLOADS_FOR_PARTNERS",
                  "CA_REGIONS_AND_SUPPORT",
                  "CJIS",
                  "EU_REGIONS_AND_SUPPORT",
                  "FEDRAMP_HIGH",
                  "FEDRAMP_MODERATE",
                  "HIPAA",
                  "HITRUST",
                  "IL4",
                  "ITAR",
                  "US_REGIONAL_ACCESS"
                ],
                "--display-name": "value",
                "--enable-sovereign-controls": "value",
                "--external-identifier": "value",
                "--labels": "value",
                "--location": "value",
                "--next-rotation-time": "value",
                "--organization": "value",
                "--partner": [
                  "LOCAL_CONTROLS_BY_S3NS",
                  "SOVEREIGN_CONTROLS_BY_T_SYSTEMS"
                ],
                "--provisioned-resources-parent": "value",
                "--resource-settings": "value",
                "--rotation-period": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--location": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--display-name": "value",
                "--etag": "value",
                "--labels": "value",
                "--location": "value",
                "--organization": "value"
              }
            },
            "violations": {
              "commands": {
                "acknowledge": {
                  "commands": {},
                  "flags": {
                    "--comment": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--workload": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--organization": "value",
                    "--workload": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--workload": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "auth": {
      "commands": {
        "activate-service-account": {
          "commands": {},
          "flags": {
            "--key-file": "value",
            "--password-file": "value",
            "--prompt-for-password": "bool"
          }
        },
        "application-default": {
          "commands": {
            "login": {
              "commands": {},
              "flags": {
                "--client-id-file": "value",
                "--disable-quota-project": "bool",
                "--launch-browser": "bool",
                "--login-config": "value",
                "--no-browser": "bool",
                "--scopes": "value"
              }
            },
            "print-access-token": {
              "commands": {},
              "flags": {
                "--scopes": "value"
              }
            },
            "revoke": {
              "commands": {},
              "flags": {}
            },
            "set-quota-project": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "configure-docker": {
          "commands": {},
          "flags": {}
        },
        "enterprise-certificate-config": {
          "commands": {
            "create": {
              "commands": {
                "linux": {
                  "commands": {},
                  "flags": {
                    "--ecp": "value",
                    "--ecp-client": "value",
                    "--label": "value",
                    "--module": "value",
                    "--output-file": "value",
                    "--slot": "value",
                    "--tls-offload": "value",
                    "--user-pin": "value"
                  }
                },
                "macos": {
                  "commands": {},
                  "flags": {
                    "--ecp": "value",
                    "--ecp-client": "value",
                    "--issuer": "value",
                    "--output-file": "value",
                    "--tls-offload": "value"
                  }
                },
                "windows": {
                  "commands": {},
                  "flags": {
                    "--ecp": "value",
                    "--ecp-client": "value",
                    "--issuer": "value",
                    "--output-file": "value",
                    "--provider": "value",
                    "--store": "value",
                    "--tls-offload": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--filter-account": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "login": {
          "commands": {},
          "flags": {
            "--activate": "bool",
            "--brief": "bool",
            "--cred-file": "value",
            "--enable-gdrive-access": "bool",
            "--force": "bool",
            "--launch-browser": "bool",
            "--login-config": "value",
            "--no-browser": "bool",
            "--update-adc": "bool"
          }
        },
        "print-access-token": {
          "commands": {},
          "flags": {}
        },
        "print-identity-token": {
          "commands": {},
          "flags": {
            "--audiences": "value",
            "--include-email": "bool",
            "--include-license": "bool",
            "--token-format": [
              "full",
              "standard"
            ]
          }
        },
        "revoke": {
          "commands": {},
          "flags": {
            "--all": "bool"
          }
        }
      },
      "flags": {}
    },
    "batch": {
      "commands": {
        "jobs": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "submit": {
              "commands": {},
              "flags": {
                "--config": "value",
                "--container-commands-file": "value",
                "--container-entrypoint": "value",
                "--container-image-uri": "value",
                "--location": "value",
                "--machine-type": "value",
                "--network": "value",
                "--priority": "value",
                "--provisioning-model": [
                  "SPOT",
                  "STANDARD"
                ],
                "--script-file-path": "value",
                "--script-text": "value",
                "--subnetwork": "value"
              }
            }
          },
          "flags": {}
        },
        "tasks": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--job": "value",
                "--location": "value",
                "--task_group": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--job": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "beta": {
      "commands": {
        "access-approval": {
          "commands": {
            "requests": {
              "commands": {
                "approve": {
                  "commands": {},
                  "flags": {}
                },
                "dismiss": {
                  "commands": {},
                  "flags": {}
                },
                "get": {
                  "commands": {},
                  "flags": {}
                },
                "invalidate": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--state": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "get": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--active_key_version": "value",
                    "--enrolled_services": "value",
                    "--folder": "value",
                    "--notification_emails": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "access-context-manager": {
          "commands": {
            "levels": {
              "commands": {
                "conditions": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--level": "value",
                        "--policy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--basic-level-spec": "value",
                    "--combine-function": [
                      "and",
                      "or"
                    ],
                    "--custom-level-spec": "value",
                    "--description": "value",
                    "--policy": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "replace-all": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--source-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--basic-level-spec": "value",
                    "--combine-function": [
                      "and",
                      "or"
                    ],
                    "--custom-level-spec": "value",
                    "--description": "value",
                    "--policy": "value",
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            },
            "perimeters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--access-levels": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--egress-policies": "value",
                    "--enable-vpc-accessible-services": "bool",
                    "--ingress-policies": "value",
                    "--perimeter-type": [
                      "bridge",
                      "regular"
                    ],
                    "--policy": "value",
                    "--resources": "value",
                    "--restricted-services": "value",
                    "--title": "value",
                    "--vpc-allowed-services": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--policy": "value"
                  }
                },
                "dry-run": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--access-levels": "value",
                        "--async": "bool",
                        "--egress-policies": "value",
                        "--enable-vpc-accessible-services": "bool",
                        "--ingress-policies": "value",
                        "--perimeter-access-levels": "value",
                        "--perimeter-description": "value",
                        "--perimeter-egress-policies": "value",
                        "--perimeter-enable-vpc-accessible-services": "bool",
                        "--perimeter-ingress-policies": "value",
                        "--perimeter-resources": "value",
                        "--perimeter-restricted-services": "value",
                        "--perimeter-title": "value",
                        "--perimeter-type": "value",
                        "--perimeter-vpc-allowed-services": "value",
                        "--policy": "value",
                        "--resources": "value",
                        "--restricted-services": "value",
                        "--vpc-allowed-services": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--policy": "value"
                      }
                    },
                    "drop": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "enforce": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--policy": "value"
                      }
                    },
                    "enforce-all": {
                      "commands": {},
                      "flags": {
                        "--etag": "value",
                        "--policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--policy": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-access-levels": "value",
                        "--add-resources": "value",
                        "--add-restricted-services": "value",
                        "--add-vpc-allowed-services": "value",
                        "--async": "bool",
                        "--clear-access-levels": "bool",
                        "--clear-egress-policies": "bool",
                        "--clear-ingress-policies": "bool",
                        "--clear-resources": "bool",
                        "--clear-restricted-services": "bool",
                        "--clear-vpc-allowed-services": "bool",
                        "--enable-vpc-accessible-services": "bool",
                        "--policy": "value",
                        "--remove-access-levels": "value",
                        "--remove-resources": "value",
                        "--remove-restricted-services": "value",
                        "--remove-vpc-allowed-services": "value",
                        "--set-egress-policies": "value",
                        "--set-ingress-policies": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--policy": "value",
                    "--sort-by": "value"
                  }
                },
                "replace-all": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--source-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-access-levels": "value",
                    "--add-resources": "value",
                    "--add-restricted-services": "value",
                    "--add-vpc-allowed-services": "value",
                    "--clear-access-levels": "bool",
                    "--clear-egress-policies": "bool",
                    "--clear-ingress-policies": "bool",
                    "--clear-resources": "bool",
                    "--clear-restricted-services": "bool",
                    "--clear-vpc-allowed-services": "bool",
                    "--description": "value",
                    "--enable-vpc-accessible-services": "bool",
                    "--policy": "value",
                    "--remove-access-levels": "value",
                    "--remove-resources": "value",
                    "--remove-restricted-services": "value",
                    "--remove-vpc-allowed-services": "value",
                    "--set-access-levels": "value",
                    "--set-egress-policies": "value",
                    "--set-ingress-policies": "value",
                    "--set-resources": "value",
                    "--set-restricted-services": "value",
                    "--title": "value",
                    "--type": [
                      "bridge",
                      "regular"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "policies": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--organization": "value",
                    "--scopes": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "active-directory": {
          "commands": {
            "domains": {
              "commands": {
                "backups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--domain": "value",
                        "--labels": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--domain": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--domain": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--domain": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--domain": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-name": "value",
                    "--async": "bool",
                    "--authorized-networks": "value",
                    "--enable-audit-logs": "bool",
                    "--labels": "value",
                    "--region": "value",
                    "--reserved-ip-range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "describe-ldaps-settings": {
                  "commands": {},
                  "flags": {}
                },
                "extend-schema": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--ldif-file": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "migration": {
                  "commands": {
                    "check-permissions": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--disable-sid-filtering-domains": "value",
                        "--onprem-domains": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "reset-admin-password": {
                  "commands": {},
                  "flags": {}
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "sql-integrations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--domain": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--domain": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "trusts": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--direction": [
                          "bidirectional",
                          "inbound",
                          "outbound",
                          "trust-direction-unspecified"
                        ],
                        "--handshake-secret": "value",
                        "--selective-authentication": "bool",
                        "--target-dns-ip-addresses": "value",
                        "--target-domain-name": "value",
                        "--type": [
                          "external",
                          "forest",
                          "trust-type-unspecified"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-domain-name": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-dns-ip-addresses": "value",
                        "--target-domain-name": "value"
                      }
                    },
                    "validate-state": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--target-domain-name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-authorized-networks": "value",
                    "--add-region": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--enable-audit-logs": "bool",
                    "--remove-authorized-networks": "value",
                    "--remove-labels": "value",
                    "--remove-region": "value",
                    "--update-labels": "value"
                  }
                },
                "update-ldaps-settings": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-password": "value",
                    "--certificate-pfx-file": "value",
                    "--clear-certificates": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "peerings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorized-network": "value",
                    "--domain": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ai": {
          "commands": {
            "custom-jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--command": "value",
                    "--config": "value",
                    "--display-name": "value",
                    "--enable-dashboard-access": "bool",
                    "--enable-web-access": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--network": "value",
                    "--python-package-uris": "value",
                    "--region": "value",
                    "--service-account": "value",
                    "--worker-pool-spec": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "local-run": {
                  "commands": {},
                  "flags": {
                    "--executor-image-uri": "value",
                    "--extra-dirs": "value",
                    "--extra-packages": "value",
                    "--gpu": "bool",
                    "--local-package-path": "value",
                    "--output-image-uri": "value",
                    "--python-module": "value",
                    "--requirements": "value",
                    "--script": "value",
                    "--service-account-key-file": "value"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--region": "dynamic",
                    "--task-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--encryption-kms-key-name": "value",
                    "--endpoint-id": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--region": "value",
                    "--request-response-logging-rate": "value",
                    "--request-response-logging-table": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "deploy-model": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--autoscaling-metric-specs": "value",
                    "--deployed-model-id": "value",
                    "--display-name": "value",
                    "--enable-access-logging": "bool",
                    "--enable-container-logging": "bool",
                    "--machine-type": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--model": "value",
                    "--region": "dynamic",
                    "--service-account": "value",
                    "--shared-resources": "dynamic",
                    "--shared-resources-region": "dynamic",
                    "--traffic-split": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "explain": {
                  "commands": {},
                  "flags": {
                    "--deployed-model-id": "value",
                    "--json-request": "value",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "predict": {
                  "commands": {},
                  "flags": {
                    "--json-request": "value",
                    "--region": "dynamic"
                  }
                },
                "raw-predict": {
                  "commands": {},
                  "flags": {
                    "--http-headers": "value",
                    "--region": "dynamic",
                    "--request": "value"
                  }
                },
                "undeploy-model": {
                  "commands": {},
                  "flags": {
                    "--deployed-model-id": "value",
                    "--region": "dynamic",
                    "--traffic-split": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-traffic-split": "bool",
                    "--description": "value",
                    "--disable-request-response-logging": "bool",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--request-response-logging-rate": "value",
                    "--request-response-logging-table": "value",
                    "--traffic-split": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "hp-tuning-jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--algorithm": [
                      "algorithm-unspecified",
                      "grid-search",
                      "random-search"
                    ],
                    "--config": "value",
                    "--display-name": "value",
                    "--enable-dashboard-access": "bool",
                    "--enable-web-access": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--max-trial-count": "value",
                    "--network": "value",
                    "--parallel-trial-count": "value",
                    "--region": "value",
                    "--service-account": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--region": "dynamic",
                    "--task-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "index-endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "deploy-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--display-name": "value",
                    "--index": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mutate-deployed-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--max-replica-count": "value",
                    "--min-replica-count": "value",
                    "--region": "dynamic"
                  }
                },
                "undeploy-index": {
                  "commands": {},
                  "flags": {
                    "--deployed-index-id": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "indexes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--metadata-file": "value",
                    "--metadata-schema-uri": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-datapoints": {
                  "commands": {},
                  "flags": {
                    "--datapoint-ids": "value",
                    "--datapoints-from-file": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--metadata-file": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                },
                "upsert-datapoints": {
                  "commands": {},
                  "flags": {
                    "--datapoints-from-file": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "model-monitoring-jobs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--analysis-instance-schema": "value",
                    "--anomaly-cloud-logging": "bool",
                    "--bigquery-uri": "value",
                    "--data-format": "value",
                    "--dataset": "value",
                    "--display-name": "value",
                    "--emails": "value",
                    "--endpoint": "value",
                    "--feature-attribution-thresholds": "value",
                    "--feature-thresholds": "value",
                    "--gcs-uris": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--log-ttl": "value",
                    "--monitoring-config-from-file": "value",
                    "--monitoring-frequency": "value",
                    "--predict-instance-schema": "value",
                    "--prediction-sampling-rate": "value",
                    "--region": "value",
                    "--sample-predict-request": "value",
                    "--target-field": "value",
                    "--training-sampling-rate": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--analysis-instance-schema": "value",
                    "--anomaly-cloud-logging": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--emails": "value",
                    "--feature-attribution-thresholds": "value",
                    "--feature-thresholds": "value",
                    "--log-ttl": "value",
                    "--monitoring-config-from-file": "value",
                    "--monitoring-frequency": "value",
                    "--prediction-sampling-rate": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "copy": {
                  "commands": {},
                  "flags": {
                    "--destination-model-id": "value",
                    "--destination-parent-model": "value",
                    "--kms-key-name": "value",
                    "--region": "value",
                    "--source-model": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "delete-version": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-version": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--artifact-uri": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-env-vars": "value",
                    "--container-health-route": "value",
                    "--container-image-uri": "value",
                    "--container-ports": "value",
                    "--container-predict-route": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--explanation-metadata-file": "value",
                    "--explanation-method": "value",
                    "--explanation-modality": "value",
                    "--explanation-nearest-neighbor-search-config-file": "value",
                    "--explanation-neighbor-count": "value",
                    "--explanation-path-count": "value",
                    "--explanation-query": "value",
                    "--explanation-step-count": "value",
                    "--labels": "value",
                    "--model-id": "value",
                    "--parent-model": "value",
                    "--region": "value",
                    "--smooth-grad-noise-sigma": "value",
                    "--smooth-grad-noise-sigma-by-feature": "value",
                    "--smooth-grad-noisy-sample-count": "value",
                    "--uris": "value",
                    "--version-aliases": "value",
                    "--version-description": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--index": "value",
                    "--index-endpoint": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-experiments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--tensorboard-id": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-runs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tensorboard-time-series": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--plugin-data": "value",
                    "--plugin-name": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--type": [
                      "blob-sequence",
                      "scalar",
                      "tensor"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic",
                    "--uri": "bool"
                  }
                },
                "read": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--max-data-points": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--plugin-data": "value",
                    "--plugin-name": "value",
                    "--region": "dynamic",
                    "--tensorboard-experiment-id": "dynamic",
                    "--tensorboard-id": "dynamic",
                    "--tensorboard-run-id": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tensorboards": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ai-platform": {
          "commands": {
            "explain": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--text-instances": "value"
              }
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--summarize": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--task-name": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "prediction": {
                      "commands": {},
                      "flags": {
                        "--batch-size": "value",
                        "--data-format": [
                          "text",
                          "tf-record",
                          "tf-record-gzip"
                        ],
                        "--input-paths": "value",
                        "--labels": "value",
                        "--max-worker-count": "value",
                        "--model": "value",
                        "--model-dir": "value",
                        "--output-path": "value",
                        "--region": "value",
                        "--runtime-version": "value",
                        "--signature-name": "value"
                      }
                    },
                    "training": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config": "value",
                        "--enable-web-access": "bool",
                        "--job-dir": "value",
                        "--kms-key": "dynamic",
                        "--kms-keyring": "dynamic",
                        "--kms-location": "dynamic",
                        "--kms-project": "dynamic",
                        "--labels": "value",
                        "--master-accelerator": "value",
                        "--master-image-uri": "value",
                        "--master-machine-type": "value",
                        "--module-name": "value",
                        "--network": "value",
                        "--package-path": "value",
                        "--packages": "value",
                        "--parameter-server-accelerator": "value",
                        "--parameter-server-count": "value",
                        "--parameter-server-image-uri": "value",
                        "--parameter-server-machine-type": "value",
                        "--python-version": "value",
                        "--region": "dynamic",
                        "--runtime-version": "value",
                        "--scale-tier": [
                          "basic",
                          "basic-gpu",
                          "basic-tpu",
                          "custom",
                          "premium-1",
                          "standard-1"
                        ],
                        "--service-account": "value",
                        "--staging-bucket": "value",
                        "--stream-logs": "bool",
                        "--tpu-tf-version": "value",
                        "--use-chief-in-tf-config": "value",
                        "--worker-accelerator": "value",
                        "--worker-count": "value",
                        "--worker-image-uri": "value",
                        "--worker-machine-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "local": {
              "commands": {
                "predict": {
                  "commands": {},
                  "flags": {
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--json-instances": "value",
                    "--json-request": "value",
                    "--model-dir": "value",
                    "--signature-name": "value",
                    "--text-instances": "value"
                  }
                },
                "train": {
                  "commands": {},
                  "flags": {
                    "--distributed": "bool",
                    "--evaluator-count": "value",
                    "--job-dir": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--parameter-server-count": "value",
                    "--start-port": "value",
                    "--worker-count": "value"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-console-logging": "bool",
                    "--enable-logging": "bool",
                    "--labels": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--regions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "predict": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "versions": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--command": "value",
                    "--config": "value",
                    "--description": "value",
                    "--env-vars": "value",
                    "--explanation-method": [
                      "integrated-gradients",
                      "sampled-shapley",
                      "xrai"
                    ],
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--health-route": "value",
                    "--image": "value",
                    "--labels": "value",
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--metric-targets": "value",
                    "--min-nodes": "value",
                    "--model": "value",
                    "--num-integral-steps": "value",
                    "--num-paths": "value",
                    "--origin": "value",
                    "--package-uris": "value",
                    "--ports": "value",
                    "--predict-route": "value",
                    "--prediction-class": "value",
                    "--python-version": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--runtime-version": "value",
                    "--service-account": "value",
                    "--staging-bucket": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--model": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-default": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bigquery-table-name": "value",
                    "--clear-labels": "bool",
                    "--config": "value",
                    "--description": "value",
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--sampling-percentage": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "alloydb": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automated-backup-days-of-week": "value",
                    "--automated-backup-encryption-key": "dynamic",
                    "--automated-backup-encryption-key-keyring": "dynamic",
                    "--automated-backup-encryption-key-location": "dynamic",
                    "--automated-backup-encryption-key-project": "dynamic",
                    "--automated-backup-retention-count": "value",
                    "--automated-backup-retention-period": "value",
                    "--automated-backup-start-times": "value",
                    "--automated-backup-window": "value",
                    "--continuous-backup-encryption-key": "dynamic",
                    "--continuous-backup-encryption-key-keyring": "dynamic",
                    "--continuous-backup-encryption-key-location": "dynamic",
                    "--continuous-backup-encryption-key-project": "dynamic",
                    "--continuous-backup-recovery-window-days": "value",
                    "--disable-automated-backup": "bool",
                    "--enable-continuous-backup": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--network": "value",
                    "--password": "value",
                    "--region": "value"
                  }
                },
                "create-secondary": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--primary-cluster": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "promote": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--network": "value",
                    "--point-in-time": "value",
                    "--region": "value",
                    "--source-cluster": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automated-backup-days-of-week": "value",
                    "--automated-backup-encryption-key": "dynamic",
                    "--automated-backup-encryption-key-keyring": "dynamic",
                    "--automated-backup-encryption-key-location": "dynamic",
                    "--automated-backup-encryption-key-project": "dynamic",
                    "--automated-backup-retention-count": "value",
                    "--automated-backup-retention-period": "value",
                    "--automated-backup-start-times": "value",
                    "--automated-backup-window": "value",
                    "--clear-automated-backup": "bool",
                    "--clear-continuous-backup-encryption-key": "bool",
                    "--continuous-backup-encryption-key": "dynamic",
                    "--continuous-backup-encryption-key-keyring": "dynamic",
                    "--continuous-backup-encryption-key-location": "dynamic",
                    "--continuous-backup-encryption-key-project": "dynamic",
                    "--continuous-backup-recovery-window-days": "value",
                    "--disable-automated-backup": "bool",
                    "--enable-continuous-backup": "bool",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--availability-type": [
                      "REGIONAL"
                    ],
                    "--cluster": "value",
                    "--cpu-count": [
                      2,
                      4,
                      8,
                      16,
                      32,
                      64
                    ],
                    "--database-flags": "value",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--instance-type": [
                      "PRIMARY",
                      "READ_POOL"
                    ],
                    "--read-pool-node-count": "value",
                    "--region": "value"
                  }
                },
                "create-secondary": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--availability-type": [
                      "REGIONAL"
                    ],
                    "--cluster": "value",
                    "--cpu-count": [
                      2,
                      4,
                      8,
                      16,
                      32,
                      64
                    ],
                    "--database-flags": "value",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--read-pool-node-count": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "anthos": {
          "commands": {
            "apply": {
              "commands": {},
              "flags": {}
            },
            "auth": {
              "commands": {
                "login": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--dry-run": "bool",
                    "--kubeconfig": "value",
                    "--login-config": "value",
                    "--login-config-cert": "value",
                    "--set-preferred-auth": "bool",
                    "--user": "value"
                  }
                }
              },
              "flags": {}
            },
            "create-login-config": {
              "commands": {},
              "flags": {
                "--kubeconfig": "value",
                "--merge-from": "value",
                "--output": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--output-directory": "value"
              }
            }
          },
          "flags": {}
        },
        "api-gateway": {
          "commands": {
            "api-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool",
                    "--backend-auth-service-account": "value",
                    "--display-name": "value",
                    "--grpc-files": "value",
                    "--labels": "value",
                    "--openapi-spec": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--view": [
                      "BASIC",
                      "FULL"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "apis": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--managed-service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "gateways": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--api-config": "value",
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--api-config": "value",
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "apigee": {
          "commands": {
            "apis": {
              "commands": {
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value",
                    "--override": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--verbose": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "undeploy": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "applications": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--developer": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "archives": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bundle-file": "value",
                    "--environment": "value",
                    "--labels": "value",
                    "--organization": "value",
                    "--source": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--environment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--environment": "value",
                    "--organization": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "deployments": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--api": "value",
                    "--environment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--revision": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "developers": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "environments": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "organizations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "products": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--all-environments": "bool",
                    "--all-proxies": "bool",
                    "--apis": "value",
                    "--attributes": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--environments": "value",
                    "--internal-access": "bool",
                    "--manual-approval": "bool",
                    "--oauth-scopes": "value",
                    "--organization": "value",
                    "--private-access": "bool",
                    "--public-access": "bool",
                    "--quota": "value",
                    "--quota-interval": "value",
                    "--quota-unit": [
                      "day",
                      "hour",
                      "minute",
                      "month"
                    ],
                    "--resources": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-api": "value",
                    "--add-attribute": "value",
                    "--add-environment": "value",
                    "--add-oauth-scope": "value",
                    "--add-resource": "value",
                    "--all-apis": "bool",
                    "--all-environments": "bool",
                    "--all-resources": "bool",
                    "--automatic-approval": "bool",
                    "--clear-attributes": "bool",
                    "--clear-description": "bool",
                    "--clear-oauth-scopes": "bool",
                    "--clear-quota": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--internal-access": "bool",
                    "--manual-approval": "bool",
                    "--organization": "value",
                    "--private-access": "bool",
                    "--public-access": "bool",
                    "--quota": "value",
                    "--quota-interval": "value",
                    "--quota-unit": [
                      "day",
                      "hour",
                      "minute",
                      "month"
                    ],
                    "--remove-api": "value",
                    "--remove-attribute": "value",
                    "--remove-environment": "value",
                    "--remove-oauth-scope": "value",
                    "--remove-resource": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "app": {
          "commands": {
            "browse": {
              "commands": {},
              "flags": {
                "--launch-browser": "bool",
                "--service": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--service-account": "value"
              }
            },
            "deploy": {
              "commands": {},
              "flags": {
                "--appyaml": "value",
                "--bucket": "value",
                "--cache": "bool",
                "--ignore-file": "value",
                "--image-url": "value",
                "--promote": "bool",
                "--service-account": "value",
                "--stop-previous-version": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "domain-mappings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate-id": "value",
                    "--certificate-management": [
                      "automatic",
                      "manual"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate-id": "value",
                    "--certificate-management": [
                      "automatic",
                      "manual"
                    ],
                    "--no-certificate-id": "bool"
                  }
                }
              },
              "flags": {}
            },
            "firewall-rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "ALLOW",
                      "DENY"
                    ],
                    "--description": "value",
                    "--source-range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "test-ip": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "ALLOW",
                      "DENY"
                    ],
                    "--description": "value",
                    "--source-range": "value"
                  }
                }
              },
              "flags": {}
            },
            "gen-config": {
              "commands": {},
              "flags": {
                "--config": "value",
                "--custom": "bool",
                "--runtime": "value"
              }
            },
            "instances": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "disable-debug": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "enable-debug": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "scp": {
                  "commands": {},
                  "flags": {
                    "--compress": "bool",
                    "--recurse": "bool",
                    "--service": "value",
                    "--tunnel-through-iap": "bool"
                  }
                },
                "ssh": {
                  "commands": {},
                  "flags": {
                    "--container": "value",
                    "--service": "value",
                    "--tunnel-through-iap": "bool"
                  }
                }
              },
              "flags": {}
            },
            "logs": {
              "commands": {
                "read": {
                  "commands": {},
                  "flags": {
                    "--level": [
                      "any",
                      "critical",
                      "debug",
                      "error",
                      "info",
                      "warning"
                    ],
                    "--limit": "value",
                    "--logs": "value",
                    "--service": "value"
                  }
                },
                "tail": {
                  "commands": {},
                  "flags": {
                    "--level": [
                      "any",
                      "critical",
                      "debug",
                      "error",
                      "info",
                      "warning"
                    ],
                    "--logs": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "migrate-config": {
              "commands": {
                "cron-xml-to-yaml": {
                  "commands": {},
                  "flags": {}
                },
                "datastore-indexes-xml-to-yaml": {
                  "commands": {},
                  "flags": {
                    "--generated-indexes-file": "value"
                  }
                },
                "dispatch-xml-to-yaml": {
                  "commands": {},
                  "flags": {}
                },
                "queue-xml-to-yaml": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "open-console": {
              "commands": {},
              "flags": {
                "--logs": "bool",
                "--service": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--pending": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "repair": {
              "commands": {},
              "flags": {}
            },
            "services": {
              "commands": {
                "browse": {
                  "commands": {},
                  "flags": {
                    "--launch-browser": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-traffic": {
                  "commands": {},
                  "flags": {
                    "--migrate": "bool",
                    "--split-by": [
                      "cookie",
                      "ip",
                      "random"
                    ],
                    "--splits": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ingress": [
                      "all",
                      "internal-and-cloud-load-balancing",
                      "internal-only"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "ssl-certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--display-name": "value",
                    "--private-key": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--display-name": "value",
                    "--private-key": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--service-account": "value",
                "--split-health-checks": "bool"
              }
            },
            "versions": {
              "commands": {
                "browse": {
                  "commands": {},
                  "flags": {
                    "--launch-browser": "bool",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--hide-no-traffic": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "migrate": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "artifacts": {
          "commands": {
            "apt": {
              "commands": {
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--gcs-source": "value",
                    "--location": "value"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            },
            "docker": {
              "commands": {
                "images": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--delete-tags": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--metadata-filter": "value",
                        "--show-all-metadata": "bool",
                        "--show-build-details": "bool",
                        "--show-deployment": "bool",
                        "--show-image-basis": "bool",
                        "--show-package-vulnerability": "bool",
                        "--show-provenance": "bool"
                      }
                    },
                    "get-operation": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--include-tags": "bool",
                        "--limit": "value",
                        "--occurrence-filter": "value",
                        "--page-size": "value",
                        "--show-occurrences": "bool",
                        "--show-occurrences-from": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list-vulnerabilities": {
                      "commands": {},
                      "flags": {
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value"
                      }
                    },
                    "scan": {
                      "commands": {},
                      "flags": {
                        "--additional-package-types": "value",
                        "--async": "bool",
                        "--location": [
                          "asia",
                          "europe",
                          "us"
                        ],
                        "--remote": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "tags": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "files": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value",
                    "--tag": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "packages": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "print-settings": {
              "commands": {
                "apt": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "gradle": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "mvn": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "npm": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value",
                    "--scope": "value"
                  }
                },
                "python": {
                  "commands": {},
                  "flags": {
                    "--json-key": "value",
                    "--location": "value",
                    "--repository": "value"
                  }
                },
                "yum": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--repository": "value"
                  }
                }
              },
              "flags": {}
            },
            "repositories": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--allow-snapshot-overwrites": "bool",
                    "--async": "bool",
                    "--description": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--repository-format": "value",
                    "--version-policy": [
                      "none",
                      "release",
                      "snapshot"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable-upgrade-redirection": {
                  "commands": {},
                  "flags": {}
                },
                "enable-upgrade-redirection": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                }
              },
              "flags": {}
            },
            "versions": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--delete-tags": "bool",
                    "--location": "value",
                    "--package": "value",
                    "--repository": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--package": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpcsc-config": {
              "commands": {
                "allow": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "deny": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "yum": {
              "commands": {
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--gcs-source": "value",
                    "--location": "value"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "asset": {
          "commands": {
            "analyze-org-policies": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "analyze-org-policy-governed-assets": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "analyze-org-policy-governed-containers": {
              "commands": {},
              "flags": {
                "--constraint": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--asset-types": "value",
                "--bigquery-dataset": "dynamic",
                "--bigquery-table": "dynamic",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--folder": "value",
                "--organization": "value",
                "--output-bigquery-force": "bool",
                "--output-path": "value",
                "--output-path-prefix": "value",
                "--partition-key": [
                  "read-time",
                  "request-time"
                ],
                "--per-asset-type": "bool",
                "--relationship-types": "value",
                "--snapshot-time": "value"
              }
            },
            "feeds": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--asset-names": "value",
                    "--asset-types": "value",
                    "--condition-description": "value",
                    "--condition-expression": "value",
                    "--condition-title": "value",
                    "--content-type": [
                      "access-policy",
                      "iam-policy",
                      "org-policy",
                      "os-inventory",
                      "relationship",
                      "resource"
                    ],
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value",
                    "--relationship-types": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-asset-names": "value",
                    "--add-asset-types": "value",
                    "--add-relationship-types": "value",
                    "--clear-asset-names": "bool",
                    "--clear-asset-types": "bool",
                    "--clear-condition-description": "bool",
                    "--clear-condition-expression": "bool",
                    "--clear-condition-title": "bool",
                    "--clear-content-type": "bool",
                    "--clear-relationship-types": "bool",
                    "--condition-description": "value",
                    "--condition-expression": "value",
                    "--condition-title": "value",
                    "--content-type": [
                      "access-policy",
                      "iam-policy",
                      "org-policy",
                      "os-inventory",
                      "relationship",
                      "resource"
                    ],
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value",
                    "--remove-asset-names": "value",
                    "--remove-asset-types": "value",
                    "--remove-relationship-types": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-history": {
              "commands": {},
              "flags": {
                "--asset-names": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--end-time": "value",
                "--organization": "value",
                "--relationship-types": "value",
                "--start-time": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--asset-types": "value",
                "--content-type": [
                  "access-policy",
                  "iam-policy",
                  "org-policy",
                  "os-inventory",
                  "relationship",
                  "resource"
                ],
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--relationship-types": "value",
                "--snapshot-time": "value",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "query": {
              "commands": {},
              "flags": {
                "--bigquery-dataset": "dynamic",
                "--bigquery-table": "dynamic",
                "--end-time": "value",
                "--folder": "value",
                "--job-reference": "value",
                "--organization": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--snapshot-time": "value",
                "--start-time": "value",
                "--statement": "value",
                "--timeout": "value",
                "--write-disposition": [
                  "write-append",
                  "write-empty",
                  "write-truncate"
                ]
              }
            },
            "saved-queries": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--folder": "value",
                    "--labels": "value",
                    "--organization": "value",
                    "--query-file-path": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--query-file-path": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "search-all-iam-policies": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--query": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            },
            "search-all-resources": {
              "commands": {},
              "flags": {
                "--asset-types": "value",
                "--filter": "value",
                "--limit": "value",
                "--order-by": "value",
                "--page-size": "value",
                "--query": "value",
                "--scope": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "assured": {
          "commands": {
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "workloads": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--compliance-regime": [
                      "ASSURED_WORKLOADS_FOR_PARTNERS",
                      "CA_REGIONS_AND_SUPPORT",
                      "CJIS",
                      "EU_REGIONS_AND_SUPPORT",
                      "FEDRAMP_HIGH",
                      "FEDRAMP_MODERATE",
                      "HIPAA",
                      "HITRUST",
                      "IL4",
                      "ITAR",
                      "US_REGIONAL_ACCESS"
                    ],
                    "--display-name": "value",
                    "--enable-sovereign-controls": "value",
                    "--external-identifier": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--next-rotation-time": "value",
                    "--organization": "value",
                    "--partner": [
                      "LOCAL_CONTROLS_BY_S3NS",
                      "SOVEREIGN_CONTROLS_BY_T_SYSTEMS"
                    ],
                    "--provisioned-resources-parent": "value",
                    "--resource-settings": "value",
                    "--rotation-period": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "violations": {
                  "commands": {
                    "acknowledge": {
                      "commands": {},
                      "flags": {
                        "--comment": "value",
                        "--location": "value",
                        "--organization": "value",
                        "--workload": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--organization": "value",
                        "--workload": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--workload": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "auth": {
          "commands": {
            "activate-service-account": {
              "commands": {},
              "flags": {
                "--key-file": "value",
                "--password-file": "value",
                "--prompt-for-password": "bool"
              }
            },
            "application-default": {
              "commands": {
                "login": {
                  "commands": {},
                  "flags": {
                    "--client-id-file": "value",
                    "--disable-quota-project": "bool",
                    "--launch-browser": "bool",
                    "--login-config": "value",
                    "--no-browser": "bool",
                    "--scopes": "value"
                  }
                },
                "print-access-token": {
                  "commands": {},
                  "flags": {
                    "--scopes": "value"
                  }
                },
                "revoke": {
                  "commands": {},
                  "flags": {}
                },
                "set-quota-project": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "configure-docker": {
              "commands": {},
              "flags": {}
            },
            "enterprise-certificate-config": {
              "commands": {
                "create": {
                  "commands": {
                    "linux": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--label": "value",
                        "--module": "value",
                        "--output-file": "value",
                        "--slot": "value",
                        "--tls-offload": "value",
                        "--user-pin": "value"
                      }
                    },
                    "macos": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--issuer": "value",
                        "--output-file": "value",
                        "--tls-offload": "value"
                      }
                    },
                    "windows": {
                      "commands": {},
                      "flags": {
                        "--ecp": "value",
                        "--ecp-client": "value",
                        "--issuer": "value",
                        "--output-file": "value",
                        "--provider": "value",
                        "--store": "value",
                        "--tls-offload": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--filter-account": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "login": {
              "commands": {},
              "flags": {
                "--activate": "bool",
                "--brief": "bool",
                "--cred-file": "value",
                "--enable-gdrive-access": "bool",
                "--force": "bool",
                "--launch-browser": "bool",
                "--login-config": "value",
                "--no-browser": "bool",
                "--update-adc": "bool"
              }
            },
            "print-access-token": {
              "commands": {},
              "flags": {}
            },
            "print-identity-token": {
              "commands": {},
              "flags": {
                "--audiences": "value",
                "--include-email": "bool",
                "--include-license": "bool",
                "--token-format": [
                  "full",
                  "standard"
                ]
              }
            },
            "revoke": {
              "commands": {},
              "flags": {
                "--all": "bool"
              }
            }
          },
          "flags": {}
        },
        "batch": {
          "commands": {
            "jobs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "submit": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--container-commands-file": "value",
                    "--container-entrypoint": "value",
                    "--container-image-uri": "value",
                    "--location": "value",
                    "--machine-type": "value",
                    "--network": "value",
                    "--priority": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--script-file-path": "value",
                    "--script-text": "value",
                    "--subnetwork": "value"
                  }
                }
              },
              "flags": {}
            },
            "tasks": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--job": "value",
                    "--location": "value",
                    "--task_group": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--job": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "beyondcorp": {
          "commands": {
            "app": {
              "commands": {
                "connections": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--application-endpoint": "value",
                        "--application-endpoint-host": "value",
                        "--application-endpoint-port": "value",
                        "--async": "bool",
                        "--connectors": "value",
                        "--display-name": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--type": [
                          "tcp"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--application-endpoint": "value",
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--connectors": "value",
                        "--display-name": "value",
                        "--location": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "connectors": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--display-name": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--member": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--details": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--display-name": "value",
                        "--location": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "client-connector": {
              "commands": {
                "gateways": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--client-connector-service": "value",
                        "--location": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "services": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config-from-file": "value",
                        "--display-name": "value",
                        "--egress-peered-vpc": "value",
                        "--ingress-config": "value",
                        "--location": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config-from-file": "value",
                        "--display-name": "value",
                        "--ingress-config": "value",
                        "--location": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "bigtable": {
          "commands": {
            "app-profiles": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--force": "bool",
                    "--instance": "dynamic",
                    "--restrict-to": "value",
                    "--route-any": "bool",
                    "--route-to": "dynamic",
                    "--transactional-writes": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--force": "bool",
                    "--instance": "dynamic",
                    "--restrict-to": "value",
                    "--route-any": "bool",
                    "--route-to": "dynamic",
                    "--transactional-writes": "bool"
                  }
                }
              },
              "flags": {}
            },
            "backups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "dynamic",
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value",
                    "--table": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--cluster": "dynamic",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--instance": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--cluster": "dynamic",
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-cpu-target": "value",
                    "--autoscaling-max-nodes": "value",
                    "--autoscaling-min-nodes": "value",
                    "--autoscaling-storage-target": "value",
                    "--instance": "dynamic",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--num-nodes": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instances": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-cpu-target": "value",
                    "--autoscaling-max-nodes": "value",
                    "--autoscaling-min-nodes": "value",
                    "--autoscaling-storage-target": "value",
                    "--disable-autoscaling": "bool",
                    "--instance": "dynamic",
                    "--num-nodes": "value"
                  }
                }
              },
              "flags": {}
            },
            "hot-tablets": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--end-time": "value",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--start-time": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "dynamic",
                    "--cluster-config": "value",
                    "--cluster-num-nodes": "value",
                    "--cluster-storage-type": [
                      "hdd",
                      "ssd"
                    ],
                    "--cluster-zone": "value",
                    "--display-name": "value",
                    "--instance-type": [
                      "DEVELOPMENT",
                      "PRODUCTION"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "tables": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--instance": "dynamic",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--column-families": "value",
                        "--deletion-protection": "bool",
                        "--instance": "dynamic",
                        "--splits": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic",
                        "--view": [
                          "encryption",
                          "full",
                          "name",
                          "replication",
                          "schema",
                          "stats"
                        ]
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instances": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--instance": "dynamic",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "restore": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--destination": "dynamic",
                        "--destination-instance": "dynamic",
                        "--source": "dynamic",
                        "--source-cluster": "dynamic",
                        "--source-instance": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deletion-protection": "bool",
                        "--instance": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "billing": {
          "commands": {
            "accounts": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "budgets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--all-updates-rule-monitoring-notification-channels": "value",
                    "--all-updates-rule-pubsub-topic": "value",
                    "--billing-account": "value",
                    "--budget-amount": "value",
                    "--calendar-period": [
                      "calendar-period-unspecified",
                      "month",
                      "quarter",
                      "year"
                    ],
                    "--credit-types-treatment": [
                      "credit-types-treatment-unspecified",
                      "exclude-all-credits",
                      "include-all-credits",
                      "include-specified-credits"
                    ],
                    "--disable-default-iam-recipients": "bool",
                    "--display-name": "value",
                    "--end-date": "value",
                    "--filter-credit-types": "value",
                    "--filter-labels": "value",
                    "--filter-projects": "value",
                    "--filter-services": "value",
                    "--filter-subaccounts": "value",
                    "--last-period-amount": "bool",
                    "--start-date": "value",
                    "--threshold-rule": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-threshold-rule": "value",
                    "--all-updates-rule-monitoring-notification-channels": "value",
                    "--all-updates-rule-pubsub-topic": "value",
                    "--billing-account": "value",
                    "--budget-amount": "value",
                    "--calendar-period": [
                      "calendar-period-unspecified",
                      "month",
                      "quarter",
                      "year"
                    ],
                    "--clear-threshold-rules": "bool",
                    "--credit-types-treatment": [
                      "credit-types-treatment-unspecified",
                      "exclude-all-credits",
                      "include-all-credits",
                      "include-specified-credits"
                    ],
                    "--disable-default-iam-recipients": "bool",
                    "--display-name": "value",
                    "--end-date": "value",
                    "--filter-credit-types": "value",
                    "--filter-labels": "value",
                    "--filter-projects": "value",
                    "--filter-services": "value",
                    "--filter-subaccounts": "value",
                    "--last-period-amount": "bool",
                    "--start-date": "value",
                    "--threshold-rules-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "projects": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "link": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "unlink": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "builds": {
          "commands": {
            "approve": {
              "commands": {},
              "flags": {
                "--comment": "value",
                "--url": "value"
              }
            },
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "connections": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {
                    "github": {
                      "commands": {},
                      "flags": {
                        "--app-installation-id": "value",
                        "--async": "bool",
                        "--authorizer-token-secret-version": "value",
                        "--region": "dynamic"
                      }
                    },
                    "github-enterprise": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--app-installation-id": "value",
                        "--app-slug": "value",
                        "--async": "bool",
                        "--host-uri": "value",
                        "--private-key-secret-version": "value",
                        "--region": "dynamic",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {
                    "github": {
                      "commands": {},
                      "flags": {
                        "--app-installation-id": "value",
                        "--async": "bool",
                        "--authorizer-token-secret-version": "value",
                        "--region": "dynamic"
                      }
                    },
                    "github-enterprise": {
                      "commands": {},
                      "flags": {
                        "--app-id": "value",
                        "--app-installation-id": "value",
                        "--app-slug": "value",
                        "--async": "bool",
                        "--host-uri": "value",
                        "--private-key-secret-version": "value",
                        "--region": "dynamic",
                        "--service-directory-service": "value",
                        "--ssl-ca-file": "value",
                        "--webhook-secret-secret-version": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--ongoing": "bool",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "log": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--stream": "bool"
              }
            },
            "reject": {
              "commands": {},
              "flags": {
                "--comment": "value",
                "--url": "value"
              }
            },
            "repositories": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--connection": "dynamic",
                    "--region": "dynamic",
                    "--remote-uri": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--connection": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--connection": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--connection": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "submit": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--config": "value",
                "--disk-size": "value",
                "--gcs-log-dir": "value",
                "--gcs-source-staging-dir": "value",
                "--ignore-file": "value",
                "--machine-type": [
                  "e2-highcpu-32",
                  "e2-highcpu-8",
                  "n1-highcpu-32",
                  "n1-highcpu-8"
                ],
                "--no-cache": "bool",
                "--no-source": "bool",
                "--pack": "value",
                "--region": "value",
                "--substitutions": "value",
                "--suppress-logs": "bool",
                "--tag": "value",
                "--timeout": "value",
                "--worker-pool": "value"
              }
            },
            "triggers": {
              "commands": {
                "create": {
                  "commands": {
                    "bitbucketserver": {
                      "commands": {},
                      "flags": {
                        "--bitbucket-server-config-resource": "value",
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--project-key": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--repo-slug": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "cloud-source-repositories": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "github": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": [
                          "COMMENTS_DISABLED",
                          "COMMENTS_ENABLED",
                          "COMMENTS_ENABLED_FOR_EXTERNAL_CONTRIBUTORS_ONLY"
                        ],
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--enterprise-config": "value",
                        "--ignored-files": "value",
                        "--include-logs-with-status": "bool",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--repo-name": "value",
                        "--repo-owner": "value",
                        "--repository": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "gitlab-enterprise": {
                      "commands": {},
                      "flags": {
                        "--branch-pattern": "value",
                        "--build-config": "value",
                        "--comment-control": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--gitlab-config-resource": "value",
                        "--ignored-files": "value",
                        "--included-files": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--project-namespace": "value",
                        "--pull-request-pattern": "value",
                        "--region": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag-pattern": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "manual": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--service-account": "value",
                        "--subscription-filter": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--topic": "value",
                        "--trigger-config": "value"
                      }
                    },
                    "webhook": {
                      "commands": {},
                      "flags": {
                        "--branch": "value",
                        "--build-config": "value",
                        "--description": "value",
                        "--dockerfile": "value",
                        "--dockerfile-dir": "value",
                        "--dockerfile-image": "value",
                        "--github-enterprise-config": "value",
                        "--inline-config": "value",
                        "--name": "value",
                        "--region": "value",
                        "--repo": "value",
                        "--repo-type": "value",
                        "--require-approval": "bool",
                        "--secret": "value",
                        "--service-account": "value",
                        "--subscription-filter": "value",
                        "--substitutions": "value",
                        "--tag": "value",
                        "--trigger-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--branch": "value",
                    "--region": "value",
                    "--sha": "value",
                    "--tag": "value"
                  }
                }
              },
              "flags": {}
            },
            "worker-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--config-from-file": "value",
                    "--no-public-egress": "bool",
                    "--peered-network": "value",
                    "--peered-network-ip-range": "value",
                    "--region": "value",
                    "--worker-disk-size": "value",
                    "--worker-machine-type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config-from-file": "value",
                    "--no-public-egress": "bool",
                    "--public-egress": "bool",
                    "--region": "value",
                    "--worker-disk-size": "value",
                    "--worker-machine-type": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "certificate-manager": {
          "commands": {
            "certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-file": "value",
                    "--description": "value",
                    "--dns-authorizations": "value",
                    "--domains": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--private-key-file": "value",
                    "--scope": [
                      "default",
                      "edge-cache"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificate-file": "value",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "dynamic",
                    "--private-key-file": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "dns-authorizations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--domain": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "issuance-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--ca-pool": "value",
                    "--description": "value",
                    "--key-algorithm": [
                      "ecdsa-p256",
                      "rsa-2048"
                    ],
                    "--labels": "value",
                    "--lifetime": "value",
                    "--location": "value",
                    "--rotation-window-percentage": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "maps": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "entries": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--certificates": "dynamic",
                        "--description": "value",
                        "--hostname": "value",
                        "--labels": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--set-primary": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--map": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--map": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--certificates": "dynamic",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--location": "dynamic",
                        "--map": "dynamic",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "cloud-shell": {
          "commands": {
            "get-mount-command": {
              "commands": {},
              "flags": {
                "--force-key-file-overwrite": "bool",
                "--ssh-key-file": "bool"
              }
            },
            "scp": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--recurse": "bool",
                "--scp-flag": "value",
                "--ssh-key-file": "bool"
              }
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--authorize-session": "bool",
                "--command": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--ssh-flag": "value",
                "--ssh-key-file": "bool"
              }
            }
          },
          "flags": {}
        },
        "code": {
          "commands": {
            "clean-up": {
              "commands": {},
              "flags": {
                "--minikube-profile": "value"
              }
            },
            "dev": {
              "commands": {},
              "flags": {
                "--allow-secret-manager": "bool",
                "--application-default-credential": "bool",
                "--builder": "value",
                "--dockerfile": "value",
                "--kube-context": "value",
                "--local-port": "value",
                "--minikube-profile": "value",
                "--secrets": "value",
                "--service-account": "value",
                "--source": "value",
                "--stop-cluster": "bool"
              }
            }
          },
          "flags": {}
        },
        "composer": {
          "commands": {
            "environments": {
              "commands": {
                "check-upgrade": {
                  "commands": {},
                  "flags": {
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--image-version": "value",
                    "--location": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--airflow-configs": "value",
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--cloud-sql-ipv4-cidr": "value",
                    "--cloud-sql-machine-type": "value",
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--composer-network-ipv4-cidr": "value",
                    "--connection-subnetwork": "value",
                    "--connection-type": [
                      "connection-type-unspecified",
                      "private-service-connect",
                      "vpc-peering"
                    ],
                    "--disk-size": "value",
                    "--enable-cloud-data-lineage-integration": "bool",
                    "--enable-ip-alias": "bool",
                    "--enable-ip-masq-agent": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-environment": "bool",
                    "--enable-privately-used-public-ips": "bool",
                    "--enable-scheduled-snapshot-creation": "bool",
                    "--enable-triggerer": "bool",
                    "--env-variables": "value",
                    "--environment-size": [
                      "large",
                      "medium",
                      "small",
                      "unspecified"
                    ],
                    "--image-version": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--location": "value",
                    "--machine-type": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--max-pods-per-node": "value",
                    "--max-workers": "value",
                    "--min-workers": "value",
                    "--network": "value",
                    "--node-count": "value",
                    "--oauth-scopes": "value",
                    "--python-version": [
                      "2",
                      "3"
                    ],
                    "--scheduler-count": "value",
                    "--scheduler-cpu": "value",
                    "--scheduler-memory": "value",
                    "--scheduler-storage": "value",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--snapshot-creation-schedule": "value",
                    "--snapshot-location": "value",
                    "--snapshot-schedule-timezone": "value",
                    "--subnetwork": "value",
                    "--tags": "value",
                    "--triggerer-count": "value",
                    "--triggerer-cpu": "value",
                    "--triggerer-memory": "value",
                    "--web-server-allow-all": "bool",
                    "--web-server-allow-ip": "value",
                    "--web-server-cpu": "value",
                    "--web-server-deny-all": "bool",
                    "--web-server-ipv4-cidr": "value",
                    "--web-server-machine-type": "value",
                    "--web-server-memory": "value",
                    "--web-server-storage": "value",
                    "--worker-cpu": "value",
                    "--worker-memory": "value",
                    "--worker-storage": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--locations": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-packages": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--tree": "bool"
                  }
                },
                "list-upgrades": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "restart-web-server": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "snapshots": {
                  "commands": {
                    "load": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--skip-airflow-overrides-setting": "bool",
                        "--skip-environment-variables-setting": "bool",
                        "--skip-gcs-data-copying": "bool",
                        "--skip-pypi-packages-installation": "bool",
                        "--snapshot-path": "value"
                      }
                    },
                    "save": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--snapshot-location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "storage": {
                  "commands": {
                    "dags": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "data": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "plugins": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        },
                        "export": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--destination": "value",
                            "--environment": "value",
                            "--location": "value",
                            "--source": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--environment": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--airflow-version": "value",
                    "--async": "bool",
                    "--clear-airflow-configs": "bool",
                    "--clear-env-variables": "bool",
                    "--clear-labels": "bool",
                    "--clear-pypi-packages": "bool",
                    "--cloud-sql-machine-type": "value",
                    "--disable-cloud-data-lineage-integration": "bool",
                    "--disable-master-authorized-networks": "bool",
                    "--disable-scheduled-snapshot-creation": "bool",
                    "--disable-triggerer": "bool",
                    "--enable-cloud-data-lineage-integration": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-scheduled-snapshot-creation": "bool",
                    "--enable-triggerer": "bool",
                    "--environment-size": [
                      "large",
                      "medium",
                      "small",
                      "unspecified"
                    ],
                    "--image-version": "value",
                    "--location": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--max-workers": "value",
                    "--min-workers": "value",
                    "--node-count": "value",
                    "--remove-airflow-configs": "value",
                    "--remove-env-variables": "value",
                    "--remove-labels": "value",
                    "--remove-pypi-packages": "value",
                    "--scheduler-count": "value",
                    "--scheduler-cpu": "value",
                    "--scheduler-memory": "value",
                    "--scheduler-storage": "value",
                    "--snapshot-creation-schedule": "value",
                    "--snapshot-location": "value",
                    "--snapshot-schedule-timezone": "value",
                    "--triggerer-count": "value",
                    "--triggerer-cpu": "value",
                    "--triggerer-memory": "value",
                    "--update-airflow-configs": "value",
                    "--update-env-variables": "value",
                    "--update-labels": "value",
                    "--update-pypi-package": "value",
                    "--update-pypi-packages-from-file": "value",
                    "--update-web-server-allow-ip": "value",
                    "--web-server-allow-all": "bool",
                    "--web-server-cpu": "value",
                    "--web-server-deny-all": "bool",
                    "--web-server-machine-type": "value",
                    "--web-server-memory": "value",
                    "--web-server-storage": "value",
                    "--worker-cpu": "value",
                    "--worker-memory": "value",
                    "--worker-storage": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--locations": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "compute": {
          "commands": {
            "accelerator-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "addresses": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--addresses": "value",
                    "--description": "value",
                    "--endpoint-type": [
                      "NETLB",
                      "VM"
                    ],
                    "--global": "bool",
                    "--ip-version": [
                      "IPV4",
                      "IPV6"
                    ],
                    "--network": "value",
                    "--network-tier": "value",
                    "--prefix-length": "value",
                    "--purpose": [
                      "GCE_ENDPOINT",
                      "IPSEC_INTERCONNECT",
                      "PRIVATE_SERVICE_CONNECT",
                      "SHARED_LOADBALANCER_VIP",
                      "VPC_PEERING"
                    ],
                    "--region": "dynamic",
                    "--subnet": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "backend-buckets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-file": "value",
                    "--key-name": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--enable-cdn": "bool",
                    "--gcs-bucket-name": "dynamic",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--signed-url-cache-max-age": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "delete-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-name": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--edge-security-policy": "dynamic",
                    "--enable-cdn": "bool",
                    "--gcs-bucket-name": "dynamic",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--no-bypass-cache-on-request-headers": "bool",
                    "--no-client-ttl": "bool",
                    "--no-custom-response-headers": "bool",
                    "--no-default-ttl": "bool",
                    "--no-max-ttl": "bool",
                    "--no-negative-caching-policies": "bool",
                    "--no-serve-while-stale": "bool",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--signed-url-cache-max-age": "value"
                  }
                }
              },
              "flags": {}
            },
            "backend-services": {
              "commands": {
                "add-backend": {
                  "commands": {},
                  "flags": {
                    "--balancing-mode": [
                      "CONNECTION",
                      "RATE",
                      "UTILIZATION"
                    ],
                    "--capacity-scaler": "value",
                    "--description": "value",
                    "--failover": "bool",
                    "--global": "bool",
                    "--global-network-endpoint-group": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--max-connections": "value",
                    "--max-connections-per-endpoint": "value",
                    "--max-connections-per-instance": "value",
                    "--max-rate": "value",
                    "--max-rate-per-endpoint": "value",
                    "--max-rate-per-instance": "value",
                    "--max-utilization": "value",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-region": "dynamic",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "value"
                  }
                },
                "add-service-bindings": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--service-bindings": "dynamic"
                  }
                },
                "add-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-file": "value",
                    "--key-name": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--affinity-cookie-ttl": "value",
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-host": "bool",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-include-named-cookie": "value",
                    "--cache-key-include-protocol": "bool",
                    "--cache-key-include-query-string": "bool",
                    "--cache-key-query-string-blacklist": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--connection-drain-on-failover": "bool",
                    "--connection-draining-timeout": "value",
                    "--connection-persistence-on-unhealthy-backends": [
                      "ALWAYS_PERSIST",
                      "DEFAULT_FOR_PROTOCOL",
                      "NEVER_PERSIST"
                    ],
                    "--custom-request-header": "value",
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--drop-traffic-if-unhealthy": "bool",
                    "--enable-cdn": "bool",
                    "--enable-logging": "bool",
                    "--enable-strong-affinity": "bool",
                    "--failover-ratio": "value",
                    "--global": "bool",
                    "--global-health-checks": "bool",
                    "--health-checks": "dynamic",
                    "--health-checks-region": "dynamic",
                    "--http-health-checks": "dynamic",
                    "--https-health-checks": "dynamic",
                    "--iap": "value",
                    "--idle-timeout-sec": "value",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--locality-lb-policy": [
                      "INVALID_LB_POLICY",
                      "LEAST_REQUEST",
                      "MAGLEV",
                      "ORIGINAL_DESTINATION",
                      "RANDOM",
                      "RING_HASH",
                      "ROUND_ROBIN",
                      "WEIGHTED_MAGLEV"
                    ],
                    "--logging-optional": [
                      "CUSTOM",
                      "EXCLUDE_ALL_OPTIONAL",
                      "INCLUDE_ALL_OPTIONAL"
                    ],
                    "--logging-optional-fields": "value",
                    "--logging-sample-rate": "value",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--network": "value",
                    "--port-name": "value",
                    "--protocol": "value",
                    "--region": "dynamic",
                    "--request-coalescing": "bool",
                    "--serve-while-stale": "value",
                    "--service-bindings": "dynamic",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_NO_DESTINATION",
                      "CLIENT_IP_PORT_PROTO",
                      "CLIENT_IP_PROTO",
                      "GENERATED_COOKIE",
                      "HEADER_FIELD",
                      "HTTP_COOKIE",
                      "NONE"
                    ],
                    "--signed-url-cache-max-age": "value",
                    "--subsetting-policy": [
                      "CONSISTENT_HASH_SUBSETTING",
                      "NONE"
                    ],
                    "--subsetting-subset-size": "value",
                    "--timeout": "value",
                    "--tracking-mode": [
                      "PER_CONNECTION",
                      "PER_SESSION"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete-signed-url-key": {
                  "commands": {},
                  "flags": {
                    "--key-name": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "edit": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "get-health": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-backend": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--global-network-endpoint-group": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-region": "dynamic",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "value"
                  }
                },
                "remove-service-bindings": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--service-bindings": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--affinity-cookie-ttl": "value",
                    "--bypass-cache-on-request-headers": "value",
                    "--cache-key-include-host": "bool",
                    "--cache-key-include-http-header": "value",
                    "--cache-key-include-named-cookie": "value",
                    "--cache-key-include-protocol": "bool",
                    "--cache-key-include-query-string": "bool",
                    "--cache-key-query-string-blacklist": "value",
                    "--cache-key-query-string-whitelist": "value",
                    "--cache-mode": [
                      "CACHE_ALL_STATIC",
                      "FORCE_CACHE_ALL",
                      "USE_ORIGIN_HEADERS"
                    ],
                    "--client-ttl": "value",
                    "--compression-mode": [
                      "AUTOMATIC",
                      "DISABLED"
                    ],
                    "--connection-drain-on-failover": "bool",
                    "--connection-draining-timeout": "value",
                    "--connection-persistence-on-unhealthy-backends": [
                      "ALWAYS_PERSIST",
                      "DEFAULT_FOR_PROTOCOL",
                      "NEVER_PERSIST"
                    ],
                    "--custom-request-header": "value",
                    "--custom-response-header": "value",
                    "--default-ttl": "value",
                    "--description": "value",
                    "--drop-traffic-if-unhealthy": "bool",
                    "--edge-security-policy": "dynamic",
                    "--enable-cdn": "bool",
                    "--enable-logging": "bool",
                    "--enable-strong-affinity": "bool",
                    "--failover-ratio": "value",
                    "--global": "bool",
                    "--global-health-checks": "bool",
                    "--health-checks": "dynamic",
                    "--health-checks-region": "dynamic",
                    "--http-health-checks": "dynamic",
                    "--https-health-checks": "dynamic",
                    "--iap": "value",
                    "--idle-timeout-sec": "value",
                    "--locality-lb-policy": [
                      "INVALID_LB_POLICY",
                      "LEAST_REQUEST",
                      "MAGLEV",
                      "ORIGINAL_DESTINATION",
                      "RANDOM",
                      "RING_HASH",
                      "ROUND_ROBIN",
                      "WEIGHTED_MAGLEV"
                    ],
                    "--logging-optional": [
                      "CUSTOM",
                      "EXCLUDE_ALL_OPTIONAL",
                      "INCLUDE_ALL_OPTIONAL"
                    ],
                    "--logging-optional-fields": "value",
                    "--logging-sample-rate": "value",
                    "--max-ttl": "value",
                    "--negative-caching": "bool",
                    "--negative-caching-policy": "value",
                    "--no-bypass-cache-on-request-headers": "bool",
                    "--no-client-ttl": "bool",
                    "--no-custom-request-headers": "bool",
                    "--no-custom-response-headers": "bool",
                    "--no-default-ttl": "bool",
                    "--no-health-checks": "bool",
                    "--no-max-ttl": "bool",
                    "--no-negative-caching-policies": "bool",
                    "--no-serve-while-stale": "bool",
                    "--no-service-bindings": "bool",
                    "--port-name": "value",
                    "--protocol": "value",
                    "--region": "dynamic",
                    "--request-coalescing": "bool",
                    "--security-policy": "dynamic",
                    "--serve-while-stale": "value",
                    "--service-bindings": "dynamic",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_NO_DESTINATION",
                      "CLIENT_IP_PORT_PROTO",
                      "CLIENT_IP_PROTO",
                      "GENERATED_COOKIE",
                      "HEADER_FIELD",
                      "HTTP_COOKIE",
                      "NONE"
                    ],
                    "--signed-url-cache-max-age": "value",
                    "--subsetting-policy": [
                      "CONSISTENT_HASH_SUBSETTING",
                      "NONE"
                    ],
                    "--subsetting-subset-size": "value",
                    "--timeout": "value",
                    "--tracking-mode": [
                      "PER_CONNECTION",
                      "PER_SESSION"
                    ]
                  }
                },
                "update-backend": {
                  "commands": {},
                  "flags": {
                    "--balancing-mode": [
                      "CONNECTION",
                      "RATE",
                      "UTILIZATION"
                    ],
                    "--capacity-scaler": "value",
                    "--description": "value",
                    "--failover": "bool",
                    "--global": "bool",
                    "--instance-group": "dynamic",
                    "--instance-group-region": "dynamic",
                    "--instance-group-zone": "dynamic",
                    "--max-connections": "value",
                    "--max-connections-per-endpoint": "value",
                    "--max-connections-per-instance": "value",
                    "--max-rate": "value",
                    "--max-rate-per-endpoint": "value",
                    "--max-rate-per-instance": "value",
                    "--max-utilization": "value",
                    "--network-endpoint-group": "value",
                    "--network-endpoint-group-zone": "dynamic",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "commitments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--auto-renew": "bool",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--maintenance-interval": [
                      "PERIODIC",
                      "RECURRENT"
                    ],
                    "--merge-source-commitments": "value",
                    "--min-cpu-platform": "value",
                    "--plan": [
                      "12-month",
                      "36-month"
                    ],
                    "--region": "dynamic",
                    "--require-specific-reservation": "bool",
                    "--reservation": "dynamic",
                    "--reservation-zone": "dynamic",
                    "--reservations-from-file": "value",
                    "--resources": "value",
                    "--resources-accelerator": "value",
                    "--share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--split-source-commitment": "value",
                    "--type": [
                      "accelerator-optimized",
                      "compute-optimized",
                      "compute-optimized-c2d",
                      "compute-optimized-c3",
                      "general-purpose",
                      "general-purpose-e2",
                      "general-purpose-n2",
                      "general-purpose-n2d",
                      "general-purpose-t2d",
                      "memory-optimized",
                      "memory-optimized-m3"
                    ],
                    "--vm-count": "value"
                  }
                },
                "create-license": {
                  "commands": {},
                  "flags": {
                    "--amount": "value",
                    "--cores-per-license": "value",
                    "--license": "value",
                    "--plan": [
                      "12-month",
                      "36-month"
                    ],
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--auto-renew": "bool",
                    "--plan": [
                      "36-month"
                    ],
                    "--region": "dynamic"
                  }
                },
                "update-reservations": {
                  "commands": {},
                  "flags": {
                    "--dest-accelerator": "value",
                    "--dest-local-ssd": "value",
                    "--dest-reservation": "value",
                    "--dest-share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--dest-share-with": "value",
                    "--region": "dynamic",
                    "--reservations-from-file": "value",
                    "--source-accelerator": "value",
                    "--source-local-ssd": "value",
                    "--source-reservation": "value",
                    "--source-share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--source-share-with": "value"
                  }
                }
              },
              "flags": {}
            },
            "config-ssh": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--remove": "bool",
                "--ssh-config-file": "value",
                "--ssh-key-file": "value"
              }
            },
            "connect-to-serial-port": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--extra-args": "value",
                "--force-key-file-overwrite": "bool",
                "--port": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--zone": "dynamic"
              }
            },
            "copy-files": {
              "commands": {},
              "flags": {
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--plain": "bool",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--zone": "value"
              }
            },
            "diagnose": {
              "commands": {
                "export-logs": {
                  "commands": {},
                  "flags": {
                    "--collect-process-traces": "bool",
                    "--zone": "dynamic"
                  }
                },
                "routes": {
                  "commands": {},
                  "flags": {
                    "--container": "value",
                    "--dry-run": "bool",
                    "--external-route-ip": "value",
                    "--force-key-file-overwrite": "bool",
                    "--plain": "bool",
                    "--regexp": "value",
                    "--reverse-traceroute": "bool",
                    "--ssh-flag": "value",
                    "--ssh-key-expiration": "value",
                    "--ssh-key-expire-after": "value",
                    "--ssh-key-file": "value",
                    "--strict-host-key-checking": [
                      "ask",
                      "no",
                      "yes"
                    ],
                    "--user": "value",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "disk-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            },
            "disks": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "add-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--erase-windows-vss-signature": "bool",
                    "--guest-os-features": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-family-scope": [
                      "global",
                      "zonal"
                    ],
                    "--image-project": "value",
                    "--interface": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--licenses": "value",
                    "--multi-writer": "bool",
                    "--physical-block-size": [
                      "16384",
                      "4096"
                    ],
                    "--provisioned-iops": "value",
                    "--region": "dynamic",
                    "--replica-zones": "value",
                    "--require-csek-key-create": "bool",
                    "--resource-policies": "value",
                    "--size": "value",
                    "--source-disk": "dynamic",
                    "--source-disk-region": "dynamic",
                    "--source-disk-zone": "dynamic",
                    "--source-snapshot": "dynamic",
                    "--type": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "remove-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--size": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "snapshot": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--chain-name": "value",
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--snapshot-names": "value",
                    "--storage-location": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-architecture": "bool",
                    "--clear-labels": "bool",
                    "--clear-user-licenses": "bool",
                    "--provisioned-iops": "value",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--update-labels": "value",
                    "--update-user-licenses": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "external-vpn-gateways": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--interfaces": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "firewall-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--folder": "value",
                        "--name": "value",
                        "--organization": "value",
                        "--replace-association-on-target": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "clone-rules": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--source-firewall-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--short-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-rules": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--layer4-configs": "value",
                        "--organization": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-threat-intelligence": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--organization": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--organization": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-threat-intelligence": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "firewall-rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "ALLOW",
                      "DENY"
                    ],
                    "--allow": "value",
                    "--description": "value",
                    "--destination-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "IN",
                      "INGRESS",
                      "OUT"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--logging-metadata": [
                      "exclude-all",
                      "include-all"
                    ],
                    "--network": "value",
                    "--priority": "value",
                    "--rules": "value",
                    "--source-ranges": "value",
                    "--source-service-accounts": "value",
                    "--source-tags": "value",
                    "--target-service-accounts": "value",
                    "--target-tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "migrate": {
                  "commands": {},
                  "flags": {
                    "--export-tag-mapping": "bool",
                    "--source-network": "dynamic",
                    "--tag-mapping-file": "value",
                    "--target-firewall-policy": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allow": "value",
                    "--description": "value",
                    "--destination-ranges": "value",
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--logging-metadata": [
                      "exclude-all",
                      "include-all"
                    ],
                    "--priority": "value",
                    "--rules": "value",
                    "--source-ranges": "value",
                    "--source-service-accounts": "value",
                    "--source-tags": "value",
                    "--target-service-accounts": "value",
                    "--target-tags": "value"
                  }
                }
              },
              "flags": {}
            },
            "forwarding-rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--address": "dynamic",
                    "--address-region": "dynamic",
                    "--allow-global-access": "bool",
                    "--allow-psc-global-access": "bool",
                    "--backend-service": "value",
                    "--backend-service-region": "dynamic",
                    "--description": "value",
                    "--disable-automate-dns-zone": "bool",
                    "--global": "bool",
                    "--global-address": "bool",
                    "--global-backend-service": "bool",
                    "--global-target-http-proxy": "bool",
                    "--global-target-https-proxy": "bool",
                    "--global-target-tcp-proxy": "bool",
                    "--ip-protocol": [
                      "AH",
                      "ESP",
                      "ICMP",
                      "L3_DEFAULT",
                      "SCTP",
                      "TCP",
                      "UDP"
                    ],
                    "--ip-version": [
                      "IPV4",
                      "IPV6"
                    ],
                    "--is-mirroring-collector": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--network": "value",
                    "--network-tier": "value",
                    "--port-range": "value",
                    "--ports": "value",
                    "--region": "dynamic",
                    "--service-directory-registration": "value",
                    "--service-label": "value",
                    "--source-ip-ranges": "value",
                    "--subnet": "value",
                    "--subnet-region": "dynamic",
                    "--target-google-apis-bundle": "value",
                    "--target-grpc-proxy": "value",
                    "--target-http-proxy": "value",
                    "--target-http-proxy-region": "dynamic",
                    "--target-https-proxy": "value",
                    "--target-https-proxy-region": "dynamic",
                    "--target-instance": "value",
                    "--target-instance-zone": "dynamic",
                    "--target-pool": "value",
                    "--target-pool-region": "dynamic",
                    "--target-service-attachment": "value",
                    "--target-service-attachment-region": "dynamic",
                    "--target-ssl-proxy": "value",
                    "--target-tcp-proxy": "value",
                    "--target-tcp-proxy-region": "dynamic",
                    "--target-vpn-gateway": "value",
                    "--target-vpn-gateway-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-target": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "value",
                    "--backend-service-region": "dynamic",
                    "--global": "bool",
                    "--global-backend-service": "bool",
                    "--global-target-http-proxy": "bool",
                    "--global-target-https-proxy": "bool",
                    "--global-target-tcp-proxy": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED",
                      "INTERNAL",
                      "INTERNAL_MANAGED",
                      "INTERNAL_SELF_MANAGED"
                    ],
                    "--network": "value",
                    "--region": "dynamic",
                    "--subnet": "value",
                    "--subnet-region": "value",
                    "--target-grpc-proxy": "value",
                    "--target-http-proxy": "value",
                    "--target-http-proxy-region": "dynamic",
                    "--target-https-proxy": "value",
                    "--target-https-proxy-region": "dynamic",
                    "--target-instance": "value",
                    "--target-instance-zone": "dynamic",
                    "--target-pool": "value",
                    "--target-pool-region": "dynamic",
                    "--target-ssl-proxy": "value",
                    "--target-tcp-proxy": "value",
                    "--target-tcp-proxy-region": "dynamic",
                    "--target-vpn-gateway": "value",
                    "--target-vpn-gateway-region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allow-global-access": "bool",
                    "--allow-psc-global-access": "bool",
                    "--clear-labels": "bool",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--source-ip-ranges": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "health-checks": {
              "commands": {
                "create": {
                  "commands": {
                    "grpc": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--grpc-service-name": "value",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http2": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "https": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "ssl": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "tcp": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--protocol": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {
                    "grpc": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--grpc-service-name": "value",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "http2": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "https": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--host": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request-path": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "ssl": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    },
                    "tcp": {
                      "commands": {},
                      "flags": {
                        "--check-interval": "value",
                        "--description": "value",
                        "--enable-logging": "bool",
                        "--global": "bool",
                        "--healthy-threshold": "value",
                        "--port": "value",
                        "--port-name": "value",
                        "--proxy-header": [
                          "NONE",
                          "PROXY_V1"
                        ],
                        "--region": "dynamic",
                        "--request": "value",
                        "--response": "value",
                        "--timeout": "value",
                        "--unhealthy-threshold": "value",
                        "--use-serving-port": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "http-health-checks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                }
              },
              "flags": {}
            },
            "https-health-checks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--request-path": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value"
                  }
                }
              },
              "flags": {}
            },
            "images": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--family": "value",
                    "--forbidden-database-file": "value",
                    "--force": "bool",
                    "--force-create": "bool",
                    "--guest-os-features": "value",
                    "--key-exchange-key-file": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--licenses": "value",
                    "--platform-key-file": "value",
                    "--require-csek-key-create": "bool",
                    "--signature-database-file": "value",
                    "--source-disk": "dynamic",
                    "--source-disk-project": "value",
                    "--source-disk-zone": "dynamic",
                    "--source-image": "value",
                    "--source-image-family": "value",
                    "--source-image-project": "value",
                    "--source-snapshot": "dynamic",
                    "--source-uri": "value",
                    "--storage-location": "value",
                    "--user-licenses": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "deprecate": {
                  "commands": {},
                  "flags": {
                    "--delete-in": "value",
                    "--delete-on": "value",
                    "--deprecate-in": "value",
                    "--deprecate-on": "value",
                    "--obsolete-in": "value",
                    "--obsolete-on": "value",
                    "--replacement": "dynamic",
                    "--state": [
                      "ACTIVE",
                      "DELETED",
                      "DEPRECATED",
                      "OBSOLETE"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "describe-from-family": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--compute-service-account": "value",
                    "--destination-uri": "value",
                    "--export-format": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--log-location": "value",
                    "--network": "value",
                    "--subnet": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--aws-access-key-id": "value",
                    "--aws-ami-export-location": "value",
                    "--aws-ami-id": "value",
                    "--aws-region": "value",
                    "--aws-secret-access-key": "value",
                    "--aws-session-token": "value",
                    "--aws-source-ami-file-path": "value",
                    "--byol": "bool",
                    "--compute-service-account": "value",
                    "--data-disk": "bool",
                    "--description": "value",
                    "--family": "value",
                    "--guest-environment": "bool",
                    "--guest-os-features": "value",
                    "--log-location": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--source-file": "value",
                    "--source-image": "dynamic",
                    "--storage-location": "value",
                    "--subnet": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--preview-images": "bool",
                    "--regexp": "value",
                    "--show-deprecated": "bool",
                    "--sort-by": "value",
                    "--standard-images": "bool",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--architecture": [
                      "ARM64",
                      "X86_64"
                    ],
                    "--clear-labels": "bool",
                    "--clear-user-licenses": "bool",
                    "--description": "value",
                    "--family": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--update-user-licenses": "value"
                  }
                }
              },
              "flags": {}
            },
            "instance-groups": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-named-ports": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "list-instances": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "managed": {
                  "commands": {
                    "abandon-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "all-instances-config": {
                      "commands": {
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--labels": "value",
                            "--metadata": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--labels": "value",
                            "--metadata": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--base-instance-name": "value",
                        "--description": "value",
                        "--force-update-on-repair": "bool",
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--instance-redistribution-type": [
                          "none",
                          "proactive"
                        ],
                        "--list-managed-instances-results": [
                          "pageless",
                          "paginated"
                        ],
                        "--region": "dynamic",
                        "--size": "value",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--target-distribution-shape": [
                          "any",
                          "any-single-zone",
                          "balanced",
                          "even"
                        ],
                        "--target-pool": "value",
                        "--template": "value",
                        "--update-policy-max-surge": "value",
                        "--update-policy-max-unavailable": "value",
                        "--update-policy-min-ready": "value",
                        "--update-policy-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-most-disruptive-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--update-policy-type": [
                          "opportunistic",
                          "proactive"
                        ],
                        "--zone": "dynamic",
                        "--zones": "dynamic"
                      }
                    },
                    "create-instance": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--region": "dynamic",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--stateful-metadata": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "delete-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--skip-instances-on-validation-error": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe-instance": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "export-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--autoscaling-file": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "get-named-ports": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "instance-configs": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--instance": "value",
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--stateful-disk": "value",
                            "--stateful-external-ip": "value",
                            "--stateful-internal-ip": "value",
                            "--stateful-metadata": "value",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--instances": "value",
                            "--region": "dynamic",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--region": "dynamic",
                            "--sort-by": "value",
                            "--zone": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--instance": "value",
                            "--instance-update-minimal-action": [
                              "none",
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--remove-stateful-disks": "value",
                            "--remove-stateful-external-ips": "value",
                            "--remove-stateful-internal-ips": "value",
                            "--remove-stateful-metadata": "value",
                            "--stateful-disk": "value",
                            "--stateful-external-ip": "value",
                            "--stateful-internal-ip": "value",
                            "--stateful-metadata": "value",
                            "--update-instance": "bool",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--regions": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zones": "value"
                      }
                    },
                    "list-errors": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "recreate-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "resize": {
                      "commands": {},
                      "flags": {
                        "--creation-retries": "bool",
                        "--region": "dynamic",
                        "--size": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "rolling-action": {
                      "commands": {
                        "replace": {
                          "commands": {},
                          "flags": {
                            "--max-surge": "value",
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--region": "dynamic",
                            "--replacement-method": [
                              "recreate",
                              "substitute"
                            ],
                            "--zone": "dynamic"
                          }
                        },
                        "restart": {
                          "commands": {},
                          "flags": {
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        },
                        "start-update": {
                          "commands": {},
                          "flags": {
                            "--canary-version": "value",
                            "--force": "bool",
                            "--max-surge": "value",
                            "--max-unavailable": "value",
                            "--min-ready": "value",
                            "--minimal-action": [
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--most-disruptive-allowed-action": [
                              "refresh",
                              "replace",
                              "restart"
                            ],
                            "--region": "dynamic",
                            "--replacement-method": [
                              "recreate",
                              "substitute"
                            ],
                            "--type": [
                              "opportunistic",
                              "proactive"
                            ],
                            "--zone": "dynamic"
                          }
                        },
                        "stop-proactive-update": {
                          "commands": {},
                          "flags": {
                            "--region": "dynamic",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "set-autohealing": {
                      "commands": {},
                      "flags": {
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "set-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--autoscaling-file": "value",
                        "--cool-down-period": "value",
                        "--cpu-utilization-predictive-method": [
                          "none",
                          "optimize-availability"
                        ],
                        "--custom-metric-utilization": "value",
                        "--description": "value",
                        "--max-num-replicas": "value",
                        "--min-num-replicas": "value",
                        "--mode": [
                          "off",
                          "on",
                          "only-scale-out",
                          "only-up"
                        ],
                        "--region": "dynamic",
                        "--remove-stackdriver-metric": "value",
                        "--scale-based-on-cpu": "bool",
                        "--scale-based-on-load-balancing": "bool",
                        "--scale-in-control": "value",
                        "--schedule-cron": "value",
                        "--schedule-description": "value",
                        "--schedule-duration-sec": "value",
                        "--schedule-min-required-replicas": "value",
                        "--schedule-time-zone": "value",
                        "--set-schedule": "value",
                        "--stackdriver-metric-filter": "value",
                        "--stackdriver-metric-single-instance-assignment": "value",
                        "--stackdriver-metric-utilization-target": "value",
                        "--stackdriver-metric-utilization-target-type": [
                          "delta-per-minute",
                          "delta-per-second",
                          "gauge"
                        ],
                        "--target-cpu-utilization": "value",
                        "--target-load-balancing-utilization": "value",
                        "--update-stackdriver-metric": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-instance-template": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--template": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-named-ports": {
                      "commands": {},
                      "flags": {
                        "--named-ports": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "set-target-pools": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--target-pools": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "stop-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-autohealing": "bool",
                        "--description": "value",
                        "--force-update-on-repair": "bool",
                        "--health-check": "dynamic",
                        "--http-health-check": "value",
                        "--https-health-check": "value",
                        "--initial-delay": "value",
                        "--instance-redistribution-type": [
                          "none",
                          "proactive"
                        ],
                        "--list-managed-instances-results": [
                          "pageless",
                          "paginated"
                        ],
                        "--region": "dynamic",
                        "--remove-stateful-disks": "value",
                        "--remove-stateful-external-ips": "value",
                        "--remove-stateful-internal-ips": "value",
                        "--stateful-disk": "value",
                        "--stateful-external-ip": "value",
                        "--stateful-internal-ip": "value",
                        "--target-distribution-shape": [
                          "any",
                          "any-single-zone",
                          "balanced",
                          "even"
                        ],
                        "--update-policy-max-surge": "value",
                        "--update-policy-max-unavailable": "value",
                        "--update-policy-min-ready": "value",
                        "--update-policy-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-most-disruptive-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--update-policy-replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--update-policy-type": [
                          "opportunistic",
                          "proactive"
                        ],
                        "--zone": "dynamic"
                      }
                    },
                    "update-autoscaling": {
                      "commands": {},
                      "flags": {
                        "--clear-scale-down-control": "bool",
                        "--clear-scale-in-control": "bool",
                        "--cpu-utilization-predictive-method": [
                          "none",
                          "optimize-availability"
                        ],
                        "--disable-schedule": "value",
                        "--enable-schedule": "value",
                        "--max-num-replicas": "value",
                        "--min-num-replicas": "value",
                        "--mode": [
                          "off",
                          "on",
                          "only-scale-out",
                          "only-up"
                        ],
                        "--region": "dynamic",
                        "--remove-schedule": "value",
                        "--scale-in-control": "value",
                        "--schedule-cron": "value",
                        "--schedule-description": "value",
                        "--schedule-duration-sec": "value",
                        "--schedule-min-required-replicas": "value",
                        "--schedule-time-zone": "value",
                        "--set-schedule": "value",
                        "--update-schedule": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "update-instances": {
                      "commands": {},
                      "flags": {
                        "--all-instances": "bool",
                        "--instances": "value",
                        "--minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--most-disruptive-allowed-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "wait-until": {
                      "commands": {},
                      "flags": {
                        "--all-instances-config-effective": "bool",
                        "--region": "dynamic",
                        "--stable": "bool",
                        "--timeout": "value",
                        "--version-target-reached": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "wait-until-stable": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--timeout": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-named-ports": {
                  "commands": {},
                  "flags": {
                    "--named-ports": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "unmanaged": {
                  "commands": {
                    "add-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "get-named-ports": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zones": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-instances": {
                      "commands": {},
                      "flags": {
                        "--instances": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "set-named-ports": {
                      "commands": {},
                      "flags": {
                        "--named-ports": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instance-templates": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--configure-disk": "value",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--key-revocation-action-type": [
                      "none",
                      "stop"
                    ],
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--mesh": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--region": "dynamic",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--service-proxy": "value",
                    "--service-proxy-labels": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance": "dynamic",
                    "--source-instance-zone": "dynamic",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--termination-time": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value"
                  }
                },
                "create-with-container": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--region": "dynamic",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--subnet": "value",
                    "--tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-access-config": {
                  "commands": {},
                  "flags": {
                    "--access-config-name": "value",
                    "--address": "value",
                    "--network-interface": "value",
                    "--network-tier": "value",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-metadata": {
                  "commands": {},
                  "flags": {
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-tags": {
                  "commands": {},
                  "flags": {
                    "--tags": "value",
                    "--zone": "dynamic"
                  }
                },
                "attach-disk": {
                  "commands": {},
                  "flags": {
                    "--boot": "bool",
                    "--csek-key-file": "value",
                    "--device-name": "value",
                    "--disk": "value",
                    "--disk-scope": [
                      "regional",
                      "zonal"
                    ],
                    "--force-attach": "bool",
                    "--mode": [
                      "ro",
                      "rw"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "bulk": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator": "value",
                        "--async": "bool",
                        "--boot-disk-auto-delete": "bool",
                        "--boot-disk-device-name": "value",
                        "--boot-disk-kms-key": "dynamic",
                        "--boot-disk-kms-keyring": "dynamic",
                        "--boot-disk-kms-location": "dynamic",
                        "--boot-disk-kms-project": "dynamic",
                        "--boot-disk-provisioned-iops": "value",
                        "--boot-disk-size": "value",
                        "--boot-disk-type": "value",
                        "--can-ip-forward": "bool",
                        "--confidential-compute": "bool",
                        "--count": "value",
                        "--create-disk": "value",
                        "--custom-cpu": "value",
                        "--custom-extensions": "bool",
                        "--custom-memory": "value",
                        "--custom-vm-type": "value",
                        "--description": "value",
                        "--disk": "value",
                        "--enable-display-device": "bool",
                        "--enable-nested-virtualization": "bool",
                        "--enable-uefi-networking": "bool",
                        "--erase-windows-vss-signature": "bool",
                        "--host-error-timeout-seconds": "value",
                        "--image": "value",
                        "--image-family": "value",
                        "--image-project": "value",
                        "--instance-termination-action": [
                          "DELETE",
                          "STOP"
                        ],
                        "--labels": "value",
                        "--local-ssd": "value",
                        "--location-policy": "value",
                        "--machine-type": "dynamic",
                        "--maintenance-policy": [
                          "MIGRATE",
                          "TERMINATE"
                        ],
                        "--max-run-duration": "value",
                        "--metadata": "value",
                        "--metadata-from-file": "value",
                        "--min-count": "value",
                        "--min-cpu-platform": "value",
                        "--min-node-cpu": "value",
                        "--name-pattern": "value",
                        "--network": "value",
                        "--network-interface": "value",
                        "--network-performance-configs": "value",
                        "--network-tier": "value",
                        "--no-address": "bool",
                        "--no-public-dns": "bool",
                        "--no-scopes": "bool",
                        "--no-service-account": "bool",
                        "--on-host-maintenance": [
                          "MIGRATE",
                          "TERMINATE"
                        ],
                        "--post-key-revocation-action-type": [
                          "noop",
                          "shutdown"
                        ],
                        "--predefined-names": "value",
                        "--preemptible": "bool",
                        "--provisioning-model": [
                          "SPOT",
                          "STANDARD"
                        ],
                        "--public-dns": "bool",
                        "--region": "value",
                        "--reservation": "value",
                        "--reservation-affinity": [
                          "any",
                          "none",
                          "specific"
                        ],
                        "--resource-manager-tags": "value",
                        "--resource-policies": "value",
                        "--restart-on-failure": "bool",
                        "--scopes": "value",
                        "--service-account": "value",
                        "--shielded-integrity-monitoring": "bool",
                        "--shielded-secure-boot": "bool",
                        "--shielded-vtpm": "bool",
                        "--source-instance-template": "dynamic",
                        "--source-snapshot": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--subnet": "value",
                        "--tags": "value",
                        "--target-distribution-shape": [
                          "ANY",
                          "ANY_SINGLE_ZONE",
                          "BALANCED"
                        ],
                        "--termination-time": "value",
                        "--threads-per-core": "value",
                        "--visible-core-count": "value",
                        "--zone": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--async": "bool",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--create-disk": "value",
                    "--csek-key-file": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--deletion-protection": "bool",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-display-device": "bool",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--erase-windows-vss-signature": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--hostname": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-family-scope": [
                      "global",
                      "zonal"
                    ],
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--ipv6-public-ptr-domain": "value",
                    "--key-revocation-action-type": [
                      "none",
                      "stop"
                    ],
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--node-project": "value",
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--require-csek-key-create": "bool",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--source-machine-image": "dynamic",
                    "--source-machine-image-csek-key-file": "value",
                    "--source-snapshot": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--termination-time": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "create-with-container": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--address": "value",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-nested-virtualization": "bool",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--host-error-timeout-seconds": "value",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--preemptible": "bool",
                    "--private-ipv6-google-access-type": [
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access",
                      "inherit-subnetwork"
                    ],
                    "--private-network-ip": "value",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--subnet": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--visible-core-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--delete-disks": [
                      "all",
                      "boot",
                      "data"
                    ],
                    "--keep-disks": [
                      "all",
                      "boot",
                      "data"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "delete-access-config": {
                  "commands": {},
                  "flags": {
                    "--access-config-name": "value",
                    "--network-interface": "value",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "detach-disk": {
                  "commands": {},
                  "flags": {
                    "--device-name": "value",
                    "--disk": "value",
                    "--disk-scope": [
                      "regional",
                      "zonal"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--zone": "value"
                  }
                },
                "get-guest-attributes": {
                  "commands": {},
                  "flags": {
                    "--query-path": "value",
                    "--zone": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "get-screenshot": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--zone": "dynamic"
                  }
                },
                "get-serial-port-output": {
                  "commands": {},
                  "flags": {
                    "--port": "value",
                    "--start": "value",
                    "--zone": "dynamic"
                  }
                },
                "get-shielded-identity": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--byol": "bool",
                    "--can-ip-forward": "bool",
                    "--compute-service-account": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--deletion-protection": "bool",
                    "--description": "value",
                    "--guest-environment": "bool",
                    "--guest-os-features": "value",
                    "--hostname": "value",
                    "--labels": "value",
                    "--log-location": "value",
                    "--machine-type": "dynamic",
                    "--network": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--private-network-ip": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--source-uri": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-zone": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "network-interfaces": {
                  "commands": {
                    "get-effective-firewalls": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network-interface": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--aliases": "value",
                        "--external-ipv6-address": "value",
                        "--external-ipv6-prefix-length": "value",
                        "--internal-ipv6-address": "value",
                        "--internal-ipv6-prefix-length": "value",
                        "--ipv6-network-tier": [
                          "PREMIUM"
                        ],
                        "--network": "value",
                        "--network-interface": "value",
                        "--private-network-ip": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--subnetwork": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "ops-agents": {
                  "commands": {
                    "policies": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--agent-rules": "value",
                            "--description": "value",
                            "--group-labels": "value",
                            "--instances": "value",
                            "--os-types": "value",
                            "--zones": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {}
                        },
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--agent-rules": "value",
                            "--clear-group-labels": "bool",
                            "--clear-instances": "bool",
                            "--clear-zones": "bool",
                            "--description": "value",
                            "--etag": "value",
                            "--group-labels": "value",
                            "--instances": "value",
                            "--os-types": "value",
                            "--zones": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "os-inventory": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list-instances": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--inventory-filter": "value",
                        "--kernel-version": "value",
                        "--limit": "value",
                        "--os-shortname": "value",
                        "--os-version": "value",
                        "--package-name": "value",
                        "--package-version": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-metadata": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--keys": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-resource-policies": {
                  "commands": {},
                  "flags": {
                    "--resource-policies": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-tags": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--tags": "value",
                    "--zone": "dynamic"
                  }
                },
                "reset": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "dynamic"
                  }
                },
                "send-diagnostic-interrupt": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "set-disk-auto-delete": {
                  "commands": {},
                  "flags": {
                    "--auto-delete": "bool",
                    "--device-name": "value",
                    "--disk": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "set-machine-type": {
                  "commands": {},
                  "flags": {
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--machine-type": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "set-name": {
                  "commands": {},
                  "flags": {
                    "--new-name": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-scheduling": {
                  "commands": {},
                  "flags": {
                    "--clear-instance-termination-action": "bool",
                    "--clear-max-run-duration": "bool",
                    "--clear-min-node-cpu": "bool",
                    "--clear-node-affinities": "bool",
                    "--clear-termination-time": "bool",
                    "--host-error-timeout-seconds": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--max-run-duration": "value",
                    "--min-node-cpu": "value",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--preemptible": "bool",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--restart-on-failure": "bool",
                    "--termination-time": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-scopes": {
                  "commands": {},
                  "flags": {
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--zone": "dynamic"
                  }
                },
                "simulate-maintenance-event": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--csek-key-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-local-ssd": "value",
                    "--zone": "dynamic"
                  }
                },
                "suspend": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-local-ssd": "value",
                    "--zone": "dynamic"
                  }
                },
                "tail-serial-port-output": {
                  "commands": {},
                  "flags": {
                    "--port": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-node-affinities": "bool",
                    "--deletion-protection": "bool",
                    "--enable-display-device": "bool",
                    "--min-cpu-platform": "value",
                    "--node": "value",
                    "--node-affinity-file": "value",
                    "--node-group": "value",
                    "--remove-labels": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-learn-integrity-policy": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--update-labels": "value",
                    "--zone": "dynamic"
                  }
                },
                "update-access-config": {
                  "commands": {},
                  "flags": {
                    "--ipv6-public-ptr-domain": "value",
                    "--network-interface": "value",
                    "--no-ipv6-public-ptr": "bool",
                    "--no-public-ptr": "bool",
                    "--no-public-ptr-domain": "bool",
                    "--public-ptr": "bool",
                    "--public-ptr-domain": "value",
                    "--zone": "dynamic"
                  }
                },
                "update-container": {
                  "commands": {},
                  "flags": {
                    "--clear-container-args": "bool",
                    "--clear-container-command": "bool",
                    "--container-arg": "value",
                    "--container-command": "value",
                    "--container-env": "value",
                    "--container-env-file": "value",
                    "--container-image": "value",
                    "--container-mount-disk": "value",
                    "--container-mount-host-path": "value",
                    "--container-mount-tmpfs": "value",
                    "--container-privileged": "bool",
                    "--container-restart-policy": [
                      "always",
                      "never",
                      "on-failure"
                    ],
                    "--container-stdin": "bool",
                    "--container-tty": "bool",
                    "--remove-container-env": "value",
                    "--remove-container-mounts": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-learn-integrity-policy": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--zone": "dynamic"
                  }
                },
                "update-from-file": {
                  "commands": {},
                  "flags": {
                    "--minimal-action": "value",
                    "--most-disruptive-allowed-action": "value",
                    "--source": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "interconnects": {
              "commands": {
                "attachments": {
                  "commands": {
                    "dedicated": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--bandwidth": [
                              "100m",
                              "10g",
                              "1g",
                              "200m",
                              "20g",
                              "2g",
                              "300m",
                              "400m",
                              "500m",
                              "50g",
                              "50m",
                              "5g"
                            ],
                            "--candidate-ipv6-subnets": "value",
                            "--candidate-subnets": "value",
                            "--cloud-router-ipv6-interface-id": "value",
                            "--customer-router-ipv6-interface-id": "value",
                            "--description": "value",
                            "--enable-admin": "bool",
                            "--encryption": [
                              "IPSEC",
                              "NONE"
                            ],
                            "--interconnect": "dynamic",
                            "--ipsec-internal-addresses": "value",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--stack-type": [
                              "IPV4_IPV6",
                              "IPV4_ONLY"
                            ],
                            "--vlan": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--bandwidth": [
                              "100m",
                              "10g",
                              "1g",
                              "200m",
                              "20g",
                              "2g",
                              "300m",
                              "400m",
                              "500m",
                              "50g",
                              "50m",
                              "5g"
                            ],
                            "--candidate-ipv6-subnets": "value",
                            "--clear-labels": "bool",
                            "--cloud-router-ipv6-interface-id": "value",
                            "--customer-router-ipv6-interface-id": "value",
                            "--description": "value",
                            "--enable-admin": "bool",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--remove-labels": "value",
                            "--stack-type": [
                              "IPV4_IPV6",
                              "IPV4_ONLY"
                            ],
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "partner": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--description": "value",
                            "--edge-availability-domain": [
                              "any",
                              "availability-domain-1",
                              "availability-domain-2"
                            ],
                            "--enable-admin": "bool",
                            "--encryption": [
                              "IPSEC",
                              "NONE"
                            ],
                            "--ipsec-internal-addresses": "value",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--clear-labels": "bool",
                            "--description": "value",
                            "--enable-admin": "bool",
                            "--mtu": "value",
                            "--region": "dynamic",
                            "--remove-labels": "value",
                            "--update-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-enabled": "bool",
                    "--customer-name": "value",
                    "--description": "value",
                    "--interconnect-type": [
                      "DEDICATED",
                      "IT_PRIVATE",
                      "PARTNER"
                    ],
                    "--link-type": [
                      "LINK_TYPE_ETHERNET_100G_LR",
                      "LINK_TYPE_ETHERNET_10G_LR"
                    ],
                    "--location": "dynamic",
                    "--noc-contact-email": "value",
                    "--requested-link-count": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-diagnostics": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--admin-enabled": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--noc-contact-email": "value",
                    "--remove-labels": "value",
                    "--requested-link-count": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "machine-images": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--require-csek-key-create": "bool",
                    "--source-disk-csek-key": "value",
                    "--source-instance": "dynamic",
                    "--source-instance-zone": "dynamic",
                    "--storage-location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--byol": "bool",
                    "--can-ip-forward": "bool",
                    "--compute-service-account": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--guest-environment": "bool",
                    "--guest-flush": "bool",
                    "--guest-os-features": "value",
                    "--labels": "value",
                    "--log-location": "value",
                    "--machine-type": "dynamic",
                    "--network": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--os": [
                      "centos-7",
                      "debian-10",
                      "debian-11",
                      "debian-8",
                      "debian-9",
                      "opensuse-15",
                      "rhel-6",
                      "rhel-6-byol",
                      "rhel-7",
                      "rhel-7-byol",
                      "rhel-8",
                      "rhel-8-byol",
                      "rhel-9",
                      "rhel-9-byol",
                      "rocky-8",
                      "sles-12",
                      "sles-12-byol",
                      "sles-15",
                      "sles-15-byol",
                      "sles-sap-12",
                      "sles-sap-12-byol",
                      "sles-sap-15",
                      "sles-sap-15-byol",
                      "ubuntu-1404",
                      "ubuntu-1604",
                      "ubuntu-1804",
                      "ubuntu-2004",
                      "ubuntu-2204",
                      "windows-10-x64-byol",
                      "windows-10-x86-byol",
                      "windows-11-x64-byol",
                      "windows-2008r2",
                      "windows-2008r2-byol",
                      "windows-2012",
                      "windows-2012-byol",
                      "windows-2012r2",
                      "windows-2012r2-byol",
                      "windows-2016",
                      "windows-2016-byol",
                      "windows-2019",
                      "windows-2019-byol",
                      "windows-2022",
                      "windows-2022-byol",
                      "windows-7-x64-byol",
                      "windows-7-x86-byol",
                      "windows-8-x64-byol",
                      "windows-8-x86-byol"
                    ],
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--source-uri": "value",
                    "--storage-location": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "machine-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-attachments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--description": "value",
                    "--producer-accept-list": "value",
                    "--producer-reject-list": "value",
                    "--region": "dynamic",
                    "--subnets": "dynamic",
                    "--subnets-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "network-edge-security-services": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--region": "dynamic",
                    "--security-policy": "dynamic",
                    "--security-policy-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--region": "dynamic",
                    "--security-policy": "dynamic",
                    "--security-policy-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-endpoint-groups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--app-engine-app": "bool",
                    "--app-engine-service": "value",
                    "--app-engine-url-mask": "value",
                    "--app-engine-version": "value",
                    "--cloud-function-name": "value",
                    "--cloud-function-url-mask": "value",
                    "--cloud-run-service": "value",
                    "--cloud-run-tag": "value",
                    "--cloud-run-url-mask": "value",
                    "--default-port": "value",
                    "--global": "bool",
                    "--network": "value",
                    "--network-endpoint-type": [
                      "gce-vm-ip",
                      "gce-vm-ip-port",
                      "internet-fqdn-port",
                      "internet-ip-port",
                      "non-gcp-private-ip-port",
                      "private-service-connect",
                      "serverless"
                    ],
                    "--psc-target-service": "value",
                    "--region": "dynamic",
                    "--serverless-deployment-platform": "value",
                    "--serverless-deployment-resource": "value",
                    "--serverless-deployment-url-mask": "value",
                    "--serverless-deployment-version": "value",
                    "--subnet": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "list-network-endpoints": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-endpoint": "value",
                    "--global": "bool",
                    "--remove-endpoint": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "network-firewall-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--name": "value",
                        "--network": "value",
                        "--replace-association-on-target": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "clone-rules": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source-firewall-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "get-effective-firewalls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--layer4-configs": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-secure-tags": "value",
                        "--src-threat-intelligence": "value",
                        "--target-secure-tags": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-address-groups": "value",
                        "--dest-fqdns": "value",
                        "--dest-ip-ranges": "value",
                        "--dest-region-codes": "value",
                        "--dest-threat-intelligence": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--disabled": "bool",
                        "--enable-logging": "bool",
                        "--firewall-policy": "value",
                        "--firewall-policy-region": "dynamic",
                        "--global-firewall-policy": "bool",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--src-address-groups": "value",
                        "--src-fqdns": "value",
                        "--src-ip-ranges": "value",
                        "--src-region-codes": "value",
                        "--src-secure-tags": "value",
                        "--src-threat-intelligence": "value",
                        "--target-secure-tags": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "networks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bgp-routing-mode": [
                      "global",
                      "regional"
                    ],
                    "--description": "value",
                    "--enable-ula-internal-ipv6": "bool",
                    "--internal-ipv6-range": "value",
                    "--mtu": "value",
                    "--network-firewall-policy-enforcement-order": [
                      "AFTER_CLASSIC_FIREWALL",
                      "BEFORE_CLASSIC_FIREWALL"
                    ],
                    "--range": "value",
                    "--subnet-mode": [
                      "auto",
                      "custom",
                      "legacy"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-effective-firewalls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "peerings": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-create-routes": "bool",
                        "--export-custom-routes": "bool",
                        "--export-subnet-routes-with-public-ip": "bool",
                        "--import-custom-routes": "bool",
                        "--import-subnet-routes-with-public-ip": "bool",
                        "--network": "value",
                        "--peer-network": "value",
                        "--peer-project": "value",
                        "--stack-type": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--network": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-routes": {
                      "commands": {},
                      "flags": {
                        "--direction": [
                          "INCOMING",
                          "OUTGOING"
                        ],
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--export-custom-routes": "bool",
                        "--export-subnet-routes-with-public-ip": "bool",
                        "--import-custom-routes": "bool",
                        "--import-subnet-routes-with-public-ip": "bool",
                        "--network": "value",
                        "--stack-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "subnets": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--enable-flow-logs": "bool",
                        "--enable-private-ip-google-access": "bool",
                        "--ipv6-access-type": [
                          "EXTERNAL",
                          "INTERNAL"
                        ],
                        "--logging-aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--logging-filter-expr": "value",
                        "--logging-flow-sampling": "value",
                        "--logging-metadata": [
                          "custom",
                          "exclude-all",
                          "include-all"
                        ],
                        "--logging-metadata-fields": "value",
                        "--network": "dynamic",
                        "--private-ipv6-google-access-type": [
                          "disable",
                          "enable-bidirectional-access",
                          "enable-outbound-vm-access"
                        ],
                        "--purpose": [
                          "INTERNAL_HTTPS_LOAD_BALANCER",
                          "PRIVATE",
                          "PRIVATE_SERVICE_CONNECT",
                          "REGIONAL_MANAGED_PROXY"
                        ],
                        "--range": "value",
                        "--region": "dynamic",
                        "--reserved-internal-range": "value",
                        "--role": [
                          "ACTIVE",
                          "BACKUP"
                        ],
                        "--secondary-range": "value",
                        "--secondary-range-with-reserved-internal-range": "value",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "expand-ip-range": {
                      "commands": {},
                      "flags": {
                        "--prefix-length": "value",
                        "--region": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--network": "value",
                        "--page-size": "value",
                        "--regexp": "value",
                        "--regions": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-usable": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--service-project": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-secondary-ranges": "value",
                        "--add-secondary-ranges-with-reserved-internal-range": "value",
                        "--drain-timeout": "value",
                        "--enable-flow-logs": "bool",
                        "--enable-private-ip-google-access": "bool",
                        "--ipv6-access-type": [
                          "EXTERNAL",
                          "INTERNAL"
                        ],
                        "--logging-aggregation-interval": [
                          "interval-1-min",
                          "interval-10-min",
                          "interval-15-min",
                          "interval-30-sec",
                          "interval-5-min",
                          "interval-5-sec"
                        ],
                        "--logging-filter-expr": "value",
                        "--logging-flow-sampling": "value",
                        "--logging-metadata": [
                          "custom",
                          "exclude-all",
                          "include-all"
                        ],
                        "--logging-metadata-fields": "value",
                        "--private-ipv6-google-access-type": [
                          "disable",
                          "enable-bidirectional-access",
                          "enable-outbound-vm-access"
                        ],
                        "--purpose": [
                          "REGIONAL_MANAGED_PROXY"
                        ],
                        "--region": "dynamic",
                        "--remove-secondary-ranges": "value",
                        "--role": [
                          "ACTIVE"
                        ],
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bgp-routing-mode": [
                      "global",
                      "regional"
                    ],
                    "--enable-ula-internal-ipv6": "bool",
                    "--internal-ipv6-range": "value",
                    "--mtu": "value",
                    "--network-firewall-policy-enforcement-order": [
                      "AFTER_CLASSIC_FIREWALL",
                      "BEFORE_CLASSIC_FIREWALL"
                    ],
                    "--switch-to-custom-subnet-mode": "bool"
                  }
                },
                "vpc-access": {
                  "commands": {
                    "connectors": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--machine-type": "value",
                            "--max-instances": "value",
                            "--max-throughput": "value",
                            "--min-instances": "value",
                            "--min-throughput": "value",
                            "--network": "value",
                            "--range": "value",
                            "--region": "dynamic",
                            "--subnet": "value",
                            "--subnet-project": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--region": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--region": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--region": "dynamic",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--machine-type": "value",
                            "--max-instances": "value",
                            "--min-instances": "value",
                            "--region": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "locations": {
                      "commands": {
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "operations": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--region": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--region": "dynamic",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            },
            "org-security-policies": {
              "commands": {
                "associations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--folder": "value",
                        "--name": "value",
                        "--organization": "value",
                        "--replace-association-on-target": "bool",
                        "--security-policy": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--folder": "value",
                        "--limit": "value",
                        "--organization": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "copy-rules": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--source-security-policy": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-rules": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-ip-ranges": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--enable-logging": "bool",
                        "--layer4-configs": "value",
                        "--organization": "value",
                        "--security-policy": "value",
                        "--src-ip-ranges": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--organization": "value",
                        "--security-policy": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny",
                          "goto_next"
                        ],
                        "--description": "value",
                        "--dest-ip-ranges": "value",
                        "--direction": [
                          "EGRESS",
                          "INGRESS"
                        ],
                        "--enable-logging": "bool",
                        "--layer4-configs": "value",
                        "--new-priority": "value",
                        "--organization": "value",
                        "--security-policy": "value",
                        "--src-ip-ranges": "value",
                        "--target-resources": "value",
                        "--target-service-accounts": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "os-config": {
              "commands": {
                "guest-policies": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "lookup": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "os-upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auto-rollback": "bool",
                    "--create-machine-backup": "bool",
                    "--log-location": "value",
                    "--source-os": [
                      "windows-2008r2"
                    ],
                    "--target-os": [
                      "windows-2012r2"
                    ],
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                },
                "patch-deployments": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "pause": {
                      "commands": {},
                      "flags": {}
                    },
                    "resume": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--file": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "patch-jobs": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "execute": {
                      "commands": {},
                      "flags": {
                        "--apt-dist": "bool",
                        "--apt-excludes": "value",
                        "--apt-exclusive-packages": "value",
                        "--async": "bool",
                        "--description": "value",
                        "--display-name": "value",
                        "--dry-run": "bool",
                        "--duration": "value",
                        "--instance-filter-all": "bool",
                        "--instance-filter-group-labels": "value",
                        "--instance-filter-name-prefixes": "value",
                        "--instance-filter-names": "value",
                        "--instance-filter-zones": "value",
                        "--mig-instances-allowed": "bool",
                        "--post-patch-linux-executable": "value",
                        "--post-patch-linux-success-codes": "value",
                        "--post-patch-windows-executable": "value",
                        "--post-patch-windows-success-codes": "value",
                        "--pre-patch-linux-executable": "value",
                        "--pre-patch-linux-success-codes": "value",
                        "--pre-patch-windows-executable": "value",
                        "--pre-patch-windows-success-codes": "value",
                        "--reboot-config": [
                          "always",
                          "default",
                          "never"
                        ],
                        "--rollout-disruption-budget": "value",
                        "--rollout-disruption-budget-percent": "value",
                        "--rollout-mode": [
                          "concurrent-zones",
                          "zone-by-zone"
                        ],
                        "--windows-classifications": "value",
                        "--windows-excludes": "value",
                        "--windows-exclusive-patches": "value",
                        "--yum-excludes": "value",
                        "--yum-exclusive-packages": "value",
                        "--yum-minimal": "bool",
                        "--yum-security": "bool",
                        "--zypper-categories": "value",
                        "--zypper-excludes": "value",
                        "--zypper-exclusive-patches": "value",
                        "--zypper-severities": "value",
                        "--zypper-with-optional": "bool",
                        "--zypper-with-update": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-instance-details": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "os-login": {
              "commands": {
                "describe-profile": {
                  "commands": {},
                  "flags": {}
                },
                "remove-profile": {
                  "commands": {},
                  "flags": {}
                },
                "ssh-keys": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value",
                        "--ttl": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--key": "value",
                        "--key-file": "value",
                        "--ttl": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "packet-mirrorings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--collector-ilb": "value",
                    "--description": "value",
                    "--enable": "bool",
                    "--filter-cidr-ranges": "value",
                    "--filter-direction": [
                      "both",
                      "egress",
                      "ingress"
                    ],
                    "--filter-protocols": "value",
                    "--mirrored-instances": "value",
                    "--mirrored-subnets": "value",
                    "--mirrored-tags": "value",
                    "--network": "value",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-filter-cidr-ranges": "value",
                    "--add-filter-protocols": "value",
                    "--add-mirrored-instances": "value",
                    "--add-mirrored-subnets": "value",
                    "--add-mirrored-tags": "value",
                    "--async": "bool",
                    "--clear-filter-cidr-ranges": "bool",
                    "--clear-filter-protocols": "bool",
                    "--clear-mirrored-instances": "bool",
                    "--clear-mirrored-subnets": "bool",
                    "--clear-mirrored-tags": "bool",
                    "--collector-ilb": "value",
                    "--description": "value",
                    "--enable": "bool",
                    "--filter-direction": [
                      "both",
                      "egress",
                      "ingress"
                    ],
                    "--region": "dynamic",
                    "--remove-filter-cidr-ranges": "value",
                    "--remove-filter-protocols": "value",
                    "--remove-mirrored-instances": "value",
                    "--remove-mirrored-subnets": "value",
                    "--remove-mirrored-tags": "value",
                    "--set-filter-cidr-ranges": "value",
                    "--set-filter-protocols": "value",
                    "--set-mirrored-instances": "value",
                    "--set-mirrored-subnets": "value",
                    "--set-mirrored-tags": "value"
                  }
                }
              },
              "flags": {}
            },
            "project-info": {
              "commands": {
                "add-metadata": {
                  "commands": {},
                  "flags": {
                    "--metadata": "value",
                    "--metadata-from-file": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "remove-metadata": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--keys": "value"
                  }
                },
                "set-usage-bucket": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--no-bucket": "bool",
                    "--prefix": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--default-network-tier": [
                      "FIXED_STANDARD",
                      "PREMIUM",
                      "STANDARD"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "public-advertised-prefixes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--dns-verification-ip": "value",
                    "--range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--status": [
                      "ptr-configured"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "public-delegated-prefixes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-live-migration": "bool",
                    "--global": "bool",
                    "--public-advertised-prefix": "value",
                    "--range": "value",
                    "--region": "dynamic"
                  }
                },
                "delegated-sub-prefixes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--create-addresses": "bool",
                        "--delegatee-project": "value",
                        "--description": "value",
                        "--global-public-delegated-prefix": "bool",
                        "--public-delegated-prefix": "value",
                        "--public-delegated-prefix-region": "dynamic",
                        "--range": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--global-public-delegated-prefix": "bool",
                        "--public-delegated-prefix": "value",
                        "--public-delegated-prefix-region": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "reservations": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--description": "value",
                    "--local-ssd": "value",
                    "--machine-type": "value",
                    "--min-cpu-platform": "value",
                    "--require-specific-reservation": "bool",
                    "--resource-policies": "value",
                    "--share-setting": [
                      "folders",
                      "local",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--source-instance-template": "value",
                    "--vm-count": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-share-with": "value",
                    "--remove-share-with": "value",
                    "--share-with": "value",
                    "--vm-count": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "reset-windows-password": {
              "commands": {},
              "flags": {
                "--user": "value",
                "--zone": "dynamic"
              }
            },
            "resource-policies": {
              "commands": {
                "create": {
                  "commands": {
                    "group-placement": {
                      "commands": {},
                      "flags": {
                        "--availability-domain-count": "value",
                        "--collocation": [
                          "collocated",
                          "unspecified-collocation"
                        ],
                        "--description": "value",
                        "--max-distance": "value",
                        "--region": "dynamic",
                        "--vm-count": "value"
                      }
                    },
                    "instance-schedule": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--end-date": "value",
                        "--initiation-date": "value",
                        "--region": "dynamic",
                        "--timezone": "value",
                        "--vm-start-schedule": "value",
                        "--vm-stop-schedule": "value"
                      }
                    },
                    "snapshot-schedule": {
                      "commands": {},
                      "flags": {
                        "--daily-schedule": "bool",
                        "--description": "value",
                        "--guest-flush": "bool",
                        "--hourly-schedule": "value",
                        "--max-retention-days": "value",
                        "--on-source-disk-delete": [
                          "apply-retention-policy",
                          "keep-auto-snapshots"
                        ],
                        "--region": "dynamic",
                        "--snapshot-labels": "value",
                        "--start-time": "value",
                        "--storage-location": "value",
                        "--weekly-schedule": [
                          "friday",
                          "monday",
                          "saturday",
                          "sunday",
                          "thursday",
                          "tuesday",
                          "wednesday"
                        ],
                        "--weekly-schedule-from-file": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create-snapshot-schedule": {
                  "commands": {},
                  "flags": {
                    "--daily-schedule": "bool",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--hourly-schedule": "value",
                    "--max-retention-days": "value",
                    "--on-source-disk-delete": [
                      "apply-retention-policy",
                      "keep-auto-snapshots"
                    ],
                    "--region": "dynamic",
                    "--snapshot-labels": "value",
                    "--start-time": "value",
                    "--storage-location": "value",
                    "--weekly-schedule": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--weekly-schedule-from-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {
                    "snapshot-schedule": {
                      "commands": {},
                      "flags": {
                        "--daily-schedule": "bool",
                        "--description": "value",
                        "--hourly-schedule": "value",
                        "--region": "dynamic",
                        "--snapshot-labels": "value",
                        "--start-time": "value",
                        "--weekly-schedule": [
                          "friday",
                          "monday",
                          "saturday",
                          "sunday",
                          "thursday",
                          "tuesday",
                          "wednesday"
                        ],
                        "--weekly-schedule-from-file": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "routers": {
              "commands": {
                "add-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--advertised-route-priority": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--async": "bool",
                    "--bfd-min-receive-interval": "value",
                    "--bfd-min-transmit-interval": "value",
                    "--bfd-multiplier": "value",
                    "--bfd-session-initialization-mode": [
                      "ACTIVE",
                      "DISABLED",
                      "PASSIVE"
                    ],
                    "--enable-ipv6": "bool",
                    "--enabled": "bool",
                    "--instance": "dynamic",
                    "--instance-zone": "dynamic",
                    "--interface": "value",
                    "--ipv6-nexthop-address": "value",
                    "--md5-authentication-key": "value",
                    "--peer-asn": "value",
                    "--peer-ip-address": "value",
                    "--peer-ipv6-nexthop-address": "value",
                    "--peer-name": "value",
                    "--region": "dynamic",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "add-interface": {
                  "commands": {},
                  "flags": {
                    "--interconnect-attachment": "dynamic",
                    "--interconnect-attachment-region": "dynamic",
                    "--interface-name": "value",
                    "--ip-address": "value",
                    "--mask-length": "value",
                    "--redundant-interface": "value",
                    "--region": "dynamic",
                    "--subnetwork": "dynamic",
                    "--subnetwork-region": "dynamic",
                    "--vpn-tunnel": "dynamic",
                    "--vpn-tunnel-region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--asn": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--encrypted-interconnect-router": "bool",
                    "--keepalive-interval": "value",
                    "--network": "dynamic",
                    "--region": "dynamic",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-nat-mapping-info": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "nats": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-allocate-nat-external-ips": "bool",
                        "--enable-dynamic-port-allocation": "bool",
                        "--enable-endpoint-independent-mapping": "bool",
                        "--enable-logging": "bool",
                        "--endpoint-types": "value",
                        "--icmp-idle-timeout": "value",
                        "--log-filter": [
                          "ALL",
                          "ERRORS_ONLY",
                          "TRANSLATIONS_ONLY"
                        ],
                        "--max-ports-per-vm": "value",
                        "--min-ports-per-vm": "value",
                        "--nat-all-subnet-ip-ranges": "bool",
                        "--nat-custom-subnet-ip-ranges": "value",
                        "--nat-external-ip-pool": "value",
                        "--nat-primary-subnet-ip-ranges": "bool",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--rules": "value",
                        "--tcp-established-idle-timeout": "value",
                        "--tcp-time-wait-timeout": "value",
                        "--tcp-transitory-idle-timeout": "value",
                        "--udp-idle-timeout": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "rules": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--match": "value",
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--source-nat-active-ips": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--clear-source-nat-drain-ips": "bool",
                            "--match": "value",
                            "--nat": "value",
                            "--region": "dynamic",
                            "--router": "dynamic",
                            "--source-nat-active-ips": "value",
                            "--source-nat-drain-ips": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--auto-allocate-nat-external-ips": "bool",
                        "--clear-icmp-idle-timeout": "bool",
                        "--clear-max-ports-per-vm": "bool",
                        "--clear-min-ports-per-vm": "bool",
                        "--clear-nat-external-drain-ip-pool": "bool",
                        "--clear-tcp-established-idle-timeout": "bool",
                        "--clear-tcp-time-wait-timeout": "bool",
                        "--clear-tcp-transitory-idle-timeout": "bool",
                        "--clear-udp-idle-timeout": "bool",
                        "--enable-dynamic-port-allocation": "bool",
                        "--enable-endpoint-independent-mapping": "bool",
                        "--enable-logging": "bool",
                        "--icmp-idle-timeout": "value",
                        "--log-filter": [
                          "ALL",
                          "ERRORS_ONLY",
                          "TRANSLATIONS_ONLY"
                        ],
                        "--max-ports-per-vm": "value",
                        "--min-ports-per-vm": "value",
                        "--nat-all-subnet-ip-ranges": "bool",
                        "--nat-custom-subnet-ip-ranges": "value",
                        "--nat-external-drain-ip-pool": "value",
                        "--nat-external-ip-pool": "value",
                        "--nat-primary-subnet-ip-ranges": "bool",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--rules": "value",
                        "--tcp-established-idle-timeout": "value",
                        "--tcp-time-wait-timeout": "value",
                        "--tcp-transitory-idle-timeout": "value",
                        "--udp-idle-timeout": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--peer-name": "value",
                    "--peer-names": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-interface": {
                  "commands": {},
                  "flags": {
                    "--interface-name": "value",
                    "--interface-names": "value",
                    "--region": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-advertisement-groups": "value",
                    "--add-advertisement-ranges": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--asn": "value",
                    "--async": "bool",
                    "--keepalive-interval": "value",
                    "--region": "dynamic",
                    "--remove-advertisement-groups": "value",
                    "--remove-advertisement-ranges": "value",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "update-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--add-advertisement-groups": "value",
                    "--add-advertisement-ranges": "value",
                    "--advertised-route-priority": "value",
                    "--advertisement-mode": [
                      "CUSTOM",
                      "DEFAULT"
                    ],
                    "--async": "bool",
                    "--bfd-min-receive-interval": "value",
                    "--bfd-min-transmit-interval": "value",
                    "--bfd-multiplier": "value",
                    "--bfd-session-initialization-mode": [
                      "ACTIVE",
                      "DISABLED",
                      "PASSIVE"
                    ],
                    "--clear-md5-authentication-key": "bool",
                    "--enable-ipv6": "bool",
                    "--enabled": "bool",
                    "--interface": "value",
                    "--ip-address": "value",
                    "--ipv6-nexthop-address": "value",
                    "--md5-authentication-key": "value",
                    "--peer-asn": "value",
                    "--peer-ip-address": "value",
                    "--peer-ipv6-nexthop-address": "value",
                    "--peer-name": "value",
                    "--region": "dynamic",
                    "--remove-advertisement-groups": "value",
                    "--remove-advertisement-ranges": "value",
                    "--set-advertisement-groups": "value",
                    "--set-advertisement-ranges": "value"
                  }
                },
                "update-interface": {
                  "commands": {},
                  "flags": {
                    "--interconnect-attachment": "dynamic",
                    "--interconnect-attachment-region": "dynamic",
                    "--interface-name": "value",
                    "--ip-address": "value",
                    "--mask-length": "value",
                    "--region": "dynamic",
                    "--vpn-tunnel": "dynamic",
                    "--vpn-tunnel-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "routes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--destination-range": "value",
                    "--network": "value",
                    "--next-hop-address": "value",
                    "--next-hop-gateway": "dynamic",
                    "--next-hop-ilb": "value",
                    "--next-hop-ilb-region": "value",
                    "--next-hop-instance": "value",
                    "--next-hop-instance-zone": "value",
                    "--next-hop-vpn-tunnel": "value",
                    "--next-hop-vpn-tunnel-region": "value",
                    "--priority": "value",
                    "--tags": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "scp": {
              "commands": {},
              "flags": {
                "--compress": "bool",
                "--dest-group": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--internal-ip": "bool",
                "--network": "value",
                "--plain": "bool",
                "--port": "value",
                "--recurse": "bool",
                "--region": "value",
                "--scp-flag": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--tunnel-through-iap": "bool",
                "--zone": "value"
              }
            },
            "security-policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic",
                    "--type": [
                      "CLOUD_ARMOR",
                      "CLOUD_ARMOR_EDGE",
                      "CLOUD_ARMOR_NETWORK"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--file-format": [
                      "json",
                      "yaml"
                    ],
                    "--file-name": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-preconfigured-expression-sets": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "rules": {
                  "commands": {
                    "add-preconfig-waf-exclusion": {
                      "commands": {},
                      "flags": {
                        "--request-cookie-to-exclude": "value",
                        "--request-header-to-exclude": "value",
                        "--request-query-param-to-exclude": "value",
                        "--request-uri-to-exclude": "value",
                        "--security-policy": "dynamic",
                        "--target-rule-ids": "value",
                        "--target-rule-set": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny-403",
                          "deny-404",
                          "deny-502",
                          "rate-based-ban",
                          "redirect",
                          "redirect-to-recaptcha",
                          "throttle"
                        ],
                        "--ban-duration-sec": "value",
                        "--ban-threshold-count": "value",
                        "--ban-threshold-interval-sec": "value",
                        "--conform-action": [
                          "allow"
                        ],
                        "--description": "value",
                        "--enforce-on-key": [
                          "all",
                          "http-cookie",
                          "http-header",
                          "http-path",
                          "ip",
                          "region-code",
                          "sni",
                          "xff-ip"
                        ],
                        "--enforce-on-key-configs": "value",
                        "--enforce-on-key-name": "value",
                        "--exceed-action": [
                          "deny-403",
                          "deny-404",
                          "deny-429",
                          "deny-502",
                          "redirect"
                        ],
                        "--exceed-redirect-target": "value",
                        "--exceed-redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--expression": "value",
                        "--preview": "bool",
                        "--rate-limit-threshold-count": "value",
                        "--rate-limit-threshold-interval-sec": "value",
                        "--redirect-target": "value",
                        "--redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--request-headers-to-add": "value",
                        "--security-policy": "dynamic",
                        "--src-ip-ranges": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--security-policy": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--security-policy": "dynamic"
                      }
                    },
                    "remove-preconfig-waf-exclusion": {
                      "commands": {},
                      "flags": {
                        "--request-cookie-to-exclude": "value",
                        "--request-header-to-exclude": "value",
                        "--request-query-param-to-exclude": "value",
                        "--request-uri-to-exclude": "value",
                        "--security-policy": "dynamic",
                        "--target-rule-ids": "value",
                        "--target-rule-set": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--action": [
                          "allow",
                          "deny-403",
                          "deny-404",
                          "deny-502",
                          "rate-based-ban",
                          "redirect",
                          "redirect-to-recaptcha",
                          "throttle"
                        ],
                        "--ban-duration-sec": "value",
                        "--ban-threshold-count": "value",
                        "--ban-threshold-interval-sec": "value",
                        "--conform-action": [
                          "allow"
                        ],
                        "--description": "value",
                        "--enforce-on-key": [
                          "all",
                          "http-cookie",
                          "http-header",
                          "http-path",
                          "ip",
                          "region-code",
                          "sni",
                          "xff-ip"
                        ],
                        "--enforce-on-key-configs": "value",
                        "--enforce-on-key-name": "value",
                        "--exceed-action": [
                          "deny-403",
                          "deny-404",
                          "deny-429",
                          "deny-502",
                          "redirect"
                        ],
                        "--exceed-redirect-target": "value",
                        "--exceed-redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--expression": "value",
                        "--preview": "bool",
                        "--rate-limit-threshold-count": "value",
                        "--rate-limit-threshold-interval-sec": "value",
                        "--redirect-target": "value",
                        "--redirect-type": [
                          "external-302",
                          "google-recaptcha"
                        ],
                        "--request-headers-to-add": "value",
                        "--security-policy": "dynamic",
                        "--src-ip-ranges": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-layer7-ddos-defense": "bool",
                    "--global": "bool",
                    "--json-custom-content-types": "value",
                    "--json-parsing": [
                      "DISABLED",
                      "STANDARD"
                    ],
                    "--layer7-ddos-defense-auto-deploy-confidence-threshold": "value",
                    "--layer7-ddos-defense-auto-deploy-expiration-sec": "value",
                    "--layer7-ddos-defense-auto-deploy-impacted-baseline-threshold": "value",
                    "--layer7-ddos-defense-auto-deploy-load-threshold": "value",
                    "--layer7-ddos-defense-rule-visibility": [
                      "PREMIUM",
                      "STANDARD"
                    ],
                    "--log-level": [
                      "NORMAL",
                      "VERBOSE"
                    ],
                    "--network-ddos-protection": [
                      "ADVANCED",
                      "ADVANCED_PREVIEW",
                      "STANDARD"
                    ],
                    "--recaptcha-redirect-site-key": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "service-attachments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--consumer-accept-list": "value",
                    "--consumer-reject-list": "value",
                    "--description": "value",
                    "--domain-names": "value",
                    "--enable-proxy-protocol": "bool",
                    "--nat-subnets": "dynamic",
                    "--nat-subnets-region": "dynamic",
                    "--producer-forwarding-rule": "dynamic",
                    "--producer-forwarding-rule-region": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--connection-preference": [
                      "ACCEPT_AUTOMATIC",
                      "ACCEPT_MANUAL"
                    ],
                    "--consumer-accept-list": "value",
                    "--consumer-reject-list": "value",
                    "--description": "value",
                    "--enable-proxy-protocol": "bool",
                    "--nat-subnets": "dynamic",
                    "--nat-subnets-region": "dynamic",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "shared-vpc": {
              "commands": {
                "associated-projects": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--host-project": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--host-project": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {}
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "get-host-project": {
                  "commands": {},
                  "flags": {}
                },
                "list-associated-resources": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "organizations": {
                  "commands": {
                    "list-host-projects": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "sign-url": {
              "commands": {},
              "flags": {
                "--expires-in": "value",
                "--key-file": "value",
                "--key-name": "value",
                "--validate": "bool"
              }
            },
            "snapshots": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "add-labels": {
                  "commands": {},
                  "flags": {
                    "--labels": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--chain-name": "value",
                    "--csek-key-file": "value",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--labels": "value",
                    "--snapshot-type": [
                      "ARCHIVE",
                      "STANDARD"
                    ],
                    "--source-disk": "dynamic",
                    "--source-disk-key-file": "value",
                    "--source-disk-region": "dynamic",
                    "--source-disk-zone": "dynamic",
                    "--storage-location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-labels": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--labels": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "sole-tenancy": {
              "commands": {
                "node-groups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--role": "dynamic",
                        "--zone": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--autoscaler-mode": [
                          "off",
                          "on",
                          "only-scale-out"
                        ],
                        "--description": "value",
                        "--maintenance-policy": [
                          "default",
                          "migrate-within-node-group",
                          "restart-in-place"
                        ],
                        "--maintenance-window-start-time": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-template": "value",
                        "--share-setting": [
                          "local",
                          "organization",
                          "projects"
                        ],
                        "--share-with": "value",
                        "--target-size": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--share-settings": "bool",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list-nodes": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--role": "dynamic",
                        "--zone": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "simulate-maintenance-event": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--nodes": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--add-nodes": "value",
                        "--autoscaler-mode": [
                          "off",
                          "on",
                          "only-scale-out"
                        ],
                        "--delete-nodes": "value",
                        "--max-nodes": "value",
                        "--min-nodes": "value",
                        "--node-template": "value",
                        "--share-setting": [
                          "local",
                          "organization",
                          "projects"
                        ],
                        "--share-with": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-templates": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator": "value",
                        "--cpu-overcommit-type": [
                          "enabled",
                          "none"
                        ],
                        "--description": "value",
                        "--disk": "value",
                        "--node-affinity-labels": "value",
                        "--node-requirements": "value",
                        "--node-type": "value",
                        "--region": "dynamic",
                        "--server-binding": [
                          "restart-node-on-any-server",
                          "restart-node-on-minimal-servers"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--member": "value",
                        "--region": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "node-types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--command": "value",
                "--container": "value",
                "--dest-group": "value",
                "--dry-run": "bool",
                "--force-key-file-overwrite": "bool",
                "--internal-ip": "bool",
                "--network": "value",
                "--plain": "bool",
                "--region": "value",
                "--ssh-flag": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--troubleshoot": "bool",
                "--tunnel-through-iap": "bool",
                "--zone": "dynamic"
              }
            },
            "ssl-certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate": "value",
                    "--description": "value",
                    "--domains": "value",
                    "--global": "bool",
                    "--private-key": "value",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "ssl-policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--custom-features": "value",
                    "--description": "value",
                    "--global": "bool",
                    "--min-tls-version": [
                      "1.0",
                      "1.1",
                      "1.2"
                    ],
                    "--profile": [
                      "COMPATIBLE",
                      "CUSTOM",
                      "MODERN",
                      "RESTRICTED"
                    ],
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-available-features": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--custom-features": "value",
                    "--global": "bool",
                    "--min-tls-version": [
                      "1.0",
                      "1.1",
                      "1.2"
                    ],
                    "--profile": [
                      "COMPATIBLE",
                      "CUSTOM",
                      "MODERN",
                      "RESTRICTED"
                    ],
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "start-iap-tunnel": {
              "commands": {},
              "flags": {
                "--dest-group": "value",
                "--iap-tunnel-disable-connection-check": "bool",
                "--local-host-port": "value",
                "--network": "value",
                "--region": "value",
                "--zone": "dynamic"
              }
            },
            "target-grpc-proxies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global-url-map": "bool",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic",
                    "--validate-for-proxyless": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "target-http-proxies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--global-url-map": "bool",
                    "--region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--global-url-map": "bool",
                    "--region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-https-proxies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--certificate-map": "dynamic",
                    "--description": "value",
                    "--global": "bool",
                    "--global-ssl-certificates": "bool",
                    "--global-ssl-policy": "bool",
                    "--global-url-map": "bool",
                    "--quic-override": [
                      "DISABLE",
                      "ENABLE",
                      "NONE"
                    ],
                    "--region": "dynamic",
                    "--ssl-certificates": "dynamic",
                    "--ssl-certificates-region": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--certificate-map": "dynamic",
                    "--clear-certificate-map": "bool",
                    "--clear-ssl-policy": "bool",
                    "--global": "bool",
                    "--global-ssl-certificates": "bool",
                    "--global-ssl-policy": "bool",
                    "--global-url-map": "bool",
                    "--quic-override": [
                      "DISABLE",
                      "ENABLE",
                      "NONE"
                    ],
                    "--region": "dynamic",
                    "--ssl-certificates": "dynamic",
                    "--ssl-certificates-region": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic",
                    "--url-map": "dynamic",
                    "--url-map-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--instance": "dynamic",
                    "--instance-zone": "dynamic",
                    "--network": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-pools": {
              "commands": {
                "add-health-checks": {
                  "commands": {},
                  "flags": {
                    "--http-health-check": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "add-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "dynamic",
                    "--instances-zone": "dynamic",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backup-pool": "value",
                    "--description": "value",
                    "--failover-ratio": "value",
                    "--health-check": "value",
                    "--http-health-check": "dynamic",
                    "--region": "dynamic",
                    "--session-affinity": [
                      "CLIENT_IP",
                      "CLIENT_IP_PROTO",
                      "NONE"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-health": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-health-checks": {
                  "commands": {},
                  "flags": {
                    "--http-health-check": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "remove-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "dynamic",
                    "--instances-zone": "dynamic",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "set-backup": {
                  "commands": {},
                  "flags": {
                    "--backup-pool": "dynamic",
                    "--failover-ratio": "value",
                    "--no-backup-pool": "bool",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-ssl-proxies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--certificate-map": "dynamic",
                    "--description": "value",
                    "--global-ssl-policy": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--ssl-certificates": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--certificate-map": "dynamic",
                    "--clear-certificate-map": "bool",
                    "--clear-ssl-certificates": "bool",
                    "--clear-ssl-policy": "bool",
                    "--global-ssl-policy": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--ssl-certificates": "dynamic",
                    "--ssl-policy": "dynamic",
                    "--ssl-policy-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "target-tcp-proxies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--backend-service-region": "dynamic",
                    "--description": "value",
                    "--global": "bool",
                    "--global-backend-service": "bool",
                    "--proxy-bind": "bool",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backend-service": "dynamic",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "target-vpn-gateways": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--network": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "tpus": {
              "commands": {
                "accelerator-types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-type": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--network": "value",
                    "--preemptible": "bool",
                    "--range": "value",
                    "--reserved": "bool",
                    "--use-service-networking": "bool",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "execution-groups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--disk-size": "value",
                        "--dry-run": "bool",
                        "--forward-ports": "bool",
                        "--gce-image": "value",
                        "--machine-type": "value",
                        "--name": "value",
                        "--network": "value",
                        "--preemptible": "bool",
                        "--preemptible-vm": "bool",
                        "--tf-version": "value",
                        "--tpu-only": "bool",
                        "--use-dl-images": "bool",
                        "--use-with-notebook": "bool",
                        "--vm-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--tpu-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "resume": {
                      "commands": {},
                      "flags": {
                        "--accelerator-type": "value",
                        "--network": "value",
                        "--preemptible": "bool",
                        "--tf-version": "value",
                        "--vm-only": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "suspend": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "reimage": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "versions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "url-maps": {
              "commands": {
                "add-host-rule": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--global": "bool",
                    "--hosts": "value",
                    "--path-matcher-name": "value",
                    "--region": "dynamic"
                  }
                },
                "add-path-matcher": {
                  "commands": {},
                  "flags": {
                    "--backend-bucket-path-rules": "value",
                    "--backend-service-path-rules": "value",
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--delete-orphaned-path-matcher": "bool",
                    "--description": "value",
                    "--existing-host": "value",
                    "--global": "bool",
                    "--new-hosts": "value",
                    "--path-matcher-name": "value",
                    "--path-rules": "value",
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--description": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "edit": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--region": "dynamic",
                    "--source": "value"
                  }
                },
                "invalidate-cdn-cache": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--global": "bool",
                    "--host": "value",
                    "--path": "value",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--global": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-cdn-cache-invalidations": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--limit": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-host-rule": {
                  "commands": {},
                  "flags": {
                    "--delete-orphaned-path-matcher": "bool",
                    "--global": "bool",
                    "--host": "value",
                    "--region": "dynamic"
                  }
                },
                "remove-path-matcher": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--path-matcher-name": "value",
                    "--region": "dynamic"
                  }
                },
                "set-default-service": {
                  "commands": {},
                  "flags": {
                    "--default-backend-bucket": "value",
                    "--default-service": "value",
                    "--global": "bool",
                    "--region": "dynamic"
                  }
                },
                "validate": {
                  "commands": {},
                  "flags": {
                    "--global": "bool",
                    "--load-balancing-scheme": [
                      "EXTERNAL",
                      "EXTERNAL_MANAGED"
                    ],
                    "--region": "value",
                    "--source": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpn-gateways": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--interconnect-attachments": "value",
                    "--network": "dynamic",
                    "--region": "dynamic",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpn-tunnels": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--ike-version": [
                      1,
                      2
                    ],
                    "--interface": [
                      0,
                      1
                    ],
                    "--local-traffic-selector": "value",
                    "--peer-address": "value",
                    "--peer-external-gateway": "dynamic",
                    "--peer-external-gateway-interface": [
                      0,
                      1,
                      2,
                      3
                    ],
                    "--peer-gcp-gateway": "dynamic",
                    "--peer-gcp-gateway-region": "dynamic",
                    "--region": "dynamic",
                    "--remote-traffic-selector": "value",
                    "--router": "dynamic",
                    "--router-region": "dynamic",
                    "--shared-secret": "value",
                    "--target-vpn-gateway": "dynamic",
                    "--target-vpn-gateway-region": "dynamic",
                    "--vpn-gateway": "dynamic",
                    "--vpn-gateway-region": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "config": {
          "commands": {
            "configurations": {
              "commands": {
                "activate": {
                  "commands": {},
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--activate": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--all": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                },
                "rename": {
                  "commands": {},
                  "flags": {
                    "--new-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "get": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            },
            "set": {
              "commands": {},
              "flags": {
                "--installation": "bool"
              }
            },
            "unset": {
              "commands": {},
              "flags": {
                "--installation": "bool"
              }
            }
          },
          "flags": {}
        },
        "container": {
          "commands": {
            "backup-restore": {
              "commands": {
                "backup-plans": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-delete-lock-days": "value",
                        "--backup-retain-days": "value",
                        "--cluster": "value",
                        "--cron-schedule": "value",
                        "--deactivated": "bool",
                        "--description": "value",
                        "--encryption-key": "value",
                        "--include-secrets": "bool",
                        "--include-volume-data": "bool",
                        "--labels": "value",
                        "--location": "value",
                        "--locked": "bool",
                        "--paused": "bool",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-delete-lock-days": "value",
                        "--backup-retain-days": "value",
                        "--clear-labels": "bool",
                        "--cron-schedule": "value",
                        "--deactivated": "bool",
                        "--description": "value",
                        "--encryption-key": "value",
                        "--etag": "value",
                        "--include-secrets": "bool",
                        "--include-volume-data": "bool",
                        "--location": "value",
                        "--locked": "bool",
                        "--paused": "bool",
                        "--remove-labels": "value",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "backups": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--delete-lock-days": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--retain-days": "value",
                        "--wait-for-completion": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--clear-labels": "bool",
                        "--delete-lock-days": "value",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--retain-days": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "locations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "restore-plans": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--backup-plan": "value",
                        "--cluster": "value",
                        "--cluster-resource-conflict-policy": [
                          "use-backup-version",
                          "use-existing-version"
                        ],
                        "--cluster-resource-restore-scope": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--namespaced-resource-restore-mode": [
                          "delete-and-restore",
                          "fail-on-conflict"
                        ],
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--substitution-rules-file": "value",
                        "--volume-data-restore-policy": [
                          "no-volume-data-restoration",
                          "restore-volume-data-from-backup",
                          "reuse-volume-handle-from-backup"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--all-namespaces": "bool",
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--cluster-resource-conflict-policy": [
                          "use-backup-version",
                          "use-existing-version"
                        ],
                        "--cluster-resource-restore-scope": "value",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--namespaced-resource-restore-mode": [
                          "delete-and-restore",
                          "fail-on-conflict"
                        ],
                        "--remove-labels": "value",
                        "--selected-applications": "value",
                        "--selected-namespaces": "value",
                        "--substitution-rules-file": "value",
                        "--update-labels": "value",
                        "--volume-data-restore-policy": [
                          "no-volume-data-restoration",
                          "restore-volume-data-from-backup",
                          "reuse-volume-handle-from-backup"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "restores": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--backup": "value",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--restore-plan": "value",
                        "--wait-for-completion": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--etag": "value",
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--etag": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--restore-plan": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "volume-backups": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--backup": "value",
                        "--backup-plan": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--backup": "value",
                        "--backup-plan": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "volume-restores": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--restore": "value",
                        "--restore-plan": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--restore": "value",
                        "--restore-plan": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "binauthz": {
              "commands": {
                "attestations": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--payload-file": "value",
                        "--public-key-id": "value",
                        "--signature-file": "value",
                        "--validate": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "sign-and-create": {
                      "commands": {},
                      "flags": {
                        "--artifact-url": "value",
                        "--attestor": "value",
                        "--attestor-project": "value",
                        "--keyversion": "value",
                        "--keyversion-key": "value",
                        "--keyversion-keyring": "value",
                        "--keyversion-location": "value",
                        "--keyversion-project": "value",
                        "--public-key-id-override": "value",
                        "--validate": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "attestors": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--attestation-authority-note": "value",
                        "--attestation-authority-note-project": "value",
                        "--description": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "public-keys": {
                      "commands": {
                        "add": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value",
                            "--comment": "value",
                            "--keyversion": "value",
                            "--keyversion-key": "value",
                            "--keyversion-keyring": "value",
                            "--keyversion-location": "value",
                            "--keyversion-project": "value",
                            "--pgp-public-key-file": "value",
                            "--pkix-public-key-algorithm": [
                              "ec-sign-p256-sha256",
                              "ec-sign-p384-sha384",
                              "ec-sign-p521-sha512",
                              "ecdsa-p256-sha256",
                              "ecdsa-p384-sha384",
                              "ecdsa-p521-sha512",
                              "rsa-pss-2048-sha256",
                              "rsa-pss-3072-sha256",
                              "rsa-pss-4096-sha256",
                              "rsa-pss-4096-sha512",
                              "rsa-sign-pkcs1-2048-sha256",
                              "rsa-sign-pkcs1-3072-sha256",
                              "rsa-sign-pkcs1-4096-sha256",
                              "rsa-sign-pkcs1-4096-sha512",
                              "rsa-sign-pss-2048-sha256",
                              "rsa-sign-pss-3072-sha256",
                              "rsa-sign-pss-4096-sha256",
                              "rsa-sign-pss-4096-sha512"
                            ],
                            "--pkix-public-key-file": "value",
                            "--public-key-id-override": "value"
                          }
                        },
                        "remove": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--attestor": "value",
                            "--comment": "value",
                            "--pgp-public-key-file": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create-signature-payload": {
                  "commands": {},
                  "flags": {
                    "--artifact-url": "value"
                  }
                },
                "policy": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--strict-validation": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "dynamic"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--additional-zones": "value",
                    "--addons": "value",
                    "--allow-route-overlap": "bool",
                    "--async": "bool",
                    "--autoprovisioning-config-file": "value",
                    "--autoprovisioning-image-type": "value",
                    "--autoprovisioning-locations": "value",
                    "--autoprovisioning-max-surge-upgrade": "value",
                    "--autoprovisioning-max-unavailable-upgrade": "value",
                    "--autoprovisioning-min-cpu-platform": "value",
                    "--autoprovisioning-network-tags": "value",
                    "--autoprovisioning-node-pool-soak-duration": "value",
                    "--autoprovisioning-scopes": "value",
                    "--autoprovisioning-service-account": "value",
                    "--autoprovisioning-standard-rollout-policy": "value",
                    "--autoscaling-profile": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--boot-disk-kms-key": "value",
                    "--cloud-run-config": "value",
                    "--cluster-dns": [
                      "clouddns",
                      "default"
                    ],
                    "--cluster-dns-domain": "value",
                    "--cluster-dns-scope": [
                      "cluster",
                      "vpc"
                    ],
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--cluster-version": "value",
                    "--create-subnetwork": "value",
                    "--database-encryption-key": "value",
                    "--default-max-pods-per-node": "value",
                    "--disable-default-snat": "bool",
                    "--disable-pod-cidr-overprovision": "bool",
                    "--disk-size": "value",
                    "--disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--enable-autoprovisioning": "bool",
                    "--enable-autoprovisioning-autorepair": "bool",
                    "--enable-autoprovisioning-autoupgrade": "bool",
                    "--enable-autoprovisioning-blue-green-upgrade": "bool",
                    "--enable-autoprovisioning-surge-upgrade": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-basic-auth": "bool",
                    "--enable-binauthz": "bool",
                    "--enable-cloud-logging": "bool",
                    "--enable-cloud-monitoring": "bool",
                    "--enable-cloud-run-alpha": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-cost-allocation": "bool",
                    "--enable-dataplane-v2": "bool",
                    "--enable-gke-oidc": "bool",
                    "--enable-google-cloud-access": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-identity-service": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-intra-node-visibility": "bool",
                    "--enable-ip-alias": "bool",
                    "--enable-kubernetes-alpha": "bool",
                    "--enable-l4-ilb-subsetting": "bool",
                    "--enable-legacy-authorization": "bool",
                    "--enable-logging-monitoring-system-only": "bool",
                    "--enable-managed-prometheus": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-master-global-access": "bool",
                    "--enable-network-egress-metering": "bool",
                    "--enable-network-policy": "bool",
                    "--enable-pod-security-policy": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-resource-consumption-metering": "bool",
                    "--enable-service-externalips": "bool",
                    "--enable-shielded-nodes": "bool",
                    "--enable-stackdriver-kubernetes": "bool",
                    "--enable-tpu": "bool",
                    "--enable-tpu-service-networking": "bool",
                    "--enable-vertical-pod-autoscaling": "bool",
                    "--ephemeral-storage": "value",
                    "--ephemeral-storage-local-ssd": "value",
                    "--gateway-api": [
                      "disabled",
                      "standard"
                    ],
                    "--identity-provider": "value",
                    "--image-type": "value",
                    "--ipv6-access-type": [
                      "external",
                      "internal"
                    ],
                    "--issue-client-certificate": "bool",
                    "--istio-config": "value",
                    "--labels": "value",
                    "--local-nvme-ssd-block": "value",
                    "--local-ssd-count": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging": "value",
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--machine-type": "value",
                    "--maintenance-window": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--max-accelerator": "value",
                    "--max-cpu": "value",
                    "--max-memory": "value",
                    "--max-nodes": "value",
                    "--max-nodes-per-pool": "value",
                    "--max-pods-per-node": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-accelerator": "value",
                    "--min-cpu": "value",
                    "--min-cpu-platform": "value",
                    "--min-memory": "value",
                    "--min-nodes": "value",
                    "--monitoring": "value",
                    "--network": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--notification-config": "value",
                    "--num-nodes": "value",
                    "--password": "value",
                    "--placement-type": [
                      "COMPACT",
                      "UNSPECIFIED"
                    ],
                    "--preemptible": "bool",
                    "--private-cluster": "bool",
                    "--private-endpoint-subnetwork": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "disabled",
                      "outbound-only"
                    ],
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-usage-bigquery-dataset": "value",
                    "--scopes": "value",
                    "--security-group": "value",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--spot": "bool",
                    "--stack-type": [
                      "ipv4",
                      "ipv4-ipv6"
                    ],
                    "--subnetwork": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--tpu-ipv4-cidr": "value",
                    "--username": "value",
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--workload-pool": "value",
                    "--zone": "value"
                  }
                },
                "create-auto": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoprovisioning-network-tags": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--boot-disk-kms-key": "value",
                    "--cluster-ipv4-cidr": "value",
                    "--cluster-secondary-range-name": "value",
                    "--cluster-version": "value",
                    "--create-subnetwork": "value",
                    "--database-encryption-key": "value",
                    "--enable-google-cloud-access": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-private-nodes": "bool",
                    "--master-authorized-networks": "value",
                    "--master-ipv4-cidr": "value",
                    "--network": "value",
                    "--private-endpoint-subnetwork": "value",
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--scopes": "value",
                    "--security-group": "value",
                    "--service-account": "value",
                    "--services-ipv4-cidr": "value",
                    "--services-secondary-range-name": "value",
                    "--subnetwork": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--internal-ip": "bool",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--node-pool": "value",
                    "--num-nodes": "value",
                    "--region": "value",
                    "--size": "value",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-maintenance-exclusion-end": "value",
                    "--add-maintenance-exclusion-name": "value",
                    "--add-maintenance-exclusion-scope": "value",
                    "--add-maintenance-exclusion-start": "value",
                    "--additional-zones": "value",
                    "--async": "bool",
                    "--autoprovisioning-config-file": "value",
                    "--autoprovisioning-image-type": "value",
                    "--autoprovisioning-locations": "value",
                    "--autoprovisioning-max-surge-upgrade": "value",
                    "--autoprovisioning-max-unavailable-upgrade": "value",
                    "--autoprovisioning-min-cpu-platform": "value",
                    "--autoprovisioning-network-tags": "value",
                    "--autoprovisioning-node-pool-soak-duration": "value",
                    "--autoprovisioning-scopes": "value",
                    "--autoprovisioning-service-account": "value",
                    "--autoprovisioning-standard-rollout-policy": "value",
                    "--autoscaling-profile": "value",
                    "--binauthz-evaluation-mode": [
                      "DISABLED",
                      "MONITORING",
                      "MONITORING_AND_PROJECT_SINGLETON_POLICY_ENFORCE",
                      "PROJECT_SINGLETON_POLICY_ENFORCE"
                    ],
                    "--binauthz-policy": "value",
                    "--clear-maintenance-window": "bool",
                    "--clear-resource-usage-bigquery-dataset": "bool",
                    "--cloud-run-config": "value",
                    "--cluster-dns": [
                      "clouddns",
                      "default"
                    ],
                    "--cluster-dns-domain": "value",
                    "--cluster-dns-scope": [
                      "cluster",
                      "vpc"
                    ],
                    "--complete-credential-rotation": "bool",
                    "--complete-ip-rotation": "bool",
                    "--database-encryption-key": "value",
                    "--disable-database-encryption": "bool",
                    "--disable-default-snat": "bool",
                    "--disable-managed-prometheus": "bool",
                    "--disable-workload-identity": "bool",
                    "--enable-autoprovisioning": "bool",
                    "--enable-autoprovisioning-autorepair": "bool",
                    "--enable-autoprovisioning-autoupgrade": "bool",
                    "--enable-autoprovisioning-blue-green-upgrade": "bool",
                    "--enable-autoprovisioning-surge-upgrade": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-basic-auth": "bool",
                    "--enable-binauthz": "bool",
                    "--enable-cost-allocation": "bool",
                    "--enable-gke-oidc": "bool",
                    "--enable-google-cloud-access": "bool",
                    "--enable-identity-service": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-intra-node-visibility": "bool",
                    "--enable-l4-ilb-subsetting": "bool",
                    "--enable-legacy-authorization": "bool",
                    "--enable-logging-monitoring-system-only": "bool",
                    "--enable-managed-prometheus": "bool",
                    "--enable-master-authorized-networks": "bool",
                    "--enable-master-global-access": "bool",
                    "--enable-network-egress-metering": "bool",
                    "--enable-network-policy": "bool",
                    "--enable-pod-security-policy": "bool",
                    "--enable-private-endpoint": "bool",
                    "--enable-resource-consumption-metering": "bool",
                    "--enable-service-externalips": "bool",
                    "--enable-shielded-nodes": "bool",
                    "--enable-stackdriver-kubernetes": "bool",
                    "--enable-tpu": "bool",
                    "--enable-tpu-service-networking": "bool",
                    "--enable-vertical-pod-autoscaling": "bool",
                    "--gateway-api": [
                      "disabled",
                      "standard"
                    ],
                    "--generate-password": "bool",
                    "--identity-provider": "value",
                    "--istio-config": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging": "value",
                    "--logging-service": "value",
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--maintenance-window": "value",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--master-authorized-networks": "value",
                    "--max-accelerator": "value",
                    "--max-cpu": "value",
                    "--max-memory": "value",
                    "--max-nodes": "value",
                    "--min-accelerator": "value",
                    "--min-cpu": "value",
                    "--min-memory": "value",
                    "--min-nodes": "value",
                    "--monitoring": "value",
                    "--monitoring-service": "value",
                    "--node-locations": "value",
                    "--node-pool": "value",
                    "--notification-config": "value",
                    "--password": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "disabled",
                      "outbound-only"
                    ],
                    "--region": "value",
                    "--release-channel": [
                      "None",
                      "rapid",
                      "regular",
                      "stable"
                    ],
                    "--remove-labels": "value",
                    "--remove-maintenance-exclusion": "value",
                    "--resource-usage-bigquery-dataset": "value",
                    "--security-group": "value",
                    "--set-password": "bool",
                    "--stack-type": [
                      "ipv4",
                      "ipv4-ipv6"
                    ],
                    "--start-credential-rotation": "bool",
                    "--start-ip-rotation": "bool",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--tpu-ipv4-cidr": "value",
                    "--update-addons": "value",
                    "--update-labels": "value",
                    "--username": "value",
                    "--workload-pool": "value",
                    "--zone": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster-version": "value",
                    "--image-type": "value",
                    "--master": "bool",
                    "--node-pool": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "fleet": {
              "commands": {
                "cloudrun": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "config-management": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "status": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "unmanage": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "upgrade": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "version": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "features": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "identity-service": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "bool",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "ingress": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "bindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "generate-gateway-rbac": {
                      "commands": {},
                      "flags": {
                        "--anthos-support": "bool",
                        "--apply": "bool",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--membership": "value",
                        "--rbac-output-file": "value",
                        "--revoke": "bool",
                        "--role": "value",
                        "--users": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "register": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--enable-workload-identity": "bool",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--has-private-issuer": "bool",
                        "--install-connect-agent": "bool",
                        "--internal-ip": "bool",
                        "--kubeconfig": "value",
                        "--manifest-output-file": "value",
                        "--proxy": "value",
                        "--public-issuer-url": "value",
                        "--service-account-key-file": "value"
                      }
                    },
                    "unregister": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--uninstall-connect-agent": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "mesh": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "multi-cluster-services": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "namespaces": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--membership": "value",
                        "--membership-location": "value",
                        "--set-namespace-in-config": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rbacrolebindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--namespace": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "scopes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "get-server-config": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "hub": {
              "commands": {
                "cloudrun": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "config-management": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    },
                    "status": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "unmanage": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "upgrade": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "version": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "features": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "identity-service": {
                  "commands": {
                    "apply": {
                      "commands": {},
                      "flags": {
                        "--config": "value",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "bool",
                        "--location": "value",
                        "--membership": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--fleet-default-member-config": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "ingress": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-membership": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "bindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--membership": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--fleet": "value",
                            "--location": "value",
                            "--membership": "value",
                            "--scope": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "generate-gateway-rbac": {
                      "commands": {},
                      "flags": {
                        "--anthos-support": "bool",
                        "--apply": "bool",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--membership": "value",
                        "--rbac-output-file": "value",
                        "--revoke": "bool",
                        "--role": "value",
                        "--users": "value"
                      }
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "register": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--enable-workload-identity": "bool",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--has-private-issuer": "bool",
                        "--install-connect-agent": "bool",
                        "--internal-ip": "bool",
                        "--kubeconfig": "value",
                        "--manifest-output-file": "value",
                        "--proxy": "value",
                        "--public-issuer-url": "value",
                        "--service-account-key-file": "value"
                      }
                    },
                    "unregister": {
                      "commands": {},
                      "flags": {
                        "--context": "value",
                        "--gke-cluster": "value",
                        "--gke-uri": "value",
                        "--kubeconfig": "value",
                        "--location": "value",
                        "--uninstall-connect-agent": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "mesh": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "multi-cluster-services": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--force": "bool"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "namespaces": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {}
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-credentials": {
                      "commands": {},
                      "flags": {
                        "--membership": "value",
                        "--membership-location": "value",
                        "--set-namespace-in-config": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rbacrolebindings": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--namespace": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--namespace": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--group": "value",
                            "--location": "value",
                            "--namespace": "value",
                            "--role": [
                              "admin",
                              "edit",
                              "view"
                            ],
                            "--user": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--scope": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "scopes": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "images": {
              "commands": {
                "add-tag": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force-delete-tags": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--metadata-filter": "value",
                    "--show-all-metadata": "bool",
                    "--show-build-details": "bool",
                    "--show-deployment": "bool",
                    "--show-image-basis": "bool",
                    "--show-package-vulnerability": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--repository": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-tags": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--occurrence-filter": "value",
                    "--page-size": "value",
                    "--show-occurrences": "bool",
                    "--show-occurrences-from": "value",
                    "--sort-by": "value"
                  }
                },
                "untag": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "node-pools": {
              "commands": {
                "complete-upgrade": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--async": "bool",
                    "--boot-disk-kms-key": "value",
                    "--cluster": "value",
                    "--create-pod-ipv4-range": "value",
                    "--disable-pod-cidr-overprovision": "bool",
                    "--disk-size": "value",
                    "--disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--enable-autoprovisioning": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-blue-green-upgrade": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-surge-upgrade": "bool",
                    "--ephemeral-storage": "value",
                    "--ephemeral-storage-local-ssd": "value",
                    "--image-type": "value",
                    "--labels": "value",
                    "--local-nvme-ssd-block": "value",
                    "--local-ssd-count": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--max-pods-per-node": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-cpu-platform": "value",
                    "--min-nodes": "value",
                    "--network-performance-configs": "value",
                    "--node-group": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-pool-soak-duration": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--num-nodes": "value",
                    "--placement-type": [
                      "COMPACT",
                      "UNSPECIFIED"
                    ],
                    "--pod-ipv4-range": "value",
                    "--preemptible": "bool",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--sandbox": "value",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--spot": "bool",
                    "--standard-rollout-policy": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--threads-per-core": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--windows-os-version": [
                      "ltsc2019",
                      "ltsc2022"
                    ],
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value",
                    "--respect-pdb": "value",
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--enable-autoprovisioning": "bool",
                    "--enable-autorepair": "bool",
                    "--enable-autoscaling": "bool",
                    "--enable-autoupgrade": "bool",
                    "--enable-blue-green-upgrade": "bool",
                    "--enable-confidential-nodes": "bool",
                    "--enable-gvnic": "bool",
                    "--enable-image-streaming": "bool",
                    "--enable-private-nodes": "bool",
                    "--enable-surge-upgrade": "bool",
                    "--labels": "value",
                    "--location-policy": [
                      "ANY",
                      "BALANCED"
                    ],
                    "--logging-variant": [
                      "DEFAULT",
                      "MAX_THROUGHPUT"
                    ],
                    "--max-nodes": "value",
                    "--max-surge-upgrade": "value",
                    "--max-unavailable-upgrade": "value",
                    "--min-nodes": "value",
                    "--network-performance-configs": "value",
                    "--node-labels": "value",
                    "--node-locations": "value",
                    "--node-pool-soak-duration": "value",
                    "--node-taints": "value",
                    "--region": "value",
                    "--standard-rollout-policy": "value",
                    "--system-config-from-file": "value",
                    "--tags": "value",
                    "--total-max-nodes": "value",
                    "--total-min-nodes": "value",
                    "--windows-os-version": [
                      "ltsc2019",
                      "ltsc2022"
                    ],
                    "--workload-metadata": [
                      "EXPOSED",
                      "GCE_METADATA",
                      "GKE_METADATA",
                      "GKE_METADATA_SERVER",
                      "SECURE"
                    ],
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "subnets": {
              "commands": {
                "list-usable": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network-project": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "data-catalog": {
          "commands": {
            "entries": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--entry-group": "value",
                    "--gcs-file-patterns": "value",
                    "--location": "value",
                    "--schema": "value",
                    "--schema-from-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--entry-group": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "lookup": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-file-patterns": "value",
                    "--clear-file-patterns": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--lookup-entry": "value",
                    "--remove-file-patterns": "value",
                    "--schema": "value",
                    "--schema-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "entry-groups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "search": {
              "commands": {},
              "flags": {
                "--include-gcp-public-datasets": "bool",
                "--include-organization-ids": "value",
                "--include-project-ids": "value",
                "--limit": "value",
                "--order-by": "value",
                "--page-size": "value",
                "--restricted-locations": "value"
              }
            },
            "tag-templates": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--field": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "fields": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--display-name": "value",
                        "--location": "value",
                        "--tag-template": "value",
                        "--type": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--force": "bool",
                        "--location": "value",
                        "--tag-template": "value"
                      }
                    },
                    "enum-values": {
                      "commands": {
                        "rename": {
                          "commands": {},
                          "flags": {
                            "--field": "value",
                            "--location": "value",
                            "--new-id": "value",
                            "--tag-template": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "rename": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--new-id": "value",
                        "--tag-template": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--display-name": "value",
                        "--enum-values": "value",
                        "--location": "value",
                        "--required": "bool",
                        "--tag-template": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--scope": "value",
                    "--tag-file": "value",
                    "--tag-template": "value",
                    "--tag-template-location": "value",
                    "--tag-template-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--entry": "value",
                    "--entry-group": "value",
                    "--location": "value",
                    "--tag-file": "value",
                    "--tag-template": "value",
                    "--tag-template-location": "value",
                    "--tag-template-project": "value"
                  }
                }
              },
              "flags": {}
            },
            "taxonomies": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "policy-tags": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--taxonomy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--taxonomy": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--taxonomy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--taxonomy": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--taxonomy": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--taxonomy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "data-fusion": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--namespace": "value",
                "--role": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--edition": [
                      "basic",
                      "developer",
                      "enterprise"
                    ],
                    "--enable_rbac": "bool",
                    "--enable_stackdriver_logging": "bool",
                    "--enable_stackdriver_monitoring": "bool",
                    "--labels": "value",
                    "--location": "value",
                    "--options": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--enable_rbac": "bool",
                    "--enable_stackdriver_logging": "bool",
                    "--enable_stackdriver_monitoring": "bool",
                    "--labels": "value",
                    "--location": "value",
                    "--options": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--namespace": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value"
              }
            }
          },
          "flags": {}
        },
        "dataflow": {
          "commands": {
            "flex-template": {
              "commands": {
                "build": {
                  "commands": {},
                  "flags": {
                    "--additional-experiments": "value",
                    "--additional-user-labels": "value",
                    "--dataflow-kms-key": "value",
                    "--disable-public-ips": "bool",
                    "--enable-streaming-engine": "bool",
                    "--env": "value",
                    "--flex-template-base-image": "value",
                    "--gcs-log-dir": "value",
                    "--go-binary-path": "value",
                    "--image": "value",
                    "--image-gcr-path": "value",
                    "--image-repository-cert-path": "value",
                    "--image-repository-password-secret-id": "value",
                    "--image-repository-username-secret-id": "value",
                    "--jar": "value",
                    "--max-workers": "value",
                    "--metadata-file": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--print-only": "bool",
                    "--py-path": "value",
                    "--sdk-language": [
                      "GO",
                      "JAVA",
                      "PYTHON"
                    ],
                    "--service-account-email": "value",
                    "--staging-location": "value",
                    "--subnetwork": "value",
                    "--temp-location": "value",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--additional-experiments": "value",
                    "--additional-user-labels": "value",
                    "--dataflow-kms-key": "value",
                    "--disable-public-ips": "bool",
                    "--enable-streaming-engine": "bool",
                    "--flexrs-goal": [
                      "COST_OPTIMIZED",
                      "SPEED_OPTIMIZED"
                    ],
                    "--max-workers": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--service-account-email": "value",
                    "--staging-location": "value",
                    "--subnetwork": "value",
                    "--temp-location": "value",
                    "--template-file-gcs-location": "value",
                    "--transform-name-mappings": "value",
                    "--update": "bool",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--full": "bool",
                    "--region": "value"
                  }
                },
                "drain": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export-steps": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--created-after": "value",
                    "--created-before": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--status": [
                      "active",
                      "all",
                      "terminated"
                    ],
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--additional-experiments": "value",
                    "--dataflow-kms-key": "value",
                    "--disable-public-ips": "bool",
                    "--enable-streaming-engine": "bool",
                    "--gcs-location": "value",
                    "--max-workers": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--service-account-email": "value",
                    "--staging-location": "value",
                    "--subnetwork": "value",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value",
                    "--zone": "value"
                  }
                },
                "show": {
                  "commands": {},
                  "flags": {
                    "--environment": "bool",
                    "--region": "value",
                    "--steps": "bool"
                  }
                }
              },
              "flags": {}
            },
            "logs": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--after": "value",
                    "--before": "value",
                    "--filter": "value",
                    "--importance": [
                      "debug",
                      "detailed",
                      "error",
                      "warning"
                    ],
                    "--limit": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--changed-after": "value",
                    "--filter": "value",
                    "--hide-committed": "bool",
                    "--limit": "value",
                    "--region": "value",
                    "--source": [
                      "all",
                      "service",
                      "user"
                    ],
                    "--tentative": "bool",
                    "--transform": "value"
                  }
                }
              },
              "flags": {}
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--job-id": "value",
                    "--region": "value",
                    "--snapshot-sources": "value",
                    "--snapshot-ttl": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--job-id": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "sql": {
              "commands": {
                "query": {
                  "commands": {},
                  "flags": {
                    "--bigquery-dataset": "value",
                    "--bigquery-project": "value",
                    "--bigquery-table": "value",
                    "--bigquery-write-disposition": [
                      "write-append",
                      "write-empty",
                      "write-truncate"
                    ],
                    "--dataflow-kms-key": "value",
                    "--disable-public-ips": "bool",
                    "--dry-run": "bool",
                    "--job-name": "value",
                    "--max-workers": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--parameter": "value",
                    "--parameters-file": "value",
                    "--pubsub-create-disposition": [
                      "create-if-not-found",
                      "fail-if-not-found"
                    ],
                    "--pubsub-project": "value",
                    "--pubsub-topic": "value",
                    "--region": "value",
                    "--service-account-email": "value",
                    "--subnetwork": "value",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "datapipelines": {
          "commands": {
            "pipeline": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--additional-experiments": "value",
                    "--additional-user-labels": "value",
                    "--dataflow-kms-key": "value",
                    "--dataflow-service-account-email": "value",
                    "--disable-public-ips": "bool",
                    "--display-name": "value",
                    "--enable-streaming-engine": "bool",
                    "--flexrs-goal": [
                      "COST_OPTIMIZED",
                      "SPEED_OPTIMIZED"
                    ],
                    "--max-workers": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--parameters": "value",
                    "--pipeline-type": [
                      "batch",
                      "streaming"
                    ],
                    "--region": "value",
                    "--schedule": "value",
                    "--subnetwork": "value",
                    "--temp-location": "value",
                    "--template-file-gcs-location": "value",
                    "--template-type": [
                      "classic",
                      "flex"
                    ],
                    "--time-zone": "value",
                    "--transform-name-mappings": "value",
                    "--update": "bool",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "job": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--pipeline": "dynamic",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--additional-experiments": "value",
                    "--additional-user-labels": "value",
                    "--dataflow-kms-key": "value",
                    "--dataflow-service-account-email": "value",
                    "--disable-public-ips": "bool",
                    "--display-name": "value",
                    "--enable-streaming-engine": "bool",
                    "--flexrs-goal": [
                      "COST_OPTIMIZED",
                      "SPEED_OPTIMIZED"
                    ],
                    "--max-workers": "value",
                    "--network": "value",
                    "--num-workers": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--schedule": "value",
                    "--subnetwork": "value",
                    "--temp-location": "value",
                    "--template-file-gcs-location": "value",
                    "--template-type": [
                      "classic",
                      "flex"
                    ],
                    "--time-zone": "value",
                    "--transform-name-mappings": "value",
                    "--update": "bool",
                    "--worker-machine-type": "value",
                    "--worker-region": "value",
                    "--worker-zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dataproc": {
          "commands": {
            "autoscaling-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "batches": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "pyspark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--batch": "value",
                        "--container-image": "value",
                        "--deps-bucket": "value",
                        "--files": "value",
                        "--history-server-cluster": "value",
                        "--jars": "value",
                        "--kms-key": "value",
                        "--labels": "value",
                        "--metastore-service": "value",
                        "--network": "value",
                        "--properties": "value",
                        "--py-files": "value",
                        "--region": "value",
                        "--request-id": "value",
                        "--service-account": "value",
                        "--subnet": "value",
                        "--tags": "value",
                        "--version": "value"
                      }
                    },
                    "spark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--batch": "value",
                        "--class": "value",
                        "--container-image": "value",
                        "--deps-bucket": "value",
                        "--files": "value",
                        "--history-server-cluster": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--kms-key": "value",
                        "--labels": "value",
                        "--metastore-service": "value",
                        "--network": "value",
                        "--properties": "value",
                        "--region": "value",
                        "--request-id": "value",
                        "--service-account": "value",
                        "--subnet": "value",
                        "--tags": "value",
                        "--version": "value"
                      }
                    },
                    "spark-r": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--batch": "value",
                        "--container-image": "value",
                        "--deps-bucket": "value",
                        "--files": "value",
                        "--history-server-cluster": "value",
                        "--kms-key": "value",
                        "--labels": "value",
                        "--metastore-service": "value",
                        "--network": "value",
                        "--properties": "value",
                        "--region": "value",
                        "--request-id": "value",
                        "--service-account": "value",
                        "--subnet": "value",
                        "--tags": "value",
                        "--version": "value"
                      }
                    },
                    "spark-sql": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--batch": "value",
                        "--container-image": "value",
                        "--deps-bucket": "value",
                        "--history-server-cluster": "value",
                        "--jars": "value",
                        "--kms-key": "value",
                        "--labels": "value",
                        "--metastore-service": "value",
                        "--network": "value",
                        "--properties": "value",
                        "--region": "value",
                        "--request-id": "value",
                        "--service-account": "value",
                        "--subnet": "value",
                        "--tags": "value",
                        "--vars": "value",
                        "--version": "value"
                      }
                    }
                  },
                  "flags": {
                    "--async": "bool",
                    "--batch": "value",
                    "--container-image": "value",
                    "--history-server-cluster": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--metastore-service": "value",
                    "--network": "value",
                    "--properties": "value",
                    "--region": "value",
                    "--request-id": "value",
                    "--service-account": "value",
                    "--subnet": "value",
                    "--tags": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-policy": "value",
                    "--bucket": "value",
                    "--dataproc-metastore": "value",
                    "--driver-pool-accelerator": "value",
                    "--driver-pool-boot-disk-size": "value",
                    "--driver-pool-boot-disk-type": "value",
                    "--driver-pool-id": "value",
                    "--driver-pool-local-ssd-interface": "value",
                    "--driver-pool-machine-type": "value",
                    "--driver-pool-min-cpu-platform": "value",
                    "--driver-pool-size": "value",
                    "--enable-component-gateway": "bool",
                    "--enable-kerberos": "bool",
                    "--expiration-time": "value",
                    "--gce-pd-kms-key": "dynamic",
                    "--gce-pd-kms-key-keyring": "dynamic",
                    "--gce-pd-kms-key-location": "dynamic",
                    "--gce-pd-kms-key-project": "dynamic",
                    "--image": "value",
                    "--image-version": "value",
                    "--initialization-action-timeout": "value",
                    "--initialization-actions": "value",
                    "--kerberos-config-file": "value",
                    "--kerberos-kms-key": "dynamic",
                    "--kerberos-kms-key-keyring": "dynamic",
                    "--kerberos-kms-key-location": "dynamic",
                    "--kerberos-kms-key-project": "dynamic",
                    "--kerberos-root-principal-password-uri": "value",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-boot-disk-size": "value",
                    "--master-boot-disk-type": "value",
                    "--master-local-ssd-interface": "value",
                    "--master-machine-type": "value",
                    "--master-min-cpu-platform": "value",
                    "--max-age": "value",
                    "--max-idle": "value",
                    "--metadata": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--node-group": "value",
                    "--num-driver-pool-local-ssds": "value",
                    "--num-master-local-ssds": "value",
                    "--num-masters": "value",
                    "--num-secondary-worker-local-ssds": "value",
                    "--num-secondary-workers": "value",
                    "--num-worker-local-ssds": "value",
                    "--num-workers": "value",
                    "--optional-components": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "inherit-subnetwork",
                      "outbound"
                    ],
                    "--properties": "value",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--scopes": "value",
                    "--secondary-worker-accelerator": "value",
                    "--secondary-worker-boot-disk-size": "value",
                    "--secondary-worker-boot-disk-type": "value",
                    "--secondary-worker-local-ssd-interface": "value",
                    "--secondary-worker-type": [
                      "non-preemptible",
                      "preemptible",
                      "spot"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--single-node": "bool",
                    "--subnet": "value",
                    "--tags": "value",
                    "--temp-bucket": "value",
                    "--worker-accelerator": "value",
                    "--worker-boot-disk-size": "value",
                    "--worker-boot-disk-type": "value",
                    "--worker-local-ssd-interface": "value",
                    "--worker-machine-type": "value",
                    "--worker-min-cpu-platform": "value",
                    "--zone": "value"
                  }
                },
                "create-from-file": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "diagnose": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "gke": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--gke-cluster": "value",
                        "--gke-cluster-location": "value",
                        "--history-server-cluster": "value",
                        "--history-server-cluster-region": "value",
                        "--metastore-service": "value",
                        "--metastore-service-location": "value",
                        "--namespace": "value",
                        "--pools": "value",
                        "--properties": "value",
                        "--region": "value",
                        "--setup-workload-identity": "bool",
                        "--spark-engine-version": "value",
                        "--staging-bucket": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--autoscaling-policy": "value",
                    "--clear-labels": "bool",
                    "--disable-autoscaling": "bool",
                    "--expiration-time": "value",
                    "--graceful-decommission-timeout": "value",
                    "--max-age": "value",
                    "--max-idle": "value",
                    "--no-max-age": "bool",
                    "--no-max-idle": "bool",
                    "--num-secondary-workers": "value",
                    "--num-workers": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "jobs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "kill": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--state-filter": [
                      "active",
                      "inactive"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "hadoop": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--class": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "hive": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "pig": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "presto": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--client-tags": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value"
                      }
                    },
                    "pyspark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--py-files": "value",
                        "--region": "value"
                      }
                    },
                    "spark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--class": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "spark-r": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "spark-sql": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value"
                      }
                    },
                    "trino": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bucket": "value",
                        "--client-tags": "value",
                        "--cluster": "value",
                        "--cluster-labels": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--max-failures-per-hour": "value",
                        "--max-failures-total": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "node-groups": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--graceful-decommission-timeout": "value",
                    "--region": "value",
                    "--size": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--state-filter": [
                      "active",
                      "inactive"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "workflow-templates": {
              "commands": {
                "add-job": {
                  "commands": {
                    "hadoop": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--class": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "hive": {
                      "commands": {},
                      "flags": {
                        "--continue-on-failure": "bool",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "pig": {
                      "commands": {},
                      "flags": {
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "presto": {
                      "commands": {},
                      "flags": {
                        "--client-tags": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "pyspark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--py-files": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--class": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--jar": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark-r": {
                      "commands": {},
                      "flags": {
                        "--archives": "value",
                        "--driver-log-levels": "value",
                        "--files": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "spark-sql": {
                      "commands": {},
                      "flags": {
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--jars": "value",
                        "--labels": "value",
                        "--params": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    },
                    "trino": {
                      "commands": {},
                      "flags": {
                        "--client-tags": "value",
                        "--continue-on-failure": "bool",
                        "--driver-log-levels": "value",
                        "--execute": "value",
                        "--file": "value",
                        "--labels": "value",
                        "--properties": "value",
                        "--properties-file": "value",
                        "--query-output-format": "value",
                        "--region": "value",
                        "--start-after": "value",
                        "--step-id": "value",
                        "--workflow-template": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dag-timeout": "value",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--source": "value"
                  }
                },
                "instantiate": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--parameters": "value",
                    "--region": "value"
                  }
                },
                "instantiate-from-file": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-dag-timeout": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "remove-job": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--step-id": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "set-cluster-selector": {
                  "commands": {},
                  "flags": {
                    "--cluster-labels": "value",
                    "--region": "value"
                  }
                },
                "set-dag-timeout": {
                  "commands": {},
                  "flags": {
                    "--dag-timeout": "value",
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "set-managed-cluster": {
                  "commands": {},
                  "flags": {
                    "--autoscaling-policy": "value",
                    "--bucket": "value",
                    "--cluster-name": "value",
                    "--dataproc-metastore": "value",
                    "--enable-component-gateway": "bool",
                    "--enable-kerberos": "bool",
                    "--image": "value",
                    "--image-version": "value",
                    "--initialization-action-timeout": "value",
                    "--initialization-actions": "value",
                    "--kerberos-config-file": "value",
                    "--kerberos-kms-key": "dynamic",
                    "--kerberos-kms-key-keyring": "dynamic",
                    "--kerberos-kms-key-location": "dynamic",
                    "--kerberos-kms-key-project": "dynamic",
                    "--kerberos-root-principal-password-uri": "value",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-boot-disk-size": "value",
                    "--master-boot-disk-type": "value",
                    "--master-local-ssd-interface": "value",
                    "--master-machine-type": "value",
                    "--master-min-cpu-platform": "value",
                    "--metadata": "value",
                    "--network": "value",
                    "--no-address": "bool",
                    "--node-group": "value",
                    "--num-master-local-ssds": "value",
                    "--num-masters": "value",
                    "--num-secondary-worker-local-ssds": "value",
                    "--num-secondary-workers": "value",
                    "--num-worker-local-ssds": "value",
                    "--num-workers": "value",
                    "--optional-components": "value",
                    "--private-ipv6-google-access-type": [
                      "bidirectional",
                      "inherit-subnetwork",
                      "outbound"
                    ],
                    "--properties": "value",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--scopes": "value",
                    "--secondary-worker-accelerator": "value",
                    "--secondary-worker-boot-disk-size": "value",
                    "--secondary-worker-boot-disk-type": "value",
                    "--secondary-worker-local-ssd-interface": "value",
                    "--secondary-worker-type": [
                      "non-preemptible",
                      "preemptible",
                      "spot"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--single-node": "bool",
                    "--subnet": "value",
                    "--tags": "value",
                    "--temp-bucket": "value",
                    "--worker-accelerator": "value",
                    "--worker-boot-disk-size": "value",
                    "--worker-boot-disk-type": "value",
                    "--worker-local-ssd-interface": "value",
                    "--worker-machine-type": "value",
                    "--worker-min-cpu-platform": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "datastore": {
          "commands": {
            "databases": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--kinds": "value",
                "--namespaces": "value",
                "--operation-labels": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--kinds": "value",
                "--namespaces": "value",
                "--operation-labels": "value"
              }
            },
            "indexes": {
              "commands": {
                "cleanup": {
                  "commands": {},
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "datastream": {
          "commands": {
            "connection-profiles": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--ca-certificate": "value",
                    "--client-certificate": "value",
                    "--client-key": "value",
                    "--database-service": "value",
                    "--display-name": "value",
                    "--forward-ssh-hostname": "value",
                    "--forward-ssh-password": "value",
                    "--forward-ssh-port": "value",
                    "--forward-ssh-private-key": "value",
                    "--forward-ssh-username": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--mysql-hostname": "value",
                    "--mysql-password": "value",
                    "--mysql-port": "value",
                    "--mysql-prompt-for-password": "bool",
                    "--mysql-username": "value",
                    "--no-connectivity": "bool",
                    "--oracle-hostname": "value",
                    "--oracle-password": "value",
                    "--oracle-port": "value",
                    "--oracle-prompt-for-password": "bool",
                    "--oracle-username": "value",
                    "--postgresql-database": "value",
                    "--postgresql-hostname": "value",
                    "--postgresql-password": "value",
                    "--postgresql-port": "value",
                    "--postgresql-prompt-for-password": "bool",
                    "--postgresql-username": "value",
                    "--private-connection-name": "value",
                    "--root-path": "value",
                    "--static-ip-connectivity": "bool",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "discover": {
                  "commands": {},
                  "flags": {
                    "--connection-profile-name": "value",
                    "--connection-profile-object-file": "value",
                    "--full-hierarchy": "bool",
                    "--hierarchy-depth": "value",
                    "--location": "value",
                    "--mysql-rdbms-file": "value",
                    "--oracle-rdbms-file": "value",
                    "--postgresql-rdbms-file": "value",
                    "--recursive": "bool",
                    "--recursive-depth": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--ca-certificate": "value",
                    "--clear-labels": "bool",
                    "--client-certificate": "value",
                    "--client-key": "value",
                    "--database-service": "value",
                    "--display-name": "value",
                    "--forward-ssh-hostname": "value",
                    "--forward-ssh-password": "value",
                    "--forward-ssh-port": "value",
                    "--forward-ssh-private-key": "value",
                    "--forward-ssh-username": "value",
                    "--location": "value",
                    "--mysql-hostname": "value",
                    "--mysql-password": "value",
                    "--mysql-port": "value",
                    "--mysql-prompt-for-password": "bool",
                    "--mysql-username": "value",
                    "--no-connectivity": "bool",
                    "--oracle-hostname": "value",
                    "--oracle-password": "value",
                    "--oracle-port": "value",
                    "--oracle-prompt-for-password": "bool",
                    "--oracle-username": "value",
                    "--postgresql-database": "value",
                    "--postgresql-hostname": "value",
                    "--postgresql-password": "value",
                    "--postgresql-port": "value",
                    "--postgresql-prompt-for-password": "bool",
                    "--postgresql-username": "value",
                    "--private-connection-name": "value",
                    "--remove-labels": "value",
                    "--root-path": "value",
                    "--static-ip-connectivity": "bool",
                    "--type": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "fetch-static-ips": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "private-connections": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--subnet": "value",
                    "--vpc-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "routes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--destination-address": "value",
                    "--destination-port": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--private-connection": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--private-connection": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--private-connection": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--private-connection": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "streams": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--backfill-all": "bool",
                    "--backfill-none": "bool",
                    "--bigquery-destination-config": "value",
                    "--destination-name": "value",
                    "--display-name": "value",
                    "--force": "bool",
                    "--gcs-destination-config": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--mysql-excluded-objects": "value",
                    "--mysql-source-config": "value",
                    "--oracle-excluded-objects": "value",
                    "--oracle-source-config": "value",
                    "--postgresql-excluded-objects": "value",
                    "--postgresql-source-config": "value",
                    "--source-name": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backfill-all": "bool",
                    "--backfill-none": "bool",
                    "--bigquery-destination-config": "value",
                    "--clear-labels": "bool",
                    "--destination-name": "value",
                    "--display-name": "value",
                    "--force": "bool",
                    "--gcs-destination-config": "value",
                    "--location": "value",
                    "--mysql-excluded-objects": "value",
                    "--mysql-source-config": "value",
                    "--oracle-excluded-objects": "value",
                    "--oracle-source-config": "value",
                    "--postgresql-excluded-objects": "value",
                    "--postgresql-source-config": "value",
                    "--remove-labels": "value",
                    "--source-name": "value",
                    "--state": "value",
                    "--update-labels": "value",
                    "--update-mask": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "debug": {
          "commands": {
            "logpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--log-level": [
                      "error",
                      "info",
                      "warning"
                    ],
                    "--target": "value",
                    "--wait": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--all-users": "bool",
                    "--include-inactive": "bool",
                    "--location": "value",
                    "--target": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--all-users": "bool",
                    "--filter": "value",
                    "--include-inactive": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--sort-by": "value",
                    "--target": "value"
                  }
                }
              },
              "flags": {
                "--target": "value"
              }
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--expression": "value",
                    "--target": "value",
                    "--wait": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--all-users": "bool",
                    "--include-inactive": "bool",
                    "--location": "value",
                    "--target": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--target": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--all-users": "bool",
                    "--filter": "value",
                    "--include-inactive": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--sort-by": "value",
                    "--target": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--all-users": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--sort-by": "value",
                    "--target": "value",
                    "--timeout": "value"
                  }
                }
              },
              "flags": {
                "--target": "value"
              }
            },
            "source": {
              "commands": {
                "gen-repo-info-file": {
                  "commands": {},
                  "flags": {
                    "--output-directory": "value",
                    "--source-directory": "value"
                  }
                }
              },
              "flags": {}
            },
            "targets": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--include-inactive": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "deploy": {
          "commands": {
            "apply": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--force": "bool",
                "--region": "value"
              }
            },
            "delivery-pipelines": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "job-runs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "releases": {
              "commands": {
                "abandon": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--build-artifacts": "value",
                    "--delivery-pipeline": "dynamic",
                    "--description": "value",
                    "--disable-initial-rollout": "bool",
                    "--enable-initial-rollout": "bool",
                    "--from-k8s-manifest": "value",
                    "--from-run-manifest": "value",
                    "--gcs-source-staging-dir": "value",
                    "--ignore-file": "value",
                    "--images": "value",
                    "--initial-rollout-annotations": "value",
                    "--initial-rollout-labels": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--skaffold-file": "value",
                    "--skaffold-version": "value",
                    "--source": "value",
                    "--to-target": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "promote": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "dynamic",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--rollout-id": "value",
                    "--to-target": "value"
                  }
                }
              },
              "flags": {}
            },
            "rollouts": {
              "commands": {
                "approve": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--release": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reject": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                },
                "retry-job": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "dynamic",
                    "--job-id": "value",
                    "--phase-id": "value",
                    "--region": "dynamic",
                    "--release": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "targets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--delivery-pipeline": "value",
                    "--list-all-pipelines": "bool",
                    "--region": "dynamic",
                    "--skip-pipeline-lookup": "bool"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "redeploy": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--rollout-id": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--region": "dynamic",
                    "--role": "dynamic"
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--delivery-pipeline": "value",
                    "--description": "value",
                    "--labels": "value",
                    "--region": "dynamic",
                    "--release": "value",
                    "--rollout-id": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "deployment-manager": {
          "commands": {
            "deployments": {
              "commands": {
                "cancel-preview": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--fingerprint": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--automatic-rollback-on-error": "bool",
                    "--composite-type": "value",
                    "--config": "value",
                    "--create-policy": [
                      "acquire",
                      "create",
                      "create-or-acquire"
                    ],
                    "--description": "value",
                    "--labels": "value",
                    "--preview": "bool",
                    "--properties": "value",
                    "--template": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--delete-policy": [
                      "abandon",
                      "delete"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--fingerprint": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--composite-type": "value",
                    "--config": "value",
                    "--create-policy": [
                      "acquire",
                      "create",
                      "create-or-acquire"
                    ],
                    "--delete-policy": [
                      "abandon",
                      "delete"
                    ],
                    "--description": "value",
                    "--fingerprint": "value",
                    "--manifest-id": "value",
                    "--preview": "bool",
                    "--properties": "value",
                    "--remove-labels": "value",
                    "--template": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "manifests": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "resources": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--simple-list": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--deployment": "value"
              }
            },
            "type-providers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--api-options-file": "value",
                    "--async": "bool",
                    "--custom-certificate-authority-roots": "value",
                    "--description": "value",
                    "--descriptor-url": "value",
                    "--labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--api-options-file": "value",
                    "--async": "bool",
                    "--custom-certificate-authority-roots": "value",
                    "--description": "value",
                    "--descriptor-url": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "types": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--status": [
                      "DEPRECATED",
                      "EXPERIMENTAL",
                      "SUPPORTED"
                    ],
                    "--template": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--provider": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--provider": "value",
                    "--provider-project": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--remove-labels": "value",
                    "--status": [
                      "DEPRECATED",
                      "EXPERIMENTAL",
                      "SUPPORTED"
                    ],
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "dns": {
          "commands": {
            "dns-keys": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "managed-zones": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--denial-of-existence": [
                      "nsec",
                      "nsec3"
                    ],
                    "--description": "value",
                    "--dns-name": "value",
                    "--dnssec-state": [
                      "off",
                      "on",
                      "transfer"
                    ],
                    "--forwarding-targets": "value",
                    "--gkeclusters": "value",
                    "--ksk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--ksk-key-length": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--log-dns-queries": "bool",
                    "--managed-reverse-lookup": "bool",
                    "--networks": "value",
                    "--private-forwarding-targets": "value",
                    "--service-directory-namespace": "value",
                    "--target-network": "value",
                    "--target-project": "value",
                    "--visibility": [
                      "private",
                      "public"
                    ],
                    "--zsk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--zsk-key-length": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--policy-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--denial-of-existence": [
                      "nsec",
                      "nsec3"
                    ],
                    "--description": "value",
                    "--dnssec-state": [
                      "off",
                      "on",
                      "transfer"
                    ],
                    "--forwarding-targets": "value",
                    "--gkeclusters": "value",
                    "--ksk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--ksk-key-length": "value",
                    "--location": "value",
                    "--log-dns-queries": "bool",
                    "--managed-reverse-lookup": "bool",
                    "--networks": "value",
                    "--private-forwarding-targets": "value",
                    "--remove-labels": "value",
                    "--target-network": "value",
                    "--target-project": "value",
                    "--update-labels": "value",
                    "--zsk-algorithm": [
                      "ecdsap256sha256",
                      "ecdsap384sha384",
                      "rsasha1",
                      "rsasha256",
                      "rsasha512"
                    ],
                    "--zsk-key-length": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--zones": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--alternative-name-servers": "value",
                    "--description": "value",
                    "--enable-inbound-forwarding": "bool",
                    "--enable-logging": "bool",
                    "--networks": "value",
                    "--private-alternative-name-servers": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--alternative-name-servers": "value",
                    "--description": "value",
                    "--enable-inbound-forwarding": "bool",
                    "--enable-logging": "bool",
                    "--networks": "value",
                    "--private-alternative-name-servers": "value"
                  }
                }
              },
              "flags": {}
            },
            "project-info": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "record-sets": {
              "commands": {
                "changes": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--sort-order": [
                          "ascending",
                          "descending"
                        ],
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--backup-data-trickle-ratio": "value",
                    "--enable-geo-fencing": "bool",
                    "--enable-health-checking": "bool",
                    "--location": "value",
                    "--routing-policy-backup-data": "value",
                    "--routing-policy-backup-data-type": [
                      "GEO"
                    ],
                    "--routing-policy-primary-data": "value",
                    "--routing_policy_data": "value",
                    "--routing_policy_type": [
                      "FAILOVER",
                      "GEO",
                      "WRR"
                    ],
                    "--rrdatas": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "dynamic",
                    "--zone-file-format": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--delete-all-existing": "bool",
                    "--location": "value",
                    "--replace-origin-ns": "bool",
                    "--zone": "dynamic",
                    "--zone-file-format": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--name": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "transaction": {
                  "commands": {
                    "abort": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "add": {
                      "commands": {},
                      "flags": {
                        "--name": "value",
                        "--transaction-file": "value",
                        "--ttl": "value",
                        "--type": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "execute": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--transaction-file": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--name": "value",
                        "--transaction-file": "value",
                        "--ttl": "value",
                        "--type": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "start": {
                      "commands": {},
                      "flags": {
                        "--transaction-file": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {
                    "--transaction-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--backup-data-trickle-ratio": "value",
                    "--enable-geo-fencing": "bool",
                    "--enable-health-checking": "bool",
                    "--location": "value",
                    "--routing-policy-backup-data": "value",
                    "--routing-policy-backup-data-type": [
                      "GEO"
                    ],
                    "--routing-policy-primary-data": "value",
                    "--routing_policy_data": "value",
                    "--routing_policy_type": [
                      "FAILOVER",
                      "GEO",
                      "WRR"
                    ],
                    "--rrdatas": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "response-policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--gkeclusters": "value",
                    "--location": "value",
                    "--networks": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--behavior": [
                          "behaviorUnspecified",
                          "bypassResponsePolicy"
                        ],
                        "--dns-name": "value",
                        "--local-data": "value",
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--behavior": [
                          "behaviorUnspecified",
                          "bypassResponsePolicy"
                        ],
                        "--dns-name": "value",
                        "--local-data": "value",
                        "--location": "value",
                        "--response-policy": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--gkeclusters": "value",
                    "--location": "value",
                    "--networks": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "domains": {
          "commands": {
            "list-user-verified": {
              "commands": {},
              "flags": {}
            },
            "registrations": {
              "commands": {
                "authorization-code": {
                  "commands": {
                    "get": {
                      "commands": {},
                      "flags": {}
                    },
                    "reset": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "configure": {
                  "commands": {
                    "contacts": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--contact-data-from-file": "value",
                        "--contact-privacy": [
                          "private-contact-data",
                          "public-contact-data",
                          "redacted-contact-data"
                        ],
                        "--notices": "value",
                        "--validate-only": "bool"
                      }
                    },
                    "dns": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cloud-dns-zone": "value",
                        "--disable-dnssec": "bool",
                        "--dns-settings-from-file": "value",
                        "--name-servers": "value",
                        "--unsafe-dns-update": "bool",
                        "--use-google-domains-dns": "bool",
                        "--validate-only": "bool"
                      }
                    },
                    "management": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--transfer-lock-state": [
                          "locked",
                          "unlocked"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "get-register-parameters": {
                  "commands": {},
                  "flags": {}
                },
                "get-transfer-parameters": {
                  "commands": {},
                  "flags": {}
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--labels": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-importable-domains": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cloud-dns-zone": "value",
                    "--contact-data-from-file": "value",
                    "--contact-privacy": [
                      "private-contact-data",
                      "public-contact-data",
                      "redacted-contact-data"
                    ],
                    "--disable-dnssec": "bool",
                    "--labels": "value",
                    "--name-servers": "value",
                    "--notices": "value",
                    "--use-google-domains-dns": "bool",
                    "--validate-only": "bool",
                    "--yearly-price": "value"
                  }
                },
                "search-domains": {
                  "commands": {},
                  "flags": {}
                },
                "transfer": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorization-code-from-file": "value",
                    "--cloud-dns-zone": "value",
                    "--contact-data-from-file": "value",
                    "--contact-privacy": [
                      "private-contact-data",
                      "public-contact-data",
                      "redacted-contact-data"
                    ],
                    "--keep-dns-settings": "bool",
                    "--labels": "value",
                    "--notices": "value",
                    "--use-google-domains-dns": "bool",
                    "--validate-only": "bool",
                    "--yearly-price": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "verify": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "emulators": {
          "commands": {
            "bigtable": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {}
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--host-port": "value"
                  }
                }
              },
              "flags": {}
            },
            "datastore": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "env-unset": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--consistency": "value",
                    "--data-dir": "value",
                    "--host-port": "value",
                    "--store-on-disk": "bool",
                    "--use-firestore-in-datastore-mode": "bool"
                  }
                }
              },
              "flags": {
                "--data-dir": "value"
              }
            },
            "firestore": {
              "commands": {
                "start": {
                  "commands": {},
                  "flags": {
                    "--host-port": "value",
                    "--rules": "value"
                  }
                }
              },
              "flags": {}
            },
            "pubsub": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--data-dir": "value",
                    "--host-port": "value"
                  }
                }
              },
              "flags": {
                "--data-dir": "value"
              }
            },
            "spanner": {
              "commands": {
                "env-init": {
                  "commands": {},
                  "flags": {}
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--enable-fault-injection": "value",
                    "--host-port": "value",
                    "--rest-port": "value",
                    "--use-docker": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "endpoints": {
          "commands": {
            "configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--service": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "check-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "error-reporting": {
          "commands": {
            "events": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "report": {
                  "commands": {},
                  "flags": {
                    "--message": "value",
                    "--message-file": "value",
                    "--service": "value",
                    "--service-version": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "essential-contacts": {
          "commands": {
            "compute": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--notification-categories": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--email": "value",
                "--folder": "value",
                "--language": "value",
                "--notification-categories": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--language": "value",
                "--notification-categories": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "eventarc": {
          "commands": {
            "attributes": {
              "commands": {
                "method-names": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--service-name": "value",
                        "--sort-by": "value",
                        "--type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "service-names": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--sort-by": "value",
                        "--type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "triggers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-run-path": "value",
                    "--destination-run-region": "value",
                    "--destination-run-service": "value",
                    "--location": "value",
                    "--matching-criteria": "value",
                    "--service-account": "value",
                    "--transport-topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-destination-run-path": "bool",
                    "--clear-service-account": "bool",
                    "--destination-run-path": "value",
                    "--destination-run-region": "value",
                    "--destination-run-service": "value",
                    "--location": "value",
                    "--matching-criteria": "value",
                    "--service-account": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "events": {
          "commands": {
            "brokers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "init": {
              "commands": {},
              "flags": {
                "--authentication": [
                  "secrets",
                  "skip",
                  "workload-identity-gsa"
                ],
                "--broker-service-account": "value",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--service-account": "value",
                "--sources-service-account": "value"
              }
            },
            "namespaces": {
              "commands": {
                "init": {
                  "commands": {},
                  "flags": {
                    "--authentication": [
                      "secrets",
                      "skip",
                      "workload-identity-gsa"
                    ],
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--copy-default-secret": "bool",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "triggers": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--broker": "value",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--custom-type": "bool",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--parameters": "value",
                    "--parameters-from-file": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--resources": "value",
                    "--secrets": "value",
                    "--source": "value",
                    "--target-service": "value",
                    "--trigger-filters": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--sort-by": "value",
                    "--target-service": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--sort-by": "value",
                    "--source": "value"
                  }
                },
                "sources": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--filter": "value",
                        "--kubeconfig": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ]
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            }
          },
          "flags": {
            "--cluster": "value",
            "--cluster-location": "value",
            "--context": "value",
            "--kubeconfig": "value",
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ]
          }
        },
        "filestore": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-zone": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--location": "value",
                    "--network": "value",
                    "--protocol": [
                      "nfs-v3",
                      "nfs-v4-1"
                    ],
                    "--region": "value",
                    "--tier": [
                      "basic-hdd",
                      "basic-ssd",
                      "enterprise",
                      "high-scale-ssd",
                      "premium",
                      "standard"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--location": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--region": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file-share": "value",
                    "--location": "value",
                    "--source-backup": "value",
                    "--source-backup-region": "value",
                    "--zone": "dynamic"
                  }
                },
                "revert": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--target-snapshot": "value"
                  }
                },
                "snapshots": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--instance": "value",
                        "--instance-location": "value",
                        "--instance-region": "value",
                        "--labels": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "value",
                        "--instance-location": "value",
                        "--instance-region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--instance-location": "value",
                        "--instance-region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "value",
                        "--instance-location": "value",
                        "--instance-region": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--instance": "value",
                        "--instance-location": "value",
                        "--instance-region": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--file-share": "value",
                    "--location": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "firebase": {
          "commands": {
            "test": {
              "commands": {
                "android": {
                  "commands": {
                    "list-device-capacities": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "locales": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "models": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "run": {
                      "commands": {},
                      "flags": {
                        "--additional-apks": "value",
                        "--app": "value",
                        "--app-package": "value",
                        "--async": "bool",
                        "--auto-google-login": "bool",
                        "--client-details": "value",
                        "--device": "value",
                        "--device-ids": "value",
                        "--directories-to-pull": "value",
                        "--environment-variables": "value",
                        "--filter": "value",
                        "--grant-permissions": [
                          "all",
                          "none"
                        ],
                        "--limit": "value",
                        "--locales": "value",
                        "--network-profile": "value",
                        "--num-flaky-test-attempts": "value",
                        "--num-uniform-shards": "value",
                        "--obb-files": "value",
                        "--orientations": "dynamic",
                        "--os-version-ids": "value",
                        "--other-files": "value",
                        "--page-size": "value",
                        "--performance-metrics": "bool",
                        "--record-video": "bool",
                        "--resign": "bool",
                        "--results-bucket": "value",
                        "--results-dir": "value",
                        "--results-history-name": "value",
                        "--robo-directives": "value",
                        "--robo-script": "value",
                        "--scenario-labels": "value",
                        "--scenario-numbers": "value",
                        "--sort-by": "value",
                        "--test": "value",
                        "--test-package": "value",
                        "--test-runner-class": "value",
                        "--test-targets": "value",
                        "--test-targets-for-shard": "value",
                        "--timeout": "value",
                        "--type": [
                          "game-loop",
                          "instrumentation",
                          "robo"
                        ],
                        "--use-orchestrator": "bool"
                      }
                    },
                    "versions": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "ios": {
                  "commands": {
                    "list-device-capacities": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "locales": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "models": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "run": {
                      "commands": {},
                      "flags": {
                        "--additional-ipas": "value",
                        "--app": "value",
                        "--async": "bool",
                        "--client-details": "value",
                        "--device": "value",
                        "--directories-to-pull": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--num-flaky-test-attempts": "value",
                        "--other-files": "value",
                        "--page-size": "value",
                        "--record-video": "bool",
                        "--results-bucket": "value",
                        "--results-dir": "value",
                        "--results-history-name": "value",
                        "--robo-script": "value",
                        "--scenario-numbers": "value",
                        "--sort-by": "value",
                        "--test": "value",
                        "--test-special-entitlements": "bool",
                        "--timeout": "value",
                        "--type": [
                          "game-loop",
                          "robo",
                          "xctest"
                        ],
                        "--xcode-version": "value",
                        "--xctestrun-file": "value"
                      }
                    },
                    "versions": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {}
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--page-size": "value",
                            "--sort-by": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "ip-blocks": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "network-profiles": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "firestore": {
          "commands": {
            "databases": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--location": "value",
                    "--type": [
                      "datastore-mode",
                      "firestore-native"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--collection-ids": "value",
                "--database": "value",
                "--namespace-ids": "value"
              }
            },
            "fields": {
              "commands": {
                "ttls": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--disable-ttl": "bool",
                        "--enable-ttl": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--collection-ids": "value",
                "--database": "value",
                "--namespace-ids": "value"
              }
            },
            "indexes": {
              "commands": {
                "composite": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--api-scope": [
                          "any-api",
                          "datastore-mode-api"
                        ],
                        "--async": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--field-config": "value",
                        "--query-scope": [
                          "collection",
                          "collection-group",
                          "collection-recursive"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--database": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--database": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "fields": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--collection-group": "value",
                        "--database": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-exemption": "bool",
                        "--collection-group": "value",
                        "--database": "value",
                        "--disable-indexes": "bool",
                        "--index": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "functions": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "add-invoker-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": "dynamic"
              }
            },
            "call": {
              "commands": {},
              "flags": {
                "--cloud-event": "value",
                "--data": "value",
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "deploy": {
              "commands": {},
              "flags": {
                "--allow-unauthenticated": "bool",
                "--build-env-vars-file": "value",
                "--build-worker-pool": "value",
                "--clear-build-env-vars": "bool",
                "--clear-build-worker-pool": "bool",
                "--clear-docker-repository": "bool",
                "--clear-env-vars": "bool",
                "--clear-kms-key": "bool",
                "--clear-labels": "bool",
                "--clear-max-instances": "bool",
                "--clear-min-instances": "bool",
                "--clear-secrets": "bool",
                "--clear-vpc-connector": "bool",
                "--concurrency": "value",
                "--cpu": "value",
                "--docker-registry": [
                  "artifact-registry",
                  "container-registry"
                ],
                "--docker-repository": "value",
                "--egress-settings": [
                  "all",
                  "private-ranges-only"
                ],
                "--entry-point": "value",
                "--env-vars-file": "value",
                "--gen2": "bool",
                "--ignore-file": "value",
                "--ingress-settings": [
                  "all",
                  "internal-and-gclb",
                  "internal-only"
                ],
                "--kms-key": "value",
                "--max-instances": "value",
                "--memory": "value",
                "--min-instances": "value",
                "--region": "dynamic",
                "--remove-build-env-vars": "value",
                "--remove-env-vars": "value",
                "--remove-labels": "value",
                "--remove-secrets": "value",
                "--retry": "bool",
                "--run-service-account": "value",
                "--runtime": "value",
                "--security-level": [
                  "secure-always",
                  "secure-optional"
                ],
                "--serve-all-traffic-latest-revision": "bool",
                "--service-account": "value",
                "--set-build-env-vars": "value",
                "--set-env-vars": "value",
                "--set-secrets": "value",
                "--source": "value",
                "--stage-bucket": "value",
                "--timeout": "value",
                "--trigger-bucket": "value",
                "--trigger-event": "value",
                "--trigger-event-filters": "value",
                "--trigger-event-filters-path-pattern": "value",
                "--trigger-http": "bool",
                "--trigger-location": "dynamic",
                "--trigger-resource": "value",
                "--trigger-service-account": "value",
                "--trigger-topic": "value",
                "--update-build-env-vars": "value",
                "--update-env-vars": "value",
                "--update-labels": "value",
                "--update-secrets": "value",
                "--vpc-connector": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            },
            "event-types": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--gen2": "bool"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--gen2": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regions": "value",
                "--sort-by": "value"
              }
            },
            "logs": {
              "commands": {
                "read": {
                  "commands": {},
                  "flags": {
                    "--end-time": "value",
                    "--execution-id": "value",
                    "--filter": "value",
                    "--gen2": "bool",
                    "--limit": "value",
                    "--min-log-level": [
                      "debug",
                      "error",
                      "info"
                    ],
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--start-time": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--gen2": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "remove-invoker-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": "dynamic"
              }
            },
            "runtimes": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--gen2": "bool",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "game": {
          "commands": {
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "servers": {
              "commands": {
                "clusters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--dry-run": "bool",
                        "--gke-cluster": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--namespace": "value",
                        "--preview-time": "value",
                        "--realm": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--realm": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--realm": "value",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--realm": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--realm": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "configs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deployment": "value",
                        "--description": "value",
                        "--fleet-configs-file": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--scaling-configs-file": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--deployment": "value",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--deployment": "value",
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--deployment": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "deployments": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "describe-rollout": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "fetch-state": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    },
                    "update-rollout": {
                      "commands": {},
                      "flags": {
                        "--clear-config-overrides": "bool",
                        "--clear-default-config": "bool",
                        "--config-overrides-file": "value",
                        "--default-config": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "realms": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--labels": "value",
                        "--location": "value",
                        "--time-zone": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--clear-labels": "bool",
                        "--description": "value",
                        "--dry-run": "bool",
                        "--location": "value",
                        "--preview-time": "value",
                        "--remove-labels": "value",
                        "--time-zone": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "healthcare": {
          "commands": {
            "annotation-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "evaluate": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--golden-store": "value",
                        "--location": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--location": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "consent-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "check-data-access": {
                  "commands": {},
                  "flags": {
                    "--data-id": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--request-attributes": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "evaluate-user-consents": {
                  "commands": {},
                  "flags": {
                    "--consent-list": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--request-attributes": "value",
                    "--resource-attributes": "value",
                    "--response-view": [
                      "basic",
                      "full",
                      "response-view-unspecified"
                    ],
                    "--user-id": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "query-accessible-data": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value",
                    "--request-attributes": "value",
                    "--resource-attributes": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--dataset": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "datasets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--time-zone": "value"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--default-fhir-config": "bool",
                    "--destination-dataset": "value",
                    "--dicom-filter-tags": "value",
                    "--location": "value",
                    "--text-redaction-mode": [
                      "all"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--time-zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "dicom-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value",
                    "--send-for-bulk-import": "bool"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--dataset": "value",
                    "--destination-store": "value",
                    "--dicom-filter-tags": "value",
                    "--location": "value",
                    "--text-redaction-mode": [
                      "all"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-table": "value",
                        "--dataset": "value",
                        "--filter-config-gcs-uri": "value",
                        "--location": "value",
                        "--overwrite-table": "bool",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--filter-config-gcs-uri": "value",
                        "--gcs-uri-prefix": "value",
                        "--location": "value",
                        "--mime-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value",
                    "--send-for-bulk-import": "bool"
                  }
                }
              },
              "flags": {}
            },
            "fhir-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--disable-referential-integrity": "bool",
                    "--disable-resource-versioning": "bool",
                    "--enable-update-create": "bool",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "deidentify": {
                  "commands": {},
                  "flags": {
                    "--annotation-store": "value",
                    "--async": "bool",
                    "--dataset": "value",
                    "--destination-store": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "bq": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--bq-dataset": "value",
                        "--dataset": "value",
                        "--location": "value",
                        "--recursive-depth": "value",
                        "--resource-type": "value",
                        "--schema-type": [
                          "analytics",
                          "analytics_v2",
                          "lossless"
                        ],
                        "--since": "value",
                        "--write-disposition": [
                          "write-append",
                          "write-empty",
                          "write-truncate"
                        ]
                      }
                    },
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value",
                        "--resource-type": "value",
                        "--since": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--content-structure": [
                          "bundle",
                          "bundle-pretty",
                          "resource",
                          "resource-pretty"
                        ],
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--enable-update-create": "bool",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "hl7v2-stores": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--end-time": "value",
                        "--gcs-uri": "value",
                        "--location": "value",
                        "--message-view": [
                          "basic",
                          "full",
                          "parsed-only",
                          "raw-only",
                          "schematized-only"
                        ],
                        "--start-time": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--dataset": "value",
                        "--gcs-uri": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "nlp": {
              "commands": {
                "analyze-entities": {
                  "commands": {},
                  "flags": {
                    "--document-content": "value",
                    "--nlp-service": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--dataset": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "help": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "iam": {
          "commands": {
            "list-grantable-roles": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--page-size": "value"
              }
            },
            "list-testable-permissions": {
              "commands": {},
              "flags": {
                "--filter": "value"
              }
            },
            "policies": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value",
                    "--policy-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--etag": "value",
                    "--kind": "value"
                  }
                },
                "get": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--filter": "value",
                    "--kind": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--page_token": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--attachment-point": "value",
                    "--kind": "value",
                    "--policy-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "roles": {
              "commands": {
                "copy": {
                  "commands": {},
                  "flags": {
                    "--dest-organization": "value",
                    "--dest-project": "value",
                    "--destination": "value",
                    "--source": "value",
                    "--source-organization": "value",
                    "--source-project": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--file": "value",
                    "--organization": "value",
                    "--permissions": "value",
                    "--stage": "value",
                    "--title": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-permissions": "value",
                    "--description": "value",
                    "--file": "value",
                    "--organization": "value",
                    "--permissions": "value",
                    "--remove-permissions": "value",
                    "--stage": "value",
                    "--title": "value"
                  }
                }
              },
              "flags": {}
            },
            "service-accounts": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {}
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "keys": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value",
                        "--key-file-type": [
                          "json",
                          "p12"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    },
                    "get-public-key": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "value",
                        "--output-file": "value",
                        "--type": [
                          "pem",
                          "raw"
                        ]
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--created-before": "value",
                        "--filter": "value",
                        "--iam-account": "value",
                        "--limit": "value",
                        "--managed-by": [
                          "any",
                          "system",
                          "user"
                        ],
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    },
                    "upload": {
                      "commands": {},
                      "flags": {
                        "--iam-account": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "sign-blob": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value"
                  }
                },
                "sign-jwt": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "workforce-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--session-duration": "value"
                  }
                },
                "create-cred-config": {
                  "commands": {},
                  "flags": {
                    "--credential-source-field-name": "value",
                    "--credential-source-file": "value",
                    "--credential-source-headers": "value",
                    "--credential-source-type": "value",
                    "--credential-source-url": "value",
                    "--executable-command": "value",
                    "--executable-interactive-timeout-millis": "value",
                    "--executable-output-file": "value",
                    "--executable-timeout-millis": "value",
                    "--output-file": "value",
                    "--service-account": "value",
                    "--service-account-token-lifetime-seconds": "value",
                    "--subject-token-type": "value",
                    "--workforce-pool-user-project": "value"
                  }
                },
                "create-login-config": {
                  "commands": {},
                  "flags": {
                    "--activate": "bool",
                    "--output-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "providers": {
                  "commands": {
                    "create-oidc": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--client-id": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--show-deleted": "bool",
                        "--sort-by": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "operations": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--provider": "dynamic",
                            "--workforce-pool": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    },
                    "update-oidc": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--client-id": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--session-duration": "value"
                  }
                }
              },
              "flags": {}
            },
            "workload-identity-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                },
                "create-cred-config": {
                  "commands": {},
                  "flags": {
                    "--app-id-uri": "value",
                    "--aws": "bool",
                    "--azure": "bool",
                    "--credential-source-field-name": "value",
                    "--credential-source-file": "value",
                    "--credential-source-headers": "value",
                    "--credential-source-type": "value",
                    "--credential-source-url": "value",
                    "--enable-imdsv2": "bool",
                    "--executable-command": "value",
                    "--executable-output-file": "value",
                    "--executable-timeout-millis": "value",
                    "--output-file": "value",
                    "--service-account": "value",
                    "--service-account-token-lifetime-seconds": "value",
                    "--subject-token-type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "providers": {
                  "commands": {
                    "create-aws": {
                      "commands": {},
                      "flags": {
                        "--account-id": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "create-oidc": {
                      "commands": {},
                      "flags": {
                        "--allowed-audiences": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--show-deleted": "bool",
                        "--sort-by": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "operations": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--provider": "dynamic",
                            "--workload-identity-pool": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "undelete": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "update-aws": {
                      "commands": {},
                      "flags": {
                        "--account-id": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    },
                    "update-oidc": {
                      "commands": {},
                      "flags": {
                        "--allowed-audiences": "value",
                        "--attribute-condition": "value",
                        "--attribute-mapping": "value",
                        "--description": "value",
                        "--disabled": "bool",
                        "--display-name": "value",
                        "--issuer-uri": "value",
                        "--location": "value",
                        "--workload-identity-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "iap": {
          "commands": {
            "oauth-brands": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--application_title": "value",
                    "--support_email": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "oauth-clients": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--display_name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "reset-secret": {
                  "commands": {},
                  "flags": {
                    "--brand": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "get": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value",
                    "--resource-type": [
                      "app-engine",
                      "compute",
                      "folder",
                      "iap_web",
                      "organization"
                    ],
                    "--service": "value"
                  }
                },
                "set": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value",
                    "--resource-type": [
                      "app-engine",
                      "compute",
                      "folder",
                      "iap_web",
                      "organization"
                    ],
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "tcp": {
              "commands": {
                "dest-groups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--dest-group": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--fqdn-list": "value",
                        "--ip-range-list": "value",
                        "--region": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--dest-group": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--dest-group": "value",
                        "--member": "value",
                        "--region": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--dest-group": "value",
                        "--region": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--fqdn-list": "value",
                        "--ip-range-list": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "web": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--role": "value",
                    "--service": "value"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--oauth2-client-id": "value",
                    "--oauth2-client-secret": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--role": "value",
                    "--service": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--resource-type": [
                      "app-engine",
                      "backend-services"
                    ],
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "identity": {
          "commands": {
            "groups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--display-name": "value",
                    "--dynamic-user-query": "value",
                    "--group-type": [
                      "discussion",
                      "dynamic",
                      "security"
                    ],
                    "--labels": "value",
                    "--organization": "value",
                    "--posix-group": "value",
                    "--with-initial-owner": [
                      "empty",
                      "with-initial-owner"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "memberships": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--expiration": "value",
                        "--group-email": "value",
                        "--member-email": "value",
                        "--organization": "value",
                        "--roles": "value"
                      }
                    },
                    "check-transitive-membership": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--member-email": "value"
                      }
                    },
                    "get-membership-graph": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--labels": "value",
                        "--member-email": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--group-email": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--page-token": "value",
                        "--sort-by": "value",
                        "--view": [
                          "basic",
                          "full"
                        ]
                      }
                    },
                    "modify-membership-roles": {
                      "commands": {},
                      "flags": {
                        "--add-roles": "value",
                        "--group-email": "value",
                        "--member-email": "value",
                        "--remove-roles": "value",
                        "--update-roles-params": "value"
                      }
                    },
                    "search-transitive-groups": {
                      "commands": {},
                      "flags": {
                        "--labels": "value",
                        "--member-email": "value",
                        "--page-size": "value",
                        "--page-token": "value"
                      }
                    },
                    "search-transitive-memberships": {
                      "commands": {},
                      "flags": {
                        "--group-email": "value",
                        "--page-size": "value",
                        "--page-token": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "preview": {
                  "commands": {},
                  "flags": {
                    "--custom-field-mask": "value",
                    "--customer": "value",
                    "--max-results": "value",
                    "--page-token": "value",
                    "--projection": [
                      "basic",
                      "custom",
                      "full"
                    ],
                    "--query": "value",
                    "--view-type": [
                      "admin-view",
                      "domain-public"
                    ]
                  }
                },
                "search": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-posix-group": "value",
                    "--clear-description": "bool",
                    "--clear-display-name": "bool",
                    "--clear-posix-groups": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--dynamic-user-query": "value",
                    "--labels": "value",
                    "--remove-posix-groups": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ids": {
          "commands": {
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--enable-traffic-logs": "bool",
                    "--labels": "value",
                    "--max-wait": "value",
                    "--network": "value",
                    "--severity": [
                      "CRITICAL",
                      "HIGH",
                      "INFORMATIONAL",
                      "LOW",
                      "MEDIUM"
                    ],
                    "--threat-exceptions": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--max-wait": "value",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--max-wait": "value",
                    "--threat-exceptions": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "init": {
          "commands": {},
          "flags": {
            "--console-only": "bool",
            "--no-browser": "bool",
            "--no-launch-browser": "bool",
            "--skip-diagnostics": "bool"
          }
        },
        "interactive": {
          "commands": {},
          "flags": {
            "--context": "value"
          }
        },
        "iot": {
          "commands": {
            "devices": {
              "commands": {
                "commands": {
                  "commands": {
                    "send": {
                      "commands": {},
                      "flags": {
                        "--command-data": "value",
                        "--command-file": "value",
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--subfolder": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "configs": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "get-value": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--config-data": "value",
                        "--config-file": "value",
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--version-to-update": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--auth-method": [
                      "association-and-device-auth-token",
                      "association-only",
                      "device-auth-token-only"
                    ],
                    "--blocked": "bool",
                    "--device-type": [
                      "gateway",
                      "non-gateway"
                    ],
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--public-key": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "credentials": {
                  "commands": {
                    "clear": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--expiration-time": "value",
                        "--path": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--type": [
                          "es256",
                          "es256-pem",
                          "es256-x509-pem",
                          "rs256",
                          "rsa-pem",
                          "rsa-x509-pem"
                        ]
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--expiration-time": "value",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "gateways": {
                  "commands": {
                    "bind": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--device-region": "value",
                        "--device-registry": "dynamic",
                        "--gateway": "dynamic",
                        "--gateway-region": "value",
                        "--gateway-registry": "dynamic"
                      }
                    },
                    "list-bound-devices": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--gateway": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    },
                    "unbind": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--device-region": "value",
                        "--device-registry": "dynamic",
                        "--gateway": "dynamic",
                        "--gateway-region": "value",
                        "--gateway-registry": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--device-field-mask": "value",
                    "--device-ids": "value",
                    "--device-num-ids": "value",
                    "--device-type": [
                      "gateway",
                      "non-gateway"
                    ],
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "states": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--device": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--auth-method": [
                      "association-and-device-auth-token",
                      "association-only",
                      "device-auth-token-only"
                    ],
                    "--blocked": "bool",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "registries": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--enable-http-config": "bool",
                    "--enable-mqtt-config": "bool",
                    "--event-notification-config": "value",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--public-key-path": "value",
                    "--region": "value",
                    "--state-pubsub-topic": "value"
                  }
                },
                "credentials": {
                  "commands": {
                    "clear": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--path": "value",
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value",
                        "--registry": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--region": "value",
                        "--registry": "dynamic",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--enable-http-config": "bool",
                    "--enable-mqtt-config": "bool",
                    "--event-notification-config": "value",
                    "--log-level": [
                      "debug",
                      "error",
                      "info",
                      "none"
                    ],
                    "--region": "value",
                    "--state-pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "kms": {
          "commands": {
            "asymmetric-decrypt": {
              "commands": {},
              "flags": {
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "asymmetric-sign": {
              "commands": {},
              "flags": {
                "--digest-algorithm": [
                  "sha256",
                  "sha384",
                  "sha512"
                ],
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "decrypt": {
              "commands": {},
              "flags": {
                "--additional-authenticated-data-file": "value",
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "ekm-connections": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--hostname": "value",
                    "--location": "dynamic",
                    "--server-certificates-files": "value",
                    "--service-directory-service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--hostname": "value",
                    "--location": "dynamic",
                    "--server-certificates-files": "value",
                    "--service-directory-service": "value"
                  }
                }
              },
              "flags": {}
            },
            "encrypt": {
              "commands": {},
              "flags": {
                "--additional-authenticated-data-file": "value",
                "--ciphertext-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--plaintext-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "import-jobs": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--import-method": [
                      "rsa-oaep-3072-sha1-aes-256",
                      "rsa-oaep-3072-sha256",
                      "rsa-oaep-3072-sha256-aes-256",
                      "rsa-oaep-4096-sha1-aes-256",
                      "rsa-oaep-4096-sha256",
                      "rsa-oaep-4096-sha256-aes-256"
                    ],
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--protection-level": [
                      "hsm",
                      "software"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--attestation-file": "value",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--keyring": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "inventory": {
              "commands": {
                "get-protected-resources-summary": {
                  "commands": {},
                  "flags": {
                    "--keyname": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "list-keys": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "search-protected-resources": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyname": "dynamic",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--scope": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "keyrings": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "keys": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--crypto-key-backend": "value",
                    "--default-algorithm": [
                      "ec-sign-p256-sha256",
                      "ec-sign-p384-sha384",
                      "ec-sign-secp256k1-sha256",
                      "external-symmetric-encryption",
                      "google-symmetric-encryption",
                      "hmac-sha1",
                      "hmac-sha224",
                      "hmac-sha256",
                      "hmac-sha384",
                      "hmac-sha512",
                      "rsa-decrypt-oaep-2048-sha1",
                      "rsa-decrypt-oaep-2048-sha256",
                      "rsa-decrypt-oaep-3072-sha1",
                      "rsa-decrypt-oaep-3072-sha256",
                      "rsa-decrypt-oaep-4096-sha1",
                      "rsa-decrypt-oaep-4096-sha256",
                      "rsa-decrypt-oaep-4096-sha512",
                      "rsa-sign-pkcs1-2048-sha256",
                      "rsa-sign-pkcs1-3072-sha256",
                      "rsa-sign-pkcs1-4096-sha256",
                      "rsa-sign-pkcs1-4096-sha512",
                      "rsa-sign-pss-2048-sha256",
                      "rsa-sign-pss-3072-sha256",
                      "rsa-sign-pss-4096-sha256",
                      "rsa-sign-pss-4096-sha512",
                      "rsa-sign-raw-pkcs1-2048",
                      "rsa-sign-raw-pkcs1-3072",
                      "rsa-sign-raw-pkcs1-4096"
                    ],
                    "--destroy-scheduled-duration": "value",
                    "--import-only": "bool",
                    "--keyring": "dynamic",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--protection-level": [
                      "external",
                      "external-vpc",
                      "hsm",
                      "software"
                    ],
                    "--purpose": [
                      "asymmetric-encryption",
                      "asymmetric-signing",
                      "encryption",
                      "mac"
                    ],
                    "--rotation-period": "value",
                    "--skip-initial-version-creation": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--keyring": "value",
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "remove-rotation-schedule": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "set-primary-version": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "set-rotation-schedule": {
                  "commands": {},
                  "flags": {
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--rotation-period": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--default-algorithm": [
                      "ec-sign-p256-sha256",
                      "ec-sign-p384-sha384",
                      "ec-sign-secp256k1-sha256",
                      "external-symmetric-encryption",
                      "google-symmetric-encryption",
                      "hmac-sha1",
                      "hmac-sha224",
                      "hmac-sha256",
                      "hmac-sha384",
                      "hmac-sha512",
                      "rsa-decrypt-oaep-2048-sha1",
                      "rsa-decrypt-oaep-2048-sha256",
                      "rsa-decrypt-oaep-3072-sha1",
                      "rsa-decrypt-oaep-3072-sha256",
                      "rsa-decrypt-oaep-4096-sha1",
                      "rsa-decrypt-oaep-4096-sha256",
                      "rsa-decrypt-oaep-4096-sha512",
                      "rsa-sign-pkcs1-2048-sha256",
                      "rsa-sign-pkcs1-3072-sha256",
                      "rsa-sign-pkcs1-4096-sha256",
                      "rsa-sign-pkcs1-4096-sha512",
                      "rsa-sign-pss-2048-sha256",
                      "rsa-sign-pss-3072-sha256",
                      "rsa-sign-pss-4096-sha256",
                      "rsa-sign-pss-4096-sha512",
                      "rsa-sign-raw-pkcs1-2048",
                      "rsa-sign-raw-pkcs1-3072",
                      "rsa-sign-raw-pkcs1-4096"
                    ],
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--next-rotation-time": "value",
                    "--primary-version": "dynamic",
                    "--remove-labels": "value",
                    "--remove-rotation-schedule": "bool",
                    "--rotation-period": "value",
                    "--update-labels": "value"
                  }
                },
                "versions": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--ekm-connection-key-path": "value",
                        "--external-key-uri": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--primary": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--attestation-file": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "destroy": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "disable": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "enable": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "get-certificate-chain": {
                      "commands": {},
                      "flags": {
                        "--certificate-chain-type": [
                          "all",
                          "cavium",
                          "google-card",
                          "google-partition"
                        ],
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--output-file": "value"
                      }
                    },
                    "get-public-key": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--output-file": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--algorithm": [
                          "ec-sign-p256-sha256",
                          "ec-sign-p384-sha384",
                          "ec-sign-secp256k1-sha256",
                          "google-symmetric-encryption",
                          "hmac-sha1",
                          "hmac-sha224",
                          "hmac-sha256",
                          "hmac-sha384",
                          "hmac-sha512",
                          "rsa-decrypt-oaep-2048-sha1",
                          "rsa-decrypt-oaep-2048-sha256",
                          "rsa-decrypt-oaep-3072-sha1",
                          "rsa-decrypt-oaep-3072-sha256",
                          "rsa-decrypt-oaep-4096-sha1",
                          "rsa-decrypt-oaep-4096-sha256",
                          "rsa-decrypt-oaep-4096-sha512",
                          "rsa-sign-pkcs1-2048-sha256",
                          "rsa-sign-pkcs1-3072-sha256",
                          "rsa-sign-pkcs1-4096-sha256",
                          "rsa-sign-pkcs1-4096-sha512",
                          "rsa-sign-pss-2048-sha256",
                          "rsa-sign-pss-3072-sha256",
                          "rsa-sign-pss-4096-sha256",
                          "rsa-sign-pss-4096-sha512",
                          "rsa-sign-raw-pkcs1-2048",
                          "rsa-sign-raw-pkcs1-3072",
                          "rsa-sign-raw-pkcs1-4096"
                        ],
                        "--import-job": "dynamic",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--public-key-file": "value",
                        "--target-key-file": "value",
                        "--wrapped-key-file": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "restore": {
                      "commands": {},
                      "flags": {
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--ekm-connection-key-path": "value",
                        "--external-key-uri": "value",
                        "--key": "dynamic",
                        "--keyring": "dynamic",
                        "--location": "dynamic",
                        "--state": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "mac-sign": {
              "commands": {},
              "flags": {
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            },
            "mac-verify": {
              "commands": {},
              "flags": {
                "--input-file": "value",
                "--key": "dynamic",
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--signature-file": "value",
                "--skip-integrity-verification": "bool"
              }
            }
          },
          "flags": {}
        },
        "lifesciences": {
          "commands": {
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "pipelines": {
              "commands": {
                "run": {
                  "commands": {},
                  "flags": {
                    "--boot-disk-size": "value",
                    "--command-line": "value",
                    "--disk-size": "value",
                    "--docker-image": "value",
                    "--env-vars": "value",
                    "--inputs": "value",
                    "--inputs-from-file": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--logging": "value",
                    "--machine-type": "value",
                    "--network": "value",
                    "--outputs": "value",
                    "--pipeline-file": "value",
                    "--preemptible": "bool",
                    "--regions": "value",
                    "--service-account-email": "value",
                    "--service-account-scopes": "value",
                    "--subnetwork": "value",
                    "--zones": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "logging": {
          "commands": {
            "buckets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cmek-kms-key-name": "value",
                    "--description": "value",
                    "--enable-analytics": "bool",
                    "--index": "value",
                    "--location": "value",
                    "--restricted-fields": "value",
                    "--retention-days": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-index": "value",
                    "--async": "bool",
                    "--clear-indexes": "bool",
                    "--cmek-kms-key-name": "value",
                    "--description": "value",
                    "--enable-analytics": "bool",
                    "--location": "value",
                    "--locked": "bool",
                    "--remove-indexes": "value",
                    "--restricted-fields": "value",
                    "--retention-days": "value",
                    "--update-index": "value"
                  }
                }
              },
              "flags": {}
            },
            "copy": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--log-filter": "value",
                "--organization": "dynamic"
              }
            },
            "links": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "logs": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--sort-by": "value",
                    "--view": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--config-from-file": "value",
                    "--description": "value",
                    "--log-filter": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bucket-name": "value",
                    "--config-from-file": "value",
                    "--description": "value",
                    "--log-filter": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--operation-filter": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "read": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--freshness": "value",
                "--limit": "value",
                "--location": "value",
                "--order": [
                  "asc",
                  "desc"
                ],
                "--organization": "dynamic",
                "--resource-names": "value",
                "--view": "value"
              }
            },
            "resource-descriptors": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "settings": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-kms-key": "bool",
                    "--disable-default-sink": "bool",
                    "--folder": "value",
                    "--kms-key-name": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--organization": "dynamic",
                    "--storage-location": "value"
                  }
                }
              },
              "flags": {}
            },
            "sinks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--exclusion": "value",
                    "--folder": "value",
                    "--include-children": "bool",
                    "--log-filter": "value",
                    "--organization": "dynamic",
                    "--use-partitioned-tables": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-exclusion": "value",
                    "--billing-account": "value",
                    "--clear-exclusions": "bool",
                    "--description": "value",
                    "--disabled": "bool",
                    "--folder": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic",
                    "--remove-exclusions": "value",
                    "--update-exclusion": "value",
                    "--use-partitioned-tables": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tail": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--buffer-window": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic",
                "--view": "value"
              }
            },
            "views": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--bucket": "value",
                    "--description": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--log-filter": "value",
                    "--organization": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "write": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic",
                "--payload-type": [
                  "json",
                  "text"
                ],
                "--severity": [
                  "ALERT",
                  "CRITICAL",
                  "DEBUG",
                  "DEFAULT",
                  "EMERGENCY",
                  "ERROR",
                  "INFO",
                  "NOTICE",
                  "WARNING"
                ]
              }
            }
          },
          "flags": {}
        },
        "memcache": {
          "commands": {
            "instances": {
              "commands": {
                "apply-parameters": {
                  "commands": {},
                  "flags": {
                    "--apply-all": "bool",
                    "--async": "bool",
                    "--node-ids": "value",
                    "--region": "dynamic"
                  }
                },
                "apply-software-update": {
                  "commands": {},
                  "flags": {
                    "--apply-all": "bool",
                    "--async": "bool",
                    "--node-ids": "value",
                    "--region": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--authorized-network": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--maintenance-window-day": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-duration": "value",
                    "--maintenance-window-start-time": "value",
                    "--memcached-version": [
                      "1.5"
                    ],
                    "--node-count": "value",
                    "--node-cpu": "value",
                    "--node-memory": "value",
                    "--parameters": "value",
                    "--region": "dynamic",
                    "--zones": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reschedule-maintenance": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic",
                    "--reschedule-type": [
                      "immediate",
                      "next-available-window",
                      "specific-time"
                    ],
                    "--schedule-time": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--labels": "value",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-duration": "value",
                    "--maintenance-window-start-time": "value",
                    "--node-count": "value",
                    "--parameters": "value",
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "metastore": {
          "commands": {
            "federations": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backends": "value",
                    "--hive-metastore-version": [
                      "2.3.6",
                      "3.1.2"
                    ],
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-backends": "bool",
                    "--clear-labels": "bool",
                    "--location": "value",
                    "--remove-backends": "value",
                    "--remove-labels": "value",
                    "--update-backends": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "alter-metadata-resource-location": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--location_uri": "value",
                    "--resource_name": "value"
                  }
                },
                "backups": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auxiliary-versions": "value",
                    "--auxiliary-versions-from-file": "value",
                    "--consumer-subnetworks": "value",
                    "--data-catalog-sync": "bool",
                    "--database-type": [
                      "mysql",
                      "spanner"
                    ],
                    "--encryption-kms-key": "value",
                    "--endpoint-protocol": [
                      "grpc",
                      "thrift"
                    ],
                    "--hive-metastore-configs": "value",
                    "--hive-metastore-configs-from-file": "value",
                    "--hive-metastore-version": "value",
                    "--instance-size": [
                      "extra_large",
                      "extra_small",
                      "large",
                      "medium",
                      "small"
                    ],
                    "--kerberos-principal": "value",
                    "--keytab": "value",
                    "--krb5-config": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--maintenance-window-day": [
                      "fri",
                      "mon",
                      "sat",
                      "sun",
                      "thu",
                      "tue",
                      "wed"
                    ],
                    "--maintenance-window-hour": "value",
                    "--network": "value",
                    "--network-config-from-file": "value",
                    "--port": "value",
                    "--release-channel": [
                      "canary",
                      "stable"
                    ],
                    "--scaling-factor": "value",
                    "--tier": [
                      "developer",
                      "enterprise"
                    ]
                  }
                },
                "databases": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--member": "value",
                        "--role": "dynamic",
                        "--service": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "tables": {
                      "commands": {
                        "add-iam-policy-binding": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--member": "value",
                            "--role": "dynamic",
                            "--service": "value"
                          }
                        },
                        "get-iam-policy": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--service": "value",
                            "--sort-by": "value"
                          }
                        },
                        "remove-iam-policy-binding": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--member": "value",
                            "--role": "dynamic",
                            "--service": "value"
                          }
                        },
                        "set-iam-policy": {
                          "commands": {},
                          "flags": {
                            "--database": "value",
                            "--location": "value",
                            "--service": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--destination-folder": "value",
                        "--dump-type": [
                          "avro",
                          "mysql"
                        ],
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "import": {
                  "commands": {
                    "gcs": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--database-dump": "value",
                        "--description": "value",
                        "--dump-type": [
                          "avro",
                          "mysql"
                        ],
                        "--import-id": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "imports": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--database-dump": "value",
                        "--database-dump-type": [
                          "mysql"
                        ],
                        "--database-type": [
                          "mysql"
                        ],
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--service": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--service": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--service": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move-table-to-database": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination_db_name": "value",
                    "--location": "value",
                    "--source_db_name": "value",
                    "--table_name": "value"
                  }
                },
                "query-metadata": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--query": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup": "value",
                    "--location": "value",
                    "--restore-type": [
                      "full",
                      "metadata-only"
                    ]
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-auxiliary-versions": "value",
                    "--async": "bool",
                    "--clear-auxiliary-versions": "bool",
                    "--clear-hive-metastore-configs": "bool",
                    "--clear-labels": "bool",
                    "--data-catalog-sync": "bool",
                    "--endpoint-protocol": [
                      "grpc",
                      "thrift"
                    ],
                    "--instance-size": [
                      "extra_large",
                      "extra_small",
                      "large",
                      "medium",
                      "small"
                    ],
                    "--kerberos-principal": "value",
                    "--keytab": "value",
                    "--krb5-config": "value",
                    "--location": "value",
                    "--maintenance-window-day": [
                      "fri",
                      "mon",
                      "sat",
                      "sun",
                      "thu",
                      "tue",
                      "wed"
                    ],
                    "--maintenance-window-hour": "value",
                    "--port": "value",
                    "--remove-hive-metastore-configs": "value",
                    "--remove-labels": "value",
                    "--scaling-factor": "value",
                    "--tier": [
                      "developer",
                      "enterprise"
                    ],
                    "--update-auxiliary-versions-from-file": "value",
                    "--update-hive-metastore-configs": "value",
                    "--update-hive-metastore-configs-from-file": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ml": {
          "commands": {
            "language": {
              "commands": {
                "analyze-entities": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-entity-sentiment": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-sentiment": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "analyze-syntax": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--encoding-type": [
                      "none",
                      "utf16",
                      "utf32",
                      "utf8"
                    ],
                    "--language": "value"
                  }
                },
                "classify-text": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--content-type": [
                      "html",
                      "plain-text"
                    ],
                    "--language": "value"
                  }
                }
              },
              "flags": {}
            },
            "speech": {
              "commands": {
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "wait": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "recognize": {
                  "commands": {},
                  "flags": {
                    "--additional-language-codes": "value",
                    "--audio-channel-count": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--encoding": [
                      "amr",
                      "amr-wb",
                      "encoding-unspecified",
                      "flac",
                      "linear16",
                      "mp3",
                      "mulaw",
                      "ogg-opus",
                      "speex-with-header-byte",
                      "webm-opus"
                    ],
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--include-word-confidence": "bool",
                    "--include-word-time-offsets": "bool",
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--max-diarization-speaker-count": "value",
                    "--min-diarization-speaker-count": "value",
                    "--model": [
                      "command_and_search",
                      "default",
                      "latest_long",
                      "latest_short",
                      "medical_conversation",
                      "medical_dictation",
                      "phone_call",
                      "phone_call_enhanced",
                      "video_enhanced"
                    ],
                    "--sample-rate": "value",
                    "--separate-channel-recognition": "bool"
                  }
                },
                "recognize-long-running": {
                  "commands": {},
                  "flags": {
                    "--additional-language-codes": "value",
                    "--async": "bool",
                    "--audio-channel-count": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--encoding": [
                      "amr",
                      "amr-wb",
                      "encoding-unspecified",
                      "flac",
                      "linear16",
                      "mp3",
                      "mulaw",
                      "ogg-opus",
                      "speex-with-header-byte",
                      "webm-opus"
                    ],
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--include-word-confidence": "bool",
                    "--include-word-time-offsets": "bool",
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--max-diarization-speaker-count": "value",
                    "--min-diarization-speaker-count": "value",
                    "--model": [
                      "command_and_search",
                      "default",
                      "latest_long",
                      "latest_short",
                      "medical_conversation",
                      "medical_dictation",
                      "phone_call",
                      "phone_call_enhanced",
                      "video_enhanced"
                    ],
                    "--output-uri": "value",
                    "--sample-rate": "value",
                    "--separate-channel-recognition": "bool"
                  }
                }
              },
              "flags": {}
            },
            "translate": {
              "commands": {
                "batch-translate-text": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--glossaries": "value",
                    "--models": "value",
                    "--source": "value",
                    "--source-language": "value",
                    "--target-language-codes": "value",
                    "--zone": "value"
                  }
                },
                "detect-language": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--mime-type": "value",
                    "--model": "value",
                    "--zone": "value"
                  }
                },
                "get-supported-languages": {
                  "commands": {},
                  "flags": {
                    "--display-language-code": "value",
                    "--model": "value",
                    "--zone": "value"
                  }
                },
                "translate-text": {
                  "commands": {},
                  "flags": {
                    "--content": "value",
                    "--content-file": "value",
                    "--glossary-config": "value",
                    "--mime-type": "value",
                    "--model": "value",
                    "--source-language": "value",
                    "--target-language": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "video": {
              "commands": {
                "detect-explicit-content": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-labels": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--detection-mode": [
                      "frame",
                      "shot",
                      "shot-and-frame"
                    ],
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-object": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-shot-changes": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "detect-text": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--language-hints": "value",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "transcribe-speech": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--audio-tracks": "value",
                    "--diarization-speaker-count": "value",
                    "--enable-automatic-punctuation": "bool",
                    "--enable-speaker-diarization": "bool",
                    "--enable-word-confidence": "bool",
                    "--filter-profanity": "bool",
                    "--hints": "value",
                    "--language-code": "value",
                    "--max-alternatives": "value",
                    "--output-uri": "value",
                    "--region": [
                      "asia-east1",
                      "europe-west1",
                      "us-east1",
                      "us-west1"
                    ],
                    "--segments": "value"
                  }
                }
              },
              "flags": {}
            },
            "vision": {
              "commands": {
                "detect-document": {
                  "commands": {},
                  "flags": {
                    "--language-hints": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-faces": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-image-properties": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-labels": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-landmarks": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-logos": {
                  "commands": {},
                  "flags": {
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-objects": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-product": {
                  "commands": {},
                  "flags": {
                    "--bounding-polygon": "value",
                    "--category": "value",
                    "--filter": "value",
                    "--max-results": "value",
                    "--product-set": "value",
                    "--product-set-location": "value"
                  }
                },
                "detect-safe-search": {
                  "commands": {},
                  "flags": {
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text": {
                  "commands": {},
                  "flags": {
                    "--language-hints": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text-pdf": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-text-tiff": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "detect-web": {
                  "commands": {},
                  "flags": {
                    "--include-geo-results": "bool",
                    "--max-results": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "product-search": {
                  "commands": {
                    "images": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--bounding-polygon": "value",
                            "--location": "value",
                            "--product": "value",
                            "--storage-location": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--product": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "product-sets": {
                      "commands": {
                        "add-product": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "create": {
                          "commands": {},
                          "flags": {
                            "--display-name": "value",
                            "--location": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "import": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--location": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "list-products": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--product-set": "value",
                            "--sort-by": "value"
                          }
                        },
                        "remove-product": {
                          "commands": {},
                          "flags": {
                            "--location": "value",
                            "--product": "value"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--display-name": "value",
                            "--location": "value"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "products": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--category": [
                              "apparel",
                              "homegoods",
                              "toys"
                            ],
                            "--description": "value",
                            "--display-name": "value",
                            "--location": "value",
                            "--product-labels": "value"
                          }
                        },
                        "delete": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--location": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--filter": "value",
                            "--limit": "value",
                            "--location": "value",
                            "--page-size": "value",
                            "--sort-by": "value",
                            "--uri": "bool"
                          }
                        },
                        "update": {
                          "commands": {},
                          "flags": {
                            "--add-product-labels": "value",
                            "--clear-product-labels": "bool",
                            "--description": "value",
                            "--display-name": "value",
                            "--location": "value",
                            "--remove-product-labels": "value"
                          }
                        }
                      },
                      "flags": {}
                    }
                  },
                  "flags": {}
                },
                "suggest-crop": {
                  "commands": {},
                  "flags": {
                    "--aspect-ratios": "value",
                    "--model-version": [
                      "builtin/latest",
                      "builtin/stable"
                    ]
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ml-engine": {
          "commands": {
            "explain": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--text-instances": "value"
              }
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--summarize": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "stream-logs": {
                  "commands": {},
                  "flags": {
                    "--allow-multiline-logs": "bool",
                    "--polling-interval": "value",
                    "--task-name": "value"
                  }
                },
                "submit": {
                  "commands": {
                    "prediction": {
                      "commands": {},
                      "flags": {
                        "--batch-size": "value",
                        "--data-format": [
                          "text",
                          "tf-record",
                          "tf-record-gzip"
                        ],
                        "--input-paths": "value",
                        "--labels": "value",
                        "--max-worker-count": "value",
                        "--model": "value",
                        "--model-dir": "value",
                        "--output-path": "value",
                        "--region": "value",
                        "--runtime-version": "value",
                        "--signature-name": "value"
                      }
                    },
                    "training": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config": "value",
                        "--enable-web-access": "bool",
                        "--job-dir": "value",
                        "--kms-key": "dynamic",
                        "--kms-keyring": "dynamic",
                        "--kms-location": "dynamic",
                        "--kms-project": "dynamic",
                        "--labels": "value",
                        "--master-accelerator": "value",
                        "--master-image-uri": "value",
                        "--master-machine-type": "value",
                        "--module-name": "value",
                        "--network": "value",
                        "--package-path": "value",
                        "--packages": "value",
                        "--parameter-server-accelerator": "value",
                        "--parameter-server-count": "value",
                        "--parameter-server-image-uri": "value",
                        "--parameter-server-machine-type": "value",
                        "--python-version": "value",
                        "--region": "dynamic",
                        "--runtime-version": "value",
                        "--scale-tier": [
                          "basic",
                          "basic-gpu",
                          "basic-tpu",
                          "custom",
                          "premium-1",
                          "standard-1"
                        ],
                        "--service-account": "value",
                        "--staging-bucket": "value",
                        "--stream-logs": "bool",
                        "--tpu-tf-version": "value",
                        "--use-chief-in-tf-config": "value",
                        "--worker-accelerator": "value",
                        "--worker-count": "value",
                        "--worker-image-uri": "value",
                        "--worker-machine-type": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "local": {
              "commands": {
                "predict": {
                  "commands": {},
                  "flags": {
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--json-instances": "value",
                    "--json-request": "value",
                    "--model-dir": "value",
                    "--signature-name": "value",
                    "--text-instances": "value"
                  }
                },
                "train": {
                  "commands": {},
                  "flags": {
                    "--distributed": "bool",
                    "--evaluator-count": "value",
                    "--job-dir": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--parameter-server-count": "value",
                    "--start-port": "value",
                    "--worker-count": "value"
                  }
                }
              },
              "flags": {}
            },
            "models": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-console-logging": "bool",
                    "--enable-logging": "bool",
                    "--labels": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--regions": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "predict": {
              "commands": {},
              "flags": {
                "--json-instances": "value",
                "--json-request": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "versions": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--command": "value",
                    "--config": "value",
                    "--description": "value",
                    "--env-vars": "value",
                    "--explanation-method": [
                      "integrated-gradients",
                      "sampled-shapley",
                      "xrai"
                    ],
                    "--framework": [
                      "scikit-learn",
                      "tensorflow",
                      "xgboost"
                    ],
                    "--health-route": "value",
                    "--image": "value",
                    "--labels": "value",
                    "--machine-type": "value",
                    "--max-nodes": "value",
                    "--metric-targets": "value",
                    "--min-nodes": "value",
                    "--model": "value",
                    "--num-integral-steps": "value",
                    "--num-paths": "value",
                    "--origin": "value",
                    "--package-uris": "value",
                    "--ports": "value",
                    "--predict-route": "value",
                    "--prediction-class": "value",
                    "--python-version": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--runtime-version": "value",
                    "--service-account": "value",
                    "--staging-bucket": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--model": "value",
                    "--page-size": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-default": {
                  "commands": {},
                  "flags": {
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--bigquery-table-name": "value",
                    "--clear-labels": "bool",
                    "--config": "value",
                    "--description": "value",
                    "--model": "value",
                    "--region": [
                      "asia-east1",
                      "asia-northeast1",
                      "asia-southeast1",
                      "australia-southeast1",
                      "europe-west1",
                      "europe-west2",
                      "europe-west3",
                      "europe-west4",
                      "global",
                      "northamerica-northeast1",
                      "us-central1",
                      "us-east1",
                      "us-east4",
                      "us-west1"
                    ],
                    "--remove-labels": "value",
                    "--sampling-percentage": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "monitoring": {
          "commands": {
            "channel-descriptors": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "channels": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--channel-content": "value",
                    "--channel-content-from-file": "value",
                    "--channel-labels": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--enabled": "bool",
                    "--type": "value",
                    "--user-labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--channel-content": "value",
                    "--channel-content-from-file": "value",
                    "--clear-channel-labels": "bool",
                    "--clear-user-labels": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--enabled": "bool",
                    "--fields": "value",
                    "--remove-channel-labels": "value",
                    "--remove-user-labels": "value",
                    "--type": "value",
                    "--update-channel-labels": "value",
                    "--update-user-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "dashboards": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--config-from-file": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--config-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "metrics-scopes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-management": {
          "commands": {
            "connectivity-tests": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--destination-cloud-sql-instance": "value",
                    "--destination-gke-master-cluster": "value",
                    "--destination-instance": "value",
                    "--destination-ip-address": "value",
                    "--destination-network": "value",
                    "--destination-port": "value",
                    "--destination-project": "value",
                    "--labels": "value",
                    "--other-projects": "value",
                    "--protocol": "value",
                    "--source-app-engine-version": "value",
                    "--source-cloud-function": "value",
                    "--source-cloud-run-revision": "value",
                    "--source-cloud-sql-instance": "value",
                    "--source-gke-master-cluster": "value",
                    "--source-instance": "value",
                    "--source-ip-address": "value",
                    "--source-network": "value",
                    "--source-network-type": [
                      "gcp-network",
                      "non-gcp-network"
                    ],
                    "--source-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "rerun": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-destination-cloud-sql-instance": "bool",
                    "--clear-destination-gke-master-cluster": "bool",
                    "--clear-destination-instance": "bool",
                    "--clear-destination-ip-address": "bool",
                    "--clear-source-app-engine-version": "bool",
                    "--clear-source-cloud-function": "bool",
                    "--clear-source-cloud-run-revision": "bool",
                    "--clear-source-cloud-sql-instance": "bool",
                    "--clear-source-gke-master-cluster": "bool",
                    "--clear-source-instance": "bool",
                    "--clear-source-ip-address": "bool",
                    "--description": "value",
                    "--destination-cloud-sql-instance": "value",
                    "--destination-gke-master-cluster": "value",
                    "--destination-instance": "value",
                    "--destination-ip-address": "value",
                    "--destination-network": "value",
                    "--destination-port": "value",
                    "--destination-project": "value",
                    "--labels": "value",
                    "--other-projects": "value",
                    "--protocol": "value",
                    "--source-app-engine-version": "value",
                    "--source-cloud-function": "value",
                    "--source-cloud-run-revision": "value",
                    "--source-cloud-sql-instance": "value",
                    "--source-gke-master-cluster": "value",
                    "--source-instance": "value",
                    "--source-ip-address": "value",
                    "--source-network": "value",
                    "--source-network-type": [
                      "gcp-network",
                      "non-gcp-network"
                    ],
                    "--source-project": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-security": {
          "commands": {
            "authorization-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "client-tls-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "server-tls-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "network-services": {
          "commands": {
            "endpoint-policies": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--destination": "value",
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "service-bindings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--service-directory-namespace": "value",
                    "--service-directory-region": "value",
                    "--service-directory-service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "notebooks": {
          "commands": {
            "environments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--container-repository": "value",
                    "--container-tag": "value",
                    "--description": "value",
                    "--display-name": "value",
                    "--location": "dynamic",
                    "--post-startup-script": "value",
                    "--vm-image-family": "value",
                    "--vm-image-name": "value",
                    "--vm-image-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-core-count": "value",
                    "--accelerator-type": [
                      "NVIDIA_TESLA_A100",
                      "NVIDIA_TESLA_K80",
                      "NVIDIA_TESLA_P100",
                      "NVIDIA_TESLA_P100_VWS",
                      "NVIDIA_TESLA_P4",
                      "NVIDIA_TESLA_P4_VWS",
                      "NVIDIA_TESLA_T4",
                      "NVIDIA_TESLA_T4_VWS",
                      "NVIDIA_TESLA_V100",
                      "TPU_V2",
                      "TPU_V3"
                    ],
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": [
                      "PD_BALANCED",
                      "PD_SSD",
                      "PD_STANDARD"
                    ],
                    "--container-repository": "value",
                    "--container-tag": "value",
                    "--custom-gpu-driver-path": "value",
                    "--disk-encryption": [
                      "CMEK",
                      "GMEK"
                    ],
                    "--environment": "dynamic",
                    "--environment-location": "dynamic",
                    "--install-gpu-driver": "bool",
                    "--instance-owners": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--machine-type": "value",
                    "--metadata": "value",
                    "--network": "dynamic",
                    "--no-proxy-access": "bool",
                    "--no-public-ip": "bool",
                    "--post-startup-script": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "ANY_RESERVATION",
                      "NO_RESERVATION",
                      "SPECIFIC_RESERVATION",
                      "TYPE_UNSPECIFIED"
                    ],
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--subnet": "dynamic",
                    "--subnet-region": "dynamic",
                    "--vm-image-family": "value",
                    "--vm-image-name": "value",
                    "--vm-image-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "diagnose": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--enable-copy-home-files": "bool",
                    "--enable-packet-capture": "bool",
                    "--enable-repair": "bool",
                    "--gcs-bucket": "value",
                    "--location": "dynamic",
                    "--relative-path": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "is-upgradeable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "reset": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--accelerator-core-count": "value",
                    "--accelerator-type": [
                      "NVIDIA_TESLA_A100",
                      "NVIDIA_TESLA_K80",
                      "NVIDIA_TESLA_P100",
                      "NVIDIA_TESLA_P100_VWS",
                      "NVIDIA_TESLA_P4",
                      "NVIDIA_TESLA_P4_VWS",
                      "NVIDIA_TESLA_T4",
                      "NVIDIA_TESLA_T4_VWS",
                      "NVIDIA_TESLA_V100",
                      "TPU_V2",
                      "TPU_V3"
                    ],
                    "--async": "bool",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--machine-type": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "organizations": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "policy-troubleshoot": {
          "commands": {
            "iam": {
              "commands": {},
              "flags": {
                "--destination-ip": "value",
                "--destination-port": "value",
                "--permission": "value",
                "--principal-email": "value",
                "--request-time": "value",
                "--resource-name": "value",
                "--resource-service": "value",
                "--resource-type": "value"
              }
            }
          },
          "flags": {}
        },
        "privateca": {
          "commands": {
            "certificates": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--cert-output-file": "value",
                    "--csr": "value",
                    "--dns-san": "value",
                    "--email-san": "value",
                    "--extended-key-usages": "value",
                    "--generate-key": "bool",
                    "--ip-san": "value",
                    "--is-ca-cert": "bool",
                    "--issuer": "dynamic",
                    "--issuer-location": "dynamic",
                    "--key-output-file": "value",
                    "--key-usages": "value",
                    "--labels": "value",
                    "--max-chain-length": "value",
                    "--reusable-config": "dynamic",
                    "--subject": "value",
                    "--uri-san": "value",
                    "--validity": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--issuer": "value",
                    "--issuer-location": "value"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--include-chain": "bool",
                    "--issuer": "dynamic",
                    "--issuer-location": "dynamic",
                    "--output-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--issuer": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "revoke": {
                  "commands": {},
                  "flags": {
                    "--certificate": "dynamic",
                    "--issuer": "dynamic",
                    "--issuer-location": "dynamic",
                    "--reason": [
                      "affiliation-changed",
                      "attribute-authority-compromise",
                      "certificate-authority-compromise",
                      "certificate-hold",
                      "cessation-of-operation",
                      "key-compromise",
                      "privilege-withdrawn",
                      "superseded",
                      "unspecified"
                    ],
                    "--serial-number": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--issuer": "dynamic",
                    "--issuer-location": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "reusable-configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "roots": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--dns-san": "value",
                    "--email-san": "value",
                    "--extended-key-usages": "value",
                    "--from-ca": "dynamic",
                    "--from-ca-location": "dynamic",
                    "--from-ca-project": "dynamic",
                    "--ip-san": "value",
                    "--issuance-policy": "value",
                    "--key-algorithm": [
                      "ec-p256-sha256",
                      "ec-p384-sha384",
                      "rsa-pkcs1-2048-sha256",
                      "rsa-pkcs1-3072-sha256",
                      "rsa-pkcs1-4096-sha256",
                      "rsa-pss-2048-sha256",
                      "rsa-pss-3078-sha256",
                      "rsa-pss-4096-sha256"
                    ],
                    "--key-usages": "value",
                    "--kms-key": "value",
                    "--kms-key-version": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "dynamic",
                    "--kms-project": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--max-chain-length": "value",
                    "--publish-ca-cert": "bool",
                    "--publish-crl": "bool",
                    "--reusable-config": "dynamic",
                    "--subject": "value",
                    "--tier": [
                      "devops",
                      "enterprise"
                    ],
                    "--uri-san": "value",
                    "--validity": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--ignore-active-certificates": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--issuance-policy": "value",
                    "--location": "dynamic",
                    "--publish-ca-cert": "bool",
                    "--publish-crl": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "subordinates": {
              "commands": {
                "activate": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--pem-chain": "value"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--bucket": "value",
                    "--create-csr": "bool",
                    "--csr-output-file": "value",
                    "--dns-san": "value",
                    "--email-san": "value",
                    "--extended-key-usages": "value",
                    "--from-ca": "dynamic",
                    "--from-ca-location": "dynamic",
                    "--from-ca-project": "dynamic",
                    "--ip-san": "value",
                    "--issuance-policy": "value",
                    "--issuer": "dynamic",
                    "--issuer-location": "dynamic",
                    "--key-algorithm": [
                      "ec-p256-sha256",
                      "ec-p384-sha384",
                      "rsa-pkcs1-2048-sha256",
                      "rsa-pkcs1-3072-sha256",
                      "rsa-pkcs1-4096-sha256",
                      "rsa-pss-2048-sha256",
                      "rsa-pss-3078-sha256",
                      "rsa-pss-4096-sha256"
                    ],
                    "--key-usages": "value",
                    "--kms-key": "value",
                    "--kms-key-version": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "dynamic",
                    "--kms-project": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--max-chain-length": "value",
                    "--publish-ca-cert": "bool",
                    "--publish-crl": "bool",
                    "--reusable-config": "dynamic",
                    "--subject": "value",
                    "--tier": [
                      "devops",
                      "enterprise"
                    ],
                    "--uri-san": "value",
                    "--validity": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--ignore-active-certificates": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-csr": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--issuance-policy": "value",
                    "--location": "dynamic",
                    "--pem-chain": "value",
                    "--publish-ca-cert": "bool",
                    "--publish-crl": "bool",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "projects": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--enable-cloud-apis": "bool",
                "--folder": "value",
                "--labels": "value",
                "--name": "value",
                "--organization": "value",
                "--set-as-default": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-ancestors": {
              "commands": {},
              "flags": {}
            },
            "get-ancestors-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--include-deny": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--name": "value"
              }
            }
          },
          "flags": {}
        },
        "publicca": {
          "commands": {
            "external-account-keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--key-output-file": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "pubsub": {
          "commands": {
            "lite-operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--done": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--subscription": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-reservations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--throughput-capacity": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list-topics": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--throughput-capacity": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-subscriptions": {
              "commands": {
                "ack-up-to": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--offset": "value",
                    "--partition": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--delivery-requirement": [
                      "deliver-after-stored",
                      "deliver-immediately"
                    ],
                    "--event-time": "value",
                    "--export-dead-letter-topic": "value",
                    "--export-desired-state": [
                      "active",
                      "paused"
                    ],
                    "--export-pubsub-topic": "value",
                    "--location": "value",
                    "--publish-time": "value",
                    "--starting-offset": [
                      "beginning",
                      "end"
                    ],
                    "--topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "seek": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--event-time": "value",
                    "--location": "value",
                    "--publish-time": "value",
                    "--starting-offset": [
                      "beginning",
                      "end"
                    ]
                  }
                },
                "subscribe": {
                  "commands": {},
                  "flags": {
                    "--auto-ack": "bool",
                    "--location": "value",
                    "--num-messages": "value",
                    "--partitions": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--delivery-requirement": [
                      "deliver-after-stored",
                      "deliver-immediately"
                    ],
                    "--export-dead-letter-topic": "value",
                    "--export-desired-state": [
                      "active",
                      "paused"
                    ],
                    "--export-pubsub-topic": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "lite-topics": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--message-retention-period": "value",
                    "--partitions": "value",
                    "--per-partition-bytes": "value",
                    "--per-partition-publish-mib": "value",
                    "--per-partition-subscribe-mib": "value",
                    "--throughput-reservation": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list-subscriptions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "publish": {
                  "commands": {},
                  "flags": {
                    "--attributes": "value",
                    "--event-time": "value",
                    "--location": "value",
                    "--message": "value",
                    "--ordering-key": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--message-retention-period": "value",
                    "--partitions": "value",
                    "--per-partition-bytes": "value",
                    "--per-partition-publish-mib": "value",
                    "--per-partition-subscribe-mib": "value",
                    "--throughput-reservation": "value"
                  }
                }
              },
              "flags": {}
            },
            "schemas": {
              "commands": {
                "commit": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "delete-revision": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "list-revisions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "rollback": {
                  "commands": {},
                  "flags": {
                    "--revision-id": "value"
                  }
                },
                "validate-message": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--message": "value",
                    "--message-encoding": [
                      "binary",
                      "encoding-unspecified",
                      "json"
                    ],
                    "--schema-name": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                },
                "validate-schema": {
                  "commands": {},
                  "flags": {
                    "--definition": "value",
                    "--definition-file": "value",
                    "--type": [
                      "avro",
                      "protocol-buffer",
                      "type-unspecified"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--subscription": "value",
                    "--subscription-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "subscriptions": {
              "commands": {
                "ack": {
                  "commands": {},
                  "flags": {
                    "--ack-ids": "value"
                  }
                },
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--bigquery-table": "value",
                    "--dead-letter-topic": "value",
                    "--dead-letter-topic-project": "value",
                    "--drop-unknown-fields": "bool",
                    "--enable-exactly-once-delivery": "bool",
                    "--enable-message-ordering": "bool",
                    "--expiration-period": "value",
                    "--labels": "value",
                    "--max-delivery-attempts": "value",
                    "--max-retry-delay": "value",
                    "--message-filter": "value",
                    "--message-retention-duration": "value",
                    "--min-retry-delay": "value",
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value",
                    "--retain-acked-messages": "bool",
                    "--topic": "value",
                    "--topic-project": "value",
                    "--use-topic-schema": "bool",
                    "--write-metadata": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "modify-ack-deadline": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--ack-ids": "value"
                  }
                },
                "modify-message-ack-deadline": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--ack-ids": "value"
                  }
                },
                "modify-push-config": {
                  "commands": {},
                  "flags": {
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value"
                  }
                },
                "pull": {
                  "commands": {},
                  "flags": {
                    "--auto-ack": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--max-messages": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--wait": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "seek": {
                  "commands": {},
                  "flags": {
                    "--snapshot": "value",
                    "--snapshot-project": "value",
                    "--time": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ack-deadline": "value",
                    "--bigquery-table": "value",
                    "--clear-bigquery-config": "bool",
                    "--clear-dead-letter-policy": "bool",
                    "--clear-labels": "bool",
                    "--clear-retry-policy": "bool",
                    "--dead-letter-topic": "value",
                    "--dead-letter-topic-project": "value",
                    "--drop-unknown-fields": "bool",
                    "--enable-exactly-once-delivery": "bool",
                    "--expiration-period": "value",
                    "--max-delivery-attempts": "value",
                    "--max-retry-delay": "value",
                    "--message-retention-duration": "value",
                    "--min-retry-delay": "value",
                    "--push-auth-service-account": "value",
                    "--push-auth-token-audience": "value",
                    "--push-endpoint": "value",
                    "--remove-labels": "value",
                    "--retain-acked-messages": "bool",
                    "--update-labels": "value",
                    "--use-topic-schema": "bool",
                    "--write-metadata": "bool"
                  }
                }
              },
              "flags": {}
            },
            "topics": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--first-revision-id": "value",
                    "--labels": "value",
                    "--last-revision-id": "value",
                    "--message-encoding": "value",
                    "--message-retention-duration": "value",
                    "--message-storage-policy-allowed-regions": "value",
                    "--schema": "value",
                    "--schema-project": "value",
                    "--topic-encryption-key": "dynamic",
                    "--topic-encryption-key-keyring": "dynamic",
                    "--topic-encryption-key-location": "dynamic",
                    "--topic-encryption-key-project": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "detach-subscription": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-subscriptions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "publish": {
                  "commands": {},
                  "flags": {
                    "--attribute": "value",
                    "--message": "value",
                    "--ordering-key": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--clear-message-retention-duration": "bool",
                    "--clear-schema-settings": "bool",
                    "--first-revision-id": "value",
                    "--last-revision-id": "value",
                    "--message-encoding": "value",
                    "--message-retention-duration": "value",
                    "--message-storage-policy-allowed-regions": "value",
                    "--recompute-message-storage-policy": "bool",
                    "--remove-labels": "value",
                    "--schema": "value",
                    "--schema-project": "value",
                    "--topic-encryption-key": "dynamic",
                    "--topic-encryption-key-keyring": "dynamic",
                    "--topic-encryption-key-location": "dynamic",
                    "--topic-encryption-key-project": "dynamic",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "recommender": {
          "commands": {
            "insight-type-config": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--billing-account": "value",
                    "--config-file": "value",
                    "--display-name": "value",
                    "--etag": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "insights": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mark-accepted": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--insight-type": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--state-metadata": "value"
                  }
                }
              },
              "flags": {}
            },
            "recommendations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--recommender": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "mark-claimed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                },
                "mark-dismissed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value"
                  }
                },
                "mark-failed": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                },
                "mark-succeeded": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--etag": "value",
                    "--folder": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--recommender": "value",
                    "--state-metadata": "value"
                  }
                }
              },
              "flags": {}
            },
            "recommender-config": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--billing-account": "value",
                    "--location": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--billing-account": "value",
                    "--config-file": "value",
                    "--display-name": "value",
                    "--etag": "value",
                    "--location": "value",
                    "--organization": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "redis": {
          "commands": {
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--alternative-zone": "value",
                    "--async": "bool",
                    "--connect-mode": [
                      "connect-mode-unspecified",
                      "direct-peering",
                      "private-service-access"
                    ],
                    "--customer-managed-key": "value",
                    "--display-name": "value",
                    "--enable-auth": "bool",
                    "--labels": "value",
                    "--maintenance-window-day": [
                      "day-of-week-unspecified",
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-hour": "value",
                    "--network": "value",
                    "--persistence-mode": [
                      "disabled",
                      "rdb"
                    ],
                    "--rdb-snapshot-period": [
                      "12h",
                      "1h",
                      "24h",
                      "6h"
                    ],
                    "--rdb-snapshot-start-time": "value",
                    "--read-replicas-mode": [
                      "read-replicas-disabled",
                      "read-replicas-enabled"
                    ],
                    "--redis-config": "value",
                    "--redis-version": [
                      "redis_3_2",
                      "redis_4_0",
                      "redis_5_0",
                      "redis_6_x"
                    ],
                    "--region": "dynamic",
                    "--replica-count": "value",
                    "--reserved-ip-range": "value",
                    "--size": "value",
                    "--tier": [
                      "basic",
                      "standard"
                    ],
                    "--transit-encryption-mode": [
                      "disabled",
                      "server-authentication"
                    ],
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--data-protection-mode": [
                      "force-data-loss",
                      "limited-data-loss"
                    ],
                    "--region": "dynamic"
                  }
                },
                "get-auth-string": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "reschedule-maintenance": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "dynamic",
                    "--reschedule-type": [
                      "immediate",
                      "next-available-window",
                      "specific-time"
                    ],
                    "--schedule-time": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--enable-auth": "bool",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "day-of-week-unspecified",
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--maintenance-window-hour": "value",
                    "--persistence-mode": [
                      "disabled",
                      "rdb"
                    ],
                    "--rdb-snapshot-period": [
                      "12h",
                      "1h",
                      "24h",
                      "6h"
                    ],
                    "--rdb-snapshot-start-time": "value",
                    "--read-replicas-mode": [
                      "read-replicas-disabled",
                      "read-replicas-enabled"
                    ],
                    "--region": "dynamic",
                    "--remove-labels": "value",
                    "--remove-redis-config": "value",
                    "--replica-count": "value",
                    "--secondary-ip-range": "value",
                    "--size": "value",
                    "--update-labels": "value",
                    "--update-redis-config": "value"
                  }
                },
                "upgrade": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--redis-version": [
                      "redis_4_0",
                      "redis_5_0",
                      "redis_6_x"
                    ],
                    "--region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "resource-config": {
          "commands": {
            "bulk-export": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--on-error": [
                  "continue",
                  "halt",
                  "ignore"
                ],
                "--organization": "value",
                "--path": "value",
                "--resource-format": [
                  "krm",
                  "terraform"
                ],
                "--resource-types": "value",
                "--resource-types-file": "value",
                "--storage-path": "value"
              }
            },
            "list-resource-types": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "terraform": {
              "commands": {
                "generate-import": {
                  "commands": {},
                  "flags": {
                    "--output-dir": "value",
                    "--output-module-file": "value",
                    "--output-script-file": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "resource-manager": {
          "commands": {
            "folders": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--display-name": "value",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-ancestors-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--include-deny": "bool",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "move": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value"
                  }
                }
              },
              "flags": {}
            },
            "org-policies": {
              "commands": {
                "allow": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "deny": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--effective": "bool",
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "disable-enforce": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "enable-enforce": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "dynamic",
                    "--page-size": "value",
                    "--show-unset": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-policy": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "tags": {
              "commands": {
                "bindings": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--parent": "value",
                        "--tag-value": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "value",
                        "--parent": "value",
                        "--tag-value": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--effective": "bool",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "holds": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--help-link": "value",
                        "--holder": "value",
                        "--location": "value",
                        "--origin": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--location": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--holder": "value",
                        "--limit": "value",
                        "--location": "value",
                        "--origin": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "keys": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--parent": "value",
                        "--purpose": [
                          "GCE_FIREWALL"
                        ],
                        "--purpose-data": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "values": {
                  "commands": {
                    "add-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value",
                        "--parent": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "get-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--parent": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "remove-iam-policy-binding": {
                      "commands": {},
                      "flags": {
                        "--all": "bool",
                        "--condition": "value",
                        "--condition-from-file": "value",
                        "--member": "value",
                        "--role": "value"
                      }
                    },
                    "set-iam-policy": {
                      "commands": {},
                      "flags": {}
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--description": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "run": {
          "commands": {
            "deploy": {
              "commands": {},
              "flags": {
                "--add-cloudsql-instances": "value",
                "--allow-unauthenticated": "bool",
                "--args": "value",
                "--async": "bool",
                "--binary-authorization": "value",
                "--breakglass": "value",
                "--clear-binary-authorization": "bool",
                "--clear-cloudsql-instances": "bool",
                "--clear-config-maps": "bool",
                "--clear-encryption-key-shutdown-hours": "bool",
                "--clear-env-vars": "bool",
                "--clear-key": "bool",
                "--clear-labels": "bool",
                "--clear-post-key-revocation-action-type": "bool",
                "--clear-secrets": "bool",
                "--clear-vpc-connector": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--command": "value",
                "--concurrency": "value",
                "--connectivity": [
                  "external",
                  "internal"
                ],
                "--context": "value",
                "--cpu": "value",
                "--cpu-boost": "bool",
                "--cpu-throttling": "bool",
                "--description": "value",
                "--encryption-key-shutdown-hours": "value",
                "--env-vars-file": "value",
                "--execution-environment": [
                  "gen1",
                  "gen2"
                ],
                "--image": "value",
                "--ingress": [
                  "all",
                  "internal",
                  "internal-and-cloud-load-balancing"
                ],
                "--key": "value",
                "--kubeconfig": "value",
                "--labels": "value",
                "--max-instances": "value",
                "--memory": "value",
                "--min-instances": "value",
                "--namespace": "value",
                "--no-traffic": "bool",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--port": "value",
                "--post-key-revocation-action-type": [
                  "prevent-new",
                  "shut-down"
                ],
                "--region": "value",
                "--remove-cloudsql-instances": "value",
                "--remove-config-maps": "value",
                "--remove-env-vars": "value",
                "--remove-labels": "value",
                "--remove-secrets": "value",
                "--revision-suffix": "value",
                "--service-account": "value",
                "--session-affinity": "bool",
                "--set-cloudsql-instances": "value",
                "--set-config-maps": "value",
                "--set-env-vars": "value",
                "--set-secrets": "value",
                "--source": "value",
                "--tag": "value",
                "--timeout": "value",
                "--update-config-maps": "value",
                "--update-env-vars": "value",
                "--update-labels": "value",
                "--update-secrets": "value",
                "--use-http2": "bool",
                "--vpc-connector": "value",
                "--vpc-egress": [
                  "all",
                  "all-traffic",
                  "private-ranges-only"
                ]
              }
            },
            "domain-mappings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--force-override": "bool",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--domain": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "integrations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--name": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--service": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--type": "value",
                    "--uri": "bool"
                  }
                },
                "types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-service": "value",
                    "--parameters": "value",
                    "--region": "value",
                    "--remove-service": "value"
                  }
                }
              },
              "flags": {
                "--region": "value"
              }
            },
            "jobs": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "deploy": {
                  "commands": {},
                  "flags": {
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-env-vars": "bool",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--remove-env-vars": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--source": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--update-env-vars": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "execute": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value",
                    "--wait": "bool"
                  }
                },
                "executions": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--job": "value",
                        "--limit": "value",
                        "--namespace": "value",
                        "--page-size": "value",
                        "--region": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "logs": {
                      "commands": {
                        "read": {
                          "commands": {},
                          "flags": {
                            "--freshness": "value",
                            "--limit": "value",
                            "--log-filter": "value",
                            "--order": [
                              "asc",
                              "desc"
                            ],
                            "--region": "value"
                          }
                        },
                        "tail": {
                          "commands": {},
                          "flags": {
                            "--log-filter": "value",
                            "--region": "value"
                          }
                        }
                      },
                      "flags": {
                        "--region": "value"
                      }
                    },
                    "tasks": {
                      "commands": {
                        "describe": {
                          "commands": {},
                          "flags": {
                            "--region": "value"
                          }
                        },
                        "list": {
                          "commands": {},
                          "flags": {
                            "--abandoned": "bool",
                            "--cancelled": "bool",
                            "--completed": "bool",
                            "--execution": "value",
                            "--failed": "bool",
                            "--filter": "value",
                            "--limit": "value",
                            "--no-completed": "bool",
                            "--no-started": "bool",
                            "--page-size": "value",
                            "--pending": "bool",
                            "--region": "value",
                            "--running": "bool",
                            "--sort-by": "value",
                            "--started": "bool",
                            "--succeeded": "bool",
                            "--uri": "bool"
                          }
                        }
                      },
                      "flags": {
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--freshness": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--log-filter": "value",
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--region": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "replace": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-cloudsql-instances": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-binary-authorization": "bool",
                    "--clear-cloudsql-instances": "bool",
                    "--clear-env-vars": "bool",
                    "--clear-labels": "bool",
                    "--clear-secrets": "bool",
                    "--clear-vpc-connector": "bool",
                    "--command": "value",
                    "--cpu": "value",
                    "--env-vars-file": "value",
                    "--execute-now": "bool",
                    "--image": "value",
                    "--key": "value",
                    "--labels": "value",
                    "--max-retries": "value",
                    "--memory": "value",
                    "--parallelism": "value",
                    "--region": "value",
                    "--remove-cloudsql-instances": "value",
                    "--remove-env-vars": "value",
                    "--remove-labels": "value",
                    "--remove-secrets": "value",
                    "--service-account": "value",
                    "--set-cloudsql-instances": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--task-timeout": "value",
                    "--tasks": "value",
                    "--update-env-vars": "value",
                    "--update-labels": "value",
                    "--update-secrets": "value",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ],
                    "--wait": "bool"
                  }
                }
              },
              "flags": {
                "--region": "value"
              }
            },
            "regions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "revisions": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--freshness": "value",
                        "--kubeconfig": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--log-filter": "value",
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--member": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--filter": "value",
                    "--kubeconfig": "value",
                    "--limit": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "logs": {
                  "commands": {
                    "read": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--freshness": "value",
                        "--kubeconfig": "value",
                        "--limit": "value",
                        "--log-filter": "value",
                        "--order": [
                          "asc",
                          "desc"
                        ],
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    },
                    "tail": {
                      "commands": {},
                      "flags": {
                        "--cluster": "value",
                        "--cluster-location": "value",
                        "--context": "value",
                        "--kubeconfig": "value",
                        "--log-filter": "value",
                        "--platform": [
                          "gke",
                          "kubernetes",
                          "managed"
                        ],
                        "--region": "value"
                      }
                    }
                  },
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "proxy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--port": "value",
                    "--region": "value",
                    "--tag": "value",
                    "--token": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--member": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "replace": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-cloudsql-instances": "value",
                    "--args": "value",
                    "--async": "bool",
                    "--binary-authorization": "value",
                    "--breakglass": "value",
                    "--clear-binary-authorization": "bool",
                    "--clear-cloudsql-instances": "bool",
                    "--clear-config-maps": "bool",
                    "--clear-encryption-key-shutdown-hours": "bool",
                    "--clear-env-vars": "bool",
                    "--clear-key": "bool",
                    "--clear-labels": "bool",
                    "--clear-post-key-revocation-action-type": "bool",
                    "--clear-secrets": "bool",
                    "--clear-vpc-connector": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--command": "value",
                    "--concurrency": "value",
                    "--connectivity": [
                      "external",
                      "internal"
                    ],
                    "--context": "value",
                    "--cpu": "value",
                    "--cpu-boost": "bool",
                    "--cpu-throttling": "bool",
                    "--encryption-key-shutdown-hours": "value",
                    "--env-vars-file": "value",
                    "--execution-environment": [
                      "gen1",
                      "gen2"
                    ],
                    "--image": "value",
                    "--ingress": [
                      "all",
                      "internal",
                      "internal-and-cloud-load-balancing"
                    ],
                    "--key": "value",
                    "--kubeconfig": "value",
                    "--labels": "value",
                    "--max-instances": "value",
                    "--memory": "value",
                    "--min-instances": "value",
                    "--namespace": "value",
                    "--no-traffic": "bool",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--port": "value",
                    "--post-key-revocation-action-type": [
                      "prevent-new",
                      "shut-down"
                    ],
                    "--region": "value",
                    "--remove-cloudsql-instances": "value",
                    "--remove-config-maps": "value",
                    "--remove-env-vars": "value",
                    "--remove-labels": "value",
                    "--remove-secrets": "value",
                    "--revision-suffix": "value",
                    "--service-account": "value",
                    "--session-affinity": "bool",
                    "--set-cloudsql-instances": "value",
                    "--set-config-maps": "value",
                    "--set-env-vars": "value",
                    "--set-secrets": "value",
                    "--tag": "value",
                    "--timeout": "value",
                    "--update-config-maps": "value",
                    "--update-env-vars": "value",
                    "--update-labels": "value",
                    "--update-secrets": "value",
                    "--use-http2": "bool",
                    "--vpc-connector": "value",
                    "--vpc-egress": [
                      "all",
                      "all-traffic",
                      "private-ranges-only"
                    ]
                  }
                },
                "update-traffic": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-tags": "bool",
                    "--cluster": "value",
                    "--cluster-location": "value",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--namespace": "value",
                    "--platform": [
                      "gke",
                      "kubernetes",
                      "managed"
                    ],
                    "--region": "value",
                    "--remove-tags": "value",
                    "--set-tags": "value",
                    "--to-latest": "bool",
                    "--to-revisions": "value",
                    "--to-tags": "value",
                    "--update-tags": "value"
                  }
                }
              },
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "runtime-config": {
          "commands": {
            "configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value"
                  }
                },
                "variables": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value"
                      }
                    },
                    "get-value": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--values": "bool"
                      }
                    },
                    "set": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--fail-if-absent": "bool",
                        "--fail-if-present": "bool",
                        "--is-text": "bool"
                      }
                    },
                    "unset": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--fail-if-absent": "bool",
                        "--recursive": "bool"
                      }
                    },
                    "watch": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--max-wait": "value",
                        "--newer-than": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "waiters": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--config-name": "value",
                        "--failure-cardinality-number": "value",
                        "--failure-cardinality-path": "value",
                        "--success-cardinality-number": "value",
                        "--success-cardinality-path": "value",
                        "--timeout": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--config-name": "value",
                        "--max-wait": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "scc": {
          "commands": {
            "assets": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "get-parent": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "get-project": {
                  "commands": {},
                  "flags": {
                    "--asset": "value",
                    "--resource-name": "value"
                  }
                },
                "group": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--filter": "value",
                    "--group-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--field-mask": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--order-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value"
                  }
                },
                "list-marks": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value"
                  }
                },
                "run-discovery": {
                  "commands": {},
                  "flags": {}
                },
                "update-marks": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--security-marks": "value",
                    "--start-time": "value",
                    "--update-mask": "value"
                  }
                }
              },
              "flags": {}
            },
            "findings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--category": "value",
                    "--event-time": "value",
                    "--external-uri": "value",
                    "--organization": "value",
                    "--resource-name": "value",
                    "--source": "dynamic",
                    "--source-properties": "value",
                    "--state": [
                      "active",
                      "inactive",
                      "state-unspecified"
                    ]
                  }
                },
                "group": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--filter": "value",
                    "--group-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--source": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--compare-duration": "value",
                    "--field-mask": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--order-by": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value",
                    "--source": "value"
                  }
                },
                "list-marks": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--read-time": "value",
                    "--sort-by": "value",
                    "--source": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--event-time": "value",
                    "--external-uri": "value",
                    "--organization": "value",
                    "--source": "dynamic",
                    "--source-properties": "value",
                    "--state": [
                      "active",
                      "inactive",
                      "state-unspecified"
                    ],
                    "--update-mask": "value"
                  }
                },
                "update-marks": {
                  "commands": {},
                  "flags": {
                    "--organization": "value",
                    "--security-marks": "value",
                    "--source": "dynamic",
                    "--start-time": "value",
                    "--update-mask": "value"
                  }
                }
              },
              "flags": {}
            },
            "notifications": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--folder": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--filter": "value",
                    "--folder": "value",
                    "--organization": "value",
                    "--pubsub-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--organization": "value"
                  }
                }
              },
              "flags": {}
            },
            "sources": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--source": "value",
                    "--source-display-name": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "scheduler": {
          "commands": {
            "jobs": {
              "commands": {
                "create": {
                  "commands": {
                    "app-engine": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--description": "value",
                        "--headers": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--relative-url": "value",
                        "--schedule": "value",
                        "--service": "value",
                        "--time-zone": "value"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--description": "value",
                        "--headers": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--oauth-service-account-email": "value",
                        "--oauth-token-scope": "value",
                        "--oidc-service-account-email": "value",
                        "--oidc-token-audience": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--uri": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--attributes": "value",
                        "--description": "value",
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--topic": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {
                    "app-engine": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--clear-headers": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-message-body": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-relative-url": "bool",
                        "--clear-service": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--relative-url": "value",
                        "--remove-headers": "value",
                        "--schedule": "value",
                        "--service": "value",
                        "--time-zone": "value",
                        "--update-headers": "value"
                      }
                    },
                    "http": {
                      "commands": {},
                      "flags": {
                        "--attempt-deadline": "value",
                        "--clear-auth-token": "bool",
                        "--clear-headers": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-message-body": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--http-method": [
                          "delete",
                          "get",
                          "head",
                          "post",
                          "put"
                        ],
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--oauth-service-account-email": "value",
                        "--oauth-token-scope": "value",
                        "--oidc-service-account-email": "value",
                        "--oidc-token-audience": "value",
                        "--remove-headers": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--update-headers": "value",
                        "--uri": "value"
                      }
                    },
                    "pubsub": {
                      "commands": {},
                      "flags": {
                        "--clear-attributes": "bool",
                        "--clear-max-backoff": "bool",
                        "--clear-max-doublings": "bool",
                        "--clear-max-retry-attempts": "bool",
                        "--clear-max-retry-duration": "bool",
                        "--clear-min-backoff": "bool",
                        "--clear-time-zone": "bool",
                        "--description": "value",
                        "--location": "value",
                        "--max-backoff": "value",
                        "--max-doublings": "value",
                        "--max-retry-attempts": "value",
                        "--max-retry-duration": "value",
                        "--message-body": "value",
                        "--message-body-from-file": "value",
                        "--min-backoff": "value",
                        "--remove-attributes": "value",
                        "--schedule": "value",
                        "--time-zone": "value",
                        "--topic": "value",
                        "--update-attributes": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "secrets": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--data-file": "value",
                "--expire-time": "value",
                "--kms-key-name": "value",
                "--labels": "value",
                "--locations": "value",
                "--next-rotation-time": "value",
                "--replication-policy": "value",
                "--replication-policy-file": "value",
                "--rotation-period": "value",
                "--set-annotations": "value",
                "--topics": "value",
                "--ttl": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--etag": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "replication": {
              "commands": {
                "get": {
                  "commands": {},
                  "flags": {}
                },
                "set": {
                  "commands": {},
                  "flags": {
                    "--replication-policy-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--remove-cmek": "bool",
                    "--set-kms-key": "value"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-topics": "value",
                "--clear-annotations": "bool",
                "--clear-labels": "bool",
                "--clear-topics": "bool",
                "--clear-version-aliases": "bool",
                "--etag": "value",
                "--expire-time": "value",
                "--next-rotation-time": "value",
                "--remove-annotations": "value",
                "--remove-expiration": "bool",
                "--remove-labels": "value",
                "--remove-next-rotation-time": "bool",
                "--remove-rotation-period": "bool",
                "--remove-rotation-schedule": "bool",
                "--remove-topics": "value",
                "--remove-version-aliases": "value",
                "--rotation-period": "value",
                "--ttl": "value",
                "--update-annotations": "value",
                "--update-labels": "value",
                "--update-version-aliases": "value"
              }
            },
            "versions": {
              "commands": {
                "access": {
                  "commands": {},
                  "flags": {
                    "--out-file": "value",
                    "--secret": "dynamic"
                  }
                },
                "add": {
                  "commands": {},
                  "flags": {
                    "--data-file": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--secret": "dynamic"
                  }
                },
                "destroy": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--secret": "dynamic"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--secret": "dynamic"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--secret": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "service-directory": {
          "commands": {
            "endpoints": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--address": "value",
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value",
                    "--network": "value",
                    "--port": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value",
                    "--service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--address": "value",
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value",
                    "--port": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "namespaces": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "services": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--namespace": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--namespace": "value",
                    "--role": "value"
                  }
                },
                "resolve": {
                  "commands": {},
                  "flags": {
                    "--endpoint-filter": "value",
                    "--location": "value",
                    "--max-endpoints": "value",
                    "--namespace": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--namespace": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--metadata": "value",
                    "--namespace": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "api-keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--allowed-application": "value",
                    "--allowed-bundle-ids": "value",
                    "--allowed-ips": "value",
                    "--allowed-referrers": "value",
                    "--annotations": "value",
                    "--api-target": "value",
                    "--async": "bool",
                    "--display-name": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-key-string": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "lookup": {
                  "commands": {},
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--key-string": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--allowed-application": "value",
                    "--allowed-bundle-ids": "value",
                    "--allowed-ips": "value",
                    "--allowed-referrers": "value",
                    "--annotations": "value",
                    "--api-target": "value",
                    "--async": "bool",
                    "--clear-annotations": "bool",
                    "--clear-restrictions": "bool",
                    "--display-name": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "disable": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "identity": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--available": "bool",
                "--enabled": "bool",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--full": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "peered-dns-domains": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dns-suffix": "value",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--network": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "vpc-peerings": {
              "commands": {
                "connect": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--ranges": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "disable-vpc-service-controls": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "enable-vpc-service-controls": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--network": "value",
                    "--service": "value"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--name": "value"
                      }
                    },
                    "wait": {
                      "commands": {},
                      "flags": {
                        "--name": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--force": "bool",
                    "--network": "value",
                    "--ranges": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "source": {
          "commands": {
            "project-configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-topic": "value",
                    "--disable-pushblock": "bool",
                    "--enable-pushblock": "bool",
                    "--message-format": [
                      "json",
                      "protobuf"
                    ],
                    "--remove-topic": "value",
                    "--service-account": "value",
                    "--topic-project": "value",
                    "--update-topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "repos": {
              "commands": {
                "clone": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--use-full-gcloud-path": "bool"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-topic": "value",
                    "--message-format": [
                      "json",
                      "protobuf"
                    ],
                    "--remove-topic": "value",
                    "--service-account": "value",
                    "--topic-project": "value",
                    "--update-topic": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "spanner": {
          "commands": {
            "backups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "copy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-backup": "value",
                    "--destination-instance": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-config-default-or-backup-encryption"
                    ],
                    "--expiration-date": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--retention-period": "value",
                    "--source-backup": "value",
                    "--source-instance": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-database-encryption"
                    ],
                    "--expiration-date": "value",
                    "--instance": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--retention-period": "value",
                    "--version-time": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "update-metadata": {
                  "commands": {},
                  "flags": {
                    "--expiration-date": "value",
                    "--instance": "dynamic",
                    "--retention-period": "value"
                  }
                }
              },
              "flags": {}
            },
            "databases": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database-dialect": [
                      "GOOGLE_STANDARD_SQL",
                      "POSTGRESQL"
                    ],
                    "--ddl": "value",
                    "--ddl-file": "dynamic",
                    "--instance": "value",
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value"
                  }
                },
                "ddl": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--ddl": "value",
                        "--ddl-file": "dynamic",
                        "--instance": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                },
                "execute-sql": {
                  "commands": {},
                  "flags": {
                    "--database-role": "value",
                    "--enable-partitioned-dml": "bool",
                    "--instance": "value",
                    "--query-mode": [
                      "NORMAL",
                      "PLAN",
                      "PROFILE"
                    ],
                    "--read-timestamp": "value",
                    "--sql": "value",
                    "--strong": "bool",
                    "--timeout": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-database": "value",
                    "--destination-instance": "value",
                    "--encryption-type": [
                      "customer-managed-encryption",
                      "google-default-encryption",
                      "use-config-default-or-backup-encryption"
                    ],
                    "--kms-key": "value",
                    "--kms-keyring": "value",
                    "--kms-location": "value",
                    "--kms-project": "value",
                    "--source-backup": "value",
                    "--source-instance": "value"
                  }
                },
                "roles": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "sessions": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--instance": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--database": "value",
                        "--filter": "value",
                        "--instance": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--server-filter": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--instance": "value"
                  }
                }
              },
              "flags": {}
            },
            "instance-configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--add-replicas": "value",
                    "--async": "bool",
                    "--base-config": "value",
                    "--clone-config": "dynamic",
                    "--display-name": "value",
                    "--etag": "value",
                    "--labels": "value",
                    "--replicas": "value",
                    "--skip-replicas": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--etag": "value",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--display-name": "value",
                    "--etag": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--config": "dynamic",
                    "--description": "value",
                    "--expire-behavior": [
                      "free-to-provisioned",
                      "remove-after-grace-period"
                    ],
                    "--instance-type": [
                      "free-instance",
                      "provisioned"
                    ],
                    "--nodes": "value",
                    "--processing-units": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--expire-behavior": [
                      "free-to-provisioned",
                      "remove-after-grace-period"
                    ],
                    "--instance-type": [
                      "free-instance",
                      "provisioned"
                    ],
                    "--nodes": "value",
                    "--processing-units": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--backup": "dynamic",
                    "--database": "dynamic",
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--instance-config": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--type": [
                      "BACKUP",
                      "DATABASE",
                      "DATABASE_CREATE",
                      "DATABASE_RESTORE",
                      "DATABASE_UPDATE_DDL",
                      "INSTANCE",
                      "INSTANCE_CONFIG_CREATE",
                      "INSTANCE_CONFIG_UPDATE"
                    ],
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "rows": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--instance": "value",
                    "--keys": "value",
                    "--table": "value"
                  }
                },
                "insert": {
                  "commands": {},
                  "flags": {
                    "--data": "value",
                    "--database": "value",
                    "--instance": "value",
                    "--table": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--data": "value",
                    "--database": "value",
                    "--instance": "value",
                    "--table": "value"
                  }
                }
              },
              "flags": {}
            },
            "samples": {
              "commands": {
                "backend": {
                  "commands": {},
                  "flags": {
                    "--database-id": "value",
                    "--duration": "value",
                    "--instance-id": "value",
                    "--port": "value"
                  }
                },
                "init": {
                  "commands": {},
                  "flags": {
                    "--database-id": "value",
                    "--instance-id": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--cleanup": "bool",
                    "--database-id": "value",
                    "--duration": "value",
                    "--instance-id": "value",
                    "--skip-init": "bool"
                  }
                },
                "workload": {
                  "commands": {},
                  "flags": {
                    "--duration": "value",
                    "--port": "value",
                    "--target-qps": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "sql": {
          "commands": {
            "backups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--instance": "dynamic",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-instance": "dynamic",
                    "--restore-instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "connect": {
              "commands": {},
              "flags": {
                "--database": "value",
                "--port": "value",
                "--user": "value"
              }
            },
            "databases": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--charset": "value",
                    "--collation": "value",
                    "--instance": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "patch": {
                  "commands": {},
                  "flags": {
                    "--charset": "value",
                    "--collation": "value",
                    "--diff": "bool",
                    "--instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "export": {
              "commands": {
                "bak": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--stripe_count": "value",
                    "--striped": "bool"
                  }
                },
                "csv": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--escape": "value",
                    "--fields-terminated-by": "value",
                    "--lines-terminated-by": "value",
                    "--offload": "bool",
                    "--query": "value",
                    "--quote": "value"
                  }
                },
                "sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--offload": "bool",
                    "--table": "value"
                  }
                }
              },
              "flags": {}
            },
            "flags": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--database-version": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "generate-login-token": {
              "commands": {},
              "flags": {
                "--application-default-credential": "bool",
                "--instance": "dynamic"
              }
            },
            "import": {
              "commands": {
                "bak": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cert-path": "value",
                    "--database": "value",
                    "--prompt-for-pvk-password": "bool",
                    "--pvk-password": "value",
                    "--pvk-path": "value",
                    "--striped": "bool"
                  }
                },
                "csv": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--columns": "value",
                    "--database": "value",
                    "--escape": "value",
                    "--fields-terminated-by": "value",
                    "--lines-terminated-by": "value",
                    "--quote": "value",
                    "--table": "value",
                    "--user": "value"
                  }
                },
                "sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--user": "value"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "clone": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bin-log-file-name": "value",
                    "--bin-log-position": "value",
                    "--point-in-time": "value",
                    "--restore-database-name": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--activation-policy": [
                      "always",
                      "never",
                      "on-demand"
                    ],
                    "--active-directory-domain": "value",
                    "--allocated-ip-range-name": "value",
                    "--assign-ip": "bool",
                    "--async": "bool",
                    "--audit-bucket-path": "value",
                    "--audit-retention-interval": "value",
                    "--audit-upload-interval": "value",
                    "--authorized-networks": "value",
                    "--availability-type": [
                      "regional",
                      "zonal"
                    ],
                    "--backup": "bool",
                    "--backup-location": "value",
                    "--backup-start-time": "value",
                    "--client-certificate-path": "value",
                    "--client-key-path": "value",
                    "--collation": "value",
                    "--connector-enforcement": [
                      "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                      "NOT_REQUIRED",
                      "REQUIRED"
                    ],
                    "--cpu": "value",
                    "--database-flags": "value",
                    "--database-version": "value",
                    "--deletion-protection": "bool",
                    "--deny-maintenance-period-end-date": "value",
                    "--deny-maintenance-period-start-date": "value",
                    "--deny-maintenance-period-time": "value",
                    "--disk-encryption-key": "dynamic",
                    "--disk-encryption-key-keyring": "dynamic",
                    "--disk-encryption-key-location": "dynamic",
                    "--disk-encryption-key-project": "dynamic",
                    "--enable-bin-log": "bool",
                    "--enable-google-private-path": "bool",
                    "--enable-password-policy": "bool",
                    "--enable-point-in-time-recovery": "bool",
                    "--failover-replica-name": "value",
                    "--gce-zone": "value",
                    "--insights-config-query-insights-enabled": "bool",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--labels": "value",
                    "--maintenance-release-channel": [
                      "preview",
                      "production"
                    ],
                    "--maintenance-window-day": [
                      "FRI",
                      "MON",
                      "SAT",
                      "SUN",
                      "THU",
                      "TUE",
                      "WED"
                    ],
                    "--maintenance-window-hour": "value",
                    "--master-ca-certificate-path": "value",
                    "--master-dump-file-path": "value",
                    "--master-instance-name": "value",
                    "--master-password": "value",
                    "--master-username": "value",
                    "--memory": "value",
                    "--network": "value",
                    "--password-policy-complexity": [
                      "COMPLEXITY_DEFAULT",
                      "COMPLEXITY_UNSPECIFIED"
                    ],
                    "--password-policy-disallow-username-substring": "bool",
                    "--password-policy-min-length": "value",
                    "--password-policy-password-change-interval": "value",
                    "--password-policy-reuse-interval": "value",
                    "--prompt-for-master-password": "bool",
                    "--region": "value",
                    "--replica-type": [
                      "FAILOVER",
                      "READ"
                    ],
                    "--replication": [
                      "asynchronous",
                      "synchronous"
                    ],
                    "--require-ssl": "bool",
                    "--retained-backups-count": "value",
                    "--retained-transaction-log-days": "value",
                    "--root-password": "value",
                    "--secondary-zone": "value",
                    "--source-ip-address": "value",
                    "--source-port": "value",
                    "--storage-auto-increase": "bool",
                    "--storage-auto-increase-limit": "value",
                    "--storage-size": "value",
                    "--storage-type": [
                      "HDD",
                      "SSD"
                    ],
                    "--tier": "value",
                    "--time-zone": "value",
                    "--timeout": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "export": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value",
                    "--table": "value"
                  }
                },
                "failover": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "patch": {
                  "commands": {},
                  "flags": {
                    "--activation-policy": [
                      "always",
                      "never",
                      "on-demand"
                    ],
                    "--active-directory-domain": "value",
                    "--allocated-ip-range-name": "value",
                    "--assign-ip": "bool",
                    "--async": "bool",
                    "--audit-bucket-path": "value",
                    "--audit-retention-interval": "value",
                    "--audit-upload-interval": "value",
                    "--authorized-gae-apps": "value",
                    "--authorized-networks": "value",
                    "--availability-type": [
                      "regional",
                      "zonal"
                    ],
                    "--backup-location": "value",
                    "--backup-start-time": "value",
                    "--clear-authorized-networks": "bool",
                    "--clear-database-flags": "bool",
                    "--clear-gae-apps": "bool",
                    "--clear-labels": "bool",
                    "--clear-password-policy": "bool",
                    "--connector-enforcement": [
                      "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                      "NOT_REQUIRED",
                      "REQUIRED"
                    ],
                    "--cpu": "value",
                    "--database-flags": "value",
                    "--database-version": "value",
                    "--deletion-protection": "bool",
                    "--deny-maintenance-period-end-date": "value",
                    "--deny-maintenance-period-start-date": "value",
                    "--deny-maintenance-period-time": "value",
                    "--diff": "bool",
                    "--enable-bin-log": "bool",
                    "--enable-database-replication": "bool",
                    "--enable-google-private-path": "bool",
                    "--enable-password-policy": "bool",
                    "--enable-point-in-time-recovery": "bool",
                    "--follow-gae-app": "value",
                    "--gce-zone": "value",
                    "--insights-config-query-insights-enabled": "bool",
                    "--insights-config-query-plans-per-minute": "value",
                    "--insights-config-query-string-length": "value",
                    "--insights-config-record-application-tags": "bool",
                    "--insights-config-record-client-address": "bool",
                    "--maintenance-release-channel": [
                      "preview",
                      "production"
                    ],
                    "--maintenance-version": "value",
                    "--maintenance-window-any": "bool",
                    "--maintenance-window-day": [
                      "FRI",
                      "MON",
                      "SAT",
                      "SUN",
                      "THU",
                      "TUE",
                      "WED"
                    ],
                    "--maintenance-window-hour": "value",
                    "--memory": "value",
                    "--network": "value",
                    "--no-backup": "bool",
                    "--password-policy-complexity": [
                      "COMPLEXITY_DEFAULT",
                      "COMPLEXITY_UNSPECIFIED"
                    ],
                    "--password-policy-disallow-username-substring": "bool",
                    "--password-policy-min-length": "value",
                    "--password-policy-password-change-interval": "value",
                    "--password-policy-reuse-interval": "value",
                    "--pricing-plan": [
                      "PACKAGE",
                      "PER_USE"
                    ],
                    "--remove-deny-maintenance-period": "bool",
                    "--remove-labels": "value",
                    "--replication": [
                      "asynchronous",
                      "synchronous"
                    ],
                    "--require-ssl": "bool",
                    "--retained-backups-count": "value",
                    "--retained-transaction-log-days": "value",
                    "--secondary-zone": "value",
                    "--storage-auto-increase": "bool",
                    "--storage-auto-increase-limit": "value",
                    "--storage-size": "value",
                    "--tier": "value",
                    "--update-labels": "value",
                    "--zone": "value"
                  }
                },
                "promote-replica": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "reset-ssl-config": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "restart": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "restore-backup": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--backup-id": "value",
                    "--backup-instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--timeout": "value"
                  }
                }
              },
              "flags": {}
            },
            "reschedule-maintenance": {
              "commands": {},
              "flags": {
                "--reschedule-type": [
                  "IMMEDIATE",
                  "NEXT_AVAILABLE_WINDOW",
                  "SPECIFIC_TIME"
                ],
                "--schedule-time": "value"
              }
            },
            "ssl": {
              "commands": {
                "client-certs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--instance": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "server-ca-certs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--instance": "dynamic",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "rollback": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    },
                    "rotate": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--instance": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ssl-certs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "tiers": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "users": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password": "value",
                    "--password-policy-allowed-failed-attempts": "value",
                    "--password-policy-enable-failed-attempts-check": "bool",
                    "--password-policy-enable-password-verification": "bool",
                    "--password-policy-password-expiration-duration": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--host": "value",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--host": "value",
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "set-password": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--discard-dual-password": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password": "value",
                    "--prompt-for-password": "bool",
                    "--retain-password": "bool"
                  }
                },
                "set-password-policy": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-password-policy": "bool",
                    "--host": "value",
                    "--instance": "dynamic",
                    "--password-policy-allowed-failed-attempts": "value",
                    "--password-policy-enable-failed-attempts-check": "bool",
                    "--password-policy-enable-password-verification": "bool",
                    "--password-policy-password-expiration-duration": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "survey": {
          "commands": {},
          "flags": {}
        },
        "tasks": {
          "commands": {
            "create-app-engine-task": {
              "commands": {},
              "flags": {
                "--body-content": "value",
                "--body-file": "value",
                "--header": "value",
                "--location": "value",
                "--method": "value",
                "--queue": "value",
                "--relative-uri": "value",
                "--routing": "value",
                "--schedule-time": "value"
              }
            },
            "create-http-task": {
              "commands": {},
              "flags": {
                "--body-content": "value",
                "--body-file": "value",
                "--header": "value",
                "--location": "value",
                "--method": "value",
                "--oauth-service-account-email": "value",
                "--oauth-token-scope": "value",
                "--oidc-service-account-email": "value",
                "--oidc-token-audience": "value",
                "--queue": "value",
                "--schedule-time": "value",
                "--url": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value",
                "--response-view": [
                  "basic",
                  "full",
                  "view-unspecified"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--queue": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "queues": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--log-sampling-ratio": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-dispatches": "value",
                    "--max-dispatches-per-second": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value",
                    "--type": "value"
                  }
                },
                "create-app-engine-queue": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--log-sampling-ratio": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-dispatches": "value",
                    "--max-dispatches-per-second": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                },
                "create-pull-queue": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--max-attempts": "value",
                    "--max-retry-duration": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "purge": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "resume": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-log-sampling-ratio": "bool",
                    "--clear-max-attempts": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-concurrent-dispatches": "bool",
                    "--clear-max-dispatches-per-second": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-routing-override": "bool",
                    "--location": "value",
                    "--log-sampling-ratio": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-dispatches": "value",
                    "--max-dispatches-per-second": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value",
                    "--type": "value"
                  }
                },
                "update-app-engine-queue": {
                  "commands": {},
                  "flags": {
                    "--clear-log-sampling-ratio": "bool",
                    "--clear-max-attempts": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-concurrent-dispatches": "bool",
                    "--clear-max-dispatches-per-second": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-routing-override": "bool",
                    "--location": "value",
                    "--log-sampling-ratio": "value",
                    "--max-attempts": "value",
                    "--max-backoff": "value",
                    "--max-concurrent-dispatches": "value",
                    "--max-dispatches-per-second": "value",
                    "--max-doublings": "value",
                    "--max-retry-duration": "value",
                    "--min-backoff": "value",
                    "--routing-override": "value"
                  }
                }
              },
              "flags": {}
            },
            "run": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--queue": "value"
              }
            }
          },
          "flags": {}
        },
        "terraform": {
          "commands": {
            "vet": {
              "commands": {},
              "flags": {
                "--policy-library": "value",
                "--region": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        },
        "topic": {
          "commands": {
            "accessibility": {
              "commands": {},
              "flags": {}
            },
            "arg-files": {
              "commands": {},
              "flags": {}
            },
            "cli-trees": {
              "commands": {},
              "flags": {}
            },
            "client-certificate": {
              "commands": {},
              "flags": {}
            },
            "command-conventions": {
              "commands": {},
              "flags": {}
            },
            "configurations": {
              "commands": {},
              "flags": {}
            },
            "datetimes": {
              "commands": {},
              "flags": {}
            },
            "endpoint-override": {
              "commands": {},
              "flags": {}
            },
            "escaping": {
              "commands": {},
              "flags": {}
            },
            "filters": {
              "commands": {},
              "flags": {}
            },
            "flags-file": {
              "commands": {},
              "flags": {}
            },
            "formats": {
              "commands": {},
              "flags": {}
            },
            "gcloudignore": {
              "commands": {},
              "flags": {}
            },
            "offline-help": {
              "commands": {},
              "flags": {}
            },
            "projections": {
              "commands": {},
              "flags": {}
            },
            "resource-keys": {
              "commands": {},
              "flags": {}
            },
            "startup": {
              "commands": {},
              "flags": {}
            },
            "uninstall": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "workflows": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "deploy": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--labels": "value",
                "--location": "value",
                "--service-account": "value",
                "--source": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "execute": {
              "commands": {},
              "flags": {
                "--call-log-level": [
                  "log-all-calls",
                  "log-errors-only",
                  "none"
                ],
                "--data": "value",
                "--location": "value"
              }
            },
            "executions": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workflow": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workflow": "value"
                  }
                },
                "describe-last": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workflow": "value"
                  }
                },
                "wait-last": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--call-log-level": [
                  "log-all-calls",
                  "log-errors-only",
                  "none"
                ],
                "--data": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "workstations": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--enable-private-endpoint": "bool",
                    "--network": "value",
                    "--region": "value",
                    "--subnetwork": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--cluster": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-custom-image": "value",
                    "--container-env": "value",
                    "--container-predefined-image": [
                      "base-image",
                      "clion",
                      "codeoss",
                      "goland",
                      "intellij",
                      "phpstorm",
                      "pycharm",
                      "rider",
                      "rubymine",
                      "webstorm"
                    ],
                    "--container-run-as-user": "value",
                    "--container-working-dir": "value",
                    "--disable-public-ip-addresses": "bool",
                    "--enable-confidential-compute": "bool",
                    "--idle-timeout": "value",
                    "--machine-type": [
                      "e2-standard-16",
                      "e2-standard-2",
                      "e2-standard-32",
                      "e2-standard-4",
                      "e2-standard-8",
                      "n1-standard-4",
                      "n2-standard-8",
                      "n2d-standard-16",
                      "n2d-standard-2",
                      "n2d-standard-32",
                      "n2d-standard-4",
                      "n2d-standard-8",
                      "t2d-standard-6"
                    ],
                    "--network-tags": "value",
                    "--pd-disk-size": [
                      10,
                      50,
                      100,
                      200,
                      500,
                      1000
                    ],
                    "--pd-disk-type": [
                      "pd-balanced",
                      "pd-ssd",
                      "pd-standard"
                    ],
                    "--pd-reclaim-policy": [
                      "delete",
                      "retain"
                    ],
                    "--pool-size": "value",
                    "--region": "value",
                    "--running-timeout": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--boot-disk-size": "value",
                    "--cluster": "value",
                    "--container-args": "value",
                    "--container-command": "value",
                    "--container-custom-image": "value",
                    "--container-env": "value",
                    "--container-predefined-image": [
                      "base-image",
                      "clion",
                      "codeoss",
                      "goland",
                      "intellij",
                      "phpstorm",
                      "pycharm",
                      "rider",
                      "rubymine",
                      "webstorm"
                    ],
                    "--container-run-as-user": "value",
                    "--container-working-dir": "value",
                    "--disable-public-ip-addresses": "bool",
                    "--enable-confidential-compute": "bool",
                    "--idle-timeout": "value",
                    "--machine-type": [
                      "e2-standard-16",
                      "e2-standard-2",
                      "e2-standard-32",
                      "e2-standard-4",
                      "e2-standard-8",
                      "n1-standard-4",
                      "n2-standard-8",
                      "n2d-standard-16",
                      "n2d-standard-2",
                      "n2d-standard-32",
                      "n2d-standard-4",
                      "n2d-standard-8",
                      "t2d-standard-6"
                    ],
                    "--network-tags": "value",
                    "--pool-size": "value",
                    "--region": "value",
                    "--running-timeout": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "ssh": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--command": "value",
                "--config": "value",
                "--port": "value",
                "--region": "value",
                "--user": "value"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            },
            "start-tcp-tunnel": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--config": "value",
                "--local-host-port": "value",
                "--region": "value"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--config": "value",
                "--region": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "bigtable": {
      "commands": {
        "app-profiles": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--force": "bool",
                "--instance": "dynamic",
                "--restrict-to": "value",
                "--route-any": "bool",
                "--route-to": "dynamic",
                "--transactional-writes": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--force": "bool",
                "--instance": "dynamic",
                "--restrict-to": "value",
                "--route-any": "bool",
                "--route-to": "dynamic",
                "--transactional-writes": "bool"
              }
            }
          },
          "flags": {}
        },
        "backups": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "dynamic",
                "--expiration-date": "value",
                "--instance": "dynamic",
                "--retention-period": "value",
                "--table": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--instance": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--cluster": "dynamic",
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--instance": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--cluster": "dynamic",
                "--expiration-date": "value",
                "--instance": "dynamic",
                "--retention-period": "value"
              }
            }
          },
          "flags": {}
        },
        "clusters": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--autoscaling-cpu-target": "value",
                "--autoscaling-max-nodes": "value",
                "--autoscaling-min-nodes": "value",
                "--autoscaling-storage-target": "value",
                "--instance": "dynamic",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--num-nodes": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instances": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--autoscaling-cpu-target": "value",
                "--autoscaling-max-nodes": "value",
                "--autoscaling-min-nodes": "value",
                "--autoscaling-storage-target": "value",
                "--disable-autoscaling": "bool",
                "--instance": "dynamic",
                "--num-nodes": "value"
              }
            }
          },
          "flags": {}
        },
        "hot-tablets": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--end-time": "value",
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--start-time": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "dynamic",
                "--cluster-config": "value",
                "--cluster-num-nodes": "value",
                "--cluster-storage-type": [
                  "hdd",
                  "ssd"
                ],
                "--cluster-zone": "value",
                "--display-name": "value",
                "--instance-type": [
                  "DEVELOPMENT",
                  "PRODUCTION"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "tables": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--column-families": "value",
                    "--deletion-protection": "bool",
                    "--instance": "dynamic",
                    "--splits": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic",
                    "--view": [
                      "encryption",
                      "full",
                      "name",
                      "replication",
                      "schema",
                      "stats"
                    ]
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instances": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--instance": "dynamic",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination": "dynamic",
                    "--destination-instance": "dynamic",
                    "--source": "dynamic",
                    "--source-cluster": "dynamic",
                    "--source-instance": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--deletion-protection": "bool",
                    "--instance": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--display-name": "value"
              }
            },
            "upgrade": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "billing": {
      "commands": {
        "budgets": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--budget-amount": "value",
                "--calendar-period": [
                  "calendar-period-unspecified",
                  "month",
                  "quarter",
                  "year"
                ],
                "--credit-types-treatment": [
                  "credit-types-treatment-unspecified",
                  "exclude-all-credits",
                  "include-all-credits",
                  "include-specified-credits"
                ],
                "--disable-default-iam-recipients": "bool",
                "--display-name": "value",
                "--end-date": "value",
                "--filter-credit-types": "value",
                "--filter-labels": "value",
                "--filter-projects": "value",
                "--filter-services": "value",
                "--filter-subaccounts": "value",
                "--last-period-amount": "bool",
                "--notifications-rule-monitoring-notification-channels": "value",
                "--notifications-rule-pubsub-topic": "value",
                "--start-date": "value",
                "--threshold-rule": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--billing-account": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-threshold-rule": "value",
                "--billing-account": "value",
                "--budget-amount": "value",
                "--calendar-period": [
                  "calendar-period-unspecified",
                  "month",
                  "quarter",
                  "year"
                ],
                "--clear-threshold-rules": "bool",
                "--credit-types-treatment": [
                  "credit-types-treatment-unspecified",
                  "exclude-all-credits",
                  "include-all-credits",
                  "include-specified-credits"
                ],
                "--disable-default-iam-recipients": "bool",
                "--display-name": "value",
                "--end-date": "value",
                "--filter-credit-types": "value",
                "--filter-labels": "value",
                "--filter-projects": "value",
                "--filter-services": "value",
                "--filter-subaccounts": "value",
                "--last-period-amount": "bool",
                "--notifications-rule-monitoring-notification-channels": "value",
                "--notifications-rule-pubsub-topic": "value",
                "--start-date": "value",
                "--threshold-rules-from-file": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "bms": {
      "commands": {
        "instances": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "disable-serial-console": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "enable-serial-console": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "reset": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--enable-hyperthreading": "bool",
                "--os-image": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "networks": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "list-ip-reservations": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-ip-range-reservation": "value",
                "--async": "bool",
                "--clear-ip-range-reservations": "bool",
                "--clear-labels": "bool",
                "--region": "dynamic",
                "--remove-ip-range-reservation": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "nfs-shares": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--allowed-client": "value",
                "--async": "bool",
                "--labels": "value",
                "--region": "dynamic",
                "--size-gib": "value",
                "--storage-type": [
                  "HDD",
                  "SSD"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-allowed-client": "value",
                "--async": "bool",
                "--clear-allowed-clients": "bool",
                "--clear-labels": "bool",
                "--region": "dynamic",
                "--remove-allowed-client": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "volumes": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "luns": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--volume": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic",
                    "--volume": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic",
                "--snapshot": "value"
              }
            },
            "snapshot": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--region": "dynamic",
                "--snapshot-name": "value"
              }
            },
            "snapshots": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--volume": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--volume": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "dynamic",
                    "--volume": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "builds": {
      "commands": {
        "cancel": {
          "commands": {},
          "flags": {
            "--region": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--region": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--ongoing": "bool",
            "--page-size": "value",
            "--region": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "log": {
          "commands": {},
          "flags": {
            "--region": "value",
            "--stream": "bool"
          }
        },
        "submit": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--config": "value",
            "--disk-size": "value",
            "--gcs-log-dir": "value",
            "--gcs-source-staging-dir": "value",
            "--ignore-file": "value",
            "--machine-type": [
              "e2-highcpu-32",
              "e2-highcpu-8",
              "n1-highcpu-32",
              "n1-highcpu-8"
            ],
            "--no-cache": "bool",
            "--no-source": "bool",
            "--pack": "value",
            "--region": "value",
            "--substitutions": "value",
            "--suppress-logs": "bool",
            "--tag": "value",
            "--timeout": "value",
            "--worker-pool": "value"
          }
        },
        "triggers": {
          "commands": {
            "create": {
              "commands": {
                "bitbucketserver": {
                  "commands": {},
                  "flags": {
                    "--bitbucket-server-config-resource": "value",
                    "--branch-pattern": "value",
                    "--build-config": "value",
                    "--comment-control": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--ignored-files": "value",
                    "--included-files": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--project-key": "value",
                    "--pull-request-pattern": "value",
                    "--region": "value",
                    "--repo-slug": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--substitutions": "value",
                    "--tag-pattern": "value",
                    "--trigger-config": "value"
                  }
                },
                "cloud-source-repositories": {
                  "commands": {},
                  "flags": {
                    "--branch-pattern": "value",
                    "--build-config": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--ignored-files": "value",
                    "--included-files": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--region": "value",
                    "--repo": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--substitutions": "value",
                    "--tag-pattern": "value",
                    "--trigger-config": "value"
                  }
                },
                "github": {
                  "commands": {},
                  "flags": {
                    "--branch-pattern": "value",
                    "--build-config": "value",
                    "--comment-control": [
                      "COMMENTS_DISABLED",
                      "COMMENTS_ENABLED",
                      "COMMENTS_ENABLED_FOR_EXTERNAL_CONTRIBUTORS_ONLY"
                    ],
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--enterprise-config": "value",
                    "--ignored-files": "value",
                    "--include-logs-with-status": "bool",
                    "--included-files": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--pull-request-pattern": "value",
                    "--region": "value",
                    "--repo-name": "value",
                    "--repo-owner": "value",
                    "--repository": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--substitutions": "value",
                    "--tag-pattern": "value",
                    "--trigger-config": "value"
                  }
                },
                "gitlab-enterprise": {
                  "commands": {},
                  "flags": {
                    "--branch-pattern": "value",
                    "--build-config": "value",
                    "--comment-control": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--gitlab-config-resource": "value",
                    "--ignored-files": "value",
                    "--included-files": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--project-namespace": "value",
                    "--pull-request-pattern": "value",
                    "--region": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--substitutions": "value",
                    "--tag-pattern": "value",
                    "--trigger-config": "value"
                  }
                },
                "manual": {
                  "commands": {},
                  "flags": {
                    "--branch": "value",
                    "--build-config": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--github-enterprise-config": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--region": "value",
                    "--repo": "value",
                    "--repo-type": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--substitutions": "value",
                    "--tag": "value",
                    "--trigger-config": "value"
                  }
                },
                "pubsub": {
                  "commands": {},
                  "flags": {
                    "--branch": "value",
                    "--build-config": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--github-enterprise-config": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--region": "value",
                    "--repo": "value",
                    "--repo-type": "value",
                    "--require-approval": "bool",
                    "--service-account": "value",
                    "--subscription-filter": "value",
                    "--substitutions": "value",
                    "--tag": "value",
                    "--topic": "value",
                    "--trigger-config": "value"
                  }
                },
                "webhook": {
                  "commands": {},
                  "flags": {
                    "--branch": "value",
                    "--build-config": "value",
                    "--description": "value",
                    "--dockerfile": "value",
                    "--dockerfile-dir": "value",
                    "--dockerfile-image": "value",
                    "--github-enterprise-config": "value",
                    "--inline-config": "value",
                    "--name": "value",
                    "--region": "value",
                    "--repo": "value",
                    "--repo-type": "value",
                    "--require-approval": "bool",
                    "--secret": "value",
                    "--service-account": "value",
                    "--subscription-filter": "value",
                    "--substitutions": "value",
                    "--tag": "value",
                    "--trigger-config": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--branch": "value",
                "--region": "value",
                "--sha": "value",
                "--tag": "value"
              }
            }
          },
          "flags": {}
        },
        "worker-pools": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--config-from-file": "value",
                "--no-public-egress": "bool",
                "--peered-network": "value",
                "--peered-network-ip-range": "value",
                "--region": "value",
                "--worker-disk-size": "value",
                "--worker-machine-type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--config-from-file": "value",
                "--no-public-egress": "bool",
                "--public-egress": "bool",
                "--region": "value",
                "--worker-disk-size": "value",
                "--worker-machine-type": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "certificate-manager": {
      "commands": {
        "certificates": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--certificate-file": "value",
                "--description": "value",
                "--dns-authorizations": "value",
                "--domains": "value",
                "--labels": "value",
                "--location": "value",
                "--private-key-file": "value",
                "--scope": [
                  "default",
                  "edge-cache"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--certificate-file": "value",
                "--clear-labels": "bool",
                "--description": "value",
                "--location": "dynamic",
                "--private-key-file": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "dns-authorizations": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--domain": "value",
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--description": "value",
                "--location": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "maps": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--labels": "value",
                "--location": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "entries": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificates": "dynamic",
                    "--description": "value",
                    "--hostname": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--map": "dynamic",
                    "--set-primary": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--map": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--map": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--map": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--certificates": "dynamic",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "dynamic",
                    "--map": "dynamic",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--description": "value",
                "--location": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "cheat-sheet": {
      "commands": {},
      "flags": {}
    },
    "cloud-shell": {
      "commands": {
        "get-mount-command": {
          "commands": {},
          "flags": {
            "--force-key-file-overwrite": "bool",
            "--ssh-key-file": "bool"
          }
        },
        "scp": {
          "commands": {},
          "flags": {
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--recurse": "bool",
            "--scp-flag": "value",
            "--ssh-key-file": "bool"
          }
        },
        "ssh": {
          "commands": {},
          "flags": {
            "--authorize-session": "bool",
            "--command": "value",
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--ssh-flag": "value",
            "--ssh-key-file": "bool"
          }
        }
      },
      "flags": {}
    },
    "components": {
      "commands": {
        "install": {
          "commands": {},
          "flags": {}
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--only-local-state": "bool",
            "--show-platform": "bool",
            "--show-versions": "bool",
            "--sort-by": "value"
          }
        },
        "reinstall": {
          "commands": {},
          "flags": {}
        },
        "remove": {
          "commands": {},
          "flags": {}
        },
        "repositories": {
          "commands": {
            "add": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            },
            "remove": {
              "commands": {},
              "flags": {
                "--all": "bool"
              }
            }
          },
          "flags": {}
        },
        "restore": {
          "commands": {},
          "flags": {}
        },
        "update": {
          "commands": {},
          "flags": {}
        }
      },
      "flags": {}
    },
    "composer": {
      "commands": {
        "environments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--airflow-configs": "value",
                "--airflow-version": "value",
                "--async": "bool",
                "--cloud-sql-ipv4-cidr": "value",
                "--cloud-sql-machine-type": "value",
                "--cluster-ipv4-cidr": "value",
                "--cluster-secondary-range-name": "value",
                "--composer-network-ipv4-cidr": "value",
                "--connection-subnetwork": "value",
                "--connection-type": [
                  "connection-type-unspecified",
                  "private-service-connect",
                  "vpc-peering"
                ],
                "--disk-size": "value",
                "--enable-ip-alias": "bool",
                "--enable-ip-masq-agent": "bool",
                "--enable-master-authorized-networks": "bool",
                "--enable-private-endpoint": "bool",
                "--enable-private-environment": "bool",
                "--enable-privately-used-public-ips": "bool",
                "--enable-scheduled-snapshot-creation": "bool",
                "--env-variables": "value",
                "--environment-size": [
                  "large",
                  "medium",
                  "small",
                  "unspecified"
                ],
                "--image-version": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--location": "value",
                "--machine-type": "value",
                "--maintenance-window-end": "value",
                "--maintenance-window-recurrence": "value",
                "--maintenance-window-start": "value",
                "--master-authorized-networks": "value",
                "--master-ipv4-cidr": "value",
                "--max-workers": "value",
                "--min-workers": "value",
                "--network": "value",
                "--node-count": "value",
                "--oauth-scopes": "value",
                "--python-version": [
                  "2",
                  "3"
                ],
                "--scheduler-count": "value",
                "--scheduler-cpu": "value",
                "--scheduler-memory": "value",
                "--scheduler-storage": "value",
                "--service-account": "value",
                "--services-ipv4-cidr": "value",
                "--services-secondary-range-name": "value",
                "--snapshot-creation-schedule": "value",
                "--snapshot-location": "value",
                "--snapshot-schedule-timezone": "value",
                "--subnetwork": "value",
                "--tags": "value",
                "--web-server-allow-all": "bool",
                "--web-server-allow-ip": "value",
                "--web-server-cpu": "value",
                "--web-server-deny-all": "bool",
                "--web-server-ipv4-cidr": "value",
                "--web-server-machine-type": "value",
                "--web-server-memory": "value",
                "--web-server-storage": "value",
                "--worker-cpu": "value",
                "--worker-memory": "value",
                "--worker-storage": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--locations": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-packages": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--tree": "bool"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "snapshots": {
              "commands": {
                "load": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--skip-airflow-overrides-setting": "bool",
                    "--skip-environment-variables-setting": "bool",
                    "--skip-gcs-data-copying": "bool",
                    "--skip-pypi-packages-installation": "bool",
                    "--snapshot-path": "value"
                  }
                },
                "save": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--snapshot-location": "value"
                  }
                }
              },
              "flags": {}
            },
            "storage": {
              "commands": {
                "dags": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "data": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "plugins": {
                  "commands": {
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    },
                    "export": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "import": {
                      "commands": {},
                      "flags": {
                        "--destination": "value",
                        "--environment": "value",
                        "--location": "value",
                        "--source": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--environment": "value",
                        "--location": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-airflow-configs": "bool",
                "--clear-env-variables": "bool",
                "--clear-labels": "bool",
                "--clear-pypi-packages": "bool",
                "--cloud-sql-machine-type": "value",
                "--disable-master-authorized-networks": "bool",
                "--disable-scheduled-snapshot-creation": "bool",
                "--enable-master-authorized-networks": "bool",
                "--enable-scheduled-snapshot-creation": "bool",
                "--environment-size": [
                  "large",
                  "medium",
                  "small",
                  "unspecified"
                ],
                "--location": "value",
                "--master-authorized-networks": "value",
                "--max-workers": "value",
                "--min-workers": "value",
                "--node-count": "value",
                "--remove-airflow-configs": "value",
                "--remove-env-variables": "value",
                "--remove-labels": "value",
                "--remove-pypi-packages": "value",
                "--scheduler-count": "value",
                "--scheduler-cpu": "value",
                "--scheduler-memory": "value",
                "--scheduler-storage": "value",
                "--snapshot-creation-schedule": "value",
                "--snapshot-location": "value",
                "--snapshot-schedule-timezone": "value",
                "--update-airflow-configs": "value",
                "--update-env-variables": "value",
                "--update-labels": "value",
                "--update-pypi-package": "value",
                "--update-pypi-packages-from-file": "value",
                "--update-web-server-allow-ip": "value",
                "--web-server-allow-all": "bool",
                "--web-server-cpu": "value",
                "--web-server-deny-all": "bool",
                "--web-server-machine-type": "value",
                "--web-server-memory": "value",
                "--web-server-storage": "value",
                "--worker-cpu": "value",
                "--worker-memory": "value",
                "--worker-storage": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--locations": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "compute": {
      "commands": {
        "accelerator-types": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "addresses": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--addresses": "value",
                "--description": "value",
                "--endpoint-type": [
                  "NETLB",
                  "VM"
                ],
                "--global": "bool",
                "--ip-version": [
                  "IPV4",
                  "IPV6"
                ],
                "--network": "value",
                "--network-tier": "value",
                "--prefix-length": "value",
                "--purpose": [
                  "GCE_ENDPOINT",
                  "IPSEC_INTERCONNECT",
                  "PRIVATE_SERVICE_CONNECT",
                  "SHARED_LOADBALANCER_VIP",
                  "VPC_PEERING"
                ],
                "--region": "dynamic",
                "--subnet": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "backend-buckets": {
          "commands": {
            "add-signed-url-key": {
              "commands": {},
              "flags": {
                "--key-file": "value",
                "--key-name": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--bypass-cache-on-request-headers": "value",
                "--cache-key-include-http-header": "value",
                "--cache-key-query-string-whitelist": "value",
                "--cache-mode": [
                  "CACHE_ALL_STATIC",
                  "FORCE_CACHE_ALL",
                  "USE_ORIGIN_HEADERS"
                ],
                "--client-ttl": "value",
                "--compression-mode": [
                  "AUTOMATIC",
                  "DISABLED"
                ],
                "--custom-response-header": "value",
                "--default-ttl": "value",
                "--description": "value",
                "--enable-cdn": "bool",
                "--gcs-bucket-name": "dynamic",
                "--max-ttl": "value",
                "--negative-caching": "bool",
                "--negative-caching-policy": "value",
                "--request-coalescing": "bool",
                "--serve-while-stale": "value",
                "--signed-url-cache-max-age": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "delete-signed-url-key": {
              "commands": {},
              "flags": {
                "--key-name": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--bypass-cache-on-request-headers": "value",
                "--cache-key-include-http-header": "value",
                "--cache-key-query-string-whitelist": "value",
                "--cache-mode": [
                  "CACHE_ALL_STATIC",
                  "FORCE_CACHE_ALL",
                  "USE_ORIGIN_HEADERS"
                ],
                "--client-ttl": "value",
                "--compression-mode": [
                  "AUTOMATIC",
                  "DISABLED"
                ],
                "--custom-response-header": "value",
                "--default-ttl": "value",
                "--description": "value",
                "--edge-security-policy": "dynamic",
                "--enable-cdn": "bool",
                "--gcs-bucket-name": "dynamic",
                "--max-ttl": "value",
                "--negative-caching": "bool",
                "--negative-caching-policy": "value",
                "--no-bypass-cache-on-request-headers": "bool",
                "--no-client-ttl": "bool",
                "--no-custom-response-headers": "bool",
                "--no-default-ttl": "bool",
                "--no-max-ttl": "bool",
                "--no-negative-caching-policies": "bool",
                "--no-serve-while-stale": "bool",
                "--request-coalescing": "bool",
                "--serve-while-stale": "value",
                "--signed-url-cache-max-age": "value"
              }
            }
          },
          "flags": {}
        },
        "backend-services": {
          "commands": {
            "add-backend": {
              "commands": {},
              "flags": {
                "--balancing-mode": [
                  "CONNECTION",
                  "RATE",
                  "UTILIZATION"
                ],
                "--capacity-scaler": "value",
                "--description": "value",
                "--failover": "bool",
                "--global": "bool",
                "--global-network-endpoint-group": "bool",
                "--instance-group": "dynamic",
                "--instance-group-region": "dynamic",
                "--instance-group-zone": "dynamic",
                "--max-connections": "value",
                "--max-connections-per-endpoint": "value",
                "--max-connections-per-instance": "value",
                "--max-rate": "value",
                "--max-rate-per-endpoint": "value",
                "--max-rate-per-instance": "value",
                "--max-utilization": "value",
                "--network-endpoint-group": "value",
                "--network-endpoint-group-region": "dynamic",
                "--network-endpoint-group-zone": "dynamic",
                "--region": "dynamic"
              }
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "add-service-bindings": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--service-bindings": "dynamic"
              }
            },
            "add-signed-url-key": {
              "commands": {},
              "flags": {
                "--key-file": "value",
                "--key-name": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--affinity-cookie-ttl": "value",
                "--bypass-cache-on-request-headers": "value",
                "--cache-key-include-host": "bool",
                "--cache-key-include-http-header": "value",
                "--cache-key-include-named-cookie": "value",
                "--cache-key-include-protocol": "bool",
                "--cache-key-include-query-string": "bool",
                "--cache-key-query-string-blacklist": "value",
                "--cache-key-query-string-whitelist": "value",
                "--cache-mode": [
                  "CACHE_ALL_STATIC",
                  "FORCE_CACHE_ALL",
                  "USE_ORIGIN_HEADERS"
                ],
                "--client-ttl": "value",
                "--compression-mode": [
                  "AUTOMATIC",
                  "DISABLED"
                ],
                "--connection-drain-on-failover": "bool",
                "--connection-draining-timeout": "value",
                "--connection-persistence-on-unhealthy-backends": [
                  "ALWAYS_PERSIST",
                  "DEFAULT_FOR_PROTOCOL",
                  "NEVER_PERSIST"
                ],
                "--custom-request-header": "value",
                "--custom-response-header": "value",
                "--default-ttl": "value",
                "--description": "value",
                "--drop-traffic-if-unhealthy": "bool",
                "--enable-cdn": "bool",
                "--enable-logging": "bool",
                "--enable-strong-affinity": "bool",
                "--failover-ratio": "value",
                "--global": "bool",
                "--global-health-checks": "bool",
                "--health-checks": "dynamic",
                "--health-checks-region": "dynamic",
                "--http-health-checks": "dynamic",
                "--https-health-checks": "dynamic",
                "--iap": "value",
                "--idle-timeout-sec": "value",
                "--load-balancing-scheme": [
                  "EXTERNAL",
                  "EXTERNAL_MANAGED",
                  "INTERNAL",
                  "INTERNAL_MANAGED",
                  "INTERNAL_SELF_MANAGED"
                ],
                "--locality-lb-policy": [
                  "INVALID_LB_POLICY",
                  "LEAST_REQUEST",
                  "MAGLEV",
                  "ORIGINAL_DESTINATION",
                  "RANDOM",
                  "RING_HASH",
                  "ROUND_ROBIN",
                  "WEIGHTED_MAGLEV"
                ],
                "--logging-optional": [
                  "CUSTOM",
                  "EXCLUDE_ALL_OPTIONAL",
                  "INCLUDE_ALL_OPTIONAL"
                ],
                "--logging-optional-fields": "value",
                "--logging-sample-rate": "value",
                "--max-ttl": "value",
                "--negative-caching": "bool",
                "--negative-caching-policy": "value",
                "--network": "value",
                "--port-name": "value",
                "--protocol": "value",
                "--region": "dynamic",
                "--request-coalescing": "bool",
                "--serve-while-stale": "value",
                "--service-bindings": "dynamic",
                "--session-affinity": [
                  "CLIENT_IP",
                  "CLIENT_IP_NO_DESTINATION",
                  "CLIENT_IP_PORT_PROTO",
                  "CLIENT_IP_PROTO",
                  "GENERATED_COOKIE",
                  "HEADER_FIELD",
                  "HTTP_COOKIE",
                  "NONE"
                ],
                "--signed-url-cache-max-age": "value",
                "--subsetting-policy": [
                  "CONSISTENT_HASH_SUBSETTING",
                  "NONE"
                ],
                "--timeout": "value",
                "--tracking-mode": [
                  "PER_CONNECTION",
                  "PER_SESSION"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "delete-signed-url-key": {
              "commands": {},
              "flags": {
                "--key-name": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "edit": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "get-health": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-backend": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--global-network-endpoint-group": "bool",
                "--instance-group": "dynamic",
                "--instance-group-region": "dynamic",
                "--instance-group-zone": "dynamic",
                "--network-endpoint-group": "value",
                "--network-endpoint-group-region": "dynamic",
                "--network-endpoint-group-zone": "dynamic",
                "--region": "dynamic"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--member": "value",
                "--region": "dynamic",
                "--role": "value"
              }
            },
            "remove-service-bindings": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--service-bindings": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--affinity-cookie-ttl": "value",
                "--bypass-cache-on-request-headers": "value",
                "--cache-key-include-host": "bool",
                "--cache-key-include-http-header": "value",
                "--cache-key-include-named-cookie": "value",
                "--cache-key-include-protocol": "bool",
                "--cache-key-include-query-string": "bool",
                "--cache-key-query-string-blacklist": "value",
                "--cache-key-query-string-whitelist": "value",
                "--cache-mode": [
                  "CACHE_ALL_STATIC",
                  "FORCE_CACHE_ALL",
                  "USE_ORIGIN_HEADERS"
                ],
                "--client-ttl": "value",
                "--compression-mode": [
                  "AUTOMATIC",
                  "DISABLED"
                ],
                "--connection-drain-on-failover": "bool",
                "--connection-draining-timeout": "value",
                "--connection-persistence-on-unhealthy-backends": [
                  "ALWAYS_PERSIST",
                  "DEFAULT_FOR_PROTOCOL",
                  "NEVER_PERSIST"
                ],
                "--custom-request-header": "value",
                "--custom-response-header": "value",
                "--default-ttl": "value",
                "--description": "value",
                "--drop-traffic-if-unhealthy": "bool",
                "--edge-security-policy": "dynamic",
                "--enable-cdn": "bool",
                "--enable-logging": "bool",
                "--enable-strong-affinity": "bool",
                "--failover-ratio": "value",
                "--global": "bool",
                "--global-health-checks": "bool",
                "--health-checks": "dynamic",
                "--health-checks-region": "dynamic",
                "--http-health-checks": "dynamic",
                "--https-health-checks": "dynamic",
                "--iap": "value",
                "--idle-timeout-sec": "value",
                "--locality-lb-policy": [
                  "INVALID_LB_POLICY",
                  "LEAST_REQUEST",
                  "MAGLEV",
                  "ORIGINAL_DESTINATION",
                  "RANDOM",
                  "RING_HASH",
                  "ROUND_ROBIN",
                  "WEIGHTED_MAGLEV"
                ],
                "--logging-optional": [
                  "CUSTOM",
                  "EXCLUDE_ALL_OPTIONAL",
                  "INCLUDE_ALL_OPTIONAL"
                ],
                "--logging-optional-fields": "value",
                "--logging-sample-rate": "value",
                "--max-ttl": "value",
                "--negative-caching": "bool",
                "--negative-caching-policy": "value",
                "--no-bypass-cache-on-request-headers": "bool",
                "--no-client-ttl": "bool",
                "--no-custom-request-headers": "bool",
                "--no-custom-response-headers": "bool",
                "--no-default-ttl": "bool",
                "--no-health-checks": "bool",
                "--no-max-ttl": "bool",
                "--no-negative-caching-policies": "bool",
                "--no-serve-while-stale": "bool",
                "--no-service-bindings": "bool",
                "--port-name": "value",
                "--protocol": "value",
                "--region": "dynamic",
                "--request-coalescing": "bool",
                "--security-policy": "dynamic",
                "--serve-while-stale": "value",
                "--service-bindings": "dynamic",
                "--session-affinity": [
                  "CLIENT_IP",
                  "CLIENT_IP_NO_DESTINATION",
                  "CLIENT_IP_PORT_PROTO",
                  "CLIENT_IP_PROTO",
                  "GENERATED_COOKIE",
                  "HEADER_FIELD",
                  "HTTP_COOKIE",
                  "NONE"
                ],
                "--signed-url-cache-max-age": "value",
                "--subsetting-policy": [
                  "CONSISTENT_HASH_SUBSETTING",
                  "NONE"
                ],
                "--timeout": "value",
                "--tracking-mode": [
                  "PER_CONNECTION",
                  "PER_SESSION"
                ]
              }
            },
            "update-backend": {
              "commands": {},
              "flags": {
                "--balancing-mode": [
                  "CONNECTION",
                  "RATE",
                  "UTILIZATION"
                ],
                "--capacity-scaler": "value",
                "--description": "value",
                "--failover": "bool",
                "--global": "bool",
                "--instance-group": "dynamic",
                "--instance-group-region": "dynamic",
                "--instance-group-zone": "dynamic",
                "--max-connections": "value",
                "--max-connections-per-endpoint": "value",
                "--max-connections-per-instance": "value",
                "--max-rate": "value",
                "--max-rate-per-endpoint": "value",
                "--max-rate-per-instance": "value",
                "--max-utilization": "value",
                "--network-endpoint-group": "value",
                "--network-endpoint-group-zone": "dynamic",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "commitments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--auto-renew": "bool",
                "--local-ssd": "value",
                "--machine-type": "value",
                "--merge-source-commitments": "value",
                "--min-cpu-platform": "value",
                "--plan": [
                  "12-month",
                  "36-month"
                ],
                "--region": "dynamic",
                "--require-specific-reservation": "bool",
                "--reservation": "dynamic",
                "--reservation-zone": "dynamic",
                "--reservations-from-file": "value",
                "--resources": "value",
                "--resources-accelerator": "value",
                "--share-setting": [
                  "folders",
                  "local",
                  "projects"
                ],
                "--share-with": "value",
                "--split-source-commitment": "value",
                "--type": [
                  "accelerator-optimized",
                  "compute-optimized",
                  "compute-optimized-c2d",
                  "compute-optimized-c3",
                  "general-purpose",
                  "general-purpose-e2",
                  "general-purpose-n2",
                  "general-purpose-n2d",
                  "general-purpose-t2d",
                  "memory-optimized",
                  "memory-optimized-m3"
                ],
                "--vm-count": "value"
              }
            },
            "create-license": {
              "commands": {},
              "flags": {
                "--amount": "value",
                "--cores-per-license": "value",
                "--license": "value",
                "--plan": [
                  "12-month",
                  "36-month"
                ],
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--auto-renew": "bool",
                "--plan": [
                  "36-month"
                ],
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "config-ssh": {
          "commands": {},
          "flags": {
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--remove": "bool",
            "--ssh-config-file": "value",
            "--ssh-key-file": "value"
          }
        },
        "connect-to-serial-port": {
          "commands": {},
          "flags": {
            "--dry-run": "bool",
            "--extra-args": "value",
            "--force-key-file-overwrite": "bool",
            "--port": "value",
            "--ssh-key-expiration": "value",
            "--ssh-key-expire-after": "value",
            "--ssh-key-file": "value",
            "--zone": "dynamic"
          }
        },
        "copy-files": {
          "commands": {},
          "flags": {
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--plain": "bool",
            "--ssh-key-expiration": "value",
            "--ssh-key-expire-after": "value",
            "--ssh-key-file": "value",
            "--strict-host-key-checking": [
              "ask",
              "no",
              "yes"
            ],
            "--zone": "value"
          }
        },
        "diagnose": {
          "commands": {
            "export-logs": {
              "commands": {},
              "flags": {
                "--collect-process-traces": "bool",
                "--zone": "dynamic"
              }
            },
            "routes": {
              "commands": {},
              "flags": {
                "--container": "value",
                "--dry-run": "bool",
                "--external-route-ip": "value",
                "--force-key-file-overwrite": "bool",
                "--plain": "bool",
                "--regexp": "value",
                "--reverse-traceroute": "bool",
                "--ssh-flag": "value",
                "--ssh-key-expiration": "value",
                "--ssh-key-expire-after": "value",
                "--ssh-key-file": "value",
                "--strict-host-key-checking": [
                  "ask",
                  "no",
                  "yes"
                ],
                "--user": "value",
                "--zones": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "disk-types": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "disks": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "add-labels": {
              "commands": {},
              "flags": {
                "--labels": "value",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "add-resource-policies": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--resource-policies": "value",
                "--zone": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--architecture": [
                  "ARM64",
                  "X86_64"
                ],
                "--csek-key-file": "value",
                "--description": "value",
                "--guest-os-features": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-family-scope": [
                  "global",
                  "zonal"
                ],
                "--image-project": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--licenses": "value",
                "--provisioned-iops": "value",
                "--region": "dynamic",
                "--replica-zones": "value",
                "--require-csek-key-create": "bool",
                "--resource-policies": "value",
                "--size": "value",
                "--source-disk": "dynamic",
                "--source-disk-region": "dynamic",
                "--source-disk-zone": "dynamic",
                "--source-snapshot": "dynamic",
                "--type": "dynamic",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "value"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--destination-zone": "dynamic",
                "--zone": "dynamic"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "remove-labels": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--labels": "value",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "remove-resource-policies": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--resource-policies": "value",
                "--zone": "dynamic"
              }
            },
            "resize": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--size": "value",
                "--zone": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "snapshot": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--chain-name": "value",
                "--csek-key-file": "value",
                "--description": "value",
                "--guest-flush": "bool",
                "--labels": "value",
                "--region": "dynamic",
                "--snapshot-names": "value",
                "--storage-location": "value",
                "--zone": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-architecture": "bool",
                "--clear-labels": "bool",
                "--provisioned-iops": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-architecture": [
                  "ARM64",
                  "X86_64"
                ],
                "--update-labels": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "external-vpn-gateways": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--interfaces": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "firewall-policies": {
          "commands": {
            "associations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--folder": "value",
                    "--name": "value",
                    "--organization": "value",
                    "--replace-association-on-target": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--organization": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--folder": "value",
                    "--limit": "value",
                    "--organization": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "clone-rules": {
              "commands": {},
              "flags": {
                "--organization": "value",
                "--source-firewall-policy": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--folder": "value",
                "--organization": "value",
                "--short-name": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-rules": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny",
                      "goto_next"
                    ],
                    "--description": "value",
                    "--dest-ip-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "INGRESS"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--firewall-policy": "value",
                    "--layer4-configs": "value",
                    "--organization": "value",
                    "--src-ip-ranges": "value",
                    "--target-resources": "value",
                    "--target-service-accounts": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--organization": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--organization": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny",
                      "goto_next"
                    ],
                    "--description": "value",
                    "--dest-ip-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "INGRESS"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--firewall-policy": "value",
                    "--layer4-configs": "value",
                    "--new-priority": "value",
                    "--organization": "value",
                    "--src-ip-ranges": "value",
                    "--target-resources": "value",
                    "--target-service-accounts": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "firewall-rules": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--action": [
                  "ALLOW",
                  "DENY"
                ],
                "--allow": "value",
                "--description": "value",
                "--destination-ranges": "value",
                "--direction": [
                  "EGRESS",
                  "IN",
                  "INGRESS",
                  "OUT"
                ],
                "--disabled": "bool",
                "--enable-logging": "bool",
                "--logging-metadata": [
                  "exclude-all",
                  "include-all"
                ],
                "--network": "value",
                "--priority": "value",
                "--rules": "value",
                "--source-ranges": "value",
                "--source-service-accounts": "value",
                "--source-tags": "value",
                "--target-service-accounts": "value",
                "--target-tags": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--allow": "value",
                "--description": "value",
                "--destination-ranges": "value",
                "--disabled": "bool",
                "--enable-logging": "bool",
                "--logging-metadata": [
                  "exclude-all",
                  "include-all"
                ],
                "--priority": "value",
                "--rules": "value",
                "--source-ranges": "value",
                "--source-service-accounts": "value",
                "--source-tags": "value",
                "--target-service-accounts": "value",
                "--target-tags": "value"
              }
            }
          },
          "flags": {}
        },
        "forwarding-rules": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--address": "dynamic",
                "--address-region": "dynamic",
                "--allow-global-access": "bool",
                "--backend-service": "value",
                "--backend-service-region": "dynamic",
                "--description": "value",
                "--disable-automate-dns-zone": "bool",
                "--global": "bool",
                "--global-address": "bool",
                "--global-backend-service": "bool",
                "--global-target-http-proxy": "bool",
                "--global-target-https-proxy": "bool",
                "--global-target-tcp-proxy": "bool",
                "--ip-protocol": [
                  "AH",
                  "ESP",
                  "ICMP",
                  "L3_DEFAULT",
                  "SCTP",
                  "TCP",
                  "UDP"
                ],
                "--ip-version": [
                  "IPV4",
                  "IPV6"
                ],
                "--is-mirroring-collector": "bool",
                "--load-balancing-scheme": [
                  "EXTERNAL",
                  "EXTERNAL_MANAGED",
                  "INTERNAL",
                  "INTERNAL_MANAGED",
                  "INTERNAL_SELF_MANAGED"
                ],
                "--network": "value",
                "--network-tier": "value",
                "--port-range": "value",
                "--ports": "value",
                "--region": "dynamic",
                "--service-directory-registration": "value",
                "--service-label": "value",
                "--subnet": "value",
                "--subnet-region": "dynamic",
                "--target-google-apis-bundle": "value",
                "--target-grpc-proxy": "value",
                "--target-http-proxy": "value",
                "--target-http-proxy-region": "dynamic",
                "--target-https-proxy": "value",
                "--target-https-proxy-region": "dynamic",
                "--target-instance": "value",
                "--target-instance-zone": "dynamic",
                "--target-pool": "value",
                "--target-pool-region": "dynamic",
                "--target-service-attachment": "value",
                "--target-service-attachment-region": "dynamic",
                "--target-ssl-proxy": "value",
                "--target-tcp-proxy": "value",
                "--target-tcp-proxy-region": "dynamic",
                "--target-vpn-gateway": "value",
                "--target-vpn-gateway-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-target": {
              "commands": {},
              "flags": {
                "--backend-service": "value",
                "--backend-service-region": "dynamic",
                "--global": "bool",
                "--global-backend-service": "bool",
                "--global-target-http-proxy": "bool",
                "--global-target-https-proxy": "bool",
                "--global-target-tcp-proxy": "bool",
                "--load-balancing-scheme": [
                  "EXTERNAL",
                  "EXTERNAL_MANAGED",
                  "INTERNAL",
                  "INTERNAL_MANAGED",
                  "INTERNAL_SELF_MANAGED"
                ],
                "--network": "value",
                "--region": "dynamic",
                "--subnet": "value",
                "--subnet-region": "value",
                "--target-grpc-proxy": "value",
                "--target-http-proxy": "value",
                "--target-http-proxy-region": "dynamic",
                "--target-https-proxy": "value",
                "--target-https-proxy-region": "dynamic",
                "--target-instance": "value",
                "--target-instance-zone": "dynamic",
                "--target-pool": "value",
                "--target-pool-region": "dynamic",
                "--target-ssl-proxy": "value",
                "--target-tcp-proxy": "value",
                "--target-tcp-proxy-region": "dynamic",
                "--target-vpn-gateway": "value",
                "--target-vpn-gateway-region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--allow-global-access": "bool",
                "--clear-labels": "bool",
                "--global": "bool",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "health-checks": {
          "commands": {
            "create": {
              "commands": {
                "grpc": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--grpc-service-name": "value",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--region": "dynamic",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "http": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "http2": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "https": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "ssl": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "tcp": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--protocol": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {
                "grpc": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--grpc-service-name": "value",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--region": "dynamic",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "http": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "http2": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "https": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--host": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request-path": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "ssl": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                },
                "tcp": {
                  "commands": {},
                  "flags": {
                    "--check-interval": "value",
                    "--description": "value",
                    "--enable-logging": "bool",
                    "--global": "bool",
                    "--healthy-threshold": "value",
                    "--port": "value",
                    "--port-name": "value",
                    "--proxy-header": [
                      "NONE",
                      "PROXY_V1"
                    ],
                    "--region": "dynamic",
                    "--request": "value",
                    "--response": "value",
                    "--timeout": "value",
                    "--unhealthy-threshold": "value",
                    "--use-serving-port": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "http-health-checks": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--check-interval": "value",
                "--description": "value",
                "--healthy-threshold": "value",
                "--host": "value",
                "--port": "value",
                "--request-path": "value",
                "--timeout": "value",
                "--unhealthy-threshold": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--check-interval": "value",
                "--description": "value",
                "--healthy-threshold": "value",
                "--host": "value",
                "--port": "value",
                "--request-path": "value",
                "--timeout": "value",
                "--unhealthy-threshold": "value"
              }
            }
          },
          "flags": {}
        },
        "https-health-checks": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--check-interval": "value",
                "--description": "value",
                "--healthy-threshold": "value",
                "--host": "value",
                "--port": "value",
                "--request-path": "value",
                "--timeout": "value",
                "--unhealthy-threshold": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--check-interval": "value",
                "--description": "value",
                "--healthy-threshold": "value",
                "--host": "value",
                "--port": "value",
                "--request-path": "value",
                "--timeout": "value",
                "--unhealthy-threshold": "value"
              }
            }
          },
          "flags": {}
        },
        "images": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "add-labels": {
              "commands": {},
              "flags": {
                "--labels": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--architecture": [
                  "ARM64",
                  "X86_64"
                ],
                "--csek-key-file": "value",
                "--description": "value",
                "--family": "value",
                "--forbidden-database-file": "value",
                "--force": "bool",
                "--guest-os-features": "value",
                "--key-exchange-key-file": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--licenses": "value",
                "--platform-key-file": "value",
                "--require-csek-key-create": "bool",
                "--signature-database-file": "value",
                "--source-disk": "dynamic",
                "--source-disk-project": "value",
                "--source-disk-zone": "dynamic",
                "--source-image": "value",
                "--source-image-family": "value",
                "--source-image-project": "value",
                "--source-snapshot": "dynamic",
                "--source-uri": "value",
                "--storage-location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "deprecate": {
              "commands": {},
              "flags": {
                "--delete-in": "value",
                "--delete-on": "value",
                "--deprecate-in": "value",
                "--deprecate-on": "value",
                "--obsolete-in": "value",
                "--obsolete-on": "value",
                "--replacement": "dynamic",
                "--state": [
                  "ACTIVE",
                  "DELETED",
                  "DEPRECATED",
                  "OBSOLETE"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "describe-from-family": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--compute-service-account": "value",
                "--destination-uri": "value",
                "--export-format": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-project": "value",
                "--log-location": "value",
                "--network": "value",
                "--subnet": "value",
                "--timeout": "value",
                "--zone": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--byol": "bool",
                "--compute-service-account": "value",
                "--data-disk": "bool",
                "--description": "value",
                "--family": "value",
                "--guest-environment": "bool",
                "--guest-os-features": "value",
                "--log-location": "value",
                "--network": "value",
                "--no-address": "bool",
                "--os": [
                  "centos-7",
                  "debian-10",
                  "debian-11",
                  "debian-8",
                  "debian-9",
                  "opensuse-15",
                  "rhel-6",
                  "rhel-6-byol",
                  "rhel-7",
                  "rhel-7-byol",
                  "rhel-8",
                  "rhel-8-byol",
                  "rhel-9",
                  "rhel-9-byol",
                  "rocky-8",
                  "sles-12",
                  "sles-12-byol",
                  "sles-15",
                  "sles-15-byol",
                  "sles-sap-12",
                  "sles-sap-12-byol",
                  "sles-sap-15",
                  "sles-sap-15-byol",
                  "ubuntu-1404",
                  "ubuntu-1604",
                  "ubuntu-1804",
                  "ubuntu-2004",
                  "ubuntu-2204",
                  "windows-10-x64-byol",
                  "windows-10-x86-byol",
                  "windows-11-x64-byol",
                  "windows-2008r2",
                  "windows-2008r2-byol",
                  "windows-2012",
                  "windows-2012-byol",
                  "windows-2012r2",
                  "windows-2012r2-byol",
                  "windows-2016",
                  "windows-2016-byol",
                  "windows-2019",
                  "windows-2019-byol",
                  "windows-2022",
                  "windows-2022-byol",
                  "windows-7-x64-byol",
                  "windows-7-x86-byol",
                  "windows-8-x64-byol",
                  "windows-8-x86-byol"
                ],
                "--source-file": "value",
                "--source-image": "dynamic",
                "--storage-location": "value",
                "--subnet": "value",
                "--timeout": "value",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--preview-images": "bool",
                "--regexp": "value",
                "--show-deprecated": "bool",
                "--sort-by": "value",
                "--standard-images": "bool",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "remove-labels": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--labels": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--architecture": [
                  "ARM64",
                  "X86_64"
                ],
                "--clear-labels": "bool",
                "--description": "value",
                "--family": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "instance-groups": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "get-named-ports": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--only-managed": "bool",
                "--only-unmanaged": "bool",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "value"
              }
            },
            "list-instances": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "dynamic"
              }
            },
            "managed": {
              "commands": {
                "abandon-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--base-instance-name": "value",
                    "--description": "value",
                    "--health-check": "dynamic",
                    "--http-health-check": "value",
                    "--https-health-check": "value",
                    "--initial-delay": "value",
                    "--instance-redistribution-type": [
                      "none",
                      "proactive"
                    ],
                    "--list-managed-instances-results": [
                      "pageless",
                      "paginated"
                    ],
                    "--region": "dynamic",
                    "--size": "value",
                    "--stateful-disk": "value",
                    "--target-distribution-shape": [
                      "any",
                      "balanced",
                      "even"
                    ],
                    "--target-pool": "value",
                    "--template": "value",
                    "--update-policy-max-surge": "value",
                    "--update-policy-max-unavailable": "value",
                    "--update-policy-minimal-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--update-policy-most-disruptive-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--update-policy-replacement-method": [
                      "recreate",
                      "substitute"
                    ],
                    "--update-policy-type": [
                      "opportunistic",
                      "proactive"
                    ],
                    "--zone": "dynamic",
                    "--zones": "dynamic"
                  }
                },
                "create-instance": {
                  "commands": {},
                  "flags": {
                    "--instance": "value",
                    "--region": "dynamic",
                    "--stateful-disk": "value",
                    "--stateful-metadata": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "delete-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "value",
                    "--region": "dynamic",
                    "--skip-instances-on-validation-error": "bool",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe-instance": {
                  "commands": {},
                  "flags": {
                    "--instance": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-named-ports": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "instance-configs": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--instance-update-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--region": "dynamic",
                        "--stateful-disk": "value",
                        "--stateful-metadata": "value",
                        "--update-instance": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--instance-update-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--instances": "value",
                        "--region": "dynamic",
                        "--update-instance": "bool",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--instance": "value",
                        "--instance-update-minimal-action": [
                          "none",
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--region": "dynamic",
                        "--remove-stateful-disks": "value",
                        "--remove-stateful-metadata": "value",
                        "--stateful-disk": "value",
                        "--stateful-metadata": "value",
                        "--update-instance": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "value"
                  }
                },
                "list-errors": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "list-instances": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "recreate-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "resize": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--size": "value",
                    "--zone": "dynamic"
                  }
                },
                "rolling-action": {
                  "commands": {
                    "replace": {
                      "commands": {},
                      "flags": {
                        "--max-surge": "value",
                        "--max-unavailable": "value",
                        "--region": "dynamic",
                        "--replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--zone": "dynamic"
                      }
                    },
                    "restart": {
                      "commands": {},
                      "flags": {
                        "--max-unavailable": "value",
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "start-update": {
                      "commands": {},
                      "flags": {
                        "--canary-version": "value",
                        "--force": "bool",
                        "--max-surge": "value",
                        "--max-unavailable": "value",
                        "--minimal-action": [
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--most-disruptive-allowed-action": [
                          "refresh",
                          "replace",
                          "restart"
                        ],
                        "--region": "dynamic",
                        "--replacement-method": [
                          "recreate",
                          "substitute"
                        ],
                        "--type": [
                          "opportunistic",
                          "proactive"
                        ],
                        "--zone": "dynamic"
                      }
                    },
                    "stop-proactive-update": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "set-autoscaling": {
                  "commands": {},
                  "flags": {
                    "--cool-down-period": "value",
                    "--cpu-utilization-predictive-method": [
                      "none",
                      "optimize-availability"
                    ],
                    "--custom-metric-utilization": "value",
                    "--description": "value",
                    "--max-num-replicas": "value",
                    "--min-num-replicas": "value",
                    "--mode": [
                      "off",
                      "on",
                      "only-scale-out",
                      "only-up"
                    ],
                    "--region": "dynamic",
                    "--remove-stackdriver-metric": "value",
                    "--scale-based-on-cpu": "bool",
                    "--scale-based-on-load-balancing": "bool",
                    "--scale-in-control": "value",
                    "--schedule-cron": "value",
                    "--schedule-description": "value",
                    "--schedule-duration-sec": "value",
                    "--schedule-min-required-replicas": "value",
                    "--schedule-time-zone": "value",
                    "--set-schedule": "value",
                    "--stackdriver-metric-filter": "value",
                    "--stackdriver-metric-single-instance-assignment": "value",
                    "--stackdriver-metric-utilization-target": "value",
                    "--stackdriver-metric-utilization-target-type": [
                      "delta-per-minute",
                      "delta-per-second",
                      "gauge"
                    ],
                    "--target-cpu-utilization": "value",
                    "--target-load-balancing-utilization": "value",
                    "--update-stackdriver-metric": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-instance-template": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--template": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-named-ports": {
                  "commands": {},
                  "flags": {
                    "--named-ports": "value",
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "set-target-pools": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--target-pools": "value",
                    "--zone": "dynamic"
                  }
                },
                "stop-autoscaling": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-autohealing": "bool",
                    "--description": "value",
                    "--health-check": "dynamic",
                    "--http-health-check": "value",
                    "--https-health-check": "value",
                    "--initial-delay": "value",
                    "--instance-redistribution-type": [
                      "none",
                      "proactive"
                    ],
                    "--list-managed-instances-results": [
                      "pageless",
                      "paginated"
                    ],
                    "--region": "dynamic",
                    "--remove-stateful-disks": "value",
                    "--stateful-disk": "value",
                    "--target-distribution-shape": [
                      "any",
                      "balanced",
                      "even"
                    ],
                    "--update-policy-max-surge": "value",
                    "--update-policy-max-unavailable": "value",
                    "--update-policy-minimal-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--update-policy-most-disruptive-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--update-policy-replacement-method": [
                      "recreate",
                      "substitute"
                    ],
                    "--update-policy-type": [
                      "opportunistic",
                      "proactive"
                    ],
                    "--zone": "dynamic"
                  }
                },
                "update-autoscaling": {
                  "commands": {},
                  "flags": {
                    "--clear-scale-in-control": "bool",
                    "--cpu-utilization-predictive-method": [
                      "none",
                      "optimize-availability"
                    ],
                    "--disable-schedule": "value",
                    "--enable-schedule": "value",
                    "--max-num-replicas": "value",
                    "--min-num-replicas": "value",
                    "--mode": [
                      "off",
                      "on",
                      "only-scale-out",
                      "only-up"
                    ],
                    "--region": "dynamic",
                    "--remove-schedule": "value",
                    "--scale-in-control": "value",
                    "--schedule-cron": "value",
                    "--schedule-description": "value",
                    "--schedule-duration-sec": "value",
                    "--schedule-min-required-replicas": "value",
                    "--schedule-time-zone": "value",
                    "--set-schedule": "value",
                    "--update-schedule": "value",
                    "--zone": "dynamic"
                  }
                },
                "update-instances": {
                  "commands": {},
                  "flags": {
                    "--all-instances": "bool",
                    "--instances": "value",
                    "--minimal-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--most-disruptive-allowed-action": [
                      "none",
                      "refresh",
                      "replace",
                      "restart"
                    ],
                    "--region": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "wait-until": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--stable": "bool",
                    "--timeout": "value",
                    "--version-target-reached": "bool",
                    "--zone": "dynamic"
                  }
                },
                "wait-until-stable": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--timeout": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "set-named-ports": {
              "commands": {},
              "flags": {
                "--named-ports": "value",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "unmanaged": {
              "commands": {
                "add-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "value",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "get-named-ports": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zones": "dynamic"
                  }
                },
                "list-instances": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "remove-instances": {
                  "commands": {},
                  "flags": {
                    "--instances": "value",
                    "--zone": "dynamic"
                  }
                },
                "set-named-ports": {
                  "commands": {},
                  "flags": {
                    "--named-ports": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "instance-templates": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--address": "value",
                "--boot-disk-auto-delete": "bool",
                "--boot-disk-device-name": "value",
                "--boot-disk-kms-key": "dynamic",
                "--boot-disk-kms-keyring": "dynamic",
                "--boot-disk-kms-location": "dynamic",
                "--boot-disk-kms-project": "dynamic",
                "--boot-disk-provisioned-iops": "value",
                "--boot-disk-size": "value",
                "--boot-disk-type": "value",
                "--can-ip-forward": "bool",
                "--confidential-compute": "bool",
                "--configure-disk": "value",
                "--create-disk": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--description": "value",
                "--disk": "value",
                "--enable-nested-virtualization": "bool",
                "--enable-uefi-networking": "bool",
                "--external-ipv6-address": "value",
                "--external-ipv6-prefix-length": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-project": "value",
                "--instance-termination-action": [
                  "DELETE",
                  "STOP"
                ],
                "--internal-ipv6-address": "value",
                "--internal-ipv6-prefix-length": "value",
                "--ipv6-network-tier": [
                  "PREMIUM"
                ],
                "--key-revocation-action-type": [
                  "none",
                  "stop"
                ],
                "--labels": "value",
                "--local-ssd": "value",
                "--machine-type": "dynamic",
                "--maintenance-policy": [
                  "MIGRATE",
                  "TERMINATE"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-cpu-platform": "value",
                "--min-node-cpu": "value",
                "--network": "value",
                "--network-interface": "value",
                "--network-performance-configs": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--node": "value",
                "--node-affinity-file": "value",
                "--node-group": "value",
                "--preemptible": "bool",
                "--private-ipv6-google-access-type": [
                  "enable-bidirectional-access",
                  "enable-outbound-vm-access",
                  "inherit-subnetwork"
                ],
                "--private-network-ip": "value",
                "--provisioning-model": [
                  "SPOT",
                  "STANDARD"
                ],
                "--region": "dynamic",
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--resource-manager-tags": "value",
                "--resource-policies": "value",
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--service-proxy": "value",
                "--service-proxy-labels": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--source-instance": "dynamic",
                "--source-instance-zone": "dynamic",
                "--stack-type": [
                  "IPV4_IPV6",
                  "IPV4_ONLY"
                ],
                "--subnet": "value",
                "--tags": "value",
                "--threads-per-core": "value",
                "--visible-core-count": "value"
              }
            },
            "create-with-container": {
              "commands": {},
              "flags": {
                "--address": "value",
                "--boot-disk-auto-delete": "bool",
                "--boot-disk-device-name": "value",
                "--boot-disk-provisioned-iops": "value",
                "--boot-disk-size": "value",
                "--boot-disk-type": "value",
                "--can-ip-forward": "bool",
                "--confidential-compute": "bool",
                "--container-arg": "value",
                "--container-command": "value",
                "--container-env": "value",
                "--container-env-file": "value",
                "--container-image": "value",
                "--container-mount-disk": "value",
                "--container-mount-host-path": "value",
                "--container-mount-tmpfs": "value",
                "--container-privileged": "bool",
                "--container-restart-policy": [
                  "always",
                  "never",
                  "on-failure"
                ],
                "--container-stdin": "bool",
                "--container-tty": "bool",
                "--create-disk": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--description": "value",
                "--disk": "value",
                "--external-ipv6-address": "value",
                "--external-ipv6-prefix-length": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-project": "value",
                "--labels": "value",
                "--machine-type": "dynamic",
                "--maintenance-policy": [
                  "MIGRATE",
                  "TERMINATE"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-cpu-platform": "value",
                "--network": "value",
                "--network-interface": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--preemptible": "bool",
                "--private-ipv6-google-access-type": [
                  "enable-bidirectional-access",
                  "enable-outbound-vm-access",
                  "inherit-subnetwork"
                ],
                "--private-network-ip": "value",
                "--region": "dynamic",
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--subnet": "value",
                "--tags": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "add-access-config": {
              "commands": {},
              "flags": {
                "--access-config-name": "value",
                "--address": "value",
                "--network-interface": "value",
                "--network-tier": "value",
                "--no-public-ptr": "bool",
                "--no-public-ptr-domain": "bool",
                "--public-ptr": "bool",
                "--public-ptr-domain": "value",
                "--zone": "dynamic"
              }
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "add-labels": {
              "commands": {},
              "flags": {
                "--labels": "value",
                "--zone": "dynamic"
              }
            },
            "add-metadata": {
              "commands": {},
              "flags": {
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--zone": "dynamic"
              }
            },
            "add-resource-policies": {
              "commands": {},
              "flags": {
                "--resource-policies": "value",
                "--zone": "dynamic"
              }
            },
            "add-tags": {
              "commands": {},
              "flags": {
                "--tags": "value",
                "--zone": "dynamic"
              }
            },
            "attach-disk": {
              "commands": {},
              "flags": {
                "--boot": "bool",
                "--csek-key-file": "value",
                "--device-name": "value",
                "--disk": "value",
                "--disk-scope": [
                  "regional",
                  "zonal"
                ],
                "--force-attach": "bool",
                "--mode": [
                  "ro",
                  "rw"
                ],
                "--zone": "dynamic"
              }
            },
            "bulk": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--async": "bool",
                    "--boot-disk-auto-delete": "bool",
                    "--boot-disk-device-name": "value",
                    "--boot-disk-kms-key": "dynamic",
                    "--boot-disk-kms-keyring": "dynamic",
                    "--boot-disk-kms-location": "dynamic",
                    "--boot-disk-kms-project": "dynamic",
                    "--boot-disk-provisioned-iops": "value",
                    "--boot-disk-size": "value",
                    "--boot-disk-type": "value",
                    "--can-ip-forward": "bool",
                    "--confidential-compute": "bool",
                    "--count": "value",
                    "--create-disk": "value",
                    "--custom-cpu": "value",
                    "--custom-extensions": "bool",
                    "--custom-memory": "value",
                    "--custom-vm-type": "value",
                    "--description": "value",
                    "--disk": "value",
                    "--enable-nested-virtualization": "bool",
                    "--enable-uefi-networking": "bool",
                    "--erase-windows-vss-signature": "bool",
                    "--image": "value",
                    "--image-family": "value",
                    "--image-project": "value",
                    "--instance-termination-action": [
                      "DELETE",
                      "STOP"
                    ],
                    "--labels": "value",
                    "--local-ssd": "value",
                    "--location-policy": "value",
                    "--machine-type": "dynamic",
                    "--maintenance-policy": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--min-count": "value",
                    "--min-cpu-platform": "value",
                    "--min-node-cpu": "value",
                    "--name-pattern": "value",
                    "--network": "value",
                    "--network-interface": "value",
                    "--network-performance-configs": "value",
                    "--network-tier": "value",
                    "--no-address": "bool",
                    "--no-public-dns": "bool",
                    "--no-scopes": "bool",
                    "--no-service-account": "bool",
                    "--on-host-maintenance": [
                      "MIGRATE",
                      "TERMINATE"
                    ],
                    "--post-key-revocation-action-type": [
                      "noop",
                      "shutdown"
                    ],
                    "--predefined-names": "value",
                    "--preemptible": "bool",
                    "--provisioning-model": [
                      "SPOT",
                      "STANDARD"
                    ],
                    "--public-dns": "bool",
                    "--region": "value",
                    "--reservation": "value",
                    "--reservation-affinity": [
                      "any",
                      "none",
                      "specific"
                    ],
                    "--resource-manager-tags": "value",
                    "--resource-policies": "value",
                    "--restart-on-failure": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-integrity-monitoring": "bool",
                    "--shielded-secure-boot": "bool",
                    "--shielded-vtpm": "bool",
                    "--source-instance-template": "dynamic",
                    "--source-snapshot": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnet": "value",
                    "--tags": "value",
                    "--target-distribution-shape": [
                      "ANY",
                      "ANY_SINGLE_ZONE",
                      "BALANCED"
                    ],
                    "--threads-per-core": "value",
                    "--visible-core-count": "value",
                    "--zone": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--address": "value",
                "--async": "bool",
                "--boot-disk-auto-delete": "bool",
                "--boot-disk-device-name": "value",
                "--boot-disk-kms-key": "dynamic",
                "--boot-disk-kms-keyring": "dynamic",
                "--boot-disk-kms-location": "dynamic",
                "--boot-disk-kms-project": "dynamic",
                "--boot-disk-provisioned-iops": "value",
                "--boot-disk-size": "value",
                "--boot-disk-type": "value",
                "--can-ip-forward": "bool",
                "--confidential-compute": "bool",
                "--create-disk": "value",
                "--csek-key-file": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--deletion-protection": "bool",
                "--description": "value",
                "--disk": "value",
                "--enable-display-device": "bool",
                "--enable-nested-virtualization": "bool",
                "--enable-uefi-networking": "bool",
                "--erase-windows-vss-signature": "bool",
                "--external-ipv6-address": "value",
                "--external-ipv6-prefix-length": "value",
                "--hostname": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-family-scope": [
                  "global",
                  "zonal"
                ],
                "--image-project": "value",
                "--instance-termination-action": [
                  "DELETE",
                  "STOP"
                ],
                "--internal-ipv6-address": "value",
                "--internal-ipv6-prefix-length": "value",
                "--ipv6-network-tier": [
                  "PREMIUM"
                ],
                "--ipv6-public-ptr-domain": "value",
                "--key-revocation-action-type": [
                  "none",
                  "stop"
                ],
                "--labels": "value",
                "--local-ssd": "value",
                "--machine-type": "dynamic",
                "--maintenance-policy": [
                  "MIGRATE",
                  "TERMINATE"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-cpu-platform": "value",
                "--min-node-cpu": "value",
                "--network": "value",
                "--network-interface": "value",
                "--network-performance-configs": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-public-ptr": "bool",
                "--no-public-ptr-domain": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--node": "value",
                "--node-affinity-file": "value",
                "--node-group": "value",
                "--node-project": "value",
                "--preemptible": "bool",
                "--private-ipv6-google-access-type": [
                  "enable-bidirectional-access",
                  "enable-outbound-vm-access",
                  "inherit-subnetwork"
                ],
                "--private-network-ip": "value",
                "--provisioning-model": [
                  "SPOT",
                  "STANDARD"
                ],
                "--public-ptr": "bool",
                "--public-ptr-domain": "value",
                "--require-csek-key-create": "bool",
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--resource-manager-tags": "value",
                "--resource-policies": "value",
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--source-instance-template": "dynamic",
                "--source-machine-image": "dynamic",
                "--source-machine-image-csek-key-file": "value",
                "--source-snapshot": "value",
                "--stack-type": [
                  "IPV4_IPV6",
                  "IPV4_ONLY"
                ],
                "--subnet": "value",
                "--tags": "value",
                "--threads-per-core": "value",
                "--visible-core-count": "value",
                "--zone": "dynamic"
              }
            },
            "create-with-container": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--address": "value",
                "--boot-disk-auto-delete": "bool",
                "--boot-disk-device-name": "value",
                "--boot-disk-provisioned-iops": "value",
                "--boot-disk-size": "value",
                "--boot-disk-type": "value",
                "--can-ip-forward": "bool",
                "--confidential-compute": "bool",
                "--container-arg": "value",
                "--container-command": "value",
                "--container-env": "value",
                "--container-env-file": "value",
                "--container-image": "value",
                "--container-mount-disk": "value",
                "--container-mount-host-path": "value",
                "--container-mount-tmpfs": "value",
                "--container-privileged": "bool",
                "--container-restart-policy": [
                  "always",
                  "never",
                  "on-failure"
                ],
                "--container-stdin": "bool",
                "--container-tty": "bool",
                "--create-disk": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--description": "value",
                "--disk": "value",
                "--enable-nested-virtualization": "bool",
                "--external-ipv6-address": "value",
                "--external-ipv6-prefix-length": "value",
                "--image": "value",
                "--image-family": "value",
                "--image-project": "value",
                "--instance-termination-action": [
                  "DELETE",
                  "STOP"
                ],
                "--labels": "value",
                "--machine-type": "dynamic",
                "--maintenance-policy": [
                  "MIGRATE",
                  "TERMINATE"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-cpu-platform": "value",
                "--network": "value",
                "--network-interface": "value",
                "--network-performance-configs": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-public-ptr": "bool",
                "--no-public-ptr-domain": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--preemptible": "bool",
                "--private-ipv6-google-access-type": [
                  "enable-bidirectional-access",
                  "enable-outbound-vm-access",
                  "inherit-subnetwork"
                ],
                "--private-network-ip": "value",
                "--provisioning-model": [
                  "SPOT",
                  "STANDARD"
                ],
                "--public-ptr": "bool",
                "--public-ptr-domain": "value",
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--source-instance-template": "dynamic",
                "--subnet": "value",
                "--tags": "value",
                "--threads-per-core": "value",
                "--visible-core-count": "value",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--delete-disks": [
                  "all",
                  "boot",
                  "data"
                ],
                "--keep-disks": [
                  "all",
                  "boot",
                  "data"
                ],
                "--zone": "dynamic"
              }
            },
            "delete-access-config": {
              "commands": {},
              "flags": {
                "--access-config-name": "value",
                "--network-interface": "value",
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "detach-disk": {
              "commands": {},
              "flags": {
                "--device-name": "value",
                "--disk": "value",
                "--disk-scope": [
                  "regional",
                  "zonal"
                ],
                "--zone": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--zone": "value"
              }
            },
            "get-guest-attributes": {
              "commands": {},
              "flags": {
                "--query-path": "value",
                "--zone": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "get-screenshot": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--zone": "dynamic"
              }
            },
            "get-serial-port-output": {
              "commands": {},
              "flags": {
                "--port": "value",
                "--start": "value",
                "--zone": "dynamic"
              }
            },
            "get-shielded-identity": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--byol": "bool",
                "--can-ip-forward": "bool",
                "--compute-service-account": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--deletion-protection": "bool",
                "--description": "value",
                "--guest-environment": "bool",
                "--guest-os-features": "value",
                "--hostname": "value",
                "--labels": "value",
                "--log-location": "value",
                "--machine-type": "dynamic",
                "--network": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--node": "value",
                "--node-affinity-file": "value",
                "--node-group": "value",
                "--os": [
                  "centos-7",
                  "debian-10",
                  "debian-11",
                  "debian-8",
                  "debian-9",
                  "opensuse-15",
                  "rhel-6",
                  "rhel-6-byol",
                  "rhel-7",
                  "rhel-7-byol",
                  "rhel-8",
                  "rhel-8-byol",
                  "rhel-9",
                  "rhel-9-byol",
                  "rocky-8",
                  "sles-12",
                  "sles-12-byol",
                  "sles-15",
                  "sles-15-byol",
                  "sles-sap-12",
                  "sles-sap-12-byol",
                  "sles-sap-15",
                  "sles-sap-15-byol",
                  "ubuntu-1404",
                  "ubuntu-1604",
                  "ubuntu-1804",
                  "ubuntu-2004",
                  "ubuntu-2204",
                  "windows-10-x64-byol",
                  "windows-10-x86-byol",
                  "windows-11-x64-byol",
                  "windows-2008r2",
                  "windows-2008r2-byol",
                  "windows-2012",
                  "windows-2012-byol",
                  "windows-2012r2",
                  "windows-2012r2-byol",
                  "windows-2016",
                  "windows-2016-byol",
                  "windows-2019",
                  "windows-2019-byol",
                  "windows-2022",
                  "windows-2022-byol",
                  "windows-7-x64-byol",
                  "windows-7-x86-byol",
                  "windows-8-x64-byol",
                  "windows-8-x86-byol"
                ],
                "--private-network-ip": "value",
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--source-uri": "value",
                "--subnet": "value",
                "--tags": "value",
                "--timeout": "value",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "dynamic"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--destination-zone": "dynamic",
                "--zone": "dynamic"
              }
            },
            "network-interfaces": {
              "commands": {
                "get-effective-firewalls": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network-interface": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--aliases": "value",
                    "--external-ipv6-address": "value",
                    "--external-ipv6-prefix-length": "value",
                    "--internal-ipv6-address": "value",
                    "--internal-ipv6-prefix-length": "value",
                    "--ipv6-network-tier": [
                      "PREMIUM"
                    ],
                    "--network": "value",
                    "--network-interface": "value",
                    "--private-network-ip": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ],
                    "--subnetwork": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "os-inventory": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list-instances": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--inventory-filter": "value",
                    "--kernel-version": "value",
                    "--limit": "value",
                    "--os-shortname": "value",
                    "--os-version": "value",
                    "--package-name": "value",
                    "--package-version": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "remove-labels": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--labels": "value",
                "--zone": "dynamic"
              }
            },
            "remove-metadata": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--keys": "value",
                "--zone": "dynamic"
              }
            },
            "remove-resource-policies": {
              "commands": {},
              "flags": {
                "--resource-policies": "value",
                "--zone": "dynamic"
              }
            },
            "remove-tags": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--tags": "value",
                "--zone": "dynamic"
              }
            },
            "reset": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "resume": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "dynamic"
              }
            },
            "send-diagnostic-interrupt": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "set-disk-auto-delete": {
              "commands": {},
              "flags": {
                "--auto-delete": "bool",
                "--device-name": "value",
                "--disk": "value",
                "--zone": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "set-machine-type": {
              "commands": {},
              "flags": {
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--machine-type": "dynamic",
                "--zone": "dynamic"
              }
            },
            "set-scheduling": {
              "commands": {},
              "flags": {
                "--clear-instance-termination-action": "bool",
                "--clear-min-node-cpu": "bool",
                "--clear-node-affinities": "bool",
                "--instance-termination-action": [
                  "DELETE",
                  "STOP"
                ],
                "--maintenance-policy": [
                  "MIGRATE",
                  "TERMINATE"
                ],
                "--min-node-cpu": "value",
                "--node": "value",
                "--node-affinity-file": "value",
                "--node-group": "value",
                "--preemptible": "bool",
                "--provisioning-model": [
                  "SPOT",
                  "STANDARD"
                ],
                "--restart-on-failure": "bool",
                "--zone": "dynamic"
              }
            },
            "set-service-account": {
              "commands": {},
              "flags": {
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--zone": "dynamic"
              }
            },
            "simulate-maintenance-event": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "dynamic"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--csek-key-file": "value",
                "--zone": "dynamic"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--discard-local-ssd": "value",
                "--zone": "dynamic"
              }
            },
            "suspend": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--discard-local-ssd": "value",
                "--zone": "dynamic"
              }
            },
            "tail-serial-port-output": {
              "commands": {},
              "flags": {
                "--port": "value",
                "--zone": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--clear-node-affinities": "bool",
                "--deletion-protection": "bool",
                "--enable-display-device": "bool",
                "--min-cpu-platform": "value",
                "--node": "value",
                "--node-affinity-file": "value",
                "--node-group": "value",
                "--remove-labels": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-learn-integrity-policy": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--update-labels": "value",
                "--zone": "dynamic"
              }
            },
            "update-access-config": {
              "commands": {},
              "flags": {
                "--ipv6-public-ptr-domain": "value",
                "--network-interface": "value",
                "--no-ipv6-public-ptr": "bool",
                "--no-public-ptr": "bool",
                "--no-public-ptr-domain": "bool",
                "--public-ptr": "bool",
                "--public-ptr-domain": "value",
                "--zone": "dynamic"
              }
            },
            "update-container": {
              "commands": {},
              "flags": {
                "--clear-container-args": "bool",
                "--clear-container-command": "bool",
                "--container-arg": "value",
                "--container-command": "value",
                "--container-env": "value",
                "--container-env-file": "value",
                "--container-image": "value",
                "--container-mount-disk": "value",
                "--container-mount-host-path": "value",
                "--container-mount-tmpfs": "value",
                "--container-privileged": "bool",
                "--container-restart-policy": [
                  "always",
                  "never",
                  "on-failure"
                ],
                "--container-stdin": "bool",
                "--container-tty": "bool",
                "--remove-container-env": "value",
                "--remove-container-mounts": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-learn-integrity-policy": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--zone": "dynamic"
              }
            },
            "update-from-file": {
              "commands": {},
              "flags": {
                "--minimal-action": "value",
                "--most-disruptive-allowed-action": "value",
                "--source": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "interconnects": {
          "commands": {
            "attachments": {
              "commands": {
                "dedicated": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--bandwidth": [
                          "100m",
                          "10g",
                          "1g",
                          "200m",
                          "20g",
                          "2g",
                          "300m",
                          "400m",
                          "500m",
                          "50g",
                          "50m",
                          "5g"
                        ],
                        "--candidate-ipv6-subnets": "value",
                        "--candidate-subnets": "value",
                        "--cloud-router-ipv6-interface-id": "value",
                        "--customer-router-ipv6-interface-id": "value",
                        "--description": "value",
                        "--enable-admin": "bool",
                        "--encryption": [
                          "IPSEC",
                          "NONE"
                        ],
                        "--interconnect": "dynamic",
                        "--ipsec-internal-addresses": "value",
                        "--mtu": "value",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ],
                        "--vlan": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--bandwidth": [
                          "100m",
                          "10g",
                          "1g",
                          "200m",
                          "20g",
                          "2g",
                          "300m",
                          "400m",
                          "500m",
                          "50g",
                          "50m",
                          "5g"
                        ],
                        "--candidate-ipv6-subnets": "value",
                        "--cloud-router-ipv6-interface-id": "value",
                        "--customer-router-ipv6-interface-id": "value",
                        "--description": "value",
                        "--enable-admin": "bool",
                        "--mtu": "value",
                        "--region": "dynamic",
                        "--stack-type": [
                          "IPV4_IPV6",
                          "IPV4_ONLY"
                        ]
                      }
                    }
                  },
                  "flags": {}
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "partner": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--edge-availability-domain": [
                          "any",
                          "availability-domain-1",
                          "availability-domain-2"
                        ],
                        "--enable-admin": "bool",
                        "--encryption": [
                          "IPSEC",
                          "NONE"
                        ],
                        "--ipsec-internal-addresses": "value",
                        "--mtu": "value",
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--description": "value",
                        "--enable-admin": "bool",
                        "--mtu": "value",
                        "--region": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--admin-enabled": "bool",
                "--customer-name": "value",
                "--description": "value",
                "--interconnect-type": [
                  "DEDICATED",
                  "PARTNER"
                ],
                "--link-type": [
                  "LINK_TYPE_ETHERNET_100G_LR",
                  "LINK_TYPE_ETHERNET_10G_LR"
                ],
                "--location": "dynamic",
                "--noc-contact-email": "value",
                "--requested-link-count": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-diagnostics": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--admin-enabled": "bool",
                "--description": "value",
                "--noc-contact-email": "value",
                "--requested-link-count": "value"
              }
            }
          },
          "flags": {}
        },
        "machine-images": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--csek-key-file": "value",
                "--description": "value",
                "--guest-flush": "bool",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--require-csek-key-create": "bool",
                "--source-disk-csek-key": "value",
                "--source-instance": "dynamic",
                "--source-instance-zone": "dynamic",
                "--storage-location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--byol": "bool",
                "--can-ip-forward": "bool",
                "--compute-service-account": "value",
                "--custom-cpu": "value",
                "--custom-extensions": "bool",
                "--custom-memory": "value",
                "--custom-vm-type": "value",
                "--description": "value",
                "--guest-environment": "bool",
                "--guest-flush": "bool",
                "--guest-os-features": "value",
                "--labels": "value",
                "--log-location": "value",
                "--machine-type": "dynamic",
                "--network": "value",
                "--network-tier": "value",
                "--no-address": "bool",
                "--no-scopes": "bool",
                "--no-service-account": "bool",
                "--os": [
                  "centos-7",
                  "debian-10",
                  "debian-11",
                  "debian-8",
                  "debian-9",
                  "opensuse-15",
                  "rhel-6",
                  "rhel-6-byol",
                  "rhel-7",
                  "rhel-7-byol",
                  "rhel-8",
                  "rhel-8-byol",
                  "rhel-9",
                  "rhel-9-byol",
                  "rocky-8",
                  "sles-12",
                  "sles-12-byol",
                  "sles-15",
                  "sles-15-byol",
                  "sles-sap-12",
                  "sles-sap-12-byol",
                  "sles-sap-15",
                  "sles-sap-15-byol",
                  "ubuntu-1404",
                  "ubuntu-1604",
                  "ubuntu-1804",
                  "ubuntu-2004",
                  "ubuntu-2204",
                  "windows-10-x64-byol",
                  "windows-10-x86-byol",
                  "windows-11-x64-byol",
                  "windows-2008r2",
                  "windows-2008r2-byol",
                  "windows-2012",
                  "windows-2012-byol",
                  "windows-2012r2",
                  "windows-2012r2-byol",
                  "windows-2016",
                  "windows-2016-byol",
                  "windows-2019",
                  "windows-2019-byol",
                  "windows-2022",
                  "windows-2022-byol",
                  "windows-7-x64-byol",
                  "windows-7-x86-byol",
                  "windows-8-x64-byol",
                  "windows-8-x86-byol"
                ],
                "--restart-on-failure": "bool",
                "--scopes": "value",
                "--service-account": "value",
                "--source-uri": "value",
                "--storage-location": "value",
                "--subnet": "value",
                "--tags": "value",
                "--timeout": "value",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "machine-types": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "network-attachments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--connection-preference": [
                  "ACCEPT_AUTOMATIC",
                  "ACCEPT_MANUAL"
                ],
                "--description": "value",
                "--producer-accept-list": "value",
                "--producer-reject-list": "value",
                "--region": "dynamic",
                "--subnets": "dynamic",
                "--subnets-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "network-edge-security-services": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--region": "dynamic",
                "--security-policy": "dynamic",
                "--security-policy-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--region": "dynamic",
                "--security-policy": "dynamic",
                "--security-policy-region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "network-endpoint-groups": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--app-engine-app": "bool",
                "--app-engine-service": "value",
                "--app-engine-url-mask": "value",
                "--app-engine-version": "value",
                "--cloud-function-name": "value",
                "--cloud-function-url-mask": "value",
                "--cloud-run-service": "value",
                "--cloud-run-tag": "value",
                "--cloud-run-url-mask": "value",
                "--default-port": "value",
                "--global": "bool",
                "--network": "value",
                "--network-endpoint-type": [
                  "gce-vm-ip",
                  "gce-vm-ip-port",
                  "internet-fqdn-port",
                  "internet-ip-port",
                  "non-gcp-private-ip-port",
                  "private-service-connect",
                  "serverless"
                ],
                "--psc-target-service": "value",
                "--region": "dynamic",
                "--subnet": "value",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "value"
              }
            },
            "list-network-endpoints": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--zone": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-endpoint": "value",
                "--global": "bool",
                "--remove-endpoint": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "network-firewall-policies": {
          "commands": {
            "associations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool",
                    "--name": "value",
                    "--network": "value",
                    "--replace-association-on-target": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool",
                    "--name": "value"
                  }
                }
              },
              "flags": {}
            },
            "clone-rules": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source-firewall-policy": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "get-effective-firewalls": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--network": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny",
                      "goto_next"
                    ],
                    "--description": "value",
                    "--dest-ip-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "INGRESS"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool",
                    "--layer4-configs": "value",
                    "--src-ip-ranges": "value",
                    "--src-secure-tags": "value",
                    "--target-secure-tags": "value",
                    "--target-service-accounts": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny",
                      "goto_next"
                    ],
                    "--description": "value",
                    "--dest-ip-ranges": "value",
                    "--direction": [
                      "EGRESS",
                      "INGRESS"
                    ],
                    "--disabled": "bool",
                    "--enable-logging": "bool",
                    "--firewall-policy": "value",
                    "--firewall-policy-region": "dynamic",
                    "--global-firewall-policy": "bool",
                    "--layer4-configs": "value",
                    "--new-priority": "value",
                    "--src-ip-ranges": "value",
                    "--src-secure-tags": "value",
                    "--target-secure-tags": "value",
                    "--target-service-accounts": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "networks": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--bgp-routing-mode": [
                  "global",
                  "regional"
                ],
                "--description": "value",
                "--enable-ula-internal-ipv6": "bool",
                "--internal-ipv6-range": "value",
                "--mtu": "value",
                "--network-firewall-policy-enforcement-order": [
                  "AFTER_CLASSIC_FIREWALL",
                  "BEFORE_CLASSIC_FIREWALL"
                ],
                "--range": "value",
                "--subnet-mode": [
                  "auto",
                  "custom",
                  "legacy"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-effective-firewalls": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "peerings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auto-create-routes": "bool",
                    "--export-custom-routes": "bool",
                    "--export-subnet-routes-with-public-ip": "bool",
                    "--import-custom-routes": "bool",
                    "--import-subnet-routes-with-public-ip": "bool",
                    "--network": "value",
                    "--peer-network": "value",
                    "--peer-project": "value",
                    "--stack-type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--network": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-routes": {
                  "commands": {},
                  "flags": {
                    "--direction": [
                      "INCOMING",
                      "OUTGOING"
                    ],
                    "--filter": "value",
                    "--limit": "value",
                    "--network": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--export-custom-routes": "bool",
                    "--export-subnet-routes-with-public-ip": "bool",
                    "--import-custom-routes": "bool",
                    "--import-subnet-routes-with-public-ip": "bool",
                    "--network": "value",
                    "--stack-type": "value"
                  }
                }
              },
              "flags": {}
            },
            "subnets": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--enable-flow-logs": "bool",
                    "--enable-private-ip-google-access": "bool",
                    "--ipv6-access-type": [
                      "EXTERNAL",
                      "INTERNAL"
                    ],
                    "--logging-aggregation-interval": [
                      "interval-1-min",
                      "interval-10-min",
                      "interval-15-min",
                      "interval-30-sec",
                      "interval-5-min",
                      "interval-5-sec"
                    ],
                    "--logging-filter-expr": "value",
                    "--logging-flow-sampling": "value",
                    "--logging-metadata": [
                      "custom",
                      "exclude-all",
                      "include-all"
                    ],
                    "--logging-metadata-fields": "value",
                    "--network": "dynamic",
                    "--private-ipv6-google-access-type": [
                      "disable",
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access"
                    ],
                    "--purpose": [
                      "INTERNAL_HTTPS_LOAD_BALANCER",
                      "PRIVATE",
                      "PRIVATE_SERVICE_CONNECT",
                      "REGIONAL_MANAGED_PROXY"
                    ],
                    "--range": "value",
                    "--region": "dynamic",
                    "--role": [
                      "ACTIVE",
                      "BACKUP"
                    ],
                    "--secondary-range": "value",
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "expand-ip-range": {
                  "commands": {},
                  "flags": {
                    "--prefix-length": "value",
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--network": "value",
                    "--page-size": "value",
                    "--regexp": "value",
                    "--regions": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-usable": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-secondary-ranges": "value",
                    "--drain-timeout": "value",
                    "--enable-flow-logs": "bool",
                    "--enable-private-ip-google-access": "bool",
                    "--ipv6-access-type": [
                      "EXTERNAL",
                      "INTERNAL"
                    ],
                    "--logging-aggregation-interval": [
                      "interval-1-min",
                      "interval-10-min",
                      "interval-15-min",
                      "interval-30-sec",
                      "interval-5-min",
                      "interval-5-sec"
                    ],
                    "--logging-filter-expr": "value",
                    "--logging-flow-sampling": "value",
                    "--logging-metadata": [
                      "custom",
                      "exclude-all",
                      "include-all"
                    ],
                    "--logging-metadata-fields": "value",
                    "--private-ipv6-google-access-type": [
                      "disable",
                      "enable-bidirectional-access",
                      "enable-outbound-vm-access"
                    ],
                    "--purpose": [
                      "REGIONAL_MANAGED_PROXY"
                    ],
                    "--region": "dynamic",
                    "--remove-secondary-ranges": "value",
                    "--role": [
                      "ACTIVE"
                    ],
                    "--stack-type": [
                      "IPV4_IPV6",
                      "IPV4_ONLY"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--bgp-routing-mode": [
                  "global",
                  "regional"
                ],
                "--enable-ula-internal-ipv6": "bool",
                "--internal-ipv6-range": "value",
                "--mtu": "value",
                "--network-firewall-policy-enforcement-order": [
                  "AFTER_CLASSIC_FIREWALL",
                  "BEFORE_CLASSIC_FIREWALL"
                ],
                "--switch-to-custom-subnet-mode": "bool"
              }
            },
            "vpc-access": {
              "commands": {
                "connectors": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--machine-type": "value",
                        "--max-instances": "value",
                        "--max-throughput": "value",
                        "--min-instances": "value",
                        "--min-throughput": "value",
                        "--network": "value",
                        "--range": "value",
                        "--region": "dynamic",
                        "--subnet": "value",
                        "--subnet-project": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--region": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "locations": {
                  "commands": {
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--region": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--region": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "value"
              }
            }
          },
          "flags": {}
        },
        "os-config": {
          "commands": {
            "inventories": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "os-policy-assignment-reports": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--assignment-id": "value",
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "os-policy-assignments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-revisions": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "operations": {
                  "commands": {
                    "cancel": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--os-policy-assignment": "value"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--os-policy-assignment": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--file": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "patch-deployments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "pause": {
                  "commands": {},
                  "flags": {}
                },
                "resume": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--file": "value"
                  }
                }
              },
              "flags": {}
            },
            "patch-jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "execute": {
                  "commands": {},
                  "flags": {
                    "--apt-dist": "bool",
                    "--apt-excludes": "value",
                    "--apt-exclusive-packages": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--display-name": "value",
                    "--dry-run": "bool",
                    "--duration": "value",
                    "--instance-filter-all": "bool",
                    "--instance-filter-group-labels": "value",
                    "--instance-filter-name-prefixes": "value",
                    "--instance-filter-names": "value",
                    "--instance-filter-zones": "value",
                    "--mig-instances-allowed": "bool",
                    "--post-patch-linux-executable": "value",
                    "--post-patch-linux-success-codes": "value",
                    "--post-patch-windows-executable": "value",
                    "--post-patch-windows-success-codes": "value",
                    "--pre-patch-linux-executable": "value",
                    "--pre-patch-linux-success-codes": "value",
                    "--pre-patch-windows-executable": "value",
                    "--pre-patch-windows-success-codes": "value",
                    "--reboot-config": [
                      "always",
                      "default",
                      "never"
                    ],
                    "--rollout-disruption-budget": "value",
                    "--rollout-disruption-budget-percent": "value",
                    "--rollout-mode": [
                      "concurrent-zones",
                      "zone-by-zone"
                    ],
                    "--windows-classifications": "value",
                    "--windows-excludes": "value",
                    "--windows-exclusive-patches": "value",
                    "--yum-excludes": "value",
                    "--yum-exclusive-packages": "value",
                    "--yum-minimal": "bool",
                    "--yum-security": "bool",
                    "--zypper-categories": "value",
                    "--zypper-excludes": "value",
                    "--zypper-exclusive-patches": "value",
                    "--zypper-severities": "value",
                    "--zypper-with-optional": "bool",
                    "--zypper-with-update": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-instance-details": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "troubleshoot": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "vulnerability-reports": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "os-login": {
          "commands": {
            "describe-profile": {
              "commands": {},
              "flags": {}
            },
            "remove-profile": {
              "commands": {},
              "flags": {}
            },
            "ssh-keys": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value",
                    "--ttl": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "remove": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--key": "value",
                    "--key-file": "value",
                    "--ttl": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "packet-mirrorings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--collector-ilb": "value",
                "--description": "value",
                "--enable": "bool",
                "--filter-cidr-ranges": "value",
                "--filter-direction": [
                  "both",
                  "egress",
                  "ingress"
                ],
                "--filter-protocols": "value",
                "--mirrored-instances": "value",
                "--mirrored-subnets": "value",
                "--mirrored-tags": "value",
                "--network": "value",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-filter-cidr-ranges": "value",
                "--add-filter-protocols": "value",
                "--add-mirrored-instances": "value",
                "--add-mirrored-subnets": "value",
                "--add-mirrored-tags": "value",
                "--async": "bool",
                "--clear-filter-cidr-ranges": "bool",
                "--clear-filter-protocols": "bool",
                "--clear-mirrored-instances": "bool",
                "--clear-mirrored-subnets": "bool",
                "--clear-mirrored-tags": "bool",
                "--collector-ilb": "value",
                "--description": "value",
                "--enable": "bool",
                "--filter-direction": [
                  "both",
                  "egress",
                  "ingress"
                ],
                "--region": "dynamic",
                "--remove-filter-cidr-ranges": "value",
                "--remove-filter-protocols": "value",
                "--remove-mirrored-instances": "value",
                "--remove-mirrored-subnets": "value",
                "--remove-mirrored-tags": "value",
                "--set-filter-cidr-ranges": "value",
                "--set-filter-protocols": "value",
                "--set-mirrored-instances": "value",
                "--set-mirrored-subnets": "value",
                "--set-mirrored-tags": "value"
              }
            }
          },
          "flags": {}
        },
        "project-info": {
          "commands": {
            "add-metadata": {
              "commands": {},
              "flags": {
                "--metadata": "value",
                "--metadata-from-file": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "remove-metadata": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--keys": "value"
              }
            },
            "set-usage-bucket": {
              "commands": {},
              "flags": {
                "--bucket": "value",
                "--no-bucket": "bool",
                "--prefix": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--default-network-tier": [
                  "FIXED_STANDARD",
                  "PREMIUM",
                  "STANDARD"
                ]
              }
            }
          },
          "flags": {}
        },
        "public-advertised-prefixes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--dns-verification-ip": "value",
                "--range": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--status": [
                  "ptr-configured"
                ]
              }
            }
          },
          "flags": {}
        },
        "public-delegated-prefixes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--enable-live-migration": "bool",
                "--global": "bool",
                "--public-advertised-prefix": "value",
                "--range": "value",
                "--region": "dynamic"
              }
            },
            "delegated-sub-prefixes": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--create-addresses": "bool",
                    "--delegatee-project": "value",
                    "--description": "value",
                    "--global-public-delegated-prefix": "bool",
                    "--public-delegated-prefix": "value",
                    "--public-delegated-prefix-region": "dynamic",
                    "--range": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--global-public-delegated-prefix": "bool",
                    "--public-delegated-prefix": "value",
                    "--public-delegated-prefix-region": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "reservations": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--description": "value",
                "--local-ssd": "value",
                "--machine-type": "value",
                "--min-cpu-platform": "value",
                "--require-specific-reservation": "bool",
                "--resource-policies": "value",
                "--share-setting": [
                  "folders",
                  "local",
                  "projects"
                ],
                "--share-with": "value",
                "--source-instance-template": "value",
                "--vm-count": "value",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic",
                "--zone": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-share-with": "value",
                "--remove-share-with": "value",
                "--vm-count": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "reset-windows-password": {
          "commands": {},
          "flags": {
            "--user": "value",
            "--zone": "dynamic"
          }
        },
        "resource-policies": {
          "commands": {
            "create": {
              "commands": {
                "group-placement": {
                  "commands": {},
                  "flags": {
                    "--availability-domain-count": "value",
                    "--collocation": [
                      "collocated",
                      "unspecified-collocation"
                    ],
                    "--description": "value",
                    "--region": "dynamic",
                    "--vm-count": "value"
                  }
                },
                "instance-schedule": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--end-date": "value",
                    "--initiation-date": "value",
                    "--region": "dynamic",
                    "--timezone": "value",
                    "--vm-start-schedule": "value",
                    "--vm-stop-schedule": "value"
                  }
                },
                "snapshot-schedule": {
                  "commands": {},
                  "flags": {
                    "--daily-schedule": "bool",
                    "--description": "value",
                    "--guest-flush": "bool",
                    "--hourly-schedule": "value",
                    "--max-retention-days": "value",
                    "--on-source-disk-delete": [
                      "apply-retention-policy",
                      "keep-auto-snapshots"
                    ],
                    "--region": "dynamic",
                    "--snapshot-labels": "value",
                    "--start-time": "value",
                    "--storage-location": "value",
                    "--weekly-schedule": [
                      "friday",
                      "monday",
                      "saturday",
                      "sunday",
                      "thursday",
                      "tuesday",
                      "wednesday"
                    ],
                    "--weekly-schedule-from-file": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "routers": {
          "commands": {
            "add-bgp-peer": {
              "commands": {},
              "flags": {
                "--advertised-route-priority": "value",
                "--advertisement-mode": [
                  "CUSTOM",
                  "DEFAULT"
                ],
                "--async": "bool",
                "--bfd-min-receive-interval": "value",
                "--bfd-min-transmit-interval": "value",
                "--bfd-multiplier": "value",
                "--bfd-session-initialization-mode": [
                  "ACTIVE",
                  "DISABLED",
                  "PASSIVE"
                ],
                "--enable-ipv6": "bool",
                "--enabled": "bool",
                "--instance": "dynamic",
                "--instance-zone": "dynamic",
                "--interface": "value",
                "--ipv6-nexthop-address": "value",
                "--md5-authentication-key": "value",
                "--peer-asn": "value",
                "--peer-ip-address": "value",
                "--peer-ipv6-nexthop-address": "value",
                "--peer-name": "value",
                "--region": "dynamic",
                "--set-advertisement-groups": "value",
                "--set-advertisement-ranges": "value"
              }
            },
            "add-interface": {
              "commands": {},
              "flags": {
                "--interconnect-attachment": "dynamic",
                "--interconnect-attachment-region": "dynamic",
                "--interface-name": "value",
                "--ip-address": "value",
                "--mask-length": "value",
                "--redundant-interface": "value",
                "--region": "dynamic",
                "--subnetwork": "dynamic",
                "--subnetwork-region": "dynamic",
                "--vpn-tunnel": "dynamic",
                "--vpn-tunnel-region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--advertisement-mode": [
                  "CUSTOM",
                  "DEFAULT"
                ],
                "--asn": "value",
                "--async": "bool",
                "--description": "value",
                "--encrypted-interconnect-router": "bool",
                "--keepalive-interval": "value",
                "--network": "dynamic",
                "--region": "dynamic",
                "--set-advertisement-groups": "value",
                "--set-advertisement-ranges": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "get-nat-mapping-info": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value"
              }
            },
            "get-status": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "nats": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auto-allocate-nat-external-ips": "bool",
                    "--enable-dynamic-port-allocation": "bool",
                    "--enable-endpoint-independent-mapping": "bool",
                    "--enable-logging": "bool",
                    "--endpoint-types": "value",
                    "--icmp-idle-timeout": "value",
                    "--log-filter": [
                      "ALL",
                      "ERRORS_ONLY",
                      "TRANSLATIONS_ONLY"
                    ],
                    "--max-ports-per-vm": "value",
                    "--min-ports-per-vm": "value",
                    "--nat-all-subnet-ip-ranges": "bool",
                    "--nat-custom-subnet-ip-ranges": "value",
                    "--nat-external-ip-pool": "value",
                    "--nat-primary-subnet-ip-ranges": "bool",
                    "--region": "dynamic",
                    "--router": "dynamic",
                    "--rules": "value",
                    "--tcp-established-idle-timeout": "value",
                    "--tcp-time-wait-timeout": "value",
                    "--tcp-transitory-idle-timeout": "value",
                    "--udp-idle-timeout": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--router": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--router": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic",
                    "--router": "dynamic"
                  }
                },
                "rules": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--match": "value",
                        "--nat": "value",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--source-nat-active-ips": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--nat": "value",
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--nat": "value",
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--nat": "value",
                        "--region": "dynamic",
                        "--router": "dynamic"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-source-nat-drain-ips": "bool",
                        "--match": "value",
                        "--nat": "value",
                        "--region": "dynamic",
                        "--router": "dynamic",
                        "--source-nat-active-ips": "value",
                        "--source-nat-drain-ips": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--auto-allocate-nat-external-ips": "bool",
                    "--clear-icmp-idle-timeout": "bool",
                    "--clear-max-ports-per-vm": "bool",
                    "--clear-min-ports-per-vm": "bool",
                    "--clear-nat-external-drain-ip-pool": "bool",
                    "--clear-tcp-established-idle-timeout": "bool",
                    "--clear-tcp-time-wait-timeout": "bool",
                    "--clear-tcp-transitory-idle-timeout": "bool",
                    "--clear-udp-idle-timeout": "bool",
                    "--enable-dynamic-port-allocation": "bool",
                    "--enable-endpoint-independent-mapping": "bool",
                    "--enable-logging": "bool",
                    "--icmp-idle-timeout": "value",
                    "--log-filter": [
                      "ALL",
                      "ERRORS_ONLY",
                      "TRANSLATIONS_ONLY"
                    ],
                    "--max-ports-per-vm": "value",
                    "--min-ports-per-vm": "value",
                    "--nat-all-subnet-ip-ranges": "bool",
                    "--nat-custom-subnet-ip-ranges": "value",
                    "--nat-external-drain-ip-pool": "value",
                    "--nat-external-ip-pool": "value",
                    "--nat-primary-subnet-ip-ranges": "bool",
                    "--region": "dynamic",
                    "--router": "dynamic",
                    "--rules": "value",
                    "--tcp-established-idle-timeout": "value",
                    "--tcp-time-wait-timeout": "value",
                    "--tcp-transitory-idle-timeout": "value",
                    "--udp-idle-timeout": "value"
                  }
                }
              },
              "flags": {}
            },
            "remove-bgp-peer": {
              "commands": {},
              "flags": {
                "--peer-name": "value",
                "--peer-names": "value",
                "--region": "dynamic"
              }
            },
            "remove-interface": {
              "commands": {},
              "flags": {
                "--interface-name": "value",
                "--interface-names": "value",
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-advertisement-groups": "value",
                "--add-advertisement-ranges": "value",
                "--advertisement-mode": [
                  "CUSTOM",
                  "DEFAULT"
                ],
                "--asn": "value",
                "--async": "bool",
                "--keepalive-interval": "value",
                "--region": "dynamic",
                "--remove-advertisement-groups": "value",
                "--remove-advertisement-ranges": "value",
                "--set-advertisement-groups": "value",
                "--set-advertisement-ranges": "value"
              }
            },
            "update-bgp-peer": {
              "commands": {},
              "flags": {
                "--add-advertisement-groups": "value",
                "--add-advertisement-ranges": "value",
                "--advertised-route-priority": "value",
                "--advertisement-mode": [
                  "CUSTOM",
                  "DEFAULT"
                ],
                "--async": "bool",
                "--bfd-min-receive-interval": "value",
                "--bfd-min-transmit-interval": "value",
                "--bfd-multiplier": "value",
                "--bfd-session-initialization-mode": [
                  "ACTIVE",
                  "DISABLED",
                  "PASSIVE"
                ],
                "--clear-md5-authentication-key": "bool",
                "--enable-ipv6": "bool",
                "--enabled": "bool",
                "--interface": "value",
                "--ip-address": "value",
                "--ipv6-nexthop-address": "value",
                "--md5-authentication-key": "value",
                "--peer-asn": "value",
                "--peer-ip-address": "value",
                "--peer-ipv6-nexthop-address": "value",
                "--peer-name": "value",
                "--region": "dynamic",
                "--remove-advertisement-groups": "value",
                "--remove-advertisement-ranges": "value",
                "--set-advertisement-groups": "value",
                "--set-advertisement-ranges": "value"
              }
            },
            "update-interface": {
              "commands": {},
              "flags": {
                "--interconnect-attachment": "dynamic",
                "--interconnect-attachment-region": "dynamic",
                "--interface-name": "value",
                "--ip-address": "value",
                "--mask-length": "value",
                "--region": "dynamic",
                "--vpn-tunnel": "dynamic",
                "--vpn-tunnel-region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "routes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--destination-range": "value",
                "--network": "value",
                "--next-hop-address": "value",
                "--next-hop-gateway": "dynamic",
                "--next-hop-ilb": "value",
                "--next-hop-ilb-region": "value",
                "--next-hop-instance": "value",
                "--next-hop-instance-zone": "value",
                "--next-hop-vpn-tunnel": "value",
                "--next-hop-vpn-tunnel-region": "value",
                "--priority": "value",
                "--tags": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "scp": {
          "commands": {},
          "flags": {
            "--compress": "bool",
            "--dest-group": "value",
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--internal-ip": "bool",
            "--network": "value",
            "--plain": "bool",
            "--port": "value",
            "--recurse": "bool",
            "--region": "value",
            "--scp-flag": "value",
            "--ssh-key-expiration": "value",
            "--ssh-key-expire-after": "value",
            "--ssh-key-file": "value",
            "--strict-host-key-checking": [
              "ask",
              "no",
              "yes"
            ],
            "--tunnel-through-iap": "bool",
            "--zone": "value"
          }
        },
        "security-policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--file-format": [
                  "json",
                  "yaml"
                ],
                "--file-name": "value",
                "--global": "bool",
                "--region": "dynamic",
                "--type": [
                  "CLOUD_ARMOR",
                  "CLOUD_ARMOR_EDGE",
                  "CLOUD_ARMOR_NETWORK"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--file-format": [
                  "json",
                  "yaml"
                ],
                "--file-name": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--file-format": [
                  "json",
                  "yaml"
                ],
                "--file-name": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-preconfigured-expression-sets": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "rules": {
              "commands": {
                "add-preconfig-waf-exclusion": {
                  "commands": {},
                  "flags": {
                    "--request-cookie-to-exclude": "value",
                    "--request-header-to-exclude": "value",
                    "--request-query-param-to-exclude": "value",
                    "--request-uri-to-exclude": "value",
                    "--security-policy": "dynamic",
                    "--target-rule-ids": "value",
                    "--target-rule-set": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny-403",
                      "deny-404",
                      "deny-502",
                      "rate-based-ban",
                      "redirect",
                      "redirect-to-recaptcha",
                      "throttle"
                    ],
                    "--ban-duration-sec": "value",
                    "--ban-threshold-count": "value",
                    "--ban-threshold-interval-sec": "value",
                    "--conform-action": [
                      "allow"
                    ],
                    "--description": "value",
                    "--enforce-on-key": [
                      "all",
                      "http-cookie",
                      "http-header",
                      "http-path",
                      "ip",
                      "region-code",
                      "sni",
                      "xff-ip"
                    ],
                    "--enforce-on-key-name": "value",
                    "--exceed-action": [
                      "deny-403",
                      "deny-404",
                      "deny-429",
                      "deny-502",
                      "redirect"
                    ],
                    "--exceed-redirect-target": "value",
                    "--exceed-redirect-type": [
                      "external-302",
                      "google-recaptcha"
                    ],
                    "--expression": "value",
                    "--preview": "bool",
                    "--rate-limit-threshold-count": "value",
                    "--rate-limit-threshold-interval-sec": "value",
                    "--redirect-target": "value",
                    "--redirect-type": [
                      "external-302",
                      "google-recaptcha"
                    ],
                    "--request-headers-to-add": "value",
                    "--security-policy": "dynamic",
                    "--src-ip-ranges": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--security-policy": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--security-policy": "dynamic"
                  }
                },
                "remove-preconfig-waf-exclusion": {
                  "commands": {},
                  "flags": {
                    "--request-cookie-to-exclude": "value",
                    "--request-header-to-exclude": "value",
                    "--request-query-param-to-exclude": "value",
                    "--request-uri-to-exclude": "value",
                    "--security-policy": "dynamic",
                    "--target-rule-ids": "value",
                    "--target-rule-set": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--action": [
                      "allow",
                      "deny-403",
                      "deny-404",
                      "deny-502",
                      "rate-based-ban",
                      "redirect",
                      "redirect-to-recaptcha",
                      "throttle"
                    ],
                    "--ban-duration-sec": "value",
                    "--ban-threshold-count": "value",
                    "--ban-threshold-interval-sec": "value",
                    "--conform-action": [
                      "allow"
                    ],
                    "--description": "value",
                    "--enforce-on-key": [
                      "all",
                      "http-cookie",
                      "http-header",
                      "http-path",
                      "ip",
                      "region-code",
                      "sni",
                      "xff-ip"
                    ],
                    "--enforce-on-key-name": "value",
                    "--exceed-action": [
                      "deny-403",
                      "deny-404",
                      "deny-429",
                      "deny-502",
                      "redirect"
                    ],
                    "--exceed-redirect-target": "value",
                    "--exceed-redirect-type": [
                      "external-302",
                      "google-recaptcha"
                    ],
                    "--expression": "value",
                    "--preview": "bool",
                    "--rate-limit-threshold-count": "value",
                    "--rate-limit-threshold-interval-sec": "value",
                    "--redirect-target": "value",
                    "--redirect-type": [
                      "external-302",
                      "google-recaptcha"
                    ],
                    "--request-headers-to-add": "value",
                    "--security-policy": "dynamic",
                    "--src-ip-ranges": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--enable-layer7-ddos-defense": "bool",
                "--global": "bool",
                "--json-custom-content-types": "value",
                "--json-parsing": [
                  "DISABLED",
                  "STANDARD"
                ],
                "--layer7-ddos-defense-rule-visibility": [
                  "PREMIUM",
                  "STANDARD"
                ],
                "--log-level": [
                  "NORMAL",
                  "VERBOSE"
                ],
                "--network-ddos-protection": [
                  "ADVANCED",
                  "STANDARD"
                ],
                "--recaptcha-redirect-site-key": "value",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "service-attachments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--connection-preference": [
                  "ACCEPT_AUTOMATIC",
                  "ACCEPT_MANUAL"
                ],
                "--consumer-accept-list": "value",
                "--consumer-reject-list": "value",
                "--description": "value",
                "--domain-names": "value",
                "--enable-proxy-protocol": "bool",
                "--nat-subnets": "dynamic",
                "--nat-subnets-region": "dynamic",
                "--producer-forwarding-rule": "dynamic",
                "--producer-forwarding-rule-region": "dynamic",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--connection-preference": [
                  "ACCEPT_AUTOMATIC",
                  "ACCEPT_MANUAL"
                ],
                "--consumer-accept-list": "value",
                "--consumer-reject-list": "value",
                "--description": "value",
                "--enable-proxy-protocol": "bool",
                "--nat-subnets": "dynamic",
                "--nat-subnets-region": "dynamic",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "shared-vpc": {
          "commands": {
            "associated-projects": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {
                    "--host-project": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove": {
                  "commands": {},
                  "flags": {
                    "--host-project": "value"
                  }
                }
              },
              "flags": {}
            },
            "disable": {
              "commands": {},
              "flags": {}
            },
            "enable": {
              "commands": {},
              "flags": {}
            },
            "get-host-project": {
              "commands": {},
              "flags": {}
            },
            "list-associated-resources": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "organizations": {
              "commands": {
                "list-host-projects": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "sign-url": {
          "commands": {},
          "flags": {
            "--expires-in": "value",
            "--key-file": "value",
            "--key-name": "value",
            "--validate": "bool"
          }
        },
        "snapshots": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "add-labels": {
              "commands": {},
              "flags": {
                "--labels": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--chain-name": "value",
                "--csek-key-file": "value",
                "--description": "value",
                "--guest-flush": "bool",
                "--labels": "value",
                "--snapshot-type": [
                  "ARCHIVE",
                  "STANDARD"
                ],
                "--source-disk": "dynamic",
                "--source-disk-key-file": "value",
                "--source-disk-region": "dynamic",
                "--source-disk-zone": "dynamic",
                "--storage-location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "remove-labels": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--labels": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "sole-tenancy": {
          "commands": {
            "node-groups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--autoscaler-mode": [
                      "off",
                      "on",
                      "only-scale-out"
                    ],
                    "--description": "value",
                    "--maintenance-policy": [
                      "default",
                      "migrate-within-node-group",
                      "restart-in-place"
                    ],
                    "--maintenance-window-start-time": "value",
                    "--max-nodes": "value",
                    "--min-nodes": "value",
                    "--node-template": "value",
                    "--share-setting": [
                      "local",
                      "organization",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--target-size": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--share-settings": "bool",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list-nodes": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--role": "dynamic",
                    "--zone": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-nodes": "value",
                    "--autoscaler-mode": [
                      "off",
                      "on",
                      "only-scale-out"
                    ],
                    "--delete-nodes": "value",
                    "--max-nodes": "value",
                    "--min-nodes": "value",
                    "--node-template": "value",
                    "--share-setting": [
                      "local",
                      "organization",
                      "projects"
                    ],
                    "--share-with": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "node-templates": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator": "value",
                    "--cpu-overcommit-type": [
                      "enabled",
                      "none"
                    ],
                    "--description": "value",
                    "--disk": "value",
                    "--node-affinity-labels": "value",
                    "--node-requirements": "value",
                    "--node-type": "value",
                    "--region": "dynamic",
                    "--server-binding": [
                      "restart-node-on-any-server",
                      "restart-node-on-minimal-servers"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "dynamic"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--member": "value",
                    "--region": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            },
            "node-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ssh": {
          "commands": {},
          "flags": {
            "--command": "value",
            "--container": "value",
            "--dest-group": "value",
            "--dry-run": "bool",
            "--force-key-file-overwrite": "bool",
            "--internal-ip": "bool",
            "--network": "value",
            "--plain": "bool",
            "--region": "value",
            "--ssh-flag": "value",
            "--ssh-key-expiration": "value",
            "--ssh-key-expire-after": "value",
            "--ssh-key-file": "value",
            "--strict-host-key-checking": [
              "ask",
              "no",
              "yes"
            ],
            "--troubleshoot": "bool",
            "--tunnel-through-iap": "bool",
            "--zone": "dynamic"
          }
        },
        "ssl-certificates": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--certificate": "value",
                "--description": "value",
                "--domains": "value",
                "--global": "bool",
                "--private-key": "value",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "ssl-policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--custom-features": "value",
                "--description": "value",
                "--global": "bool",
                "--min-tls-version": [
                  "1.0",
                  "1.1",
                  "1.2"
                ],
                "--profile": [
                  "COMPATIBLE",
                  "CUSTOM",
                  "MODERN",
                  "RESTRICTED"
                ],
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-available-features": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--custom-features": "value",
                "--global": "bool",
                "--min-tls-version": [
                  "1.0",
                  "1.1",
                  "1.2"
                ],
                "--profile": [
                  "COMPATIBLE",
                  "CUSTOM",
                  "MODERN",
                  "RESTRICTED"
                ],
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "start-iap-tunnel": {
          "commands": {},
          "flags": {
            "--dest-group": "value",
            "--iap-tunnel-disable-connection-check": "bool",
            "--local-host-port": "value",
            "--network": "value",
            "--region": "value",
            "--zone": "dynamic"
          }
        },
        "target-grpc-proxies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--global-url-map": "bool",
                "--url-map": "dynamic",
                "--url-map-region": "dynamic",
                "--validate-for-proxyless": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "target-http-proxies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--global": "bool",
                "--global-url-map": "bool",
                "--region": "dynamic",
                "--url-map": "dynamic",
                "--url-map-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--global-url-map": "bool",
                "--region": "dynamic",
                "--url-map": "dynamic",
                "--url-map-region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "target-https-proxies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--certificate-map": "dynamic",
                "--description": "value",
                "--global": "bool",
                "--global-ssl-certificates": "bool",
                "--global-ssl-policy": "bool",
                "--global-url-map": "bool",
                "--quic-override": [
                  "DISABLE",
                  "ENABLE",
                  "NONE"
                ],
                "--region": "dynamic",
                "--ssl-certificates": "dynamic",
                "--ssl-certificates-region": "dynamic",
                "--ssl-policy": "dynamic",
                "--ssl-policy-region": "dynamic",
                "--url-map": "dynamic",
                "--url-map-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--certificate-map": "dynamic",
                "--clear-certificate-map": "bool",
                "--clear-ssl-policy": "bool",
                "--global": "bool",
                "--global-ssl-certificates": "bool",
                "--global-ssl-policy": "bool",
                "--global-url-map": "bool",
                "--quic-override": [
                  "DISABLE",
                  "ENABLE",
                  "NONE"
                ],
                "--region": "dynamic",
                "--ssl-certificates": "dynamic",
                "--ssl-certificates-region": "dynamic",
                "--ssl-policy": "dynamic",
                "--ssl-policy-region": "dynamic",
                "--url-map": "dynamic",
                "--url-map-region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "target-instances": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--instance": "dynamic",
                "--instance-zone": "dynamic",
                "--network": "value",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zones": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "target-pools": {
          "commands": {
            "add-health-checks": {
              "commands": {},
              "flags": {
                "--http-health-check": "dynamic",
                "--region": "dynamic"
              }
            },
            "add-instances": {
              "commands": {},
              "flags": {
                "--instances": "dynamic",
                "--instances-zone": "dynamic",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--backup-pool": "value",
                "--description": "value",
                "--failover-ratio": "value",
                "--health-check": "value",
                "--http-health-check": "dynamic",
                "--region": "dynamic",
                "--session-affinity": [
                  "CLIENT_IP",
                  "CLIENT_IP_PROTO",
                  "NONE"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "get-health": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-health-checks": {
              "commands": {},
              "flags": {
                "--http-health-check": "dynamic",
                "--region": "dynamic"
              }
            },
            "remove-instances": {
              "commands": {},
              "flags": {
                "--instances": "dynamic",
                "--instances-zone": "dynamic",
                "--region": "dynamic",
                "--zone": "dynamic"
              }
            },
            "set-backup": {
              "commands": {},
              "flags": {
                "--backup-pool": "dynamic",
                "--failover-ratio": "value",
                "--no-backup-pool": "bool",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "target-ssl-proxies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--backend-service": "dynamic",
                "--certificate-map": "dynamic",
                "--description": "value",
                "--global-ssl-policy": "bool",
                "--proxy-header": [
                  "NONE",
                  "PROXY_V1"
                ],
                "--ssl-certificates": "dynamic",
                "--ssl-policy": "dynamic",
                "--ssl-policy-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--backend-service": "dynamic",
                "--certificate-map": "dynamic",
                "--clear-certificate-map": "bool",
                "--clear-ssl-certificates": "bool",
                "--clear-ssl-policy": "bool",
                "--global-ssl-policy": "bool",
                "--proxy-header": [
                  "NONE",
                  "PROXY_V1"
                ],
                "--ssl-certificates": "dynamic",
                "--ssl-policy": "dynamic",
                "--ssl-policy-region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "target-tcp-proxies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--backend-service": "dynamic",
                "--backend-service-region": "dynamic",
                "--description": "value",
                "--global": "bool",
                "--global-backend-service": "bool",
                "--proxy-bind": "bool",
                "--proxy-header": [
                  "NONE",
                  "PROXY_V1"
                ],
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--backend-service": "dynamic",
                "--proxy-header": [
                  "NONE",
                  "PROXY_V1"
                ]
              }
            }
          },
          "flags": {}
        },
        "target-vpn-gateways": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--network": "dynamic",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "tpus": {
          "commands": {
            "accelerator-types": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator-type": "value",
                "--async": "bool",
                "--description": "value",
                "--network": "value",
                "--preemptible": "bool",
                "--range": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "execution-groups": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-type": "value",
                    "--disk-size": "value",
                    "--dry-run": "bool",
                    "--forward-ports": "bool",
                    "--gce-image": "value",
                    "--machine-type": "value",
                    "--name": "value",
                    "--network": "value",
                    "--preemptible": "bool",
                    "--preemptible-vm": "bool",
                    "--tf-version": "value",
                    "--tpu-only": "bool",
                    "--use-dl-images": "bool",
                    "--use-with-notebook": "bool",
                    "--vm-only": "bool",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--tpu-only": "bool",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "dynamic"
              }
            },
            "locations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "reimage": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "value"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "value"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--zone": "value"
              }
            },
            "tpu-vm": {
              "commands": {
                "accelerator-types": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--accelerator-type": "value",
                    "--async": "bool",
                    "--data-disk": "value",
                    "--description": "value",
                    "--internal-ips": "bool",
                    "--labels": "value",
                    "--metadata": "value",
                    "--metadata-from-file": "value",
                    "--network": "value",
                    "--preemptible": "bool",
                    "--range": "value",
                    "--reserved": "bool",
                    "--scopes": "value",
                    "--service-account": "value",
                    "--shielded-secure-boot": "bool",
                    "--subnetwork": "value",
                    "--tags": "value",
                    "--zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "get-guest-attributes": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--query-path": "value",
                    "--sort-by": "value",
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "scp": {
                  "commands": {},
                  "flags": {
                    "--compress": "bool",
                    "--dry-run": "bool",
                    "--force-key-file-overwrite": "bool",
                    "--internal-ip": "bool",
                    "--plain": "bool",
                    "--recurse": "bool",
                    "--scp-flag": "value",
                    "--ssh-key-expiration": "value",
                    "--ssh-key-expire-after": "value",
                    "--ssh-key-file": "value",
                    "--strict-host-key-checking": [
                      "ask",
                      "no",
                      "yes"
                    ],
                    "--worker": "value",
                    "--zone": "dynamic"
                  }
                },
                "service-identity": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "ssh": {
                  "commands": {},
                  "flags": {
                    "--command": "value",
                    "--dry-run": "bool",
                    "--force-key-file-overwrite": "bool",
                    "--internal-ip": "bool",
                    "--output-directory": "value",
                    "--plain": "bool",
                    "--ssh-flag": "value",
                    "--ssh-key-expiration": "value",
                    "--ssh-key-expire-after": "value",
                    "--ssh-key-file": "value",
                    "--strict-host-key-checking": [
                      "ask",
                      "no",
                      "yes"
                    ],
                    "--worker": "value",
                    "--zone": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "stop": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--zone": "value"
                  }
                },
                "versions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--zone": "value"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "versions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "url-maps": {
          "commands": {
            "add-host-rule": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--global": "bool",
                "--hosts": "value",
                "--path-matcher-name": "value",
                "--region": "dynamic"
              }
            },
            "add-path-matcher": {
              "commands": {},
              "flags": {
                "--backend-bucket-path-rules": "value",
                "--backend-service-path-rules": "value",
                "--default-backend-bucket": "value",
                "--default-service": "value",
                "--delete-orphaned-path-matcher": "bool",
                "--description": "value",
                "--existing-host": "value",
                "--global": "bool",
                "--new-hosts": "value",
                "--path-matcher-name": "value",
                "--path-rules": "value",
                "--region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--default-backend-bucket": "value",
                "--default-service": "value",
                "--description": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "edit": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--region": "dynamic",
                "--source": "value"
              }
            },
            "invalidate-cdn-cache": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--global": "bool",
                "--host": "value",
                "--path": "value",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--global": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-cdn-cache-invalidations": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--limit": "value",
                "--region": "dynamic"
              }
            },
            "remove-host-rule": {
              "commands": {},
              "flags": {
                "--delete-orphaned-path-matcher": "bool",
                "--global": "bool",
                "--host": "value",
                "--region": "dynamic"
              }
            },
            "remove-path-matcher": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--path-matcher-name": "value",
                "--region": "dynamic"
              }
            },
            "set-default-service": {
              "commands": {},
              "flags": {
                "--default-backend-bucket": "value",
                "--default-service": "value",
                "--global": "bool",
                "--region": "dynamic"
              }
            },
            "validate": {
              "commands": {},
              "flags": {
                "--global": "bool",
                "--load-balancing-scheme": [
                  "EXTERNAL",
                  "EXTERNAL_MANAGED"
                ],
                "--region": "value",
                "--source": "value"
              }
            }
          },
          "flags": {}
        },
        "vpn-gateways": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--interconnect-attachments": "value",
                "--network": "dynamic",
                "--region": "dynamic",
                "--stack-type": [
                  "IPV4_IPV6",
                  "IPV4_ONLY"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "get-status": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "vpn-tunnels": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--ike-version": [
                  1,
                  2
                ],
                "--interface": [
                  0,
                  1
                ],
                "--local-traffic-selector": "value",
                "--peer-address": "value",
                "--peer-external-gateway": "dynamic",
                "--peer-external-gateway-interface": [
                  0,
                  1,
                  2,
                  3
                ],
                "--peer-gcp-gateway": "dynamic",
                "--peer-gcp-gateway-region": "dynamic",
                "--region": "dynamic",
                "--remote-traffic-selector": "value",
                "--router": "dynamic",
                "--router-region": "dynamic",
                "--shared-secret": "value",
                "--target-vpn-gateway": "dynamic",
                "--target-vpn-gateway-region": "dynamic",
                "--vpn-gateway": "dynamic",
                "--vpn-gateway-region": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--regions": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "zones": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--regexp": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "config": {
      "commands": {
        "configurations": {
          "commands": {
            "activate": {
              "commands": {},
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--activate": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {
                "--all": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            },
            "rename": {
              "commands": {},
              "flags": {
                "--new-name": "value"
              }
            }
          },
          "flags": {}
        },
        "get": {
          "commands": {},
          "flags": {}
        },
        "list": {
          "commands": {},
          "flags": {
            "--all": "bool",
            "--filter": "value",
            "--limit": "value",
            "--sort-by": "value"
          }
        },
        "set": {
          "commands": {},
          "flags": {
            "--installation": "bool"
          }
        },
        "unset": {
          "commands": {},
          "flags": {
            "--installation": "bool"
          }
        }
      },
      "flags": {}
    },
    "container": {
      "commands": {
        "attached": {
          "commands": {
            "clusters": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--ignore-errors": "bool",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "generate-install-manifest": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--output-file": "value",
                    "--platform-version": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--context": "value",
                    "--distribution": "value",
                    "--fleet-membership": "value",
                    "--fleet-membership-location": "value",
                    "--kubeconfig": "value",
                    "--location": "value",
                    "--platform-version": "value",
                    "--validate-only": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--context": "value",
                    "--description": "value",
                    "--distribution": "value",
                    "--enable-managed-prometheus": "bool",
                    "--fleet-project": "value",
                    "--has-private-issuer": "bool",
                    "--issuer-url": "value",
                    "--kubeconfig": "value",
                    "--location": "value",
                    "--logging": "value",
                    "--platform-version": "value",
                    "--validate-only": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--async": "bool",
                    "--clear-description": "bool",
                    "--description": "value",
                    "--disable-managed-prometheus": "bool",
                    "--enable-managed-prometheus": "bool",
                    "--location": "value",
                    "--logging": "value",
                    "--platform-version": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "get-server-config": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "aws": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--async": "bool",
                    "--aws-region": "value",
                    "--cluster-version": "value",
                    "--config-encryption-kms-key-arn": "value",
                    "--database-encryption-kms-key-arn": "value",
                    "--description": "value",
                    "--fleet-project": "value",
                    "--iam-instance-profile": "value",
                    "--instance-type": "value",
                    "--location": "value",
                    "--logging": "value",
                    "--main-volume-iops": "value",
                    "--main-volume-kms-key-arn": "value",
                    "--main-volume-size": "value",
                    "--main-volume-type": [
                      "gp2",
                      "gp3"
                    ],
                    "--pod-address-cidr-blocks": "value",
                    "--proxy-secret-arn": "value",
                    "--proxy-secret-version-id": "value",
                    "--role-arn": "value",
                    "--role-session-name": "value",
                    "--root-volume-iops": "value",
                    "--root-volume-kms-key-arn": "value",
                    "--root-volume-size": "value",
                    "--root-volume-type": [
                      "gp2",
                      "gp3"
                    ],
                    "--security-group-ids": "value",
                    "--service-address-cidr-blocks": "value",
                    "--ssh-ec2-key-pair": "value",
                    "--subnet-ids": "value",
                    "--tags": "value",
                    "--validate-only": "bool",
                    "--vpc-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--private-endpoint": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--async": "bool",
                    "--clear-annotations": "bool",
                    "--clear-description": "bool",
                    "--clear-proxy-config": "bool",
                    "--clear-security-group-ids": "bool",
                    "--clear-ssh-ec2-key-pair": "bool",
                    "--clear-tags": "bool",
                    "--cluster-version": "value",
                    "--config-encryption-kms-key-arn": "value",
                    "--description": "value",
                    "--iam-instance-profile": "value",
                    "--instance-type": "value",
                    "--location": "value",
                    "--logging": "value",
                    "--proxy-secret-arn": "value",
                    "--proxy-secret-version-id": "value",
                    "--role-arn": "value",
                    "--role-session-name": "value",
                    "--root-volume-iops": "value",
                    "--root-volume-kms-key-arn": "value",
                    "--root-volume-size": "value",
                    "--root-volume-type": [
                      "gp2",
                      "gp3"
                    ],
                    "--security-group-ids": "value",
                    "--ssh-ec2-key-pair": "value",
                    "--tags": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "get-server-config": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "node-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--async": "bool",
                    "--autoscaling-metrics": "value",
                    "--autoscaling-metrics-granularity": "value",
                    "--cluster": "value",
                    "--config-encryption-kms-key-arn": "value",
                    "--iam-instance-profile": "value",
                    "--instance-type": "value",
                    "--location": "value",
                    "--max-nodes": "value",
                    "--max-pods-per-node": "value",
                    "--min-nodes": "value",
                    "--node-labels": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--proxy-secret-arn": "value",
                    "--proxy-secret-version-id": "value",
                    "--root-volume-iops": "value",
                    "--root-volume-kms-key-arn": "value",
                    "--root-volume-size": "value",
                    "--root-volume-type": [
                      "gp2",
                      "gp3"
                    ],
                    "--security-group-ids": "value",
                    "--ssh-ec2-key-pair": "value",
                    "--subnet-id": "value",
                    "--tags": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--cluster": "value",
                    "--location": "value",
                    "--validate-only": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--async": "bool",
                    "--autoscaling-metrics": "value",
                    "--autoscaling-metrics-granularity": "value",
                    "--clear-annotations": "bool",
                    "--clear-autoscaling-metrics": "bool",
                    "--clear-proxy-config": "bool",
                    "--clear-security-group-ids": "bool",
                    "--clear-ssh-ec2-key-pair": "bool",
                    "--clear-tags": "bool",
                    "--cluster": "value",
                    "--config-encryption-kms-key-arn": "value",
                    "--iam-instance-profile": "value",
                    "--location": "value",
                    "--max-nodes": "value",
                    "--min-nodes": "value",
                    "--node-version": "value",
                    "--proxy-secret-arn": "value",
                    "--proxy-secret-version-id": "value",
                    "--root-volume-iops": "value",
                    "--root-volume-kms-key-arn": "value",
                    "--root-volume-size": "value",
                    "--root-volume-type": [
                      "gp2",
                      "gp3"
                    ],
                    "--security-group-ids": "value",
                    "--ssh-ec2-key-pair": "value",
                    "--tags": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "azure": {
          "commands": {
            "clients": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--application-id": "value",
                    "--async": "bool",
                    "--location": "value",
                    "--tenant-id": "value",
                    "--validate-only": "bool"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-public-cert": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--output-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--async": "bool",
                    "--azure-application-id": "value",
                    "--azure-region": "value",
                    "--azure-tenant-id": "value",
                    "--client": "value",
                    "--cluster-version": "value",
                    "--config-encryption-key-id": "value",
                    "--config-encryption-public-key": "value",
                    "--database-encryption-key-id": "value",
                    "--description": "value",
                    "--endpoint-subnet-id": "value",
                    "--fleet-project": "value",
                    "--location": "value",
                    "--logging": "value",
                    "--main-volume-size": "value",
                    "--pod-address-cidr-blocks": "value",
                    "--proxy-resource-group-id": "value",
                    "--proxy-secret-id": "value",
                    "--replica-placements": "value",
                    "--resource-group-id": "value",
                    "--root-volume-size": "value",
                    "--service-address-cidr-blocks": "value",
                    "--service-load-balancer-subnet-id": "value",
                    "--ssh-public-key": "value",
                    "--subnet-id": "value",
                    "--tags": "value",
                    "--validate-only": "bool",
                    "--vm-size": "value",
                    "--vnet-id": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--private-endpoint": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--annotations": "value",
                    "--async": "bool",
                    "--azure-application-id": "value",
                    "--azure-tenant-id": "value",
                    "--clear-annotations": "bool",
                    "--clear-client": "bool",
                    "--clear-description": "bool",
                    "--client": "value",
                    "--cluster-version": "value",
                    "--description": "value",
                    "--location": "value",
                    "--logging": "value",
                    "--ssh-public-key": "value",
                    "--validate-only": "bool",
                    "--vm-size": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-server-config": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "node-pools": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--async": "bool",
                    "--azure-availability-zone": "value",
                    "--cluster": "value",
                    "--config-encryption-key-id": "value",
                    "--config-encryption-public-key": "value",
                    "--location": "value",
                    "--max-nodes": "value",
                    "--max-pods-per-node": "value",
                    "--min-nodes": "value",
                    "--node-labels": "value",
                    "--node-taints": "value",
                    "--node-version": "value",
                    "--proxy-resource-group-id": "value",
                    "--proxy-secret-id": "value",
                    "--root-volume-size": "value",
                    "--ssh-public-key": "value",
                    "--subnet-id": "value",
                    "--tags": "value",
                    "--validate-only": "bool",
                    "--vm-size": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--allow-missing": "bool",
                    "--async": "bool",
                    "--cluster": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--cluster": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--annotations": "value",
                    "--async": "bool",
                    "--clear-annotations": "bool",
                    "--cluster": "value",
                    "--location": "value",
                    "--max-nodes": "value",
                    "--min-nodes": "value",
                    "--node-version": "value",
                    "--ssh-public-key": "value",
                    "--validate-only": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "binauthz": {
          "commands": {
            "attestations": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--artifact-url": "value",
                    "--attestor": "value",
                    "--attestor-project": "value",
                    "--payload-file": "value",
                    "--public-key-id": "value",
                    "--signature-file": "value",
                    "--validate": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--artifact-url": "value",
                    "--attestor": "value",
                    "--attestor-project": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "attestors": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--attestation-authority-note": "value",
                    "--attestation-authority-note-project": "value",
                    "--description": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "public-keys": {
                  "commands": {
                    "add": {
                      "commands": {},
                      "flags": {
                        "--attestor": "value",
                        "--comment": "value",
                        "--keyversion": "value",
                        "--keyversion-key": "value",
                        "--keyversion-keyring": "value",
                        "--keyversion-location": "value",
                        "--keyversion-project": "value",
                        "--pgp-public-key-file": "value",
                        "--pkix-public-key-algorithm": [
                          "ec-sign-p256-sha256",
                          "ec-sign-p384-sha384",
                          "ec-sign-p521-sha512",
                          "ecdsa-p256-sha256",
                          "ecdsa-p384-sha384",
                          "ecdsa-p521-sha512",
                          "rsa-pss-2048-sha256",
                          "rsa-pss-3072-sha256",
                          "rsa-pss-4096-sha256",
                          "rsa-pss-4096-sha512",
                          "rsa-sign-pkcs1-2048-sha256",
                          "rsa-sign-pkcs1-3072-sha256",
                          "rsa-sign-pkcs1-4096-sha256",
                          "rsa-sign-pkcs1-4096-sha512",
                          "rsa-sign-pss-2048-sha256",
                          "rsa-sign-pss-3072-sha256",
                          "rsa-sign-pss-4096-sha256",
                          "rsa-sign-pss-4096-sha512"
                        ],
                        "--pkix-public-key-file": "value",
                        "--public-key-id-override": "value"
                      }
                    },
                    "remove": {
                      "commands": {},
                      "flags": {
                        "--attestor": "value"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--attestor": "value",
                        "--comment": "value",
                        "--pgp-public-key-file": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--description": "value"
                  }
                }
              },
              "flags": {}
            },
            "create-signature-payload": {
              "commands": {},
              "flags": {
                "--artifact-url": "value"
              }
            },
            "policy": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "export": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--strict-validation": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "dynamic"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "clusters": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--additional-zones": "value",
                "--addons": "value",
                "--async": "bool",
                "--autoprovisioning-config-file": "value",
                "--autoprovisioning-image-type": "value",
                "--autoprovisioning-locations": "value",
                "--autoprovisioning-max-surge-upgrade": "value",
                "--autoprovisioning-max-unavailable-upgrade": "value",
                "--autoprovisioning-min-cpu-platform": "value",
                "--autoprovisioning-network-tags": "value",
                "--autoprovisioning-node-pool-soak-duration": "value",
                "--autoprovisioning-scopes": "value",
                "--autoprovisioning-service-account": "value",
                "--autoprovisioning-standard-rollout-policy": "value",
                "--autoscaling-profile": "value",
                "--binauthz-evaluation-mode": [
                  "DISABLED",
                  "PROJECT_SINGLETON_POLICY_ENFORCE"
                ],
                "--boot-disk-kms-key": "value",
                "--cloud-run-config": "value",
                "--cluster-dns": [
                  "clouddns",
                  "default"
                ],
                "--cluster-dns-domain": "value",
                "--cluster-dns-scope": [
                  "cluster",
                  "vpc"
                ],
                "--cluster-ipv4-cidr": "value",
                "--cluster-secondary-range-name": "value",
                "--cluster-version": "value",
                "--create-subnetwork": "value",
                "--database-encryption-key": "value",
                "--default-max-pods-per-node": "value",
                "--disable-default-snat": "bool",
                "--disk-size": "value",
                "--disk-type": [
                  "pd-balanced",
                  "pd-ssd",
                  "pd-standard"
                ],
                "--enable-autoprovisioning": "bool",
                "--enable-autoprovisioning-autorepair": "bool",
                "--enable-autoprovisioning-autoupgrade": "bool",
                "--enable-autoprovisioning-blue-green-upgrade": "bool",
                "--enable-autoprovisioning-surge-upgrade": "bool",
                "--enable-autorepair": "bool",
                "--enable-autoscaling": "bool",
                "--enable-autoupgrade": "bool",
                "--enable-basic-auth": "bool",
                "--enable-binauthz": "bool",
                "--enable-cloud-logging": "bool",
                "--enable-cloud-monitoring": "bool",
                "--enable-cloud-run-alpha": "bool",
                "--enable-confidential-nodes": "bool",
                "--enable-cost-allocation": "bool",
                "--enable-dataplane-v2": "bool",
                "--enable-google-cloud-access": "bool",
                "--enable-gvnic": "bool",
                "--enable-identity-service": "bool",
                "--enable-image-streaming": "bool",
                "--enable-intra-node-visibility": "bool",
                "--enable-ip-alias": "bool",
                "--enable-kubernetes-alpha": "bool",
                "--enable-l4-ilb-subsetting": "bool",
                "--enable-legacy-authorization": "bool",
                "--enable-managed-prometheus": "bool",
                "--enable-master-authorized-networks": "bool",
                "--enable-master-global-access": "bool",
                "--enable-network-egress-metering": "bool",
                "--enable-network-policy": "bool",
                "--enable-private-endpoint": "bool",
                "--enable-private-nodes": "bool",
                "--enable-resource-consumption-metering": "bool",
                "--enable-service-externalips": "bool",
                "--enable-shielded-nodes": "bool",
                "--enable-stackdriver-kubernetes": "bool",
                "--enable-tpu": "bool",
                "--enable-vertical-pod-autoscaling": "bool",
                "--ephemeral-storage-local-ssd": "value",
                "--gateway-api": [
                  "disabled",
                  "standard"
                ],
                "--image-type": "value",
                "--ipv6-access-type": [
                  "external",
                  "internal"
                ],
                "--issue-client-certificate": "bool",
                "--labels": "value",
                "--local-nvme-ssd-block": "value",
                "--local-ssd-count": "value",
                "--location-policy": [
                  "ANY",
                  "BALANCED"
                ],
                "--logging": "value",
                "--logging-variant": [
                  "DEFAULT",
                  "MAX_THROUGHPUT"
                ],
                "--machine-type": "value",
                "--maintenance-window": "value",
                "--maintenance-window-end": "value",
                "--maintenance-window-recurrence": "value",
                "--maintenance-window-start": "value",
                "--master-authorized-networks": "value",
                "--master-ipv4-cidr": "value",
                "--max-accelerator": "value",
                "--max-cpu": "value",
                "--max-memory": "value",
                "--max-nodes": "value",
                "--max-nodes-per-pool": "value",
                "--max-pods-per-node": "value",
                "--max-surge-upgrade": "value",
                "--max-unavailable-upgrade": "value",
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-accelerator": "value",
                "--min-cpu": "value",
                "--min-cpu-platform": "value",
                "--min-memory": "value",
                "--min-nodes": "value",
                "--monitoring": "value",
                "--network": "value",
                "--node-labels": "value",
                "--node-locations": "value",
                "--node-taints": "value",
                "--node-version": "value",
                "--notification-config": "value",
                "--num-nodes": "value",
                "--password": "value",
                "--placement-type": [
                  "COMPACT",
                  "UNSPECIFIED"
                ],
                "--preemptible": "bool",
                "--private-endpoint-subnetwork": "value",
                "--private-ipv6-google-access-type": [
                  "bidirectional",
                  "disabled",
                  "outbound-only"
                ],
                "--region": "value",
                "--release-channel": [
                  "None",
                  "rapid",
                  "regular",
                  "stable"
                ],
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--resource-usage-bigquery-dataset": "value",
                "--scopes": "value",
                "--security-group": "value",
                "--service-account": "value",
                "--services-ipv4-cidr": "value",
                "--services-secondary-range-name": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--spot": "bool",
                "--stack-type": [
                  "ipv4",
                  "ipv4-ipv6"
                ],
                "--subnetwork": "value",
                "--system-config-from-file": "value",
                "--tags": "value",
                "--threads-per-core": "value",
                "--total-max-nodes": "value",
                "--total-min-nodes": "value",
                "--tpu-ipv4-cidr": "value",
                "--username": "value",
                "--workload-metadata": [
                  "GCE_METADATA",
                  "GKE_METADATA"
                ],
                "--workload-pool": "value",
                "--zone": "value"
              }
            },
            "create-auto": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--autoprovisioning-network-tags": "value",
                "--binauthz-evaluation-mode": [
                  "DISABLED",
                  "PROJECT_SINGLETON_POLICY_ENFORCE"
                ],
                "--boot-disk-kms-key": "value",
                "--cluster-ipv4-cidr": "value",
                "--cluster-secondary-range-name": "value",
                "--cluster-version": "value",
                "--create-subnetwork": "value",
                "--database-encryption-key": "value",
                "--enable-google-cloud-access": "bool",
                "--enable-master-authorized-networks": "bool",
                "--enable-private-endpoint": "bool",
                "--enable-private-nodes": "bool",
                "--master-authorized-networks": "value",
                "--master-ipv4-cidr": "value",
                "--network": "value",
                "--private-endpoint-subnetwork": "value",
                "--region": "value",
                "--release-channel": [
                  "None",
                  "rapid",
                  "regular",
                  "stable"
                ],
                "--scopes": "value",
                "--security-group": "value",
                "--service-account": "value",
                "--services-ipv4-cidr": "value",
                "--services-secondary-range-name": "value",
                "--subnetwork": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "get-credentials": {
              "commands": {},
              "flags": {
                "--internal-ip": "bool",
                "--region": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "value"
              }
            },
            "resize": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--node-pool": "value",
                "--num-nodes": "value",
                "--region": "value",
                "--size": "value",
                "--zone": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-maintenance-exclusion-end": "value",
                "--add-maintenance-exclusion-name": "value",
                "--add-maintenance-exclusion-scope": "value",
                "--add-maintenance-exclusion-start": "value",
                "--async": "bool",
                "--autoprovisioning-config-file": "value",
                "--autoprovisioning-image-type": "value",
                "--autoprovisioning-locations": "value",
                "--autoprovisioning-max-surge-upgrade": "value",
                "--autoprovisioning-max-unavailable-upgrade": "value",
                "--autoprovisioning-min-cpu-platform": "value",
                "--autoprovisioning-network-tags": "value",
                "--autoprovisioning-node-pool-soak-duration": "value",
                "--autoprovisioning-scopes": "value",
                "--autoprovisioning-service-account": "value",
                "--autoprovisioning-standard-rollout-policy": "value",
                "--autoscaling-profile": "value",
                "--binauthz-evaluation-mode": [
                  "DISABLED",
                  "PROJECT_SINGLETON_POLICY_ENFORCE"
                ],
                "--clear-maintenance-window": "bool",
                "--clear-resource-usage-bigquery-dataset": "bool",
                "--cloud-run-config": "value",
                "--cluster-dns": [
                  "clouddns",
                  "default"
                ],
                "--cluster-dns-domain": "value",
                "--cluster-dns-scope": [
                  "cluster",
                  "vpc"
                ],
                "--complete-credential-rotation": "bool",
                "--complete-ip-rotation": "bool",
                "--database-encryption-key": "value",
                "--disable-database-encryption": "bool",
                "--disable-default-snat": "bool",
                "--disable-managed-prometheus": "bool",
                "--disable-workload-identity": "bool",
                "--enable-autoprovisioning": "bool",
                "--enable-autoprovisioning-autorepair": "bool",
                "--enable-autoprovisioning-autoupgrade": "bool",
                "--enable-autoprovisioning-blue-green-upgrade": "bool",
                "--enable-autoprovisioning-surge-upgrade": "bool",
                "--enable-autoscaling": "bool",
                "--enable-basic-auth": "bool",
                "--enable-binauthz": "bool",
                "--enable-cost-allocation": "bool",
                "--enable-google-cloud-access": "bool",
                "--enable-identity-service": "bool",
                "--enable-image-streaming": "bool",
                "--enable-intra-node-visibility": "bool",
                "--enable-l4-ilb-subsetting": "bool",
                "--enable-legacy-authorization": "bool",
                "--enable-managed-prometheus": "bool",
                "--enable-master-authorized-networks": "bool",
                "--enable-master-global-access": "bool",
                "--enable-network-egress-metering": "bool",
                "--enable-network-policy": "bool",
                "--enable-private-endpoint": "bool",
                "--enable-resource-consumption-metering": "bool",
                "--enable-service-externalips": "bool",
                "--enable-shielded-nodes": "bool",
                "--enable-stackdriver-kubernetes": "bool",
                "--enable-vertical-pod-autoscaling": "bool",
                "--gateway-api": [
                  "disabled",
                  "standard"
                ],
                "--generate-password": "bool",
                "--location-policy": [
                  "ANY",
                  "BALANCED"
                ],
                "--logging": "value",
                "--logging-service": "value",
                "--logging-variant": [
                  "DEFAULT",
                  "MAX_THROUGHPUT"
                ],
                "--maintenance-window": "value",
                "--maintenance-window-end": "value",
                "--maintenance-window-recurrence": "value",
                "--maintenance-window-start": "value",
                "--master-authorized-networks": "value",
                "--max-accelerator": "value",
                "--max-cpu": "value",
                "--max-memory": "value",
                "--max-nodes": "value",
                "--min-accelerator": "value",
                "--min-cpu": "value",
                "--min-memory": "value",
                "--min-nodes": "value",
                "--monitoring": "value",
                "--monitoring-service": "value",
                "--node-locations": "value",
                "--node-pool": "value",
                "--notification-config": "value",
                "--password": "value",
                "--private-ipv6-google-access-type": [
                  "bidirectional",
                  "disabled",
                  "outbound-only"
                ],
                "--region": "value",
                "--release-channel": [
                  "None",
                  "rapid",
                  "regular",
                  "stable"
                ],
                "--remove-labels": "value",
                "--remove-maintenance-exclusion": "value",
                "--resource-usage-bigquery-dataset": "value",
                "--security-group": "value",
                "--set-password": "bool",
                "--stack-type": [
                  "ipv4",
                  "ipv4-ipv6"
                ],
                "--start-credential-rotation": "bool",
                "--start-ip-rotation": "bool",
                "--total-max-nodes": "value",
                "--total-min-nodes": "value",
                "--update-addons": "value",
                "--update-labels": "value",
                "--username": "value",
                "--workload-pool": "value",
                "--zone": "value"
              }
            },
            "upgrade": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster-version": "value",
                "--image-type": "value",
                "--master": "bool",
                "--node-pool": "value",
                "--region": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {
            "--region": "value",
            "--zone": "value"
          }
        },
        "fleet": {
          "commands": {
            "cloudrun": {
              "commands": {
                "apply": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--context": "value",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--kubeconfig": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "features": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "identity-service": {
              "commands": {
                "apply": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--location": "value",
                    "--membership": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--membership": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ingress": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--config-membership": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config-membership": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "memberships": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "generate-gateway-rbac": {
                  "commands": {},
                  "flags": {
                    "--anthos-support": "bool",
                    "--apply": "bool",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--membership": "value",
                    "--rbac-output-file": "value",
                    "--revoke": "bool",
                    "--role": "value",
                    "--users": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--context": "value",
                    "--enable-workload-identity": "bool",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--has-private-issuer": "bool",
                    "--install-connect-agent": "bool",
                    "--internal-ip": "bool",
                    "--kubeconfig": "value",
                    "--manifest-output-file": "value",
                    "--proxy": "value",
                    "--public-issuer-url": "value",
                    "--service-account-key-file": "value"
                  }
                },
                "unregister": {
                  "commands": {},
                  "flags": {
                    "--context": "value",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--kubeconfig": "value",
                    "--location": "value",
                    "--uninstall-connect-agent": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "mesh": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--control-plane": [
                      "automatic",
                      "manual"
                    ],
                    "--location": "value",
                    "--management": [
                      "automatic",
                      "manual"
                    ],
                    "--memberships": "value"
                  }
                }
              },
              "flags": {}
            },
            "multi-cluster-services": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "get-server-config": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--region": "value",
            "--sort-by": "value",
            "--zone": "value"
          }
        },
        "hub": {
          "commands": {
            "cloudrun": {
              "commands": {
                "apply": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--context": "value",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--kubeconfig": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "features": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "identity-service": {
              "commands": {
                "apply": {
                  "commands": {},
                  "flags": {
                    "--config": "value",
                    "--location": "value",
                    "--membership": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--membership": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ingress": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--config-membership": "value",
                    "--location": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config-membership": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "memberships": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "generate-gateway-rbac": {
                  "commands": {},
                  "flags": {
                    "--anthos-support": "bool",
                    "--apply": "bool",
                    "--context": "value",
                    "--kubeconfig": "value",
                    "--membership": "value",
                    "--rbac-output-file": "value",
                    "--revoke": "bool",
                    "--role": "value",
                    "--users": "value"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "register": {
                  "commands": {},
                  "flags": {
                    "--context": "value",
                    "--enable-workload-identity": "bool",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--has-private-issuer": "bool",
                    "--install-connect-agent": "bool",
                    "--internal-ip": "bool",
                    "--kubeconfig": "value",
                    "--manifest-output-file": "value",
                    "--proxy": "value",
                    "--public-issuer-url": "value",
                    "--service-account-key-file": "value"
                  }
                },
                "unregister": {
                  "commands": {},
                  "flags": {
                    "--context": "value",
                    "--gke-cluster": "value",
                    "--gke-uri": "value",
                    "--kubeconfig": "value",
                    "--location": "value",
                    "--uninstall-connect-agent": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "mesh": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--control-plane": [
                      "automatic",
                      "manual"
                    ],
                    "--location": "value",
                    "--management": [
                      "automatic",
                      "manual"
                    ],
                    "--memberships": "value"
                  }
                }
              },
              "flags": {}
            },
            "multi-cluster-services": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--force": "bool"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "images": {
          "commands": {
            "add-tag": {
              "commands": {},
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force-delete-tags": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--repository": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-tags": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "untag": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "node-pools": {
          "commands": {
            "complete-upgrade": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--region": "value",
                "--zone": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--async": "bool",
                "--boot-disk-kms-key": "value",
                "--cluster": "value",
                "--create-pod-ipv4-range": "value",
                "--disk-size": "value",
                "--disk-type": [
                  "pd-balanced",
                  "pd-ssd",
                  "pd-standard"
                ],
                "--enable-autoprovisioning": "bool",
                "--enable-autorepair": "bool",
                "--enable-autoscaling": "bool",
                "--enable-autoupgrade": "bool",
                "--enable-blue-green-upgrade": "bool",
                "--enable-confidential-nodes": "bool",
                "--enable-gvnic": "bool",
                "--enable-image-streaming": "bool",
                "--enable-private-nodes": "bool",
                "--enable-surge-upgrade": "bool",
                "--ephemeral-storage-local-ssd": "value",
                "--image-type": "value",
                "--labels": "value",
                "--local-nvme-ssd-block": "value",
                "--local-ssd-count": "value",
                "--location-policy": [
                  "ANY",
                  "BALANCED"
                ],
                "--logging-variant": [
                  "DEFAULT",
                  "MAX_THROUGHPUT"
                ],
                "--machine-type": "value",
                "--max-nodes": "value",
                "--max-pods-per-node": "value",
                "--max-surge-upgrade": "value",
                "--max-unavailable-upgrade": "value",
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--min-cpu-platform": "value",
                "--min-nodes": "value",
                "--network-performance-configs": "value",
                "--node-group": "value",
                "--node-labels": "value",
                "--node-locations": "value",
                "--node-pool-soak-duration": "value",
                "--node-taints": "value",
                "--node-version": "value",
                "--num-nodes": "value",
                "--placement-type": [
                  "COMPACT",
                  "UNSPECIFIED"
                ],
                "--pod-ipv4-range": "value",
                "--preemptible": "bool",
                "--region": "value",
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--sandbox": "value",
                "--scopes": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--spot": "bool",
                "--standard-rollout-policy": "value",
                "--system-config-from-file": "value",
                "--tags": "value",
                "--threads-per-core": "value",
                "--total-max-nodes": "value",
                "--total-min-nodes": "value",
                "--windows-os-version": [
                  "ltsc2019",
                  "ltsc2022"
                ],
                "--workload-metadata": [
                  "GCE_METADATA",
                  "GKE_METADATA"
                ],
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--region": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--region": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "value"
              }
            },
            "rollback": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--region": "value",
                "--respect-pdb": "value",
                "--zone": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--enable-autoprovisioning": "bool",
                "--enable-autorepair": "bool",
                "--enable-autoscaling": "bool",
                "--enable-autoupgrade": "bool",
                "--enable-blue-green-upgrade": "bool",
                "--enable-confidential-nodes": "bool",
                "--enable-gvnic": "bool",
                "--enable-image-streaming": "bool",
                "--enable-private-nodes": "bool",
                "--enable-surge-upgrade": "bool",
                "--labels": "value",
                "--location-policy": [
                  "ANY",
                  "BALANCED"
                ],
                "--logging-variant": [
                  "DEFAULT",
                  "MAX_THROUGHPUT"
                ],
                "--max-nodes": "value",
                "--max-surge-upgrade": "value",
                "--max-unavailable-upgrade": "value",
                "--min-nodes": "value",
                "--network-performance-configs": "value",
                "--node-labels": "value",
                "--node-locations": "value",
                "--node-pool-soak-duration": "value",
                "--node-taints": "value",
                "--region": "value",
                "--standard-rollout-policy": "value",
                "--system-config-from-file": "value",
                "--tags": "value",
                "--total-max-nodes": "value",
                "--total-min-nodes": "value",
                "--windows-os-version": [
                  "ltsc2019",
                  "ltsc2022"
                ],
                "--workload-metadata": [
                  "GCE_METADATA",
                  "GKE_METADATA"
                ],
                "--zone": "value"
              }
            }
          },
          "flags": {
            "--region": "value",
            "--zone": "value"
          }
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--zone": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {
            "--region": "value",
            "--zone": "value"
          }
        },
        "subnets": {
          "commands": {
            "list-usable": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--network-project": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "data-catalog": {
      "commands": {
        "entries": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--entry-group": "value",
                "--gcs-file-patterns": "value",
                "--linked-resource": "value",
                "--location": "value",
                "--schema": "value",
                "--schema-from-file": "value",
                "--source-system-create-time": "value",
                "--source-system-update-time": "value",
                "--type": [
                  "cluster",
                  "dashboard",
                  "data-source-connection",
                  "data-stream",
                  "database",
                  "database-schema",
                  "entry-type-unspecified",
                  "explore",
                  "fileset",
                  "lake",
                  "look",
                  "model",
                  "routine",
                  "service",
                  "table",
                  "zone"
                ],
                "--user-specified-system": "value",
                "--user-specified-type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--entry-group": "value",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--entry-group": "value",
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--entry-group": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "lookup": {
              "commands": {},
              "flags": {}
            },
            "star": {
              "commands": {},
              "flags": {
                "--entry-group": "value",
                "--location": "value"
              }
            },
            "unstar": {
              "commands": {},
              "flags": {
                "--entry-group": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-file-patterns": "value",
                "--clear-file-patterns": "bool",
                "--description": "value",
                "--display-name": "value",
                "--entry-group": "value",
                "--linked-resource": "value",
                "--location": "value",
                "--lookup-entry": "value",
                "--remove-file-patterns": "value",
                "--schema": "value",
                "--schema-from-file": "value",
                "--user-specified-system": "value",
                "--user-specified-type": "value"
              }
            }
          },
          "flags": {}
        },
        "entry-groups": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "search": {
          "commands": {},
          "flags": {
            "--include-gcp-public-datasets": "bool",
            "--include-organization-ids": "value",
            "--include-project-ids": "value",
            "--limit": "value",
            "--order-by": "value",
            "--page-size": "value",
            "--restricted-locations": "value"
          }
        },
        "tag-templates": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--display-name": "value",
                "--field": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "fields": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--location": "value",
                    "--tag-template": "value",
                    "--type": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--force": "bool",
                    "--location": "value",
                    "--tag-template": "value"
                  }
                },
                "enum-values": {
                  "commands": {
                    "rename": {
                      "commands": {},
                      "flags": {
                        "--field": "value",
                        "--location": "value",
                        "--new-id": "value",
                        "--tag-template": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "rename": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--new-id": "value",
                    "--tag-template": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--display-name": "value",
                    "--enum-values": "value",
                    "--location": "value",
                    "--required": "bool",
                    "--tag-template": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--display-name": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "tags": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--entry": "value",
                "--entry-group": "value",
                "--location": "value",
                "--scope": "value",
                "--tag-file": "value",
                "--tag-template": "value",
                "--tag-template-location": "value",
                "--tag-template-project": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--entry": "value",
                "--entry-group": "value",
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--entry": "value",
                "--entry-group": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--entry": "value",
                "--entry-group": "value",
                "--location": "value",
                "--tag-file": "value",
                "--tag-template": "value",
                "--tag-template-location": "value",
                "--tag-template-project": "value"
              }
            }
          },
          "flags": {}
        },
        "taxonomies": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "policy-tags": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--taxonomy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--taxonomy": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--taxonomy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--taxonomy": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--taxonomy": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--taxonomy": "value"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "database-migration": {
      "commands": {
        "connection-profiles": {
          "commands": {
            "create": {
              "commands": {
                "alloydb": {
                  "commands": {},
                  "flags": {
                    "--cluster-labels": "value",
                    "--cpu-count": [
                      2,
                      4,
                      8,
                      16,
                      32,
                      64
                    ],
                    "--database-flags": "value",
                    "--display-name": "value",
                    "--labels": "value",
                    "--network": "value",
                    "--no-async": "bool",
                    "--password": "value",
                    "--primary-id": "value",
                    "--primary-labels": "value",
                    "--region": "dynamic"
                  }
                },
                "cloudsql": {
                  "commands": {},
                  "flags": {
                    "--activation-policy": [
                      "ALWAYS",
                      "NEVER"
                    ],
                    "--authorized-networks": "value",
                    "--auto-storage-increase": "bool",
                    "--data-disk-size": "value",
                    "--data-disk-type": [
                      "PD_HDD",
                      "PD_SSD"
                    ],
                    "--database-flags": "value",
                    "--database-version": [
                      "MYSQL_5_6",
                      "MYSQL_5_7",
                      "MYSQL_8_0",
                      "POSTGRES_10",
                      "POSTGRES_11",
                      "POSTGRES_12",
                      "POSTGRES_13",
                      "POSTGRES_14",
                      "POSTGRES_9_6"
                    ],
                    "--display-name": "value",
                    "--enable-ip-v4": "bool",
                    "--labels": "value",
                    "--no-async": "bool",
                    "--private-network": "value",
                    "--provider": [
                      "CLOUDSQL",
                      "RDS"
                    ],
                    "--region": "dynamic",
                    "--require-ssl": "bool",
                    "--root-password": "value",
                    "--source-id": "dynamic",
                    "--storage-auto-resize-limit": "value",
                    "--tier": "value",
                    "--user-labels": "value",
                    "--zone": "value"
                  }
                },
                "mysql": {
                  "commands": {},
                  "flags": {
                    "--ca-certificate": "value",
                    "--client-certificate": "value",
                    "--cloudsql-instance": "value",
                    "--display-name": "value",
                    "--host": "value",
                    "--labels": "value",
                    "--no-async": "bool",
                    "--password": "value",
                    "--port": "value",
                    "--private-key": "value",
                    "--prompt-for-password": "bool",
                    "--provider": [
                      "CLOUDSQL",
                      "RDS"
                    ],
                    "--region": "dynamic",
                    "--username": "value"
                  }
                },
                "postgresql": {
                  "commands": {},
                  "flags": {
                    "--ca-certificate": "value",
                    "--client-certificate": "value",
                    "--cloudsql-instance": "value",
                    "--display-name": "value",
                    "--host": "value",
                    "--labels": "value",
                    "--no-async": "bool",
                    "--password": "value",
                    "--port": "value",
                    "--private-key": "value",
                    "--prompt-for-password": "bool",
                    "--region": "dynamic",
                    "--username": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--ca-certificate": "value",
                "--clear-labels": "bool",
                "--client-certificate": "value",
                "--cloudsql-instance": "value",
                "--display-name": "value",
                "--host": "value",
                "--password": "value",
                "--port": "value",
                "--private-key": "value",
                "--prompt-for-password": "bool",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value",
                "--username": "value"
              }
            }
          },
          "flags": {}
        },
        "migration-jobs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--destination": "dynamic",
                "--display-name": "value",
                "--dump-path": "value",
                "--labels": "value",
                "--no-async": "bool",
                "--peer-vpc": "value",
                "--region": "dynamic",
                "--source": "dynamic",
                "--static-ip": "bool",
                "--type": [
                  "CONTINUOUS",
                  "ONE_TIME"
                ],
                "--vm": "value",
                "--vm-ip": "value",
                "--vm-port": "value",
                "--vpc": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "generate-ssh-script": {
              "commands": {},
              "flags": {
                "--region": "dynamic",
                "--subnet": "value",
                "--vm": "value",
                "--vm-machine-type": "value",
                "--vm-port": "value",
                "--vm-zone": "value",
                "--vm-zone-create": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "promote": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "restart": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "resume": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--destination": "dynamic",
                "--display-name": "value",
                "--dump-path": "value",
                "--no-async": "bool",
                "--peer-vpc": "value",
                "--region": "dynamic",
                "--remove-labels": "value",
                "--source": "dynamic",
                "--static-ip": "bool",
                "--type": [
                  "CONTINUOUS",
                  "ONE_TIME"
                ],
                "--update-labels": "value",
                "--vm": "value",
                "--vm-ip": "value",
                "--vm-port": "value",
                "--vpc": "value"
              }
            },
            "verify": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "dataflow": {
      "commands": {
        "flex-template": {
          "commands": {
            "build": {
              "commands": {},
              "flags": {
                "--additional-experiments": "value",
                "--additional-user-labels": "value",
                "--dataflow-kms-key": "value",
                "--disable-public-ips": "bool",
                "--enable-streaming-engine": "bool",
                "--env": "value",
                "--flex-template-base-image": "value",
                "--gcs-log-dir": "value",
                "--go-binary-path": "value",
                "--image": "value",
                "--image-gcr-path": "value",
                "--image-repository-cert-path": "value",
                "--image-repository-password-secret-id": "value",
                "--image-repository-username-secret-id": "value",
                "--jar": "value",
                "--max-workers": "value",
                "--metadata-file": "value",
                "--network": "value",
                "--num-workers": "value",
                "--print-only": "bool",
                "--py-path": "value",
                "--sdk-language": [
                  "GO",
                  "JAVA",
                  "PYTHON"
                ],
                "--service-account-email": "value",
                "--staging-location": "value",
                "--subnetwork": "value",
                "--temp-location": "value",
                "--worker-machine-type": "value",
                "--worker-region": "value",
                "--worker-zone": "value"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--additional-experiments": "value",
                "--additional-user-labels": "value",
                "--dataflow-kms-key": "value",
                "--disable-public-ips": "bool",
                "--enable-streaming-engine": "bool",
                "--flexrs-goal": [
                  "COST_OPTIMIZED",
                  "SPEED_OPTIMIZED"
                ],
                "--max-workers": "value",
                "--network": "value",
                "--num-workers": "value",
                "--parameters": "value",
                "--region": "value",
                "--service-account-email": "value",
                "--staging-location": "value",
                "--subnetwork": "value",
                "--temp-location": "value",
                "--template-file-gcs-location": "value",
                "--transform-name-mappings": "value",
                "--update": "bool",
                "--worker-machine-type": "value",
                "--worker-region": "value",
                "--worker-zone": "value"
              }
            }
          },
          "flags": {}
        },
        "jobs": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--full": "bool",
                "--region": "value"
              }
            },
            "drain": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--created-after": "value",
                "--created-before": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--status": [
                  "active",
                  "all",
                  "terminated"
                ],
                "--uri": "bool"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--additional-experiments": "value",
                "--dataflow-kms-key": "value",
                "--disable-public-ips": "bool",
                "--enable-streaming-engine": "bool",
                "--gcs-location": "value",
                "--max-workers": "value",
                "--network": "value",
                "--num-workers": "value",
                "--parameters": "value",
                "--region": "value",
                "--service-account-email": "value",
                "--staging-location": "value",
                "--subnetwork": "value",
                "--worker-machine-type": "value",
                "--worker-region": "value",
                "--worker-zone": "value",
                "--zone": "value"
              }
            },
            "show": {
              "commands": {},
              "flags": {
                "--environment": "bool",
                "--region": "value",
                "--steps": "bool"
              }
            }
          },
          "flags": {}
        },
        "snapshots": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--job-id": "value",
                "--region": "value",
                "--snapshot-sources": "value",
                "--snapshot-ttl": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--job-id": "value",
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "sql": {
          "commands": {
            "query": {
              "commands": {},
              "flags": {
                "--bigquery-dataset": "value",
                "--bigquery-project": "value",
                "--bigquery-table": "value",
                "--bigquery-write-disposition": [
                  "write-append",
                  "write-empty",
                  "write-truncate"
                ],
                "--dataflow-kms-key": "value",
                "--disable-public-ips": "bool",
                "--dry-run": "bool",
                "--job-name": "value",
                "--max-workers": "value",
                "--network": "value",
                "--num-workers": "value",
                "--parameter": "value",
                "--parameters-file": "value",
                "--pubsub-create-disposition": [
                  "create-if-not-found",
                  "fail-if-not-found"
                ],
                "--pubsub-project": "value",
                "--pubsub-topic": "value",
                "--region": "value",
                "--service-account-email": "value",
                "--subnetwork": "value",
                "--worker-machine-type": "value",
                "--worker-region": "value",
                "--worker-zone": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "dataplex": {
      "commands": {
        "assets": {
          "commands": {
            "actions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--asset": "dynamic",
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value",
                "--zone": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--csv-delimiter": "value",
                "--csv-disable-type-inference": "bool",
                "--csv-encoding": "value",
                "--csv-header-rows": "value",
                "--description": "value",
                "--discovery-enabled": "bool",
                "--discovery-exclude-patterns": "value",
                "--discovery-include-patterns": "value",
                "--discovery-schedule": "value",
                "--display-name": "value",
                "--json-disable-type-inference": "bool",
                "--json-encoding": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--resource-name": "value",
                "--resource-type": [
                  "BIGQUERY_DATASET",
                  "STORAGE_BUCKET"
                ],
                "--validate-only": "bool",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--lake": "dynamic",
                "--location": "dynamic",
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic",
                "--zone": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "dynamic"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value",
                "--zone": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--zone": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--csv-delimiter": "value",
                "--csv-disable-type-inference": "bool",
                "--csv-encoding": "value",
                "--csv-header-rows": "value",
                "--description": "value",
                "--discovery-enabled": "bool",
                "--discovery-exclude-patterns": "value",
                "--discovery-include-patterns": "value",
                "--discovery-schedule": "value",
                "--display-name": "value",
                "--json-disable-type-inference": "bool",
                "--json-encoding": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--validate-only": "bool",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        },
        "content": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--data-text": "value",
                "--description": "value",
                "--kernel-type": [
                  "PYTHON3"
                ],
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--path": "value",
                "--query-engine": [
                  "SPARK"
                ],
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic",
                "--view": [
                  "basic",
                  "full"
                ]
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--data-text": "value",
                "--description": "value",
                "--kernel-type": [
                  "PYTHON3"
                ],
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--path": "value",
                "--query-engine": [
                  "SPARK"
                ],
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        },
        "environments": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--compute-disk-size-gb": "value",
                "--compute-max-node-count": "value",
                "--compute-node-count": "value",
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--os-image-java-libraries": "value",
                "--os-image-properties": "value",
                "--os-image-python-packages": "value",
                "--os-image-version": "value",
                "--session-enable-fast-startup": "bool",
                "--session-max-idle-duration": "value",
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "sessions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--environment": "dynamic",
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--compute-disk-size-gb": "value",
                "--compute-max-node-count": "value",
                "--compute-node-count": "value",
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--os-image-java-libraries": "value",
                "--os-image-properties": "value",
                "--os-image-python-packages": "value",
                "--os-image-version": "value",
                "--session-enable-fast-startup": "bool",
                "--session-max-idle-duration": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        },
        "lakes": {
          "commands": {
            "actions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "authorize": {
              "commands": {},
              "flags": {
                "--bigquery-dataset-resource": "value",
                "--project-resource": "value",
                "--secondary-project": "value",
                "--storage-bucket-resource": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--location": "value",
                "--metastore-service": "value",
                "--validate-only": "bool"
              }
            },
            "deauthorize": {
              "commands": {},
              "flags": {
                "--bigquery-dataset-resource": "value",
                "--project-resource": "value",
                "--secondary-project": "value",
                "--storage-bucket-resource": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--display-name": "value",
                "--labels": "value",
                "--location": "value",
                "--metastore-service": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        },
        "tasks": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--batch-executors-count": "value",
                "--batch-max-executors-count": "value",
                "--container-image": "value",
                "--container-image-java-jars": "value",
                "--container-image-properties": "value",
                "--container-image-python-packages": "value",
                "--description": "value",
                "--display-name": "value",
                "--execution-args": "value",
                "--execution-project": "value",
                "--execution-service-account": "value",
                "--kms-key": "value",
                "--labels": "value",
                "--lake": "dynamic",
                "--location": "dynamic",
                "--max-job-execution-lifetime": "value",
                "--notebook": "value",
                "--notebook-archive-uris": "value",
                "--notebook-batch-executors-count": "value",
                "--notebook-batch-max-executors-count": "value",
                "--notebook-container-image": "value",
                "--notebook-container-image-java-jars": "value",
                "--notebook-container-image-properties": "value",
                "--notebook-file-uris": "value",
                "--notebook-vpc-network-name": "value",
                "--notebook-vpc-network-tags": "value",
                "--notebook-vpc-sub-network-name": "value",
                "--spark-archive-uris": "value",
                "--spark-file-uris": "value",
                "--spark-main-class": "value",
                "--spark-main-jar-file-uri": "value",
                "--spark-python-script-file": "value",
                "--spark-sql-script": "value",
                "--spark-sql-script-file": "value",
                "--trigger-disabled": "bool",
                "--trigger-max-retires": "value",
                "--trigger-schedule": "value",
                "--trigger-start-time": "value",
                "--trigger-type": [
                  "on-demand",
                  "recurring"
                ],
                "--vpc-network-name": "value",
                "--vpc-network-tags": "value",
                "--vpc-sub-network-name": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "jobs": {
              "commands": {
                "cancel": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--task": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--lake": "dynamic",
                    "--location": "dynamic",
                    "--task": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--task": "dynamic",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--batch-executors-count": "value",
                "--batch-max-executors-count": "value",
                "--clear-labels": "bool",
                "--container-image": "value",
                "--container-image-java-jars": "value",
                "--container-image-properties": "value",
                "--container-image-python-packages": "value",
                "--description": "value",
                "--display-name": "value",
                "--execution-args": "value",
                "--execution-project": "value",
                "--execution-service-account": "value",
                "--kms-key": "value",
                "--lake": "dynamic",
                "--location": "dynamic",
                "--max-job-execution-lifetime": "value",
                "--notebook": "value",
                "--notebook-archive-uris": "value",
                "--notebook-batch-executors-count": "value",
                "--notebook-batch-max-executors-count": "value",
                "--notebook-container-image": "value",
                "--notebook-container-image-java-jars": "value",
                "--notebook-container-image-properties": "value",
                "--notebook-file-uris": "value",
                "--notebook-vpc-network-name": "value",
                "--notebook-vpc-network-tags": "value",
                "--notebook-vpc-sub-network-name": "value",
                "--remove-labels": "value",
                "--spark-archive-uris": "value",
                "--spark-file-uris": "value",
                "--spark-main-class": "value",
                "--spark-main-jar-file-uri": "value",
                "--spark-python-script-file": "value",
                "--spark-sql-script": "value",
                "--spark-sql-script-file": "value",
                "--trigger-disabled": "bool",
                "--trigger-max-retires": "value",
                "--trigger-schedule": "value",
                "--trigger-start-time": "value",
                "--update-labels": "value",
                "--vpc-network-name": "value",
                "--vpc-network-tags": "value",
                "--vpc-sub-network-name": "value"
              }
            }
          },
          "flags": {}
        },
        "zones": {
          "commands": {
            "actions": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--lake": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--csv-delimiter": "value",
                "--csv-disable-type-inference": "bool",
                "--csv-encoding": "value",
                "--csv-header-rows": "value",
                "--description": "value",
                "--discovery-enabled": "bool",
                "--discovery-exclude-patterns": "value",
                "--discovery-include-patterns": "value",
                "--discovery-schedule": "value",
                "--display-name": "value",
                "--json-disable-type-inference": "bool",
                "--json-encoding": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--resource-location-type": [
                  "MULTI_REGION",
                  "SINGLE_REGION"
                ],
                "--type": [
                  "CURATED",
                  "RAW"
                ],
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--lake": "dynamic",
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--lake": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--lake": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--csv-delimiter": "value",
                "--csv-disable-type-inference": "bool",
                "--csv-encoding": "value",
                "--csv-header-rows": "value",
                "--description": "value",
                "--discovery-enabled": "bool",
                "--discovery-exclude-patterns": "value",
                "--discovery-include-patterns": "value",
                "--discovery-schedule": "value",
                "--display-name": "value",
                "--json-disable-type-inference": "bool",
                "--json-encoding": "value",
                "--labels": "value",
                "--lake": "value",
                "--location": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "dataproc": {
      "commands": {
        "autoscaling-policies": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "batches": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "submit": {
              "commands": {
                "pyspark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--batch": "value",
                    "--container-image": "value",
                    "--deps-bucket": "value",
                    "--files": "value",
                    "--history-server-cluster": "value",
                    "--jars": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--metastore-service": "value",
                    "--network": "value",
                    "--properties": "value",
                    "--py-files": "value",
                    "--region": "value",
                    "--request-id": "value",
                    "--service-account": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--version": "value"
                  }
                },
                "spark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--batch": "value",
                    "--class": "value",
                    "--container-image": "value",
                    "--deps-bucket": "value",
                    "--files": "value",
                    "--history-server-cluster": "value",
                    "--jar": "value",
                    "--jars": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--metastore-service": "value",
                    "--network": "value",
                    "--properties": "value",
                    "--region": "value",
                    "--request-id": "value",
                    "--service-account": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--version": "value"
                  }
                },
                "spark-r": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--batch": "value",
                    "--container-image": "value",
                    "--deps-bucket": "value",
                    "--files": "value",
                    "--history-server-cluster": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--metastore-service": "value",
                    "--network": "value",
                    "--properties": "value",
                    "--region": "value",
                    "--request-id": "value",
                    "--service-account": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--version": "value"
                  }
                },
                "spark-sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--batch": "value",
                    "--container-image": "value",
                    "--deps-bucket": "value",
                    "--history-server-cluster": "value",
                    "--jars": "value",
                    "--kms-key": "value",
                    "--labels": "value",
                    "--metastore-service": "value",
                    "--network": "value",
                    "--properties": "value",
                    "--region": "value",
                    "--request-id": "value",
                    "--service-account": "value",
                    "--subnet": "value",
                    "--tags": "value",
                    "--vars": "value",
                    "--version": "value"
                  }
                }
              },
              "flags": {
                "--async": "bool",
                "--batch": "value",
                "--container-image": "value",
                "--history-server-cluster": "value",
                "--kms-key": "value",
                "--labels": "value",
                "--metastore-service": "value",
                "--network": "value",
                "--properties": "value",
                "--region": "value",
                "--request-id": "value",
                "--service-account": "value",
                "--subnet": "value",
                "--tags": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "clusters": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--action-on-failed-primary-workers": [
                  "DELETE",
                  "FAILURE_ACTION_UNSPECIFIED",
                  "NO_ACTION"
                ],
                "--async": "bool",
                "--autoscaling-policy": "value",
                "--bucket": "value",
                "--confidential-compute": "bool",
                "--dataproc-metastore": "value",
                "--driver-pool-accelerator": "value",
                "--driver-pool-boot-disk-size": "value",
                "--driver-pool-boot-disk-type": "value",
                "--driver-pool-id": "value",
                "--driver-pool-local-ssd-interface": "value",
                "--driver-pool-machine-type": "value",
                "--driver-pool-min-cpu-platform": "value",
                "--driver-pool-size": "value",
                "--enable-component-gateway": "bool",
                "--enable-kerberos": "bool",
                "--expiration-time": "value",
                "--gce-pd-kms-key": "dynamic",
                "--gce-pd-kms-key-keyring": "dynamic",
                "--gce-pd-kms-key-location": "dynamic",
                "--gce-pd-kms-key-project": "dynamic",
                "--identity-config-file": "value",
                "--image": "value",
                "--image-version": "value",
                "--initialization-action-timeout": "value",
                "--initialization-actions": "value",
                "--kerberos-config-file": "value",
                "--kerberos-kms-key": "dynamic",
                "--kerberos-kms-key-keyring": "dynamic",
                "--kerberos-kms-key-location": "dynamic",
                "--kerberos-kms-key-project": "dynamic",
                "--kerberos-root-principal-password-uri": "value",
                "--labels": "value",
                "--master-accelerator": "value",
                "--master-boot-disk-size": "value",
                "--master-boot-disk-type": "value",
                "--master-local-ssd-interface": "value",
                "--master-machine-type": "value",
                "--master-min-cpu-platform": "value",
                "--max-age": "value",
                "--max-idle": "value",
                "--metadata": "value",
                "--metric-overrides": "value",
                "--metric-overrides-file": "value",
                "--metric-sources": "value",
                "--network": "value",
                "--no-address": "bool",
                "--node-group": "value",
                "--num-driver-pool-local-ssds": "value",
                "--num-master-local-ssds": "value",
                "--num-masters": "value",
                "--num-secondary-worker-local-ssds": "value",
                "--num-secondary-workers": "value",
                "--num-worker-local-ssds": "value",
                "--num-workers": "value",
                "--optional-components": "value",
                "--private-ipv6-google-access-type": [
                  "bidirectional",
                  "inherit-subnetwork",
                  "outbound"
                ],
                "--properties": "value",
                "--region": "value",
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--scopes": "value",
                "--secondary-worker-accelerator": "value",
                "--secondary-worker-boot-disk-size": "value",
                "--secondary-worker-boot-disk-type": "value",
                "--secondary-worker-local-ssd-interface": "value",
                "--secondary-worker-type": [
                  "non-preemptible",
                  "preemptible",
                  "spot"
                ],
                "--secure-multi-tenancy-user-mapping": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--single-node": "bool",
                "--subnet": "value",
                "--tags": "value",
                "--temp-bucket": "value",
                "--worker-accelerator": "value",
                "--worker-boot-disk-size": "value",
                "--worker-boot-disk-type": "value",
                "--worker-local-ssd-interface": "value",
                "--worker-machine-type": "value",
                "--worker-min-cpu-platform": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "diagnose": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "gke": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--gke-cluster": "value",
                    "--gke-cluster-location": "value",
                    "--history-server-cluster": "value",
                    "--history-server-cluster-region": "value",
                    "--metastore-service": "value",
                    "--metastore-service-location": "value",
                    "--namespace": "value",
                    "--pools": "value",
                    "--properties": "value",
                    "--region": "value",
                    "--setup-workload-identity": "bool",
                    "--spark-engine-version": "value",
                    "--staging-bucket": "value"
                  }
                }
              },
              "flags": {}
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--autoscaling-policy": "value",
                "--clear-labels": "bool",
                "--disable-autoscaling": "bool",
                "--expiration-time": "value",
                "--graceful-decommission-timeout": "value",
                "--max-age": "value",
                "--max-idle": "value",
                "--no-max-age": "bool",
                "--no-max-idle": "bool",
                "--num-secondary-workers": "value",
                "--num-workers": "value",
                "--region": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "jobs": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "kill": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--state-filter": [
                  "active",
                  "inactive"
                ]
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "submit": {
              "commands": {
                "hadoop": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--bucket": "value",
                    "--class": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jar": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "hive": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--continue-on-failure": "bool",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "pig": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "presto": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--client-tags": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--query-output-format": "value",
                    "--region": "value"
                  }
                },
                "pyspark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--bucket": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--py-files": "value",
                    "--region": "value"
                  }
                },
                "spark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--bucket": "value",
                    "--class": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jar": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "spark-r": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--async": "bool",
                    "--bucket": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "spark-sql": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value"
                  }
                },
                "trino": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--client-tags": "value",
                    "--cluster": "value",
                    "--cluster-labels": "value",
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--labels": "value",
                    "--max-failures-per-hour": "value",
                    "--max-failures-total": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--query-output-format": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {
                "--async": "bool",
                "--bucket": "value",
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--region": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "node-groups": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--region": "value"
              }
            },
            "resize": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--graceful-decommission-timeout": "value",
                "--region": "value",
                "--size": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--state-filter": [
                  "active",
                  "inactive"
                ]
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "workflow-templates": {
          "commands": {
            "add-job": {
              "commands": {
                "hadoop": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--class": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jar": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "hive": {
                  "commands": {},
                  "flags": {
                    "--continue-on-failure": "bool",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "pig": {
                  "commands": {},
                  "flags": {
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "presto": {
                  "commands": {},
                  "flags": {
                    "--client-tags": "value",
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--query-output-format": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "pyspark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--py-files": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "spark": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--class": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--jar": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "spark-r": {
                  "commands": {},
                  "flags": {
                    "--archives": "value",
                    "--driver-log-levels": "value",
                    "--files": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "spark-sql": {
                  "commands": {},
                  "flags": {
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--jars": "value",
                    "--labels": "value",
                    "--params": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                },
                "trino": {
                  "commands": {},
                  "flags": {
                    "--client-tags": "value",
                    "--continue-on-failure": "bool",
                    "--driver-log-levels": "value",
                    "--execute": "value",
                    "--file": "value",
                    "--labels": "value",
                    "--properties": "value",
                    "--properties-file": "value",
                    "--query-output-format": "value",
                    "--region": "value",
                    "--start-after": "value",
                    "--step-id": "value",
                    "--workflow-template": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--dag-timeout": "value",
                "--labels": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--source": "value"
              }
            },
            "instantiate": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--parameters": "value",
                "--region": "value"
              }
            },
            "instantiate-from-file": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--file": "value",
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "remove-dag-timeout": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "remove-job": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--step-id": "value"
              }
            },
            "set-cluster-selector": {
              "commands": {},
              "flags": {
                "--cluster-labels": "value",
                "--region": "value"
              }
            },
            "set-dag-timeout": {
              "commands": {},
              "flags": {
                "--dag-timeout": "value",
                "--region": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "set-managed-cluster": {
              "commands": {},
              "flags": {
                "--autoscaling-policy": "value",
                "--bucket": "value",
                "--cluster-name": "value",
                "--confidential-compute": "bool",
                "--dataproc-metastore": "value",
                "--enable-component-gateway": "bool",
                "--enable-kerberos": "bool",
                "--identity-config-file": "value",
                "--image": "value",
                "--image-version": "value",
                "--initialization-action-timeout": "value",
                "--initialization-actions": "value",
                "--kerberos-config-file": "value",
                "--kerberos-kms-key": "dynamic",
                "--kerberos-kms-key-keyring": "dynamic",
                "--kerberos-kms-key-location": "dynamic",
                "--kerberos-kms-key-project": "dynamic",
                "--kerberos-root-principal-password-uri": "value",
                "--labels": "value",
                "--master-accelerator": "value",
                "--master-boot-disk-size": "value",
                "--master-boot-disk-type": "value",
                "--master-local-ssd-interface": "value",
                "--master-machine-type": "value",
                "--master-min-cpu-platform": "value",
                "--metadata": "value",
                "--metric-overrides": "value",
                "--metric-overrides-file": "value",
                "--metric-sources": "value",
                "--network": "value",
                "--no-address": "bool",
                "--node-group": "value",
                "--num-master-local-ssds": "value",
                "--num-masters": "value",
                "--num-secondary-worker-local-ssds": "value",
                "--num-secondary-workers": "value",
                "--num-worker-local-ssds": "value",
                "--num-workers": "value",
                "--optional-components": "value",
                "--private-ipv6-google-access-type": [
                  "bidirectional",
                  "inherit-subnetwork",
                  "outbound"
                ],
                "--properties": "value",
                "--region": "value",
                "--reservation": "value",
                "--reservation-affinity": [
                  "any",
                  "none",
                  "specific"
                ],
                "--scopes": "value",
                "--secondary-worker-accelerator": "value",
                "--secondary-worker-boot-disk-size": "value",
                "--secondary-worker-boot-disk-type": "value",
                "--secondary-worker-local-ssd-interface": "value",
                "--secondary-worker-type": [
                  "non-preemptible",
                  "preemptible",
                  "spot"
                ],
                "--secure-multi-tenancy-user-mapping": "value",
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--single-node": "bool",
                "--subnet": "value",
                "--tags": "value",
                "--temp-bucket": "value",
                "--worker-accelerator": "value",
                "--worker-boot-disk-size": "value",
                "--worker-boot-disk-type": "value",
                "--worker-local-ssd-interface": "value",
                "--worker-machine-type": "value",
                "--worker-min-cpu-platform": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "datastore": {
      "commands": {
        "databases": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "export": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--kinds": "value",
            "--namespaces": "value",
            "--operation-labels": "value"
          }
        },
        "import": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--kinds": "value",
            "--namespaces": "value",
            "--operation-labels": "value"
          }
        },
        "indexes": {
          "commands": {
            "cleanup": {
              "commands": {},
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "datastream": {
      "commands": {
        "connection-profiles": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--bucket": "value",
                "--ca-certificate": "value",
                "--client-certificate": "value",
                "--client-key": "value",
                "--database-service": "value",
                "--display-name": "value",
                "--force": "bool",
                "--forward-ssh-hostname": "value",
                "--forward-ssh-password": "value",
                "--forward-ssh-port": "value",
                "--forward-ssh-private-key": "value",
                "--forward-ssh-username": "value",
                "--labels": "value",
                "--location": "value",
                "--mysql-hostname": "value",
                "--mysql-password": "value",
                "--mysql-port": "value",
                "--mysql-prompt-for-password": "bool",
                "--mysql-username": "value",
                "--oracle-hostname": "value",
                "--oracle-password": "value",
                "--oracle-port": "value",
                "--oracle-prompt-for-password": "bool",
                "--oracle-username": "value",
                "--postgresql-database": "value",
                "--postgresql-hostname": "value",
                "--postgresql-password": "value",
                "--postgresql-port": "value",
                "--postgresql-prompt-for-password": "bool",
                "--postgresql-username": "value",
                "--private-connection": "value",
                "--root-path": "value",
                "--static-ip-connectivity": "bool",
                "--type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "discover": {
              "commands": {},
              "flags": {
                "--connection-profile-name": "value",
                "--connection-profile-object-file": "value",
                "--full-hierarchy": "bool",
                "--hierarchy-depth": "value",
                "--location": "value",
                "--mysql-rdbms-file": "value",
                "--oracle-rdbms-file": "value",
                "--postgresql-rdbms-file": "value",
                "--recursive": "bool",
                "--recursive-depth": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--bucket": "value",
                "--ca-certificate": "value",
                "--clear-labels": "bool",
                "--client-certificate": "value",
                "--client-key": "value",
                "--database-service": "value",
                "--display-name": "value",
                "--force": "bool",
                "--forward-ssh-hostname": "value",
                "--forward-ssh-password": "value",
                "--forward-ssh-port": "value",
                "--forward-ssh-private-key": "value",
                "--forward-ssh-username": "value",
                "--location": "value",
                "--mysql-hostname": "value",
                "--mysql-password": "value",
                "--mysql-port": "value",
                "--mysql-prompt-for-password": "bool",
                "--mysql-username": "value",
                "--oracle-hostname": "value",
                "--oracle-password": "value",
                "--oracle-port": "value",
                "--oracle-prompt-for-password": "bool",
                "--oracle-username": "value",
                "--postgresql-database": "value",
                "--postgresql-hostname": "value",
                "--postgresql-password": "value",
                "--postgresql-port": "value",
                "--postgresql-prompt-for-password": "bool",
                "--postgresql-username": "value",
                "--private-connection": "value",
                "--remove-labels": "value",
                "--root-path": "value",
                "--static-ip-connectivity": "bool",
                "--type": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "fetch-static-ips": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "objects": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--stream": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--stream": "value",
                "--uri": "bool"
              }
            },
            "lookup": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--mysql-database": "value",
                "--mysql-table": "value",
                "--oracle-schema": "value",
                "--oracle-table": "value",
                "--postgresql-schema": "value",
                "--postgresql-table": "value",
                "--stream": "value"
              }
            },
            "start-backfill": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--stream": "dynamic"
              }
            },
            "stop-backfill": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--stream": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "private-connections": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--display-name": "value",
                "--labels": "value",
                "--location": "value",
                "--subnet": "value",
                "--vpc": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "routes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--destination-address": "value",
                "--destination-port": "value",
                "--display-name": "value",
                "--labels": "value",
                "--location": "value",
                "--private-connection": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--private-connection": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--private-connection": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--private-connection": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "streams": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--backfill-all": "bool",
                "--backfill-none": "bool",
                "--bigquery-destination-config": "value",
                "--destination": "value",
                "--display-name": "value",
                "--force": "bool",
                "--gcs-destination-config": "value",
                "--labels": "value",
                "--location": "value",
                "--mysql-excluded-objects": "value",
                "--mysql-source-config": "value",
                "--oracle-excluded-objects": "value",
                "--oracle-source-config": "value",
                "--postgresql-excluded-objects": "value",
                "--postgresql-source-config": "value",
                "--source": "value",
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--backfill-all": "bool",
                "--backfill-none": "bool",
                "--bigquery-destination-config": "value",
                "--clear-labels": "bool",
                "--destination": "value",
                "--display-name": "value",
                "--force": "bool",
                "--gcs-destination-config": "value",
                "--location": "value",
                "--mysql-excluded-objects": "value",
                "--mysql-source-config": "value",
                "--oracle-excluded-objects": "value",
                "--oracle-source-config": "value",
                "--postgresql-excluded-objects": "value",
                "--postgresql-source-config": "value",
                "--remove-labels": "value",
                "--source": "value",
                "--state": "value",
                "--update-labels": "value",
                "--update-mask": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "debug": {
      "commands": {
        "logpoints": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--log-level": [
                  "error",
                  "info",
                  "warning"
                ],
                "--target": "value",
                "--wait": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--all-users": "bool",
                "--include-inactive": "bool",
                "--location": "value",
                "--target": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--all-users": "bool",
                "--filter": "value",
                "--include-inactive": "value",
                "--limit": "value",
                "--location": "value",
                "--sort-by": "value",
                "--target": "value"
              }
            }
          },
          "flags": {
            "--target": "value"
          }
        },
        "snapshots": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--expression": "value",
                "--target": "value",
                "--wait": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--all-users": "bool",
                "--include-inactive": "bool",
                "--location": "value",
                "--target": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--target": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--all-users": "bool",
                "--filter": "value",
                "--include-inactive": "value",
                "--limit": "value",
                "--location": "value",
                "--sort-by": "value",
                "--target": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--all-users": "bool",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--sort-by": "value",
                "--target": "value",
                "--timeout": "value"
              }
            }
          },
          "flags": {
            "--target": "value"
          }
        },
        "source": {
          "commands": {
            "gen-repo-info-file": {
              "commands": {},
              "flags": {
                "--output-directory": "value",
                "--source-directory": "value"
              }
            }
          },
          "flags": {}
        },
        "targets": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--include-inactive": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "deploy": {
      "commands": {
        "apply": {
          "commands": {},
          "flags": {
            "--file": "value",
            "--region": "value"
          }
        },
        "delete": {
          "commands": {},
          "flags": {
            "--file": "value",
            "--force": "bool",
            "--region": "value"
          }
        },
        "delivery-pipelines": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--region": "dynamic",
                "--role": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--region": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--region": "dynamic",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "job-runs": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic",
                "--release": "dynamic",
                "--rollout": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--release": "dynamic",
                "--rollout": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "releases": {
          "commands": {
            "abandon": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--build-artifacts": "value",
                "--delivery-pipeline": "dynamic",
                "--description": "value",
                "--disable-initial-rollout": "bool",
                "--enable-initial-rollout": "bool",
                "--from-k8s-manifest": "value",
                "--from-run-manifest": "value",
                "--gcs-source-staging-dir": "value",
                "--ignore-file": "value",
                "--images": "value",
                "--initial-rollout-annotations": "value",
                "--initial-rollout-labels": "value",
                "--labels": "value",
                "--region": "dynamic",
                "--skaffold-file": "value",
                "--skaffold-version": "value",
                "--source": "value",
                "--to-target": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "promote": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--delivery-pipeline": "dynamic",
                "--labels": "value",
                "--region": "dynamic",
                "--release": "dynamic",
                "--rollout-id": "value",
                "--to-target": "value"
              }
            }
          },
          "flags": {}
        },
        "rollouts": {
          "commands": {
            "approve": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic",
                "--release": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic",
                "--release": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--release": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "reject": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--region": "dynamic",
                "--release": "dynamic"
              }
            },
            "retry-job": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "dynamic",
                "--job-id": "value",
                "--phase-id": "value",
                "--region": "dynamic",
                "--release": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "targets": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--region": "dynamic",
                "--role": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--delivery-pipeline": "value",
                "--list-all-pipelines": "bool",
                "--region": "dynamic",
                "--skip-pipeline-lookup": "bool"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--region": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "redeploy": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--delivery-pipeline": "value",
                "--description": "value",
                "--labels": "value",
                "--region": "dynamic",
                "--rollout-id": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--region": "dynamic",
                "--role": "dynamic"
              }
            },
            "rollback": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--delivery-pipeline": "value",
                "--description": "value",
                "--labels": "value",
                "--region": "dynamic",
                "--release": "value",
                "--rollout-id": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "deployment-manager": {
      "commands": {
        "deployments": {
          "commands": {
            "cancel-preview": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--fingerprint": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--automatic-rollback-on-error": "bool",
                "--composite-type": "value",
                "--config": "value",
                "--description": "value",
                "--labels": "value",
                "--preview": "bool",
                "--properties": "value",
                "--template": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--delete-policy": [
                  "abandon",
                  "delete"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--simple-list": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--fingerprint": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--composite-type": "value",
                "--config": "value",
                "--create-policy": [
                  "acquire",
                  "create-or-acquire"
                ],
                "--delete-policy": [
                  "abandon",
                  "delete"
                ],
                "--description": "value",
                "--fingerprint": "value",
                "--preview": "bool",
                "--properties": "value",
                "--remove-labels": "value",
                "--template": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "manifests": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--deployment": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--deployment": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--simple-list": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--simple-list": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "resources": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--deployment": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--deployment": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--simple-list": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {
            "--deployment": "value"
          }
        },
        "types": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "dns": {
      "commands": {
        "dns-keys": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "managed-zones": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--denial-of-existence": [
                  "nsec",
                  "nsec3"
                ],
                "--description": "value",
                "--dns-name": "value",
                "--dnssec-state": [
                  "off",
                  "on",
                  "transfer"
                ],
                "--forwarding-targets": "value",
                "--gkeclusters": "value",
                "--ksk-algorithm": [
                  "ecdsap256sha256",
                  "ecdsap384sha384",
                  "rsasha1",
                  "rsasha256",
                  "rsasha512"
                ],
                "--ksk-key-length": "value",
                "--labels": "value",
                "--location": "value",
                "--log-dns-queries": "bool",
                "--managed-reverse-lookup": "bool",
                "--networks": "value",
                "--private-forwarding-targets": "value",
                "--service-directory-namespace": "value",
                "--target-network": "value",
                "--target-project": "value",
                "--visibility": [
                  "private",
                  "public"
                ],
                "--zsk-algorithm": [
                  "ecdsap256sha256",
                  "ecdsap384sha384",
                  "rsasha1",
                  "rsasha256",
                  "rsasha512"
                ],
                "--zsk-key-length": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--policy-file": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--denial-of-existence": [
                  "nsec",
                  "nsec3"
                ],
                "--description": "value",
                "--dnssec-state": [
                  "off",
                  "on",
                  "transfer"
                ],
                "--forwarding-targets": "value",
                "--gkeclusters": "value",
                "--ksk-algorithm": [
                  "ecdsap256sha256",
                  "ecdsap384sha384",
                  "rsasha1",
                  "rsasha256",
                  "rsasha512"
                ],
                "--ksk-key-length": "value",
                "--location": "value",
                "--log-dns-queries": "bool",
                "--managed-reverse-lookup": "bool",
                "--networks": "value",
                "--private-forwarding-targets": "value",
                "--remove-labels": "value",
                "--target-network": "value",
                "--target-project": "value",
                "--update-labels": "value",
                "--zsk-algorithm": [
                  "ecdsap256sha256",
                  "ecdsap384sha384",
                  "rsasha1",
                  "rsasha256",
                  "rsasha512"
                ],
                "--zsk-key-length": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value",
                "--zones": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--alternative-name-servers": "value",
                "--description": "value",
                "--enable-inbound-forwarding": "bool",
                "--enable-logging": "bool",
                "--networks": "value",
                "--private-alternative-name-servers": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--alternative-name-servers": "value",
                "--description": "value",
                "--enable-inbound-forwarding": "bool",
                "--enable-logging": "bool",
                "--networks": "value",
                "--private-alternative-name-servers": "value"
              }
            }
          },
          "flags": {}
        },
        "project-info": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "record-sets": {
          "commands": {
            "changes": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--sort-order": [
                      "ascending",
                      "descending"
                    ],
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--backup-data-trickle-ratio": "value",
                "--enable-geo-fencing": "bool",
                "--enable-health-checking": "bool",
                "--location": "value",
                "--routing-policy-backup-data": "value",
                "--routing-policy-backup-data-type": [
                  "GEO"
                ],
                "--routing-policy-data": "value",
                "--routing-policy-primary-data": "value",
                "--routing-policy-type": [
                  "FAILOVER",
                  "GEO",
                  "WRR"
                ],
                "--rrdatas": "value",
                "--ttl": "value",
                "--type": "value",
                "--zone": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--type": "value",
                "--zone": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--type": "value",
                "--zone": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--zone": "dynamic",
                "--zone-file-format": "bool"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--delete-all-existing": "bool",
                "--location": "value",
                "--replace-origin-ns": "bool",
                "--zone": "dynamic",
                "--zone-file-format": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--name": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--type": "value",
                "--uri": "bool",
                "--zone": "dynamic"
              }
            },
            "transaction": {
              "commands": {
                "abort": {
                  "commands": {},
                  "flags": {
                    "--transaction-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "add": {
                  "commands": {},
                  "flags": {
                    "--name": "value",
                    "--transaction-file": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--transaction-file": "value",
                    "--zone": "dynamic"
                  }
                },
                "execute": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--transaction-file": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "remove": {
                  "commands": {},
                  "flags": {
                    "--name": "value",
                    "--transaction-file": "value",
                    "--ttl": "value",
                    "--type": "value",
                    "--zone": "dynamic"
                  }
                },
                "start": {
                  "commands": {},
                  "flags": {
                    "--transaction-file": "value",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {
                "--transaction-file": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--backup-data-trickle-ratio": "value",
                "--enable-geo-fencing": "bool",
                "--enable-health-checking": "bool",
                "--location": "value",
                "--routing-policy-backup-data": "value",
                "--routing-policy-backup-data-type": [
                  "GEO"
                ],
                "--routing-policy-data": "value",
                "--routing-policy-primary-data": "value",
                "--routing-policy-type": [
                  "FAILOVER",
                  "GEO",
                  "WRR"
                ],
                "--rrdatas": "value",
                "--ttl": "value",
                "--type": "value",
                "--zone": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "response-policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--gkeclusters": "value",
                "--location": "value",
                "--networks": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "rules": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--behavior": [
                      "behaviorUnspecified",
                      "bypassResponsePolicy"
                    ],
                    "--dns-name": "value",
                    "--local-data": "value",
                    "--location": "value",
                    "--response-policy": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--response-policy": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--response-policy": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--behavior": [
                      "behaviorUnspecified",
                      "bypassResponsePolicy"
                    ],
                    "--dns-name": "value",
                    "--local-data": "value",
                    "--location": "value",
                    "--response-policy": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--gkeclusters": "value",
                "--location": "value",
                "--networks": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "docker": {
      "commands": {},
      "flags": {
        "--authorize-only": "bool",
        "--docker-host": "value",
        "--server": "value"
      }
    },
    "domains": {
      "commands": {
        "list-user-verified": {
          "commands": {},
          "flags": {}
        },
        "registrations": {
          "commands": {
            "authorization-code": {
              "commands": {
                "get": {
                  "commands": {},
                  "flags": {}
                },
                "reset": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "configure": {
              "commands": {
                "contacts": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--contact-data-from-file": "value",
                    "--contact-privacy": [
                      "private-contact-data",
                      "public-contact-data",
                      "redacted-contact-data"
                    ],
                    "--notices": "value",
                    "--validate-only": "bool"
                  }
                },
                "dns": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cloud-dns-zone": "value",
                    "--disable-dnssec": "bool",
                    "--dns-settings-from-file": "value",
                    "--name-servers": "value",
                    "--unsafe-dns-update": "bool",
                    "--use-google-domains-dns": "bool",
                    "--validate-only": "bool"
                  }
                },
                "management": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--transfer-lock-state": [
                      "locked",
                      "unlocked"
                    ]
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "get-register-parameters": {
              "commands": {},
              "flags": {}
            },
            "get-transfer-parameters": {
              "commands": {},
              "flags": {}
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--labels": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-importable-domains": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "register": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cloud-dns-zone": "value",
                "--contact-data-from-file": "value",
                "--contact-privacy": [
                  "private-contact-data",
                  "public-contact-data",
                  "redacted-contact-data"
                ],
                "--disable-dnssec": "bool",
                "--labels": "value",
                "--name-servers": "value",
                "--notices": "value",
                "--use-google-domains-dns": "bool",
                "--validate-only": "bool",
                "--yearly-price": "value"
              }
            },
            "search-domains": {
              "commands": {},
              "flags": {}
            },
            "transfer": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--authorization-code-from-file": "value",
                "--cloud-dns-zone": "value",
                "--contact-data-from-file": "value",
                "--contact-privacy": [
                  "private-contact-data",
                  "public-contact-data",
                  "redacted-contact-data"
                ],
                "--keep-dns-settings": "bool",
                "--labels": "value",
                "--notices": "value",
                "--use-google-domains-dns": "bool",
                "--validate-only": "bool",
                "--yearly-price": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "verify": {
          "commands": {},
          "flags": {}
        }
      },
      "flags": {}
    },
    "edge-cache": {
      "commands": {
        "keysets": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--labels": "value",
                "--location": "value",
                "--public-key": "value",
                "--validation-shared-key": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--labels": "value",
                "--location": "value",
                "--public-key": "value",
                "--validation-shared-key": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "origins": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--failover-origin": "value",
                "--labels": "value",
                "--location": "value",
                "--max-attempts": "value",
                "--origin-address": "value",
                "--port": "value",
                "--protocol": [
                  "http",
                  "http2",
                  "https"
                ],
                "--response-timeout": "value",
                "--retry-conditions": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--failover-origin": "value",
                "--labels": "value",
                "--location": "value",
                "--max-attempts": "value",
                "--origin-address": "value",
                "--port": "value",
                "--protocol": [
                  "http",
                  "http2",
                  "https"
                ],
                "--response-timeout": "value",
                "--retry-conditions": "value"
              }
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "invalidate-cache": {
              "commands": {},
              "flags": {
                "--host": "value",
                "--location": "value",
                "--path": "value",
                "--tags": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--edge-security-policy": "value",
                "--edge-ssl-certificate": "value",
                "--enable-logging": "bool",
                "--labels": "value",
                "--location": "value",
                "--logging-sample-rate": "value",
                "--require-tls": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "edge-cloud": {
      "commands": {
        "container": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--admin-users": "value",
                    "--async": "bool",
                    "--cluster-ipv4-cidr": "value",
                    "--default-max-pods-per-node": "value",
                    "--fleet-project": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value",
                    "--services-ipv4-cidr": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "get-credentials": {
                  "commands": {},
                  "flags": {
                    "--auth-provider-cmd-path": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "node-pools": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cluster": "dynamic",
                        "--labels": "value",
                        "--local-disk-kms-key": "value",
                        "--location": "dynamic",
                        "--machine-filter": "value",
                        "--node-count": "value",
                        "--node-location": "value"
                      }
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--cluster": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--cluster": "dynamic",
                        "--location": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--cluster": "dynamic",
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    },
                    "update": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--clear-labels": "bool",
                        "--cluster": "dynamic",
                        "--location": "dynamic",
                        "--machine-filter": "value",
                        "--node-count": "value",
                        "--remove-labels": "value",
                        "--update-labels": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-maintenance-window": "bool",
                    "--location": "dynamic",
                    "--maintenance-window-end": "value",
                    "--maintenance-window-recurrence": "value",
                    "--maintenance-window-start": "value"
                  }
                }
              },
              "flags": {}
            },
            "machines": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "regions": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "vpn-connections": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--cluster": "value",
                    "--high-availability": "bool",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--nat-gateway-ip": "value",
                    "--vpc-network": "value",
                    "--vpc-project": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "networking": {
          "commands": {
            "interconnects": {
              "commands": {
                "attachments": {
                  "commands": {
                    "dedicated": {
                      "commands": {
                        "create": {
                          "commands": {},
                          "flags": {
                            "--async": "bool",
                            "--description": "value",
                            "--interconnect": "value",
                            "--labels": "value",
                            "--location": "dynamic",
                            "--mtu": "value",
                            "--network": "value",
                            "--vlan-id": "value",
                            "--zone": "dynamic"
                          }
                        }
                      },
                      "flags": {}
                    },
                    "delete": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--zone": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--sort-by": "value",
                        "--uri": "bool",
                        "--zone": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-diagnostics": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "networks": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--mtu": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "routers": {
              "commands": {
                "add-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--interface": "value",
                    "--location": "dynamic",
                    "--peer-asn": "value",
                    "--peer-ipv4-range": "value",
                    "--peer-name": "value",
                    "--zone": "dynamic"
                  }
                },
                "add-interface": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--interconnect-attachment": "value",
                    "--interface-name": "value",
                    "--ip-address": "value",
                    "--ip-mask-length": "value",
                    "--location": "dynamic",
                    "--loopback-ip-addresses": "value",
                    "--subnetwork": "value",
                    "--zone": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--asn": "value",
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--network": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "get-status": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                },
                "remove-bgp-peer": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--peer-name": "value",
                    "--peer-names": "value",
                    "--zone": "dynamic"
                  }
                },
                "remove-interface": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--interface-name": "value",
                    "--interface-names": "value",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "subnets": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--ipv4-range": "value",
                    "--ipv6-range": "value",
                    "--labels": "value",
                    "--location": "dynamic",
                    "--network": "value",
                    "--vlan-id": "value",
                    "--zone": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--zone": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--zone": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "zones": {
              "commands": {
                "init": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "emulators": {
      "commands": {
        "firestore": {
          "commands": {
            "start": {
              "commands": {},
              "flags": {
                "--host-port": "value",
                "--rules": "value"
              }
            }
          },
          "flags": {}
        },
        "spanner": {
          "commands": {
            "env-init": {
              "commands": {},
              "flags": {}
            },
            "start": {
              "commands": {},
              "flags": {
                "--enable-fault-injection": "value",
                "--host-port": "value",
                "--rest-port": "value",
                "--use-docker": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "endpoints": {
      "commands": {
        "configs": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--service": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--service": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--service": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "check-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "deploy": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool",
                "--validate-only": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "essential-contacts": {
      "commands": {
        "compute": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--notification-categories": "value",
            "--organization": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "create": {
          "commands": {},
          "flags": {
            "--email": "value",
            "--folder": "value",
            "--language": "value",
            "--notification-categories": "value",
            "--organization": "value"
          }
        },
        "delete": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--organization": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--organization": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--organization": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "update": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--language": "value",
            "--notification-categories": "value",
            "--organization": "value"
          }
        }
      },
      "flags": {}
    },
    "eventarc": {
      "commands": {
        "audit-logs-provider": {
          "commands": {
            "method-names": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--service-name": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "service-names": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "channel-connections": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--activation-token": "value",
                "--async": "bool",
                "--channel": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "channels": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--crypto-key": "value",
                "--location": "value",
                "--provider": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-crypto-key": "bool",
                "--crypto-key": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "google-channels": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-crypto-key": "bool",
                "--crypto-key": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "providers": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--name": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "triggers": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--channel": "value",
                "--destination-gke-cluster": "value",
                "--destination-gke-location": "value",
                "--destination-gke-namespace": "value",
                "--destination-gke-path": "value",
                "--destination-gke-service": "value",
                "--destination-run-path": "value",
                "--destination-run-region": "value",
                "--destination-run-service": "value",
                "--event-filters": "value",
                "--event-filters-path-pattern": "value",
                "--location": "value",
                "--service-account": "value",
                "--transport-topic": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-destination-gke-path": "bool",
                "--clear-destination-run-path": "bool",
                "--clear-service-account": "bool",
                "--destination-gke-namespace": "value",
                "--destination-gke-path": "value",
                "--destination-gke-service": "value",
                "--destination-run-path": "value",
                "--destination-run-region": "value",
                "--destination-run-service": "value",
                "--event-filters": "value",
                "--event-filters-path-pattern": "value",
                "--location": "value",
                "--service-account": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "feedback": {
      "commands": {},
      "flags": {
        "--log-file": "value"
      }
    },
    "filestore": {
      "commands": {
        "backups": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--file-share": "value",
                "--instance": "value",
                "--instance-zone": "value",
                "--labels": "value",
                "--region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--description": "value",
                "--region": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--file-share": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--location": "value",
                "--network": "value",
                "--region": "value",
                "--tier": [
                  "basic-hdd",
                  "basic-ssd",
                  "enterprise",
                  "high-scale-ssd",
                  "premium",
                  "standard"
                ],
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool",
                "--location": "value",
                "--region": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--region": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "value"
              }
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--file-share": "value",
                "--source-backup": "value",
                "--source-backup-region": "value",
                "--zone": "dynamic"
              }
            },
            "snapshots": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-region": "value",
                    "--labels": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-region": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-region": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--instance": "value",
                    "--instance-location": "value",
                    "--instance-region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--description": "value",
                "--file-share": "value",
                "--location": "value",
                "--region": "value",
                "--remove-labels": "value",
                "--update-labels": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "zones": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "firebase": {
      "commands": {
        "test": {
          "commands": {
            "android": {
              "commands": {
                "list-device-capacities": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "locales": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "models": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--additional-apks": "value",
                    "--app": "value",
                    "--app-package": "value",
                    "--async": "bool",
                    "--auto-google-login": "bool",
                    "--client-details": "value",
                    "--device": "value",
                    "--device-ids": "value",
                    "--directories-to-pull": "value",
                    "--environment-variables": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--locales": "value",
                    "--network-profile": "value",
                    "--num-flaky-test-attempts": "value",
                    "--obb-files": "value",
                    "--orientations": "dynamic",
                    "--os-version-ids": "value",
                    "--other-files": "value",
                    "--page-size": "value",
                    "--performance-metrics": "bool",
                    "--record-video": "bool",
                    "--results-bucket": "value",
                    "--results-dir": "value",
                    "--results-history-name": "value",
                    "--robo-directives": "value",
                    "--robo-script": "value",
                    "--scenario-labels": "value",
                    "--scenario-numbers": "value",
                    "--sort-by": "value",
                    "--test": "value",
                    "--test-package": "value",
                    "--test-runner-class": "value",
                    "--test-targets": "value",
                    "--timeout": "value",
                    "--type": [
                      "game-loop",
                      "instrumentation",
                      "robo"
                    ],
                    "--use-orchestrator": "bool"
                  }
                },
                "versions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "ios": {
              "commands": {
                "list-device-capacities": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "locales": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "models": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                },
                "run": {
                  "commands": {},
                  "flags": {
                    "--app": "value",
                    "--async": "bool",
                    "--client-details": "value",
                    "--device": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--num-flaky-test-attempts": "value",
                    "--page-size": "value",
                    "--record-video": "bool",
                    "--results-bucket": "value",
                    "--results-dir": "value",
                    "--results-history-name": "value",
                    "--sort-by": "value",
                    "--test": "value",
                    "--test-special-entitlements": "bool",
                    "--timeout": "value",
                    "--type": [
                      "game-loop",
                      "robo",
                      "xctest"
                    ],
                    "--xcode-version": "value",
                    "--xctestrun-file": "value"
                  }
                },
                "versions": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {}
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--page-size": "value",
                        "--sort-by": "value"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "network-profiles": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "firestore": {
      "commands": {
        "databases": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "export": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--collection-ids": "value",
            "--database": "value",
            "--namespace-ids": "value"
          }
        },
        "fields": {
          "commands": {
            "ttls": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--collection-group": "value",
                    "--database": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--collection-group": "value",
                    "--database": "value",
                    "--disable-ttl": "bool",
                    "--enable-ttl": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "import": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--collection-ids": "value",
            "--database": "value",
            "--namespace-ids": "value"
          }
        },
        "indexes": {
          "commands": {
            "composite": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--api-scope": [
                      "any-api",
                      "datastore-mode-api"
                    ],
                    "--async": "bool",
                    "--collection-group": "value",
                    "--database": "value",
                    "--field-config": "value",
                    "--query-scope": [
                      "collection",
                      "collection-group",
                      "collection-recursive"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "fields": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--collection-group": "value",
                    "--database": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--collection-group": "value",
                    "--database": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-exemption": "bool",
                    "--collection-group": "value",
                    "--database": "value",
                    "--disable-indexes": "bool",
                    "--index": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--database": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--database": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--database": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--database": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "functions": {
      "commands": {
        "add-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--gen2": "bool",
            "--member": "value",
            "--region": "dynamic",
            "--role": "value"
          }
        },
        "add-invoker-policy-binding": {
          "commands": {},
          "flags": {
            "--member": "value",
            "--region": "dynamic"
          }
        },
        "call": {
          "commands": {},
          "flags": {
            "--cloud-event": "value",
            "--data": "value",
            "--gen2": "bool",
            "--region": "dynamic"
          }
        },
        "delete": {
          "commands": {},
          "flags": {
            "--gen2": "bool",
            "--region": "dynamic"
          }
        },
        "deploy": {
          "commands": {},
          "flags": {
            "--allow-unauthenticated": "bool",
            "--build-env-vars-file": "value",
            "--build-worker-pool": "value",
            "--clear-build-env-vars": "bool",
            "--clear-build-worker-pool": "bool",
            "--clear-docker-repository": "bool",
            "--clear-env-vars": "bool",
            "--clear-kms-key": "bool",
            "--clear-labels": "bool",
            "--clear-max-instances": "bool",
            "--clear-min-instances": "bool",
            "--clear-secrets": "bool",
            "--clear-vpc-connector": "bool",
            "--docker-registry": [
              "artifact-registry",
              "container-registry"
            ],
            "--docker-repository": "value",
            "--egress-settings": [
              "all",
              "private-ranges-only"
            ],
            "--entry-point": "value",
            "--env-vars-file": "value",
            "--gen2": "bool",
            "--ignore-file": "value",
            "--ingress-settings": [
              "all",
              "internal-and-gclb",
              "internal-only"
            ],
            "--kms-key": "value",
            "--max-instances": "value",
            "--memory": "value",
            "--min-instances": "value",
            "--region": "dynamic",
            "--remove-build-env-vars": "value",
            "--remove-env-vars": "value",
            "--remove-labels": "value",
            "--remove-secrets": "value",
            "--retry": "bool",
            "--run-service-account": "value",
            "--runtime": "value",
            "--security-level": [
              "secure-always",
              "secure-optional"
            ],
            "--serve-all-traffic-latest-revision": "bool",
            "--service-account": "value",
            "--set-build-env-vars": "value",
            "--set-env-vars": "value",
            "--set-secrets": "value",
            "--source": "value",
            "--stage-bucket": "value",
            "--timeout": "value",
            "--trigger-bucket": "value",
            "--trigger-event": "value",
            "--trigger-event-filters": "value",
            "--trigger-event-filters-path-pattern": "value",
            "--trigger-http": "bool",
            "--trigger-location": "dynamic",
            "--trigger-resource": "value",
            "--trigger-service-account": "value",
            "--trigger-topic": "value",
            "--update-build-env-vars": "value",
            "--update-env-vars": "value",
            "--update-labels": "value",
            "--update-secrets": "value",
            "--vpc-connector": "dynamic"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--gen2": "bool",
            "--region": "dynamic"
          }
        },
        "event-types": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--gen2": "bool"
              }
            }
          },
          "flags": {}
        },
        "get-iam-policy": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--gen2": "bool",
            "--limit": "value",
            "--page-size": "value",
            "--region": "dynamic",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--regions": "value",
            "--sort-by": "value"
          }
        },
        "logs": {
          "commands": {
            "read": {
              "commands": {},
              "flags": {
                "--end-time": "value",
                "--execution-id": "value",
                "--filter": "value",
                "--gen2": "bool",
                "--limit": "value",
                "--min-log-level": [
                  "debug",
                  "error",
                  "info"
                ],
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--start-time": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--gen2": "bool",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "remove-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--gen2": "bool",
            "--member": "value",
            "--region": "dynamic",
            "--role": "value"
          }
        },
        "remove-invoker-policy-binding": {
          "commands": {},
          "flags": {
            "--member": "value",
            "--region": "dynamic"
          }
        },
        "runtimes": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "set-iam-policy": {
          "commands": {},
          "flags": {
            "--gen2": "bool",
            "--region": "dynamic"
          }
        }
      },
      "flags": {}
    },
    "game": {
      "commands": {
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "servers": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--description": "value",
                    "--dry-run": "bool",
                    "--gke-cluster": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--namespace": "value",
                    "--preview-time": "value",
                    "--realm": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--dry-run": "bool",
                    "--location": "value",
                    "--preview-time": "value",
                    "--realm": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--realm": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--realm": "value",
                    "--sort-by": "value",
                    "--uri": "bool",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--dry-run": "bool",
                    "--location": "value",
                    "--preview-time": "value",
                    "--realm": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "configs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--deployment": "value",
                    "--description": "value",
                    "--fleet-configs-file": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--scaling-configs-file": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--deployment": "value",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--deployment": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "deployments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "describe-rollout": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "fetch-state": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                },
                "update-rollout": {
                  "commands": {},
                  "flags": {
                    "--clear-config-overrides": "bool",
                    "--clear-default-config": "bool",
                    "--config-overrides-file": "value",
                    "--default-config": "value",
                    "--dry-run": "bool",
                    "--location": "value",
                    "--preview-time": "value"
                  }
                }
              },
              "flags": {}
            },
            "realms": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--labels": "value",
                    "--location": "value",
                    "--time-zone": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--dry-run": "bool",
                    "--location": "value",
                    "--preview-time": "value",
                    "--remove-labels": "value",
                    "--time-zone": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "healthcare": {
      "commands": {
        "consent-stores": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "check-data-access": {
              "commands": {},
              "flags": {
                "--data-id": "value",
                "--dataset": "value",
                "--location": "value",
                "--request-attributes": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "evaluate-user-consents": {
              "commands": {},
              "flags": {
                "--consent-list": "value",
                "--dataset": "value",
                "--location": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--request-attributes": "value",
                "--resource-attributes": "value",
                "--response-view": [
                  "basic",
                  "full",
                  "response-view-unspecified"
                ],
                "--user-id": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "query-accessible-data": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--dataset": "value",
                "--gcs-uri": "value",
                "--location": "value",
                "--request-attributes": "value",
                "--resource-attributes": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--dataset": "value",
                "--location": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "datasets": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--time-zone": "value"
              }
            },
            "deidentify": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--default-fhir-config": "bool",
                "--destination-dataset": "value",
                "--dicom-filter-tags": "value",
                "--location": "value",
                "--text-redaction-mode": [
                  "all"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--time-zone": "value"
              }
            }
          },
          "flags": {}
        },
        "dicom-stores": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value",
                "--pubsub-topic": "value"
              }
            },
            "deidentify": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--dataset": "value",
                "--destination-store": "value",
                "--dicom-filter-tags": "value",
                "--location": "value",
                "--text-redaction-mode": [
                  "all"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "export": {
              "commands": {
                "bq": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bq-table": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--overwrite-table": "bool",
                    "--write-disposition": [
                      "write-append",
                      "write-empty",
                      "write-truncate"
                    ]
                  }
                },
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri-prefix": "value",
                    "--location": "value",
                    "--mime-type": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value",
                "--pubsub-topic": "value"
              }
            }
          },
          "flags": {}
        },
        "fhir-stores": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--disable-referential-integrity": "bool",
                "--disable-resource-versioning": "bool",
                "--enable-update-create": "bool",
                "--location": "value",
                "--pubsub-topic": "value"
              }
            },
            "deidentify": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--dataset": "value",
                "--destination-store": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "export": {
              "commands": {
                "bq": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bq-dataset": "value",
                    "--dataset": "value",
                    "--location": "value",
                    "--recursive-depth": "value",
                    "--resource-type": "value",
                    "--schema-type": [
                      "analytics",
                      "analytics_v2"
                    ],
                    "--since": "value",
                    "--write-disposition": [
                      "write-append",
                      "write-empty",
                      "write-truncate"
                    ]
                  }
                },
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value",
                    "--resource-type": "value",
                    "--since": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--content-structure": [
                      "bundle",
                      "bundle-pretty",
                      "resource",
                      "resource-pretty"
                    ],
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--enable-update-create": "bool",
                "--location": "value",
                "--pubsub-topic": "value"
              }
            }
          },
          "flags": {}
        },
        "hl7v2-stores": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value",
                "--notification-config": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "export": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--end-time": "value",
                    "--gcs-uri": "value",
                    "--location": "value",
                    "--message-view": [
                      "basic",
                      "full",
                      "parsed-only",
                      "raw-only",
                      "schematized-only"
                    ],
                    "--start-time": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--dataset": "value",
                    "--gcs-uri": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--dataset": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value",
                "--notification-config": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "help": {
      "commands": {},
      "flags": {
        "--filter": "value",
        "--limit": "value",
        "--page-size": "value",
        "--sort-by": "value"
      }
    },
    "iam": {
      "commands": {
        "list-grantable-roles": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--page-size": "value"
          }
        },
        "list-testable-permissions": {
          "commands": {},
          "flags": {
            "--filter": "value"
          }
        },
        "policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--attachment-point": "value",
                "--kind": "value",
                "--policy-file": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--attachment-point": "value",
                "--etag": "value",
                "--kind": "value"
              }
            },
            "get": {
              "commands": {},
              "flags": {
                "--attachment-point": "value",
                "--kind": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--attachment-point": "value",
                "--filter": "value",
                "--kind": "value",
                "--limit": "value",
                "--page-size": "value",
                "--page_token": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--attachment-point": "value",
                "--kind": "value",
                "--policy-file": "value"
              }
            }
          },
          "flags": {}
        },
        "roles": {
          "commands": {
            "copy": {
              "commands": {},
              "flags": {
                "--dest-organization": "value",
                "--dest-project": "value",
                "--destination": "value",
                "--source": "value",
                "--source-organization": "value",
                "--source-project": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--file": "value",
                "--organization": "value",
                "--permissions": "value",
                "--stage": "value",
                "--title": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--show-deleted": "bool",
                "--sort-by": "value"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-permissions": "value",
                "--description": "value",
                "--file": "value",
                "--organization": "value",
                "--permissions": "value",
                "--remove-permissions": "value",
                "--stage": "value",
                "--title": "value"
              }
            }
          },
          "flags": {}
        },
        "service-accounts": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "disable": {
              "commands": {},
              "flags": {}
            },
            "enable": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "keys": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value",
                    "--key-file-type": [
                      "json",
                      "p12"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "value"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "dynamic"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--created-before": "value",
                    "--filter": "value",
                    "--iam-account": "value",
                    "--limit": "value",
                    "--managed-by": [
                      "any",
                      "system",
                      "user"
                    ],
                    "--page-size": "value",
                    "--sort-by": "value"
                  }
                },
                "upload": {
                  "commands": {},
                  "flags": {
                    "--iam-account": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "sign-blob": {
              "commands": {},
              "flags": {
                "--iam-account": "value"
              }
            },
            "sign-jwt": {
              "commands": {},
              "flags": {
                "--iam-account": "value"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--display-name": "value"
              }
            }
          },
          "flags": {}
        },
        "simulator": {
          "commands": {
            "replay-recent-access": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "workforce-pools": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--disabled": "bool",
                "--display-name": "value",
                "--location": "value",
                "--organization": "value",
                "--session-duration": "value"
              }
            },
            "create-cred-config": {
              "commands": {},
              "flags": {
                "--credential-source-field-name": "value",
                "--credential-source-file": "value",
                "--credential-source-headers": "value",
                "--credential-source-type": "value",
                "--credential-source-url": "value",
                "--executable-command": "value",
                "--executable-interactive-timeout-millis": "value",
                "--executable-output-file": "value",
                "--executable-timeout-millis": "value",
                "--output-file": "value",
                "--service-account": "value",
                "--service-account-token-lifetime-seconds": "value",
                "--subject-token-type": "value",
                "--workforce-pool-user-project": "value"
              }
            },
            "create-login-config": {
              "commands": {},
              "flags": {
                "--activate": "bool",
                "--output-file": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "value",
                "--page-size": "value",
                "--show-deleted": "bool",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "providers": {
              "commands": {
                "create-oidc": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--client-id": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--issuer-uri": "value",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "create-saml": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--idp-metadata-path": "value",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--provider": "dynamic",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "update-oidc": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--client-id": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--issuer-uri": "value",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "update-saml": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--idp-metadata-path": "value",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "subjects": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--subject": "value",
                        "--workforce-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--workforce-pool": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--disabled": "bool",
                "--display-name": "value",
                "--location": "value",
                "--session-duration": "value"
              }
            }
          },
          "flags": {}
        },
        "workload-identity-pools": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--disabled": "bool",
                "--display-name": "value",
                "--location": "value"
              }
            },
            "create-cred-config": {
              "commands": {},
              "flags": {
                "--app-id-uri": "value",
                "--aws": "bool",
                "--azure": "bool",
                "--credential-source-field-name": "value",
                "--credential-source-file": "value",
                "--credential-source-headers": "value",
                "--credential-source-type": "value",
                "--credential-source-url": "value",
                "--enable-imdsv2": "bool",
                "--executable-command": "value",
                "--executable-output-file": "value",
                "--executable-timeout-millis": "value",
                "--output-file": "value",
                "--service-account": "value",
                "--service-account-token-lifetime-seconds": "value",
                "--subject-token-type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--show-deleted": "bool",
                "--sort-by": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "providers": {
              "commands": {
                "create-aws": {
                  "commands": {},
                  "flags": {
                    "--account-id": "value",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "create-oidc": {
                  "commands": {},
                  "flags": {
                    "--allowed-audiences": "value",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--issuer-uri": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "create-saml": {
                  "commands": {},
                  "flags": {
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--idp-metadata-path": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--show-deleted": "bool",
                    "--sort-by": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "operations": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "value",
                        "--provider": "dynamic",
                        "--workload-identity-pool": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                },
                "undelete": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "update-aws": {
                  "commands": {},
                  "flags": {
                    "--account-id": "value",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "update-oidc": {
                  "commands": {},
                  "flags": {
                    "--allowed-audiences": "value",
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--issuer-uri": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                },
                "update-saml": {
                  "commands": {},
                  "flags": {
                    "--attribute-condition": "value",
                    "--attribute-mapping": "value",
                    "--description": "value",
                    "--disabled": "bool",
                    "--display-name": "value",
                    "--idp-metadata-path": "value",
                    "--location": "value",
                    "--workload-identity-pool": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--disabled": "bool",
                "--display-name": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "iap": {
      "commands": {
        "oauth-brands": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--application_title": "value",
                "--support_email": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "oauth-clients": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--display_name": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--brand": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--brand": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "reset-secret": {
              "commands": {},
              "flags": {
                "--brand": "value"
              }
            }
          },
          "flags": {}
        },
        "settings": {
          "commands": {
            "get": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value",
                "--resource-type": [
                  "app-engine",
                  "compute",
                  "folder",
                  "iap_web",
                  "organization"
                ],
                "--service": "value"
              }
            },
            "set": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value",
                "--resource-type": [
                  "app-engine",
                  "compute",
                  "folder",
                  "iap_web",
                  "organization"
                ],
                "--service": "value"
              }
            }
          },
          "flags": {}
        },
        "tcp": {
          "commands": {
            "dest-groups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dest-group": "value",
                    "--member": "value",
                    "--region": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--fqdn-list": "value",
                    "--ip-range-list": "value",
                    "--region": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dest-group": "value",
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--sort-by": "value"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--dest-group": "value",
                    "--member": "value",
                    "--region": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--dest-group": "value",
                    "--region": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--fqdn-list": "value",
                    "--ip-range-list": "value",
                    "--region": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "web": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--role": "value",
                "--service": "value"
              }
            },
            "disable": {
              "commands": {},
              "flags": {
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--service": "value"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--oauth2-client-id": "value",
                "--oauth2-client-secret": "value",
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--service": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--service": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--role": "value",
                "--service": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--resource-type": [
                  "app-engine",
                  "backend-services"
                ],
                "--service": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "identity": {
      "commands": {
        "groups": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--customer": "value",
                "--description": "value",
                "--display-name": "value",
                "--dynamic-user-query": "value",
                "--group-type": [
                  "discussion",
                  "dynamic",
                  "security"
                ],
                "--labels": "value",
                "--organization": "value",
                "--with-initial-owner": [
                  "empty",
                  "with-initial-owner"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "memberships": {
              "commands": {
                "add": {
                  "commands": {},
                  "flags": {
                    "--expiration": "value",
                    "--group-email": "value",
                    "--member-email": "value",
                    "--roles": "value"
                  }
                },
                "check-transitive-membership": {
                  "commands": {},
                  "flags": {
                    "--group-email": "value",
                    "--member-email": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--group-email": "value",
                    "--member-email": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--group-email": "value",
                    "--member-email": "value"
                  }
                },
                "get-membership-graph": {
                  "commands": {},
                  "flags": {
                    "--group-email": "value",
                    "--labels": "value",
                    "--member-email": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--group-email": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--page-token": "value",
                    "--sort-by": "value",
                    "--view": [
                      "basic",
                      "full"
                    ]
                  }
                },
                "modify-membership-roles": {
                  "commands": {},
                  "flags": {
                    "--add-roles": "value",
                    "--group-email": "value",
                    "--member-email": "value",
                    "--remove-roles": "value",
                    "--update-roles-params": "value"
                  }
                },
                "search-transitive-groups": {
                  "commands": {},
                  "flags": {
                    "--labels": "value",
                    "--member-email": "value",
                    "--page-size": "value",
                    "--page-token": "value"
                  }
                },
                "search-transitive-memberships": {
                  "commands": {},
                  "flags": {
                    "--group-email": "value",
                    "--page-size": "value",
                    "--page-token": "value"
                  }
                }
              },
              "flags": {}
            },
            "search": {
              "commands": {},
              "flags": {
                "--customer": "value",
                "--labels": "value",
                "--organization": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--view": [
                  "basic",
                  "full"
                ]
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-description": "bool",
                "--clear-display-name": "bool",
                "--description": "value",
                "--display-name": "value",
                "--dynamic-user-query": "value",
                "--labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "ids": {
      "commands": {
        "endpoints": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--enable-traffic-logs": "bool",
                "--labels": "value",
                "--max-wait": "value",
                "--network": "value",
                "--severity": [
                  "CRITICAL",
                  "HIGH",
                  "INFORMATIONAL",
                  "LOW",
                  "MEDIUM"
                ],
                "--threat-exceptions": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--max-wait": "value",
                "--zone": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--zone": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--max-wait": "value",
                "--threat-exceptions": "value",
                "--zone": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "immersive-stream": {
      "commands": {
        "xr": {
          "commands": {
            "contents": {
              "commands": {
                "build": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--location": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--bucket": "value",
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "instances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--add-region": "value",
                    "--async": "bool",
                    "--content": "value",
                    "--fallback-url": "value",
                    "--location": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--add-region": "value",
                    "--async": "bool",
                    "--fallback-url": "value",
                    "--location": "value",
                    "--remove-region": "value",
                    "--update-region": "value"
                  }
                }
              },
              "flags": {}
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "info": {
      "commands": {},
      "flags": {
        "--anonymize": "bool",
        "--run-diagnostics": "bool",
        "--show-log": "bool"
      }
    },
    "init": {
      "commands": {},
      "flags": {
        "--console-only": "bool",
        "--no-browser": "bool",
        "--no-launch-browser": "bool",
        "--skip-diagnostics": "bool"
      }
    },
    "iot": {
      "commands": {
        "devices": {
          "commands": {
            "commands": {
              "commands": {
                "send": {
                  "commands": {},
                  "flags": {
                    "--command-data": "value",
                    "--command-file": "value",
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--subfolder": "value"
                  }
                }
              },
              "flags": {}
            },
            "configs": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "get-value": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--config-data": "value",
                    "--config-file": "value",
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--version-to-update": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--auth-method": [
                  "association-and-device-auth-token",
                  "association-only",
                  "device-auth-token-only"
                ],
                "--blocked": "bool",
                "--device-type": [
                  "gateway",
                  "non-gateway"
                ],
                "--log-level": [
                  "debug",
                  "error",
                  "info",
                  "none"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--public-key": "value",
                "--region": "value",
                "--registry": "dynamic"
              }
            },
            "credentials": {
              "commands": {
                "clear": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--expiration-time": "value",
                    "--path": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--type": [
                      "es256",
                      "es256-pem",
                      "es256-x509-pem",
                      "rs256",
                      "rsa-pem",
                      "rsa-x509-pem"
                    ]
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--expiration-time": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--registry": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value",
                "--registry": "dynamic"
              }
            },
            "gateways": {
              "commands": {
                "bind": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--device-region": "value",
                    "--device-registry": "dynamic",
                    "--gateway": "dynamic",
                    "--gateway-region": "value",
                    "--gateway-registry": "dynamic"
                  }
                },
                "list-bound-devices": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--gateway": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value"
                  }
                },
                "unbind": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--device-region": "value",
                    "--device-registry": "dynamic",
                    "--gateway": "dynamic",
                    "--gateway-region": "value",
                    "--gateway-registry": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--device-field-mask": "value",
                "--device-ids": "value",
                "--device-num-ids": "value",
                "--device-type": [
                  "gateway",
                  "non-gateway"
                ],
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--registry": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "states": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--device": "dynamic",
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--blocked": "bool",
                "--log-level": [
                  "debug",
                  "error",
                  "info",
                  "none"
                ],
                "--metadata": "value",
                "--metadata-from-file": "value",
                "--region": "value",
                "--registry": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "registries": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--enable-http-config": "bool",
                "--enable-mqtt-config": "bool",
                "--event-notification-config": "value",
                "--log-level": [
                  "debug",
                  "error",
                  "info",
                  "none"
                ],
                "--public-key-path": "value",
                "--region": "value",
                "--state-pubsub-topic": "value"
              }
            },
            "credentials": {
              "commands": {
                "clear": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--path": "value",
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--region": "value",
                    "--registry": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--region": "value",
                    "--registry": "dynamic",
                    "--sort-by": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--enable-http-config": "bool",
                "--enable-mqtt-config": "bool",
                "--event-notification-config": "value",
                "--log-level": [
                  "debug",
                  "error",
                  "info",
                  "none"
                ],
                "--region": "value",
                "--state-pubsub-topic": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "kms": {
      "commands": {
        "asymmetric-decrypt": {
          "commands": {},
          "flags": {
            "--ciphertext-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--plaintext-file": "value",
            "--skip-integrity-verification": "bool"
          }
        },
        "asymmetric-sign": {
          "commands": {},
          "flags": {
            "--digest-algorithm": [
              "sha256",
              "sha384",
              "sha512"
            ],
            "--input-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--signature-file": "value",
            "--skip-integrity-verification": "bool"
          }
        },
        "decrypt": {
          "commands": {},
          "flags": {
            "--additional-authenticated-data-file": "value",
            "--ciphertext-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--plaintext-file": "value",
            "--skip-integrity-verification": "bool"
          }
        },
        "ekm-connections": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--endpoint-filter": "value",
                "--hostname": "value",
                "--location": "dynamic",
                "--server-certificates-files": "value",
                "--service-directory-service": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--endpoint-filter": "value",
                "--hostname": "value",
                "--location": "dynamic",
                "--server-certificates-files": "value",
                "--service-directory-service": "value"
              }
            }
          },
          "flags": {}
        },
        "encrypt": {
          "commands": {},
          "flags": {
            "--additional-authenticated-data-file": "value",
            "--ciphertext-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--plaintext-file": "value",
            "--skip-integrity-verification": "bool"
          }
        },
        "import-jobs": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--keyring": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--import-method": [
                  "rsa-oaep-3072-sha1-aes-256",
                  "rsa-oaep-3072-sha256",
                  "rsa-oaep-3072-sha256-aes-256",
                  "rsa-oaep-4096-sha1-aes-256",
                  "rsa-oaep-4096-sha256",
                  "rsa-oaep-4096-sha256-aes-256"
                ],
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--protection-level": [
                  "hsm",
                  "software"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--attestation-file": "value",
                "--keyring": "dynamic",
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--keyring": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--keyring": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--keyring": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--keyring": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "keyrings": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "keys": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--keyring": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--crypto-key-backend": "value",
                "--default-algorithm": [
                  "ec-sign-p256-sha256",
                  "ec-sign-p384-sha384",
                  "ec-sign-secp256k1-sha256",
                  "external-symmetric-encryption",
                  "google-symmetric-encryption",
                  "hmac-sha1",
                  "hmac-sha224",
                  "hmac-sha256",
                  "hmac-sha384",
                  "hmac-sha512",
                  "rsa-decrypt-oaep-2048-sha1",
                  "rsa-decrypt-oaep-2048-sha256",
                  "rsa-decrypt-oaep-3072-sha1",
                  "rsa-decrypt-oaep-3072-sha256",
                  "rsa-decrypt-oaep-4096-sha1",
                  "rsa-decrypt-oaep-4096-sha256",
                  "rsa-decrypt-oaep-4096-sha512",
                  "rsa-sign-pkcs1-2048-sha256",
                  "rsa-sign-pkcs1-3072-sha256",
                  "rsa-sign-pkcs1-4096-sha256",
                  "rsa-sign-pkcs1-4096-sha512",
                  "rsa-sign-pss-2048-sha256",
                  "rsa-sign-pss-3072-sha256",
                  "rsa-sign-pss-4096-sha256",
                  "rsa-sign-pss-4096-sha512",
                  "rsa-sign-raw-pkcs1-2048",
                  "rsa-sign-raw-pkcs1-3072",
                  "rsa-sign-raw-pkcs1-4096"
                ],
                "--destroy-scheduled-duration": "value",
                "--import-only": "bool",
                "--keyring": "dynamic",
                "--labels": "value",
                "--location": "dynamic",
                "--next-rotation-time": "value",
                "--protection-level": [
                  "external",
                  "external-vpc",
                  "hsm",
                  "software"
                ],
                "--purpose": [
                  "asymmetric-encryption",
                  "asymmetric-signing",
                  "encryption",
                  "mac"
                ],
                "--rotation-period": "value",
                "--skip-initial-version-creation": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--keyring": "dynamic",
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--keyring": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--keyring": "dynamic",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--keyring": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "remove-rotation-schedule": {
              "commands": {},
              "flags": {
                "--keyring": "dynamic",
                "--location": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--keyring": "dynamic",
                "--location": "dynamic"
              }
            },
            "set-primary-version": {
              "commands": {},
              "flags": {
                "--keyring": "dynamic",
                "--location": "dynamic"
              }
            },
            "set-rotation-schedule": {
              "commands": {},
              "flags": {
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--next-rotation-time": "value",
                "--rotation-period": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--default-algorithm": [
                  "ec-sign-p256-sha256",
                  "ec-sign-p384-sha384",
                  "ec-sign-secp256k1-sha256",
                  "external-symmetric-encryption",
                  "google-symmetric-encryption",
                  "hmac-sha1",
                  "hmac-sha224",
                  "hmac-sha256",
                  "hmac-sha384",
                  "hmac-sha512",
                  "rsa-decrypt-oaep-2048-sha1",
                  "rsa-decrypt-oaep-2048-sha256",
                  "rsa-decrypt-oaep-3072-sha1",
                  "rsa-decrypt-oaep-3072-sha256",
                  "rsa-decrypt-oaep-4096-sha1",
                  "rsa-decrypt-oaep-4096-sha256",
                  "rsa-decrypt-oaep-4096-sha512",
                  "rsa-sign-pkcs1-2048-sha256",
                  "rsa-sign-pkcs1-3072-sha256",
                  "rsa-sign-pkcs1-4096-sha256",
                  "rsa-sign-pkcs1-4096-sha512",
                  "rsa-sign-pss-2048-sha256",
                  "rsa-sign-pss-3072-sha256",
                  "rsa-sign-pss-4096-sha256",
                  "rsa-sign-pss-4096-sha512",
                  "rsa-sign-raw-pkcs1-2048",
                  "rsa-sign-raw-pkcs1-3072",
                  "rsa-sign-raw-pkcs1-4096"
                ],
                "--keyring": "dynamic",
                "--location": "dynamic",
                "--next-rotation-time": "value",
                "--primary-version": "dynamic",
                "--remove-labels": "value",
                "--remove-rotation-schedule": "bool",
                "--rotation-period": "value",
                "--update-labels": "value"
              }
            },
            "versions": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--ekm-connection-key-path": "value",
                    "--external-key-uri": "value",
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--primary": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--attestation-file": "value",
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "destroy": {
                  "commands": {},
                  "flags": {
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "disable": {
                  "commands": {},
                  "flags": {
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "enable": {
                  "commands": {},
                  "flags": {
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "get-certificate-chain": {
                  "commands": {},
                  "flags": {
                    "--certificate-chain-type": [
                      "all",
                      "cavium",
                      "google-card",
                      "google-partition"
                    ],
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--output-file": "value"
                  }
                },
                "get-public-key": {
                  "commands": {},
                  "flags": {
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--output-file": "value"
                  }
                },
                "import": {
                  "commands": {},
                  "flags": {
                    "--algorithm": [
                      "ec-sign-p256-sha256",
                      "ec-sign-p384-sha384",
                      "ec-sign-secp256k1-sha256",
                      "google-symmetric-encryption",
                      "hmac-sha1",
                      "hmac-sha224",
                      "hmac-sha256",
                      "hmac-sha384",
                      "hmac-sha512",
                      "rsa-decrypt-oaep-2048-sha1",
                      "rsa-decrypt-oaep-2048-sha256",
                      "rsa-decrypt-oaep-3072-sha1",
                      "rsa-decrypt-oaep-3072-sha256",
                      "rsa-decrypt-oaep-4096-sha1",
                      "rsa-decrypt-oaep-4096-sha256",
                      "rsa-decrypt-oaep-4096-sha512",
                      "rsa-sign-pkcs1-2048-sha256",
                      "rsa-sign-pkcs1-3072-sha256",
                      "rsa-sign-pkcs1-4096-sha256",
                      "rsa-sign-pkcs1-4096-sha512",
                      "rsa-sign-pss-2048-sha256",
                      "rsa-sign-pss-3072-sha256",
                      "rsa-sign-pss-4096-sha256",
                      "rsa-sign-pss-4096-sha512",
                      "rsa-sign-raw-pkcs1-2048",
                      "rsa-sign-raw-pkcs1-3072",
                      "rsa-sign-raw-pkcs1-4096"
                    ],
                    "--import-job": "dynamic",
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--public-key-file": "value",
                    "--target-key-file": "value",
                    "--wrapped-key-file": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "restore": {
                  "commands": {},
                  "flags": {
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--ekm-connection-key-path": "value",
                    "--external-key-uri": "value",
                    "--key": "dynamic",
                    "--keyring": "dynamic",
                    "--location": "dynamic",
                    "--state": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "mac-sign": {
          "commands": {},
          "flags": {
            "--input-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--signature-file": "value",
            "--skip-integrity-verification": "bool"
          }
        },
        "mac-verify": {
          "commands": {},
          "flags": {
            "--input-file": "value",
            "--key": "dynamic",
            "--keyring": "dynamic",
            "--location": "dynamic",
            "--signature-file": "value",
            "--skip-integrity-verification": "bool"
          }
        }
      },
      "flags": {}
    },
    "logging": {
      "commands": {
        "buckets": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cmek-kms-key-name": "value",
                "--description": "value",
                "--enable-analytics": "bool",
                "--index": "value",
                "--location": "value",
                "--restricted-fields": "value",
                "--retention-days": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-index": "value",
                "--async": "bool",
                "--clear-indexes": "bool",
                "--cmek-kms-key-name": "value",
                "--description": "value",
                "--enable-analytics": "bool",
                "--location": "value",
                "--locked": "bool",
                "--remove-indexes": "value",
                "--restricted-fields": "value",
                "--retention-days": "value",
                "--update-index": "value"
              }
            }
          },
          "flags": {}
        },
        "copy": {
          "commands": {},
          "flags": {
            "--billing-account": "value",
            "--folder": "value",
            "--location": "value",
            "--log-filter": "value",
            "--organization": "dynamic"
          }
        },
        "links": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--billing-account": "value",
                "--bucket": "value",
                "--description": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "logs": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--bucket": "value",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--sort-by": "value",
                "--view": "value"
              }
            }
          },
          "flags": {}
        },
        "metrics": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--bucket-name": "value",
                "--config-from-file": "value",
                "--description": "value",
                "--log-filter": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--bucket-name": "value",
                "--config-from-file": "value",
                "--description": "value",
                "--log-filter": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--limit": "value",
                "--location": "value",
                "--operation-filter": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "read": {
          "commands": {},
          "flags": {
            "--billing-account": "value",
            "--bucket": "value",
            "--folder": "value",
            "--freshness": "value",
            "--limit": "value",
            "--location": "value",
            "--order": [
              "asc",
              "desc"
            ],
            "--organization": "dynamic",
            "--resource-names": "value",
            "--view": "value"
          }
        },
        "resource-descriptors": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "settings": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-kms-key": "bool",
                "--disable-default-sink": "bool",
                "--folder": "value",
                "--kms-key-name": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--organization": "dynamic",
                "--storage-location": "value"
              }
            }
          },
          "flags": {}
        },
        "sinks": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--description": "value",
                "--disabled": "bool",
                "--exclusion": "value",
                "--folder": "value",
                "--include-children": "bool",
                "--log-filter": "value",
                "--organization": "dynamic",
                "--use-partitioned-tables": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "dynamic",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-exclusion": "value",
                "--billing-account": "value",
                "--clear-exclusions": "bool",
                "--description": "value",
                "--disabled": "bool",
                "--folder": "value",
                "--log-filter": "value",
                "--organization": "dynamic",
                "--remove-exclusions": "value",
                "--update-exclusion": "value",
                "--use-partitioned-tables": "bool"
              }
            }
          },
          "flags": {}
        },
        "views": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--description": "value",
                "--folder": "value",
                "--location": "value",
                "--log-filter": "value",
                "--organization": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--bucket": "value",
                "--description": "value",
                "--folder": "value",
                "--location": "value",
                "--log-filter": "value",
                "--organization": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "write": {
          "commands": {},
          "flags": {
            "--billing-account": "value",
            "--folder": "value",
            "--organization": "dynamic",
            "--payload-type": [
              "json",
              "text"
            ],
            "--severity": [
              "ALERT",
              "CRITICAL",
              "DEBUG",
              "DEFAULT",
              "EMERGENCY",
              "ERROR",
              "INFO",
              "NOTICE",
              "WARNING"
            ]
          }
        }
      },
      "flags": {}
    },
    "memcache": {
      "commands": {
        "instances": {
          "commands": {
            "apply-parameters": {
              "commands": {},
              "flags": {
                "--apply-all": "bool",
                "--async": "bool",
                "--node-ids": "value",
                "--region": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--authorized-network": "value",
                "--display-name": "value",
                "--labels": "value",
                "--maintenance-window-day": [
                  "friday",
                  "monday",
                  "saturday",
                  "sunday",
                  "thursday",
                  "tuesday",
                  "wednesday"
                ],
                "--maintenance-window-duration": "value",
                "--maintenance-window-start-time": "value",
                "--memcached-version": [
                  "1.5"
                ],
                "--node-count": "value",
                "--node-cpu": "value",
                "--node-memory": "value",
                "--parameters": "value",
                "--region": "dynamic",
                "--zones": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "reschedule-maintenance": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic",
                "--reschedule-type": [
                  "immediate",
                  "next-available-window",
                  "specific-time"
                ],
                "--schedule-time": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--display-name": "value",
                "--labels": "value",
                "--maintenance-window-any": "bool",
                "--maintenance-window-day": [
                  "friday",
                  "monday",
                  "saturday",
                  "sunday",
                  "thursday",
                  "tuesday",
                  "wednesday"
                ],
                "--maintenance-window-duration": "value",
                "--maintenance-window-start-time": "value",
                "--node-count": "value",
                "--parameters": "value",
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "metastore": {
      "commands": {
        "federations": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backends": "value",
                "--hive-metastore-version": [
                  "2.3.6",
                  "3.1.2"
                ],
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-backends": "bool",
                "--clear-labels": "bool",
                "--location": "value",
                "--remove-backends": "value",
                "--remove-labels": "value",
                "--update-backends": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "backups": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--service": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--service": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--service": "value"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--service": "value"
                  }
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--service": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--member": "value",
                    "--role": "dynamic",
                    "--service": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {
                    "--location": "value",
                    "--service": "value"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--auxiliary-versions": "value",
                "--auxiliary-versions-from-file": "value",
                "--consumer-subnetworks": "value",
                "--database-type": [
                  "mysql",
                  "spanner"
                ],
                "--encryption-kms-key": "value",
                "--hive-metastore-configs": "value",
                "--hive-metastore-configs-from-file": "value",
                "--hive-metastore-version": "value",
                "--instance-size": [
                  "extra_large",
                  "extra_small",
                  "large",
                  "medium",
                  "small"
                ],
                "--kerberos-principal": "value",
                "--keytab": "value",
                "--krb5-config": "value",
                "--labels": "value",
                "--location": "value",
                "--maintenance-window-day": [
                  "fri",
                  "mon",
                  "sat",
                  "sun",
                  "thu",
                  "tue",
                  "wed"
                ],
                "--maintenance-window-hour": "value",
                "--network": "value",
                "--network-config-from-file": "value",
                "--port": "value",
                "--release-channel": [
                  "canary",
                  "stable"
                ],
                "--scaling-factor": "value",
                "--tier": [
                  "developer",
                  "enterprise"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--destination-folder": "value",
                    "--dump-type": [
                      "avro",
                      "mysql"
                    ],
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "import": {
              "commands": {
                "gcs": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--database-dump": "value",
                    "--description": "value",
                    "--dump-type": [
                      "avro",
                      "mysql"
                    ],
                    "--import-id": "value",
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backup": "value",
                "--location": "value",
                "--restore-type": [
                  "full",
                  "metadata-only"
                ]
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-auxiliary-versions": "value",
                "--async": "bool",
                "--clear-auxiliary-versions": "bool",
                "--clear-hive-metastore-configs": "bool",
                "--clear-labels": "bool",
                "--instance-size": [
                  "extra_large",
                  "extra_small",
                  "large",
                  "medium",
                  "small"
                ],
                "--kerberos-principal": "value",
                "--keytab": "value",
                "--krb5-config": "value",
                "--location": "value",
                "--maintenance-window-day": [
                  "fri",
                  "mon",
                  "sat",
                  "sun",
                  "thu",
                  "tue",
                  "wed"
                ],
                "--maintenance-window-hour": "value",
                "--port": "value",
                "--remove-hive-metastore-configs": "value",
                "--remove-labels": "value",
                "--scaling-factor": "value",
                "--tier": [
                  "developer",
                  "enterprise"
                ],
                "--update-auxiliary-versions-from-file": "value",
                "--update-hive-metastore-configs": "value",
                "--update-hive-metastore-configs-from-file": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "ml": {
      "commands": {
        "language": {
          "commands": {
            "analyze-entities": {
              "commands": {},
              "flags": {
                "--content": "value",
                "--content-file": "value",
                "--content-type": [
                  "html",
                  "plain-text"
                ],
                "--encoding-type": [
                  "none",
                  "utf16",
                  "utf32",
                  "utf8"
                ],
                "--language": "value"
              }
            },
            "analyze-entity-sentiment": {
              "commands": {},
              "flags": {
                "--content": "value",
                "--content-file": "value",
                "--content-type": [
                  "html",
                  "plain-text"
                ],
                "--encoding-type": [
                  "none",
                  "utf16",
                  "utf32",
                  "utf8"
                ],
                "--language": "value"
              }
            },
            "analyze-sentiment": {
              "commands": {},
              "flags": {
                "--content": "value",
                "--content-file": "value",
                "--content-type": [
                  "html",
                  "plain-text"
                ],
                "--encoding-type": [
                  "none",
                  "utf16",
                  "utf32",
                  "utf8"
                ],
                "--language": "value"
              }
            },
            "analyze-syntax": {
              "commands": {},
              "flags": {
                "--content": "value",
                "--content-file": "value",
                "--content-type": [
                  "html",
                  "plain-text"
                ],
                "--encoding-type": [
                  "none",
                  "utf16",
                  "utf32",
                  "utf8"
                ],
                "--language": "value"
              }
            },
            "classify-text": {
              "commands": {},
              "flags": {
                "--content": "value",
                "--content-file": "value",
                "--content-type": [
                  "html",
                  "plain-text"
                ],
                "--language": "value"
              }
            }
          },
          "flags": {}
        },
        "speech": {
          "commands": {
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "wait": {
                  "commands": {},
                  "flags": {}
                }
              },
              "flags": {}
            },
            "recognize": {
              "commands": {},
              "flags": {
                "--audio-channel-count": "value",
                "--enable-automatic-punctuation": "bool",
                "--encoding": [
                  "amr",
                  "amr-wb",
                  "encoding-unspecified",
                  "flac",
                  "linear16",
                  "mulaw",
                  "ogg-opus",
                  "speex-with-header-byte",
                  "webm-opus"
                ],
                "--filter-profanity": "bool",
                "--hints": "value",
                "--include-word-time-offsets": "bool",
                "--language-code": "value",
                "--max-alternatives": "value",
                "--model": [
                  "command_and_search",
                  "default",
                  "latest_long",
                  "latest_short",
                  "medical_conversation",
                  "medical_dictation",
                  "phone_call",
                  "phone_call_enhanced",
                  "video_enhanced"
                ],
                "--sample-rate": "value",
                "--separate-channel-recognition": "bool"
              }
            },
            "recognize-long-running": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--audio-channel-count": "value",
                "--enable-automatic-punctuation": "bool",
                "--encoding": [
                  "amr",
                  "amr-wb",
                  "encoding-unspecified",
                  "flac",
                  "linear16",
                  "mulaw",
                  "ogg-opus",
                  "speex-with-header-byte",
                  "webm-opus"
                ],
                "--filter-profanity": "bool",
                "--hints": "value",
                "--include-word-time-offsets": "bool",
                "--language-code": "value",
                "--max-alternatives": "value",
                "--model": [
                  "command_and_search",
                  "default",
                  "latest_long",
                  "latest_short",
                  "medical_conversation",
                  "medical_dictation",
                  "phone_call",
                  "phone_call_enhanced",
                  "video_enhanced"
                ],
                "--output-uri": "value",
                "--sample-rate": "value",
                "--separate-channel-recognition": "bool"
              }
            }
          },
          "flags": {}
        },
        "video": {
          "commands": {
            "detect-explicit-content": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--output-uri": "value",
                "--region": [
                  "asia-east1",
                  "europe-west1",
                  "us-east1",
                  "us-west1"
                ],
                "--segments": "value"
              }
            },
            "detect-labels": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--detection-mode": [
                  "frame",
                  "shot",
                  "shot-and-frame"
                ],
                "--output-uri": "value",
                "--region": [
                  "asia-east1",
                  "europe-west1",
                  "us-east1",
                  "us-west1"
                ],
                "--segments": "value"
              }
            },
            "detect-shot-changes": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--output-uri": "value",
                "--region": [
                  "asia-east1",
                  "europe-west1",
                  "us-east1",
                  "us-west1"
                ],
                "--segments": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "vision": {
          "commands": {
            "detect-document": {
              "commands": {},
              "flags": {
                "--language-hints": "value"
              }
            },
            "detect-faces": {
              "commands": {},
              "flags": {
                "--max-results": "value"
              }
            },
            "detect-image-properties": {
              "commands": {},
              "flags": {}
            },
            "detect-labels": {
              "commands": {},
              "flags": {
                "--max-results": "value"
              }
            },
            "detect-landmarks": {
              "commands": {},
              "flags": {
                "--max-results": "value"
              }
            },
            "detect-logos": {
              "commands": {},
              "flags": {
                "--max-results": "value"
              }
            },
            "detect-objects": {
              "commands": {},
              "flags": {}
            },
            "detect-safe-search": {
              "commands": {},
              "flags": {}
            },
            "detect-text": {
              "commands": {},
              "flags": {
                "--language-hints": "value"
              }
            },
            "detect-text-pdf": {
              "commands": {},
              "flags": {
                "--batch-size": "value"
              }
            },
            "detect-text-tiff": {
              "commands": {},
              "flags": {
                "--batch-size": "value"
              }
            },
            "detect-web": {
              "commands": {},
              "flags": {
                "--max-results": "value"
              }
            },
            "suggest-crop": {
              "commands": {},
              "flags": {
                "--aspect-ratios": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "ml-engine": {
      "commands": {
        "jobs": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {
                "--summarize": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "stream-logs": {
              "commands": {},
              "flags": {
                "--allow-multiline-logs": "bool",
                "--polling-interval": "value",
                "--task-name": "value"
              }
            },
            "submit": {
              "commands": {
                "prediction": {
                  "commands": {},
                  "flags": {
                    "--batch-size": "value",
                    "--data-format": [
                      "text",
                      "tf-record",
                      "tf-record-gzip"
                    ],
                    "--input-paths": "value",
                    "--labels": "value",
                    "--max-worker-count": "value",
                    "--model": "value",
                    "--model-dir": "value",
                    "--output-path": "value",
                    "--region": "value",
                    "--runtime-version": "value",
                    "--signature-name": "value"
                  }
                },
                "training": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--config": "value",
                    "--enable-web-access": "bool",
                    "--job-dir": "value",
                    "--kms-key": "dynamic",
                    "--kms-keyring": "dynamic",
                    "--kms-location": "dynamic",
                    "--kms-project": "dynamic",
                    "--labels": "value",
                    "--master-accelerator": "value",
                    "--master-image-uri": "value",
                    "--master-machine-type": "value",
                    "--module-name": "value",
                    "--package-path": "value",
                    "--packages": "value",
                    "--parameter-server-accelerator": "value",
                    "--parameter-server-count": "value",
                    "--parameter-server-image-uri": "value",
                    "--parameter-server-machine-type": "value",
                    "--python-version": "value",
                    "--region": "dynamic",
                    "--runtime-version": "value",
                    "--scale-tier": [
                      "basic",
                      "basic-gpu",
                      "basic-tpu",
                      "custom",
                      "premium-1",
                      "standard-1"
                    ],
                    "--service-account": "value",
                    "--staging-bucket": "value",
                    "--stream-logs": "bool",
                    "--use-chief-in-tf-config": "value",
                    "--worker-accelerator": "value",
                    "--worker-count": "value",
                    "--worker-image-uri": "value",
                    "--worker-machine-type": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "local": {
          "commands": {
            "predict": {
              "commands": {},
              "flags": {
                "--framework": [
                  "scikit-learn",
                  "tensorflow",
                  "xgboost"
                ],
                "--json-instances": "value",
                "--json-request": "value",
                "--model-dir": "value",
                "--signature-name": "value",
                "--text-instances": "value"
              }
            },
            "train": {
              "commands": {},
              "flags": {
                "--distributed": "bool",
                "--evaluator-count": "value",
                "--job-dir": "value",
                "--module-name": "value",
                "--package-path": "value",
                "--parameter-server-count": "value",
                "--start-port": "value",
                "--worker-count": "value"
              }
            }
          },
          "flags": {}
        },
        "models": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--enable-logging": "bool",
                "--labels": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--regions": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--description": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            }
          },
          "flags": {}
        },
        "predict": {
          "commands": {},
          "flags": {
            "--json-instances": "value",
            "--json-request": "value",
            "--model": "value",
            "--region": [
              "asia-east1",
              "asia-northeast1",
              "asia-southeast1",
              "australia-southeast1",
              "europe-west1",
              "europe-west2",
              "europe-west3",
              "europe-west4",
              "global",
              "northamerica-northeast1",
              "us-central1",
              "us-east1",
              "us-east4",
              "us-west1"
            ],
            "--signature-name": "value",
            "--text-instances": "value"
          }
        },
        "versions": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--accelerator": "value",
                "--async": "bool",
                "--config": "value",
                "--description": "value",
                "--framework": [
                  "scikit-learn",
                  "tensorflow",
                  "xgboost"
                ],
                "--labels": "value",
                "--machine-type": "value",
                "--max-nodes": "value",
                "--metric-targets": "value",
                "--min-nodes": "value",
                "--model": "value",
                "--origin": "value",
                "--python-version": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--runtime-version": "value",
                "--staging-bucket": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--model": "value",
                "--page-size": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-default": {
              "commands": {},
              "flags": {
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ]
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--config": "value",
                "--description": "value",
                "--model": "value",
                "--region": [
                  "asia-east1",
                  "asia-northeast1",
                  "asia-southeast1",
                  "australia-southeast1",
                  "europe-west1",
                  "europe-west2",
                  "europe-west3",
                  "europe-west4",
                  "global",
                  "northamerica-northeast1",
                  "us-central1",
                  "us-east1",
                  "us-east4",
                  "us-west1"
                ],
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "monitoring": {
      "commands": {
        "dashboards": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--config": "value",
                "--config-from-file": "value",
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--config": "value",
                "--config-from-file": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "network-connectivity": {
      "commands": {
        "hubs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--labels": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--description": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "internal-ranges": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--ip-cidr-range": "value",
                "--labels": "value",
                "--network": "value",
                "--overlaps": "value",
                "--peering": [
                  "for-peer",
                  "for-self",
                  "not-shared"
                ],
                "--prefix-length": "value",
                "--region": "value",
                "--target-cidr-range": "value",
                "--usage": [
                  "external-to-vpc",
                  "for-vpc"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--ip-cidr-range": "value",
                "--prefix-length": "value",
                "--region": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "policy-based-routes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--destination-range": "value",
                "--interconnect-attachment-region": "value",
                "--ip-protocol": "value",
                "--labels": "value",
                "--network": "value",
                "--next-hop-ilb-ip": "value",
                "--priority": "value",
                "--protocol-version": [
                  "ipv4",
                  "protocol-version-unspecified"
                ],
                "--source-range": "value",
                "--tags": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        },
        "spokes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "value"
              }
            },
            "linked-interconnect-attachments": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--hub": "value",
                    "--interconnect-attachments": "value",
                    "--labels": "value",
                    "--region": "value",
                    "--site-to-site-data-transfer": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "linked-router-appliances": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--hub": "value",
                    "--labels": "value",
                    "--region": "value",
                    "--router-appliance": "value",
                    "--site-to-site-data-transfer": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--router-appliance": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "linked-vpn-tunnels": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--hub": "value",
                    "--labels": "value",
                    "--region": "value",
                    "--site-to-site-data-transfer": "bool",
                    "--vpn-tunnels": "value"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--clear-labels": "bool",
                    "--description": "value",
                    "--region": "value",
                    "--remove-labels": "value",
                    "--update-labels": "value"
                  }
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "network-management": {
      "commands": {
        "connectivity-tests": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--destination-cloud-sql-instance": "value",
                "--destination-gke-master-cluster": "value",
                "--destination-instance": "value",
                "--destination-ip-address": "value",
                "--destination-network": "value",
                "--destination-port": "value",
                "--destination-project": "value",
                "--labels": "value",
                "--other-projects": "value",
                "--protocol": "value",
                "--source-app-engine-version": "value",
                "--source-cloud-function": "value",
                "--source-cloud-run-revision": "value",
                "--source-cloud-sql-instance": "value",
                "--source-gke-master-cluster": "value",
                "--source-instance": "value",
                "--source-ip-address": "value",
                "--source-network": "value",
                "--source-network-type": [
                  "gcp-network",
                  "non-gcp-network"
                ],
                "--source-project": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "rerun": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-destination-cloud-sql-instance": "bool",
                "--clear-destination-gke-master-cluster": "bool",
                "--clear-destination-instance": "bool",
                "--clear-destination-ip-address": "bool",
                "--clear-source-app-engine-version": "bool",
                "--clear-source-cloud-function": "bool",
                "--clear-source-cloud-run-revision": "bool",
                "--clear-source-cloud-sql-instance": "bool",
                "--clear-source-gke-master-cluster": "bool",
                "--clear-source-instance": "bool",
                "--clear-source-ip-address": "bool",
                "--description": "value",
                "--destination-cloud-sql-instance": "value",
                "--destination-gke-master-cluster": "value",
                "--destination-instance": "value",
                "--destination-ip-address": "value",
                "--destination-network": "value",
                "--destination-port": "value",
                "--destination-project": "value",
                "--labels": "value",
                "--other-projects": "value",
                "--protocol": "value",
                "--source-app-engine-version": "value",
                "--source-cloud-function": "value",
                "--source-cloud-run-revision": "value",
                "--source-cloud-sql-instance": "value",
                "--source-gke-master-cluster": "value",
                "--source-instance": "value",
                "--source-ip-address": "value",
                "--source-network": "value",
                "--source-network-type": [
                  "gcp-network",
                  "non-gcp-network"
                ],
                "--source-project": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "network-security": {
      "commands": {
        "authorization-policies": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "client-tls-policies": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "server-tls-policies": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "network-services": {
      "commands": {
        "endpoint-policies": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "gateways": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "grpc-routes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "http-routes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "meshes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "service-bindings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--location": "value",
                "--service-directory-namespace": "value",
                "--service-directory-region": "value",
                "--service-directory-service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "tcp-routes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "tls-routes": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--destination": "value",
                "--location": "value"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "notebooks": {
      "commands": {
        "environments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--container-repository": "value",
                "--container-tag": "value",
                "--description": "value",
                "--display-name": "value",
                "--location": "dynamic",
                "--post-startup-script": "value",
                "--vm-image-family": "value",
                "--vm-image-name": "value",
                "--vm-image-project": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--accelerator-core-count": "value",
                "--accelerator-type": [
                  "NVIDIA_TESLA_A100",
                  "NVIDIA_TESLA_K80",
                  "NVIDIA_TESLA_P100",
                  "NVIDIA_TESLA_P100_VWS",
                  "NVIDIA_TESLA_P4",
                  "NVIDIA_TESLA_P4_VWS",
                  "NVIDIA_TESLA_T4",
                  "NVIDIA_TESLA_T4_VWS",
                  "NVIDIA_TESLA_V100",
                  "TPU_V2",
                  "TPU_V3"
                ],
                "--async": "bool",
                "--boot-disk-size": "value",
                "--boot-disk-type": [
                  "PD_BALANCED",
                  "PD_SSD",
                  "PD_STANDARD"
                ],
                "--container-repository": "value",
                "--container-tag": "value",
                "--custom-gpu-driver-path": "value",
                "--disk-encryption": [
                  "CMEK",
                  "GMEK"
                ],
                "--environment": "dynamic",
                "--environment-location": "dynamic",
                "--install-gpu-driver": "bool",
                "--instance-owners": "value",
                "--kms-key": "dynamic",
                "--kms-keyring": "dynamic",
                "--kms-location": "dynamic",
                "--kms-project": "dynamic",
                "--labels": "value",
                "--location": "dynamic",
                "--machine-type": "value",
                "--metadata": "value",
                "--network": "dynamic",
                "--no-proxy-access": "bool",
                "--no-public-ip": "bool",
                "--post-startup-script": "value",
                "--reservation": "value",
                "--reservation-affinity": [
                  "ANY_RESERVATION",
                  "NO_RESERVATION",
                  "SPECIFIC_RESERVATION",
                  "TYPE_UNSPECIFIED"
                ],
                "--service-account": "value",
                "--shielded-integrity-monitoring": "bool",
                "--shielded-secure-boot": "bool",
                "--shielded-vtpm": "bool",
                "--subnet": "dynamic",
                "--subnet-region": "dynamic",
                "--vm-image-family": "value",
                "--vm-image-name": "value",
                "--vm-image-project": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "diagnose": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--enable-copy-home-files": "bool",
                "--enable-packet-capture": "bool",
                "--enable-repair": "bool",
                "--gcs-bucket": "value",
                "--location": "dynamic",
                "--relative-path": "value"
              }
            },
            "get-health": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "is-upgradeable": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "register": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "reset": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "rollback": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic",
                "--target-snapshot": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--accelerator-core-count": "value",
                "--accelerator-type": [
                  "NVIDIA_TESLA_A100",
                  "NVIDIA_TESLA_K80",
                  "NVIDIA_TESLA_P100",
                  "NVIDIA_TESLA_P100_VWS",
                  "NVIDIA_TESLA_P4",
                  "NVIDIA_TESLA_P4_VWS",
                  "NVIDIA_TESLA_T4",
                  "NVIDIA_TESLA_T4_VWS",
                  "NVIDIA_TESLA_V100",
                  "TPU_V2",
                  "TPU_V3"
                ],
                "--async": "bool",
                "--labels": "value",
                "--location": "dynamic",
                "--machine-type": "value"
              }
            },
            "upgrade": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "runtimes": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--custom-gpu-driver-path": "value",
                "--idle-shutdown-timeout": "value",
                "--install-gpu-driver": "value",
                "--interface": "value",
                "--location": "dynamic",
                "--machine-type": "value",
                "--mode": "value",
                "--post-startup-script": "value",
                "--post-startup-script-behavior": "value",
                "--runtime-access-type": "value",
                "--runtime-owner": "value",
                "--runtime-type": "value",
                "--source": "value",
                "--type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "diagnose": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--enable-copy-home-files": "bool",
                "--enable-packet-capture": "bool",
                "--enable-repair": "bool",
                "--gcs-bucket": "value",
                "--location": "dynamic",
                "--relative-path": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "reset": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "start": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "stop": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "switch": {
              "commands": {},
              "flags": {
                "--accelerator-core-count": "value",
                "--accelerator-type": [
                  "NVIDIA_TESLA_A100",
                  "NVIDIA_TESLA_K80",
                  "NVIDIA_TESLA_P100",
                  "NVIDIA_TESLA_P100_VWS",
                  "NVIDIA_TESLA_P4",
                  "NVIDIA_TESLA_P4_VWS",
                  "NVIDIA_TESLA_T4",
                  "NVIDIA_TESLA_T4_VWS",
                  "NVIDIA_TESLA_V100",
                  "TPU_V2",
                  "TPU_V3"
                ],
                "--async": "bool",
                "--location": "dynamic",
                "--machine-type": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "org-policies": {
      "commands": {
        "delete": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--organization": "dynamic"
          }
        },
        "delete-custom-constraint": {
          "commands": {},
          "flags": {
            "--organization": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--effective": "bool",
            "--folder": "value",
            "--organization": "dynamic"
          }
        },
        "describe-custom-constraint": {
          "commands": {},
          "flags": {
            "--organization": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--organization": "dynamic",
            "--page-size": "value",
            "--show-unset": "bool",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "list-custom-constraints": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--organization": "value",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "reset": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--organization": "dynamic"
          }
        },
        "set-custom-constraint": {
          "commands": {},
          "flags": {}
        },
        "set-policy": {
          "commands": {},
          "flags": {
            "--update-mask": "value"
          }
        }
      },
      "flags": {}
    },
    "organizations": {
      "commands": {
        "add-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "describe": {
          "commands": {},
          "flags": {}
        },
        "get-iam-policy": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "remove-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--all": "bool",
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "set-iam-policy": {
          "commands": {},
          "flags": {}
        }
      },
      "flags": {}
    },
    "policy-intelligence": {
      "commands": {
        "query-activity": {
          "commands": {},
          "flags": {
            "--activity-type": [
              "serviceAccountKeyLastAuthentication",
              "serviceAccountLastAuthentication"
            ],
            "--limit": "value",
            "--page-size": "value",
            "--query-filter": "value"
          }
        }
      },
      "flags": {}
    },
    "policy-troubleshoot": {
      "commands": {
        "iam": {
          "commands": {},
          "flags": {
            "--destination-ip": "value",
            "--destination-port": "value",
            "--permission": "value",
            "--principal-email": "value",
            "--request-time": "value",
            "--resource-name": "value",
            "--resource-service": "value",
            "--resource-type": "value"
          }
        }
      },
      "flags": {}
    },
    "privateca": {
      "commands": {
        "certificates": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--ca": "value",
                "--cert-output-file": "value",
                "--csr": "value",
                "--dns-san": "value",
                "--email-san": "value",
                "--extended-key-usages": "value",
                "--generate-key": "bool",
                "--ip-san": "value",
                "--is-ca-cert": "bool",
                "--issuer-location": "dynamic",
                "--issuer-pool": "dynamic",
                "--key-output-file": "value",
                "--key-usages": "value",
                "--kms-key": "value",
                "--kms-key-version": "value",
                "--kms-keyring": "value",
                "--kms-location": "dynamic",
                "--kms-project": "value",
                "--labels": "value",
                "--max-chain-length": "value",
                "--name-constraints-critical": "bool",
                "--name-excluded-dns": "value",
                "--name-excluded-email": "value",
                "--name-excluded-ip": "value",
                "--name-excluded-uri": "value",
                "--name-permitted-dns": "value",
                "--name-permitted-email": "value",
                "--name-permitted-ip": "value",
                "--name-permitted-uri": "value",
                "--subject": "value",
                "--template": "value",
                "--template-location": "dynamic",
                "--unconstrained-chain-length": "bool",
                "--uri-san": "value",
                "--use-preset-profile": "value",
                "--validate-only": "bool",
                "--validity": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--issuer-location": "value",
                "--issuer-pool": "value"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--include-chain": "bool",
                "--issuer-location": "dynamic",
                "--issuer-pool": "dynamic",
                "--output-file": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--issuer-pool": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "revoke": {
              "commands": {},
              "flags": {
                "--certificate": "dynamic",
                "--issuer-location": "dynamic",
                "--issuer-pool": "value",
                "--reason": [
                  "affiliation-changed",
                  "attribute-authority-compromise",
                  "certificate-authority-compromise",
                  "certificate-hold",
                  "cessation-of-operation",
                  "key-compromise",
                  "privilege-withdrawn",
                  "superseded",
                  "unspecified"
                ],
                "--serial-number": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--issuer-location": "dynamic",
                "--issuer-pool": "dynamic",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "pools": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--issuance-policy": "value",
                "--labels": "value",
                "--location": "dynamic",
                "--publish-ca-cert": "bool",
                "--publish-crl": "bool",
                "--tier": [
                  "devops",
                  "enterprise"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-ca-certs": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--output-file": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--issuance-policy": "value",
                "--location": "dynamic",
                "--publish-ca-cert": "bool",
                "--publish-crl": "bool",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "roots": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--auto-enable": "bool",
                "--bucket": "value",
                "--dns-san": "value",
                "--email-san": "value",
                "--extended-key-usages": "value",
                "--from-ca": "value",
                "--ip-san": "value",
                "--key-algorithm": [
                  "ec-p256-sha256",
                  "ec-p384-sha384",
                  "rsa-pkcs1-2048-sha256",
                  "rsa-pkcs1-3072-sha256",
                  "rsa-pkcs1-4096-sha256",
                  "rsa-pss-2048-sha256",
                  "rsa-pss-3072-sha256",
                  "rsa-pss-4096-sha256"
                ],
                "--key-usages": "value",
                "--kms-key": "value",
                "--kms-key-version": "value",
                "--kms-keyring": "value",
                "--kms-location": "dynamic",
                "--kms-project": "value",
                "--labels": "value",
                "--location": "dynamic",
                "--max-chain-length": "value",
                "--name-constraints-critical": "bool",
                "--name-excluded-dns": "value",
                "--name-excluded-email": "value",
                "--name-excluded-ip": "value",
                "--name-excluded-uri": "value",
                "--name-permitted-dns": "value",
                "--name-permitted-email": "value",
                "--name-permitted-ip": "value",
                "--name-permitted-uri": "value",
                "--pool": "value",
                "--subject": "value",
                "--unconstrained-chain-length": "bool",
                "--uri-san": "value",
                "--use-preset-profile": "value",
                "--validity": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--ignore-active-certificates": "bool",
                "--location": "dynamic",
                "--pool": "value",
                "--skip-grace-period": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--pool": "value"
              }
            },
            "disable": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--pool": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--location": "dynamic",
                "--pool": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "subordinates": {
          "commands": {
            "activate": {
              "commands": {},
              "flags": {
                "--auto-enable": "bool",
                "--location": "dynamic",
                "--pem-chain": "value",
                "--pool": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--auto-enable": "bool",
                "--bucket": "value",
                "--create-csr": "bool",
                "--csr-output-file": "value",
                "--dns-san": "value",
                "--email-san": "value",
                "--extended-key-usages": "value",
                "--from-ca": "value",
                "--ip-san": "value",
                "--issuer-ca": "value",
                "--issuer-location": "dynamic",
                "--issuer-pool": "value",
                "--key-algorithm": [
                  "ec-p256-sha256",
                  "ec-p384-sha384",
                  "rsa-pkcs1-2048-sha256",
                  "rsa-pkcs1-3072-sha256",
                  "rsa-pkcs1-4096-sha256",
                  "rsa-pss-2048-sha256",
                  "rsa-pss-3072-sha256",
                  "rsa-pss-4096-sha256"
                ],
                "--key-usages": "value",
                "--kms-key": "value",
                "--kms-key-version": "value",
                "--kms-keyring": "value",
                "--kms-location": "dynamic",
                "--kms-project": "value",
                "--labels": "value",
                "--location": "dynamic",
                "--max-chain-length": "value",
                "--name-constraints-critical": "bool",
                "--name-excluded-dns": "value",
                "--name-excluded-email": "value",
                "--name-excluded-ip": "value",
                "--name-excluded-uri": "value",
                "--name-permitted-dns": "value",
                "--name-permitted-email": "value",
                "--name-permitted-ip": "value",
                "--name-permitted-uri": "value",
                "--pool": "value",
                "--subject": "value",
                "--unconstrained-chain-length": "bool",
                "--uri-san": "value",
                "--use-preset-profile": "value",
                "--validity": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--ignore-active-certificates": "bool",
                "--location": "dynamic",
                "--pool": "value",
                "--skip-grace-period": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--pool": "value"
              }
            },
            "disable": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "get-csr": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--pool": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--location": "dynamic",
                "--pool": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--location": "dynamic",
                "--pem-chain": "value",
                "--pool": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        },
        "templates": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--copy-all-requested-extensions": "bool",
                "--copy-extensions-by-oid": "value",
                "--copy-known-extensions": "value",
                "--copy-sans": "bool",
                "--copy-subject": "bool",
                "--description": "value",
                "--identity-cel-expression": "value",
                "--labels": "value",
                "--location": "dynamic",
                "--predefined-values-file": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "replicate": {
              "commands": {},
              "flags": {
                "--all-locations": "bool",
                "--continue-on-error": "bool",
                "--location": "dynamic",
                "--overwrite": "bool",
                "--target-locations": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--copy-all-requested-extensions": "bool",
                "--copy-extensions-by-oid": "value",
                "--copy-known-extensions": "value",
                "--copy-sans": "bool",
                "--copy-subject": "bool",
                "--description": "value",
                "--drop-known-extensions": "bool",
                "--drop-oid-extensions": "bool",
                "--identity-cel-expression": "value",
                "--location": "dynamic",
                "--predefined-values-file": "value",
                "--remove-labels": "value",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "projects": {
      "commands": {
        "add-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "create": {
          "commands": {},
          "flags": {
            "--enable-cloud-apis": "bool",
            "--folder": "value",
            "--labels": "value",
            "--name": "value",
            "--organization": "value",
            "--set-as-default": "bool"
          }
        },
        "delete": {
          "commands": {},
          "flags": {}
        },
        "describe": {
          "commands": {},
          "flags": {}
        },
        "get-ancestors": {
          "commands": {},
          "flags": {}
        },
        "get-ancestors-iam-policy": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "get-iam-policy": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "remove-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--all": "bool",
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "set-iam-policy": {
          "commands": {},
          "flags": {}
        },
        "undelete": {
          "commands": {},
          "flags": {}
        },
        "update": {
          "commands": {},
          "flags": {
            "--name": "value"
          }
        }
      },
      "flags": {}
    },
    "pubsub": {
      "commands": {
        "lite-operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--done": "bool",
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--subscription": "value"
              }
            }
          },
          "flags": {}
        },
        "lite-reservations": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--throughput-capacity": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list-topics": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--throughput-capacity": "value"
              }
            }
          },
          "flags": {}
        },
        "lite-subscriptions": {
          "commands": {
            "ack-up-to": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--offset": "value",
                "--partition": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--delivery-requirement": [
                  "deliver-after-stored",
                  "deliver-immediately"
                ],
                "--event-time": "value",
                "--export-dead-letter-topic": "value",
                "--export-desired-state": [
                  "active",
                  "paused"
                ],
                "--export-pubsub-topic": "value",
                "--location": "value",
                "--publish-time": "value",
                "--starting-offset": [
                  "beginning",
                  "end"
                ],
                "--topic": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "seek": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--event-time": "value",
                "--location": "value",
                "--publish-time": "value",
                "--starting-offset": [
                  "beginning",
                  "end"
                ]
              }
            },
            "subscribe": {
              "commands": {},
              "flags": {
                "--auto-ack": "bool",
                "--location": "value",
                "--num-messages": "value",
                "--partitions": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--delivery-requirement": [
                  "deliver-after-stored",
                  "deliver-immediately"
                ],
                "--export-dead-letter-topic": "value",
                "--export-desired-state": [
                  "active",
                  "paused"
                ],
                "--export-pubsub-topic": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "lite-topics": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--message-retention-period": "value",
                "--partitions": "value",
                "--per-partition-bytes": "value",
                "--per-partition-publish-mib": "value",
                "--per-partition-subscribe-mib": "value",
                "--throughput-reservation": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list-subscriptions": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "publish": {
              "commands": {},
              "flags": {
                "--attributes": "value",
                "--location": "value",
                "--message": "value",
                "--ordering-key": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--message-retention-period": "value",
                "--partitions": "value",
                "--per-partition-bytes": "value",
                "--per-partition-publish-mib": "value",
                "--per-partition-subscribe-mib": "value",
                "--throughput-reservation": "value"
              }
            }
          },
          "flags": {}
        },
        "schemas": {
          "commands": {
            "commit": {
              "commands": {},
              "flags": {
                "--definition": "value",
                "--definition-file": "value",
                "--type": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--definition": "value",
                "--definition-file": "value",
                "--type": [
                  "avro",
                  "protocol-buffer",
                  "type-unspecified"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "delete-revision": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--view": [
                  "basic",
                  "full"
                ]
              }
            },
            "list-revisions": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool",
                "--view": [
                  "basic",
                  "full"
                ]
              }
            },
            "rollback": {
              "commands": {},
              "flags": {
                "--revision-id": "value"
              }
            },
            "validate-message": {
              "commands": {},
              "flags": {
                "--definition": "value",
                "--definition-file": "value",
                "--message": "value",
                "--message-encoding": [
                  "binary",
                  "encoding-unspecified",
                  "json"
                ],
                "--schema-name": "value",
                "--type": [
                  "avro",
                  "protocol-buffer",
                  "type-unspecified"
                ]
              }
            },
            "validate-schema": {
              "commands": {},
              "flags": {
                "--definition": "value",
                "--definition-file": "value",
                "--type": [
                  "avro",
                  "protocol-buffer",
                  "type-unspecified"
                ]
              }
            }
          },
          "flags": {}
        },
        "snapshots": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--labels": "value",
                "--subscription": "value",
                "--subscription-project": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "subscriptions": {
          "commands": {
            "ack": {
              "commands": {},
              "flags": {
                "--ack-ids": "value"
              }
            },
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--ack-deadline": "value",
                "--bigquery-table": "value",
                "--dead-letter-topic": "value",
                "--dead-letter-topic-project": "value",
                "--drop-unknown-fields": "bool",
                "--enable-exactly-once-delivery": "bool",
                "--enable-message-ordering": "bool",
                "--expiration-period": "value",
                "--labels": "value",
                "--max-delivery-attempts": "value",
                "--max-retry-delay": "value",
                "--message-filter": "value",
                "--message-retention-duration": "value",
                "--min-retry-delay": "value",
                "--push-auth-service-account": "value",
                "--push-auth-token-audience": "value",
                "--push-endpoint": "value",
                "--retain-acked-messages": "bool",
                "--topic": "value",
                "--topic-project": "value",
                "--use-topic-schema": "bool",
                "--write-metadata": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "modify-message-ack-deadline": {
              "commands": {},
              "flags": {
                "--ack-deadline": "value",
                "--ack-ids": "value"
              }
            },
            "modify-push-config": {
              "commands": {},
              "flags": {
                "--push-auth-service-account": "value",
                "--push-auth-token-audience": "value",
                "--push-endpoint": "value"
              }
            },
            "pull": {
              "commands": {},
              "flags": {
                "--auto-ack": "bool",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "seek": {
              "commands": {},
              "flags": {
                "--snapshot": "value",
                "--snapshot-project": "value",
                "--time": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--ack-deadline": "value",
                "--bigquery-table": "value",
                "--clear-bigquery-config": "bool",
                "--clear-dead-letter-policy": "bool",
                "--clear-labels": "bool",
                "--clear-retry-policy": "bool",
                "--dead-letter-topic": "value",
                "--dead-letter-topic-project": "value",
                "--drop-unknown-fields": "bool",
                "--enable-exactly-once-delivery": "bool",
                "--expiration-period": "value",
                "--max-delivery-attempts": "value",
                "--max-retry-delay": "value",
                "--message-retention-duration": "value",
                "--min-retry-delay": "value",
                "--push-auth-service-account": "value",
                "--push-auth-token-audience": "value",
                "--push-endpoint": "value",
                "--remove-labels": "value",
                "--retain-acked-messages": "bool",
                "--update-labels": "value",
                "--use-topic-schema": "bool",
                "--write-metadata": "bool"
              }
            }
          },
          "flags": {}
        },
        "topics": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--first-revision-id": "value",
                "--labels": "value",
                "--last-revision-id": "value",
                "--message-encoding": "value",
                "--message-retention-duration": "value",
                "--message-storage-policy-allowed-regions": "value",
                "--schema": "value",
                "--schema-project": "value",
                "--topic-encryption-key": "dynamic",
                "--topic-encryption-key-keyring": "dynamic",
                "--topic-encryption-key-location": "dynamic",
                "--topic-encryption-key-project": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "detach-subscription": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "list-subscriptions": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "publish": {
              "commands": {},
              "flags": {
                "--attribute": "value",
                "--message": "value",
                "--ordering-key": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-labels": "bool",
                "--clear-message-retention-duration": "bool",
                "--clear-schema-settings": "bool",
                "--first-revision-id": "value",
                "--last-revision-id": "value",
                "--message-encoding": "value",
                "--message-retention-duration": "value",
                "--message-storage-policy-allowed-regions": "value",
                "--recompute-message-storage-policy": "bool",
                "--remove-labels": "value",
                "--schema": "value",
                "--schema-project": "value",
                "--topic-encryption-key": "dynamic",
                "--topic-encryption-key-keyring": "dynamic",
                "--topic-encryption-key-location": "dynamic",
                "--topic-encryption-key-project": "dynamic",
                "--update-labels": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "recaptcha": {
      "commands": {
        "keys": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--allow-all-bundle-ids": "bool",
                "--allow-all-domains": "bool",
                "--allow-all-package-names": "bool",
                "--allow-amp-traffic": "bool",
                "--android": "bool",
                "--bundle-ids": "value",
                "--display-name": "value",
                "--domains": "value",
                "--integration-type": [
                  "checkbox",
                  "invisible",
                  "score"
                ],
                "--ios": "bool",
                "--labels": "value",
                "--package-names": "value",
                "--security-preference": [
                  "balance",
                  "challenge-security-preference-unspecified",
                  "security",
                  "usability"
                ],
                "--testing-challenge": [
                  "challenge",
                  "nocaptcha"
                ],
                "--testing-score": "value",
                "--waf-feature": [
                  "action-token",
                  "challenge-page",
                  "express",
                  "session-token"
                ],
                "--waf-service": [
                  "ca",
                  "fastly",
                  "unspecified"
                ],
                "--web": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "migrate": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--allow-all-bundle-ids": "bool",
                "--allow-all-domains": "bool",
                "--allow-all-package-names": "bool",
                "--allow-amp-traffic": "bool",
                "--android": "bool",
                "--bundle-ids": "value",
                "--display-name": "value",
                "--domains": "value",
                "--ios": "bool",
                "--labels": "value",
                "--package-names": "value",
                "--security-preference": [
                  "balance",
                  "challenge-security-preference-unspecified",
                  "security",
                  "usability"
                ],
                "--web": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "recommender": {
      "commands": {
        "insight-type-config": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--location": "value",
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--billing-account": "value",
                "--config-file": "value",
                "--display-name": "value",
                "--etag": "value",
                "--location": "value",
                "--organization": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        },
        "insights": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--insight-type": "value",
                "--location": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--folder": "value",
                "--insight-type": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "mark-accepted": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--etag": "value",
                "--folder": "value",
                "--insight-type": "value",
                "--location": "value",
                "--organization": "value",
                "--state-metadata": "value"
              }
            }
          },
          "flags": {}
        },
        "recommendations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "value",
                "--recommender": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--location": "value",
                "--organization": "value",
                "--page-size": "value",
                "--recommender": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "mark-claimed": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--etag": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "value",
                "--recommender": "value",
                "--state-metadata": "value"
              }
            },
            "mark-dismissed": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--etag": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "value",
                "--recommender": "value"
              }
            },
            "mark-failed": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--etag": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "value",
                "--recommender": "value",
                "--state-metadata": "value"
              }
            },
            "mark-succeeded": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--etag": "value",
                "--folder": "value",
                "--location": "value",
                "--organization": "value",
                "--recommender": "value",
                "--state-metadata": "value"
              }
            }
          },
          "flags": {}
        },
        "recommender-config": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--billing-account": "value",
                "--location": "value",
                "--organization": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--billing-account": "value",
                "--config-file": "value",
                "--display-name": "value",
                "--etag": "value",
                "--location": "value",
                "--organization": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "redis": {
      "commands": {
        "instances": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--alternative-zone": "value",
                "--async": "bool",
                "--connect-mode": [
                  "connect-mode-unspecified",
                  "direct-peering",
                  "private-service-access"
                ],
                "--customer-managed-key": "value",
                "--display-name": "value",
                "--enable-auth": "bool",
                "--labels": "value",
                "--maintenance-window-day": [
                  "day-of-week-unspecified",
                  "friday",
                  "monday",
                  "saturday",
                  "sunday",
                  "thursday",
                  "tuesday",
                  "wednesday"
                ],
                "--maintenance-window-hour": "value",
                "--network": "value",
                "--persistence-mode": [
                  "disabled",
                  "rdb"
                ],
                "--rdb-snapshot-period": [
                  "12h",
                  "1h",
                  "24h",
                  "6h"
                ],
                "--rdb-snapshot-start-time": "value",
                "--read-replicas-mode": [
                  "read-replicas-disabled",
                  "read-replicas-enabled"
                ],
                "--redis-config": "value",
                "--redis-version": [
                  "redis_3_2",
                  "redis_4_0",
                  "redis_5_0",
                  "redis_6_x"
                ],
                "--region": "dynamic",
                "--replica-count": "value",
                "--reserved-ip-range": "value",
                "--size": "value",
                "--tier": [
                  "basic",
                  "standard"
                ],
                "--transit-encryption-mode": [
                  "disabled",
                  "server-authentication"
                ],
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "failover": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--data-protection-mode": [
                  "force-data-loss",
                  "limited-data-loss"
                ],
                "--region": "dynamic"
              }
            },
            "get-auth-string": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "reschedule-maintenance": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--region": "dynamic",
                "--reschedule-type": [
                  "immediate",
                  "next-available-window",
                  "specific-time"
                ],
                "--schedule-time": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--enable-auth": "bool",
                "--maintenance-window-any": "bool",
                "--maintenance-window-day": [
                  "day-of-week-unspecified",
                  "friday",
                  "monday",
                  "saturday",
                  "sunday",
                  "thursday",
                  "tuesday",
                  "wednesday"
                ],
                "--maintenance-window-hour": "value",
                "--persistence-mode": [
                  "disabled",
                  "rdb"
                ],
                "--rdb-snapshot-period": [
                  "12h",
                  "1h",
                  "24h",
                  "6h"
                ],
                "--rdb-snapshot-start-time": "value",
                "--read-replicas-mode": [
                  "read-replicas-disabled",
                  "read-replicas-enabled"
                ],
                "--region": "dynamic",
                "--remove-labels": "value",
                "--remove-redis-config": "value",
                "--replica-count": "value",
                "--secondary-ip-range": "value",
                "--size": "value",
                "--update-labels": "value",
                "--update-redis-config": "value"
              }
            },
            "upgrade": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--redis-version": [
                  "redis_4_0",
                  "redis_5_0",
                  "redis_6_x"
                ],
                "--region": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--region": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "dynamic",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "regions": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "zones": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--region": "value",
                "--sort-by": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "resource-manager": {
      "commands": {
        "folders": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--display-name": "value",
                "--folder": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-ancestors-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "move": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--folder": "value",
                "--organization": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--display-name": "value"
              }
            }
          },
          "flags": {}
        },
        "org-policies": {
          "commands": {
            "allow": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "deny": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--effective": "bool",
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "disable-enforce": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "enable-enforce": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "dynamic",
                "--page-size": "value",
                "--show-unset": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-policy": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "tags": {
          "commands": {
            "bindings": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--parent": "value",
                    "--tag-value": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "value",
                    "--parent": "value",
                    "--tag-value": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--effective": "bool",
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--page-size": "value",
                    "--parent": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "holds": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--help-link": "value",
                    "--holder": "value",
                    "--location": "value",
                    "--origin": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--location": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--holder": "value",
                    "--limit": "value",
                    "--location": "value",
                    "--origin": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "keys": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--parent": "value",
                    "--purpose": [
                      "GCE_FIREWALL"
                    ],
                    "--purpose-data": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--parent": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value"
                  }
                }
              },
              "flags": {}
            },
            "values": {
              "commands": {
                "add-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value",
                    "--parent": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "get-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--parent": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "remove-iam-policy-binding": {
                  "commands": {},
                  "flags": {
                    "--all": "bool",
                    "--condition": "value",
                    "--condition-from-file": "value",
                    "--member": "value",
                    "--role": "value"
                  }
                },
                "set-iam-policy": {
                  "commands": {},
                  "flags": {}
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--description": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "resource-settings": {
      "commands": {
        "describe": {
          "commands": {},
          "flags": {
            "--effective": "bool",
            "--folder": "value",
            "--organization": "dynamic"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--organization": "dynamic",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "list-values": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--folder": "value",
            "--limit": "value",
            "--organization": "dynamic",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "set-value": {
          "commands": {},
          "flags": {
            "--value-file": "dynamic"
          }
        },
        "unset-value": {
          "commands": {},
          "flags": {
            "--folder": "value",
            "--organization": "dynamic"
          }
        }
      },
      "flags": {}
    },
    "run": {
      "commands": {
        "deploy": {
          "commands": {},
          "flags": {
            "--add-cloudsql-instances": "value",
            "--allow-unauthenticated": "bool",
            "--args": "value",
            "--async": "bool",
            "--binary-authorization": "value",
            "--breakglass": "value",
            "--clear-binary-authorization": "bool",
            "--clear-cloudsql-instances": "bool",
            "--clear-config-maps": "bool",
            "--clear-env-vars": "bool",
            "--clear-key": "bool",
            "--clear-labels": "bool",
            "--clear-post-key-revocation-action-type": "bool",
            "--clear-secrets": "bool",
            "--clear-vpc-connector": "bool",
            "--cluster": "value",
            "--cluster-location": "value",
            "--command": "value",
            "--concurrency": "value",
            "--connectivity": [
              "external",
              "internal"
            ],
            "--context": "value",
            "--cpu": "value",
            "--cpu-throttling": "bool",
            "--description": "value",
            "--env-vars-file": "value",
            "--execution-environment": [
              "gen1",
              "gen2"
            ],
            "--image": "value",
            "--ingress": [
              "all",
              "internal",
              "internal-and-cloud-load-balancing"
            ],
            "--key": "value",
            "--kubeconfig": "value",
            "--labels": "value",
            "--max-instances": "value",
            "--memory": "value",
            "--min-instances": "value",
            "--namespace": "value",
            "--no-traffic": "bool",
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ],
            "--port": "value",
            "--post-key-revocation-action-type": [
              "prevent-new",
              "shut-down"
            ],
            "--region": "value",
            "--remove-cloudsql-instances": "value",
            "--remove-config-maps": "value",
            "--remove-env-vars": "value",
            "--remove-labels": "value",
            "--remove-secrets": "value",
            "--revision-suffix": "value",
            "--service-account": "value",
            "--set-cloudsql-instances": "value",
            "--set-config-maps": "value",
            "--set-env-vars": "value",
            "--set-secrets": "value",
            "--source": "value",
            "--tag": "value",
            "--timeout": "value",
            "--update-config-maps": "value",
            "--update-env-vars": "value",
            "--update-labels": "value",
            "--update-secrets": "value",
            "--use-http2": "bool",
            "--vpc-connector": "value",
            "--vpc-egress": [
              "all",
              "all-traffic",
              "private-ranges-only"
            ]
          }
        },
        "domain-mappings": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--domain": "value",
                "--force-override": "bool",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--domain": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--domain": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ]
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--filter": "value",
                "--kubeconfig": "value",
                "--limit": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {
            "--cluster": "value",
            "--cluster-location": "value",
            "--context": "value",
            "--kubeconfig": "value",
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ]
          }
        },
        "regions": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ]
          }
        },
        "revisions": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--filter": "value",
                "--kubeconfig": "value",
                "--limit": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--service": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {
            "--cluster": "value",
            "--cluster-location": "value",
            "--context": "value",
            "--kubeconfig": "value",
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ],
            "--region": "value"
          }
        },
        "services": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--member": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--role": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--filter": "value",
                "--kubeconfig": "value",
                "--limit": "value",
                "--page-size": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--filter": "value",
                "--kubeconfig": "value",
                "--limit": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--member": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--role": "dynamic"
              }
            },
            "replace": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-cloudsql-instances": "value",
                "--args": "value",
                "--async": "bool",
                "--binary-authorization": "value",
                "--breakglass": "value",
                "--clear-binary-authorization": "bool",
                "--clear-cloudsql-instances": "bool",
                "--clear-config-maps": "bool",
                "--clear-env-vars": "bool",
                "--clear-key": "bool",
                "--clear-labels": "bool",
                "--clear-post-key-revocation-action-type": "bool",
                "--clear-secrets": "bool",
                "--clear-vpc-connector": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--command": "value",
                "--concurrency": "value",
                "--connectivity": [
                  "external",
                  "internal"
                ],
                "--context": "value",
                "--cpu": "value",
                "--cpu-throttling": "bool",
                "--env-vars-file": "value",
                "--execution-environment": [
                  "gen1",
                  "gen2"
                ],
                "--image": "value",
                "--ingress": [
                  "all",
                  "internal",
                  "internal-and-cloud-load-balancing"
                ],
                "--key": "value",
                "--kubeconfig": "value",
                "--labels": "value",
                "--max-instances": "value",
                "--memory": "value",
                "--min-instances": "value",
                "--namespace": "value",
                "--no-traffic": "bool",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--port": "value",
                "--post-key-revocation-action-type": [
                  "prevent-new",
                  "shut-down"
                ],
                "--region": "value",
                "--remove-cloudsql-instances": "value",
                "--remove-config-maps": "value",
                "--remove-env-vars": "value",
                "--remove-labels": "value",
                "--remove-secrets": "value",
                "--revision-suffix": "value",
                "--service-account": "value",
                "--set-cloudsql-instances": "value",
                "--set-config-maps": "value",
                "--set-env-vars": "value",
                "--set-secrets": "value",
                "--tag": "value",
                "--timeout": "value",
                "--update-config-maps": "value",
                "--update-env-vars": "value",
                "--update-labels": "value",
                "--update-secrets": "value",
                "--use-http2": "bool",
                "--vpc-connector": "value",
                "--vpc-egress": [
                  "all",
                  "all-traffic",
                  "private-ranges-only"
                ]
              }
            },
            "update-traffic": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-tags": "bool",
                "--cluster": "value",
                "--cluster-location": "value",
                "--context": "value",
                "--kubeconfig": "value",
                "--namespace": "value",
                "--platform": [
                  "gke",
                  "kubernetes",
                  "managed"
                ],
                "--region": "value",
                "--remove-tags": "value",
                "--set-tags": "value",
                "--to-latest": "bool",
                "--to-revisions": "value",
                "--to-tags": "value",
                "--update-tags": "value"
              }
            }
          },
          "flags": {
            "--cluster": "value",
            "--cluster-location": "value",
            "--context": "value",
            "--kubeconfig": "value",
            "--platform": [
              "gke",
              "kubernetes",
              "managed"
            ],
            "--region": "value"
          }
        }
      },
      "flags": {}
    },
    "scc": {
      "commands": {
        "assets": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--asset": "value",
                "--resource-name": "value"
              }
            },
            "get-parent": {
              "commands": {},
              "flags": {
                "--asset": "value",
                "--resource-name": "value"
              }
            },
            "get-project": {
              "commands": {},
              "flags": {
                "--asset": "value",
                "--resource-name": "value"
              }
            },
            "group": {
              "commands": {},
              "flags": {
                "--compare-duration": "value",
                "--filter": "value",
                "--group-by": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--compare-duration": "value",
                "--field-mask": "value",
                "--filter": "value",
                "--limit": "value",
                "--order-by": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value",
                "--sort-by": "value"
              }
            },
            "list-marks": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value",
                "--sort-by": "value"
              }
            },
            "run-discovery": {
              "commands": {},
              "flags": {}
            },
            "update-marks": {
              "commands": {},
              "flags": {
                "--organization": "value",
                "--security-marks": "value",
                "--start-time": "value",
                "--update-mask": "value"
              }
            }
          },
          "flags": {}
        },
        "bqexports": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "get": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--dataset": "value",
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value",
                "--update-mask": "value"
              }
            }
          },
          "flags": {}
        },
        "findings": {
          "commands": {
            "bulk-mute": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--organization": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--category": "value",
                "--event-time": "value",
                "--external-uri": "value",
                "--organization": "value",
                "--resource-name": "value",
                "--source": "dynamic",
                "--source-properties": "value",
                "--state": [
                  "active",
                  "inactive",
                  "state-unspecified"
                ]
              }
            },
            "group": {
              "commands": {},
              "flags": {
                "--compare-duration": "value",
                "--filter": "value",
                "--group-by": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value",
                "--source": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--compare-duration": "value",
                "--field-mask": "value",
                "--filter": "value",
                "--limit": "value",
                "--order-by": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value",
                "--sort-by": "value",
                "--source": "value"
              }
            },
            "list-marks": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--page-token": "value",
                "--read-time": "value",
                "--sort-by": "value",
                "--source": "dynamic"
              }
            },
            "set-mute": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--mute": [
                  "muted",
                  "unmuted"
                ],
                "--organization": "value",
                "--source": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--event-time": "value",
                "--external-uri": "value",
                "--organization": "value",
                "--source": "dynamic",
                "--source-properties": "value",
                "--state": [
                  "active",
                  "inactive",
                  "state-unspecified"
                ],
                "--update-mask": "value"
              }
            },
            "update-marks": {
              "commands": {},
              "flags": {
                "--organization": "value",
                "--security-marks": "value",
                "--source": "dynamic",
                "--start-time": "value",
                "--update-mask": "value"
              }
            }
          },
          "flags": {}
        },
        "muteconfigs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "get": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--folder": "value",
                "--limit": "value",
                "--organization": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value",
                "--update-mask": "value"
              }
            }
          },
          "flags": {}
        },
        "notifications": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value",
                "--pubsub-topic": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--folder": "value",
                "--organization": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--description": "value",
                "--filter": "value",
                "--folder": "value",
                "--organization": "value",
                "--pubsub-topic": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--organization": "value"
              }
            }
          },
          "flags": {}
        },
        "sources": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--source": "value",
                "--source-display-name": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "scheduler": {
      "commands": {
        "jobs": {
          "commands": {
            "create": {
              "commands": {
                "app-engine": {
                  "commands": {},
                  "flags": {
                    "--attempt-deadline": "value",
                    "--description": "value",
                    "--headers": "value",
                    "--http-method": [
                      "delete",
                      "get",
                      "head",
                      "post",
                      "put"
                    ],
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--relative-url": "value",
                    "--schedule": "value",
                    "--service": "value",
                    "--time-zone": "value"
                  }
                },
                "http": {
                  "commands": {},
                  "flags": {
                    "--attempt-deadline": "value",
                    "--description": "value",
                    "--headers": "value",
                    "--http-method": [
                      "delete",
                      "get",
                      "head",
                      "post",
                      "put"
                    ],
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--oauth-service-account-email": "value",
                    "--oauth-token-scope": "value",
                    "--oidc-service-account-email": "value",
                    "--oidc-token-audience": "value",
                    "--schedule": "value",
                    "--time-zone": "value",
                    "--uri": "value"
                  }
                },
                "pubsub": {
                  "commands": {},
                  "flags": {
                    "--attributes": "value",
                    "--description": "value",
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--schedule": "value",
                    "--time-zone": "value",
                    "--topic": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "pause": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "resume": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {
                "app-engine": {
                  "commands": {},
                  "flags": {
                    "--attempt-deadline": "value",
                    "--clear-headers": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-attempts": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-message-body": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-relative-url": "bool",
                    "--clear-service": "bool",
                    "--clear-time-zone": "bool",
                    "--description": "value",
                    "--http-method": [
                      "delete",
                      "get",
                      "head",
                      "post",
                      "put"
                    ],
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--relative-url": "value",
                    "--remove-headers": "value",
                    "--schedule": "value",
                    "--service": "value",
                    "--time-zone": "value",
                    "--update-headers": "value"
                  }
                },
                "http": {
                  "commands": {},
                  "flags": {
                    "--attempt-deadline": "value",
                    "--clear-auth-token": "bool",
                    "--clear-headers": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-attempts": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-message-body": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-time-zone": "bool",
                    "--description": "value",
                    "--http-method": [
                      "delete",
                      "get",
                      "head",
                      "post",
                      "put"
                    ],
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--oauth-service-account-email": "value",
                    "--oauth-token-scope": "value",
                    "--oidc-service-account-email": "value",
                    "--oidc-token-audience": "value",
                    "--remove-headers": "value",
                    "--schedule": "value",
                    "--time-zone": "value",
                    "--update-headers": "value",
                    "--uri": "value"
                  }
                },
                "pubsub": {
                  "commands": {},
                  "flags": {
                    "--clear-attributes": "bool",
                    "--clear-max-backoff": "bool",
                    "--clear-max-doublings": "bool",
                    "--clear-max-retry-attempts": "bool",
                    "--clear-max-retry-duration": "bool",
                    "--clear-min-backoff": "bool",
                    "--clear-time-zone": "bool",
                    "--description": "value",
                    "--location": "value",
                    "--max-backoff": "value",
                    "--max-doublings": "value",
                    "--max-retry-attempts": "value",
                    "--max-retry-duration": "value",
                    "--message-body": "value",
                    "--message-body-from-file": "value",
                    "--min-backoff": "value",
                    "--remove-attributes": "value",
                    "--schedule": "value",
                    "--time-zone": "value",
                    "--topic": "value",
                    "--update-attributes": "value"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "secrets": {
      "commands": {
        "add-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "create": {
          "commands": {},
          "flags": {
            "--data-file": "value",
            "--expire-time": "value",
            "--kms-key-name": "value",
            "--labels": "value",
            "--locations": "value",
            "--next-rotation-time": "value",
            "--replication-policy": "value",
            "--replication-policy-file": "value",
            "--rotation-period": "value",
            "--set-annotations": "value",
            "--topics": "value",
            "--ttl": "value"
          }
        },
        "delete": {
          "commands": {},
          "flags": {
            "--etag": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {}
        },
        "get-iam-policy": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "remove-iam-policy-binding": {
          "commands": {},
          "flags": {
            "--all": "bool",
            "--condition": "value",
            "--condition-from-file": "value",
            "--member": "value",
            "--role": "dynamic"
          }
        },
        "replication": {
          "commands": {
            "get": {
              "commands": {},
              "flags": {}
            },
            "set": {
              "commands": {},
              "flags": {
                "--replication-policy-file": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--remove-cmek": "bool",
                "--set-kms-key": "value"
              }
            }
          },
          "flags": {}
        },
        "set-iam-policy": {
          "commands": {},
          "flags": {}
        },
        "update": {
          "commands": {},
          "flags": {
            "--add-topics": "value",
            "--clear-annotations": "bool",
            "--clear-labels": "bool",
            "--clear-topics": "bool",
            "--clear-version-aliases": "bool",
            "--etag": "value",
            "--expire-time": "value",
            "--next-rotation-time": "value",
            "--remove-annotations": "value",
            "--remove-expiration": "bool",
            "--remove-labels": "value",
            "--remove-next-rotation-time": "bool",
            "--remove-rotation-period": "bool",
            "--remove-rotation-schedule": "bool",
            "--remove-topics": "value",
            "--remove-version-aliases": "value",
            "--rotation-period": "value",
            "--ttl": "value",
            "--update-annotations": "value",
            "--update-labels": "value",
            "--update-version-aliases": "value"
          }
        },
        "versions": {
          "commands": {
            "access": {
              "commands": {},
              "flags": {
                "--out-file": "value",
                "--secret": "dynamic"
              }
            },
            "add": {
              "commands": {},
              "flags": {
                "--data-file": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--secret": "dynamic"
              }
            },
            "destroy": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--secret": "dynamic"
              }
            },
            "disable": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--secret": "dynamic"
              }
            },
            "enable": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--secret": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "service-directory": {
      "commands": {
        "endpoints": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--address": "value",
                "--annotations": "value",
                "--location": "value",
                "--namespace": "value",
                "--network": "value",
                "--port": "value",
                "--service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value",
                "--service": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value",
                "--service": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--location": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--service": "value",
                "--sort-by": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--address": "value",
                "--annotations": "value",
                "--location": "value",
                "--namespace": "value",
                "--port": "value",
                "--service": "value"
              }
            }
          },
          "flags": {}
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "namespaces": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--labels": "value",
                "--location": "value"
              }
            }
          },
          "flags": {}
        },
        "services": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--namespace": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--location": "value",
                "--namespace": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--location": "value",
                "--namespace": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--namespace": "value",
                "--role": "value"
              }
            },
            "resolve": {
              "commands": {},
              "flags": {
                "--endpoint-filter": "value",
                "--location": "value",
                "--max-endpoints": "value",
                "--namespace": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--namespace": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--annotations": "value",
                "--location": "value",
                "--namespace": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "services": {
      "commands": {
        "api-keys": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--show-deleted": "bool",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "disable": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--force": "bool"
          }
        },
        "enable": {
          "commands": {},
          "flags": {
            "--async": "bool"
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--available": "bool",
            "--enabled": "bool",
            "--filter": "value",
            "--limit": "value",
            "--page-size": "value",
            "--sort-by": "value"
          }
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--full": "value"
              }
            },
            "wait": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "peered-dns-domains": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--dns-suffix": "value",
                "--network": "value",
                "--service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--network": "value",
                "--service": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--network": "value",
                "--service": "value"
              }
            }
          },
          "flags": {}
        },
        "vpc-peerings": {
          "commands": {
            "connect": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--network": "value",
                "--ranges": "value",
                "--service": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--network": "value",
                "--service": "value"
              }
            },
            "disable-vpc-service-controls": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--network": "value",
                "--service": "value"
              }
            },
            "enable-vpc-service-controls": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--network": "value",
                "--service": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--network": "value",
                "--service": "value"
              }
            },
            "operations": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--name": "value"
                  }
                },
                "wait": {
                  "commands": {},
                  "flags": {
                    "--name": "value"
                  }
                }
              },
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--force": "bool",
                "--network": "value",
                "--ranges": "value",
                "--service": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "source": {
      "commands": {
        "project-configs": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-topic": "value",
                "--disable-pushblock": "bool",
                "--enable-pushblock": "bool",
                "--message-format": [
                  "json",
                  "protobuf"
                ],
                "--remove-topic": "value",
                "--service-account": "value",
                "--topic-project": "value",
                "--update-topic": "value"
              }
            }
          },
          "flags": {}
        },
        "repos": {
          "commands": {
            "clone": {
              "commands": {},
              "flags": {
                "--dry-run": "bool"
              }
            },
            "create": {
              "commands": {},
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--force": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--add-topic": "value",
                "--message-format": [
                  "json",
                  "protobuf"
                ],
                "--remove-topic": "value",
                "--service-account": "value",
                "--topic-project": "value",
                "--update-topic": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "spanner": {
      "commands": {
        "backups": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "copy": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--destination-backup": "value",
                "--destination-instance": "value",
                "--encryption-type": [
                  "customer-managed-encryption",
                  "google-default-encryption",
                  "use-config-default-or-backup-encryption"
                ],
                "--expiration-date": "value",
                "--kms-key": "value",
                "--kms-keyring": "value",
                "--kms-location": "value",
                "--kms-project": "value",
                "--retention-period": "value",
                "--source-backup": "value",
                "--source-instance": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--encryption-type": [
                  "customer-managed-encryption",
                  "google-default-encryption",
                  "use-database-encryption"
                ],
                "--expiration-date": "value",
                "--instance": "value",
                "--kms-key": "value",
                "--kms-keyring": "value",
                "--kms-location": "value",
                "--kms-project": "value",
                "--retention-period": "value",
                "--version-time": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--database": "value",
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "update-metadata": {
              "commands": {},
              "flags": {
                "--expiration-date": "value",
                "--instance": "dynamic",
                "--retention-period": "value"
              }
            }
          },
          "flags": {}
        },
        "databases": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database-dialect": [
                  "GOOGLE_STANDARD_SQL",
                  "POSTGRESQL"
                ],
                "--ddl": "value",
                "--ddl-file": "dynamic",
                "--instance": "value",
                "--kms-key": "value",
                "--kms-keyring": "value",
                "--kms-location": "value",
                "--kms-project": "value"
              }
            },
            "ddl": {
              "commands": {
                "describe": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--ddl": "value",
                    "--ddl-file": "dynamic",
                    "--instance": "value"
                  }
                }
              },
              "flags": {}
            },
            "delete": {
              "commands": {},
              "flags": {
                "--instance": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "value"
              }
            },
            "execute-sql": {
              "commands": {},
              "flags": {
                "--database-role": "value",
                "--enable-partitioned-dml": "bool",
                "--instance": "value",
                "--query-mode": [
                  "NORMAL",
                  "PLAN",
                  "PROFILE"
                ],
                "--read-timestamp": "value",
                "--sql": "value",
                "--strong": "bool",
                "--timeout": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--instance": "dynamic",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--destination-database": "value",
                "--destination-instance": "value",
                "--encryption-type": [
                  "customer-managed-encryption",
                  "google-default-encryption",
                  "use-config-default-or-backup-encryption"
                ],
                "--kms-key": "value",
                "--kms-keyring": "value",
                "--kms-location": "value",
                "--kms-project": "value",
                "--source-backup": "value",
                "--source-instance": "value"
              }
            },
            "roles": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "sessions": {
              "commands": {
                "delete": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--instance": "value"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--database": "value",
                    "--filter": "value",
                    "--instance": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--server-filter": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--instance": "value"
              }
            }
          },
          "flags": {}
        },
        "instance-configs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--add-replicas": "value",
                "--async": "bool",
                "--base-config": "value",
                "--clone-config": "dynamic",
                "--display-name": "value",
                "--etag": "value",
                "--labels": "value",
                "--replicas": "value",
                "--skip-replicas": "value",
                "--validate-only": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--etag": "value",
                "--validate-only": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-labels": "bool",
                "--display-name": "value",
                "--etag": "value",
                "--remove-labels": "value",
                "--update-labels": "value",
                "--validate-only": "bool"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--config": "dynamic",
                "--description": "value",
                "--expire-behavior": [
                  "free-to-provisioned",
                  "remove-after-grace-period"
                ],
                "--instance-type": [
                  "free-instance",
                  "provisioned"
                ],
                "--nodes": "value",
                "--processing-units": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--expire-behavior": [
                  "free-to-provisioned",
                  "remove-after-grace-period"
                ],
                "--instance-type": [
                  "free-instance",
                  "provisioned"
                ],
                "--nodes": "value",
                "--processing-units": "value"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--backup": "dynamic",
                "--database": "dynamic",
                "--instance": "dynamic",
                "--instance-config": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--backup": "dynamic",
                "--database": "dynamic",
                "--instance": "dynamic",
                "--instance-config": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--backup": "dynamic",
                "--database": "dynamic",
                "--filter": "value",
                "--instance": "dynamic",
                "--instance-config": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--type": [
                  "BACKUP",
                  "DATABASE",
                  "DATABASE_CREATE",
                  "DATABASE_RESTORE",
                  "DATABASE_UPDATE_DDL",
                  "INSTANCE",
                  "INSTANCE_CONFIG_CREATE",
                  "INSTANCE_CONFIG_UPDATE"
                ],
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "rows": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--database": "value",
                "--instance": "value",
                "--keys": "value",
                "--table": "value"
              }
            },
            "insert": {
              "commands": {},
              "flags": {
                "--data": "value",
                "--database": "value",
                "--instance": "value",
                "--table": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--data": "value",
                "--database": "value",
                "--instance": "value",
                "--table": "value"
              }
            }
          },
          "flags": {}
        },
        "samples": {
          "commands": {
            "backend": {
              "commands": {},
              "flags": {
                "--database-id": "value",
                "--duration": "value",
                "--instance-id": "value",
                "--port": "value"
              }
            },
            "init": {
              "commands": {},
              "flags": {
                "--database-id": "value",
                "--instance-id": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "run": {
              "commands": {},
              "flags": {
                "--cleanup": "bool",
                "--database-id": "value",
                "--duration": "value",
                "--instance-id": "value",
                "--skip-init": "bool"
              }
            },
            "workload": {
              "commands": {},
              "flags": {
                "--duration": "value",
                "--port": "value",
                "--target-qps": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "sql": {
      "commands": {
        "backups": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--instance": "dynamic",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "restore": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backup-instance": "dynamic",
                "--restore-instance": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "connect": {
          "commands": {},
          "flags": {
            "--database": "value",
            "--user": "value"
          }
        },
        "databases": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--charset": "value",
                "--collation": "value",
                "--instance": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "patch": {
              "commands": {},
              "flags": {
                "--charset": "value",
                "--collation": "value",
                "--diff": "bool",
                "--instance": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "export": {
          "commands": {
            "bak": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--stripe_count": "value",
                "--striped": "bool"
              }
            },
            "csv": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--escape": "value",
                "--fields-terminated-by": "value",
                "--lines-terminated-by": "value",
                "--offload": "bool",
                "--query": "value",
                "--quote": "value"
              }
            },
            "sql": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--offload": "bool",
                "--table": "value"
              }
            }
          },
          "flags": {}
        },
        "flags": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--database-version": [
                  "MYSQL_5_6",
                  "MYSQL_5_7",
                  "MYSQL_8_0",
                  "POSTGRES_10",
                  "POSTGRES_11",
                  "POSTGRES_12",
                  "POSTGRES_13",
                  "POSTGRES_14",
                  "POSTGRES_9_6",
                  "SQLSERVER_2017_ENTERPRISE",
                  "SQLSERVER_2017_EXPRESS",
                  "SQLSERVER_2017_STANDARD",
                  "SQLSERVER_2017_WEB",
                  "SQLSERVER_2019_ENTERPRISE",
                  "SQLSERVER_2019_EXPRESS",
                  "SQLSERVER_2019_STANDARD",
                  "SQLSERVER_2019_WEB"
                ],
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "generate-login-token": {
          "commands": {},
          "flags": {
            "--application-default-credential": "bool",
            "--instance": "dynamic"
          }
        },
        "import": {
          "commands": {
            "bak": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cert-path": "value",
                "--database": "value",
                "--prompt-for-pvk-password": "bool",
                "--pvk-password": "value",
                "--pvk-path": "value",
                "--striped": "bool"
              }
            },
            "csv": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--columns": "value",
                "--database": "value",
                "--escape": "value",
                "--fields-terminated-by": "value",
                "--lines-terminated-by": "value",
                "--quote": "value",
                "--table": "value",
                "--user": "value"
              }
            },
            "sql": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--user": "value"
              }
            }
          },
          "flags": {}
        },
        "instances": {
          "commands": {
            "clone": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--bin-log-file-name": "value",
                "--bin-log-position": "value",
                "--point-in-time": "value",
                "--restore-database-name": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--activation-policy": [
                  "always",
                  "never",
                  "on-demand"
                ],
                "--active-directory-domain": "value",
                "--assign-ip": "bool",
                "--async": "bool",
                "--audit-bucket-path": "value",
                "--audit-retention-interval": "value",
                "--audit-upload-interval": "value",
                "--authorized-networks": "value",
                "--availability-type": [
                  "regional",
                  "zonal"
                ],
                "--backup": "bool",
                "--backup-location": "value",
                "--backup-start-time": "value",
                "--collation": "value",
                "--connector-enforcement": [
                  "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                  "NOT_REQUIRED",
                  "REQUIRED"
                ],
                "--cpu": "value",
                "--database-flags": "value",
                "--database-version": [
                  "MYSQL_5_6",
                  "MYSQL_5_7",
                  "MYSQL_8_0",
                  "POSTGRES_10",
                  "POSTGRES_11",
                  "POSTGRES_12",
                  "POSTGRES_13",
                  "POSTGRES_14",
                  "POSTGRES_9_6",
                  "SQLSERVER_2017_ENTERPRISE",
                  "SQLSERVER_2017_EXPRESS",
                  "SQLSERVER_2017_STANDARD",
                  "SQLSERVER_2017_WEB",
                  "SQLSERVER_2019_ENTERPRISE",
                  "SQLSERVER_2019_EXPRESS",
                  "SQLSERVER_2019_STANDARD",
                  "SQLSERVER_2019_WEB"
                ],
                "--deletion-protection": "bool",
                "--deny-maintenance-period-end-date": "value",
                "--deny-maintenance-period-start-date": "value",
                "--deny-maintenance-period-time": "value",
                "--disk-encryption-key": "dynamic",
                "--disk-encryption-key-keyring": "dynamic",
                "--disk-encryption-key-location": "dynamic",
                "--disk-encryption-key-project": "dynamic",
                "--enable-bin-log": "bool",
                "--enable-google-private-path": "bool",
                "--enable-password-policy": "bool",
                "--enable-point-in-time-recovery": "bool",
                "--failover-replica-name": "value",
                "--gce-zone": "value",
                "--insights-config-query-insights-enabled": "bool",
                "--insights-config-query-plans-per-minute": "value",
                "--insights-config-query-string-length": "value",
                "--insights-config-record-application-tags": "bool",
                "--insights-config-record-client-address": "bool",
                "--maintenance-release-channel": [
                  "preview",
                  "production"
                ],
                "--maintenance-window-day": [
                  "FRI",
                  "MON",
                  "SAT",
                  "SUN",
                  "THU",
                  "TUE",
                  "WED"
                ],
                "--maintenance-window-hour": "value",
                "--master-instance-name": "value",
                "--memory": "value",
                "--network": "value",
                "--password-policy-complexity": [
                  "COMPLEXITY_DEFAULT",
                  "COMPLEXITY_UNSPECIFIED"
                ],
                "--password-policy-disallow-username-substring": "bool",
                "--password-policy-min-length": "value",
                "--password-policy-password-change-interval": "value",
                "--password-policy-reuse-interval": "value",
                "--region": "value",
                "--replica-type": [
                  "FAILOVER",
                  "READ"
                ],
                "--replication": [
                  "asynchronous",
                  "synchronous"
                ],
                "--require-ssl": "bool",
                "--retained-backups-count": "value",
                "--retained-transaction-log-days": "value",
                "--root-password": "value",
                "--secondary-zone": "value",
                "--storage-auto-increase": "bool",
                "--storage-size": "value",
                "--storage-type": [
                  "HDD",
                  "SSD"
                ],
                "--tier": "value",
                "--time-zone": "value",
                "--timeout": "value",
                "--zone": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "export": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value",
                "--table": "value"
              }
            },
            "failover": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "import": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--database": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "patch": {
              "commands": {},
              "flags": {
                "--activation-policy": [
                  "always",
                  "never",
                  "on-demand"
                ],
                "--active-directory-domain": "value",
                "--assign-ip": "bool",
                "--async": "bool",
                "--audit-bucket-path": "value",
                "--audit-retention-interval": "value",
                "--audit-upload-interval": "value",
                "--authorized-gae-apps": "value",
                "--authorized-networks": "value",
                "--availability-type": [
                  "regional",
                  "zonal"
                ],
                "--backup-location": "value",
                "--backup-start-time": "value",
                "--clear-authorized-networks": "bool",
                "--clear-database-flags": "bool",
                "--clear-gae-apps": "bool",
                "--clear-password-policy": "bool",
                "--connector-enforcement": [
                  "CONNECTOR_ENFORCEMENT_UNSPECIFIED",
                  "NOT_REQUIRED",
                  "REQUIRED"
                ],
                "--cpu": "value",
                "--database-flags": "value",
                "--database-version": [
                  "MYSQL_5_6",
                  "MYSQL_5_7",
                  "MYSQL_8_0",
                  "POSTGRES_10",
                  "POSTGRES_11",
                  "POSTGRES_12",
                  "POSTGRES_13",
                  "POSTGRES_14",
                  "POSTGRES_9_6",
                  "SQLSERVER_2017_ENTERPRISE",
                  "SQLSERVER_2017_EXPRESS",
                  "SQLSERVER_2017_STANDARD",
                  "SQLSERVER_2017_WEB",
                  "SQLSERVER_2019_ENTERPRISE",
                  "SQLSERVER_2019_EXPRESS",
                  "SQLSERVER_2019_STANDARD",
                  "SQLSERVER_2019_WEB"
                ],
                "--deletion-protection": "bool",
                "--deny-maintenance-period-end-date": "value",
                "--deny-maintenance-period-start-date": "value",
                "--deny-maintenance-period-time": "value",
                "--diff": "bool",
                "--enable-bin-log": "bool",
                "--enable-database-replication": "bool",
                "--enable-google-private-path": "bool",
                "--enable-password-policy": "bool",
                "--enable-point-in-time-recovery": "bool",
                "--follow-gae-app": "value",
                "--gce-zone": "value",
                "--insights-config-query-insights-enabled": "bool",
                "--insights-config-query-plans-per-minute": "value",
                "--insights-config-query-string-length": "value",
                "--insights-config-record-application-tags": "bool",
                "--insights-config-record-client-address": "bool",
                "--maintenance-release-channel": [
                  "preview",
                  "production"
                ],
                "--maintenance-version": "value",
                "--maintenance-window-any": "bool",
                "--maintenance-window-day": [
                  "FRI",
                  "MON",
                  "SAT",
                  "SUN",
                  "THU",
                  "TUE",
                  "WED"
                ],
                "--maintenance-window-hour": "value",
                "--memory": "value",
                "--network": "value",
                "--no-backup": "bool",
                "--password-policy-complexity": [
                  "COMPLEXITY_DEFAULT",
                  "COMPLEXITY_UNSPECIFIED"
                ],
                "--password-policy-disallow-username-substring": "bool",
                "--password-policy-min-length": "value",
                "--password-policy-password-change-interval": "value",
                "--password-policy-reuse-interval": "value",
                "--pricing-plan": [
                  "PACKAGE",
                  "PER_USE"
                ],
                "--remove-deny-maintenance-period": "bool",
                "--replication": [
                  "asynchronous",
                  "synchronous"
                ],
                "--require-ssl": "bool",
                "--retained-backups-count": "value",
                "--retained-transaction-log-days": "value",
                "--secondary-zone": "value",
                "--storage-auto-increase": "bool",
                "--storage-size": "value",
                "--tier": "value",
                "--zone": "value"
              }
            },
            "promote-replica": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "reset-ssl-config": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "restart": {
              "commands": {},
              "flags": {
                "--async": "bool"
              }
            },
            "restore-backup": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--backup-id": "value",
                "--backup-instance": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--timeout": "value"
              }
            }
          },
          "flags": {}
        },
        "reschedule-maintenance": {
          "commands": {},
          "flags": {
            "--reschedule-type": [
              "IMMEDIATE",
              "NEXT_AVAILABLE_WINDOW",
              "SPECIFIC_TIME"
            ],
            "--schedule-time": "value"
          }
        },
        "ssl": {
          "commands": {
            "client-certs": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--instance": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--instance": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--instance": "dynamic",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        },
        "ssl-certs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "tiers": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "users": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--host": "value",
                "--instance": "dynamic",
                "--password": "value",
                "--password-policy-allowed-failed-attempts": "value",
                "--password-policy-enable-failed-attempts-check": "bool",
                "--password-policy-enable-password-verification": "bool",
                "--password-policy-password-expiration-duration": "value",
                "--type": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--host": "value",
                "--instance": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--host": "value",
                "--instance": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--instance": "dynamic",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "set-password": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--discard-dual-password": "bool",
                "--host": "value",
                "--instance": "dynamic",
                "--password": "value",
                "--prompt-for-password": "bool",
                "--retain-password": "bool"
              }
            },
            "set-password-policy": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--clear-password-policy": "bool",
                "--host": "value",
                "--instance": "dynamic",
                "--password-policy-allowed-failed-attempts": "value",
                "--password-policy-enable-failed-attempts-check": "bool",
                "--password-policy-enable-password-verification": "bool",
                "--password-policy-password-expiration-duration": "value"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "storage": {
      "commands": {
        "buckets": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--default-encryption-key": "value",
                "--default-storage-class": "value",
                "--enable-autoclass": "bool",
                "--location": "value",
                "--pap": "bool",
                "--placement": "value",
                "--public-access-prevention": "bool",
                "--recovery-point-objective": [
                  "ASYNC_TURBO",
                  "DEFAULT"
                ],
                "--retention-period": "value",
                "--rpo": [
                  "ASYNC_TURBO",
                  "DEFAULT"
                ],
                "--uniform-bucket-level-access": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--continue-on-error": "bool"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--additional-headers": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "notifications": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--custom-attributes": "value",
                    "--event-types": "value",
                    "--object-prefix": "value",
                    "--payload-format": [
                      "json",
                      "none"
                    ],
                    "--skip-topic-setup": "bool",
                    "--topic": "value"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {}
                },
                "describe": {
                  "commands": {},
                  "flags": {}
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--page-size": "value",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--condition": "value",
                "--condition-from-file": "value",
                "--member": "value",
                "--role": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--continue-on-error": "bool",
                "--etag": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--acl-file": "value",
                "--add-acl-grant": "value",
                "--add-default-object-acl-grant": "value",
                "--additional-headers": "value",
                "--canned-acl": "value",
                "--clear-cors": "bool",
                "--clear-default-encryption-key": "bool",
                "--clear-labels": "bool",
                "--clear-lifecycle": "bool",
                "--clear-log-bucket": "bool",
                "--clear-log-object-prefix": "bool",
                "--clear-pap": "bool",
                "--clear-public-access-prevention": "bool",
                "--clear-retention-period": "bool",
                "--clear-web-error-page": "bool",
                "--clear-web-main-page-suffix": "bool",
                "--continue-on-error": "bool",
                "--cors-file": "value",
                "--default-encryption-key": "value",
                "--default-event-based-hold": "bool",
                "--default-object-acl-file": "value",
                "--enable-autoclass": "bool",
                "--labels-file": "value",
                "--lifecycle-file": "value",
                "--lock-retention-period": "bool",
                "--log-bucket": "value",
                "--log-object-prefix": "value",
                "--pap": "bool",
                "--predefined-acl": "value",
                "--predefined-default-object-acl": "value",
                "--public-access-prevention": "bool",
                "--recovery-point-objective": [
                  "ASYNC_TURBO",
                  "DEFAULT"
                ],
                "--remove-acl-grant": "value",
                "--remove-default-object-acl-grant": "value",
                "--remove-labels": "value",
                "--requester-pays": "bool",
                "--retention-period": "value",
                "--rpo": [
                  "ASYNC_TURBO",
                  "DEFAULT"
                ],
                "--uniform-bucket-level-access": "bool",
                "--update-labels": "value",
                "--versioning": "bool",
                "--web-error-page": "value",
                "--web-main-page-suffix": "value"
              }
            }
          },
          "flags": {}
        },
        "cat": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--decryption-keys": "value",
            "--display-url": "bool",
            "--range": "value"
          }
        },
        "cp": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--all-versions": "bool",
            "--cache-control": "value",
            "--canned-acl": "value",
            "--clear-custom-metadata": "bool",
            "--content-disposition": "value",
            "--content-encoding": "value",
            "--content-language": "value",
            "--content-md5": "value",
            "--content-type": "value",
            "--continue-on-error": "bool",
            "--custom-metadata": "value",
            "--custom-time": "value",
            "--daisy-chain": "bool",
            "--decryption-keys": "value",
            "--do-not-decompress": "bool",
            "--encryption-key": "value",
            "--gzip-in-flight": "value",
            "--gzip-in-flight-all": "bool",
            "--gzip-local": "value",
            "--gzip-local-all": "bool",
            "--if-generation-match": "value",
            "--if-metageneration-match": "value",
            "--ignore-symlinks": "bool",
            "--manifest-path": "value",
            "--no-clobber": "bool",
            "--predefined-acl": "value",
            "--preserve-acl": "bool",
            "--preserve-posix": "bool",
            "--print-created-message": "bool",
            "--read-paths-from-stdin": "bool",
            "--recursive": "bool",
            "--remove-custom-metadata": "value",
            "--skip-unsupported": "bool",
            "--storage-class": "value",
            "--update-custom-metadata": "value"
          }
        },
        "hash": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--decryption-keys": "value",
            "--hex": "bool",
            "--skip-crc32c": "bool",
            "--skip-md5": "bool"
          }
        },
        "ls": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--all-versions": "bool",
            "--buckets": "bool",
            "--decryption-keys": "value",
            "--etag": "bool",
            "--fetch-encrypted-object-hashes": "bool",
            "--full": "bool",
            "--json": "bool",
            "--long": "bool",
            "--readable-sizes": "bool",
            "--recursive": "bool"
          }
        },
        "mv": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--all-versions": "bool",
            "--cache-control": "value",
            "--canned-acl": "value",
            "--clear-custom-metadata": "bool",
            "--content-disposition": "value",
            "--content-encoding": "value",
            "--content-language": "value",
            "--content-md5": "value",
            "--content-type": "value",
            "--continue-on-error": "bool",
            "--custom-metadata": "value",
            "--custom-time": "value",
            "--daisy-chain": "bool",
            "--decryption-keys": "value",
            "--do-not-decompress": "bool",
            "--encryption-key": "value",
            "--gzip-in-flight": "value",
            "--gzip-in-flight-all": "bool",
            "--gzip-local": "value",
            "--gzip-local-all": "bool",
            "--if-generation-match": "value",
            "--if-metageneration-match": "value",
            "--ignore-symlinks": "bool",
            "--manifest-path": "value",
            "--no-clobber": "bool",
            "--predefined-acl": "value",
            "--preserve-acl": "bool",
            "--preserve-posix": "bool",
            "--print-created-message": "bool",
            "--read-paths-from-stdin": "bool",
            "--remove-custom-metadata": "value",
            "--skip-unsupported": "bool",
            "--storage-class": "value",
            "--update-custom-metadata": "value"
          }
        },
        "objects": {
          "commands": {
            "compose": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--if-generation-match": "value",
                "--if-metageneration-match": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--decryption-keys": "value",
                "--fetch-encrypted-object-hashes": "bool"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--additional-headers": "value",
                "--decryption-keys": "value",
                "--fetch-encrypted-object-hashes": "bool",
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--stat": "bool",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--acl-file": "value",
                "--add-acl-grant": "value",
                "--additional-headers": "value",
                "--all-versions": "bool",
                "--cache-control": "value",
                "--canned-acl": "value",
                "--clear-cache-control": "bool",
                "--clear-content-disposition": "bool",
                "--clear-content-encoding": "bool",
                "--clear-content-language": "bool",
                "--clear-content-md5": "bool",
                "--clear-content-type": "bool",
                "--clear-custom-metadata": "bool",
                "--clear-custom-time": "bool",
                "--clear-encryption-key": "bool",
                "--content-disposition": "value",
                "--content-encoding": "value",
                "--content-language": "value",
                "--content-md5": "value",
                "--content-type": "value",
                "--continue-on-error": "bool",
                "--custom-metadata": "value",
                "--custom-time": "value",
                "--decryption-keys": "value",
                "--encryption-key": "value",
                "--event-based-hold": "bool",
                "--if-generation-match": "value",
                "--if-metageneration-match": "value",
                "--predefined-acl": "value",
                "--preserve-acl": "bool",
                "--read-paths-from-stdin": "bool",
                "--recursive": "bool",
                "--remove-acl-grant": "value",
                "--remove-custom-metadata": "value",
                "--storage-class": "value",
                "--temporary-hold": "bool",
                "--update-custom-metadata": "value"
              }
            }
          },
          "flags": {}
        },
        "rm": {
          "commands": {},
          "flags": {
            "--additional-headers": "value",
            "--all-versions": "bool",
            "--continue-on-error": "bool",
            "--if-generation-match": "value",
            "--if-metageneration-match": "value",
            "--read-paths-from-stdin": "bool",
            "--recursive": "bool"
          }
        },
        "service-agent": {
          "commands": {},
          "flags": {
            "--authorize-cmek": "value"
          }
        }
      },
      "flags": {}
    },
    "survey": {
      "commands": {},
      "flags": {}
    },
    "tasks": {
      "commands": {
        "create-app-engine-task": {
          "commands": {},
          "flags": {
            "--body-content": "value",
            "--body-file": "value",
            "--header": "value",
            "--location": "value",
            "--method": "value",
            "--queue": "value",
            "--relative-uri": "value",
            "--routing": "value",
            "--schedule-time": "value"
          }
        },
        "create-http-task": {
          "commands": {},
          "flags": {
            "--body-content": "value",
            "--body-file": "value",
            "--header": "value",
            "--location": "value",
            "--method": "value",
            "--oauth-service-account-email": "value",
            "--oauth-token-scope": "value",
            "--oidc-service-account-email": "value",
            "--oidc-token-audience": "value",
            "--queue": "value",
            "--schedule-time": "value",
            "--url": "value"
          }
        },
        "delete": {
          "commands": {},
          "flags": {
            "--location": "value",
            "--queue": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--location": "value",
            "--queue": "value",
            "--response-view": [
              "basic",
              "full",
              "view-unspecified"
            ]
          }
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--location": "value",
            "--page-size": "value",
            "--queue": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "locations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "queues": {
          "commands": {
            "add-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "create": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--log-sampling-ratio": "value",
                "--max-attempts": "value",
                "--max-backoff": "value",
                "--max-concurrent-dispatches": "value",
                "--max-dispatches-per-second": "value",
                "--max-doublings": "value",
                "--max-retry-duration": "value",
                "--min-backoff": "value",
                "--routing-override": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "get-iam-policy": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "pause": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "purge": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "remove-iam-policy-binding": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--member": "value",
                "--role": "dynamic"
              }
            },
            "resume": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "set-iam-policy": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-log-sampling-ratio": "bool",
                "--clear-max-attempts": "bool",
                "--clear-max-backoff": "bool",
                "--clear-max-concurrent-dispatches": "bool",
                "--clear-max-dispatches-per-second": "bool",
                "--clear-max-doublings": "bool",
                "--clear-max-retry-duration": "bool",
                "--clear-min-backoff": "bool",
                "--clear-routing-override": "bool",
                "--location": "value",
                "--log-sampling-ratio": "value",
                "--max-attempts": "value",
                "--max-backoff": "value",
                "--max-concurrent-dispatches": "value",
                "--max-dispatches-per-second": "value",
                "--max-doublings": "value",
                "--max-retry-duration": "value",
                "--min-backoff": "value",
                "--routing-override": "value"
              }
            }
          },
          "flags": {}
        },
        "run": {
          "commands": {},
          "flags": {
            "--location": "value",
            "--queue": "value"
          }
        }
      },
      "flags": {}
    },
    "topic": {
      "commands": {
        "accessibility": {
          "commands": {},
          "flags": {}
        },
        "arg-files": {
          "commands": {},
          "flags": {}
        },
        "cli-trees": {
          "commands": {},
          "flags": {}
        },
        "client-certificate": {
          "commands": {},
          "flags": {}
        },
        "command-conventions": {
          "commands": {},
          "flags": {}
        },
        "configurations": {
          "commands": {},
          "flags": {}
        },
        "datetimes": {
          "commands": {},
          "flags": {}
        },
        "endpoint-override": {
          "commands": {},
          "flags": {}
        },
        "escaping": {
          "commands": {},
          "flags": {}
        },
        "filters": {
          "commands": {},
          "flags": {}
        },
        "flags-file": {
          "commands": {},
          "flags": {}
        },
        "formats": {
          "commands": {},
          "flags": {}
        },
        "gcloudignore": {
          "commands": {},
          "flags": {}
        },
        "offline-help": {
          "commands": {},
          "flags": {}
        },
        "projections": {
          "commands": {},
          "flags": {}
        },
        "resource-keys": {
          "commands": {},
          "flags": {}
        },
        "startup": {
          "commands": {},
          "flags": {}
        },
        "uninstall": {
          "commands": {},
          "flags": {}
        }
      },
      "flags": {}
    },
    "transcoder": {
      "commands": {
        "jobs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--input-uri": "value",
                "--json": "value",
                "--labels": "value",
                "--location": "value",
                "--output-uri": "value",
                "--template-id": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "templates": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--file": "value",
                "--json": "value",
                "--labels": "value",
                "--location": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "transfer": {
      "commands": {
        "agent-pools": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--bandwidth-limit": "value",
                "--display-name": "value",
                "--no-async": "bool"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--limit": "value",
                "--names": "value",
                "--page-size": "value"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--bandwidth-limit": "value",
                "--clear-bandwidth-limit": "bool",
                "--clear-display-name": "bool",
                "--display-name": "value"
              }
            }
          },
          "flags": {}
        },
        "agents": {
          "commands": {
            "delete": {
              "commands": {},
              "flags": {
                "--all": "bool",
                "--ids": "value",
                "--uninstall": "bool"
              }
            },
            "install": {
              "commands": {},
              "flags": {
                "--count": "value",
                "--creds-file": "value",
                "--enable-multipart": "bool",
                "--id-prefix": "value",
                "--logs-directory": "value",
                "--memlock-limit": "value",
                "--mount-directories": "value",
                "--pool": "value",
                "--proxy": "value",
                "--s3-compatible-mode": "bool"
              }
            }
          },
          "flags": {}
        },
        "authorize": {
          "commands": {},
          "flags": {
            "--add-missing": "bool",
            "--creds-file": "value"
          }
        },
        "jobs": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--custom-storage-class": "value",
                "--delete-from": [
                  "destination-if-unique",
                  "source-after-transfer"
                ],
                "--description": "value",
                "--destination-agent-pool": "value",
                "--do-not-run": "bool",
                "--enable-posix-transfer-logs": "bool",
                "--event-stream-expires": "value",
                "--event-stream-name": "value",
                "--event-stream-starts": "value",
                "--exclude-prefixes": "value",
                "--include-modified-after-absolute": "value",
                "--include-modified-after-relative": "value",
                "--include-modified-before-absolute": "value",
                "--include-modified-before-relative": "value",
                "--include-prefixes": "value",
                "--intermediate-storage-path": "value",
                "--log-action-states": "value",
                "--log-actions": "value",
                "--manifest-file": "value",
                "--name": "value",
                "--no-async": "bool",
                "--notification-event-types": "value",
                "--notification-payload-format": [
                  "json",
                  "none"
                ],
                "--notification-pubsub-topic": "value",
                "--overwrite-when": [
                  "always",
                  "different",
                  "never"
                ],
                "--preserve-metadata": "value",
                "--schedule-repeats-every": "value",
                "--schedule-repeats-until": "value",
                "--schedule-starts": "value",
                "--source-agent-pool": "value",
                "--source-auth-method": [
                  "AWS_SIGNATURE_V2",
                  "AWS_SIGNATURE_V4"
                ],
                "--source-creds-file": "value",
                "--source-endpoint": "value",
                "--source-list-api": [
                  "LIST_OBJECTS",
                  "LIST_OBJECTS_V2"
                ],
                "--source-network-protocol": [
                  "HTTP",
                  "HTTPS"
                ],
                "--source-request-model": [
                  "PATH_STYLE",
                  "VIRTUAL_HOSTED_STYLE"
                ],
                "--source-signing-region": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--expand-table": "bool",
                "--job-names": "value",
                "--job-statuses": "value",
                "--limit": "value",
                "--page-size": "value"
              }
            },
            "monitor": {
              "commands": {},
              "flags": {}
            },
            "run": {
              "commands": {},
              "flags": {
                "--no-async": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--clear-custom-storage-class": "bool",
                "--clear-delete-from": "bool",
                "--clear-description": "bool",
                "--clear-destination-agent-pool": "bool",
                "--clear-event-stream": "bool",
                "--clear-exclude-prefixes": "bool",
                "--clear-include-modified-after-absolute": "bool",
                "--clear-include-modified-after-relative": "bool",
                "--clear-include-modified-before-absolute": "bool",
                "--clear-include-modified-before-relative": "bool",
                "--clear-include-prefixes": "bool",
                "--clear-intermediate-storage-path": "bool",
                "--clear-log-config": "bool",
                "--clear-manifest-file": "bool",
                "--clear-notification-config": "bool",
                "--clear-notification-event-types": "bool",
                "--clear-preserve-metadata": "bool",
                "--clear-schedule": "bool",
                "--clear-source-agent-pool": "bool",
                "--clear-source-auth-method": "bool",
                "--clear-source-creds-file": "bool",
                "--clear-source-endpoint": "bool",
                "--clear-source-list-api": "bool",
                "--clear-source-network-protocol": "bool",
                "--clear-source-request-model": "bool",
                "--clear-source-signing-region": "bool",
                "--custom-storage-class": "value",
                "--delete-from": [
                  "destination-if-unique",
                  "source-after-transfer"
                ],
                "--description": "value",
                "--destination": "value",
                "--destination-agent-pool": "value",
                "--enable-posix-transfer-logs": "bool",
                "--event-stream-expires": "value",
                "--event-stream-name": "value",
                "--event-stream-starts": "value",
                "--exclude-prefixes": "value",
                "--include-modified-after-absolute": "value",
                "--include-modified-after-relative": "value",
                "--include-modified-before-absolute": "value",
                "--include-modified-before-relative": "value",
                "--include-prefixes": "value",
                "--intermediate-storage-path": "value",
                "--log-action-states": "value",
                "--log-actions": "value",
                "--manifest-file": "value",
                "--notification-event-types": "value",
                "--notification-payload-format": [
                  "json",
                  "none"
                ],
                "--notification-pubsub-topic": "value",
                "--overwrite-when": [
                  "always",
                  "different",
                  "never"
                ],
                "--preserve-metadata": "value",
                "--schedule-repeats-every": "value",
                "--schedule-repeats-until": "value",
                "--schedule-starts": "value",
                "--source": "value",
                "--source-agent-pool": "value",
                "--source-auth-method": [
                  "AWS_SIGNATURE_V2",
                  "AWS_SIGNATURE_V4"
                ],
                "--source-creds-file": "value",
                "--source-endpoint": "value",
                "--source-list-api": [
                  "LIST_OBJECTS",
                  "LIST_OBJECTS_V2"
                ],
                "--source-network-protocol": [
                  "HTTP",
                  "HTTPS"
                ],
                "--source-request-model": [
                  "PATH_STYLE",
                  "VIRTUAL_HOSTED_STYLE"
                ],
                "--source-signing-region": "value",
                "--status": [
                  "deleted",
                  "disabled",
                  "enabled"
                ]
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {}
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--expand-table": "bool",
                "--job-names": "value",
                "--limit": "value",
                "--operation-names": "value",
                "--operation-statuses": "value",
                "--page-size": "value"
              }
            },
            "monitor": {
              "commands": {},
              "flags": {}
            },
            "pause": {
              "commands": {},
              "flags": {}
            },
            "resume": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "version": {
      "commands": {},
      "flags": {}
    },
    "vmware": {
      "commands": {
        "locations": {
          "commands": {
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "network-policies": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--edge-services-cidr": "value",
                "--external-ip-access": "bool",
                "--internet-access": "bool",
                "--location": "dynamic",
                "--vmware-engine-network": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--edge-services-cidr": "value",
                "--external-ip-access": "bool",
                "--internet-access": "bool",
                "--location": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "networks": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--location": "dynamic",
                "--type": [
                  "LEGACY",
                  "STANDARD"
                ]
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--location": "dynamic"
              }
            }
          },
          "flags": {}
        },
        "node-types": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "operations": {
          "commands": {
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            }
          },
          "flags": {}
        },
        "private-clouds": {
          "commands": {
            "clusters": {
              "commands": {
                "create": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--node-type-config": "value",
                    "--private-cloud": "dynamic"
                  }
                },
                "delete": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--private-cloud": "dynamic"
                  }
                },
                "describe": {
                  "commands": {},
                  "flags": {
                    "--location": "dynamic",
                    "--private-cloud": "dynamic"
                  }
                },
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--private-cloud": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                },
                "update": {
                  "commands": {},
                  "flags": {
                    "--async": "bool",
                    "--location": "dynamic",
                    "--node-type-config": "value",
                    "--private-cloud": "dynamic"
                  }
                }
              },
              "flags": {}
            },
            "create": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--cluster": "dynamic",
                "--description": "value",
                "--location": "dynamic",
                "--management-range": "value",
                "--node-type-config": "value",
                "--vmware-engine-network": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--delay-hours": [
                  0,
                  1,
                  2,
                  3,
                  4,
                  5,
                  6,
                  7,
                  8
                ],
                "--location": "dynamic"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "dynamic"
              }
            },
            "hcx": {
              "commands": {
                "activationkeys": {
                  "commands": {
                    "create": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    },
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    },
                    "list": {
                      "commands": {},
                      "flags": {
                        "--filter": "value",
                        "--limit": "value",
                        "--location": "dynamic",
                        "--page-size": "value",
                        "--private-cloud": "dynamic",
                        "--sort-by": "value",
                        "--uri": "bool"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "dynamic",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "nsx": {
              "commands": {
                "credentials": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    },
                    "reset": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            },
            "subnets": {
              "commands": {
                "list": {
                  "commands": {},
                  "flags": {
                    "--filter": "value",
                    "--limit": "value",
                    "--location": "dynamic",
                    "--page-size": "value",
                    "--private-cloud": "dynamic",
                    "--sort-by": "value",
                    "--uri": "bool"
                  }
                }
              },
              "flags": {}
            },
            "undelete": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--location": "dynamic"
              }
            },
            "update": {
              "commands": {},
              "flags": {
                "--async": "bool",
                "--description": "value",
                "--location": "dynamic"
              }
            },
            "vcenter": {
              "commands": {
                "credentials": {
                  "commands": {
                    "describe": {
                      "commands": {},
                      "flags": {
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    },
                    "reset": {
                      "commands": {},
                      "flags": {
                        "--async": "bool",
                        "--location": "dynamic",
                        "--private-cloud": "dynamic"
                      }
                    }
                  },
                  "flags": {}
                }
              },
              "flags": {}
            }
          },
          "flags": {}
        }
      },
      "flags": {}
    },
    "workflows": {
      "commands": {
        "delete": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--location": "value"
          }
        },
        "deploy": {
          "commands": {},
          "flags": {
            "--async": "bool",
            "--description": "value",
            "--labels": "value",
            "--location": "value",
            "--service-account": "value",
            "--source": "value"
          }
        },
        "describe": {
          "commands": {},
          "flags": {
            "--location": "value",
            "--revision-id": "value"
          }
        },
        "execute": {
          "commands": {},
          "flags": {
            "--call-log-level": [
              "log-all-calls",
              "log-errors-only",
              "none"
            ],
            "--data": "value",
            "--labels": "value",
            "--location": "value"
          }
        },
        "executions": {
          "commands": {
            "cancel": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--workflow": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--workflow": "value"
              }
            },
            "describe-last": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--location": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "wait": {
              "commands": {},
              "flags": {
                "--location": "value",
                "--workflow": "value"
              }
            },
            "wait-last": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "list": {
          "commands": {},
          "flags": {
            "--filter": "value",
            "--limit": "value",
            "--location": "value",
            "--page-size": "value",
            "--sort-by": "value",
            "--uri": "bool"
          }
        },
        "run": {
          "commands": {},
          "flags": {
            "--call-log-level": [
              "log-all-calls",
              "log-errors-only",
              "none"
            ],
            "--data": "value",
            "--labels": "value",
            "--location": "value"
          }
        }
      },
      "flags": {}
    },
    "workspace-add-ons": {
      "commands": {
        "deployments": {
          "commands": {
            "create": {
              "commands": {},
              "flags": {
                "--deployment-file": "value",
                "--deployment-object": "value"
              }
            },
            "delete": {
              "commands": {},
              "flags": {
                "--etag": "value"
              }
            },
            "describe": {
              "commands": {},
              "flags": {}
            },
            "install": {
              "commands": {},
              "flags": {}
            },
            "install-status": {
              "commands": {},
              "flags": {}
            },
            "list": {
              "commands": {},
              "flags": {
                "--filter": "value",
                "--limit": "value",
                "--page-size": "value",
                "--sort-by": "value",
                "--uri": "bool"
              }
            },
            "replace": {
              "commands": {},
              "flags": {
                "--deployment-file": "value",
                "--deployment-object": "value",
                "--etag": "value"
              }
            },
            "uninstall": {
              "commands": {},
              "flags": {}
            }
          },
          "flags": {}
        },
        "get-authorization": {
          "commands": {},
          "flags": {}
        }
      },
      "flags": {}
    }
  },
  "flags": {
    "--access-token-file": "value",
    "--account": "value",
    "--billing-project": "value",
    "--configuration": "value",
    "--flags-file": "value",
    "--flatten": "value",
    "--format": "value",
    "--help": "bool",
    "--impersonate-service-account": "value",
    "--log-http": "bool",
    "--project": "dynamic",
    "--quiet": "bool",
    "--trace-token": "value",
    "--user-output-enabled": "bool",
    "--verbosity": [
      "critical",
      "debug",
      "error",
      "info",
      "none",
      "warning"
    ],
    "--version": "bool"
  }
}
